@@ -23,8 +23,11 @@ class IceWarpIDP extends iAPIInterface
 		return icewarp_apiobjectcall_persistent($this->base, 'AddFiles', $archive, $path);
 	}
 
-	function AddFilesEx($archive, $path, $pass)
-	{
+	function AddFilesEx(
+		$archive,
+		$path,
+		 		$pass
+	) {
 		return icewarp_apiobjectcall_persistent($this->base, 'AddFilesEx', $archive, $path, $pass);
 	}
 
@@ -35,13 +38,16 @@ class IceWarpIDP extends iAPIInterface
 
 	function RestoreFiles($archive, $path)
 	{
-		return icewarp_apiobjectcal_persistent($this->base, 'RestoreFiles', $archive, $path);
+		return icewarp_apiobjectcall_persistent($this->base, 'RestoreFiles', $archive, $path);
 	}
 
-	function RestoreFilesEx($archive, $path, $list = '', $pass = '')
-	{
-		return icewarp_apiobjectcall_persistent($this->base, 'RestoreFilesEx', $archive, $path, $list,
-			$pass);
+	function RestoreFilesEx(
+		$archive,
+		$path,
+		$list = '',
+		 		$pass = ''
+	) {
+		return icewarp_apiobjectcall_persistent($this->base, 'RestoreFilesEx', $archive, $path, $list, $pass);
 	}
 
 	function CopyFile($source, $dest)
