@@ -44,9 +44,9 @@ class IceWarpStatistics extends iAPIInterface
 
    function GetSessions ($service, $history)
    {
-      return $this->FunctionCall( 'GetSessions', $service, $history );
+      return $this->FunctionCall( 'GetSessions', $service, (int)$history );
    }
 }
 
  class MerakStatistics extends IceWarpStatistics {}
-?>
\ No newline at end of file
+?>
