@@ -38,8 +38,22 @@ abstract class slPHPMailer
     var $tempFile  = '';
     public $BackupConnections;
 
-    
-    
+    static public $smartAttachSkipExt = [
+        'ics',
+        'vcf',
+        'eml'
+    ];
+
+    static public $smartAttachSkipType = [
+        'text/calendar',
+        'text/x-vcard',
+        'text-vcard',
+        'message/rfc822',
+        'application/x-pkcs7-signature',
+        'application/x-pkcs7-mime',
+        'application/pkcs7-signature',
+        'application/pkcs7-mime'
+    ];
           
     var $Host        = "127.0.0.1";
      
@@ -80,37 +94,40 @@ abstract class slPHPMailer
     public $DeferredDelivery;
     public $SmartAttachKey;
     public $mime_encode_base64;
+    public $isDraft = false;
     public $smartAttach;
-
     public $inlineAttachmentsHashCid = [];
     public $inlineAttachmentsCidDedup = [];
 
     abstract public function setCharset($charset);
 
-    function PHPMailer($charset){
+    function PHPMailer($charset)
+    {
 	     $this->charSet = $charset; 
     }
 
-    function IsHTML($bool) {
-        if($bool == true)
-            $this->ContentType = "text/html";
-        else
-            $this->ContentType = "text/plain";
+    function IsHTML($bool)
+    {
+        $this->ContentType = $bool ? "text/html" : "text/plain";
     }
 
-    function IsSMTP() {
+    function IsSMTP()
+    {
         $this->Mailer = "smtp";
     }
 
-    function IsMail() {
+    function IsMail()
+    {
         $this->Mailer = "mail";
     }
 
-    function IsSendmail() {
+    function IsSendmail()
+    {
         $this->Mailer = "sendmail";
     }
 
-    function IsQmail() {
+    function IsQmail()
+    {
         $this->Sendmail = "/var/qmail/bin/sendmail";
         $this->Mailer = "sendmail";
     }
@@ -120,130 +137,124 @@ abstract class slPHPMailer
     	$this->iMIP = $status;
     }
     
-         function AddAddress($address, $name = "") {
+         function AddAddress($address, $name = "")
+    {
     	$address = self::addressToIDN($address);
         $cur = count($this->to);
         $this->to[$cur][0] = trim($address);
         $this->to[$cur][1] = $name;
     }
 
-    function AddCC($address, $name = "") {
+    function AddCC($address, $name = "")
+    {
     	$address = self::addressToIDN($address);
         $cur = count($this->cc);
         $this->cc[$cur][0] = trim($address);
         $this->cc[$cur][1] = $name;
     }
 
-    function AddBCC($address, $name = "") {
+    function AddBCC($address, $name = "")
+    {
     	$address = self::addressToIDN($address);
         $cur = count($this->bcc);
         $this->bcc[$cur][0] = trim($address);
         $this->bcc[$cur][1] = $name;
     }
 
-    function AddReplyTo($address, $name = "") {
+    function AddReplyTo($address, $name = "")
+    {
     	$address = self::addressToIDN($address);
         $cur = count($this->ReplyTo);
         $this->ReplyTo[$cur][0] = trim($address);
         $this->ReplyTo[$cur][1] = $name;
     }
 
-    function setMode( $mode )
+    function setMode($mode)
     {
       $this->sendMode = $mode;
     }
 
-         function Send($header = false,$body = false,$charset = 'UTF-8',&$filename = false,$message_id = false, $deleteFile = true) 
+         function Send($header = false, $body = false, $charset = 'UTF-8', &$filename = false, $message_id = false, $deleteFile = true)
     {
     	$this->time = time();
-    	if($message_id){
-    		$this->message_id = $message_id;
-    	}
-    	
+    	if($message_id) $this->message_id = $message_id;
     	$result = true;
-    	if($this->sendMode=='file'){
+    	if($this->sendMode == 'file'){
 	    	$filename = $this->prepareMessageTempFile($charset,$filename,$header,$body);
 	    	if((count($this->to) + count($this->cc) + count($this->bcc)) < 1){
 	    		$this->SetError('no_rcpt',$this->Lang("provide_address"));
 	    		return false;
 	    	}
-	    	$result = $this->sendFromPreparedFile($filename,$deleteFile,false,false,false);
+	    	$result = $this->sendFromPreparedFile($filename, $deleteFile,false,false,false);
     	}else{
-    		if($charset) {
-    			$this->setCharset($charset);
-    		}
-    		if((count($this->to) + count($this->cc) + count($this->bcc)) < 1)
-    		{
+    		if($charset) $this->setCharset($charset);
+    		if((count($this->to) + count($this->cc) + count($this->bcc)) < 1) {
     			$this->SetError('no_rcpt',$this->Lang("provide_address"));
     			return false;
     		}
-    		if(!empty($this->AltBody)){
-    			$this->ContentType = "multipart/alternative";
-    		}
+    		if(!empty($this->AltBody)) $this->ContentType = "multipart/alternative";
+
     		$this->error_count = 0;
     		$this->SetMessageType();
     		
     		$message = "";
-    		$filename = false;
-    		$fp = false;
+    		$filename = $fp = false;
     		
-    		if (!$header) $this->CreateHeader($fp, $message);
-    		else $this->messageOutput($fp,$message,$header . "\r\n");
+    		if (!$header){
+    		    $this->CreateHeader($fp, $message);
+            } else {
+    		    $this->messageOutput($fp,$message,$header . "\r\n");
+            }
             $messageFile = false;
-    		if (!$body) $this->CreateBody($fp,$messageFile, $message);
-    		else $this->messageOutput($fp,$message,$body);
+    		if (!$body) {
+    		    $this->CreateBody($fp,$messageFile, $message);
+            } else {
+    		    $this->messageOutput($fp,$message,$body);
+            }
     		
-    		if($this->smartAttach){
-    			$this->smartAttach();
-    		}
+    		if($this->smartAttach && !$this->isDraft) $this->smartAttach();
     		if($this->EncryptMessage || $this->SignMessage){
     		    $filename = null;
     			$this->encryptAndSign($message, $filename);
     		}
-    		
-    		switch($this->Mailer)
-    		{
-    			case "sendmail":
-    				$result = $this->SendmailSend($filename, $fp, $message);
-    				break;
-    			case "mail":
-    				$result = $this->MailSend($filename, $fp, $message);
-    				break;
-    			case "smtp":
-    				$result = $this->SmtpSend($filename, $fp, $message);
-    				break;
-    			default:
-    				$this->SetError('mailer_support',$this->Mailer . $this->Lang("mailer_not_supported"));
-    				$result = false;
-    			break;
-    		}
+
+    		if($this->Mailer == 'sendmail'){
+                $result = $this->SendmailSend($filename, $fp, $message);
+            }elseif($this->Mailer == 'mail'){
+                $result = $this->MailSend($filename, $fp, $message);
+            }elseif($this->Mailer == 'smtp'){
+                $result = $this->SmtpSend($filename, $fp, $message);
+    		}else{
+                $this->SetError('mailer_support',$this->Mailer . $this->Lang("mailer_not_supported"));
+                $result = false;
+            }
     	}
     	return $result;
     }
     
-    function messageOutput(&$fp,&$message,$string)
+    function messageOutput(&$fp, &$message, $string)
     {
-      if($this->sendMode == 'file'){
-        fwrite($fp,$string);
-      }else{
-        $message.=$string;
-      }
+        if ($this->sendMode == 'file') {
+            fwrite($fp, $string);
+        } else {
+            $message .= $string;
+        }
     }
 
-    function SendmailSend($filename, &$fp, &$message) {
-        if ($this->Sender != "")
+    function SendmailSend($filename, &$fp, &$message)
+    {
+        if ($this->Sender != '') {
             $sendmail = sprintf("%s -oi -f %s -t", $this->Sendmail, $this->Sender);
-        else
+        } else {
             $sendmail = sprintf("%s -oi -t", $this->Sendmail);
+        }
 
-        if(!@$mail = popen($sendmail, "w"))
-        {
+        if(!@$mail = popen($sendmail, "w")) {
             $this->SetError('sendmail_open',$this->Lang("execute") . $this->Sendmail);
             return false;
         }
 
-        if ($this->SignMessage || $this->EncryptMessage)
-          $this->encryptAndSign($fp, $filename, $message);
+        if ($this->SignMessage || $this->EncryptMessage) $this->encryptAndSign($fp, $filename, $message);
         
         if($this->sendMode == 'file'){
 	        fseek($fp,0);
@@ -255,78 +266,59 @@ abstract class slPHPMailer
         }
         
         $result = pclose($mail) >> 8 & 0xFF;
-        if($result != 0)
-        {
+        if($result != 0) {
             $this->SetError('sendmail_send',$this->Lang("execute") . $this->Sendmail);
             return false;
         }
-
         return true;
     }
 
-    function MailSend($filename, &$fp, &$message) {
-        $to = "";
-        for($i = 0; $i < count($this->to); $i++)
-        {
-            if($i != 0) { $to .= ", "; }
-            $to .= $this->to[$i][0];
-        }
+    function MailSend($filename, &$fp, &$message)
+    {
+        $to = implode(', ', $this->to);
 
-        if ($this->SignMessage || $this->EncryptMessage) {
-          $this->encryptAndSign($fp,$filename, $message);
-        }
-          if($this->sendMode == 'file'){
-            $message = file_get_contents($filename);
-          }
-		      $delimiter = strpos($message, "\r\n");
-		      $header = substr($message, 0, $delimiter);
-		      $body = substr($message, $delimiter);
-        
+        if ($this->SignMessage || $this->EncryptMessage) $this->encryptAndSign($fp, $filename, $message);
+        if ($this->sendMode == 'file') $message = file_get_contents($filename);
+        $delimiter = strpos($message, "\r\n");
+        $header = substr($message, 0, $delimiter);
+        $body = substr($message, $delimiter);
 
-        if ($this->Sender != "" && strlen(ini_get("safe_mode"))< 1)
-        {
+        if ($this->Sender != "" && strlen(ini_get("safe_mode")) < 1) {
             $old_from = ini_get("sendmail_from");
             ini_set("sendmail_from", $this->Sender);
             $params = sprintf("-oi -f %s", $this->Sender);
-            $rt = @mail($to, $this->EncodeHeader($this->Subject), $body, 
-                        $header, $params);
-        }
-        else
+            $rt = @mail($to, $this->EncodeHeader($this->Subject), $body, $header, $params);
+        } else{
             $rt = @mail($to, $this->EncodeHeader($this->Subject), $body, $header);
+        }
 
-        if (isset($old_from))
-            ini_set("sendmail_from", $old_from);
+        if (isset($old_from)) ini_set("sendmail_from", $old_from);
 
-        if(!$rt)
-        {
-            $this->SetError('mail_send',$this->Lang("instantiate"));
+        if (!$rt) {
+            $this->SetError('mail_send', $this->Lang("instantiate"));
             return false;
         }
-
         return true;
     }
 
-		function SenderDomain() {
-			return substr($this->From, strrpos($this->From, '@') + 1);
-		}
-
-    function SmtpSend(&$filename,&$fp, &$message) {
+    function SenderDomain()
+    {
+        return substr($this->From, strrpos($this->From, '@') + 1);
+    }
 
+    function SmtpSend(&$filename,&$fp, &$message)
+    {
         include_once($this->PluginDir . "smtp.php");
 
-        $error = "";
-        $bad_rcpt = array();
+        $error = '';
+        $bad_rcpt = [];
 
-        if(!$this->SmtpConnect()){
-        	return false;
-        }
+        if(!$this->SmtpConnect()) return false;
 
         $smtp_from = ($this->Sender == "") ? $this->From : $this->Sender;
         $smtp_from = $this->addressToIDN($smtp_from);
         
-        if(!$this->smtp->Mail($smtp_from))
-        {
-            $error = $this->Lang("from_failed") . $smtp_from;
+        if(!$this->smtp->Mail($smtp_from)) {
             $this->SetSMTPError("from_failed", $smtp_from);
             $this->smtp->Reset();
             return false;
@@ -337,54 +329,27 @@ abstract class slPHPMailer
 				$aErrorMsg[$this->rawRecipient] = slToolsPHP::htmlspecialchars($this->smtp->error['smtp_msg']);
         	}
         }else{
-			if (count($this->to)+count($this->cc)+count($this->bcc) > 1){
-			  if($this->Port!='25'){
-			  	$host = $this->Host.'#'.$this->Port;
-			  }else{
-			  	$host = $this->Host;
-			  }
-			  $prefix = $this->relay?("@".$host.":"):"";
-			}else
-			  $prefix = "";
-			
-			for($i = 0; $i < count($this->to); $i++)
-			{
-			    if(!$this->smtp->Recipient($prefix.$this->addressToIDN($this->to[$i][0]))){
-			        $bad_rcpt[] = $prefix.$this->to[$i][0];
-			        $aErrorMsg[$this->to[$i][0]] = slToolsPHP::htmlspecialchars($this->smtp->error['smtp_msg']);
-			    }
-			}
-			for($i = 0; $i < count($this->cc); $i++)
-			{
-			    if(!$this->smtp->Recipient($prefix.$this->addressToIDN($this->cc[$i][0]))){
-			        $bad_rcpt[] = $prefix.$this->cc[$i][0];
-			        $aErrorMsg[$this->cc[$i][0]] = slToolsPHP::htmlspecialchars($this->smtp->error['smtp_msg']);
-			    }
-			}
-			
-			for($i = 0; $i < count($this->bcc); $i++)
-			{
-			    if(!$this->smtp->Recipient($prefix.$this->addressToIDN($this->bcc[$i][0]))){
-			        $bad_rcpt[] = $prefix.$this->bcc[$i][0];
-			        $aErrorMsg[$this->bcc[$i][0]] = slToolsPHP::htmlspecialchars($this->smtp->error['smtp_msg']);
-			    }
-			}
+            $prefix = '';
+            $recipient_list = array_merge($this->to, $this->cc ?? [], $this->bcc ?? []);
+			if (count($recipient_list) > 0){
+			  	$host = $this->Host.($this->Port != '25'?('#'.$this->Port):'');
+                $prefix = $this->relay ? ("@".$host) : "";
+                foreach ($recipient_list as $recipient){
+                    $sep = $this->relay?(strpos($recipient[0],'@')===0?',':':'):'';
+                    if(!$this->smtp->Recipient($prefix.$sep.$this->addressToIDN($recipient[0]))){
+                        $bad_rcpt[] = $prefix.$sep.$recipient[0];
+                        $aErrorMsg[$to[0]] = slToolsPHP::htmlspecialchars($this->smtp->error['smtp_msg']);
+                    }
+                }
+            }
         }
         
-        
-        if(count($bad_rcpt) > 0)
-        {
-            for($i = 0; $i < count($bad_rcpt); $i++)
-            {
-                if($i != 0) { $error .= ", "; }
-                $error .= $bad_rcpt[$i];
-            }
+        if(count($bad_rcpt) > 0) {
+            $error = implode(', ', $bad_rcpt);
             foreach($aErrorMsg as $key => $msg){
             	$aErrorMsg[$key] = $msg.'('.$key.')';
             }
-
             $errormsg = implode('<br/>',$aErrorMsg);
-            
             $this->SetSMTPError("recipients_failed", $errormsg);
             $this->smtp->Reset();
             return false;
@@ -397,11 +362,11 @@ abstract class slPHPMailer
             return false;
         }
 
-        if($this->SMTPKeepAlive == true)
+        if($this->SMTPKeepAlive == true){
             $this->smtp->Reset();
-        else            
+        } else{
             $this->SmtpClose();
-
+        }
         return true;
     }
 
@@ -450,7 +415,7 @@ abstract class slPHPMailer
                          $sslext = defined('OPENSSL_ALGO_SHA1');
             if ('tls' === $secure || 'ssl' === $secure) {
                                  if (!$sslext) {
-                                         throw new phpmailerException($this->lang('extension_missing').'openssl', self::STOP_CRITICAL);
+                    throw new Exception($this->lang('extension_missing').'openssl');
                 }
             }    
             if($this->smtp->Connect($prefix.$host, $port, $this->Timeout)){
@@ -506,7 +471,7 @@ abstract class slPHPMailer
             return false;
         }
         global $PHPMAILER_LANG;
-        $this->language = $PHPMAILER_LANG;
+        $this->language = $PHPMAILER_LANG ?? [];
         return true;
     }
     
@@ -622,16 +587,27 @@ abstract class slPHPMailer
 
        $this->messageOutput($fp,$message,"Subject: ".$this->putheader($this->Subject,$this->charSet)."\r\n");
 
-        if ($this->XMailer)
-					$XMailer = $this->XMailer;
-				else
-					$XMailer = "PHPMailer [version " . $this->Version . "]";
+        if ($this->XMailer) {
+            $XMailer = $this->XMailer;
+        } else {
+            $XMailer = "PHPMailer [version " . $this->Version . "]";
+        }
 				
-				$this->messageOutput($fp,$message,sprintf("Message-ID: %s%s", $this->last_message_id, $this->LE));
+        $this->messageOutput($fp,$message,sprintf("Message-ID: %s%s", $this->last_message_id, $this->LE));
 
-	if ($this->Priority) $this->messageOutput($fp,$message,$this->HeaderLine("X-Priority", $this->Priority));
+	    if ($this->Priority) $this->messageOutput($fp,$message,$this->HeaderLine("X-Priority", $this->Priority));
         $this->messageOutput($fp,$message,$this->HeaderLine("X-Mailer", $XMailer));
-        
+        if($this->isDraft){
+            foreach ($this->attachment as $attachment){
+                if(self::isForbiddenSmartAttach($attachment[2], $attachment[4])) continue;
+                $smartParams = $attachment[12];
+                
+                $encoded_url = '=?UTF-8?B?'.base64_encode($smartParams['url']).'?=';
+                $encoded_path = '=?UTF-8?B?'.base64_encode($smartParams['fullpath']).'?=';
+                $encoded_name = '=?UTF-8?B?'.$attachment[2].'?=';
+                $this->messageOutput($fp,$message,$this->HeaderLine("X-IceWarp-SmartAttach-Draft", "fullpath=\"$encoded_path\"; class=\"".$smartParams['class']."\"; name=\"$encoded_name\"; type=\"".$smartParams['type']."\"; size=\"".$smartParams['size']."\"; url=\"$encoded_url\";"));
+            }
+        }
         if($this->ConfirmReadingTo != "")
             $this->messageOutput($fp,$message,$this->HeaderLine("Disposition-Notification-To", 
                        "<" . trim($this->ConfirmReadingTo) . ">"));
@@ -785,8 +761,8 @@ abstract class slPHPMailer
 						$this->AttachAll($fp,$filename, $message,false,$smarter_smart,'inline');
 						 						if($this->NormalAttachmentExists()){
 		               		$this->AttachAll($fp,$filename, $message,$this->boundary[3],$smarter_smart,'normal');
+                            $this->messageOutput($fp,$message,sprintf("--%s--%s", $this->boundary[3], $this->LE));
 						}
-	                	 	                	$this->messageOutput($fp,$message,sprintf("--%s--%s", $this->boundary[3], $this->LE));
 					}else{
 						$this->AttachAll($fp,$filename, $message,false,$smarter_smart);
 					}
@@ -918,73 +894,64 @@ abstract class slPHPMailer
         return true;
     }
 
-    function AttachAll(&$fp = false,$messageFile = false, &$message, $boundary = false,$smarter_smart = true, $attachmenttype = 'all') 
+    function AttachAll(&$fp = false,$messageFile = false, &$message = '', $boundary = false,$smarter_smart = true, $attachmenttype = 'all') 
     {
         $mime = array();
         if(!$boundary){       	
         	$boundary = $this->boundary[1];
         }
 
-        for($i = 0; $i < count($this->attachment); $i++)
+        foreach($this->attachment as $i => $attachment)
         {
-        	
-            $bString = $this->attachment[$i][5];
-            if ($bString)
-                $string = $this->attachment[$i][0];
-            else
-                $path = $this->attachment[$i][0];
+            $bString = $attachment[5];
+            if ($bString){
+                $string = $attachment[0];
+            } else{
+                $path = $attachment[0];
+            }
 
-            $filename    = $this->attachment[$i][1];
-            $name        = $this->attachment[$i][2];
-            $encoding    = $this->attachment[$i][3];
-            $type        = $this->attachment[$i][4];
-            $disposition = $this->attachment[$i][6];
-            $cid         = $this->attachment[$i][7];
-            $encode      = $this->attachment[$i][8];
-            $method      = $this->attachment[$i][9];
-            $dummy       = $this->attachment[$i][10];
-            $smart_attach= $this->attachment[$i][11];
-            $smart_params   = $this->attachment[$i][12];
+            $filename    = $attachment[1];
+            $name        = $attachment[2];
+            $encoding    = $attachment[3];
+            $type        = $attachment[4];
+            $disposition = $attachment[6];
+            $cid         = $attachment[7];
+            $encode      = $attachment[8];
+            $method      = $attachment[9];
+            $dummy       = $attachment[10];
+            $smart_attach= $attachment[11];
+            $smart_params   = $attachment[12];
 
             $charset      = "UTF-8";
 
             if(empty($disposition)) $disposition = 'attachment';
 
-       		if($attachmenttype=='inline'){
-            	if($disposition!='inline'){
-            		continue;
-            	}
-            }
-        	if($attachmenttype=='normal'){
-            	if($disposition=='inline'){
-            		continue;
-            	}
-            }
+       		if($attachmenttype == 'inline' && $disposition != 'inline') continue;
+        	if($attachmenttype == 'normal' && $disposition == 'inline')	continue;
             
-	                       if (slToolsString::isabovechar($name)){
+        	             if (slToolsString::isabovechar($name)){
               slSystem::import('tools/charset');
               $name = "=?" . strtoupper($this->charSet) . "?Q?" .$this->converttoquoted(slToolsCharset::my_iconv('UTF-8', $this->charSet . '//IGNORE', $name)) . "?=";
             }
-                                      $this->messageOutput($fp,$message,sprintf("--%s%s", $boundary, $this->LE));
+            $this->messageOutput($fp,$message,sprintf("--%s%s", $boundary, $this->LE));
             
             $sHead = "Content-Type: %s; name=\"%s\";";
             if($charset) $sHead.= " charset=\"".$charset."\";";
             if($method) $sHead.= " method=".$method;
-            if($dummy){
-            	$sHead.= "; dummyimg=true";
-            }
+            if($dummy) $sHead.= "; dummyimg=true";
             $sHead.="%s";
             
             $this->messageOutput($fp,$message,sprintf($sHead, $type, $name, $this->LE, $this->LE));
-            
-            if($smarter_smart && $smart_attach && $disposition != "inline" && $type!='message/rfc822'){
+
+            if($smarter_smart && $smart_attach && $disposition != "inline" && !self::isForbiddenSmartAttach($name, $type)){
             	$key = $this->SmartAttachKey;
             	if($smart_params){
  					$hash = md5($key.$smart_params['url']);
  					$encoded_url = '=?UTF-8?B?'.base64_encode($smart_params['url']).'?=';
  					$encoded_zipid = '=?UTF-8?B?'.base64_encode($smart_params['zipid']).'?=';
- 					
- 					$this->messageOutput($fp,$message,"X-IceWarp-SmartAttach: url=\"".$encoded_url."\"; zipid=\"".$encoded_zipid."\"; size=\"".$smart_params['size']."\"; hash=\"".$hash."\"; algorithm=\"MD5\"".$this->LE.$this->LE);
+ 					$encoded_fullpath = '=?UTF-8?B?'.base64_encode($smart_params['fullpath']).'?=';
+
+ 					$this->messageOutput($fp,$message,"X-IceWarp-SmartAttach: url=\"".$encoded_url."\"; zipid=\"".$encoded_zipid."\"; size=\"".$smart_params['size']."\"; hash=\"".$hash."\"; algorithm=\"MD5\"; fullpath=\"".$encoded_fullpath."\"".$this->LE.$this->LE);
             	}else{
             		$hash = md5($key.$path);
             		$encoded_path = '=?UTF-8?B?'.base64_encode($path).'?=';
@@ -998,20 +965,22 @@ abstract class slPHPMailer
             if($disposition == "inline")
                 $this->messageOutput($fp,$message,sprintf("Content-ID: <%s>%s", $cid, $this->LE));
             if(!$this->iMIP){
-                 $this->messageOutput($fp,$message,sprintf("Content-Disposition: %s; filename=\"%s\"%s", 
-                              $disposition, $name, $this->LE.$this->LE));
+                 $this->messageOutput($fp,$message,sprintf("Content-Disposition: %s; filename=\"%s\"%s", $disposition, $name, $this->LE.$this->LE));
             }else{
             	$this->messageOutput($fp,$message,$this->LE.$this->LE);
             }
             if($bString){
-                if ($encode) $this->messageOutput($fp,$message,$this->EncodeString($string, $encoding));
-                else         $this->messageOutput($fp,$message,$string);
+                if ($encode){
+                    $this->messageOutput($fp,$message,$this->EncodeString($string, $encoding));
+                } else {
+                    $this->messageOutput($fp,$message,$string);
+                }
                 
-                if($this->IsError()) { return ""; }
+                if($this->IsError()) return '';
                 $this->messageOutput($fp,$message,$this->LE.$this->LE);
             } else {                             
                 $this->EncodeFile($fp,$messageFile,$message, $path, $encoding,$encode);
-                if($this->IsError()) { return ""; }
+                if($this->IsError()) return '';
                 $this->messageOutput($fp,$message,$this->LE.$this->LE);
             }
         }
@@ -1020,66 +989,48 @@ abstract class slPHPMailer
         return true;
     }
 
-    function EncodeFile (&$fp,$filename,$message, $path, $encoding = "base64",$encode = true) 
+    public function EncodeFile(&$fp, $filename, $message, $path, $encoding = "base64", $encode = true)
     {
-    	
-        if($encode){
-          if($this->sendMode == 'file'){
+        if ($this->sendMode != 'file') return true;
+        if ($encode) {
             $newFile = slToolsFilesystem::randomFilename($this->getTempDir());
-            
             fclose($fp);
-            switch($encoding){
-	          	case 'base64':
-	          		if(defined('IW_DISABLED') && IW_DISABLED){
-	          			file_put_contents($newFile,base64_encode(file_get_contents($path)));
-	          		}else{
-	          			icewarp_base64_encode_file($path,$newFile);
-
-	          			$api = IceWarpAPI::instance('Shared/PHPMailer');
-	          			$api->CacheFileWithUpdate($newFile);
-	          		}
-	          	break;
-	          	case 'quoted-printable':
-	          	default:
-	          		file_put_contents($newFile,file_get_contents($path));
-	          		break;
-	          }
-            
-            $fp = fopen($filename,"ab");
-            
-            if(!$fd = fopen($newFile, "rb")){
-              $this->SetError('encode_file',$this->Lang("file_open") . $path);
-              return "";
+            if ($encoding == 'base64') {
+                if (defined('IW_DISABLED') && IW_DISABLED) {
+                    file_put_contents($newFile, base64_encode(file_get_contents($path)));
+                } else {
+                    icewarp_base64_encode_file($path, $newFile);
+                    $api = IceWarpAPI::instance('Shared/PHPMailer');
+                    $api->CacheFileWithUpdate($newFile);
+                }
+            } else {
+                file_put_contents($newFile, file_get_contents($path));
             }
 
-            while($buffer = fread($fd,4096)){
-            	if($this->Encoding=='quoted-printable' && $encoding!='base64')
-            		$buffer = $this->EncodeQP($buffer);
-              
-              fwrite($fp,$buffer);
+            $fp = fopen($filename, "ab");
+
+            if (!$fd = fopen($newFile, "rb")) {
+                $this->SetError('encode_file', $this->Lang("file_open") . $path);
+                return "";
             }
-            
+
+            while ($buffer = fread($fd, 4096)) {
+                if ($this->Encoding == 'quoted-printable' && $encoding != 'base64') $buffer = $this->EncodeQP($buffer);
+                fwrite($fp, $buffer);
+            }
+
             fclose($fd);
-            if(defined('IW_DISABLED') && IW_DISABLED){
-            	unlink($newFile);
-            }else{
-            	slSystem::import('tools/icewarp');
-            	slToolsIcewarp::iw_delete($newFile);
+            if (defined('IW_DISABLED') && IW_DISABLED) {
+                unlink($newFile);
+            } else {
+                slSystem::import('tools/icewarp');
+                slToolsIcewarp::iw_delete($newFile);
             }
-          }else{
-             $message = base64_encode(file_get_contents($path));
-          }
-        }else{
-          if($this->sendMode == 'file'){
+        } else {
             $fd = fopen($path, "rb");
-            while($buffer = fread($fd,4096))
-              fwrite($fp,$buffer);
+            while ($buffer = fread($fd, 4096)) fwrite($fp, $buffer);
             fclose($fd);
-          }else{
-            $message = file_get_contents($path);
-          }
         }
-
         return true;
     }
 
@@ -1507,6 +1458,8 @@ self::fillstr(strtoupper(dechex(ord($string[$i]))), 2, "0");
 	
 	public function smartAttach($filename = null)
 	{
+        
+
 	    $api = IceWarpAPI::instance('Shared/PHPMailer');
 		$api->SmartAttach(
 				$filename,
@@ -1609,14 +1562,15 @@ self::fillstr(strtoupper(dechex(ord($string[$i]))), 2, "0");
 		}else{
 			$this->CreateBody($fp,$messageFile);
 		}
-		@fclose($fp);
-		
-		 		if ($this->SmartAttach)
+		@fclose($fp);	
+		 		if ($this->SmartAttach && !$this->isDraft)
 			$this->smartAttach($messageFile);
 			
 	    $api = IceWarpAPI::instance('Shared/PHPMailer');
 		$api->CacheFileWithUpdate($messageFile);
-	
+
+
+
 		return $messageFile;
 	}
 	
@@ -1654,11 +1608,13 @@ self::fillstr(strtoupper(dechex(ord($string[$i]))), 2, "0");
 				break;
 			case "smtp":
 				if ($this->SignMessage || $this->EncryptMessage){
-					$fp = fopen($$variable,'rw+');
+                    $fp = fopen($$variable,'rw+');
 					$this->encryptAndSign($fp,$$variable);
 				}
 				$result = $this->SmtpSend($$variable,$fp,$message);
-				@fclose($fp);
+				if($fp) {
+                    fclose($fp);
+                }
 				if($deleteFile){
 					slSystem::import('tools/icewarp');
 					@slToolsIcewarp::iw_delete($$variable);
@@ -1675,6 +1631,12 @@ self::fillstr(strtoupper(dechex(ord($string[$i]))), 2, "0");
 		}
 		return $result;
 	}
+
+    static public function isForbiddenSmartAttach($name, $type)
+    {
+        $ext = substr($name, strrpos($name, '.') + 1);
+        return in_array($type, self::$smartAttachSkipType) || in_array($ext , self::$smartAttachSkipExt);
+    }
 	
 }
 
