@@ -83,7 +83,7 @@ class slSMTP
 
         if(substr(PHP_OS, 0, 3) != "WIN")
            socket_set_timeout($this->smtp_conn, $tval, 0);
-                 $first_char =  fgets($this->smtp_conn,1);
+                 $first_char =  fgets($this->smtp_conn,2);
         if(strlen($first_char)!=1){
         	usleep(1000000);
         }
@@ -96,8 +96,11 @@ class slSMTP
         return true;
     }
 
-    function Authenticate($username, $password, $authtype = null) 
-    {
+    function Authenticate(
+        $username,
+                 $password,
+        $authtype = null
+    ) {
         if (!$this->server_caps) {
             $this->setError('Authentication is not allowed before HELO/EHLO');
             return false;
