@@ -17,9 +17,9 @@ class slObservable
 	public $aObservers;
 	public $state;
 	
-	public function setState($state)
+	public function setState($state, $force_notify = false)
 	{
-		if($state!==$this->getState()){
+		if($state!==$this->getState() || $force_notify){
 			$this->state = $state;
 			$this->notify();
 		}
