@@ -44,6 +44,7 @@ class slToolsImage
 	protected $memLimit;
 	 
 	protected $shutDownHandler;
+	private $forcedImageType;
 
     const MAX_MEMORY = 2147483648;
     const MAX_MEMORY_STRING = '2048M';
@@ -137,7 +138,7 @@ class slToolsImage
 	{
 		if(!$this->mkdir($directory,$permissions))
 		{
-			$this->throwException('Save Failed. Insufficient rights.', 'Save Failed! Insufficient rights for mkdir "' . $directory . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED_RIGHTS_MKDIR');
+			$this->throwException('Save Failed. Insufficient rights.', 'Save Failed! Insufficient rights for mkdir "' . htmlentities($directory) . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED_RIGHTS_MKDIR');
 			return false;
 		}
 		
@@ -282,7 +283,7 @@ class slToolsImage
 
 	private function imagebmp(&$img, $filename = false)
 	{
-	    if(!is_resource($img)) throw new Exc('Img is not resource');
+	    if(!is_object($img)) throw new Exc('Img is not resource');
 	    $wid = imagesx($img);
 	    $hei = imagesy($img);
 	    $wid_pad = str_pad('', $wid % 4, "\0");
@@ -395,7 +396,7 @@ class slToolsImage
 		}
 		catch(exception $e){
             $this->afterImageManipulation();
-			$this->throwException('Load Failed. BMP error.', 'Load Failed! BMP error in "' . $p_sFile . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED_BMP');
+			$this->throwException('Load Failed. BMP error.', 'Load Failed! BMP error in "' . htmlentities($p_sFile) . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED_BMP');
 			return false;
 		}
 	}
@@ -442,7 +443,7 @@ class slToolsImage
     }
 
      
-	public function load($source,$autorotate=true)
+	public function load($source,$autorotate=true) : bool
 	{
 		$this->setFile($source);
 		if($this->useCache) {
@@ -451,35 +452,36 @@ class slToolsImage
             return true;
         }
         if (!file_exists($source)) {
-            $this->throwException('Load Failed. File not found.', 'Load Failed! File not found "' . $source . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED_FILE_NOT_FOUND');
+            $this->throwException('Load Failed. File not found.', 'Load Failed! File not found "' . htmlentities($source) . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED_FILE_NOT_FOUND');
         }
         if(pathinfo($source, PATHINFO_EXTENSION) == 'svg' || mime_content_type($source) == 'image/svg+xml'){
             $this->image = file_get_contents($source);
             return true;
         }
-        $image_info = @getimagesize($source);
+        $image_info = getimagesize($source);
         if (!$image_info) {
-            $this->throwException('Load Failed. Invalid filetype or damaged image.', 'Load Failed! Invalid filetype or damaged image "' . $source . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED');
+            $this->throwException('Load Failed. Invalid filetype or damaged image.', 'Load Failed! Invalid filetype or damaged image "' . htmlentities($source) . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED');
         }
         $this->beforeImageManipulation($source);
         if(!$this->enoughMemory($image_info[0], $image_info[1], $image_info['channels'] ?? 3)){
             $this->afterImageManipulation();
-            $this->throwException('Load Failed. Image too large.', 'Load Failed. Image too large. "' . $source . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED_IMAGE_LARGE');
+            $this->throwException('Load Failed. Image too large.', 'Load Failed. Image too large. "' . htmlentities($source) . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED_IMAGE_LARGE');
         }
         $error = false;
         try {
             $this->image_type = $image_info[2];
             if ($this->image_type == IMAGETYPE_JPEG) {
-                if (!$this->image = @imagecreatefromjpeg($source)) $error = true;
+                if (!$this->image = imagecreatefromjpeg($source)) $error = true;
             } elseif ($this->image_type == IMAGETYPE_GIF) {
-                if (!$this->image = @imagecreatefromgif($source)) $error = true;
+                if (!$this->image = imagecreatefromgif($source)) $error = true;
             } elseif ($this->image_type == IMAGETYPE_BMP) {
                 if (!$this->image = $this->imagecreatefrombmp($source)) $error = true;
             } elseif ($this->image_type == IMAGETYPE_PNG) {
-                if (!$this->image = @imagecreatefrompng($source)) {
+                if (!$this->image = imagecreatefrompng($source)) {
                     $error = true;
                 } else {
-                    $output = imagecreatetruecolor($this->getWidth(), $this->getHeight());
+					list($width, $height) = getimagesize($source);
+                    $output = imagecreatetruecolor($width, $height);
                     $trnprt_indx = imagecolortransparent($this->image);
                                          if ($trnprt_indx >= 0) {
                                                  $trnprt_color = imagecolorsforindex($this->image, $trnprt_indx);
@@ -490,19 +492,19 @@ class slToolsImage
                         imagealphablending($output, false);
                         imagesavealpha($output, true);
                         $transparent = imagecolorallocatealpha($output, 255, 255, 255, 127);
-                        imagefilledrectangle($output, 0, 0, $this->getWidth(), $this->getHeight(), $transparent);
+                        imagefilledrectangle($output, 0, 0, $width, $height, $transparent);
                     }
-                    imagecopy($output, $this->image, 0, 0, 0, 0, $this->getWidth(), $this->getHeight());
+                    imagecopy($output, $this->image, 0, 0, 0, 0, $width, $height);
                     $this->image = $output;
                 }
             } else {
-                $this->throwException('Load Failed. Unsupported image type "' . $this->resolveImageType($this->image_type) . '".', 'Load Failed! Unsupported image type "' . $this->resolveImageType($this->image_type) . '" in "' . $source . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED_UNSUPPORTED_TYPE');
+                $this->throwException('Load Failed. Unsupported image type "' . $this->resolveImageType($this->image_type) . '".', 'Load Failed! Unsupported image type "' . $this->resolveImageType($this->image_type) . '" in "' . htmlentities($source) . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED_UNSUPPORTED_TYPE');
             }
             if ($error) {
-                $this->throwException('Load Failed. Corrupted image.', 'Load Failed! Corrupted image "' . $source . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED_UNSUPPORTED_TYPE');
+                $this->throwException('Load Failed. Corrupted image.', 'Load Failed! Corrupted image "' . htmlentities($source) . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED_UNSUPPORTED_TYPE');
             }
             if ($autorotate) {
-                @$exif = exif_read_data($source);
+                $exif = exif_read_data($source);
                 if (isset($exif['Orientation']) && ($exif['Orientation'] == 8 || $exif['Orientation'] == 6) && $image_info[0] > $image_info[1]) {
                     $degrees = 90;
                     if ($exif['Orientation'] != 8) {
@@ -515,7 +517,8 @@ class slToolsImage
             return true;
         } catch (Exception $e) {}
         $this->afterImageManipulation();
-        $this->throwException('Load Failed. Invalid filetype or damaged image.', 'Load Failed! Invalid filetype or damaged image "' . $source . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED');
+        $this->throwException('Load Failed. Invalid filetype or damaged image.', 'Load Failed! Invalid filetype or damaged image "' . htmlentities($source) . '" [Method:"' . __METHOD__ . '"]', 'LOAD_FAILED');
+		return false;
 	}
 	
 	private function fixFileName($filename,$image_type)
@@ -543,7 +546,11 @@ class slToolsImage
 		if($compression==''){$compression=false;}
 		if($image_type==''){$image_type=false;}
 		if($filename==''){$filename=false;}
-		
+
+		if ($this->forcedImageType) {
+			$image_type = $this->forcedImageType;
+		}
+
 		if($image_type===false){$image_type=$this->getImageType();}else{$image_type=$this->resolveImageType($image_type,true);}
 		if($filepath){$filepath=trim(str_replace('\\','/',$filepath),'/').'/';if($filepath=='/'){$filepath='./';}}
 		if($filename){$filename=$this->fixFileName($filename,$image_type);}
@@ -556,15 +563,17 @@ class slToolsImage
 			if(!$filename){$filename=$this->fixFileName($this->getFileName(),$image_type);}
 			
 			$this->runActions();
-			if($compression){$this->setQuality($compression);}
-			$compression=$this->getQuality();
+			if ($compression !== -1) {
+				if($compression){$this->setQuality($compression);}
+				$compression=$this->getQuality();
+			}
 			
 			try
 			{
 				if(!file_exists($filepath.$filename)){
 					 					if(!$this->mkdir($filepath,$permissions))
 					{
-						$this->throwException('Save Failed. Insufficient rights.', 'Save Failed! Insufficient rights for mkdir "' . $filepath . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED_RIGHTS_MKDIR');
+						$this->throwException('Save Failed. Insufficient rights.', 'Save Failed! Insufficient rights for mkdir "' . htmlentities($filepath) . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED_RIGHTS_MKDIR');
 						return false;
 					}
 				}
@@ -575,19 +584,18 @@ class slToolsImage
 			}
 			catch(Exception $e)
 			{
-				$this->throwException('Save Failed! Insufficient rights.', 'Save Failed! Insufficient rights "' . $filepath . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED_RIGHTS');
+				$this->throwException('Save Failed! Insufficient rights.', 'Save Failed! Insufficient rights "' . htmlentities($filepath) . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED_RIGHTS');
 				return false;
 			}
-			
 			if($this->needChange())
 			{
                 $this->beforeImageManipulation($filepath.$filename);
 				if($image_type==IMAGETYPE_JPEG)
 				{
-					if(!@imagejpeg($this->image,$filepath.$filename,$compression))
+					if(!imagejpeg($this->image,$filepath.$filename,$compression))
 					{
                         $this->afterImageManipulation();
-						$this->throwException('Save Failed! Image couldn\'t be created', 'Save Failed! Image couldn\'t be created "' . $filepath . $filename . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED');
+						$this->throwException('Save Failed! Image couldn\'t be created', 'Save Failed! Image couldn\'t be created "' . htmlentities($filepath . $filename) . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED');
 						return false;
 					}
 				}
@@ -596,7 +604,7 @@ class slToolsImage
 					if(!imagegif($this->image,$filepath.$filename))
 					{
                         $this->afterImageManipulation();
-						$this->throwException('Save Failed! Image couldn\'t be created', 'Save Failed! Image couldn\'t be created "' . $filepath . $filename . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED');
+						$this->throwException('Save Failed! Image couldn\'t be created', 'Save Failed! Image couldn\'t be created "' . htmlentities($filepath . $filename) . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED');
 						return false;
 					}
 				}
@@ -606,24 +614,33 @@ class slToolsImage
 					if(!$this->imagebmp($this->image,$filepath.$filename))
 					{
                         $this->afterImageManipulation();
-						$this->throwException('Save Failed! Image couldn\'t be created', 'Save Failed! Image couldn\'t be created "' . $filepath . $filename . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED');
+						$this->throwException('Save Failed! Image couldn\'t be created', 'Save Failed! Image couldn\'t be created "' . htmlentities($filepath . $filename) . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED');
 						return false;
 					}
 					
 					 				}
 				elseif($image_type==IMAGETYPE_PNG)
 				{
+					if ($compression !== -1) {
+						 						if(!imagejpeg($this->image,$filepath.'tmp'.$filename, $compression))
+						{
+							$this->afterImageManipulation();
+							$this->throwException('Save Failed! Image couldn\'t be created', 'Save Failed! Image couldn\'t be created "' . htmlentities($filepath . 'tmp' . $filename) . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED');
+							return false;
+						}
+						$this->image = imagecreatefromjpeg($filepath . 'tmp' . $filename);
+					}
 					if(!imagepng($this->image,$filepath.$filename))
 					{
                         $this->afterImageManipulation();
-						$this->throwException('Save Failed! Image couldn\'t be created', 'Save Failed! Image couldn\'t be created "' . $filepath . $filename . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED');
+						$this->throwException('Save Failed! Image couldn\'t be created', 'Save Failed! Image couldn\'t be created "' . htmlentities($filepath . $filename) . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED');
 						return false;
 					}
 				}
 				else
 				{
                     $this->afterImageManipulation();
-					$this->throwException('Save Failed! Unsupported image type "' . $this->resolveImageType($image_type) . '".', 'Save Failed! Unsupported image type "' . $this->resolveImageType($image_type) . '" in "' . $filepath . $filename . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED_UNSUPPORTED_TYPE');
+					$this->throwException('Save Failed! Unsupported image type "' . $this->resolveImageType($image_type) . '".', 'Save Failed! Unsupported image type "' . $this->resolveImageType($image_type) . '" in "' . htmlentities($filepath . $filename) . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED_UNSUPPORTED_TYPE');
 					return false;
 				}
                 $this->afterImageManipulation();
@@ -636,13 +653,12 @@ class slToolsImage
 					{
 						if(!copy($this->getFilePath().$this->getFileName(),$filepath.$filename))
 						{
-							$this->throwException('Save Failed! Copy failed.', 'Save Failed! Copy failed "' . $this->getFilePath() . $this->getFileName() . '" to "' . $filepath . $filename . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED_UNSUPPORTED_TYPE');
+							$this->throwException('Save Failed! Copy failed.', 'Save Failed! Copy failed "' . $this->getFilePath() . $this->getFileName() . '" to "' . htmlentities($filepath . $filename) . '" [Method:"' . __METHOD__ . '"]', 'SAVE_FAILED_UNSUPPORTED_TYPE');
 							return false;
 						}
 					}
 				}
 			}
-
 			return $filename;
 		}
 		else
@@ -680,7 +696,7 @@ class slToolsImage
 
 	public function getWidth()
 	{
-		if(!$this->useCache && is_resource($this->image))
+		if(!$this->useCache && is_object($this->image))
 		{
 			return imagesx($this->image);
 		}
@@ -692,7 +708,7 @@ class slToolsImage
 	
 	public function getHeight()
 	{
-		if(!$this->useCache && is_resource($this->image))
+		if(!$this->useCache && is_object($this->image))
 		{
 			return imagesy($this->image);
 		}
@@ -730,10 +746,10 @@ class slToolsImage
 	
 	public function resizeToWidth($width)
 	{
-		$width=$this->clearSize($width);
+		$width = $this->clearSize($width);
 		if(!$this->useCache)
 		{
-			$ratio=$width / $this->getWidth();
+			$ratio = $width / $this->getWidth();
 			$height=$this->getheight() * $ratio;
 			if(!$this->ignoreImageSize && $width>=$this->getWidth() && $height>=$this->getHeight())
 			{
@@ -843,7 +859,7 @@ class slToolsImage
 		if(!$this->useCache)
 		{
             $this->beforeImageManipulation();
-			if($new_image=@imagecreatetruecolor($width,$height))
+			if($new_image=imagecreatetruecolor($width,$height))
 			{
 				$trnprt_indx = imagecolortransparent($this->image);
 				 				if ($trnprt_indx >= 0) {
@@ -860,7 +876,7 @@ class slToolsImage
 					imagefilledrectangle($new_image, 0, 0, $width, $height, $transparent);
 				}
 				
-				if(@imagecopyresampled($new_image,$this->image,0,0,0,0,$width,$height,$this->getWidth(),$this->getHeight()))
+				if(imagecopyresampled($new_image,$this->image,0,0,0,0,$width,$height,$this->getWidth(),$this->getHeight()))
 				{
 					$this->image=$new_image;
                     $this->afterImageManipulation();
@@ -898,7 +914,7 @@ class slToolsImage
 			
 			if($position)
 			{
-				if(is_string($position))
+				if(is_string($position) && $position !== 'true')
 				{
 					$left=0;
 					$top=0;
@@ -971,8 +987,10 @@ class slToolsImage
 		}
 	}
 	
-	public function edit($width=false,$height=false,$crop=false,$fitDefinedBoxInside=false)
+	public function edit($width=false,$height=false,$crop=false,$fitDefinedBoxInside=false, $forcedImageType = false)
 	{
+		$this->forcedImageType = $forcedImageType;
+
 		if($width && !$height)
 		{
 			return $this->resizeToWidth($width);
