@@ -65,7 +65,11 @@ class SearchTool
         'beforeend',
         'taglist',
         'simplefulltext',
+        'evncomevnid',
         'rid',
+		'last',
+		'with',
+		'message-id',
 	);
 	static $operators = array(
 		'creationdate'=>'=',
@@ -82,6 +86,14 @@ class SearchTool
 			'header_cc',
 			'header_bcc',
 			'header_sms'
+		),
+		'SNOOZED'=>array(
+			'header_from',
+			'header_to',
+			'subject',
+			'header_cc',
+			'header_bcc',
+			'header_sms'
 		),
 		 		'C'=>array(
 			'ItmTitle',
@@ -182,21 +194,13 @@ class SearchTool
     public $account;
     public $filter;
 
-	 
-	public function parse($condition, &$filter = array())
+	public function __construct(&$filter = false)
 	{
-		if(!$this->type){
-			throw new Exception('search_type_undefined');
-		}
 		$this->configTableName($_SESSION['DBTYPE']);
 		$this->dbtype = $_SESSION['DBTYPE'];
 		$this->filter = &$filter;
-
-		$tokens = $this->tokenizeCondition($condition);  		
-		$this->prepareTokensToSql($tokens);  
-		$result = $this->processTokens($tokens);  		return $result;
 	}
-	
+
 	public function setFolder($oFolder)
 	{
 		$this->oFolder = $oFolder;
@@ -209,16 +213,10 @@ class SearchTool
 	
 	public function configTableName($dbType)
 	{
+		$this->dbtype = $dbType;
 		$this->setTableName(false);
-		if($this->oFolder 
-		&& $this->oFolder->type=='V'
-		&& $this->oFolder->getType()=='M'
-		&& $this->oFolder->isEmpty()){
-			switch(strtoupper($dbType)){
-				case 'MYSQL':
-				$this->setTableName('i');
-				break;
-			}
+		if($this->oFolder && $this->oFolder->type=='V' && $this->oFolder->getType()=='M' && $this->oFolder->isEmpty()){
+            if (strtoupper($dbType) == 'MYSQL') $this->setTableName('i');
 		}
 	}
 	
@@ -252,378 +250,22 @@ class SearchTool
 		return $this->account;
 	}
 
-	 
-	private function tokenizeCondition($condition) {
-		$characterLimit = 2048;
-		$tokensLimit = 100;
-		$condition = trim($condition);
-		if (strlen($condition) > $characterLimit) {
-			throw new Exception('Character limit has been exceeded (' . $characterLimit . ').');
-		}
-
-		 		 		 		 		 		$return = preg_match_all('/([\(\-]*".*?[^\\\\"]"[\)]*)|([\(\-]*[\w]+?:[\w\@\.\/\=]*[\)]*?)|(\S+)/u', $condition, $tokens);
-		 
-		if (false === $return) {
-			return array();
-		}
-
-		$tokens = $tokens[0];
-
-		 		 		$loop = 0;
-		for ($i = 0; $i < count($tokens); $i++) {
-			$loop++;
-			if ($loop > $tokensLimit) {
-				throw new Exception('Search words limit has been exceeded (' . $tokensLimit . ').');
-			}
-
-			$token = $tokens[$i];
-			$firstChar = substr($token, 0, 1);
-			$lastChar = substr($token, -1);
-
-			if ('-' === $firstChar && strlen($token) > 1) {
-				 				if (isset($tokens[$i - 1]) && 1 === strlen($tokens[$i - 1]) && '-' === substr($tokens[$i - 1], 0, 1)) {
-					continue;
-				}
-				$tokens[$i] = substr($token, 1);
-				array_splice($tokens, $i, 0, $firstChar);
-				$i = 0;  				continue;
-			}
-
-			if ('(' === $firstChar && strlen($token) > 1) {
-				$tokens[$i] = substr($token, 1);
-				array_splice($tokens, $i, 0, $firstChar);
-				$i = 0;  				continue;
-			}
-
-			if (')' === $lastChar && strlen($token) > 1 && false === strpos($token, '(')) {
-				$tokens[$i] = substr($token, 0, -1);
-				array_splice($tokens, $i + 1, 0, $lastChar);
-				$i = -1;  				continue;
-			}
-
-			 		}
-
-		 		$loop = 0;
-		for ($i = 0; $i < count($tokens); $i++) {
-			$loop++;
-			if ($loop > $tokensLimit) {
-				throw new Exception('Search words limit has been exceeded (' . $tokensLimit . ').');
-			}
-
-			$token = $tokens[$i];
-			$lastChar = substr($token, -1);
-
-			 			if (':' === $lastChar && strlen($token) > 1 && isset($tokens[$i + 1])) {
-				$nextToken = $tokens[$i + 1];
-				if ($nextToken !== '-' && $nextToken !== ')' && $nextToken !== '(' && strlen($nextToken) > 0) {
-					$tokens[$i] = $token . $nextToken;
-					array_splice($tokens, $i + 1, 1);
-					$i = -1;  					continue;
-				}
-			}
-
-		}
-
-		$this->validateTokens($tokens);
-
-		return $tokens;
-	}
-
-	 
-	private function separateContentInBracketsToString($tokens) {
-		$cnt = count($tokens);
-		$openingIndex = false;
-		$closingIndex = false;
-		 		for ($i = $cnt - 1; $i >= 0; $i--) {
-			if ('(' === $tokens[$i]) {
-				$openingIndex = $i;
-				break;
-			}
-		}
-		for ($i = $openingIndex + 1; $i < $cnt; $i++) {
-			if (')' === $tokens[$i]) {
-				$closingIndex = $i;
-				break;
-			}
-		}
-
-		if ($openingIndex !== false && $closingIndex !== false) {
-			$offset = $openingIndex + 1;
-			$length = $closingIndex - ($openingIndex + 1);
-			$tokensSliced = array_slice($tokens, $offset, $length);
-			$tokensBefore = array_slice($tokens, 0, $openingIndex);
-			$tokensAfter = array_slice($tokens, $closingIndex + 1);
-			if (count($tokensSliced) < 1) {
-				$lastToken = $tokensBefore[count($tokensBefore) - 1];
-				if (':' === substr($lastToken, -1)) {
-					return array_merge($tokensBefore, array('AND', '()'), $tokensAfter);
-				}
-			}
-			$tokensSlicedSimplified = $this->prepareTokensToSqlSimple($tokensSliced);
-			return array_merge($tokensBefore, array(':::::' . json_encode($tokensSlicedSimplified)), $tokensAfter);
-		}
-
-		return '';
-	}
-
-	 
-	private function prepareTokensToSql(&$tokens) {
-		$cnt = count($tokens);
-		if ($cnt < 1) {
-			return;  		}
-
-		$openingBracketIndexes = [];
-		for ($i = $cnt - 1; $i >= 0; $i--) {
-			if ('(' === $tokens[$i]) {
-				$openingBracketIndexes[] = $i;
-			}
-		}
-		$cnt = count($openingBracketIndexes);
-		if ($cnt < 1) {
-			$tokens = $this->prepareTokensToSqlSimple($tokens);  			return;
-		}
-
-		 		$cnt = count($openingBracketIndexes) + 1;
-		for ($i = 0; $i < $cnt; $i++) {
-
-			$openingBracketCount = 0;
-			foreach ($tokens as $token) {
-				if ('(' === $token) {
-					$openingBracketCount++;
-				}
-			}
-
-			 			if ('(' === $tokens[0] && ')' === $tokens[count($tokens) - 1] && 1 === $openingBracketCount) {
-				$tokensToProcess = array_slice($tokens, 1, -1);
-				if (0 === count($tokensToProcess)) {
-					$tokens = array('(', ')');
-				}
-				else {
-					$tokens = array_merge(array('('), $this->prepareTokensToSqlSimple($tokensToProcess), array(')'));
-				}
-			}
-			 			elseif ($openingBracketCount > 0) {
-				$tokens = $this->separateContentInBracketsToString($tokens);
-			}
-			 			elseif (0 === $openingBracketCount) {
-				$tokens = $this->prepareTokensToSqlSimple($tokens);
-			}
-		}
-
-		 		for ($i = 0; $i < count($tokens); $i++) {
-			$token = $tokens[$i];
-			if (':::::' === substr($token, 0, 5)) {
-				$tokensDecoded = json_decode(substr($token, 5));
-				if (JSON_ERROR_NONE === json_last_error() && is_array($tokensDecoded)) {
-					array_unshift($tokensDecoded, '(');
-					$tokensDecoded[] = ')';
-				}
-				array_splice($tokens, $i, 1, $tokensDecoded);
-				continue;
-			}
-		}
-
-		 	}
-
-	 
-	 private function processTokens($tokens)
-     {
-		$logicOperators = array('AND', 'OR', 'NOT');
-		$bracketOperators = array('(', ')');
-		$allOperators = array_merge($logicOperators, $bracketOperators);
-		$resultArray = array();
-		if (isset($tokens[0]) && '(' === $tokens[0] && ')' === $tokens[count($tokens) - 1]) {
-			array_pop($tokens);
-			array_shift($tokens);
-		}
-
-		for ($i = 0; $i < count($tokens); $i++) {
-			$token = $tokens[$i];
-			if (in_array($token, $allOperators)) {
-				$resultArray[] = $token;
-				continue;
-			}
-			$isSemicolon = false;
-			$isSlicedTokens = false;
-			if (strpos($token, ':') !== false) {
-				$parts = explode(':', $token);
-				if (in_array(strtolower($parts[0]), self::$knownCommands)) {
-					$isSemicolon = true;
-					$nextToken = isset($tokens[$i + 1]) ? $tokens[$i + 1] : false;
-					if ($nextToken && '(' === $nextToken) {
-						$closingBracketIndex = false;
-						for ($j = $i + 1; $j < count($tokens); $j++) {
-							if (')' === $tokens[$j]) {
-								$closingBracketIndex = $j;
-								break;
-							}
-						}
-						if ($closingBracketIndex) {
-							$slicedTokens = array_slice($tokens, $i + 1, $closingBracketIndex - $i);
-							$isSlicedTokens = true;
-							$resultArray[] = $this->processFunction(false, array_merge(array($token), $slicedTokens));
-							$i = $closingBracketIndex;
-							continue;
-						}
-					}
-				}
-				if (!$isSlicedTokens) {
-					$resultArray[] = $this->processFunction($token);
-					continue;
-				}
-			}
-			if (!$isSemicolon) {
-                $token = trim($token,'"');
-                if(strlen($token) >= 3 && $this->type != 'Z'){
-                    $prefix = 'fulltext:';
-                    if($this->type == 'M'){
-                        $resultArray[] = $this->processFunction($token);
-                        $resultArray[] = 'OR';
-                    }else{
-                        $prefix = 'simple'. $prefix;
-                    }
-                    $token = $prefix . $token;
-                }
-                $resultArray[] = $this->processFunction($token);
-			}
-		}
-		return implode(' ', $resultArray);
-	}
-
-	 
-	private function prepareTokensToSqlSimple($tokens) {
-		$logicOperators = array('AND', 'OR');
-		$negationOperators = array('NOT', '-');
-		$allOperators = array_merge($logicOperators, $negationOperators);
-
-		 		for ($i = 0; $i < count($tokens); $i++) {
-			$tokenPrev = isset($tokens[$i - 1]) ? $tokens[$i - 1] : false;
-			$tokenCurrent = $tokens[$i];
-			$tokenNext = isset($tokens[$i + 1]) ? $tokens[$i + 1] : false;
-			$tokenPrevUp = $tokenPrev ? strtoupper($tokenPrev) : false;
-			$tokenCurrentUp = $tokenCurrent ? strtoupper($tokenCurrent) : false;
-			$tokenNextUp = $tokenNext ? strtoupper($tokenNext) : false;
-
-			 			if (false === $tokenPrev &&
-			    $tokenCurrent !== false && true === in_array($tokenCurrentUp, $logicOperators)) {
-				$tokens[$i] = '"' . $tokenCurrent . '"';
-			}
-			 			elseif ($tokenCurrent !== false && true === in_array($tokenCurrentUp, $logicOperators) &&
-			        false === $tokenNext) {
-				$tokens[$i] = '"' . $tokenCurrent . '"';
-			}
-			 			elseif ($tokenCurrent !== false && true === in_array($tokenCurrentUp, $negationOperators) &&
-			        false === $tokenNext) {
-				$tokens[$i] = '"' . $tokenCurrent . '"';
-			}
-			 			elseif ($tokenPrev !== false && true === in_array($tokenPrevUp, $negationOperators) &&
-			        $tokenCurrent !== false && true === in_array($tokenCurrentUp, $negationOperators)) {
-				$tokens[$i] = '"' . $tokenCurrent . '"';
-			}
-		}
-
-		 		for ($i = 0; $i < count($tokens); $i++) {
-			$tokenPrev = isset($tokens[$i - 1]) ? $tokens[$i - 1] : false;
-			$tokenCurrent = $tokens[$i];
-			$tokenNext = isset($tokens[$i + 1]) ? $tokens[$i + 1] : false;
-			$tokenNextNext = isset($tokens[$i + 2]) ? $tokens[$i + 2] : false;
-			$tokenNextNextNext = isset($tokens[$i + 3]) ? $tokens[$i + 3] : false;
-			$tokenPrevUp = $tokenPrev ? strtoupper($tokenPrev) : false;
-			$tokenPrevLow = $tokenPrev ? strtolower($tokenPrev) : false;
-			$tokenCurrentUp = $tokenCurrent ? strtoupper($tokenCurrent) : false;
-			$tokenNextUp = $tokenNext ? strtoupper($tokenNext) : false;
-			$tokenNextNextUp = $tokenNextNext ? strtoupper($tokenNextNext) : false;
-			$tokenNextNextNextUp = $tokenNextNextNext ? strtoupper($tokenNextNextNext) : false;
-			$tokenPrevIsKeyword = ':' === substr($tokenPrev, -1, 1) && in_array(substr($tokenPrevLow, 0, -1), self::$knownCommands) !== false;
-
-			 			if ($tokenPrev !== false && $tokenPrevIsKeyword &&
-			    $tokenCurrent !== false && false === in_array($tokenCurrentUp, $allOperators)) {
-				 			}
-			 			elseif ($tokenPrev !== false && $tokenPrevIsKeyword &&
-			        $tokenCurrent !== false && true === in_array($tokenCurrentUp, $negationOperators) &&
-			        $tokenNext !== false && false === in_array($tokenNextUp, $allOperators)) {
-				 			}
-			 			elseif ($tokenPrev !== false && false === in_array($tokenPrevUp, $allOperators) &&
-			        $tokenCurrent !== false && false === in_array($tokenCurrentUp, $allOperators)) {
-				array_splice($tokens, $i, 0, 'AND');
-			}
-			 			elseif ($tokenPrev !== false && true === in_array($tokenPrevUp, $negationOperators) &&
-			        $tokenCurrent !== false && false === in_array($tokenCurrentUp, $negationOperators) &&
-			        false === $tokenNext) {
-				$tokens[$i - 1] = 'NOT';
-			}
-			 			elseif (false === $tokenPrev &&
-			        $tokenCurrent !== false && true === in_array($tokenCurrentUp, $negationOperators) &&
-			        $tokenNext !== false && false === in_array($tokenNextUp, $allOperators)) {
-				$tokens[$i] = 'NOT';
-			}
-			 			elseif ($tokenPrev !== false && false === in_array($tokenPrevUp, $allOperators) &&
-			        $tokenCurrent !== false && true === in_array($tokenCurrentUp, $negationOperators) &&
-			        $tokenNext !== false && false === in_array($tokenNextUp, $allOperators)) {
-				$tokens[$i] = 'NOT';
-				array_splice($tokens, $i, 0, 'AND');
-			}
-			 			elseif ($tokenPrev !== false && false === in_array($tokenPrevUp, $allOperators) &&
-			        $tokenCurrent !== false && true === in_array($tokenCurrentUp, $logicOperators) &&
-			        $tokenNext !== false && false === in_array($tokenNextUp, $allOperators)) {
-				$tokens[$i] = $tokenCurrentUp;
-			}
-			 			elseif ($tokenPrev !== false && false === in_array($tokenPrevUp, $allOperators) &&
-			        $tokenCurrent !== false && true === in_array($tokenCurrentUp, $logicOperators) &&
-			        $tokenNext !== false && true === in_array($tokenNextUp, $negationOperators) &&
-			        $tokenNextNext !== false && false === in_array($tokenNextNextUp, $allOperators)) {
-				$tokens[$i] = $tokenCurrentUp;
-				$tokens[$i + 1] = 'NOT';
-			}
-			 			elseif ($tokenPrev !== false && true === in_array($tokenPrevUp, $logicOperators) &&
-			        $tokenCurrent !== false && true === in_array($tokenCurrentUp, $logicOperators) &&
-			        $tokenNext !== false && true === in_array($tokenNextUp, $logicOperators)) {
-				$tokens[$i - 1] = '"' . $tokenPrev . '"';
-				$tokens[$i + 1] = '"' . $tokenNext . '"';
-				$tokens[$i] = $tokenCurrentUp;
-			}
-			 			elseif ($tokenPrev !== false && false === in_array($tokenPrevUp, $allOperators) &&
-			        $tokenCurrent !== false && true === in_array($tokenCurrentUp, $logicOperators) &&
-			        $tokenNext !== false && true === in_array($tokenNextUp, $negationOperators) &&
-			        $tokenNextNext !== false && true === in_array($tokenNextNextUp, $logicOperators) &&
-			        $tokenNextNextNext !== false && false === in_array($tokenNextNextNextUp, $allOperators)) {
-				$tokens[$i] = $tokenCurrentUp;
-				$tokens[$i + 1] = 'NOT';
-				array_splice($tokens, $i + 3, 0, 'AND');
-			}
-		}
-
-		return $tokens;
-	}
-
-	 
-	private function validateTokens(&$tokens) {
-		$bracketsCounter = 0;
-		for ($i = 0; $i < count($tokens); $i++) {
-			$token = $tokens[$i];
-			if ('(' === $token) {
-				$bracketsCounter++;
-			}
-			if (')' === $token) {
-				if ($bracketsCounter > 0) {
-					$bracketsCounter--;
-				}
-				else {
-					$tokens[$i] = '"' . $token . '"';  				}
-			}
-		}
-		if ($bracketsCounter > 0) {
-			for ($i = count($tokens) - 1; $i >= 0 ; $i--) {
-				if (0 === $bracketsCounter) {
-					break;
-				}
-				$token = $tokens[$i];
-				if ('(' === $token && $bracketsCounter > 0) {
-					$tokens[$i] = '"' . $token . '"';  					$bracketsCounter--;
-				}
-			}
-		}
+	public function termToSql($term)
+	{
+		$keyword = strtolower($term['keyword']);
+		$value = strtolower($term['value']);
+		return $this->processFunction($keyword, $value);
 	}
+	
+	static public function urlEnquoteEscape($str)
+    {
+        return slToolsString::urlenquote($str);
+    }
+	
+	static public function urlEnquoteEscapeTag($str)
+    {
+        return str_replace('+', "'20", slToolsString::urlenquote($str));
+    }
 
 	private function escapeWildcards($value)
 	{
@@ -638,11 +280,6 @@ class SearchTool
 			     				$value = icewarp_escape_db_string($value,'antispam');
 				break;
 			default:
-				if($this->type=='M'){
-					if($this->dbtype=='MYSQL' &&!$exact_match){
-						$value = preg_replace('/\\\\(?!(_|%|\[|\]))/si','\\\\\\\\',$value);
-					}
-				}
                 $value = str_replace("'","''",$value);
 				break;
 		}
@@ -650,70 +287,22 @@ class SearchTool
 	}
 	
 	 
-	private function processValue($str_value ,$arr_value, $condition, $count, $value_callback = array(), $condition_callback = array(), $exact_match = false)
+	private function processValue($value , $condition, $count, $value_callback = array(), $condition_callback = array(), $exact_match = false)
 	{	
-		if($str_value || trim($str_value,'"')==='0' || trim($str_value,"'")==='0'){
-			$str_value = self::escapeValue($str_value, $exact_match);
-			if(strpos($condition,'{ESCAPE \'\\\'}')!==false){
-				$str_value = preg_replace('/\\\\(?!(_|%|\[|\]))/si','\\\\\\\\',$str_value);
-			}
-			$str_value = $this->executeCallback($str_value, $value_callback);
-			if($str_value || trim($str_value,'"')==='0' || trim($str_value,"'")==='0'){
-				if($count > 1){
-					$condition = vsprintf($condition,array_fill(0,$count,$str_value));
-				}else{
-					$condition = sprintf($condition,$str_value);
-				}			
-			}
-			
-			$condition = $this->executeCallback($str_value, $condition_callback,$condition);
-		}else if($arr_value){
-			$result = '(';
-			foreach($arr_value as $val){
-				switch(strtoupper(trim($val))){
-					case 'NOT':
-						$result.= ' NOT ';
-						break;
-					case '(':
-						$result.=' (';
-						break;
-					case ')':
-						$result.=') ';
-						break;
-					break;
-					case 'AND':
-					case 'OR':
-						$result.=' '.strtoupper($val).' ';
-						break;
-					default:
-						if(($val[0]=='"' || $val[0]=="'") && (strlen($val) > 1)){
-							$val = substr($val,1,strlen($val)-2);
-							$val = str_replace(array('\\\\','\\"'),array('\\','"'),$val);  
-						}
-						$val = self::escapeValue($val, $exact_match);
-						if(strpos($condition,'{ESCAPE \'\\\'}')!==false){
-							$val = preg_replace('/\\\\(?!(_|%|\[|\]))/si','\\\\\\\\\\\\\\\\',$val);
-						}	
-						$val = $this->executeCallback($val, $value_callback);
-						if(trim($val) || $val===0){
-							$c = '';
-							if($count > 1){
-								$c = vsprintf($condition,array_fill(0,$count,$val));
-							}else{
-								
-								$c = sprintf($condition,$val);
-							}
-						}
-						$c = $this->executeCallback($val, $condition_callback, $c);
-						$result.=$c;
-						break;
-				}
-			}
-			$result.=')';
-			$condition = $result;
-		}else{
-			$condition = '';
+		 		$value = slToolsString::removeQuotes($value);
+		
+		 		$value = $this->escapeValue($value, $exact_match);  		
+		 		$value = $this->executeCallback($value, $value_callback);
+		
+		 		if($value || trim($value,'"')==='0' || trim($value,"'")==='0'){
+			if($count > 1){
+				$condition = vsprintf($condition,array_fill(0,$count,$value));
+			}elseif ($count!=0){
+				$condition = sprintf($condition,$value);
+			}			
 		}
+		$condition = $this->executeCallback($value, $condition_callback,$condition);
+
 		return $condition;
 	}
 	
@@ -737,38 +326,9 @@ class SearchTool
 	}
 	
 	 
-	private function processFunction($str_token = '',$arr_token = array())
-	{
-        $str_value = $value_callback = $condition_callback = $arr_value = $condition = $command = null;
-		$escape = true;
-		if($str_token){
-			if(($str_token[0]=='"' || $str_token[0]=="'") && (strlen($str_token) > 1)){
-				$str_value = substr($str_token,1,strlen($str_token)-2);
-			}else{
-				if(strpos($str_token,':')!==false){
-					$info = explode(':',$str_token,2);
-					if(in_array(strtolower($info[0]),self::$knownCommands)){
-						$str_value = $info[1];
-						$command = $info[0];
-					}else{
-						$str_value = $str_token;
-					}
-					
-				}else{
-					$str_value = $str_token;
-				}
-				if(($str_value[0]=='"' || $str_value[0]=="'") && (strlen($str_value) > 1)){
-					$str_value = substr($str_value,1,strlen($str_value)-2);
-				}
-			}
-		}
-		if($arr_token){
-			$command = substr($arr_token[0],0,strlen($arr_token[0])-1);
-			$arr_value = $arr_token;
-			array_shift($arr_value);
-		}
-		
-		$complete = false;  		$exact_match = false;  		$count = 1;  		$table = $this->getTableName();
+	private function processFunction($command, $value)
+	{		
+		$exact_match = false;  		$count = 1;  		$table = $this->getTableName();
 		 		switch(strtolower($command)){
 			 			case 'from':
 				switch($this->type){
@@ -776,6 +336,7 @@ class SearchTool
 					case 'QL':
 						$condition = "(".$table."SndEmail IS NOT NULL AND ".$table."SndEmail LIKE '%%%s%%' {ESCAPE '\\'})";
 						break;
+					case 'SNOOZED':
 					case 'M':
 						$condition = "(".$table."header_from IS NOT NULL AND ".$table."header_from LIKE '%%%s%%' {ESCAPE '\\'})";
 						break;
@@ -794,6 +355,7 @@ class SearchTool
 					case 'QL':
 						$condition = "(SndOwner IS NOT NULL AND SndOwner LIKE '%%%s%%' {ESCAPE '\\'})";
 						break;
+					case 'SNOOZED':
 					case 'M':
 						$condition = "(".$table."header_to IS NOT NULL AND ".$table."header_to LIKE '%%%s%%' {ESCAPE '\\'})";
 						break;
@@ -804,6 +366,7 @@ class SearchTool
 					case 'Q':
 						$condition = "(SndSubject IS NOT NULL AND SndSubject LIKE '%%%s%%' {ESCAPE '\\'})";
 						break;
+					case 'SNOOZED':
 					case 'M':
 						$condition = "(".$table."subject IS NOT NULL AND ".$table."subject LIKE '%%%s%%' {ESCAPE '\\'})";
 						break;
@@ -830,18 +393,33 @@ class SearchTool
 					'class'=>&$this,
 					'func'=>'colorCondition'
 				);				
+			break;
+			case 'last':
+				$condition_callback = array(
+					'class'=>&$this,
+					'func'=>'lastCondition'
+				);	
+			break;
+			case 'with':
+				$condition = "(".  
+				"(".$table."header_from IS NOT NULL AND ".$table."header_from LIKE '%%%s%%' {ESCAPE '\\'})".
+				" OR (".$table."header_to IS NOT NULL AND ".$table."header_to LIKE '%%%s%%' {ESCAPE '\\'})".
+				" OR (".$table."header_cc IS NOT NULL AND ".$table."header_cc LIKE '%%%s%%' {ESCAPE '\\'})".
+				")";
+				$count = 3;
 			break;
 			 			case 'note':
 			case 'description':
-				if($this->type == 'C'){
+				if($this->type=='C'){
 					$condition = "(ItmDescription IS NOT NULL AND ItmDescription LIKE '%%%s%%' {ESCAPE '\\'})";
-				}elseif($this->type == 'N'){
-                    $condition = "{ITEMATTR}LIKE '%%%s%%'{/ITEMATTR}";
-                }else{
+				}else{
 					$condition = "(EvnNote IS NOT NULL AND EvnNote LIKE '%%%s%%' {ESCAPE '\\'})";
 				}
 				break;
 			break;
+            case 'evncomevnid':
+                $condition = "(evncomevnid IS NOT NULL AND evncomevnid LIKE '%%%s%%' {ESCAPE '\\'})";
+                break;
             case 'rid':
                 if(in_array($this->type,['C', 'G'])){
                     $condition = "(ItmRID = '%s')";
@@ -850,30 +428,8 @@ class SearchTool
                 }
                 break;
 			case 'gchat':
-				switch($str_value){
-					case 'main':
-						$this->filter['groupchat_type'] = 'main';
-						$condition = "(EvnClass = 'Z' Or EvnClass = 'I' OR EvnClass='Q' OR EvnClass='R' OR EvnClass='D' Or EvnClass='S' Or EvnClass='W' Or EvnClass='B' Or EvnClass='Y')";
-						break;
-					case 'files':
-						$this->filter['groupchat_type'] = 'files';
-						$condition = "(EvnClass='F' OR EvnClass='M')";
-						break;
-					case 'events':
-						$this->filter['groupchat_type'] = 'events';
-						$condition = "(EvnClass='E' Or EvnClass='O' Or EvnClass='V')";
-						break;
-					case 'comments':
-						$this->filter['groupchat_type'] = 'comments';
-						$condition = "(EvnClass = 'Z' Or EvnClass = 'I' OR EvnClass='Q' OR EvnClass='R' OR EvnClass='D' Or EvnClass='S' Or EvnClass='W' Or EvnClass='Y')";						
-						break;
-					case 'global_pins':
-					case 'my_pins':
-					case 'mentions':
-						$this->filter['groupchat_type'] = trim($str_value);
-						$condition = "(EvnClass = 'Z' Or EvnClass = 'I' OR EvnClass='Q' OR EvnClass='R' OR EvnClass='D' Or EvnClass='S' Or EvnClass='W' Or EvnClass='Y')";
-						break;
-				}
+                $this->filter['groupchat_type'] = trim($value);
+                $condition = "1=1";
 				break;
 			case 'title':
 				switch($this->type){
@@ -899,7 +455,7 @@ class SearchTool
 						$condition = "(header_from IS NOT NULL AND header_from LIKE '%%%s%%' {ESCAPE '\\'})";
 						break;
 					case 'Z':
-						$this->filter['name'] = $str_value;
+						$this->filter['name'] = $value;
 						 
 						break;
 				}
@@ -907,7 +463,7 @@ class SearchTool
 			case 'everywhere':
 				switch($this->type)	{
 					case 'Z':
-						$this->filter['everywhere'] = $str_value;
+						$this->filter['everywhere'] = $value;
 						break;
 					case 'I':						
 						$condition_callback = array(
@@ -926,7 +482,7 @@ class SearchTool
 						$count = 3;
 						break;
 					case 'Z':
-						$this->filter['email'] = $str_value;
+						$this->filter['email'] = $value;
 						break;
 					case 'I':
 						$condition_callback = array(
@@ -948,7 +504,7 @@ class SearchTool
 						$count = 1;
 						break;
 					case 'Z':
-						$this->filter['email'] = $str_value;
+						$this->filter['email'] = $value;
 						break;
 					case 'I':
 						$condition_callback = array(
@@ -982,6 +538,7 @@ class SearchTool
 			 			case 'greater':
 				switch($this->type){
 					case 'M':
+					case 'SNOOZED':
 						$condition = $this->getTableName().'size > %s';
 						$value_callback = array(
 							'class'=>&$this,
@@ -1001,6 +558,7 @@ class SearchTool
 			case 'smaller':
 				switch($this->type){
 					case 'M':
+					case 'SNOOZED':
 						$condition = $this->getTableName().'size < %s';
 						$value_callback = array(
 							'class'=>&$this,
@@ -1019,45 +577,31 @@ class SearchTool
 			break;
 
 			case 'keyword':			case 'tag':
-				
 				switch($this->type){
 					case 'Q':
 					case 'QL':
 						break;
+					case 'SNOOZED':
 					case 'M':
-						$condition = "(taglist IS NOT NULL and taglist LIKE '%%%s%%' {ESCAPE '\\'})";
-						break;
-					case 'K':
-							$condition = "(tagname IS NOT NULL and tagname LIKE '%%%s%%' {ESCAPE '\\'})";
+							$condition = $this->getTagListCondition($value);
+							$count = 0;
 							break;
+					case 'K':
+						$condition = "(tagname IS NOT NULL AND tagname LIKE \"%%%s%%\" {ESCAPE '\\'})";
+						break;
 					default:
-						$condition = "{TAG}%s{/TAG}";
+						$condition = "";
 						$exact_match = true;
 						break;
 				}
 				break;
             case 'taglist':
-                $condition = "(taglist IS NOT NULL and taglist LIKE \"%%%s%%\" {ESCAPE '\\'})";
-                $value_callback = ['class' => &$this, 'func' => 'urlEnquoteEscape'];
+                $condition = $this->getTagListCondition($value);
+				$count = 0;
 				break;
 			case 'fulltext':
-			    if($this->type == 'M'){
-                    $condition = "{FULLTEXT}%s{/FULLTEXT}";
-                    $exact_match = true;
-                }elseif(in_array($this->type, ['F','C','E','N','T','I','K','Q','QL'])){
-                    $conditions = [];
-                    foreach(self::$folderDefaults[$this->type] as $field){
-                        $conditions[] = "(".$this->getTableName().$field." IS NOT NULL AND ".$this->getTableName().$field." LIKE '%%%s%%' {ESCAPE '\\'})";
-                    }
-                    $condition = join(' OR ',$conditions);
-                    $condition .= ' OR {FULLTEXT}%s{/FULLTEXT}';
-                    $condition .= " OR {ITEMATTR}LIKE '%%%s%%'{/ITEMATTR}";
-                    $condition = '('.$condition.')';
-                    $count = count(self::$folderDefaults[$this->type]) + 2;
-                }
-			    break;
             case 'simplefulltext':
-                if(in_array($this->type, ['F','C','E','N','T','I','K','Q','QL'])){
+                if(in_array($this->type, ['M','F','C','E','N','T','I','K','Q','QL', 'SNOOZED'])){
                     $conditions = [];
                     foreach(self::$folderDefaults[$this->type] as $field){
                         $conditions[] = "(".$this->getTableName().$field." IS NOT NULL AND ".$this->getTableName().$field." LIKE '%%%s%%' {ESCAPE '\\'})";
@@ -1065,7 +609,9 @@ class SearchTool
                     $condition = join(' OR ',$conditions);
                     $condition = '('.$condition.')';
                     $count = count(self::$folderDefaults[$this->type]);
-                }
+                }elseif($this->type=='Z'){
+					$this->filter['everywhere'] = $value;
+				}
                 break;
 			case 'has':
 				$condition_callback = array(
@@ -1089,14 +635,14 @@ class SearchTool
                 if($this->type == 'E') {
                     $this->filter['idinterval'] = 'FUTURE';
                     $condition = null;
-                    $str_value = null;
+                    $value = null;
                 }
                 break;
             case 'beforeend':
                 if($this->type == 'E') {
                     $this->filter['idinterval'] = 'PAST';
                     $condition = null;
-                    $str_value = null;
+                    $value = null;
                 }
                 break;
 			case 'after':
@@ -1107,6 +653,7 @@ class SearchTool
 				$operator = self::$operators[strtolower($command)];
 				$function_suffix  = strtolower($command)=='before'?'before':'';
 				switch($this->type){
+					case 'SNOOZED':
 					case 'M':
 						$condition= $this->getTableName().'date '.$operator.' %s';
 						$value_callback = array(
@@ -1172,18 +719,18 @@ class SearchTool
 				break;
 			case 'afterid':
                 if($this->type == 'E' && preg_match('/(?:afterend|beforeend)/i', $this->filter['search'])){
-                    $this->filter['idinterval'] = ($this->filter['idinterval'] ?? '') . '+' . $str_value;
+                    $this->filter['idinterval'] = ($this->filter['idinterval'] ?? '') . '+' . $value;
                     $condition = null;
-                    $str_value = null;
+                    $value = null;
                 }else{
                     $condition = "(Evn_ID > '%s')";
                 }
 				break; 	
 			case 'beforeid':
                 if($this->type == 'E' && preg_match('/(?:afterend|beforeend)/i', $this->filter['search'])){
-                    $this->filter['idinterval'] = ($this->filter['idinterval'] ?? '') . '+' . $str_value;
+                    $this->filter['idinterval'] = ($this->filter['idinterval'] ?? '') . '+' . $value;
                     $condition = null;
-                    $str_value = null;
+                    $value = null;
                 }else {
                     $condition = "(Evn_ID < '%s')";
                 }
@@ -1192,7 +739,7 @@ class SearchTool
 				$condition = "(Evn_ID <= '%s')";
 				break; 
 			case 'next':
-				$this->filter['next'] = $str_value;			
+				$this->filter['next'] = $value;			
 				$condition = "(1=1)";
 				break;
 			case 'folder':
@@ -1200,16 +747,20 @@ class SearchTool
 				$exact_match = true;
 				break;
 			case 'flags':
-				$condition = "(".$table."flags IS NOT NULL AND ".$table."flags = %s)";
-				$exact_match = true;
-				break;
+				if($this->type == 'M' ){
+					$condition = "(".$table."flags IS NOT NULL AND ".$table."flags = %s)";
+					$exact_match = true;
+				}else{
+					$condition = "(1=1)";
+				}
+			break;
 			case 'class':
 				switch($this->type){
 					case 'C':
 						$condition = "(ItmClass = '%s')";
 						break;
 					case 'I':
-						$condition = "(EvnClass = '%s')";
+						$condition = "(1=1)";
 						break;
 				}
 				break;
@@ -1260,19 +811,29 @@ class SearchTool
 				);		
 				break;
 			case 'partid':
+				$draft_pos = strpos($value,'draft_sa');
+				if($draft_pos!==false){
+					$value = substr($value, $draft_pos);
+				}
 				$condition = "(EvnUID = '%s')";
 			break;
+			case 'message-id':
+				$condition = "(message_id = '%s')";
+			break;
+			 			case 'body':
+				return '';
+				break;
 			default:
-				if(stripos($str_value,'{TAG}')!==false){
-					return $str_value;
+				if(stripos($value,'{TAG}')!==false){
+					return $value;
 				}
 				$conditions = array();
 				switch($this->type){
 					case 'HIPAA':
-							return $str_value;
+							return $value;
 						break;
 					case 'Z':
-						$this->filter['everywhere'] = $str_value;
+						$this->filter['everywhere'] = $value;
 						break;
 					default:	
 						foreach(self::$folderDefaults[$this->type] as $field){
@@ -1293,9 +854,8 @@ class SearchTool
 				
 			break;
 		}
-		if(!$complete){
-			$condition = $this->processValue($str_value, $arr_value, $condition, $count, $value_callback, $condition_callback, $exact_match);
-		}
+		$condition = $this->processValue($value, $condition, $count, $value_callback, $condition_callback, $exact_match);
+
 		if($condition == ''){
 			return '1 = 1';
 		}
@@ -1305,6 +865,7 @@ class SearchTool
 	private function itemsCondition($token)
 	{
 		switch($this->type){
+			case 'SNOOZED':
 			case 'M':
 				$result = "(item_id = '".$token."' )";
 				break;
@@ -1327,6 +888,7 @@ class SearchTool
 	private function hasCondition($token)
 	{
 		switch($this->type){
+			case 'SNOOZED':
 			case 'M':
 				switch($token){
 					case 'attachment':
@@ -1416,6 +978,7 @@ class SearchTool
 			case 'done':
 				switch($this->type){
 					case 'M':
+					case 'SNOOZED':
 						$result = $this->getTableName()."color = 'Y'";
 					break;
 					case 'E':
@@ -1501,7 +1064,7 @@ class SearchTool
 			'lowest'=>5
 		);
 		
-		$priority_val = $priority[trim(strtolower(self::escapeValue($value)))];
+		$priority_val = $priority[trim(strtolower($this->escapeValue($value)))];
 		if(!$priority_val){
 			$priority_val = is_numeric($value)?$value:3;
 		}
@@ -1520,6 +1083,7 @@ class SearchTool
 		try{
 			switch($this->type)
 			{
+				case 'SNOOZED':
 				case 'M':
 					$folder = str_replace("/","\\",$folder);
 					$fdr = $oAccount->getFolder($folder);
@@ -1540,6 +1104,7 @@ class SearchTool
 	private function colorCondition($token)
 	{
 		switch($this->type){
+			case 'SNOOZED':
 			case 'M':
 				switch(strtolower($token)){
 					case 'none':
@@ -1571,6 +1136,24 @@ class SearchTool
 		}
 		return $result;
 	}
+
+	private function lastCondition($token)
+	{
+		if(!is_numeric($token)){
+			$timestamp = strtotime($token);
+		} else {
+			$daysBack = intval($token)-1;  			$date = new \DateTime();
+			$date->modify('-' . $daysBack . ' days midnight');
+			$timestamp = $date->getTimestamp();
+		}
+		switch($this->type){
+			case 'SNOOZED':
+			case 'M':
+				$result = "(date > ".$timestamp.")";
+				break;
+		}
+		return $result;		
+	}
 	
 	private function conferenceCondition($token)
 	{
@@ -1615,11 +1198,6 @@ class SearchTool
 	{
 		return self::strtotime($str)+86400;
 	}
-	
-	protected function urlEnquoteEscape($str)
-    {
-        return slToolsString::urlenquote($str);
-    }
 
 
 	private function kbtob($str)
@@ -1660,7 +1238,17 @@ class SearchTool
 		}
 		return $condition;		
 	}
-	
+
+	protected function getTagListCondition($value)
+	{
+		$value = slToolsString::removeQuotes($value);
+		$tag_name = $this->getTableName().'taglist';
+		$condition = "(".$tag_name." IS NOT NULL AND (".$tag_name." LIKE \"%%%s%%\" {ESCAPE '\\'} OR ".$tag_name." LIKE \"%%%s%%\" {ESCAPE '\\'}))";
+		$urlenquoteValue = self::urlEnquoteEscape($value);
+		$urlenquoteTagValue = self::urlEnquoteEscapeTag($value);
+		$condition = sprintf($condition, $urlenquoteValue, $urlenquoteTagValue);
+		return $condition;				
+	}
 	
 	protected function teamChatTimeCondition($value,$condition,$type)
 	{
@@ -1690,16 +1278,13 @@ class SearchTool
 					$timeCondition = '(Evn_Created '.$operator.' '.$dtNow->getTimestamp().')';
 				}
 				if($type=='before' || $type=='after'){
-					$eventCondition = "((EvnClass = 'E') AND (EvnStartDate ".$operator." ".unixtojd($dtNow->getTimestamp())." ))";
-					$teamchatCondition = "((EvnClass = 'I' OR EvnClass='Q' OR EvnClass='R' OR EvnClass='D' Or EvnClass='S' Or EvnClass='W' Or EvnClass='B' Or EvnClass='F') AND ".$timeCondition.")";
-					$timeCondition = '('.$timeCondition.' OR '.$eventCondition.')';
+					$eventCondition = "(EvnStartDate ".$operator." ".unixtojd($dtNow->getTimestamp())." )";
+					$timeCondition = ''.$timeCondition.' OR '.$eventCondition.'';
 				}
 				return $timeCondition;						
 			break;	
 		}
 	}
-
-	
 }
 
-?>
\ No newline at end of file
+?>
