@@ -47,9 +47,9 @@ class slToolsString
 	{
 
 		$emailPattern = '/(((?<!(\.)))\b[&\*\'a-z0-9\.\+_-]{1,255}@[a-z0-9\.\+_-]{1,252}\.[a-z]{2,6})(\?[^\s]+)?/i';
-		$urlPattern = '{(([A-Za-z]{3,9})://?([-;:&=\+\$\w]+@{1})?|(www)\.)((([-A-Za-z0-9\[]{1,}(\.|:))+([-:A-Za-z0-9\]\[]{2,}|[0-9\]\[]{1,}))|([a-zA-Z0-9_-]{1,})):?(\d+)?((/((\?)?[-\(\):\+!\~/&%@\.\w\*,]+)?\??([-\(\)\+!\:=;&%@\.\w\\\\{}\*,/]+)?((["-\(\)\+!\:=;&%@\.\w\\\\{}\*,/]+)([-\(\)\+!\:=;&%@\.\w\\\\{}\*,/]+)))?(#[^\s\r\n\t]+)?)?(/)?}';
+		$urlPattern = '{((\w{3,9})://?([-;:&=\+\$\p{L}\p{M}]+@{1})?|(www)\.)((([-\p{L}\p{M}\d_[]{1,}(\.|:))+([-:\p{L}\p{M}\d_\][]{2,}|[\d\][]{1,}))|([\p{L}\p{M}\d_-]{1,})):?(\d+)?((/((\?)?[-\(\):\+!\~/&%@\.\p{L}\p{M}\d_\*,]+)?\??([-\(\)\+!\:=;&%@\.\p{L}\p{M}\d_\\\\{}\*,/]+)?((["-\(\)\+!\:=;&%@\.\p{L}\p{M}\d_\\\\{}\*,/]+)([-\(\)\+!\:=;&%@\.\p{L}\p{M}\d_\\\\{}\*,/]+)))?(#[^\s\r\n\t]+)?)?(/)?}u';
 		
-		 		 		$stringToProcess = substr($string, 0, MAX_LINKS_SIZE);
+		$stringToProcess = substr($string, 0, MAX_LINKS_SIZE);
 		$string = substr($string,MAX_LINKS_SIZE);
 		 		$replaceFunction = function($matches){
 		    $rep = str_replace("@", "#__w_at#", $matches[0]); $http = $matches[4]=="www"?"http://":""; return "#__w_o#a href=#__w_q#$http$rep#__w_q##__w_c#$rep#__w_o#/a#__w_c#";
@@ -151,6 +151,8 @@ class slToolsString
 
 		         $config->set('URI.Host', $_SERVER['SERVER_NAME']);
 
+		 		$config->set('Cache.SerializerPermissions', null);
+
 		return $config;
 	}
 
@@ -292,10 +294,9 @@ class slToolsString
             }
         }
 
-		libxml_disable_entity_loader(true);
 		try {
 			$string = self::fixSpace($string);
-			if(preg_match_all('/\\<[^a-z!\\/][^\\>]+\\>/i', $string, $matches)){
+			if(preg_match_all('/\\<[^a-z!\\/][^\\>-]+\\>/i', $string, $matches)){
                 foreach ($matches[0] as $match) {
                     $string = str_replace($match, htmlentities($match), $string);
 			    }
@@ -402,7 +403,7 @@ class slToolsString
 
 		if($metaElm){
 			$content = $metaElm->getAttribute('content');
-			 			$content = preg_replace('/(charset=([^"]+))/si','charset=utf-8',$content);
+			 			$content = preg_replace('/(charset=([^"]+))/si','charset=UTF-8',$content);
 			$metaElm->setAttribute('content',$content);
 		}
 		 		$baseElm = $xml->query("//base");
@@ -463,8 +464,8 @@ class slToolsString
 					if($block_external){
 						if(strpos($src,'http')!==false && strpos($src,'http')===0){
 							$blocked = true;
-							$img->setAttribute('src','#');
-
+							$img->setAttribute('src','data:image/gif;base64,R0lGODlhAQABAAD/ACwAAAAAAQABAA');
+							$img->setAttribute('class', $img->getAttribute('class') . ' iw-placeholder');
 						}
 					}else{
 						if(strpos($src,'http')!==false && strpos(ltrim(trim($src),'%20'),'http')===0){
@@ -477,6 +478,28 @@ class slToolsString
 			}
 		}
 
+		 		$iframes = $xml->query("//iframe");
+		if(!empty($iframes)){
+			foreach($iframes as $iframe){
+				if($src = $iframe->getAttribute('src')){
+					
+					if($block_external){
+						if(strpos($src,'http')!==false && strpos($src,'http')===0){
+							$blocked = true;
+							$iframe->setAttribute('src', 'about:blank');
+							$iframe->setAttribute('class', $iframe->getAttribute('class') . ' iw-placeholder');
+						}
+					}else{
+						if(strpos($src,'http')!==false && strpos(ltrim(trim($src),'%20'),'http')===0){
+							if(trim($src)!=$src || ltrim($src,'%20')!=$src){
+								$iframe->setAttribute('src',ltrim(trim($src),'%20'));
+							}
+						}
+					}
+				}
+			}
+		}
+
 		 		if($block_external){
 			$elements = $xml->query("//input");
 			if(!empty($elements)){
@@ -652,6 +675,8 @@ class slToolsString
 	
 	public static function basicSanitizeHTML($string, &$base = '', $block_external = false, &$blocked = false, $textLinks = false, $wrap_html = false)
 	{
+        $string = preg_replace('/<!--\s{2,}-->/', '', $string);
+        $string = preg_replace('/\\s{10,}/', ' ', $string);
         $blocked = $blocked ?? false;
 	    $string = str_replace('<?php', '', $string);
 
@@ -673,6 +698,7 @@ class slToolsString
 	    }while ($oldstring != $string);
 
 	    $string = str_replace("&", "&amp;", $string);
+		 		$string = preg_replace('/\<\?xml.*?\?\>\r?\n?/', '', $string);
 
 	     	    $string = preg_replace('/]]&amp;gt;/i', ']]>', $string);
 
@@ -683,14 +709,13 @@ class slToolsString
 		}
 	    }
 
-	    $xml = self::getXmlFromHtml($string);
-	    libxml_disable_entity_loader(false);
+		$xml = self::getXmlFromHtml($string);
 	     	    $metaElm = $xml->query("//meta[@content]");
 	    $metaElm = $metaElm->item(0);
 
 	    if ($metaElm) {
 		$content = $metaElm->getAttribute('content');
-		 		$content = preg_replace('/(charset=([^"]+))/si', 'charset=utf-8', $content);
+		 		$content = preg_replace('/(charset=([^"]+))/si', 'charset=UTF-8', $content);
 		$metaElm->setAttribute('content', $content);
 	    }
 	     	    $baseElm = $xml->query("//base");
@@ -732,7 +757,8 @@ class slToolsString
 			if ($block_external) {
 			    if (strpos($src, 'http') === 0) {
 				$blocked = true;
-				$img->setAttribute('src', '#');
+				$img->setAttribute('src', 'data:image/gif;base64,R0lGODlhAQABAAD/ACwAAAAAAQABAA');
+				$img->setAttribute('class', $img->getAttribute('class') . ' iw-placeholder');
 			    }
 			} else {
 			    if (strpos(ltrim(trim($src), '%20'), 'http') === 0 && (trim($src) != $src || ltrim($src, '%20') != $src)) {
@@ -741,8 +767,27 @@ class slToolsString
 			}
 		    }
 		}
-	    }
-	     	    if ($block_external) {
+		}
+
+		 		if ($iframes = $xml->query("//iframe")) {
+		foreach ($iframes as $iframe) {
+		    if (($src = $iframe->getAttribute('src')) && strpos($src, 'http') !== false) {
+			if ($block_external) {
+			    if (strpos($src, 'http') === 0) {
+				$blocked = true;
+				$iframe->setAttribute('src', 'about:blank');
+				$iframe->setAttribute('class', $iframe->getAttribute('class') . ' iw-placeholder');
+			    }
+			} else {
+			    if (strpos(ltrim(trim($src), '%20'), 'http') === 0 && (trim($src) != $src || ltrim($src, '%20') != $src)) {
+				$iframe->setAttribute('src', ltrim(trim($src), '%20'));
+			    }
+			}
+		    }
+		}
+		}
+
+		 	    if ($block_external) {
 		self::removeUrlFromAttribute($xml, $blocked, '//input', 'src');
 		 		self::removeUrlFromAttribute($xml, $blocked, "//*[contains(@background,'http')]", 'background', 'http', 'none');
 		 		if ($elements = $xml->query("//style[contains(.,'@import')]")) {
@@ -776,7 +821,7 @@ class slToolsString
 		}
 	    }
 	    if ($textLinks) {
-		if ($texts = $xml->query("//text()[not(parent::a)]")) {
+		if ($texts = $xml->query("//text()[not(parent::a)][not(parent::style)]")) {
 		    $textLinksCount = 0;
 		    foreach ($texts as $text) {
 			if ($textLinksCount++ > MAX_URL_LINKS) {
@@ -802,8 +847,7 @@ class slToolsString
 		$string = preg_replace('/background-image:url\([^\)]+\)/si', '', $string);
 		$string = preg_replace('/background:url\([^\)]+\)/si', '', $string);
 	    }
-
-	    $string = str_replace("&amp;", "&", $string);
+		$string = str_replace("&amp;", "&", $string);
 	    $string = trim($string);
 	    return $string;
 	}
@@ -956,15 +1000,6 @@ class slToolsString
 		return $result;
 	}
 	
-	public static function cidrMatch($ip, $cidr)
-	{
-	    list($subnet, $mask) = explode('/', $cidr);
-	    if ((ip2long($ip) & ~((1 << (32 - $mask)) - 1) ) == ip2long($subnet)) { 
-	        return true;
-	    }
-	    return false;
-	}
-	
 	static public function urlenquote($string)
 	{
 		$string = urlencode($string);
@@ -978,6 +1013,14 @@ class slToolsString
 		$string = urldecode($string);
 		return $string;
 	}	
+
+	public static function removeQuotes($string)
+	{
+		if(substr($string,0,1)=='"' && substr($string,strlen($string)-1,1)=='"') {
+			$string = substr($string,1,strlen($string)-2);
+		}
+		return $string;
+	}
 	
 	static public function fixDistributionListHeader($header)
 	{
@@ -989,5 +1032,13 @@ class slToolsString
 		}
 		return $header;
 	}
+
+	public static function paragraphLines($string, $htmlentities = false)
+    {
+        $string = preg_replace('/(?:<|(?:\&lt;))br\s?\/?(?:>|(?:\&gt;))/i', PHP_EOL, $string);
+        $replace = '<p>\\1</p>';
+        if($htmlentities) $replace = htmlentities($replace);
+        return str_replace(["\r", "\n"], '', preg_replace('/^(?:(.+)[\r\n]*)$/m', $replace, $string));
+    }
 }
-?>
\ No newline at end of file
+?>
