@@ -23,7 +23,7 @@ _me.__constructor = function(s){
 			icon:'user',
 			value:'generic::new_user',
 			onclick:function(name){
-				me._newAccount(0);
+				me._newAccount(Account.USER);
 				return false;
 			}
 		},
@@ -33,7 +33,7 @@ _me.__constructor = function(s){
 			icon:'collaboration',
 			value:'generic::new_group',
 			onclick:function(name){
-				me._newAccount(7);
+				me._newAccount(Account.GROUP);
 				return false;
 			}
 		},
@@ -43,7 +43,7 @@ _me.__constructor = function(s){
 			icon:'mailing-list',
 			value:'generic::new_mailing_list',
 			onclick:function(name){
-				me._newAccount(1);
+				me._newAccount(Account.MAILINGLIST);
 				return false;
 			}
 		},
@@ -53,7 +53,16 @@ _me.__constructor = function(s){
 			icon:'resources',
 			value:'generic::new_resource',
 			onclick:function(name){
-				me._newAccount(8);
+				me._newAccount(Account.RESOURCE);
+				return false;
+			}
+		},
+		{
+			name: 'new_static_route',
+			icon: 'hypervisor',
+			value: 'generic::new_static_route',
+			onclick: function(){
+				me._newAccount(Account.STATIC_ROUTE);
 				return false;
 			}
 		},
@@ -63,7 +72,7 @@ _me.__constructor = function(s){
 			icon:'server',
 			value:'generic::new_domain',
 			onclick:function(name){
-				me._newAccount('d');
+				me._newAccount(Account.DOMAIN);
 				return false;
 			}
 		}
@@ -124,10 +133,19 @@ _me.__constructor = function(s){
 				location.hash='menu=whitelabeling';
 				return false;
 			}
-		});
+		}, {
+			name: 'features',
+			icon: 'webclient',
+			value: 'main::features',
+			onclick: function(name) {
+				me._onouterclick();
+				location.hash = 'menu=features';
+				return false;
+			}
+		},);
 	}
 
-/*	if(gui._globalInfo.admintype==USER_ADMIN){
+	if(gui._globalInfo.admintype==USER_ADMIN){
 		this._asideMainMenu.push({
 			name:'content_rules',
 			icon:'rules',
@@ -139,7 +157,7 @@ _me.__constructor = function(s){
 			}
 		});
 	}
-*/
+
 	if(gui._globalInfo.admintype==USER_ADMIN || gui._globalInfo.admintype==USER_WEB){
 		this._asideMainMenu.push({
 			name:'server_settings',
@@ -153,7 +171,7 @@ _me.__constructor = function(s){
 		});
 	}
 
-	if(gui._globalInfo.admintype!=USER_USER){
+	if(gui._globalInfo.admintype!=USER_USER && gui._globalInfo.admintype!=USER_WEB){
 		this._asideMainMenu.push({
 			name:'api_console',
 			icon:'terminal',
@@ -222,8 +240,15 @@ _me.__constructor = function(s){
 				me._userMenu.unshift({
 					name:'webclient',
 					value:'main::webclient',
-					href:helper.trim(gui._globalInfo.webmail_url,'/')+'/?sid='+dataSet.get('main',['sid'])+'&from=WebAdmin&language=' + (storage.aStorage.language._ACTIVE_LANG || 'en'),
 					onclick:function(name){
+						com.server.getAToken([function(atoken) {
+							if (atoken) {
+								open(helper.trim(gui._globalInfo.webmail_url, '/') + '/?atoken=' + encodeURIComponent(atoken) + '&language=' + (storage.aStorage.language._ACTIVE_LANG || 'en'), 'webmail');
+							// } else {
+								// an error getting atoken, fallback to unsafe sid
+								// open(helper.trim(gui._globalInfo.webmail_url, '/') + '/?sid=' + dataSet.get('main', ['sid']) + '&from=WebAdmin&language=' + (storage.aStorage.language._ACTIVE_LANG || 'en'), 'webmail');
+							}
+						}]);
 						me._onouterclick();
 						return false;
 					}
@@ -504,6 +529,28 @@ _me.__constructor = function(s){
 	if(Cookies.get('night_mode') == 1) {
 		NightMode().activate();
 	}
+
+	if (gui._globalInfo.resources.restrictions.admin_autologout_timeout > 0) {
+		var activity_last = +new Date(),
+			activity_time = gui._globalInfo.resources.restrictions.admin_autologout_timeout * 60000;
+
+		this.__activity = function(){
+			activity_last = +new Date();
+		};
+
+		//Bind events
+		gui._obeyEvent('click',[this,'__activity']);
+		gui._obeyEvent('mousemove',[this,'__activity']);
+		gui._obeyEvent('focus',[this,'__activity']);
+		gui._obeyEvent('keydown',[this,'__activity']);
+
+		//Check for Activity
+		setInterval(function() {
+			if (+new Date() - activity_last > activity_time) {
+				logout();
+			}
+		}, 10000);
+	}
 };
 
 _me._fillUsermenu=function(){
@@ -767,6 +814,11 @@ _me.__hash_handler = function(e,aData)
 					this.main._create('content_rules','obj_content_rules','main_content');
 					this.main.content_rules._hash_handler(e,aData);
 				break;
+				case "features":
+					this.main._clean('main_content');
+					this.main._create('features', 'obj_features', 'main_content');
+					this.main.features._hash_handler(e, aData);
+				break;
 				case "server_settings":
 					this.main._clean('main_content');
 					this.main._create('server_settings','obj_server_settings','main_content');
