@@ -635,6 +635,7 @@ _me.__genPathName = function(){
 							case 'fill' :
 							case 'filllang' :
 							case 'value':
+							case 'data':
 							case 'src':
 							case 'title':
 							case 'text' :
@@ -794,6 +795,10 @@ _me.__addObjects = function(obj,str,aData){
 		if (obj[i]['value'] && Is.Function(newObj._value))
 			newObj._value(obj[i]['value']);
 		
+		// _data
+		if (obj[i]['data'] && Is.Function(newObj._data))
+			newObj._data(obj[i]['data']);
+		
 		// _src
 		if (obj[i]['src'] && Is.Function(newObj._src))
 			newObj._src(obj[i]['src']);
