@@ -112,6 +112,11 @@ helper.base64_decode=function(data) {
   return dec.replace(/\0+$/, '');
 }
 
+helper.b64DecodeUnicode = function(str) {
+	return decodeURIComponent(atob(str).split('').map(function(c) {
+		return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
+	}).join(''));
+}
 
 helper.array_diff=function(arr1) {
   //  discuss at: http://phpjs.org/functions/array_diff/
@@ -221,9 +226,9 @@ helper.clone=function(obj) {
 
 helper.translateHash=function(hash,parameters){
 	if(!parameters&&location.parsed_query){parameters=location.parsed_query;}
-	var str=hash;
+	var str=hash.replace(/^#/, '');
 	for(var key in parameters){
-		str=encodeURIComponent(str.replace('/'+key.toUpperCase()+'/',parameters[key]));
+		str=str.replace('/'+key.toUpperCase()+'/',encodeURIComponent(parameters[key]));
 	}
 	return str;
 }
