@@ -11,7 +11,7 @@ _me.__constructor = function(s){
 
 	storage.library('wm_user');
 
-	var view = this._view = new AccountInfoView(this);
+	this._view = new AccountInfoView(this);
 
 	gui.frm_main.main._setHeadingButton('generic::save',function(){me._save();},'button text success');
 
@@ -19,12 +19,80 @@ _me.__constructor = function(s){
 
 };
 
+_me._loadStaticRoute = function() {
+	var me = this;
+	this._draw('obj_staticrouteinfo');
+	gui.frm_main.main._getAnchor('main_content').scrollTop=0;
+
+	var user = new Account(location.parsed_query.account);
+	user.getPropertySet([
+		'U_Alias',			// alias
+		'U_Name',			// description
+		'R_Activity',		// action
+		'R_ExternalDomain',	// forward older
+		'R_ActivityValue',	// value
+		'R_SaveTo',			// forward to
+		'U_BlackWhiteFilter',	// Rules enabled
+		// 'R_FilterFile',
+		// 'R_ExternalFilter',
+		// 'R_ExternalFilterFile',
+		// 'R_ExternalFilterType'
+	], function (p) {
+		this._data = p;
+
+		this.multiple_add_alias._label("@" + this._accountDomain);
+		this.multiple_add_alias._setValue(p.A_AliasList);
+		if (p.A_AliasList.propertyRights != 2){
+			this.button_add_alias._disabled(true);
+		} else {
+			this.button_add_alias._onclick = function() {
+				me.multiple_add_alias._add(me._data.A_AliasList.addItem('item', ''));
+			}
+		}
+
+		this.input_static_route_description._setValue(p.U_Name);
+
+		this.dropdown_static_route_action._fill({
+			'0': getLang('static_route::forward_to_address'),
+			'1': getLang('static_route::forward_to_domain'),
+			'2': getLang('static_route::forward_to_host'),
+			'3': getLang('static_route::deliver_to_this_domain'),
+			'4': getLang('static_route::delete')
+		});
+		this.dropdown_static_route_action._onchange = function(event, target) {
+			me.checkbox_static_route_forward_older._disabled(target.value > 1);
+			me.input_static_route_value._disabled(target.value > 2);
+		};
+		this.dropdown_static_route_action._setValue(p.R_Activity);
+
+		this.checkbox_static_route_forward_older._setValue(p.R_ExternalDomain);
+
+		this.input_static_route_value._setValue(p.R_ActivityValue);
+
+		this.input_static_route_forward_to._setValue(p.R_SaveTo);
+
+		this.checkbox_static_route_enable_rules._onchange = function(event, target) {
+			if (this._checked()) {
+				gui.frm_main.main.left_menu._enableTab('rules');
+			} else {
+				gui.frm_main.main.left_menu._disableTab('rules');
+			}
+		};
+		this.checkbox_static_route_enable_rules._setValue(p.U_BlackWhiteFilter);
+
+	}.bind(this));
+};
+
 _me._load = function(domain)
 {
 	try
 	{
 		var me=this;
 
+		if (location.parsed_query.type === Account.STATIC_ROUTE) {
+			return this._loadStaticRoute();
+		}
+
 		// set guestaccountstatus
 		if(this._isguestaccount === void 0){
 			com.console.item(function(result){
@@ -56,7 +124,7 @@ _me._load = function(domain)
 
 		// Check if outlook and desktop licences are allowed on domain level
 		var domain = new Domain(
-			location.parsed_query.account.split("@")[1]
+			location.parsed_query.account.split("@")[1] 
 		).getProperties([
 			'D_ConnectorSupport',
 			'D_DesktopSupport',
@@ -74,6 +142,12 @@ _me._load = function(domain)
 			me._initPlans();
 		});
 
+		if (me.button_groups) {
+			me.button_groups._onclick = function() {
+				me._openGroups();
+			};
+		}
+
 		var user = new Account(location.parsed_query.account);
 		user.getPropertySet([
 			'u_name',
@@ -83,13 +157,14 @@ _me._load = function(domain)
 			'u_spamadmin',
 			'u_2f_enabled',
 			'u_saas_plan',
+			'u_fulltext_status',
 			'u_meetingsupport',
 			'u_recordingsupport',
 			'u_activesyncsupport',
-			'u_desktopsupport'
+			'u_desktopsupport',
+			'u_domainadmin'
 		],function(p){
 			this._data = p;
-console.log(p);
 			['meeting', 'recording', 'activesync', 'desktop'].forEach(function(service) {
 				this._getAnchor('service-' + service).classList.toggle('disabled', p['u_' + service + 'support'].value === "0");
 			}, this);
@@ -119,7 +194,7 @@ console.log(p);
 			if(loginip) {
 				this._getAnchor('last_login_ip').textContent = loginip;
 			}
-			if(logintime && !isNaN(logintime)) {
+			if(logintime && !isNaN(logintime) && logintime !== 4294963696) {
 				this._getAnchor('last_login_date').innerHTML=helper.date(getLang('datetime::php_date'),logintime);
 				this._getAnchor('last_login_time').innerHTML=helper.date(getLang('datetime::php_time'),logintime);
 			} else {
@@ -176,6 +251,7 @@ console.log(p);
 			}
 
 			this._initPlans();
+			this._initFulltext();
 
 			// Account type
 			var aFill;
@@ -190,18 +266,30 @@ console.log(p);
 				if (gui._globalInfo.admintype != USER_ADMIN)
 					this.dropdown_account_type._disabled(true);
 			}
-			else{
+			else if (gui._globalInfo.admintype == USER_WEB){
 				aFill = {
 					'0':getLang('userlist::user'),
 					'2':getLang('userlist::domain_admin'),
 					'3':getLang('userlist::webadmin')
 				};
 			}
+			else{
+				aFill = {
+					'0':getLang('userlist::user'),
+					'1':getLang('userlist::domain_admin'),
+				};
+			}
 
 			this.dropdown_account_type._fill(aFill);
-			this.dropdown_account_type._setValue(p.A_AdminType);
 
-			this._getAnchor('user_type_icon')._admintype='subtype_'+p.A_AdminType;
+			if (gui._globalInfo.admintype == USER_ADMIN || gui._globalInfo.admintype == USER_WEB) {
+				this.dropdown_account_type._setValue(p.A_AdminType);
+				this._getAnchor('user_type_icon')._admintype='subtype_'+p.A_AdminType;
+			} else {
+				this.dropdown_account_type._setValue(p.u_domainadmin);
+				this._getAnchor('user_type_icon')._admintype='subtype_'+(p.u_domainadmin.value == '1' ? 2 : 0);
+			}
+
 			addcss(this._getAnchor('user_type_icon'),this._getAnchor('user_type_icon')._admintype);
 
 			// Account state
@@ -274,7 +362,11 @@ console.log(p);
 		this.dropdown_account_type._onchange=function(e){
 			var val=me.dropdown_account_type._value();
 			removecss(me._getAnchor('user_type_icon'),me._getAnchor('user_type_icon')._admintype);
-			me._getAnchor('user_type_icon')._admintype="subtype_"+val;
+			if (gui._globalInfo.admintype == USER_ADMIN) {
+				me._getAnchor('user_type_icon')._admintype="subtype_" + val;
+			} else {
+				me._getAnchor('user_type_icon')._admintype="subtype_" + (val == '1' ? 2 : 0);
+			}
 			addcss(me._getAnchor('user_type_icon'),me._getAnchor('user_type_icon')._admintype);
 
 		}
@@ -336,10 +428,10 @@ console.log(p);
 
 		com.general.install_url(function(url){
 			me._getAnchor('button_account_icewarp_outlook_sync_download').target="_blank";
-			me._getAnchor('button_account_icewarp_outlook_sync_download').href=url+'download/outlook-sync.exe';
+			me._getAnchor('button_account_icewarp_outlook_sync_download').href = 'https://icewarp.com/apps/outlook-sync/';
 
 			me._getAnchor('button_account_icewarp_desktop_download').target="_blank";
-			me._getAnchor('button_account_icewarp_desktop_download').href=url+'download/desktop-setup.msi';
+			me._getAnchor('button_account_icewarp_desktop_download').href = 'https://icewarp.com/apps/icewarp-desktop-suite/';
 		});
 
 		this.button_add_alias._onclick=function(e){
@@ -370,7 +462,42 @@ console.log(p);
 	{
 		log.error(e);
 	}
-}
+};
+
+_me._initFulltext = function() {
+	var me = this, status;
+
+	function setFulltext(id) {
+		var value = 0,
+			label = getLang('VCARD::UNKNOWN'),
+			data = status[id] || {};
+
+		switch(data.status_code) {
+			case 1:
+			case 2:
+			case 3:
+				label = getLang('ACCOUNTDETAIL::DISABLED');
+				break;
+			case 4:
+			case 5:
+				label = data.status_usrpct ? getLang('ACCOUNTDETAIL::INDEXED', [Math.round(data.status_usrpct * 100) / 100 + '%']) : getLang('ACCOUNTDETAIL::ENABLED');
+				value = data.status_usrpct || 100;
+				break;
+		}
+
+		me['fulltext_' + id]._label(label);
+		me['fulltext_' + id]._value(value);
+	}
+
+	try {
+		status = JSON.parse(this._data.u_fulltext_status);
+		setFulltext('mail');
+		setFulltext('archive');
+		setFulltext('gw');
+	} catch (e) {
+		this._getAnchor('fb_fulltext').setAttribute('hidden', '');
+	}
+};
 
 _me._initPlans = function() {
 	var me = this;
@@ -385,7 +512,7 @@ _me._initPlans = function() {
 		selectable: true,
 		show_prices: +cloudinfo.cloudshowprice,
 		yearly: (cloudinfo.cloudplanbillingperiod || '').toString() !== 'MONTH',
-		plans: gui._globalInfo.licence.plans.map(function(plan) {
+		plans: (gui._globalInfo.licence.plans || []).map(function(plan) {
 			if(!+me._props.D_Saas_Plan || +plan.planid <= maxPlan) {
 				var label = getLang("SUBSCRIPTION_PLANS::" + plan.planlabel);
 				return options = {
@@ -426,7 +553,7 @@ _me._save = function(callback,allowSameUsername){
 	var me = this;
 
 	var user = this.input_account_username;
-	if(!allowSameUsername && user._value()==gui._globalInfo.accountProperties.u_mailbox && me.__initial_username!=gui._globalInfo.accountProperties.u_mailbox) {
+	if(user && !allowSameUsername && user._value()==gui._globalInfo.accountProperties.u_mailbox && me.__initial_username!=gui._globalInfo.accountProperties.u_mailbox) {
 		// Do not allow to set user name to the same as the currently logged in user
 		if(this._accountDomain==gui._globalInfo.domain) {
 			// This is you own email, can't be used
@@ -467,11 +594,13 @@ _me._save = function(callback,allowSameUsername){
 			}
 
 			// If main alias changed, change hash
-			if(newusername && r==1) {
+			if(r==1) {
 				gui._globalInfo._update(function(){
 					gui.frm_main._fillUsermenu();
 				});
-				location.hash = helper.translateHash(hash.replace('/ACCOUNT/',encodeURIComponent(newusername)),location.parsed_query);
+				if (newusername) {
+					location.hash = helper.translateHash(hash.replace('/ACCOUNT/',encodeURIComponent(newusername)));
+				}
 			}
 		});
 	}
@@ -567,6 +696,24 @@ _me._openPermissions=function(){
 	popup.content._load(location.parsed_query.account);
 }
 
+_me._openGroups=function(){
+	var popup = gui._create('popup', 'obj_popup');
+	popup._init({
+		fixed: false,
+		iwattr: {
+			height: 'full',
+			width: 'medium'
+		},
+		name: 'accountgroups',
+		heading: {
+			value: getLang('accountdetail::list_of_memberships')
+		},
+		content: 'obj_accountgroups'
+	});
+
+	popup.content._load(location.parsed_query.account);
+};
+
 /* View */
 
 var AccountInfoView = function(controller) {
