@@ -18,7 +18,7 @@ _me.__constructor = function(s){
 /** obbefore destruct listener */
 _me.__onbeforedestruct=function(){
 
-}
+};
 
 _me._hash_handler=function(){
 	var me=this;
@@ -32,34 +32,6 @@ _me._hash_handler=function(){
 		}
 	});
 
-}
-
-/*
-// Hash handler for tabbed menu
-_me._hash_handler=function(){
-	var me=this;
-	
-	try
-	{
-		me._getMenuDefinition({},function(menuDefinition,defaultTab){
-			me._defaultTab=defaultTab;
-			
-			gui.frm_main.main._init({
-				name:'content_rules',
-				heading:{
-					value:getLang('main::content_rules')
-				},
-				menu:{
-					hashTemplate:"menu=/MENU/",
-					items:menuDefinition
-				}
-			});
-		});
-	}
-	catch(e)
-	{
-		log.error([e,me]);
-	}
-
-	
-}*/
\ No newline at end of file
+	this._create('rules','obj_rules','');
+	this.rules._load();
+};
\ No newline at end of file
