@@ -235,7 +235,7 @@ _me._updateTrafficData=function(items){
 
 	log.log(['dashboard-updatetrafficdata',data]);
 
-	var web=0;
+	var web=0, web_max = 0;
 	log.log(['dashboard-_updateTrafficData',items]);
 	if(data.control && data.control.connections){
 		web=data.control.connections;
