@@ -534,7 +534,7 @@ _me.__activateActionCard=function(){
 						{
 							var id=data.id;
 								id=id.substr(1,id.length-1);
-								id=helper.base64_decode(id);
+								id=helper.b64DecodeUnicode(id);
 							me[(name=='copyfolder'?'input_copy_to_folder':'input_move_to_folder')]._value(id);
 						}
 						catch(e)
@@ -763,14 +763,14 @@ _me.__setAction=function(data){
 		case 'flags':
 			this.checkbox_flagged._checked(data.flagged);
 			this.checkbox_seen._checked(data.seen);
-			this.checkbox_junk._checked(data.junk);
-			this.checkbox_non_junk._checked(data.notjunk);
-			this.checkbox_label_1._checked(data.label1);
-			this.checkbox_label_2._checked(data.label2);
-			this.checkbox_label_3._checked(data.label3);
-			this.checkbox_label_4._checked(data.label4);
-			this.checkbox_label_5._checked(data.label5);
-			this.checkbox_label_6._checked(data.label6);
+			// this.checkbox_junk._checked(data.junk);
+			// this.checkbox_non_junk._checked(data.notjunk);
+			// this.checkbox_label_1._checked(data.label1);
+			// this.checkbox_label_2._checked(data.label2);
+			// this.checkbox_label_3._checked(data.label3);
+			// this.checkbox_label_4._checked(data.label4);
+			// this.checkbox_label_5._checked(data.label5);
+			// this.checkbox_label_6._checked(data.label6);
 			this.input_custom_flags._value(data.customflags);
 		break;
 	}
@@ -906,15 +906,15 @@ _me._updateStorage=function(){
 				break;
 				case 'flags':
 					this._setValue('flagged',this.checkbox_flagged._checked());
-					this._setValue('junk',this.checkbox_junk._checked());
-					this._setValue('notjunk',this.checkbox_non_junk._checked());
+					// this._setValue('junk',this.checkbox_junk._checked());
+					// this._setValue('notjunk',this.checkbox_non_junk._checked());
 					this._setValue('seen',this.checkbox_seen._checked());
-					this._setValue('label1',this.checkbox_label_1._checked());
-					this._setValue('label2',this.checkbox_label_2._checked());
-					this._setValue('label3',this.checkbox_label_3._checked());
-					this._setValue('label4',this.checkbox_label_4._checked());
-					this._setValue('label5',this.checkbox_label_5._checked());
-					this._setValue('label6',this.checkbox_label_6._checked());
+					// this._setValue('label1',this.checkbox_label_1._checked());
+					// this._setValue('label2',this.checkbox_label_2._checked());
+					// this._setValue('label3',this.checkbox_label_3._checked());
+					// this._setValue('label4',this.checkbox_label_4._checked());
+					// this._setValue('label5',this.checkbox_label_5._checked());
+					// this._setValue('label6',this.checkbox_label_6._checked());
 					this._setValue('customflags',this.input_custom_flags._value());
 				break;
 			}
