@@ -21,7 +21,7 @@ _me.__constructor = function(s){
 		'*1':getLang('userlist::mailing_list'),
 		//'*2':getLang('userlist::executable'),
 		//'*3':getLang('userlist::notification'),
-		//'*4':getLang('userlist::static_route'),
+		'*4':getLang('userlist::static_route'),
 		//'*5':getLang('userlist::catalog'),
 		//'*6':getLang('userlist::list_server'),
 		'*7':getLang('userlist::group'),
@@ -33,15 +33,26 @@ _me.__constructor = function(s){
 		'1':getLang('userlist::admin'),
 		'2':getLang('userlist::domain_admin'),
 		'3':getLang('userlist::webadmin')
+	};
+
+	me.plans = {
+		'-': getLang('accountdetail::all_plans')
+	};
+	var plans = gui._globalInfo.licence.plans || [];
+	if (plans.length) {
+		plans.forEach(function(plan) {
+			var label = getLang("SUBSCRIPTION_PLANS::" + plan.planlabel);
+			me.plans[plan.planid] = ~label.indexOf("::") ? plan.planlabel : label;
+		});
 	}
 
 	me.services = {
-		'0': getLang('accountdetail::all_services'),
+		'-': getLang('accountdetail::all_services'),
 		'1': getLang('userlist_header::service_conferences'),
 		'4': getLang('userlist_header::service_recording'),
 		'2': getLang('userlist_header::service_activesync'),
 		'3': getLang('userlist_header::service_desktop_client')
-	}
+	};
 
 	if(!gui.frm_main.main.actions){
 		gui.frm_main.main._cleanHeadingButtonsAnchor();
@@ -49,6 +60,7 @@ _me.__constructor = function(s){
 			if(!box._alternativeButtons){
 				box._alternativeButtons=[];
 			}
+
 			var actionobject = box._create('actions','obj_actionselect',target_anchor, 'userlist');
 			actionobject._value('generic::select_action');
 			if(target_anchor=='heading_buttons_mobile'){actionobject._addcss('full',true);}
@@ -58,10 +70,104 @@ _me.__constructor = function(s){
 			actionobject._disabled(true);
 
 			box._alternativeButtons.push(actionobject);
+
+			if (plans.length) {
+				var exportButton = box._create('export', 'obj_button', target_anchor, 'text primary');
+				exportButton._value('generic::export');
+				exportButton._onclick = function() {
+					exportButton._disabled(true);
+					me.__export(function() {
+						exportButton._disabled(false);
+					});
+				}
+				exportButton._disabled(true);
+				if(target_anchor == 'heading_buttons_mobile') {
+					exportButton._addcss('full', true);
+				}
+				box._alternativeButtons.push(exportButton);
+			}
 		});
 	}
 };
 
+_me.__export = function(callback) {
+	var me = this;
+	var domain = new Domain(location.parsed_query.domain);
+	var list = domain.getAccountList({type: Account.USER});
+
+	var search = gui.frm_main._getSearch(true);
+	list.filter(search);
+
+	var type = me.list.dropdown_userlist_filter;
+	if(!type) {
+		type = 0;
+	} else if (type._value() != '-') {
+		type = type._value();
+	} else {
+		type = undefined;
+	}
+	list.type(type);
+
+	if(me.list.dropdown_userlist_plans) {
+		var plan = me.list.dropdown_userlist_plans._value();
+		list.setFilter({ planmask: plan });
+	}
+
+	if(me.list.dropdown_userlist_services) {
+		var service = me.list.dropdown_userlist_services._value();
+		list.setFilter({ servicemask: service });
+	}
+
+	var plans = gui._globalInfo.licence.plans || [];
+	list.load(function(items) {
+		var data = [].map.call(items || [], function(item) {
+			var plan, planlabel;
+			var size = +item.quota.mailboxsize.value;
+			var max = +item.quota.mailboxquota.value;
+
+			if(plans.length && +item.planid.value) {
+				plan = plans.filter(function(plan) {
+					return +plan.planid.value === +item.planid.value;
+				})[0] || {};
+				planlabel = getLang("SUBSCRIPTION_PLANS::" + plan.planlabel);
+				planlabel = planlabel.indexOf("::")==-1 ? planlabel : (plan.planlabel ? plan.planlabel.value : item.planid.value);
+			}
+
+			return {
+				[getLang('userlist_header::account')]: punycode.ToUnicode(item.email.value),
+				[getLang('userlist_header::name')]: item.name.value,
+				[getLang('userlist_header::type')]: item.accounttype.value == 0 ? me.admintypes[item.admintype.value || 0] : me.accounttypes['*' + item.accounttype.value],
+				[getLang('userlist_header::quota')]: [size ? relativeByteSize(size, 2, 1) : '', max ? relativeByteSize(max, 2, 1) : ''].filter(Boolean).join(' / '),
+				[getLang('accountdetail::disabled')]: +((item.accountstate || {}).state || {}).value,
+				[getLang('accountdetail::subscription_plan') + ' ID']: ((plan || {}).planid || {}).value || '',
+				[getLang('accountdetail::subscription_plan')]: item.accounttype.value == 0 ? planlabel : '',
+				[getLang('userlist_header::service_conferences')]: item.accounttype.value == 0 ? +item.serviceinfo.meetingvalue.value : '',
+				[getLang('userlist_header::service_recording')]: item.accounttype.value == 0 ? +item.serviceinfo.recordingvalue.value : '',
+				[getLang('userlist_header::service_activesync')]: item.accounttype.value == 0 ? +item.serviceinfo.activesyncvalue.value : '',
+				[getLang('userlist_header::service_desktop_client')]: item.accounttype.value == 0 ? +item.serviceinfo.desktopvalue.value : ''
+			};
+		});
+		
+		if (data.length) {
+			var csv = [Object.keys(data[0])].concat(data.map(function(item) {
+				return Object.values(item).map(function(data) {
+					return '"' + data.toString().replace(/^[=+\-@\r\t]/, "'$&").replace(/"/g, '""') + '"';
+				});
+			}));
+
+			var link = document.createElement("a");
+			link.setAttribute("href", encodeURI("data:text/csv;charset=utf-8," + csv.map(row => row.join(",")).join("\n")));
+			link.setAttribute("download", location.parsed_query.domain + '.csv');
+			document.body.appendChild(link);
+			link.click();
+			document.body.removeChild(link);
+		}
+
+		callback && callback();
+
+	}, {chunk: 9999 });
+};
+
 _me._updateActions = function() {
 	var me = this;
 	var options = [];
@@ -142,7 +248,12 @@ _me._load = function(settings)
 		me._onSearch(string);
 	});
 
-	this._draw('obj_userlist' + (settings && settings.subTemplate?"_" + settings.subTemplate:""), '', {items:{}});
+	this._draw('obj_userlist' + (settings && settings.subTemplate?"_" + settings.subTemplate:""), '', {
+		items:{},
+		options: JSON.stringify({
+			plans: (gui._globalInfo.licence.plans || []).length
+		})
+	});
 
 	if(settings){
 		if(settings.domain){location.parsed_query.domain=settings.domain;}
@@ -170,6 +281,22 @@ _me._load = function(settings)
 		};
 	}
 
+	if(this.list.dropdown_userlist_plans && Object.keys(me.plans).length > 1){
+		this.list.dropdown_userlist_plans._fill(me.plans);
+		this.list.dropdown_userlist_plans._value('-');
+
+		this.list.dropdown_userlist_plans._onchange = function(e){
+			var val = this._value();
+			dataSet.add('accountlist-filter', ['plans-' + location.parsed_query.domain], val);
+			me._lastAccount = false;
+			me.list._emptySelectedList();
+			me.list._empty();
+			me.list._load();
+		};
+	} else if (this.list.dropdown_userlist_plans) {
+		this.list.dropdown_userlist_plans._destruct();
+	}
+
 	if(this.list.dropdown_userlist_services) {
 		this.list.dropdown_userlist_services._fill(me.services);
 		this.list.dropdown_userlist_services._value('-');
@@ -196,13 +323,13 @@ _me._load = function(settings)
 			if(me.list._getSelectedCount()==0){
 				for(var i=0; i<gui.frm_main.main._alternativeButtons.length; i++)
 				{
-					gui.frm_main.main._alternativeButtons[i]._text(getLang("generic::select_action"));
+					gui.frm_main.main._alternativeButtons[0]._text(getLang("generic::select_action"));
 					gui.frm_main.main._alternativeButtons[i]._disabled(true);
 				}
 			}else{
 				for(var i=0; i<gui.frm_main.main._alternativeButtons.length; i++)
 				{
-					gui.frm_main.main._alternativeButtons[i]._text(getLang("generic::select_action")+" ("+me.list._getSelectedCount()+")");
+					gui.frm_main.main._alternativeButtons[0]._text(getLang("generic::select_action")+" ("+me.list._getSelectedCount()+")");
 					gui.frm_main.main._alternativeButtons[i]._disabled(false);
 				}
 			}
@@ -225,6 +352,12 @@ _me._load = function(settings)
 			me.list.dropdown_userlist_filter._value(filter_type,true);
 		}
 	}
+	var plan=dataSet.get('accountlist-filter',['plans-'+location.parsed_query.domain]);
+	if(typeof plan != 'undefined'){
+		if(me.list.dropdown_userlist_plans){
+			me.list.dropdown_userlist_plans._value(plan,true);
+		}
+	}
 	var services = dataSet.get('accountlist-filter', ['services-' + location.parsed_query.domain]);
 	if(typeof services != 'undefined') {
 		if(me.list.dropdown_userlist_services) {
@@ -256,29 +389,35 @@ _me._load = function(settings)
 		}
 		list.type(type);
 
-		if(me.list.dropdown_userlist_services) {
-			var service = me.list.dropdown_userlist_services._value();
-			list.setFilter({ servicemask: service === '-' ? void 0 : service });
+		if(me.list.dropdown_userlist_plans) {
+			var plan = me.list.dropdown_userlist_plans._value();
+			list.setFilter({ planmask: plan });
 		}
 
-		list.load(function(items){
+		if(me.list.dropdown_userlist_services) {
+			var service = me.list.dropdown_userlist_services._value();
+			list.setFilter({ servicemask: service });
+		}
 
+		me.__aData = [];
+		list.load(function(items){
 			gui.frm_main._setSearchResults(items.total);
 			me.list._setMax(items.total);
 			gui.frm_main.main.left_menu._setItemValue('userlist',getLang('domaindetail::users')+' ('+items.total+')');
 
-			var plans = gui._globalInfo.licence.plans;
+			var plans = gui._globalInfo.licence.plans || [];
 			for(var i=0, l=items.length; i<l; i++) {
+				me.__aData.push(items[i]);
 				var plan, planlabel;
 				var size = +items[i].quota.mailboxsize.value;
 				var max = +items[i].quota.mailboxquota.value;
 
-				if(plans && +items[i].planid.value) {
+				if(plans.length && +items[i].planid.value) {
 					plan = plans.filter(function(plan) {
 						return +plan.planid.value === +items[i].planid.value;
-					})[0];
+					})[0] || {};
 					planlabel = getLang("SUBSCRIPTION_PLANS::" + plan.planlabel);
-					planlabel = planlabel.indexOf("::")==-1 ? planlabel : plan.planlabel;
+					planlabel = planlabel.indexOf("::")==-1 ? planlabel : plan.planlabel || items[i].planid.value;
 				}
 
 				var item = {
@@ -293,6 +432,7 @@ _me._load = function(settings)
 					subtype_str: me.admintypes[items[i].admintype.value || 0],
 					size: relativeByteSize(size,2,1),
 					disabled: +((items[i].accountstate || {}).state || {}).value,
+					plans: plans.length,
 					planlabel: items[i].accounttype.value == 0 ? planlabel : '',
 					planid: (plan || {}).planid,
 					hash: md5(items[i].accounttype.value == 0 ? 'ad-' + (items[i].admintype.value || 0) : ('ac-' + (items[i].accounttype.value || 0))),
@@ -316,6 +456,7 @@ _me._load = function(settings)
 					if(!me._lastAccount || me._lastAccount.subtype!=item.subtype){
 						me.list._drawItem({
 							group: item.hash,
+							plans: plans.length,
 							selectable: false,
 							name: +items[i].accounttype.value ? me.accounttypes['*'+items[i].accounttype.value] : me.admintypes[items[i].admintype.value]
 						},function(elm,checked){
@@ -338,15 +479,19 @@ _me._load = function(settings)
 					['meeting', 'recording', 'activesync', 'desktop'].forEach(function(service, i) {
 						var button = item.line._objects[i + 1];
 						if (button) {
-							button._title(item['u_' + service] ? 'USERLIST::DISABLE_' + service : 'USERLIST::ENABLE_' + service);
-							button._onclick = function (item) {
-								this._disabled(true);
-								me._setAccountsService([item], 'u_' + service, +!item['u_' + service], function() {
-									this._disabled(false);
-									this._title(item['u_' + service] ? 'USERLIST::DISABLE_' + service : 'USERLIST::ENABLE_' + service);
-									this._main.classList.toggle('active', item['u_' + service]);
-								}.bind(this));
-							}.bind(button, item);
+							if (item['u_' + service + 'editable']) {
+								button._title(item['u_' + service] ? 'USERLIST::DISABLE_' + service : 'USERLIST::ENABLE_' + service);
+								button._onclick = function (item) {
+									this._disabled(true);
+									me._setAccountsService([item], 'u_' + service, +!item['u_' + service], function() {
+										this._disabled(false);
+										this._title(item['u_' + service] ? 'USERLIST::DISABLE_' + service : 'USERLIST::ENABLE_' + service);
+										this._main.classList.toggle('active', item['u_' + service]);
+									}.bind(this));
+								}.bind(button, item);
+							} else {
+								button._title('USERLIST::' + service + '_DISABLED');
+							}
 						}
 					});
 				}
@@ -437,14 +582,32 @@ _me._groupSelectedAccounts=function(){
 			if(me.list.dropdown_userlist_filter){
 				type = me.list.dropdown_userlist_filter._value();
 			}
+			var planmask='*';
+			var plan=0;
+			if(me.list.dropdown_userlist_plans){
+				plan = me.list.dropdown_userlist_plans._value();
+			}
+			var servicemask='*';
+			var service=0;
+			if(me.list.dropdown_userlist_services){
+				service = me.list.dropdown_userlist_services._value();
+			}
 
 			if(type!='-'){
 				typemask=type;
 			}
+			if(plan!='-'){
+				planmask=plan;
+			}
+			if(service!='-'){
+				servicemask=service;
+			}
 
 			com.user.addAllToGroup(groups,location.parsed_query.domain,{
 				namemask:namemask,
-				typemask:typemask
+				typemask:typemask,
+				planmask:planmask,
+				servicemask:servicemask
 			},function(result){
 				if(!result.Array.IQ[0].QUERY[0].RESULT || result.Array.IQ[0].QUERY[0].RESULT[0].VALUE==0){
 					gui.message.error(getLang("error::save_unsuccessful"));
@@ -529,7 +692,7 @@ _me._moveUser=function(domain, accounts) {
 	var me = this;
 
 	com.domain.moveAccounts(domain,accounts,function(result){
-		if(result.error) {
+		if(result.error || result == 0) {
 			gui.message.error(getLang("error::save_unsuccessful"));
 		} else {
 			gui.message.toast(getLang("message::save_successfull"));
@@ -540,10 +703,66 @@ _me._moveUser=function(domain, accounts) {
 	});
 };
 
-_me._deleteSelectedAccounts=function(){
+_me._deleteSelectedAccounts=function(bForceDeleteLoggedInUser, bForceDeleteLastAdmin) {
 	var me = this;
 	var num = this.list._getSelectedCount();
 	var selected=me.list._getSelectedList();
+	
+	if (!bForceDeleteLoggedInUser) {
+		for(var i in selected) {
+			if ((gui._globalInfo.email === selected[i].id || gui._globalInfo.email === selected[i].unpunied)) {
+				return gui.message.warning([getLang("warning::delete_logged_in_user")], false, [
+					{
+						value:getLang("generic::cancel"),
+						method:'close'
+					},
+					{
+						value:getLang("generic::delete"),
+						onclick:function(closeCallback){
+							closeCallback();
+							me._deleteSelectedAccounts(true);
+						},
+						type:'text error'
+					},
+				]);
+			}
+		}
+	}
+
+	if (!bForceDeleteLastAdmin) {
+		var admins = [];
+		var hasAdmin = false;
+		for (var i in this.__aData) {
+			if (this.__aData[i].accounttype == 0 && this.__aData[i].admintype > 0) {
+				admins.push(this.__aData[i].email.value);
+				hasAdmin = true;
+			}
+		}
+		for (var i in selected) {
+			if (selected[i].type === 0 && selected[i].subtype > 0) {
+				admins = admins.filter(function(admin) {
+					return admin !== selected[i].id;
+				});
+			}
+		}
+
+		if (hasAdmin && !admins.length) {
+			return gui.message.warning([getLang("warning::delete_last_admin")], false, [
+				{
+					value:getLang("generic::cancel"),
+					method:'close'
+				},
+				{
+					value:getLang("generic::delete"),
+					onclick:function(closeCallback){
+						closeCallback();
+						me._deleteSelectedAccounts(bForceDeleteLoggedInUser, true);
+					},
+					type:'text error'
+				},
+			]);
+		}
+	}
 
 	// check for deleting superior account
 	var superior = false;
@@ -613,13 +832,31 @@ _me._deleteSelectedAccounts=function(){
 					if(me.list.dropdown_userlist_filter){
 						type = me.list.dropdown_userlist_filter._value();
 					}
+					var planmask='*';
+					var plan=0;
+					if(me.list.dropdown_userlist_plans){
+						plan = me.list.dropdown_userlist_plans._value();
+					}
+					var servicemask='*';
+					var service=0;
+					if(me.list.dropdown_userlist_services){
+						service = me.list.dropdown_userlist_services._value();
+					}
 
 					if(type!='-'){
 						typemask=type;
 					}
+					if(plan!='-'){
+						planmask=plan;
+					}
+					if(service!='-'){
+						servicemask=service;
+					}
 
 					filter.namemask=namemask;
 					filter.typemask=typemask;
+					filter.planmask=planmask;
+					filter.servicemask=servicemask;
 
 					com.domain.deleteAllAccounts(location.parsed_query.domain,filter,function(result){
 						closeCallback();
