@@ -4,10 +4,8 @@ function obj_whitelabeling(){};
 obj_whitelabeling.LOGIN_SETTINGS = {
 	layout_settings: [
 		'login_logo_name','logo_file',
-		'login_color',
-		'login_background','login_background_name','login_background_file',
 		'search_phrase',
-		'facebook_link','twitter_link','linkedin_link'
+		'website_link','facebook_link','twitter_link','linkedin_link'
 	],
 	custom_login_fields: [
 		'login_verification_enabled','login_verification_type',
@@ -24,17 +22,17 @@ obj_whitelabeling.LOGIN_SETTINGS = {
 		'homepage_enabled','homepage_required'
 	],
 	restrictions: [
-		'disable_languages','disable_remember','disable_autofill','disable_signup',
+		'disable_languages','disable_remember','disable_autofill','disable_signup','disable_guest_signin',
 		'contact_support','support_require_number','show_search',
-		'facebook_disabled','twitter_disabled','linkedin_disabled'
+		'website_disabled','facebook_disabled','twitter_disabled','linkedin_disabled'
 	]
 };
 obj_whitelabeling.WEBMAIL_SETTINGS = {
 	layout_settings: [
 		'title',
 		'skin',
-		'skin_style'
-	]
+		'login_background', 'login_background_name', 'login_background_file', 'login_background_color', 'login_background_wallpaper_size', 'login_background_solid', 'login_interface_color', 'login_favicon_name', 'login_favicon_file', 'login_favicon_uid'
+	],
 };
 obj_whitelabeling.WEBADMIN_SETTINGS = {
 	layout_settings_admin: [
@@ -42,25 +40,17 @@ obj_whitelabeling.WEBADMIN_SETTINGS = {
 		'skin_style'
 	]
 };
-obj_whitelabeling.ICECHAT_SETTINGS = {
-	icechat_settings: [
-		'login_color',
-		'login_background_name',
-		'login_background_file'
-	]
-};
 obj_whitelabeling.COMBINED_SETTINGS = {
 	layout_settings: obj_whitelabeling.WEBMAIL_SETTINGS.layout_settings,
 	layout_settings_admin: obj_whitelabeling.WEBADMIN_SETTINGS.layout_settings_admin,
-	icechat_settings: obj_whitelabeling.ICECHAT_SETTINGS.icechat_settings
 }
 obj_whitelabeling.BANNER_SETTINGS = {
 	banner_options: [
 		'customer_id',
 		'desktop_type','desktop_url','desktop_code',
 		'mobile_type','mobile_url','mobile_code',
-		'top_type','top_url','top_code',
-		'below_type','below_url','below_code'
+		'top_type','top_url','top_link','top_code',
+		'below_type','below_url','below_link','below_code'
 	],
 	restrictions: ['enable_adsense']
 };
@@ -76,6 +66,7 @@ obj_whitelabeling.CONFERENCING = {
  * @date : 26.10.2014
  **/
 _me.__constructor = function(s){
+	storage.library('extract-colors', 'extract-colors');
 	var me = this;
 
 	var view = this._view = new WhiteLabelingView(this);
@@ -121,34 +112,28 @@ _me.__constructor = function(s){
 	this.__settings={};
 
 	this.__settings.skin={};
-	this.__settings.skin_style={
-		'default':getLang('whitelabeling::blue'),
-		'black':getLang('whitelabeling::black'),
-		'brown':getLang('whitelabeling::brown'),
-		'graphite':getLang('whitelabeling::graphite'),
-		'green':getLang('whitelabeling::green'),
-		'orange':getLang('whitelabeling::orange'),
-		'pink':getLang('whitelabeling::pink'),
-		'purple':getLang('whitelabeling::purple'),
-		'red':getLang('whitelabeling::red'),
-		'yellow':getLang('whitelabeling::yellow')
-	}
 	this.__settings.background_urls = {
-		default_0: "background--default.jpg",
-		default_1: "background--krivan.jpg",
-		default_2: "background--prague2.jpg",
-		default_3: "background--adrspach1.jpg",
-		default_4: "background--czechcity.jpg",
-		default_5: "background--prague1.jpg",
-		default_6: "background--slovakia.jpg",
-		default_7: "background--adrspach2.jpg",
-		default_8: "background--czech.jpg",
-		default_9: "background--czech1.jpg",
-		default_10: "background--pleso.jpg",
-		default_11: "background--tatras.jpg",
-		default_12: "background--vine.jpg",
-		default_13: "background--pragueboats.jpg",
-		default_14: ''
+		'default_-1': "custom",
+		default_0: "background--default.png",
+		default_1: "background--abstract.png",
+		default_2: "background--architecture.png",
+		default_3: "background--car.png",
+		default_4: "background--city.png",
+		default_5: "background--fox.png",
+		default_6: "background--mountains.png",
+		default_7: "background--nature.png",
+		default_8: "background--ship.png",
+		default_9: "background--space.png",
+		default_10: "background--vacation.png"
+	};
+	this.__settings.background_colors = {
+		custom: 'custom',
+		"#702EB7": "#702EB7",
+		"#0086E5": "#0086E5",
+		"#45A154": "#45A154",
+		"#AF3A3A": "#AF3A3A",
+		"#CF9517": "#CF9517",
+		"#A1456D": "#A1456D"
 	};
 	this.__settings._getFileLabel = function(file) {
 		var files = this.background_urls;
@@ -161,29 +146,6 @@ _me.__constructor = function(s){
 
 };
 
-/** activate colorpickers to switch background images and activate imagepicker */
-_me._bindColorPickerWithImagepicker = function(preview,object,imageTemplate){
-	var me=this;
-
-	// Make sure that background in preview changes
-	var defaultImage = me[imageTemplate.replace("#COLOR#","default")+"0"];
-	defaultImage.__moreGroupOnchange = defaultImage.__moreGroupOnchange || [];
-	defaultImage.__moreGroupOnchange.push(function(that){
-		preview._setAttribute("background",that._groupValue());
-		// Set background image for preview
-		var img = me.radio_login_background_image_default_0._groupValue();
-		if(img) {
-			var url = "./client/skins/default/login/images/";
-			url += me.__settings.background_urls[img];
-			preview._main.style.backgroundImage = "url("+url+")";
-		}
-		// Clear out any previously uploaded image name
-		me.input_login_background_image_input._value("");
-//		me._getAnchor('fb_login_background_image')._value("");
-	});
-
-}
-
 _me._hash_handler=function(){
 	var me=this;
 
@@ -197,25 +159,10 @@ _me._hash_handler=function(){
 		me.dropdown_language._disabled(true);
 	}
 
-	me._getAnchor('fi_wc_default').removeAttribute('is-hidden');
 	me._getAnchor('fi_login_default').removeAttribute('is-hidden');
 	me._getAnchor('fi_wa_default').removeAttribute('is-hidden');
 	me.__is_default_enabled=true;
 
-	// activate imagepickers
-	me._bindColorPickerWithImagepicker(
-		me.slider_login.preview_login,
-		me.radio_login_colorpicker_default,
-		"radio_login_background_image_#COLOR#_"
-	);
-
-	// Set background used in login preview
-	var url = "url(./server/download.php?class=background";
-	url += "&fullpath="+encodeURIComponent(me.__domaindetail || '__@@GLOBAL@@__');
-	url += "&uid="+Date.now();
-	url += "&resize=1&width=572)";
-	this.slider_login.preview_login._main.style.backgroundImage = url;
-
 	// Set logo used in preview
 	var logo_custom = this.slider_login.preview_login._getAnchor('login_logo_custom');
 	var logo_default = this.slider_login.preview_login._getAnchor('login_logo_default');
@@ -225,15 +172,11 @@ _me._hash_handler=function(){
 	logo_custom.addEventListener('load', function() {
 		logo_custom.removeAttribute('is-hidden');
 		logo_default.setAttribute('is-hidden', '');
-	});
-	logo_custom.src = url;
 
-	// Set background used in chat preview
-	var url = "url(./server/download.php?class=icechat_background";
-	url += "&fullpath="+encodeURIComponent(me.__domaindetail || '__@@GLOBAL@@__');
-	url += "&uid="+Date.now();
-	url += "&resize=1&width=464)";
-	this.slider_icechat_skin.preview_icechat_skin._main.style.backgroundImage = url;
+		this.button_login_logo_default._disabled(false);
+	}.bind(this));
+	this.button_login_logo_default._disabled(true);
+	logo_custom.src = url;
 
 	/* Form user interaction functionality */
 
@@ -244,6 +187,9 @@ _me._hash_handler=function(){
 	}
 
 	// Social integration
+	me.toggle_website._onchange=function(checked){
+		me.input_website._readonly(checked);
+	};
 	me.toggle_facebook._onchange=function(checked){
 		me.input_facebook._readonly(checked);
 	};
@@ -294,10 +240,12 @@ _me._hash_handler=function(){
 	me.radio_banner_top_url._onchange=
 	me.radio_banner_top_code._onchange=function(checked){
 		me.input_banner_top_url._main.parentNode.setAttribute('is-hidden','1');
+		me.input_banner_top_link._main.parentNode.setAttribute('is-hidden','1');
 		me.input_banner_top_code._main.parentNode.setAttribute('is-hidden','1');
 		switch(me.radio_banner_top._groupValue()) {
 			case 'url':
 				me.input_banner_top_url._main.parentNode.removeAttribute('is-hidden');
+				me.input_banner_top_link._main.parentNode.removeAttribute('is-hidden');
 				break;
 			case 'code':
 				me.input_banner_top_code._main.parentNode.removeAttribute('is-hidden');
@@ -308,10 +256,12 @@ _me._hash_handler=function(){
 	me.radio_banner_bottom_url._onchange=
 	me.radio_banner_bottom_code._onchange=function(checked){
 		me.input_banner_bottom_url._main.parentNode.setAttribute('is-hidden','1');
+		me.input_banner_bottom_link._main.parentNode.setAttribute('is-hidden','1');
 		me.input_banner_bottom_code._main.parentNode.setAttribute('is-hidden','1');
 		switch(me.radio_banner_bottom._groupValue()) {
 			case 'url':
 				me.input_banner_bottom_url._main.parentNode.removeAttribute('is-hidden');
+				me.input_banner_bottom_link._main.parentNode.removeAttribute('is-hidden');
 				break;
 			case 'code':
 				me.input_banner_bottom_code._main.parentNode.removeAttribute('is-hidden');
@@ -321,12 +271,7 @@ _me._hash_handler=function(){
 
 	// Make backgrounds images visible and initialise
 	for(var i in this.__settings.background_urls) {
-		var bg = this['radio_login_background_image_'+i];
-		if(bg) {
-			bg._addcss("is-visible");
-			bg._value(this.__settings.background_urls[i]);
-		}
-		var bg = this['radio_ic_background_image_'+i];
+		var bg = this['radio_background_image_'+i];
 		if(bg) {
 			bg._addcss("is-visible");
 			bg._value(this.__settings.background_urls[i]);
@@ -397,71 +342,6 @@ _me._hash_handler=function(){
 			}
 		]);
 	}
-	me.button_wc_default._onclick=function(){
-		gui.message.warning(getLang("warning::set_to_default"),false,[
-			{
-				value:getLang("generic::cancel"),
-				type:'text borderless',
-				method:'close'
-			},
-			{
-				value:getLang("generic::set_to_default"),
-				type:'success text',
-				onclick:function(closeCallback){
-					me._data.webmail.resetAll(function(r){
-						if(r==1) {
-							me._level.getSettings(obj_whitelabeling.WEBMAIL_SETTINGS,function(wm){
-								me._data.removeItem('webmail');
-								me._data.addItem(wm,'webmail');
-								me._loadWebMail(wm);
-								gui.message.toast(getLang("message::reset_to_default"));
-							});
-						} else {
-							gui.message.error(getLang("error::default_reset_failed"));
-						}
-					});
-					closeCallback();
-				}
-			}
-		]);
-	}
-	me.button_ic_default._onclick=function(){
-		gui.message.warning(getLang("warning::set_to_default"),false,[
-			{
-				value:getLang("generic::cancel"),
-				type:'text borderless',
-				method:'close'
-			},
-			{
-				value:getLang("generic::set_to_default"),
-				type:'success text',
-				onclick:function(closeCallback){
-					me._data.icechat.resetAll(function(r){
-						if(r==1) {
-							me._level.getSettings(obj_whitelabeling.ICECHAT_SETTINGS,function(ic){
-								me._data.removeItem('icechat');
-								me._data.addItem(ic,'icechat');
-								me._loadIceChat(ic);
-							});
-						} else {
-							// Fail!
-						}
-					});
-					closeCallback();
-
-					// Server does not return default values so set manually
-			/*		me.radio_ic_colorpicker_default._checked(true);
-					me.radio_ic_background_image_default_0._checked(true);
-					me.input_ic_background_image_input._value("");
-					var url = "url(./server/download.php?class=icechat_background";
-					url += "&fullpath="+encodeURIComponent(me.__domaindetail || '__@@GLOBAL@@__');
-					url += "&uid="+Date.now();
-					url += "&resize=1&width=464)";
-					me.slider_icechat_skin.preview_icechat_skin._main.style.backgroundImage = url;
-			*/	}
-			}
-		]);
-	}
 
 	// Load using unified settings
 	this._load();
@@ -499,7 +379,7 @@ _me._load = function() {
 		this._activateLogin();
 	}.bind(this));
 
-	// WebMail, WebAdmin and IceChat Settings
+	// WebMail and WebAdmin Settings
 	this._level.getSettings(obj_whitelabeling.COMBINED_SETTINGS,function(s){
 		if(s.error) {
 			gui.message.error(getLang("error::server_failure")+s.error);
@@ -508,27 +388,27 @@ _me._load = function() {
 
 		this._data.addItem(s.layout_settings,'webmail');
 		this._data.addItem(s.layout_settings_admin,'webadmin');
-		this._data.addItem(s.icechat_settings,'icechat');
 
 		// For WebMail, get available design layouts
 		com.settings.get('skins', function(layouts) {
 			for(var id in layouts) {
-				this.__settings.skin[id] = layouts[id];
+				if (id !== 'banner' && id !== 'biggerrtl') {
+					this.__settings.skin[id] = layouts[id];
+				}
 			}
 			this.dropdown_wc_skin._fill(this.__settings.skin);
 
 			// Fill in WebMail section and activate preview
-			this._loadWebMail(s.layout_settings);
+			this._loadWebMail(s);
 			this._activateWebMail();
 		}.bind(this));
 
 		// Fill in WebAdmin section and activate preview
 		this._loadWebAdmin(s.layout_settings_admin);
 		this._activateWebAdmin();
+		this._loadFavicon(s.layout_settings);
 
-		// Fill in IceChat section and activate preview
-		this._loadIceChat(s.icechat_settings);
-		this._activateIceChat();
+		this._loadTheme(s.layout_settings);
 
 	}.bind(this));
 
@@ -555,29 +435,343 @@ _me._load = function() {
 
 }
 
+_me._setTheme = function(sColor, sImagePath) {
+	if (!sColor && !sImagePath) {
+		sColor = this.__sColor;
+		sImagePath = this.__sImagePath;
+	} else if (sColor) {
+		this.__sColor = sColor;
+		this.__sImagePath = '';
+	} else if (sImagePath) {
+		this.__sImagePath = sImagePath;
+	}
+
+	this.slider_login.preview_login._main.style.backgroundColor = this.__sColor;
+	this.slider_webmail_skin.preview_webclient_skin._main.style.backgroundColor = this.__sColor;
+	this.slider_login.preview_login._main.style.backgroundImage = this.__sImagePath ? "url(" + this.__sImagePath + ")" : '';
+	this.slider_webmail_skin.preview_webclient_skin._main.style.backgroundImage = this.__sImagePath ? "url(" + this.__sImagePath + ")" : '';
+
+	if (sImagePath) {
+		extractColors.extractColorsFromSrc(sImagePath, {
+			crossOrigin: 'anonymous'
+		}).then(function(swatches) {
+			var hsl, rgb, hex, hex_light;
+			var mostPopulous = swatches[0];
+			for (var i in swatches) {
+				if (swatches[i].area * swatches[i].saturation > mostPopulous.area * mostPopulous.saturation) {
+					mostPopulous = swatches[i];
+				}
+			}
+
+			hsl = colors.rgb2hsl(mostPopulous.red, mostPopulous.green, mostPopulous.blue);
+
+			rgb = colors.hsl2rgb(hsl[0], hsl[1], 0.45);
+			hex = colors.rgb2hex.apply(colors, rgb);
+
+			this.radio_interface_color_background_background._main.style.color = hex;
+			if (this._data.webmail.login_interface_color.value === 'background') {
+				this.slider_login.preview_login._main.style.setProperty('--theme-color', hex);
+				this.slider_webmail_skin.preview_webclient_skin._main.style.setProperty('--theme-color', hex);
+	
+				rgb = colors.hsl2rgb(hsl[0], hsl[1], 0.95);
+				hex_light = colors.rgb2hex.apply(colors, rgb);
+				this.slider_login.preview_login._main.style.setProperty('--theme-color-light', hex_light);
+				this.slider_webmail_skin.preview_webclient_skin._main.style.setProperty('--theme-color-light', hex_light);
+			}
+		}.bind(this));
+
+		if (this._data.webmail.login_interface_color.value && this._data.webmail.login_interface_color.value !== 'background') {
+			hex = this._data.webmail.login_interface_color.value;
+			this.slider_login.preview_login._main.style.setProperty('--theme-color', hex);
+			this.slider_webmail_skin.preview_webclient_skin._main.style.setProperty('--theme-color', hex);
+
+			hsl = colors.rgb2hsl.apply(colors, colors.hex2rgb(hex));
+			rgb = colors.hsl2rgb(hsl[0], hsl[1], 0.95);
+			hex_light = colors.rgb2hex.apply(colors, rgb);
+			this.slider_login.preview_login._main.style.setProperty('--theme-color-light', hex_light);
+			this.slider_webmail_skin.preview_webclient_skin._main.style.setProperty('--theme-color-light', hex_light);
+		}
+	} else {
+		var hsl = colors.rgb2hsl.apply(colors, colors.hex2rgb(sColor));
+		var rgb = colors.hsl2rgb(hsl[0], hsl[1], 0.45);
+		this.radio_interface_color_background_background._main.style.color = colors.rgb2hex.apply(colors, rgb);
+
+		if (this._data.webmail.login_interface_color.value && this._data.webmail.login_interface_color.value !== 'background') {
+			sColor = this._data.webmail.login_interface_color.value;
+		}
+		hsl = colors.rgb2hsl.apply(colors, colors.hex2rgb(sColor));
+
+		rgb = colors.hsl2rgb(hsl[0], hsl[1], 0.45);
+		var hex = colors.rgb2hex.apply(colors, rgb);
+		this.slider_login.preview_login._main.style.setProperty('--theme-color', hex);
+		this.slider_webmail_skin.preview_webclient_skin._main.style.setProperty('--theme-color', hex);
+
+		rgb = colors.hsl2rgb(hsl[0], hsl[1], 0.95);
+		var hex_light = colors.rgb2hex.apply(colors, rgb);
+		this.slider_login.preview_login._main.style.setProperty('--theme-color-light', hex_light);
+		this.slider_webmail_skin.preview_webclient_skin._main.style.setProperty('--theme-color-light', hex_light);
+
+	}
+}
+
+_me._loadTheme = function(g) {
+	this.slider_login.preview_login._main.classList.remove('cover', 'contain', 'tile', 'center');
+	this.slider_login.preview_login._main.classList.add(g.login_background_wallpaper_size.value || 'cover');
+	this.slider_webmail_skin.preview_webclient_skin._main.classList.remove('cover', 'contain', 'tile', 'center');
+	this.slider_webmail_skin.preview_webclient_skin._main.classList.add(g.login_background_wallpaper_size.value || 'cover');
+
+	this.dropdown_wallpaper_size._setValue(g.login_background_wallpaper_size);
+	this.dropdown_wallpaper_size._fill({
+		cover: getLang('WHITELABELING::WALLPAPER_COVER'),
+		contain: getLang('WHITELABELING::WALLPAPER_CONTAIN'),
+		tile: getLang('WHITELABELING::WALLPAPER_TILE'),
+		center: getLang('WHITELABELING::WALLPAPER_CENTER')
+	});
+	this.dropdown_wallpaper_size._onchange = function(e, dropdown) {
+		this.slider_login.preview_login._main.classList.remove('cover', 'contain', 'tile', 'center');
+		this.slider_login.preview_login._main.classList.add(dropdown.value);
+		this.slider_webmail_skin.preview_webclient_skin._main.classList.remove('cover', 'contain', 'tile', 'center');
+		this.slider_webmail_skin.preview_webclient_skin._main.classList.add(dropdown.value);
+	}.bind(this);
+
+	// Default values when not defined on server
+	g.login_background_name.default = this.__settings.background_urls['default_0'];
+	g.login_interface_color.value = g.login_interface_color.value;
+
+	this.upload_background_file._onfile = function(file) {
+		if(!~['jpg', 'jpeg', 'png', 'gif'].indexOf((file.extension || '').toLowerCase())) {
+			return gui.message.toast(getLang("error::unsupported_image_format"));
+		}
+		var filename = 'background.' + file.extension;
+		g.login_background_name.initialValue = '';
+		g.login_background_name.value = filename;
+		g.login_background_file.value = file.content;
+		this.__backgroundFileContent = file.content;
+		this.__backgroundFileName = filename;
+		g.login_background_color.value = '';
+		g.login_background_solid.value = false;
+
+		var checked = this.radio_background_color_custom._groupValue();
+		if (checked) {
+			(this['radio_background_color_' + checked] || this['radio_background_color_custom'])._checked(false);
+		}
+		var checked = this.radio_background_image_default_0._groupValue();
+		if (checked) {
+			checked = this.__settings._getFileLabel(checked);
+			this['radio_background_image_' + checked]._checked(false);
+		}
+
+		this['radio_background_image_default_-1']._main.querySelector('i').style.backgroundImage = 'url(' + file.content + ')';
+		this['radio_background_image_default_-1']._checked(true);
+
+		this._setTheme(false, file.content);
+
+	}.bind(this);
+	this.upload_background_file._imagesOnly();
+
+	if (g.login_background_solid.value !== 'true' && g.login_background_solid.value !== true) {
+		this._setTheme(false, "./server/download.php?class=background&fullpath=" + encodeURIComponent(this.__domaindetail || '__@@GLOBAL@@__') + "&uid="+Date.now());
+		this.radio_background_image_default_0._setValue(g.login_background_name);
+		if (g.login_background_name.value && !this.__settings._getFileLabel(g.login_background_name.value)) {
+			this.__backgroundFileName = g.login_background_name.value;
+			var url = './server/download.php?class=background&fullpath=' + encodeURIComponent(this.__domaindetail || '__@@GLOBAL@@__') + '&uid=' + Date.now();
+			this['radio_background_image_default_-1']._main.querySelector('i').style.backgroundImage = "url(" + url + ")";
+			this['radio_background_image_default_-1']._checked(true);
+			fetch(url).then((res) => res.blob()).then((blob) => {
+				const reader = new FileReader();
+				reader.onloadend = () => {
+					this.__backgroundFileContent = reader.result;
+				};
+				reader.readAsDataURL(blob);
+			});
+		}
+	} else {
+		this._setTheme(g.login_background_color.value);
+	}
+	this.radio_background_color_custom._setValue(g.login_background_color);
+	if (!this.__settings.background_colors[g.login_background_color.value]) {
+		this.radio_background_color_custom._checked(true);
+		this.radio_background_color_custom._main.style.color = g.login_background_color.value;
+	}
+
+	if (g.login_interface_color.value && !this.__settings.background_colors[g.login_interface_color.value] && g.login_interface_color.value !== 'background') {
+		this.radio_interface_color_custom._main.style.color = g.login_interface_color.value;
+	}
+
+	this.radio_background_color_custom._onclick = function(e) {
+		e.preventDefault();
+		e.stopPropagation();
+		e.stopImmediatePropagation();
+
+		var popup = gui._create('popup', 'obj_popup');
+		popup._init({
+			name: 'colorpicker',
+			heading: {
+				value: getLang('whitelabeling::background_color')
+			},
+			fixed: false,
+			footer: 'obj_accountinfo_changepwd_footer',
+			content: 'obj_colorpicker'
+		});
+		popup.content._load({
+			value: this.__customBackgroundColor || g.login_background_color.value || g.login_background_color.initialValue
+		}, function(v) {
+			this.__customBackgroundColor = v;
+			g.login_background_color.value = v;
+			this._setTheme(v);
+			g.login_background_solid.value = true;
+
+			var checked = this.radio_background_image_default_0._groupValue();
+			this.radio_background_color_custom._main.style.color = v;
+			if (checked) {
+				checked = this.__settings._getFileLabel(checked);
+				this['radio_background_image_' + checked]._checked(false);
+			}
+			var checked = this.radio_background_color_custom._groupValue();
+			if (checked) {
+				(this['radio_background_color_' + checked] || this['radio_background_color_custom'])._checked(false);
+			}
+			this.radio_background_color_custom._checked(true);
+		}.bind(this));
+	}.bind(this);
+
+	this.radio_background_color_custom._groupOnchange = function(v) {
+		g.login_background_color.value = v._value();
+		this._setTheme(v._value());
+		g.login_background_solid.value = true;
+		g.login_background_file.value = null;
+
+		var checked = this.radio_background_image_default_0._groupValue();
+		if (checked) {
+			checked = this.__settings._getFileLabel(checked);
+			this['radio_background_image_' + checked]._checked(false);
+		}
+	}.bind(this);
+
+	this['radio_background_image_default_-1']._onclick = function(e) {
+		e.preventDefault();
+		e.stopPropagation();
+		e.stopImmediatePropagation();
+		if (this.__backgroundFileName) {
+			g.login_background_name.value = this.__backgroundFileName;
+			g.login_background_file.value = this.__backgroundFileContent;
+			this._setTheme(false, this.__backgroundFileContent);
+
+			var checked = this.radio_background_color_custom._groupValue();
+			if (checked) {
+				(this['radio_background_color_' + checked] || this['radio_background_color_custom'])._checked(false);
+			}
+			var checked = this.radio_background_image_default_0._groupValue();
+			if (checked) {
+				checked = this.__settings._getFileLabel(checked);
+				this['radio_background_image_' + checked]._checked(false);
+			}
+
+			setTimeout(function() {
+				this['radio_background_image_default_-1']._checked(true)
+			}.bind(this), 5);
+		} else {
+			this.upload_background_file.__eIN.click();
+		}
+	}.bind(this);
+
+	this.radio_background_image_default_0._groupOnchange = function(v) {
+		g.login_background_color.value = '';
+		g.login_background_solid.value = false;
+		g.login_background_file.value = null;
+
+		this._setTheme(false, './client/skins/default/login/images/'+v._value());
+
+		var checked = this.radio_background_color_custom._groupValue();
+		if (checked) {
+			(this['radio_background_color_' + checked] || this['radio_background_color_custom'])._checked(false);
+		}
+	}.bind(this);
+
+	this.radio_interface_color_custom._onclick = function(e) {
+		e.preventDefault();
+		e.stopPropagation();
+		e.stopImmediatePropagation();
+
+		var popup = gui._create('popup', 'obj_popup');
+		popup._init({
+			name: 'colorpicker',
+			heading: {
+				value: getLang('whitelabeling::interface_color')
+			},
+			fixed: false,
+			footer: 'obj_accountinfo_changepwd_footer',
+			content: 'obj_colorpicker'
+		});
+		popup.content._load({
+			theme: true,
+			lightness: false,
+			hex: false,
+			value: this.__customInterfaceColor || (g.login_interface_color.value === 'background' ? '' : g.login_interface_color.value) || g.login_interface_color.initialValue
+		}, function(v) {
+			this.__customInterfaceColor = v;
+			g.login_interface_color.value = v;
+			this.radio_interface_color_custom._main.style.color = v;
+			this._setTheme();
+			this.radio_interface_color_background_background._checked(false);
+			var checked = this.radio_interface_color_custom._groupValue();
+			if (checked) {
+				this['radio_interface_color_' + checked]._checked(false);
+			}
+			this['radio_interface_color_custom']._checked(true);
+		}.bind(this));
+	}.bind(this);
+	
+	this.radio_interface_color_custom._groupOnchange = function(v) {
+		g.login_interface_color.value = v._value();
+		this._setTheme();
+
+		this.radio_interface_color_background_background._checked(false);
+	}.bind(this);
+
+	this.radio_interface_color_background_background._groupOnchange = function(v) {
+		g.login_interface_color.value = 'background';
+		this._setTheme();
+
+		var checked = this.radio_interface_color_custom._groupValue();
+		if (checked) {
+			(this['radio_interface_color_' + checked] || this['radio_interface_color_custom'])._checked(false);
+		}
+	}.bind(this);
+
+	if (g.login_interface_color.value === 'background') {
+		this.radio_interface_color_background_background._checked(true);
+	} else if (!this.__settings.background_colors[g.login_interface_color.value]) {
+		this.radio_interface_color_custom._checked(true);
+	} else {
+		this['radio_interface_color_' + g.login_interface_color.value]._checked(true);
+	}
+	// this.radio_interface_color_custom._setValue(g.login_interface_color);
+
+	// this.
+};
+
 // Fill in values into Login section
 _me._loadLogin = function(l) {
 	var r = l.restrictions;
 	var f = l.custom_login_fields;
 	var g = l.layout_settings;
 
-	// Default values when not defined on server
-	g.login_color.default = 'default';
-	g.login_background_name.default = this.__settings.background_urls['default_0'];
-
 	// Some restriction have values that appear opposite to UI elements
 	r.disable_languages.inversed = true;
 	r.disable_remember.inversed = true;
 	r.disable_autofill.inversed = true;
 	r.disable_signup.inversed = true;
+	r.disable_guest_signin.inversed = true;
+	r.website_disabled.inversed = true;
 	r.facebook_disabled.inversed = true;
 	r.twitter_disabled.inversed = true;
 	r.linkedin_disabled.inversed = true;
 
-	// Logo, colour and background for Login
+	// Logo for Login
 	var preview = this.slider_login.preview_login;
 	this.upload_logo_file._onfile = function(file) {
-		if(!~['jpg', 'png', 'gif', 'svg'].indexOf((file.extension || '').toLowerCase())) {
+		if(!~['jpg', 'jpeg', 'png', 'gif', 'svg'].indexOf((file.extension || '').toLowerCase())) {
 			return gui.message.toast(getLang("error::unsupported_image_format"));
 		}
 		// Set uploaded logo as preview logo and show it
@@ -589,34 +783,27 @@ _me._loadLogin = function(l) {
 		svg.setAttribute('is-hidden',1);
 		// Save upload filename
 		g.logo_file.value = file.content;
+		g.login_logo_name.initialValue = '';
 		g.login_logo_name.value = 'login_logo.'+file.extension;
+		this.button_login_logo_default._disabled(false);
 	}.bind(this);
-	this.upload_background_file._onfile = function(file) {
-		if(!~['jpg', 'png', 'gif'].indexOf((file.extension || '').toLowerCase())) {
-			return gui.message.toast(getLang("error::unsupported_image_format"));
-		}
-		var filename = 'login_background.' + file.extension;
-		g.login_background_name.initialValue = '';
-		g.login_background_name.value = filename;
-		g.login_background_file.value = file.content;
-		// Uncheck any previously selected background after upload
-		var checked = this.radio_login_background_image_default_0._groupValue();
-		if(checked) {
-			checked = this.__settings._getFileLabel(checked);
-			this['radio_login_background_image_'+checked]._checked(false);
-		}
-		// Set uploaded image in preview
-		preview._main.style.backgroundImage = 'url('+file.content+')';
 
+	this.button_login_logo_default._onclick = function() {
+		this.button_login_logo_default._disabled(true);
+		var img = preview._main.getElementsByTagName('img')[0];
+		var svg = img.parentNode.firstElementChild;
+		svg.removeAttribute('is-hidden');
+		img.setAttribute('is-hidden', 1);
+		g.logo_file.value = '';
+		g.login_logo_name.value = '';
 	}.bind(this);
-	this.radio_login_colorpicker_default._setValue(g.login_color);
-	this.radio_login_background_image_default_0._setValue(g.login_background_name);
 
 	// Set restriction values
 	this.toggle_disable_language._setValue(r.disable_languages);
 	this.toggle_disable_remember_me._setValue(r.disable_remember);
 	this.toggle_disable_autofill._setValue(r.disable_autofill);
 	this.toggle_disable_sign_up._setValue(r.disable_signup);
+	this.toggle_disable_guest_signin._setValue(r.disable_guest_signin);
 	// Restrictions for Support
 	var disable_support = this.toggle_disable_support_link;
 	var require_phone = this.checkbox_phone_required;
@@ -673,6 +860,8 @@ _me._loadLogin = function(l) {
 	}
 
 	// Social integration
+	this.toggle_website._setValue(r.website_disabled);
+	this.input_website._setValue(g.website_link);
 	this.toggle_facebook._setValue(r.facebook_disabled);
 	this.input_facebook._setValue(g.facebook_link);
 	this.toggle_twitter._setValue(r.twitter_disabled);
@@ -681,42 +870,27 @@ _me._loadLogin = function(l) {
 	this.input_linkedin._setValue(g.linkedin_link);
 
 	// Set preview according to values
-	var preview = this.slider_login.preview_login;
-
-	preview._main.style.backgroundImage = "url(./server/download.php?class=background&fullpath=" + encodeURIComponent(this.__domaindetail || '__@@GLOBAL@@__') + "&uid=" + Date.now() + "&resize=1&width=572)";
-	preview._main.className = preview._main.className.replace(/skin-[-a-z]+/,'');
-	preview._main.classList.add('skin-'+(g.login_color.value || g.login_color.default));
+	// preview._main.style.backgroundImage = "url(./server/download.php?class=background&fullpath=" + encodeURIComponent(this.__domaindetail || '__@@GLOBAL@@__') + "&uid=" + Date.now() + ")";
+	// preview._main.className = preview._main.className.replace(/skin-[-a-z]+/,'');
 
 	preview._enable('language',r.disable_languages!=1);
 	preview._enable('remember',r.disable_remember!=1);
 	preview._enable('autofill',r.disable_autofill!=1);
 	preview._enable('sign_up',r.disable_signup);
+	preview._enable('guest_signin',r.disable_guest_signin);
 	preview._enable('support',r.contact_support==1);
 	preview._enable('search',r.show_search==1);
 
+	preview._enable('website',r.website_disabled!=1);
 	preview._enable('facebook',r.facebook_disabled!=1);
 	preview._enable('twitter',r.twitter_disabled!=1);
 	preview._enable('linkedin',r.linkedin_disabled!=1);
 
 }
+
 // Activate preview logic for Login
 _me._activateLogin = function() {
 	var preview = this.slider_login.preview_login;
-	var main = preview._main;
-	var file = this._data.login.layout_settings.login_background_file;
-	var name = this._data.login.layout_settings.login_background_name;
-
-	this.radio_login_colorpicker_default._groupOnchange = function(v) {
-		main.className = main.className.replace(/skin-[-a-z]+/,'');
-		main.classList.add('skin-'+v._value());
-	}
-
-	this.radio_login_background_image_default_0._groupOnchange = function(v) {
-		main.style.backgroundImage = 'url(./client/skins/default/login/images/'+v._value()+')';
-		// Reset file data in case image was just uploaded
-		file.value = null;
-	}
-
 	this.toggle_disable_language._onchange = function(check) {
 		preview._enable('language',check);
 	}
@@ -729,6 +903,9 @@ _me._activateLogin = function() {
 	this.toggle_disable_sign_up._onchange = function(check) {
 		preview._enable('sign_up',check);
 	}
+	this.toggle_disable_guest_signin._onchange = function(check) {
+		preview._enable('guest_signin',check);
+	}
 	this.toggle_disable_support_link._onchange = function(check) {
 		preview._enable('support',check);
 	}
@@ -736,6 +913,7 @@ _me._activateLogin = function() {
 		preview._enable('search',check);
 	}
 
+	this.toggle_website._onchange =
 	this.toggle_facebook._onchange =
 	this.toggle_twitter._onchange =
 	this.toggle_linkedin._onchange =
@@ -747,17 +925,26 @@ _me._activateLogin = function() {
 
 // Fill in values into WebClient section
 _me._loadWebMail = function(wm) {
-	wm.skin.default = 'default';
-	wm.skin_style.default = 'default';
+	var me = this;
+	var skin = wm.layout_settings.skin;
+	skin.default = 'default';
+	if (skin.currentValue === 'banner') {
+		skin.currentValue = 'default';
+	} else if (skin.currentValue === 'biggerrtl') {
+		skin.currentValue = 'bigger';
+	}
+	if (skin.initialValue === 'banner') {
+		skin.initialValue = 'default';
+	} else if (skin.initialValue === 'biggerrtl') {
+		skin.initialValue = 'bigger';
+	}
 
-	this.input_wc_page_title._setValue(wm.title);
-	this.dropdown_wc_skin._setValue(wm.skin);
-	this.radio_wc_colorpicker_default._setValue(wm.skin_style);
+	this.input_wc_page_title._setValue(wm.layout_settings.title);
+	this.dropdown_wc_skin._setValue(skin);
 
 	// Set preview according to values
 	var preview = this.slider_webmail_skin.preview_webclient_skin;
-	preview._setAttribute('skin',wm.skin.value || wm.skin.default);
-	preview._setAttribute('style',wm.skin_style.value || wm.skin_style.default);
+	preview._setAttribute('skin',skin.value || skin.default);
 }
 // Activate preview logic for WebMail
 _me._activateWebMail = function() {
@@ -768,9 +955,6 @@ _me._activateWebMail = function() {
 			preview._setAttribute('skin',e.target.value);
 		}
 	}
-	this.radio_wc_colorpicker_default._groupOnchange = function(v) {
-		preview._setAttribute('style',v._value());
-	}
 }
 
 // Fill in values into WebAdmin section
@@ -792,55 +976,6 @@ _me._activateWebAdmin = function() {
 	}
 }
 
-// Fill in values into IceChat section
-_me._loadIceChat = function(ic) {
-	ic.login_color.default = 'default';
-	ic.login_background_name.default = this.__settings.background_urls['default_0'];
-
-	this.radio_ic_colorpicker_default._setValue(ic.login_color);
-	this.radio_ic_background_image_default_0._setValue(ic.login_background_name);
-
-	var preview = this.slider_icechat_skin.preview_icechat_skin;
-	this.upload_ic_upload_image._onfile = function(file) {
-		if(!~['jpg', 'png', 'gif'].indexOf((file.extension || '').toLowerCase())) {
-			return gui.message.toast(getLang("error::unsupported_image_format"));
-		}
-		// Custom filename
-		var filename = 'login_background.' + file.extension;
-		ic.login_background_name.value = filename;
-		ic.login_background_file.value = file.content;
-		// Uncheck any previously selected background after upload
-		var checked = this.radio_ic_background_image_default_0._groupValue();
-		if(checked) {
-			checked = this.__settings._getFileLabel(checked);
-			this['radio_ic_background_image_'+checked]._checked(false);
-		}
-		// Set uploaded image in preview
-		preview._main.style.backgroundImage = 'url('+file.content+')';
-	}.bind(this);
-
-	// Set preview according to values
-	var preview = this.slider_icechat_skin.preview_icechat_skin;
-
-}
-// Activate preview logic for IceChat
-_me._activateIceChat = function() {
-	var preview = this.slider_icechat_skin.preview_icechat_skin;
-	var main = preview._main;
-	var file = this._data.icechat.login_background_file;
-
-	this.radio_ic_colorpicker_default._groupOnchange = function(v) {
-		main.className = main.className.replace(/skin-[a-z]+/,'');
-		main.classList.add('skin-'+v._value());
-	}
-
-	this.radio_ic_background_image_default_0._groupOnchange = function(v) {
-		main.style.backgroundImage = 'url(./client/skins/default/login/images/icechat/'+v._value()+')';
-		// Reset file data in case image was just uploaded
-		file.value = null;
-	}
-}
-
 // Fill in values into Banners section
 _me._loadBanners = function(s) {
 	this.toggle_adsense._setValue(s.restrictions.enable_adsense);
@@ -864,10 +999,12 @@ _me._loadBanners = function(s) {
 
 	this.radio_banner_top._setValue(b.top_type);
 	this.input_banner_top_url._setValue(b.top_url);
+	this.input_banner_top_link._setValue(b.top_link);
 	this.input_banner_top_code._setValue(b.top_code);
 
 	this.radio_banner_bottom._setValue(b.below_type);
 	this.input_banner_bottom_url._setValue(b.below_url);
+	this.input_banner_bottom_link._setValue(b.below_link);
 	this.input_banner_bottom_code._setValue(b.below_code);
 }
 
@@ -878,6 +1015,7 @@ _me._loadConferencing = function(c) {
 			return;
 		}
 		c.jitsi_logo_file.value = file.content;
+		c.jitsi_logo_name.initialValue = '';
 		c.jitsi_logo_name.value = 'jitsi_logo.'+file.extension;
 		this._getAnchor('jitsi_logo_preview').src = file.content;
 		this._getAnchor('fi_jitsi_logo_default').classList.remove('hide');
@@ -899,6 +1037,36 @@ _me._loadConferencing = function(c) {
 	this.input_jitsi_redirect_url._setValue(c.jitsi_redirect_url);
 }
 
+// Fill in values into Conferencing section
+_me._loadFavicon = function(c) {
+	this.upload_favicon_file._onfile = function(file) {
+		if ((file.extension || '').toLowerCase() !== 'png') {
+			return;
+		}
+
+		c.login_favicon_file.value = file.content;
+		c.login_favicon_name.initialValue = '';
+		c.login_favicon_name.value = 'favicon.png';
+		this._getAnchor('favicon_preview').src = file.content;
+		this._getAnchor('fi_favicon_default').classList.remove('hide');
+	}.bind(this);
+
+	if (c.login_favicon_name.value) {
+		this._getAnchor('favicon_preview').src = './server/download.php?class=favicon&fullpath=' + (this.__domaindetail || '__@@GLOBAL@@__') + '&uid=' + (+new Date());
+		this._getAnchor('fi_favicon_default').classList.remove('hide');
+	} else {
+		this._getAnchor('favicon_preview').src = './server/download.php?class=favicon&fullpath=' + (this.__domaindetail || '__@@GLOBAL@@__') + '&uid=1';
+		this._getAnchor('fi_favicon_default').classList.add('hide');
+	}
+
+	this.button_favicon_default._onclick = function() {
+		c.login_favicon_file.value = void 0;
+		c.login_favicon_name.value = void 0;
+		this._getAnchor('favicon_preview').src = './server/download.php?class=favicon&fullpath=' + (this.__domaindetail || '__@@GLOBAL@@__') + '&uid=1';
+		this._getAnchor('fi_favicon_default').classList.add('hide');
+	}.bind(this);
+}
+
 _me._issaved = function() {
 	return !this._data.hasChanged();
 }
