@@ -14,6 +14,9 @@ Account.USER = '0';
 Account.GROUP = '7';
 Account.RESOURCE = '8';
 Account.MAILINGLIST = '1';
+Account.STATIC_ROUTE = '4';
+Account.DOMAIN = 'd';
+Account.BULK_USERS = 'b';
 
 // Convenience access to properties for accounts
 Account.prototype.getProperty = function(props,cb,opts) {
