@@ -302,6 +302,15 @@ var wm_generic = (function () {
         }
         this.create_iq(aRequest, [this, '__response', [aHandler]]);
     };
+    wm_generic.prototype.getDNSZoneFile = function (sDomain, aHandler) {
+        var aRequest = {
+            commandname: [{ VALUE: 'getdnszonefile' }],
+            commandparams: [{
+                domain: [{ VALUE: sDomain }]
+            }]
+        };
+        this.create_iq(aRequest, [this, '__response', [aHandler]]);
+    };
     /**
         * Response preprocessor
         * @param preprocessor
