@@ -401,7 +401,7 @@ _me.change_user_password = function(account, pwd, old_pwd, cb) {
 	return true;
 }
 
-_me.expire_password=function(account,cb){
+_me.expire_password=function(account,cb, params){
 	var cb=cb;
 	var aRequest = {
 		commandname:[{VALUE:'expireaccountpassword'}],
@@ -409,6 +409,10 @@ _me.expire_password=function(account,cb){
 			accountemail:[{VALUE:account}],
 		}]
 	};
+
+	for(var i in params || {}) {
+		aRequest.commandparams[0][i] = [{VALUE:params[i]}];
+	}
 	
 	this.create_iq(aRequest,[this,'__response',[[function(aResponse){
 		//log.info(['expire',aResponse]);
@@ -796,6 +800,12 @@ _me.addAllToGroup=function(groups,domain,filter,aHandler){
 	if(filter.typemask){
 		masks.typemask=[{VALUE:filter.typemask}];
 	}
+	if(filter && filter.planmask){
+		masks.planmask=[{VALUE:filter.planmask}];
+	}
+	if(filter && filter.servicemask){
+		masks.servicemask=[{VALUE:filter.servicemask}];
+	}
 	
 	if(typeof groups != 'object'){
 		groups=[groups];
