@@ -7,7 +7,19 @@
     }
 }
 require_once(SHAREDLIB_PATH.'system.php');
- define('CLIENT','client');
+ if(!defined('CLIENT')){
+	define('CLIENT','client');
+}
+if(!defined('APP')){
+	define('APP', 'SIGNUP');
+}
+
+if (ini_get('sentry.enabled')) {
+	require_once(SHAREDLIB_PATH . 'vendor/autoload.php');
+	\Sentry\init([
+	'dsn' => 'https://cb525daef5ee40c9663dd153ef1dd7c4@sentry.icewarp.com/24'
+	]);
+}
 
 class SignupAncestor
 {
@@ -66,7 +78,7 @@ class SignupAncestor
 	public $base_login;
 	public $self;
 	public $api;
-
+	public $SentryConfig;
 
     public function __construct()
 	{
@@ -252,6 +264,8 @@ class SignupAncestor
 		
 		$this->templateData['login_action'] = $login_action?$login_action:'basic/';
 
+		$this->sanitize($this->templateData);
+		
 		return $this->templateData;
 	}
 	
@@ -285,23 +299,9 @@ class SignupAncestor
 							 || isset($_REQUEST['frm']) || isset($_GET['!'])
 							 ) && $this->licenseType!='simple';
 
-		if (isset($_GET['meeting']) && !isset($_GET['member']) && (!isset($_COOKIE['lastUsername']) || isset($_GET['force']) || $this->OSCode=='device')){
-			$this->meeting = $_GET['meeting'];
-		}
-
 		if(isset($_GET['pda'])){$this->pda=true;}
         $browserInfo = $this->browser_info();
-		if (isset($_GET['videoBox'])) {
-			$this->template = $this->getTemplateFile($defaultSkin, 'video.box.tpl', CLIENT);
-		} elseif ($this->meeting) {
-            
-			 			if (!isset($_GET['old'])) {
-				 
-				$this->template = $this->getTemplateFile($defaultSkin, 'meeting.webrtc.tpl', CLIENT);
-			} else {
-				$this->template = $this->getTemplateFile($defaultSkin, 'meeting.tpl', CLIENT);
-			}
-		} elseif ($this->directLogin) {
+		if ($this->directLogin) {
 			 			if (isset($_COOKIE['login_sid'])) {
 				$this->templateData['activeSID'] = $_COOKIE['login_sid'];
 			}
@@ -347,16 +347,33 @@ class SignupAncestor
 
 			$this->template = $this->getTemplateFile($defaultSkin, 'advanced.tpl', CLIENT);
             
-		} elseif (($this->pda && !$this->desktop)) {
-			 			$this->template = $this->getTemplateFile($defaultSkin, $this->forcedTemplate ? $this->forcedTemplate : 'mobile.tpl', CLIENT);
 		} else {
-			 			if(($browserInfo['browser'] == 'msie' && $browserInfo['version'] <= 8)){
-				return $this->template = $this->getTemplateFile($defaultSkin, $this->forcedTemplate ? $this->forcedTemplate : 'mobile.tpl', CLIENT);
-			}
 			$this->template = $this->getTemplateFile($defaultSkin, $this->forcedTemplate ? $this->forcedTemplate : 'pc.tpl', CLIENT);
 		}
 		return $this->template;
 	}
+
+	protected function getDefaultIconPath() {
+		if (defined('WORKING_MODEL_PATH')) {
+			require_once(WORKING_MODEL_PATH . 'model/settings.php');
+		} else {
+			require_once('inc/exception.php');
+			require_once('inc/tools.php');
+			require_once('../webmail/basic/model/settings.php');
+		}
+		$this->api = IceWarpAPI::instance('api');
+		
+		$request = slRequest::instance();
+		$session = slSession::instance($request,false);
+		$sets = WebmailSettings::instance($session);
+		$layoutData = $sets->getPublic('layout_settings');
+
+		$login_favicon_uid = @$layoutData['@childnodes']['item'][0]['@childnodes']['login_favicon_uid'][0]['@value'];
+		if ($login_favicon_uid){
+			return 'server/download.php?class=favicon&fullpath=' . urlencode($_SERVER['HTTP_HOST']) . '&uid=' . $login_favicon_uid . '&resize=1';
+		}
+	}
+
 	private function errorHandler()
 	{
 		if (isset($_GET['eid'])){
@@ -465,7 +482,7 @@ class SignupAncestor
 			$this->cookie->setcookie('permanentLogin','',mktime(0,0,0,1,1,2000),'/');
 			unset($_COOKIE['permanentLogin']);
 		}
-		 		if (!isset($_GET['*']) && isset($_COOKIE['permanentLogin'])) {
+		 		if (!isset($_GET['*']) && !empty($_COOKIE['permanentLogin'] ?? null)) {
 			$this->permanentLogin=$_COOKIE['permanentLogin'];
 			if ($this->interface!='pro' && !isset($_GET['sid']) && !isset($_COOKIE['login_sid']) && !isset($_REQUEST['atoken'])){
 				$_POST['to']=$this->interface;
@@ -606,6 +623,7 @@ class SignupAncestor
 
 	public function noJSLogin()
 	{
+        $this->checkNoJsTwoFactorAuth();
 		 
 		if (isset($_POST['to']) && !isset($_POST['contact'])){
 			if ($this->encodeCredentials){
@@ -832,11 +850,6 @@ class SignupAncestor
 			$this->username='';
 		}
 		if (isset($_GET['interface'])) {
-			switch($_GET['interface']){
-				case 'pda':
-					$this->forcedTemplate = 'mobile.tpl';
-					break;
-			}
 			$this->interface=$_GET['interface'];
 		}
 		
@@ -845,10 +858,9 @@ class SignupAncestor
 		}
 		$this->settings['logging_type']=(int) $serverData->item->logging_type;
 		$this->language_data['login_screen']['copy']="<span title=\"".$version."\">".$this->language_data['login_screen']['copy'].' &copy; 1999-'.date('Y')."</span>";  		$this->language_data['login_screen']['copy_new']="<span class=\"copyright\">".$this->language_data['login_screen']['copy_new'].' &copy;'.date('Y')."</span>";  		$this->templateData['version'] = $version;
-		$this->settings['layout']['login_logo']=$this->webmail_url.'server/download.php?class=logo&fullpath='.urlencode($_SERVER['HTTP_HOST']);
+		$this->settings['layout']['login_logo']='server/download.php?class=logo&fullpath='.urlencode($_SERVER['HTTP_HOST']);
 		$this->settings['main']=(array)$sets;
-		 		WebmailIqAuth::checkServerKeys();
-		$this->hash = @file_get_contents(WM_CONFIGPATH.'public.key');
+		 		$this->hash = WebmailRSA::instance()->getServerData('public');
 		
 		 		
 		$this->facebook_disabled = $restrictionsData['@childnodes']['item'][0]['@childnodes']['facebook_disabled'][0]['@value'];
@@ -1109,13 +1121,42 @@ class SignupAncestor
             if (!preg_match('/\/$/',$uri))
                 $uri = dirname($uri).'/';
 
-			$base = 'http'.($HTTPS ? 's' : '').'://'.$_SERVER['HTTP_HOST'].'/-.._._.--.._'.max(filemtime(CLIENT.'/inc/javascript.js'), filemtime(CLIENT.'/inc/startscript.js')).$uri;
+			if ($serverData->item->cdn_path) {
+				$version = $this->api->GetProperty('C_Version');
+				preg_match('/(\d+\.?)+/', $version, $version);
+				$cdnPath = $serverData->item->cdn_path . $version[0] . '/';
+			}
+
+			$base = filter_var($cdnPath ?? ('http'.($HTTPS ? 's' : '').'://'.$_SERVER['HTTP_HOST'].'/-.._._.--.._'.max(filemtime(CLIENT.'/inc/javascript.js'), filemtime(CLIENT.'/inc/startscript.js')).$uri), FILTER_VALIDATE_URL);
+
+			$this->SentryConfig = '';
+			$sentryEnabled = filter_var($serverData->item->sentry_enabled ?? false, FILTER_VALIDATE_BOOLEAN);
+			if ($sentryEnabled) {
+				$this->SentryConfig = "window.SentryEnabled = true;\n";
+				$sentryEnv = $serverData->item->sentry_environment ?? false;
+				if ($sentryEnv) {
+					$this->SentryConfig .= "window.SentryEnvironment = '" . $sentryEnv . "';\n";
+				}
+				$sentryDSN = filter_var($serverData->item->sentry_webmail_dsn ?? '', FILTER_VALIDATE_URL);
+				if ($sentryDSN) {
+					$this->SentryConfig .= "window.SentryWebmailDSN = '" . $sentryDSN . "';\n";
+				}
+				$sentryLoginDSN = filter_var($serverData->item->sentry_login_dsn ?? '', FILTER_VALIDATE_URL);
+				if ($sentryLoginDSN) {
+					$this->SentryConfig .= "window.SentryLoginDSN = '" . $sentryLoginDSN . "';\n";
+				}
+				$sentryAdminDSN = filter_var($serverData->item->sentry_admin_dsn ?? '', FILTER_VALIDATE_URL);
+				if ($sentryAdminDSN) {
+					$this->SentryConfig .= "window.SentryAdminDSN = '" . $sentryAdminDSN . "';\n";
+				}
+			}
+
 			$bundleFile = file_exists('bundle.min.js')?'bundle.min.js':'bundle.js';
 			$frameworkFile = file_exists('framework.min.js')?'framework.min.js':'framework.js';
-            $base_login = 'http'.($HTTPS ? 's' : '').'://'.$_SERVER['HTTP_HOST'].'/-.._._.--.._'.max(filemtime($bundleFile), filemtime($frameworkFile)).$uri;
+            $base_login = filter_var('http'.($HTTPS ? 's' : '').'://'.$_SERVER['HTTP_HOST'].'/-.._._.--.._'.max(filemtime($bundleFile), filemtime($frameworkFile)).$uri, FILTER_VALIDATE_URL);
             if ($_SERVER["REQUEST_URI"]!='') {$ruri=$_SERVER["REQUEST_URI"].'/';} else {$ruri='';}
             
-            $self= 'http'.($HTTPS ? 's' : '').'://'.$_SERVER['HTTP_HOST'].$uri;
+            $self= filter_var('http'.($HTTPS ? 's' : '').'://'.$_SERVER['HTTP_HOST'].$uri, FILTER_VALIDATE_URL);
         }
         $this->base = $base;
         $this->base_login = $base_login;
@@ -1133,9 +1174,14 @@ class SignupAncestor
         if(defined('SECURE_COOKIE') && SECURE_COOKIE){
        		header('Strict-Transport-Security: max-age=31536000; includeSubDomains; preload');
         }
+		if (!defined('APP_PATH')) {
+			define('APP_PATH', '../webmail/');
+		}
 
         $xml = simplexml_load_file($this->api->GetProperty('C_ConfigPath').'_webmail/server.xml');
-        if(!filter_var($xml->item->allow_iframe, FILTER_VALIDATE_BOOLEAN)) header('X-Frame-Options: SAMEORIGIN');
+        if(!filter_var($xml->item->allow_iframe, FILTER_VALIDATE_BOOLEAN)) {
+			header("Content-Security-Policy: frame-ancestors 'self' *.icewarp.com");
+		}
     }
     
     private function handleExternal()
@@ -1194,5 +1240,31 @@ class SignupAncestor
 		}
 		 
     }
+
+    protected function checkNoJsTwoFactorAuth()
+    {
+        if(!isset($_POST['iw_username'])) return;
+        $merakAccount = createobject('account');
+        $merakAccount->Open($_POST['iw_username']);
+        if(filter_var($merakAccount->getProperty("U_2F_ENABLED"), FILTER_VALIDATE_BOOLEAN)) {
+            header("Location: ./?eid=auth_two_factor_invalid&_s[action]=error");
+            exit();
+        }
+    }
+
+	protected function sanitize(&$data) {
+		if (is_object($data)) {
+			foreach($data as $key => $value) {
+				$data->$key = $this->sanitize($value);
+			}
+		} elseif (is_array($data)) {
+			foreach($data as $key => $value) {
+				$data[$key] = $this->sanitize($value);
+			}
+		} elseif (is_string($data)) {
+			return str_replace('<', '&lt;', $data);
+		}
+		return $data;
+	}
 }
 ?>
\ No newline at end of file
