@@ -1,7 +1,7 @@
 #!/bin/bash
 #
 # IceWarp Server
-# Copyright (c) 2008-2022 IceWarp Ltd. All rights reserved.
+# Copyright (c) 2008-2025 IceWarp Ltd. All rights reserved.
 #
 # http://www.icewarp.com
 #
@@ -32,6 +32,6 @@ LOGDIR=$(get_api_variable system c_system_storage_dir_logpath)
 mkdir -p "${LOGDIR}/purple"
 
 cd "${IWS_INSTALL_DIR}/purple" &> /dev/null
-"${IWS_INSTALL_DIR}/purple/purpleserv" &> "${LOGDIR}/purple/purplestartup.log" & echo $! > "${IWS_INSTALL_DIR}/var/purpleserv.pid"
+"${IWS_INSTALL_DIR}/purple/purpleserv" "$@" &> "${LOGDIR}/purple/purplestartup.log" & echo $! > "${IWS_INSTALL_DIR}/var/purpleserv.pid"
 
 exit 0
