@@ -102,6 +102,8 @@ class ChallengeApplication
 			foreach($this->bypass as $key => $val){
 				$this->bypass[$key] = trim($val);
 			}
+		}else{
+			$this->bypass = [];
 		}
 		if(!$enabled){
 			$this->error('reports_are_disabled');
@@ -275,9 +277,9 @@ class ChallengeApplication
 						$this->message('Spam reports start URI: executed from console',4);
 					}
 					$this->cache = ChallengeCache::instance();
-					$this->report = Report::instance( $this->cache );
-					$this->spam = Spam::instance( $this->cache );
-					$this->quarantine = Quarantine::instance( $this->cache );
+					$this->report = Report::instance( 'Report', 'report', $this->cache );
+					$this->spam = Spam::instance( 'Spam', 'spam', $this->cache );
+					$this->quarantine = Quarantine::instance('Quarantine', 'quarantine', $this->cache );
 					$this->admin = ChallengeCacheTable::instance(
 						'ChallengeCacheTable',
 						'admin',
@@ -309,6 +311,8 @@ class ChallengeApplication
 	{
 		 		switch ($action) {
 			case 'report':
+				$activeAccounts = 0;
+				$adminAccount = 0;
 				$cache = ChallengeCache::instance();
 				if ($this->adminAccounts) {
 					$this->processSpamAdmin($this->adminAccounts);
@@ -317,8 +321,12 @@ class ChallengeApplication
 					$this->report->delete($this->id);
 				}
 				$api = IcewarpAPI::instance();
-				$activeAccounts = count($this->activeAccounts);
-				$adminAccounts = count($this->adminAccounts);
+				if(is_countable($this->activeAccounts)){
+					$activeAccounts = count($this->activeAccounts);
+				}
+				if(is_countable($this->adminAccounts)){
+					$adminAccounts = count($this->adminAccounts);
+				}
 				$time = $this->duration();
 				if( $this->getLogging() > 0 ){
 					$api->DoLog(
