@@ -104,8 +104,7 @@ class ChallengeCache extends slCache
 			
 			$this->query('CREATE INDEX IDX_itm_spam_report_id ON spam (itm_report_id)');
 			$this->query('CREATE INDEX IDX_itm_quarantine_report_id ON quarantine (itm_report_id)');
-            $this->commit();
-            $this->transaction();
+
 			 			$stmt = $this->prepare('INSERT INTO challenge_metadata (item_key, item_value) ' .
 				'VALUES (?, ?)');
 			$stmt->execute(array('db_version', strval(CHALLENGE_DB_VERSION)));
@@ -172,9 +171,6 @@ class ChallengeCache extends slCache
 				);
 			}
 		}
-		$this->commit();
-		
-		$this->transaction();
 		 		try{
 			$stmt = $this->prepare('UPDATE challenge_metadata SET item_value = ? WHERE item_key = ?');
 			$stmt->execute(array(strval(CHALLENGE_DB_VERSION), 'db_version'));
@@ -582,7 +578,7 @@ class ChallengeCacheTable
 			unset($new[$sender]);
 		}
 
-		if(count($new)){
+		if(is_countable($new) && count($new)){
 			$hasRecent  = true;
 		}else{
 			$hasRecent = false;
