@@ -4,7 +4,7 @@ require_once('cache.php');
 
 class Report extends ChallengeCacheTable
 {
-	static public function instance( &$cache )
+	static public function instance( $class = "Report", $table = 'report', &$cache = null )
 	{
 		return parent::instance( 'Report', 'report', $cache );
 	}
