@@ -73,7 +73,8 @@ class caldav_session extends webdav_session
 			$destination_resource = resource::get_resource($this->_datastore, $request->_destination_uri);
 			if ($_SERVER['HTTP_OVERWRITE'] == 'F')
 				throw new precondition_failed_exception();
-			$destination_resource->delete();
+
+			$destination_resource->delete($_SERVER['HTTP_SCHEDULE_REPLY'] != 'F');
 			$overwritten = true;
 		}
 		catch (not_found_exception $e)
@@ -99,17 +100,16 @@ class caldav_session extends webdav_session
 			$location = 'Location: http://';
 		else
 			$location = 'Location: https://';
+
 		if (isset($_SERVER['HTTP_HOST']))
 			$location .= $_SERVER['HTTP_HOST'];
 		else
 			$location .= $_SERVER['SERVER_NAME'];
+
 		$location .= $request->_destination_uri->to_string();
 		$headers = array($location);
-
-		if ($overwritten)
-			$response->http_response('204 No content', $headers);
-		else
-			$response->http_response('201 Created', $headers);
+ 
+		$response->http_response('201 Created', $headers);
 	}
 
 	function handle_copy($request, $response)
@@ -406,12 +406,13 @@ class caldav_session extends webdav_session
 			$data = $dav_resource->get_content();
 			 			if (is_resource($data))
 			{
-				$data_path = $dav_resource->get_content_path();  
+				$data_path = $dav_resource->get_content_path();
+				$is_zip = $dav_resource->is_zip();
 				if (($data_path!='') && $_SERVER["SERVER_XFILEOPERATION_SUPPORT"])
 				{
 					 
 					fclose($data);
-					$response->http_response_stream_xoperation('200 OK', $headers, $data_path);
+					$response->http_response_stream_xoperation('200 OK', $headers, $data_path, $is_zip);
 				}
 				else
 				{
@@ -749,7 +750,7 @@ class caldav_session extends webdav_session
 		$writer->startElementNS(null, 'multistatus', 'DAV:');
 
 		if ($_SESSION["use_gal_for_search"] == '0')
-			$child_resources = $dav_resource->get_children();
+			$child_resources = $dav_resource->get_children(null, $properties);
 		else 
 			$child_resources = $dav_resource->get_items_filtered($filter, null, true);
  
@@ -1016,7 +1017,7 @@ class caldav_session extends webdav_session
 				else if ($child->get_name() == 'urn:ietf:params:xml:ns:caldav:prop-filter')
 				{
 					$name = strtolower($child->_attribs['name']);
-					if ($name == 'uid' || $name == 'summary' || $name == 'description' || $name == 'location')
+					if ($name == 'uid' || $name == 'summary' || $name == 'description' || $name == 'location' || $name == 'completed' || $name == 'status')
 					{
 						if ($child->get_child('urn:ietf:params:xml:ns:caldav:is-not-defined'))
 						{
@@ -1024,11 +1025,17 @@ class caldav_session extends webdav_session
 						}
 						else if (($text_match = $child->get_child('urn:ietf:params:xml:ns:caldav:text-match')) != null)
 						{
-							 							$filter[$comp_type][$name]['text'] = $text_match->_children[0]->get_data();
+							$text = $text_match->_children[0]->get_data();
+							$lc_text = strtolower($text ?? 'x');
+							$statuses = array('needs-action', 'in-process', 'completed', 'cancelled');
+							if ($name == 'status' && in_array($lc_text, $statuses))
+								$name .= '_' . strtolower($lc_text);
+
+							 							$filter[$comp_type][$name]['text'] = $text;
 							$filter[$comp_type][$name]['negate'] = $text_match->_attribs['negate-condition'] == 'yes' ? true : false;
 							$filter[$comp_type][$name]['type'] = $text_match->_attribs['match-type'];
-							if (!$filter[$name]['type'])
-							  $filter[$name]['type'] = 'contains';
+							if (!$filter[$comp_type][$name]['type'])
+								$filter[$comp_type][$name]['type'] = 'contains';
 						}
 						else if ($child->children_count() == 0)
 						{
@@ -1230,7 +1237,7 @@ class caldav_session extends webdav_session
 			$writer->writeElementNS($xml_dav_prefix, 'href', null, $dav_resource->get_uri()->to_string() . '/');
 		else
 			$writer->writeElementNS($xml_dav_prefix, 'href', null, $dav_resource->get_uri()->to_string());
-		$in_propstat = false;		
+		$in_propstat = false;
 		foreach ($properties_xml->children() as $property_xml)
 		{
 			$name = $property_xml->_attribs['name'];
@@ -1269,7 +1276,11 @@ class caldav_session extends webdav_session
 					foreach ($href_xml->children() as $href)
 					{
 						$uri = new caldav_uri($href->_children[0]->get_data());
-						$resource = resource::get_resource($this->_datastore, $uri);
+						$group_info = array('displayname' => $href->_attribs['displayname'], 'email' => $href->_attribs['email']);
+						if (is_null_or_empty($group_info['email']))
+							$resource = resource::get_resource($this->_datastore, $uri);
+						else
+							$resource = new principal_resource($this->_datastore, $uri, $group_info);
 						$this->expand_properties($writer, $xml_dav_prefix, $resource, $property_xml);
 					}
 
@@ -1346,7 +1357,7 @@ class caldav_session extends webdav_session
 		  $filter = array('is_collection' => 'true');
 		else
 		  $filter = null;
-		foreach ($dav_resource->get_children($filter) as $child_resource)
+		foreach ($dav_resource->get_children($filter, $properties) as $child_resource)
 		{
 			if (!($child_resource instanceof collection_resource))
 				$child_resource->write_response($writer, $this->_xml_dav_prefix, $properties, null, null, null, $request->_brief);
@@ -1396,7 +1407,7 @@ class caldav_session extends webdav_session
 		if (!($dav_resource instanceof collection_resource))
 			return 0;
 
-		$children = $dav_resource->get_children($filter);
+		$children = $dav_resource->get_children($filter, $properties);
 		$resource_count = count($children);
 		foreach ($children as $child_resource)
 		{
