@@ -10,7 +10,6 @@ define('E_NO_VEVENT',14);
 class datastore
 {
 	var $_gw;
-	var $_user;
 	var $_email;
 	var $_gsid;
 	var $_fsid;
@@ -24,6 +23,7 @@ class datastore
 	var $_contacts_folder;
 	var $_folders_version;
 	var $_folders_new_version;
+	var $_webfolders;
 	public $_folder_info_cache;
 
 	function __construct()
@@ -34,6 +34,7 @@ class datastore
 		$this->_gw = new IceWarpGWAPI();
 		$this->_gw->identity = 'com.icewarp.webdav';
 		$this->_gw->deviceid = $_SESSION['WEBDAV_DEVICE_ID'];
+		$this->_webfolders = (substr_compare($_SERVER['SCRIPT_NAME'], '/webfolders', 0, 11, true) == 0) ? true : false;
 	}
 
 	function is_folder($group_name, $folder)
@@ -47,8 +48,7 @@ class datastore
 
 	function login($username, $password, $scheme = '', $challenge = '')
 	{
- 
-		$account = new IceWarpAccount();
+ 		$account = new IceWarpAccount();
 		if ($scheme == 'GSSAPI')
 		{
  			$host = $_SERVER['HTTP_HOST'];
@@ -67,50 +67,51 @@ class datastore
 			$scheme = 'PLAIN';
 			header('WWW-Authenticate: Negotiate ' . $otoken[0]);
 		}
- 		if ($account->Open($username) && $account->ValidateUser(catWebDav))
+
+		 		if (!$account->AuthenticateUserHash($username, $password, '', $scheme . '|' . $challenge. '|' .$_SERVER['REMOTE_ADDR']))
+			return false;
+
+		 		$impersonate = $_SERVER['HTTP_X_IMPERSONATE'];
+		if (!is_null_or_empty($impersonate) && (($account->getProperty('u_admin') != '1') || (!$account->Open($impersonate))))
+			return false;
+
+		 		if (!$account->ValidateUser(catWebDav))
+			return false;
+
+ 		 		if ($this->_gw->sessid)
 		{
-			 			if ($this->_gw->sessid)
-			{
-				$this->_fsid = array();
-				$this->_gw->Logout();
-			}
- 			$this->_user = $username;
-			$this->_gw->user = $account->EmailAddress;
-			$this->_gw->setPassword($password, $scheme == 'PLAIN');
-			$this->_gw->authscheme = $scheme;
-			$this->_gw->authchallenge = $challenge;
-			$this->_email = $account->EmailAddress;
- 			if ($this->_gw->Login())
-			{
- 				$gsid = $this->_gsid = $this->_gw->OpenGroup('*');
-				$this->cache_folder_list($gsid);
-				$this->_folders_new_version = $this->_folders_version;
-				return true;
-			}
+			$this->_fsid = array();
+			$this->_gw->Logout();
 		}
-		return false;
+
+		$this->_email = $account->EmailAddress;
+		$this->_gw->user = $account->EmailAddress;
+		$this->_gw->setPassword($password, $scheme == 'PLAIN');
+		$this->_gw->authscheme = $scheme;
+		$this->_gw->authchallenge = $challenge;
+		$this->_gw->impersonate = !is_null_or_empty($impersonate);
+ 		if (!$this->_gw->Login())
+			return false;
+
+ 		$gsid = $this->_gsid = $this->_gw->OpenGroup('*');
+		$this->cache_folder_list($gsid);
+		$this->_folders_new_version = $this->_folders_version;
+
+		return true;
 	}
 
 	function recheck_login($username, $password, $scheme = '', $challenge = '')
 	{
- 
-		$account = new IceWarpAccount();
-		if (!$account->Open($username) || !$account->ValidateUser(catWebDav))
+ 		 		$account = new IceWarpAccount();
+		if (!$account->AuthenticateUserHash($username, $password, '', $scheme . '|' . $challenge. '|' .$_SERVER['REMOTE_ADDR']))
 			return false;
 
-		if ($this->_user != $username)
+		 		$impersonate = $_SERVER['HTTP_X_IMPERSONATE'];
+		if (!is_null_or_empty($impersonate) && (($account->getProperty('u_admin') != '1') || (!$account->Open($impersonate))))
 			return false;
 
-		if ($scheme != 'PLAIN')
-		{
-			if (!$account->AuthenticateUserHash($this->_email, $password, '', $scheme . '|' . $challenge. '|' .$_SERVER['REMOTE_ADDR']))
-				return false;
-		}
-		else
-		{
-			if ($this->_gw->getPassword() != $password)
-				return false;
-		}
+		 		if (!$account->ValidateUser(catWebDav))
+			return false;
 
 		if ($this->_folders_new_version != $this->_folders_version)
 		{
@@ -230,16 +231,15 @@ class datastore
 	function user_logged_on()
 	{
 		$this->_folder_info_cache = array();
-		$this->_folders_new_version = $this->_gw->FunctionCall('GetFolderUpdate', $this->_gsid);
-		return $this->_folders_new_version != 0;
+		$result = $this->_gw->FunctionCall('GetFolderUpdate', $this->_gsid);
+		$this->_folders_new_version = $result + 3;  		return $result != 0;
 	}
 
 	function open_group($group_name)
 	{
 		if ($this->_ticket || $group_name == $this->_email)
 			return $this->_gsid;
-
-		 		 		 		$folder = $_SESSION['shared_prefix'] . $group_name;
+ 		 		 		 		$folder = $_SESSION['shared_prefix'] . $group_name;
 		if (!isset($this->_fsid[$folder]))
 		{
 			$this->_fsid[$folder] = $this->_gw->OpenFolder($this->_gsid, $folder, true);
@@ -273,6 +273,9 @@ class datastore
 		if ($folder == '@@UPLOAD@@')
 			return $this->_fsid[$folder] = $this->_gw->OpenFolder($this->_gsid, $folder);
 
+		if ($this->_webfolders)
+			$folder = $this->_folders[$folder]['folderid'];
+
 		if (!($gsid = $this->open_group($group_name)))
 			return false;
  		$folder = $this->get_group_folder_path($group_name, $folder);
@@ -295,20 +298,64 @@ class datastore
 		$this->_contacts_folder = 'Contacts';
 
 		$folders = $this->_gw->ParseParamLine($this->_gw->FunctionCall('GetFolderList', $gsid, 'FdrDisplay,FdrDesc,FdrTZ'));
+
+		if ($this->_webfolders)
+		{
+			$parent_folders = array();
+			$separator_len = strlen(DAV_PATH_SEPARATOR);
+			foreach ($folders as $folder)
+			{
+				$name = $folder['FDR_ID'];
+				$parent = '';
+				if (($pos = strrpos($folder['FDR_ID'], DAV_PATH_SEPARATOR)) !== false)
+				{
+					$parent = substr($name, 0, $pos);
+					$name = substr($name, $pos + $separator_len);
+				}
+
+				$parent_folders[$folder['FDR_ID']] = array('parent' => $parent, 'name' => strlen($folder['FDRDISPLAY'])?$folder['FDRDISPLAY']:$name);
+			}
+		}
+
 		foreach ($folders as $folder)
 		{
-			if ($folder['FDRTYPE'] == 'M')
+			if (!strpos(' CEFJNPRSTX', $folder['FDRTYPE']))
 				continue;
 
-			$name = $folder['FDR_ID'];
-			$is_shared = $folder['FDRTYPE'] == 'P' || $folder['FDRTYPE'] == 'S' || $folder['FDRTYPE'] == 'R';
+			if ($this->_webfolders)
+			{
+				$name = $parent_folders[$folder['FDR_ID']]['name'];
+				$parent = $parent_folders[$folder['FDR_ID']]['parent'];
+				for(;;)
+				{
+					if (is_null_or_empty($parent))
+						break;
+
+					if ($parent_folders[$parent])
+					{
+						$name = $parent_folders[$parent]['name'].DAV_PATH_SEPARATOR.$name;
+						$parent = $parent_folders[$parent]['parent'];
+					}
+					else
+					{
+						$name = $parent.DAV_PATH_SEPARATOR.$name;
+						break;
+					}
+				}
+			}
+			else
+			{
+				$name = $folder['FDR_ID'];
+			}
+
+			$is_shared = $folder['FDRTYPE'] == 'P' || $folder['FDRTYPE'] == 'S' || $folder['FDRTYPE'] == 'R' || $folder['FDRTYPE'] == 'X';
 
 			if (($pos = strrpos($name, DAV_PATH_SEPARATOR)) !== false)
 				$parent_name = substr($name, 0, $pos);
 			else
 				$parent_name = '';
 
-			if ($parent_name != '')
+			if (!is_null_or_empty($parent_name))
 			{
 				$this->_folders[$parent_name]['subfolders'][] = $name;
 				if (!$is_shared)
@@ -335,6 +382,10 @@ class datastore
 					$this->_journal_folder = $name;
 			}
 
+			$color = null;
+			if (strpos(' EJT', $folder['FDRTYPE']))
+				$color = manage_parameters($folder['FDRPARAM'], 'color');
+
 			$this->_folders[$name] = array(
 				'type' => $folder['FDRTYPE'],
 				'creationdate' => date('Y-m-d\TH:i:s', intval($folder['FDR_CREATED'])) . 'Z',
@@ -346,30 +397,42 @@ class datastore
 				'subfolders' => is_array($this->_folders[$name])?$this->_folders[$name]['subfolders']:null,
 				'owner' => empty($folder['GRPOWNER'])?$this->_email:$folder['GRPOWNER'],
 				'id' => $folder['GRPRELATIVEFOLDER'],
-				'subtype' => $folder['FDRSUBTYPE']);
+				'subtype' => $folder['FDRSUBTYPE'],
+				'folderid' => $folder['FDR_ID'],
+				'color' => $color);
 		}
 
 		date_default_timezone_set($tz);
+
+		do_log('Folder cache updated!!!',2);
 	}
 
 	function build_item_query($local_id)
 	{
 		$quoted_id = str_replace("'", "''", $local_id);
-		$id_query = "((ItmRID IS NULL AND ITM_ID='$quoted_id') OR ItmRID='$quoted_id')";
+		if (preg_match("/^[a-zA-Z0-9]{12}$/", $quoted_id))
+			$id_query = "((ItmRID IS NULL AND ITM_ID='$quoted_id') OR ItmRID='$quoted_id')";
+		else
+			$id_query = "(ItmRID='$quoted_id')";
+
 		return $id_query;
 	}
 
 	function build_event_query($local_id)
 	{
 		$quoted_id = str_replace("'", "''", $local_id);
-		$id_query = "(EvnClass<>'O' AND ((EvnRID IS NULL AND EVN_ID='$quoted_id') OR EvnRID='$quoted_id'))";
+		if (preg_match("/^[a-zA-Z0-9]{12}$/", $quoted_id))
+			$id_query = "(EvnClass<>'O' AND ((EvnRID IS NULL AND EVN_ID='$quoted_id') OR EvnRID='$quoted_id'))";
+		else
+			$id_query = "(EvnClass<>'O' AND EvnRID='$quoted_id')";
+
 		return $id_query;
 	}
 
 	function expand_lock_info($lockinfo)
 	{
 		$result = array();
-		foreach (explode("&", $lockinfo) as $chunk)
+		foreach (explode("&", $lockinfo ?? '') as $chunk)
 		{
 			if (!empty($chunk))
 			{
@@ -377,6 +440,7 @@ class datastore
 				$result[rawurldecode($param[0])] = rawurldecode($param[1]);
 			}
 		}
+
 		return $result;
 	}
 
@@ -404,7 +468,7 @@ class datastore
 		$item = @fopen('iw.file://' . $path, 'rb');
 		if (!$item)
 			throw new not_found_exception();
-		$parameters = explode('&', $attachment_list[0]["ATTPARAMS"]);
+		$parameters = explode('&', $attachment_list[0]["ATTPARAMS"] ?? '');
 		foreach ($parameters as $parameter)
 		{
 			if (substr_compare($parameter, 'mimetype=', 0, 9, true) == 0)
@@ -421,14 +485,14 @@ class datastore
 			$result['etag'] = null;
 
 		if (isset($item_info[0]['EVN_CREATED']))
-			$result['creationdate'] = date('Y-m-d\TH:i:s', $item_info[0]['EVN_CREATED']) . 'Z';
+			$result['creationdate'] = date('Y-m-d\TH:i:s', intval($item_info[0]['EVN_CREATED'])) . 'Z';
 		else if (isset($item_info[0]['ITM_CREATED']))
-			$result['creationdate'] = date('Y-m-d\TH:i:s', $item_info[0]['ITM_CREATED']) . 'Z';
+			$result['creationdate'] = date('Y-m-d\TH:i:s', intval($item_info[0]['ITM_CREATED'])) . 'Z';
 
 		if (isset($item_info[0]['EVN_MODIFIED']))
-			$result['lastmodified'] = date(DATE_RFC1123, $item['EVN_MODIFIED']);
+			$result['lastmodified'] = date(DATE_RFC1123, intval($item['EVN_MODIFIED']));
 		else if (isset($item_info[0]['ITM_MODIFIED']))
-			$result['lastmodified'] = date(DATE_RFC1123, $item['ITM_MODIFIED']);
+			$result['lastmodified'] = date(DATE_RFC1123, intval($item['ITM_MODIFIED']));
 
 		if ($item_info[0]['EVNCLASS'] == 'F')
 			$result['size'] = intval($item_info[0]['EVNCOMPLETE']);
@@ -442,12 +506,20 @@ class datastore
 		return $result;
 	}
 
+	function _file_size()
+	{
+		if (!is_null_or_empty($_SERVER['SERVER_PUT_FILENAME']))
+			return filesize($_SERVER['SERVER_PUT_FILENAME']);
+		else
+			return $_SERVER['CONTENT_LENGTH'];
+	}
+
 	function put_attachment($group_name, $folder, $item_id, $attachment_id, $stream)
-	{  	
+	{
 		$rights = $this->rights_to_string($this->_gw->FunctionCall('GetGroupAccessRights', $this->_gsid, $folder));
 		if (strpos($rights, 'i') === false || strpos($rights, 'w') === false)
 			throw new forbidden_exception();
-			
+
 		if (!$this->_ticket)
 		{
 			$fsid = $this->open_folder($group_name, $folder);
@@ -464,6 +536,7 @@ class datastore
 		$attachment_list = $this->_gw->ParseParamLine($this->_gw->FunctionCall('GetAttachmentList', $fsid, $item_id, $attachment_id));
 		
 		$time = time();
+		$size = $this->_file_size();
 		$attachment['AttName'] = $attachment_id;
 		$attachment['AttType'] = 'F';
 		$attachment['AttTime'] = $time;
@@ -473,20 +546,19 @@ class datastore
 		$path = $this->_gw->FunctionCall('GetAttachmentPath', $fsid, $item_id, $attachment_id, "LOCK");
 		if (!$path)
 			throw new locked_exception();
-		$dest_stream = @fopen('iw.file://' . $path, 'wb');
-		if (!$dest_stream)
-			throw new forbidden_exception();
-		stream_copy_to_stream($stream, $dest_stream);
-		$size = ftell($dest_stream);
-		fclose($dest_stream);
+
+		if (!is_null_or_empty($_SERVER['SERVER_PUT_FILENAME']))
+			rename($_SERVER['SERVER_PUT_FILENAME'], $path);
+		else
+			file_put_contents($path, stream_get_contents($stream));
+
 		$result = $this->_gw->FunctionCall('GetAttachmentPath', $fsid, $item_id, $attachment_id, "UNLOCK");
 		if (!$result)
 			throw new forbidden_exception();
 
 		$event['EvnComplete'] = empty($size) ? 0 : $size;
 		$event['EvnStartDate'] = unixtojd($time);
-		$event['EvnStartTime'] = ($time / 60) % 3600;
-		$event['EvnComplete'] = $size;
+		$event['EvnStartTime'] = intval($time / 60) % 3600;
 		$result = $this->_gw->FunctionCall('AddEventInfo', $fsid, $this->_gw->CreateParamLine($event), $item_id);
 		if (!$result)
 			throw new forbidden_exception();
@@ -528,7 +600,7 @@ class datastore
 		if (count($attachment_list) == 0)
 			throw new not_found_exception();
 
-		$parameters = explode('&', $attachment_list[0]["ATTPARAMS"]);
+		$parameters = explode('&', $attachment_list[0]["ATTPARAMS"] ?? '');
 		foreach ($parameters as $parameter)
 		if (substr_compare($parameter, 'mimetype=', 0, 9, true) == 0)
 			$content_type = rawurldecode(substr($parameter, 9));
@@ -602,8 +674,11 @@ class datastore
 				$attachment_list = $this->_gw->ParseParamLine($this->_gw->FunctionCall('GetAttachmentList', $fsid, $raw_item_id, ''));
 				if (count($attachment_list)) {
 					$path = $this->_gw->FunctionCall('GetAttachmentPath', $fsid, $raw_item_id, $attachment_list[0]['ATTNAME']);
-					$item = @fopen('iw.file://' . $path, "rb");
-					if (!$item) throw new not_found_exception();
+					$data = @fopen('iw.file://' . $path, "rb");
+					if (!$data) 
+						throw new not_found_exception();
+
+					$item = array('data' => $data, 'path' => $path);
 				} else{
 					throw new not_found_exception();
 				}
@@ -622,19 +697,36 @@ class datastore
 			else
 				$prefix = '';
 			if ($filter_component['type'] == 'null')
-				return $prefix . "$name == ''";
+				return $prefix . "$name = ''";
 			else if ($filter_component['type'] == 'ends-with')
 				return $prefix . "$name LIKE '%" . str_replace("'", "''", $filter_component['text']) . "'";
 			else if ($filter_component['type'] == 'starts-with')
 				return $prefix . "$name LIKE '" . str_replace("'", "''", $filter_component['text']) . "%'";
 			else if ($filter_component['type'] == 'equals')
-				return $prefix . "$name =='" . str_replace("'", "''", $filter_component['text']) . "'";
+				return $prefix . "$name ='" . str_replace("'", "''", $filter_component['text']) . "'";
 			else if ($filter_component['type'] == 'contains')
 				return $prefix . "$name LIKE '%" . str_replace("'", "''", $filter_component['text']) . "%'";
 		}
 		return false;
 	}
 
+	function tasks_status_filter_to_query($filter_component, &$query)
+	{
+		$statuses = array('completed' => 'M', 'status_needs-action' => 'B', 'status_in-process' => 'I', 'status_completed' => 'M', 'status_cancelled' => 'Q', 'status' => '#');
+		foreach($statuses as $name => $status)
+		{
+			if (!$filter_component[$name])
+				continue;
+
+			if ($filter_component[$name]['negate'] || $filter_component[$name]['type'] == 'null')
+				$prefix = 'NOT ';
+			else
+				$prefix = '';
+
+			$query[] = $prefix . "EvnStatus ='$status'";
+		}
+	}
+
 	function time_range_to_julian($time_range)
 	{
 		 		$start = $time_range['start'];
@@ -654,7 +746,7 @@ class datastore
 	}
 
 	 
-	function get_item_info($group_name, $folder, $local_id = null, $filter = null)
+	function get_item_info($group_name, $folder, $local_id = null, $filter = null, $properties = null)
 	{
 		$result = $custom_queries = [];
 		if ($this->_ticket != null)
@@ -736,8 +828,8 @@ class datastore
 				$result[$key]['etag'] = $item['ITM_ID'] . ':' . $item['ITM_EDITCOUNTER'];
 				$result[$key]['title'] = $item['ITMCLASSIFYAS'];
 				$result[$key]['type'] = $object_type;
-				$result[$key]['creationdate'] = date('Y-m-d\TH:i:s', $item['ITM_CREATED']) . 'Z';
-				$result[$key]['lastmodified'] = date(DATE_RFC1123, $item['ITM_MODIFIED']);
+				$result[$key]['creationdate'] = date('Y-m-d\TH:i:s', intval($item['ITM_CREATED'])) . 'Z';
+				$result[$key]['lastmodified'] = date(DATE_RFC1123, intval($item['ITM_MODIFIED']));
 				$result[$key]['uid'] = $item['ITMUID']?$item['ITMUID']:$item['ITM_ID'];
 				$result[$key]['lockinfo'] = $this->expand_lock_info($item['ITMLOCKINFO']);
 				$result[$key]['lockhash'] = $item['ITMLOCKHASH'];
@@ -771,6 +863,9 @@ class datastore
 						$custom_query[] = $this->text_filter_to_query($comp_filter['summary'], 'EvnTitle');
 						$custom_query[] = $this->text_filter_to_query($comp_filter['description'], 'EvnNote');
 						$custom_query[] = $this->text_filter_to_query($comp_filter['location'], 'EvnLocation');
+						if ($component = 'T' )
+							$this->tasks_status_filter_to_query($comp_filter, $custom_query);
+
 						if (count($custom_query) > 0)
 							$custom_queries[] = '(' . implode(' AND ', array_filter($custom_query)) . ')';
 
@@ -796,7 +891,7 @@ class datastore
 
 			if (isset($local_id))
 			{
-				if (strlen($query) > 0)
+				if (!is_null_or_empty($query))
 					$query = ' AND ' . $query;
 
 				$query = $this->build_event_query($local_id) . $query;
@@ -823,14 +918,18 @@ class datastore
 				if ($query=='(())')
 				{
 					$query = '';
-					do_log('WARNING: get_item_info still has invalid query!!!', 2);
-				}
+					 				}
 
-				$result_line = $this->_gw->FunctionCall('GetIntervalEvents', $fsid, $this->time_range_to_julian($filter['time-range']) . $tz, $query, 'Evn_EditCounter,EvnTitle,EvnClass,EvnRID,EvnComplete,EvnTimeFormat,EvnStartDate,EvnStartTime,EvnEndDate,EvnEndTime,EvnUID');
+				$result_line = $this->_gw->FunctionCall('GetIntervalMasterEvents', $fsid, $this->time_range_to_julian($filter['time-range']) . $tz, $query, 'Evn_EditCounter,EvnTitle,EvnClass,EvnRID,EvnComplete,EvnTimeFormat,EvnStartDate,EvnStartTime,EvnEndDate,EvnEndTime,EvnUID');
 			}
-			else
+			else 
+			{
 				$result_line = $this->_gw->FunctionCall('GetEventList', $fsid, $query, 'Evn_EditCounter,EvnTitle,EvnClass,EvnRID,EvnComplete,Evn_Created,Evn_Modified,EvnUID,EvnLockInfo,EvnLockHash', 'private_busy=1');
 
+				 				if (!$result_line)
+					throw new service_unavailable_exception();
+			}
+
 			foreach ($this->_gw->ParseParamLine($result_line) as $item)
 			{
 				$key = $item['EVNRID']?$item['EVNRID']:$item['EVN_ID'];
@@ -840,8 +939,8 @@ class datastore
 					$result[$key]['etag'] = $item['EVN_ID'] . ':' . $item['EVN_EDITCOUNTER'];
 					$result[$key]['title'] = $item['EVNTITLE'];
 					$result[$key]['type'] = ($item['EVNCLASS'] == 'V' ? 'E': $item['EVNCLASS']);
-					$result[$key]['creationdate'] = date('Y-m-d\TH:i:s', $item['EVN_CREATED']) . 'Z';
-					$result[$key]['lastmodified'] = date(DATE_RFC1123, $item['EVN_MODIFIED']);
+					$result[$key]['creationdate'] = date('Y-m-d\TH:i:s', intval($item['EVN_CREATED'])) . 'Z';
+					$result[$key]['lastmodified'] = date(DATE_RFC1123, intval($item['EVN_MODIFIED']));
 					$result[$key]['uid'] = $item['EVNUID']?$item['EVNUID']:$item['EVN_ID'];
 					$result[$key]['lockinfo'] = $this->expand_lock_info($item['EVNLOCKINFO']);
 					$result[$key]['lockhash'] = $item['EVNLOCKHASH'];
@@ -850,15 +949,21 @@ class datastore
 					{
 						$result[$key]['size'] = intval($item['EVNCOMPLETE']);
 
-						 						$attributes_line = $this->_gw->FunctionCall('GetItemAttributes', $fsid, $item['EVN_ID'],"AtrParam='X_ICEWARP_FILEHASH'");
-						$attributes = $this->_gw->ParseParamLine($attributes_line);
-						if (count($attributes))
-							$result[$key]['filehash'] = $attributes[0]['DATAVALUE'];
+						 						if (is_null($properties) || ($properties == 'allprop') || (is_array($properties) && $properties['filehash']))
+						{
+							$attributes_line = $this->_gw->FunctionCall('GetItemAttributes', $fsid, $item['EVN_ID'],"AtrParam='X_ICEWARP_FILEHASH'");
+							$attributes = $this->_gw->ParseParamLine($attributes_line);
+							if (count($attributes))
+								$result[$key]['filehash'] = $attributes[0]['DATAVALUE'];
+						}
 
-						 						$attachments_line = $this->_gw->FunctionCall('GetAttachmentList', $fsid, $item['EVN_ID']);
-                        $attachments = $this->_gw->ParseParamLine($attachments_line);
-                        if (count($attachments))
-                            $result[$key]['filepublicurl'] = $this->_gw->FunctionCall('GetAttachmentPathLocal', $fsid, $item['EVN_ID'], $attachments[0]['ATTNAME'], 'TICKET');
+						 						if (is_null($properties) || ($properties == 'allprop') || (is_array($properties) && $properties['filepublicurl']))
+						{
+							$attachments_line = $this->_gw->FunctionCall('GetAttachmentList', $fsid, $item['EVN_ID']);
+							$attachments = $this->_gw->ParseParamLine($attachments_line);
+							if (count($attachments))
+								$result[$key]['filepublicurl'] = $this->_gw->FunctionCall('GetAttachmentPathLocal', $fsid, $item['EVN_ID'], $attachments[0]['ATTNAME'], 'TICKET');
+						}
 					}
 				}
 				else
@@ -887,7 +992,7 @@ class datastore
 		foreach ($attributes as $attribute)
 			$this->_gw->FunctionCall('DeleteItemAttribute', $fsid, $item_id, $attribute['ATR_ID']);
 
-		 		if (strlen($file_hash)>0)
+		 		if (!is_null_or_empty($file_hash))
 			$this->_gw->FunctionCall('AddItemAttribute', $fsid, $item_id, $file_hash, 'AtrType=x&AtrParam=X_ICEWARP_FILEHASH');
 
 		return true;
@@ -897,6 +1002,10 @@ class datastore
 	function make_collection($group_name, $folder, $item_id, $folder_info)
 	{
 		$gsid = $this->open_group($group_name);
+		
+		if ($this->_webfolders)
+			$folder = $this->_folders[$folder]['folderid'];
+
 		$full_path = $this->get_group_folder_path($group_name, $folder);
 
 		$params = array();
@@ -907,6 +1016,12 @@ class datastore
 		if (isset($folder_info['timezone']))
 			$params['FdrTZ'] = $folder_info['timezone'];
 
+		if (isset($folder_info['color']))
+		{
+			$new_parameters = manage_parameters('', 'color', $folder_info['color']);
+			$params['FdrParam'] = $new_parameters;
+		}
+
 		$result = $this->_gw->FunctionCall('AddFolder', $gsid, trim($full_path . DAV_PATH_SEPARATOR . $item_id, DAV_PATH_SEPARATOR), $folder_info['type'], $this->_gw->CreateParamLine($params));
 		if (!$result)
 		{
@@ -915,8 +1030,7 @@ class datastore
 			throw new forbidden_exception();
 		}
 
-		$this->cache_folder_list($gsid);
-	}
+		 	}
 
 	 
 	function modify_collection($group_name, $folder, $folder_info)
@@ -972,7 +1086,7 @@ class datastore
 
 	function rights_to_string($rights)
 	{
-		if (!($rights &0x80000000))
+		if (!($rights &(int)0x80000000))
 		{
 			switch ($rights)
 			{
@@ -1000,7 +1114,7 @@ class datastore
 
 	function string_to_rights($symbolic_rights)
 	{
-		$rights = 0x80000000;
+		$rights = (int)0x80000000;
 		foreach (str_split($symbolic_rights) as $symbolic_right)
 		{
 			switch ($symbolic_right)
@@ -1037,7 +1151,7 @@ class datastore
 		{
 			$principal = $ace['FRTEMAIL'];
 			$rights = intval($ace['FRTRIGHT']);
-			if (($rights & 0x80000001) == 0x80000001)  			{
+			if (($rights & (int)0x80000001) == 0x80000001)  			{
 				$result[$principal]['owner'] = true;
 				$found_owner = true;
 			}
@@ -1124,7 +1238,7 @@ class datastore
 		else
 			$group_flags = 1 | 2 | 8 | 32;
 
-		$groups = $this->_gw->ParseParamLine($this->_gw->FunctionCall('GetShareAccountList', $this->_gsid, $group_flags));
+		$groups = $this->_gw->ParseParamLine($this->_gw->FunctionCall('GetShareAccountList', $this->_gw->sessid, $group_flags));
 		foreach($groups as $group)
 			$result[$group['FRTEMAIL']] = array('displayname' => $group['OWNERNAME'], 'email' => $group['FRTEMAIL']);
 
@@ -1234,6 +1348,14 @@ class datastore
 			return $this->endsWith($id,'Calendar.ics');
 		}
 
+	function _stream_get_contents($stream)
+	{
+		if (!is_null_or_empty($_SERVER['SERVER_PUT_FILENAME']))
+			return file_get_contents($_SERVER['SERVER_PUT_FILENAME']);
+		else
+			return stream_get_contents($stream);
+	}
+
 	function put_item($group_name, $folder, $item_id, $stream)
 	{
 		$add_info = $id_name = null;
@@ -1245,7 +1367,7 @@ class datastore
 		if (!$fsid)
 			throw new not_found_exception();
 
-		$pos = strpos($_SERVER['CONTENT_TYPE'], ';');
+		$pos = strpos($_SERVER['CONTENT_TYPE'] ?? '', ';');
 		$content_type = $pos !== false ? substr($_SERVER['CONTENT_TYPE'], 0, $pos) : $_SERVER['CONTENT_TYPE'];
 
 		switch ($object_type)
@@ -1263,7 +1385,7 @@ class datastore
 					$edit_counter = 0;
 				}
 
-				$add_info = $this->_gw->ParseParamLine($this->_gw->FunctionCall('AddvCard', $fsid, stream_get_contents($stream), $raw_item_id, ';SETUID;EXRESULT;PARAMS=ItmRID=' . rawurlencode($item_id)));
+				$add_info = $this->_gw->ParseParamLine($this->_gw->FunctionCall('AddvCard', $fsid, $this->_stream_get_contents($stream), $raw_item_id, ';SETUID;EXRESULT;PARAMS=ItmRID=' . rawurlencode($item_id)));
 				$id_name = 'ITM_ID';
 				break;
 
@@ -1273,10 +1395,13 @@ class datastore
 			case 'T':
 				 				if (!empty($content_type))
 				{
-					if ($object_type != 'N' && $content_type != "text/calendar")
-						throw new precondition_failed_exception('urn:ietf:params:xml:ns:caldav', 'supported-calendar-data');
-					if ($object_type == 'N' && $content_type != "text/x-vnote")
-						throw new precondition_failed_exception('urn:ietf:params:xml:ns:caldav', 'supported-calendar-data');
+					 					if (strpos($_SERVER['HTTP_USER_AGENT'], 'ezdavplugin')===false)
+					{
+						if ($object_type != 'N' && $content_type != "text/calendar")
+							throw new precondition_failed_exception('urn:ietf:params:xml:ns:caldav', 'supported-calendar-data');
+						if ($object_type == 'N' && $content_type != "text/x-vnote")
+							throw new precondition_failed_exception('urn:ietf:params:xml:ns:caldav', 'supported-calendar-data');
+					}
 				}
 
 				$list = $this->_gw->ParseParamLine($this->_gw->FunctionCall('GetEventList', $fsid, $this->build_event_query($item_id), 'Evn_EditCounter,EvnMeetingId', 'private_busy=1'));
@@ -1304,9 +1429,9 @@ class datastore
 				$id_name = 'EVN_ID';
 
 				if ($object_type == 'N' || $object_type == 'J' || $_SERVER['HTTP_SCHEDULE_REPLY'] == 'F')
-					$add_info = $this->_gw->ParseParamLine($this->_gw->FunctionCall('AddvCalendar', $fsid, stream_get_contents($stream), $raw_item_id, ';SETUID;EXRESULT;PARAMS=EvnRID=' . rawurlencode($item_id)));
+					$add_info = $this->_gw->ParseParamLine($this->_gw->FunctionCall('AddvCalendar', $fsid, $this->_stream_get_contents($stream), $raw_item_id, ';SETUID;EXRESULT;PARAMS=EvnRID=' . rawurlencode($item_id)));
 				else
-					$add_info = $this->_gw->ParseParamLine($this->_gw->FunctionCall('ProcessvCalendar', $fsid, $raw_item_id, stream_get_contents($stream), 'rfc6638=&action=edit&versitparams=' . rawurlencode(';EXRESULT;PARAMS=EvnRID=' . $item_id)));
+					$add_info = $this->_gw->ParseParamLine($this->_gw->FunctionCall('ProcessvCalendar', $fsid, $raw_item_id, $this->_stream_get_contents($stream), 'rfc6638=&action=edit&versitparams=' . rawurlencode(';EXRESULT;PARAMS=EvnRID=' . $item_id)));
 
 				if (!$add_info || count($add_info) == 0 || !$add_info[0][$id_name])
 				{
@@ -1330,14 +1455,14 @@ class datastore
 			case 'F':
 				$list = $this->_gw->ParseParamLine($this->_gw->FunctionCall('GetEventList', $fsid, $this->build_event_query($item_id), 'Evn_EditCounter', 'private_busy=1'));
 
-				$size = $_SERVER["CONTENT_LENGTH"];
+				$size = $this->_file_size();
 				$time = time();
 				$event['EvnRID'] = $item_id;
 				$event['EvnTitle'] = $item_id;
 				$event['EvnLocation'] = $item_id;
 				$event['EvnComplete'] = empty($size) ? 0 : $size;
 				$event['EvnStartDate'] = unixtojd($time);
-				$event['EvnStartTime'] = ($time / 60) % 3600;
+				$event['EvnStartTime'] = intval($time / 60) % 3600;
 				$event['EvnClass'] = 'F';
 				$event['EvnShareType'] = 'U';
 
@@ -1368,19 +1493,10 @@ class datastore
 					if (!$path)
 						throw new locked_exception();
 
-					 					$dest_stream = @fopen('iw.file://' . $path, 'wb');
-					if (!$dest_stream)
-						throw new forbidden_exception();
-
-					stream_copy_to_stream($stream, $dest_stream);
-					if (empty($size))
-					{
-						 						$size = ftell($dest_stream);
-						$event['EvnComplete'] = $size;
-						$this->_gw->FunctionCall('AddEventInfo', $fsid, $this->_gw->CreateParamLine($event), $event_id);
-					}
-
-					fclose($dest_stream);
+					if (!is_null_or_empty($_SERVER['SERVER_PUT_FILENAME']))
+						rename($_SERVER['SERVER_PUT_FILENAME'], $path);
+					else
+						file_put_contents($path, stream_get_contents($stream));
 
 					 					$result = $this->_gw->FunctionCall('GetAttachmentPath', $fsid, $event_id, $lattid, 'UNLOCK');
 					if (!$result)
@@ -1430,6 +1546,10 @@ class datastore
 		$gsid = $this->open_group($group_name);
 		if (!$gsid)
 			throw new not_found_exception();
+
+		if ($this->_webfolders)
+			$folder = $this->_folders[$folder]['folderid'];
+
 		$full_path = $this->get_group_folder_path($group_name, $folder);
 		$result = $this->_gw->FunctionCall('DeleteFolder', $gsid, $full_path);
 		if ($result)
@@ -1536,6 +1656,10 @@ class datastore
 				unset($event_info[0]['EVNFOLDER']);
 				unset($event_info[0]['EVNGRP_ID']);
 				unset($event_info[0]['EVNRCR_ID']);
+				unset($event_info[0]['GPINWHEN']);
+				unset($event_info[0]['GPINOWNNAME']);
+				unset($event_info[0]['GPINOWNEMAIL']);
+				unset($event_info[0]['PINWHEN']);
 				 				$event_info[0]['EVNRID'] = $target_item_id;
 				 
 				$result = $this->_gw->FunctionCall('AddEventInfo', $target_fsid, $this->_gw->CreateParamLine($event_info[0]));
@@ -1578,7 +1702,7 @@ class datastore
 		}
 	}
 
-	function rename_item($group_name, $folder, $raw_item_id, $target_item_id)
+	function rename_item($group_name, $folder, $raw_item_id, $source_item_id, $target_item_id)
 	{
 		$result = null;
 		$fsid = $this->open_folder($group_name, $folder);
@@ -1608,7 +1732,14 @@ class datastore
 				$event['EvnLocation'] = $target_item_id;
 				$result = $this->_gw->FunctionCall('AddEventInfo', $fsid, $this->_gw->CreateParamLine($event), $raw_item_id);
 				if ($result)
+				{
+					$attach['AttName'] = $source_item_id;
+					$attach['AttRename'] = $target_item_id;
+					$attach['AttDesc'] = $target_item_id;
+					$attach['AttSkipValue'] = 1;
+					$this->_gw->FunctionCall('AddAttachment', $fsid, $raw_item_id, $this->_gw->CreateParamLine($attach));
 					$this->_gw->FunctionCall('AddEventInfo', $fsid, null, $result);
+				}
 				break;
 		}
 
@@ -1624,6 +1755,19 @@ class datastore
 		$gsid = $this->open_group($group_name);
 		if (!$gsid)
 			throw new not_found_exception();
+
+		if ($this->_webfolders)
+		{
+			$folder = $this->_folders[$folder]['folderid'];
+
+			if (($pos = strrpos($target_folder, DAV_PATH_SEPARATOR)) !== false)
+			{
+				$parent_name = substr($target_folder, 0, $pos);
+				$parent_name = $this->_folders[$parent_name]['folderid'];
+				$target_folder = $parent_name . substr($target_folder,$pos);
+			}
+		}
+
 		$source = $this->get_group_folder_path($group_name, $folder);
 		$target = $this->get_group_folder_path($target_group_name, $target_folder);
 		$result = $this->_gw->FunctionCall('RenameFolder', $gsid, $source, $target);
@@ -1652,35 +1796,42 @@ class datastore
 		$gsid = $this->open_group($group_name);
 		if (!$gsid)
 			throw new not_found_exception();
-		$xmlResult = $this->_gw->FunctionCall('ProcessvCalendar', $gsid, '', '', 'rfc6638=&action=list_inbox&stateonly=1' . '&owner=' . rawurlencode($group_name));
 
-		$reader = new XMLReader();
-		$reader->XML($xmlResult);
-		$reader->read();  		$last_state = $reader->getAttribute('laststate');
-		$reader->close();
+		$last_state = '';
+		$xmlResult = $this->_gw->FunctionCall('ProcessvCalendar', $gsid, '', '', 'rfc6638=&action=list_inbox&stateonly=1' . '&owner=' . rawurlencode($group_name));
+		if ($xmlResult)
+		{
+			$reader = new XMLReader();
+			$reader->XML($xmlResult);
+			$reader->read();  			$last_state = $reader->getAttribute('laststate');
+			$reader->close();
+		}
 
 		return $last_state;
 	}
 
 	function list_inbox_items($group_name, &$last_state)
 	{
-		$result = [];
 		$gsid = $this->open_group($group_name);
 		if (!$gsid)
 			throw new not_found_exception();
 
+		$result = [];
 		$xmlResult = $this->_gw->FunctionCall('ProcessvCalendar', $gsid, '', '', 'rfc6638=&action=list_inbox' . ($last_state == null ? '' : '&laststate=' . rawurlencode($last_state)) . '&owner=' . rawurlencode($group_name));
-
-		$reader = new XMLReader();
-		$reader->XML($xmlResult);
-		$reader->read();  		$last_state = $reader->getAttribute('laststate');
-		$reader->read();  		while ($reader->localName == 'item')
+		if ($xmlResult)
 		{
-			$reader->read();  			$uid = $reader->readString();
-			$reader->read();  			$reader->read();  			$reader->read();  			$reader->read();  			$inbox_item_data = array('uid' => $uid);
-			$result[$uid] = $inbox_item_data;
+			$reader = new XMLReader();
+			$reader->XML($xmlResult);
+			$reader->read();  			$last_state = $reader->getAttribute('laststate');
+			$reader->read();  			while ($reader->localName == 'item')
+			{
+				$reader->read();  				$uid = $reader->readString();
+				$reader->read();  				$reader->read();  				$reader->read();  				$reader->read();  				$inbox_item_data = array('uid' => $uid);
+				$result[$uid] = $inbox_item_data;
+			}
+
+			$reader->close();
 		}
-		$reader->close();
 
 		return $result;
 	}
