@@ -28,7 +28,7 @@ class collection_resource extends resource
 		return true;
 	}
 
-	public function get_children($filter=NULL)
+	public function get_children($filter = null, $properties = null)
 	{
 		return array();
 	}
