@@ -18,7 +18,7 @@ class root_resource extends collection_resource
 		parent::write_property_value($namespace_uri, $property_name, $xml_writer, $xml_dav_prefix);
 	}
 
-	public function get_children($filter=NULL)
+	public function get_children($filter = null, $properties = null)
 	{
 		$resources = array();
 		foreach ($this->_datastore->get_groups(true) as $group => $group_info)
