@@ -74,25 +74,26 @@ class user_collection_resource extends collection_resource
 		return array_merge($subfolders, $subsubfolders);
 	}
 
-	public function get_children($filter=null)
+	public function get_children($filter=null, $properties = null)
 	{
 		$resources = array();
+		$agent = $_SERVER['HTTP_USER_AGENT'] ?? '';
 
 		$iCal =
-			strpos($_SERVER['HTTP_USER_AGENT'], 'iCal/') !== false ||
-			strpos($_SERVER['HTTP_USER_AGENT'], 'CalendarAgent/') !== false;
+			strpos($agent, 'iCal/') !== false ||
+			strpos($agent, 'CalendarAgent/') !== false;
 		$use_flat_folders =
 			($iCal ||
-			 strpos($_SERVER['HTTP_USER_AGENT'], 'iCalendar/') !== false ||
-			 strpos($_SERVER['HTTP_USER_AGENT'], 'DataAccess/') !== false ||
-			 strpos($_SERVER['HTTP_USER_AGENT'], 'Address%20Book/') !== false ||
-			 strpos($_SERVER['HTTP_USER_AGENT'], 'AddressBook/') !== false ||
-			 strpos($_SERVER['HTTP_USER_AGENT'], 'iOS/') !== false);
+			 strpos($agent, 'iCalendar/') !== false ||
+			 strpos($agent, 'DataAccess/') !== false ||
+			 strpos($agent, 'Address%20Book/') !== false ||
+			 strpos($agent, 'AddressBook/') !== false ||
+			 strpos($agent, 'iOS/') !== false);
 		
 		 		if (count($this->get_uri()->get_segments()) == 1 && $use_flat_folders)
 		{
-			if (strpos($_SERVER['HTTP_USER_AGENT'], 'Address%20Book/') !== false ||
-			strpos($_SERVER['HTTP_USER_AGENT'], 'AddressBook/') !== false)
+			if (strpos($agent, 'Address%20Book/') !== false ||
+			strpos($agent, 'AddressBook/') !== false)
 				$subfolders = array($this->_datastore->_contacts_folder);
 			else
 				$subfolders = $this->get_subfolders_recursive($this->_group, $this->_folder_path);
@@ -112,7 +113,7 @@ class user_collection_resource extends collection_resource
 
 		if ($filter == null || !isset($filter['is_collection']))
 		{
-			foreach ($this->_datastore->get_item_info($this->_group, $this->_folder_path) as $id => $item)
+			foreach ($this->_datastore->get_item_info($this->_group, $this->_folder_path, null, null, $properties) as $id => $item)
 				$resources[] = new item_resource($this->_datastore, new caldav_uri($this->get_uri(), $id), $item, $this);
 		}
 
@@ -183,6 +184,8 @@ class user_collection_resource extends collection_resource
 					$additional_properties[] = array('urn:ietf:params:xml:ns:caldav', 'calendar-timezone');
 				$additional_properties[] = array('urn:ietf:params:xml:ns:caldav', 'supported-calendar-component-set');
 				$additional_properties[] = array('urn:ietf:params:xml:ns:caldav', 'supported-calendar-data');
+				if ($this->_folder_info['color'] !== null)
+					$additional_properties[] = array('http://apple.com/ns/ical/', 'calendar-color');
 				$additional_properties[] = array('http://apple.com/ns/ical/', 'calendar-order');
 			}
 			else if ($this->_folder_info['type'] == 'C')
@@ -672,15 +675,21 @@ class user_collection_resource extends collection_resource
 			return;
 		}
 
-		if ($namespace_uri == 'http://apple.com/ns/ical/' && $property_name == 'calendar-order')
+		if ($namespace_uri == 'http://apple.com/ns/ical/')
 		{
-			 			if ($this->_folder_path == $this->_datastore->_events_folder)
-				return $xml_writer->text('1');
-			if ($this->_folder_path == $this->_datastore->_tasks_folder)
-				return $xml_writer->text('2');
-			if ($this->_folder_path == $this->_datastore->_journal_folder)
-				return $xml_writer->text('3');
-			return $xml_writer->text('666');
+			if ($property_name == 'calendar-color' && $this->_folder_info['color'] !== null)
+				return $xml_writer->text($this->_folder_info['color']);
+
+			 			if ($property_name == 'calendar-order')
+			{
+				if ($this->_folder_path == $this->_datastore->_events_folder)
+					return $xml_writer->text('1');
+				if ($this->_folder_path == $this->_datastore->_tasks_folder)
+					return $xml_writer->text('2');
+				if ($this->_folder_path == $this->_datastore->_journal_folder)
+					return $xml_writer->text('3');
+				return $xml_writer->text('666');
+			}
 		}
 
 		if ($namespace_uri == 'http://icewarp.com/ns/')
@@ -719,8 +728,8 @@ class user_collection_resource extends collection_resource
 			if ($property[0] == 'DAV:' && $property[1] == 'displayname')
 			{
 				$displayname = $value->_children[0]->get_data();
+				$updated = $folder_info['displayname'] != $displayname;
 				$folder_info['displayname'] = $displayname;
-				$updated = true;
 				continue;
 			}
 
@@ -770,7 +779,15 @@ class user_collection_resource extends collection_resource
 
 			if ($property[0] == 'http://apple.com/ns/ical/')
 			{
-				if ($property[1] == 'calendar-color' || $property[1] == 'calendar-order')
+				if ($property[1] == 'calendar-color')
+				{
+					$color = $value->_children[0]->get_data();
+					$updated = $folder_info['color'] != $color;
+					$folder_info['color'] = $color;
+					continue;
+				}
+
+				if ($property[1] == 'calendar-order')
 				{
 					continue;
 				}
@@ -842,7 +859,7 @@ class user_collection_resource extends collection_resource
 			$deny = $ace->get_child('DAV::deny');
 			if ($deny)
 				throw new forbidden_precondition_exception('DAV:', 'grant-only');
-			if (!$principal || !grant)
+			if (!$principal || !$grant)
 				throw new bad_request_exception();
 			 			if ($principal->get_child('DAV::all'))
 				$principal_name = 'anyone';
