@@ -30,6 +30,7 @@ function fdrCompare($a,$b)
 
 class slHelperFolder extends slHelper{
 	static $personalFolders;
+	static $mappedFolders;
 
 	public function check()
 	{
@@ -359,8 +360,9 @@ class slHelperFolder extends slHelper{
 						}
 					}
 					if(is_array($init_folders) && !empty($init_folders)){
+						$foldersByType = [];
 						foreach($init_folders as $init_folder){
-							 							if(strpos($init_folder->name,'~')!==0 && !$folder->archive){
+							 							if(strpos($init_folder->name,'~')!==0 && !$init_folder->archive){
 								if($forbidden) foreach($forbidden as $pub){
 									if(substr($init_folder->name,0,strlen($pub))==$pub){
 										$skip = true;
@@ -384,7 +386,7 @@ class slHelperFolder extends slHelper{
 					if($xml->folders->$tagName){
 						foreach($xml->folders->$tagName->folder as $tag => $value){
 							$val = strval($value);
-							if(!in_array($val,$this->mappedFolders) || ($type=='C' && $folder!='__@@ADDRESSBOOK@@__')){
+							if(!in_array($val,$this->mappedFolders) || ($type=='C' && $val!='__@@ADDRESSBOOK@@__')){
 								try{
 									$fdr = $account->getFolder($val);
 									$folders[$val] = $this->getInfo($type,$val);
