@@ -6,6 +6,9 @@ require_once('inc/user.php');
 class slHelperSettings extends slHelper
 {
 	public static $storage_function;
+	public static $initialized;
+	public static $logged;
+	public static $language;
 	public function check()
 	{
 		if(!$this->initialized){
@@ -18,7 +21,6 @@ class slHelperSettings extends slHelper
 		$session = slSession::instance();
 
 		 		if(!($data = $session->getMain('["cache"]["helper"]["settings"]["data"]'))){
-			WebmailIqAuth::checkServerKeys();
 			 
 			$remember['login']='';
 			$remember['password']='';
@@ -47,7 +49,7 @@ class slHelperSettings extends slHelper
 			$data['remember']=$remember;
 			 
 
-			$data['hash'] = file_get_contents(WM_CONFIGPATH.'public.key');
+			$data['hash'] = WebmailRSA::instance()->getServerData('public');
 			if($oUser = $_SESSION['user']){
 				$this->logged = true;
 				$oPrimaryAccount = $_SESSION['user']->getAccount($_SESSION['EMAIL']);
@@ -451,13 +453,9 @@ class slHelperSettings extends slHelper
 						$old = self::$storage_function['changepwd']['values']['oldpassword'];
 						
 						if($new || $old || $confirm){
-							WebmailIqAuth::checkServerKeys();
-							$privateData = file_get_contents(WM_CONFIGPATH.'private.key');
-							$privateKey = openssl_pkey_get_private($privateData);
-							
-							openssl_private_decrypt(WebmailIqAuth::hexStringToBinary($old), $sPassword, $privateKey);
-							openssl_private_decrypt(WebmailIqAuth::hexStringToBinary($new), $sNewPassword, $privateKey);
-							openssl_private_decrypt(WebmailIqAuth::hexStringToBinary($confirm), $sCheckPassword, $privateKey);
+							$old = WebmailRSA::instance()->rsaToPlain($sPassword);
+							$new = WebmailRSA::instance()->rsaToPlain($sNewPassword);
+							$confirm = WebmailRSA::instance()->rsaToPlain($sCheckPassword);
 							
 							if($sCheckPassword!=$sNewPassword){
 								throw new Exc('password_confirmation');
