@@ -5,6 +5,13 @@ slSystem::import('controller/grid',APP_PATH);
 class slControllerItem extends slControllerGrid
 {
 	private $phones=array();
+	public $actionMethod;
+	public $oAccount;
+	public $oQuarantineFolder;
+	public $part_id;
+	public $accept_folder;
+	public $oFolder;
+	public $application;
 	
 	public function mergeItems()
 	{
@@ -205,7 +212,7 @@ class slControllerItem extends slControllerGrid
 		$i = 0;
 		if($items) foreach($items as $itm => $item){
 			if($item['selected']){
-				$address = Mailparse::parseAddresses($item['address']);
+				$address = MailParse::parseAddresses($item['address']);
 				$address = reset($address);
 				$lct['LCTDESCRIPTION'] = $address['display'];
 				$lct['LCTEMAIL1'] = $address['address'];
@@ -239,7 +246,7 @@ class slControllerItem extends slControllerGrid
 		$aItem = $this->prepareItem();
 				
 		$aItem = array_change_key_case($aItem,CASE_UPPER);
-		$id = $this->storeItem($aItem,$edit_items);
+		$id = $this->storeItem($aItem);
 		return $this->loadDistributionListFromSession($id,$folder,'distribution_contact',$action);
 	}
 	
@@ -420,7 +427,7 @@ class slControllerItem extends slControllerGrid
 				foreach($aItems as $sID => $selected){
 					$oItem = $this->oFolder->getItem($sID);
 					$aItemsToMove[] = $oItem;
-					$aFrom = Mailparse::parseAddresses($oItem->from);
+					$aFrom = MailParse::parseAddresses($oItem->from);
 					$sFrom = $aFrom[0]['address'];
 					$subresult = $this->oQuarantineFolder->createItem(array('email'=>$sFrom));
 					$result = $result && $subresult;
@@ -548,7 +555,7 @@ class slControllerItem extends slControllerGrid
 			$filter = array(
 				'tag'=>'header_from, header_to, subject, html, text, date, reply_to'
 			);
-			$message['data'] = WebmailIQItems::cnvMailItem($item,$filter,1,false);
+			$message['data'] = WebmailIqItems::cnvMailItem($item,$filter,1,false);
 			$message['data']['date'] = date('Y/m/d H:i:d',$message['data']['date']);
 			$itemHelperConstructor = array('helper'=>'item');
 			$itemHelper = slHelperFactory::instance($itemHelperConstructor);
@@ -774,6 +781,7 @@ class slControllerItem extends slControllerGrid
 			$response->error = $e;
 			return $response;
 		}
+		$redirect = ($request->get('all._n.target'))?false:true;
 		if($redirect){
 			$response->redirectURL = $this->getContainerLink($selector);
 			$response->redirect = true;
@@ -972,6 +980,7 @@ class slControllerItem extends slControllerGrid
 			$read_confirmation_text = $language->get('mail_compose','read_confirmation_text');
 		}
 		
+		$mail = new Mail('UTF-8');
 		$data['SUBJECT'] = $oItem->subject;
 		$data['FROM'] = $oItem->from;
 		$data['TO'] = $oItem->to;
@@ -981,12 +990,11 @@ class slControllerItem extends slControllerGrid
 		}else{
 			$data['DATE'] = date("Y/m/d H:i:s O",$oItem->date);
 		}
-		$aItem = WebmailIQItems::cnvMailItem($oItem,array('tag'=>'subject,header_from,header_to,confirm_addr,date,size'));
+		$aItem = WebmailIqItems::cnvMailItem($oItem,array('tag'=>'subject,header_from,header_to,confirm_addr,date,size'));
 		
 		$read_confirmation_subject = $this->replaceVar($read_confirmation_subject,$data);
 		$read_confirmation_text = $this->replaceVar($read_confirmation_text,$data);
 		
-		$mail = new Mail('UTF-8');
 		$mail->XMailer.='-Tablet';
 		$mail->setSMTPAuth($_SESSION['SMTP_AUTH']);
 		$mail->setSMTPUsername($this->oAccount->username);
