@@ -2,6 +2,8 @@
 
 class slControllerMailview extends slControllerDefault
 {
+	public $controller;
+	public $appplication;
 	public function check( &$action, &$data)
 	{
 		parent::check( $action, $data );
