@@ -18,6 +18,11 @@ class slControllerMessage extends slControllerDefault
 	public $files;
 	public $data;
 	public $js;
+	public $charset;
+	public $bcc;
+	public $aEmbeddedAttachments;
+	public $aAttrs;
+	public $EA;
 	
 	 	public function check( &$action , &$data )
 	{
@@ -45,6 +50,7 @@ class slControllerMessage extends slControllerDefault
 		$result = new stdClass();
 		$session = slSession::instance();
 		$router = slRouter::instance();
+		$id = '';
 		try{
 			$mail = $this->createMessage(true,$failedAttachments);
 			$this->processDistributionLists($mail);
@@ -67,7 +73,7 @@ class slControllerMessage extends slControllerDefault
 			if($failedAttachments){
 				throw new Exception();
 			}
-			 			$preparedFile = $mail->prepareMessageTempFile($this->sCharset);
+			 			$preparedFile = $mail->prepareMessageTempFile($this->charset);
 			if($this->info['options']['encrypt']){
 				
 				$gwmanage = new GroupWareManagement();
@@ -468,7 +474,7 @@ class slControllerMessage extends slControllerDefault
 		
 		 		$bcc = $request->get('all._frm.bcc');
 		if ($bcc || $this->bccDlAddrList){
-			$bcc = Mailparse::parseAddresses($bcc);
+			$bcc = MailParse::parseAddresses($bcc);
 			$bccCount = 0;
 			if($bcc) foreach($bcc as $bccAddress){
 				if(!$bccAddress['display'] || $bccAddress['display']==$bccAddress['address']){
@@ -479,7 +485,7 @@ class slControllerMessage extends slControllerDefault
 			}
 			if($this->bccDlAddrList){
 				foreach($this->bccDlAddrList as $address){
-					$aFixedBCC[] = $oMail->AddrFormat(
+					$aFixedBCC[] = $mail->AddrFormat(
 						array(
 							$address['address'],
 							$address['display']
@@ -515,7 +521,7 @@ class slControllerMessage extends slControllerDefault
 		}
 		
 		if($this->info['options']['encrypt'] || $this->info['options']['sign']){
-			$mail->publicCertificates = slSMIME::personalToPublic($mail->personalCertificates);
+			$mail->publicCertificates = slSMime::personalToPublic($mail->personalCertificates);
 			$mail->overrideRecipient($_SESSION['EMAIL']);
 			$fp = fopen($preparedFile,'r+');
 			$mail->encryptAndSign($fp,$filename);
@@ -648,7 +654,7 @@ class slControllerMessage extends slControllerDefault
 						if(!$error){
 							$oMail->$methodName($email,$name!=$email?$name:'');
 							if(strtoupper($dest) == 'BCC'){
-								$iq->bccDlAddrList[] = array(
+								$this->bccDlAddrList[] = array(
 									'address'=>$email,
 									'display'=>$name
 								);
@@ -674,8 +680,7 @@ class slControllerMessage extends slControllerDefault
 		$script = $_SERVER['SCRIPT_NAME'];
 		$script = str_replace(WEBMAIL_PHP, DOWNLOAD_PHP, $script);
 		$script = preg_replace('/-.._._.--.._[0-9]{0,}\//si','',$script);
-		$absolute = 'http' . (($_SERVER['HTTPS'] == 'ON' || ($_SESSION['ALWAYSHTTPS'] ?? false)) ? 's' : '') . '://' . $_SERVER['HTTP_HOST'] .
-			$script;
+		$absolute = filter_var('http' . (($_SERVER['HTTPS'] == 'ON' || ($_SESSION['ALWAYSHTTPS'] ?? false)) ? 's' : '') . '://' . $_SERVER['HTTP_HOST'] . $script, FILTER_VALIDATE_URL);
 		$prefixmatch = '('.preg_quote(DOWNLOAD_PHP).
 						'|'.preg_quote($absolute).
 						'|'.preg_quote($this->getDottedURI()).')';
@@ -722,7 +727,6 @@ class slControllerMessage extends slControllerDefault
 	
 	private function getDottedURI()
 	{
-		$host = $_SERVER['HTTP_HOST'];
 		$uri = $_SERVER['REQUEST_URI'];
 		if(strpos($uri,'?')!==false){
 			$uri = substr($uri,0,strpos($uri,'?'));
@@ -740,12 +744,13 @@ class slControllerMessage extends slControllerDefault
 	}
 	
  
-	static public function addAttachment(&$oMail, $class, $type, $data, $cid = false)
+	public function addAttachment(&$oMail, $class, $type, $data, $cid = false)
 	{
 		$oUser = &$_SESSION['user'];
 		if($cid!==false){
 			$data['cid'] = $cid;
 		}
+		$info = [];
 		switch ($class){
 			 				 			case 'session':
 				$session = slSession::instance();
@@ -802,7 +807,7 @@ class slControllerMessage extends slControllerDefault
 				$file = $oItem->getAttachmentDataFileCID($attData['part'], $info);
 				if ($info['encoding'] == 'uu'){
 					$info['encoding'] = 'base64';
-					$part = base64_encode($part);
+					$part = base64_encode($attData['part']);
 				}
 				switch ($type)
 				{
@@ -838,7 +843,7 @@ class slControllerMessage extends slControllerDefault
 				$file = $oItem->getAttachmentDataFile($attData['part'], $info);
 				if ($info['encoding'] == 'uu'){
 					$info['encoding'] = 'base64';
-					$part = base64_encode($part);
+					$part = base64_encode($attData['part']);
 				}
 				switch ($type)
 				{
