@@ -6,6 +6,8 @@ require_once('inc/exception.php');
 class WebmailModel
 {
 	private static $instance;
+	public $api;
+	public $settings;
 	
 	static public function instance( &$settings = false )
 	{
