@@ -15,10 +15,8 @@ $(document).ready(function(){
 		}
 	});
 	$('table.datagrid td a,table.datagrid td input').click(function(event) {
-		var agent = jQuery.browser;
-		if(agent && agent.msie) {
-			event.cancelBubble = true;
-		} else {
+		event.cancelBubble = true;
+		if (event.stopPropagation) {
 			event.stopPropagation();
 		}
 	});
