@@ -1,13 +1,10 @@
 <?php
 
-class slWidgetWelcome extends Widget
+class slWidgetWelcome extends slWidget
 {
 	public function create($aConstruct)
 	{
 		parent::create($aConstruct);
-		$language = Language::instance();
-		$request = Request::instance();
-		$xml = $aConstruct['xml'];
 		 		$this->setTemplateData('sid',$_SESSION['SID']);
 		if($_SESSION['DISK_QUOTA']){
 			$quota = $_SESSION['DISK_QUOTA'];
