@@ -21,8 +21,11 @@ function NewMessage(id) {
 		this.__id = id;
 
 	this.onSentCallback = [];
+	// this.onTChCallback = [];
 	this.onDisposeCallback  = [];
 	this.oHeaders = {};
+	this.sTemplate = '';
+	this.sBody = '';
 
 	this.signatureID = 0;
 };
@@ -33,11 +36,11 @@ NewMessage.crlf = '<div><br></div>';
  * @params:	sRcp - comma separated list of recipient emails
  **/
 NewMessage.signature = function(sRcp){
-	var iAliasID = null,
+	var i, tmp, iAliasID = null,
 		aTmp = dataSet.get('storage',['ALIASES','ITEMS']);
 
-    if (GWOthers.getItem('RESTRICTIONS', 'disable_personalities')>0){
-		for(var i in aTmp)
+    if (GWOthers.getItem('RESTRICTIONS', 'disable_personalities')>0 && GWOthers.getItem('RESTRICTIONS', 'disable_delegated_aliases')>0){
+		for(i in aTmp)
 			if (aTmp[i].VALUES.EMAIL.VALUE.toLowerCase() == sPrimaryAccount){
 				iAliasID = i;
 				break;
@@ -46,12 +49,12 @@ NewMessage.signature = function(sRcp){
 	else{
 		var aRcp = {};
 		if (sRcp){
-			var tmp = MailAddress.splitEmailsAndNames(sRcp);
-			for (var i in tmp)
+			tmp = MailAddress.splitEmailsAndNames(sRcp);
+			for (i in tmp)
 				aRcp[tmp[i].email.toLowerCase()] = true;
 		}
 		else{
-			var tmp = GWOthers.getItem('MAIL_SETTINGS_DEFAULT', 'from');
+			tmp = GWOthers.getItem('MAIL_SETTINGS_DEFAULT', 'from');
 			if (tmp && (tmp = MailAddress.splitEmailsAndNames(tmp))){
 			    if (tmp[0] && tmp[0].email.toLowerCase() != sPrimaryAccount.toLowerCase())
 			    	aRcp[tmp[0].email.toLowerCase()] = true;
@@ -62,7 +65,7 @@ NewMessage.signature = function(sRcp){
 				aRcp[sPrimaryAccount.toLowerCase()] = true;
 		}
 
-		for(var i in aTmp)
+		for(i in aTmp)
 			if (aTmp[i].VALUES && ((iAliasID == null && aTmp[i].VALUES.EMAIL.VALUE.toLowerCase() == sPrimaryAccount) || (aTmp[i].VALUES.ENABLED && aTmp[i].VALUES.ENABLED.VALUE == '1' && aRcp[aTmp[i].VALUES.EMAIL.VALUE.toLowerCase()])))
 				iAliasID = i;
 	}
@@ -71,8 +74,11 @@ NewMessage.signature = function(sRcp){
 		var bDelegate = aTmp[iAliasID].VALUES.ISDELEGATE && aTmp[iAliasID].VALUES.ISDELEGATE.VALUE == '1';
 		aTmp = aTmp[iAliasID].VALUES[Is.Defined(sRcp)?'SIGN2':'SIGN1'] || {VALUE:'0'};
 		var aSign = dataSet.get('storage',['SIGNATURE','ITEMS']);
-		var iLastUsed = dataSet.get('cookies', ['last_used_signature']);
-		for (var i in aSign)
+		var iLastUsed;
+		if (GWOthers.getItem('MAIL_SETTINGS_DEFAULT', 'remember_last_used_signature') > 0) {
+			iLastUsed = dataSet.get('cookies', ['last_used_signature']);
+		}
+		for (i in aSign)
 	    	if ((iLastUsed && iLastUsed == i) || !iLastUsed && (aSign[i].VALUES.ID && aSign[i].VALUES.ID.VALUE == aTmp.VALUE || (aTmp.VALUE == '0' && !aSign[i].VALUES.ID)))
                 if (aSign[i].VALUES.TEXT){
                 	var signature = aSign[i].VALUES.TEXT.VALUE;
@@ -102,7 +108,7 @@ NewMessage.linkFix = function(v){
 		try{
 			elm.href = (elm.protocol?elm.protocol + (elm.protocol == 'mailto:'?'':'//'):'') + elm.host + elm.pathname + elm.search + elm.hash;
 		}
-		catch(r){
+		catch{
 			elm.removeAttribute('href');
 		}
 	});
@@ -133,8 +139,8 @@ NewMessage.compose_sms = function(aResponse) {
 		NewMessage.compose({sms:true});
 };
 
-NewMessage.compose = function(aRecipients, bIsHTML) {
-
+NewMessage.compose = async function(aRecipients, bIsHTML, oGui) {
+	oGui = oGui || gui;
 	var newMessage = new NewMessage();
 
 	if (aRecipients instanceof OldMessage){
@@ -147,7 +153,11 @@ NewMessage.compose = function(aRecipients, bIsHTML) {
 		newMessage.sSMS = aRecipients.getSms();
 		newMessage.sSubject = aRecipients.getSubject();
 		newMessage.setHtml(aRecipients.isHtml(true));
-		newMessage.sBody = aRecipients.getBody();
+		if ((aRecipients.__mailInfo.aid + '/' + aRecipients.__mailInfo.fid) === GWOthers.getItem('DEFAULT_FOLDERS','templates')) {
+			newMessage.sTemplate = aRecipients.getBody();
+		} else {
+			newMessage.sBody = aRecipients.getBody();
+		}
 		newMessage.aAttachments = aRecipients.copyAttachments(aRecipients.__id);
 		delete newMessage.__id;
 	}
@@ -178,7 +188,7 @@ NewMessage.compose = function(aRecipients, bIsHTML) {
 				newMessage.sBody = aRecipients.htmlBody;
 			}
 
-			newMessage.addSignature();
+			// newMessage.addSignature();
 		}
 
 		if (aRecipients.teamchat){
@@ -197,9 +207,10 @@ NewMessage.compose = function(aRecipients, bIsHTML) {
 	    	newMessage.sBody = aRecipients.body;
 			newMessage.setHtml(false);
 		}
+		if (aRecipients.oSave) {
+			newMessage.oSave = aRecipients.oSave;
+		}
 	}
-	else
-		newMessage.addSignature();
 
 	if (!aRecipients || (!aRecipients.sms && !aRecipients.template))
 		newMessage.addvCard();
@@ -207,15 +218,24 @@ NewMessage.compose = function(aRecipients, bIsHTML) {
 	if(bIsHTML !== void 0) {
 		newMessage.setHtml(bIsHTML);
 	}
-	return gui._create('frm_compose', 'frm_compose', '', '', newMessage);
+	
+	var args = [].slice.call(arguments).filter(function(arg) {
+		return arg !== oGui;
+	});
+	return await oGui._create('frm_compose', 'frm_compose', '', '', newMessage, void 0, {
+		context: 'NewMessage',
+		fun: ['compose'],
+		args: args
+	});
 };
 
-NewMessage.composeTemplate = function(){
+NewMessage.composeTemplate = async function(){
 	var f = Path.split(GWOthers.getItem('DEFAULT_FOLDERS','templates')),
-		frm = gui._create('template','frm_insert_item','','',[
-		function(aPath){
-			if (aPath = aPath[0]){
-				var oldTemplate = new OldMessage([aPath.aid, aPath.fid, aPath.id]);
+		frm = await gui._create('template','frm_insert_item','','frm_insert_item_nobottomdiv',[
+		async function(aPath){
+			if ((aPath = aPath[0])){
+				var oldTemplate = new OldMessage();
+				await oldTemplate.init([aPath.aid, aPath.fid, aPath.id]);
 				NewMessage.compose(oldTemplate);
 			}
 		}],
@@ -236,7 +256,7 @@ NewMessage.prototype.setHtml = function(bHtml) {
 	this.__bHtml = bHtml?true:false;
 };
 
-NewMessage.prototype.save = function(bKeepAttachments, aNotify, aSettingsDefault, bTeamChat) {
+NewMessage.prototype.save = async function(bKeepAttachments, aHandler, aSettingsDefault, bTeamChat) {
 
 	var aMessageIQ = this.__loadCommonMessageIQ(bKeepAttachments, aSettingsDefault);
 	aMessageIQ.to = this.sTo;
@@ -248,8 +268,8 @@ NewMessage.prototype.save = function(bKeepAttachments, aNotify, aSettingsDefault
 		aMessageIQ.teamchat = this.sTeamchat;
 		aMessageIQ.teamchat_comment = this.sComment || '';
 
-		if (!this.__id_chat || this.__id_chat[1] != this.sTeamchat.replace(/(::.+)$/g,''))
-			this.__id_chat = {0:sPrimaryAccount, 1:this.sTeamchat.replace(/(::.+)$/g,'')};
+		if (!this.__id_chat || this.__id_chat[1] != this.sTeamchat.replace(/::.+$/g,''))
+			this.__id_chat = {0:sPrimaryAccount, 1:this.sTeamchat.replace(/::.+$/g,'')};
 
 		aMessageIQ['action']['save_chat'] = {aid: this.__id_chat[0], fid: this.__id_chat[1]};
 		if (this.__id_chat[2]) aMessageIQ['action']['save_chat']['iid'] = WMItems.__serverID(Array.isArray(this.__id_chat[2]) ? this.__id_chat[2][0] : this.__id_chat[2]);
@@ -266,25 +286,43 @@ NewMessage.prototype.save = function(bKeepAttachments, aNotify, aSettingsDefault
 		if (!this.__id) {
 			this.__id = [];
 
-			var sTarget = Path.split(GWOthers.getItem('DEFAULT_FOLDERS', (this.template?'templates':'drafts')));
+			var aTarget = [];
+			if (this.template) {
+				aTarget = Path.split(GWOthers.getItem('DEFAULT_FOLDERS', ('templates')));
+			} else {
+				aTarget = Path.split(GWOthers.getItem('DEFAULT_FOLDERS', ('drafts')));
+				var from = MailAddress.splitEmailsAndNames(aMessageIQ.from)[0].email;
+				var aliases = dataSet.get('storage',['ALIASES','ITEMS']);
 
-			this.__id[0] = sTarget[0];
-			this.__id[1] = sTarget[1];
+				for(var i in aliases) {
+					if (aliases[i].VALUES.EMAIL.VALUE.toLowerCase() === from && (aliases[i].VALUES.DRAFTFOLDER || {}).VALUE) {
+						aTarget = Path.split(aliases[i].VALUES.DRAFTFOLDER.VALUE);
+					}
+				}
+			}
 
-			if (!dataSet.get('folders', sTarget))
+			this.__id[0] = aTarget.shift();
+			this.__id[1] = aTarget.join('/');
+
+			if (!dataSet.get('folders', this.__id))
 				dataSet.add('folders', this.__id, {'NAME': getLang('COMMON_FOLDERS::' + (this.template ? 'TEMPLATES' : 'DRAFTS')),'TYPE':'M','RIGHTS':'rwmd'});
 		}
 
+		if (this.oSave) {
+			this.__id[0] = this.oSave.aid;
+			this.__id[1] = this.oSave.fid;
+		}
+
 		aMessageIQ['action']['save'] = {"aid": this.__id[0], "fid": this.__id[1]};
 		if (this.__id[2]) aMessageIQ['action']['save']['iid'] = Array.isArray(this.__id[2]) ? this.__id[2][0] : this.__id[2];
 	}
 
-	storage.library('wm_messages');
-	return message.add(aMessageIQ, 'dummy', '', [this, '__messageSaved', [aNotify, bTeamChat]], aNotify);
+	await storage.library('wm_messages');
+	return await message.add(aMessageIQ, '', '', [this, '__messageSaved', [aHandler, bTeamChat]]);
 };
 
-NewMessage.prototype.send = function(bKeepAttachments, aNotify, aSettingsDefault) {
-	var aSettingsDefault = aSettingsDefault || {};
+NewMessage.prototype.send = async function(bKeepAttachments, aHandler, aSettingsDefault) {
+	aSettingsDefault = aSettingsDefault || {};
 	var aMessageIQ = this.__loadCommonMessageIQ(bKeepAttachments, aSettingsDefault);
 
 	if (this.sTo == '' && this.sCc == '' && this.sBcc == '' && this.sSMS == '')
@@ -296,6 +334,7 @@ NewMessage.prototype.send = function(bKeepAttachments, aNotify, aSettingsDefault
 	if ((typeof aSettingsDefault == 'object' && parseInt(aSettingsDefault['save_sent_message'])>0) || (typeof aSettingsDefault != 'object' && parseInt(GWOthers.getItem('MAIL_SETTINGS_DEFAULT', 'save_sent_message'))>0)) {
 		var sSentFolder = Path.split(aSettingsDefault['sent'] || GWOthers.getItem('DEFAULT_FOLDERS', 'sent'));
 		aMessageIQ['action']['save'] = {"aid": sSentFolder[0], "fid": sSentFolder[1]};
+		aMessageIQ['action']['include_in_sent_folder'] = aSettingsDefault.include_in_sent_folder || 0;
 	}
 
 	//Save to Teamchat
@@ -303,8 +342,8 @@ NewMessage.prototype.send = function(bKeepAttachments, aNotify, aSettingsDefault
 		aMessageIQ.teamchat = this.sTeamchat;
 		aMessageIQ.teamchat_comment = this.sComment || '';
 
-		if (!this.__id_chat || this.__id_chat[1] != this.sTeamchat.replace(/(::.+)$/g,''))
-			this.__id_chat = {0:sPrimaryAccount, 1:this.sTeamchat.replace(/(::.+)$/g,'')};
+		if (!this.__id_chat || this.__id_chat[1] != this.sTeamchat.replace(/::.+$/g,''))
+			this.__id_chat = {0:sPrimaryAccount, 1:this.sTeamchat.replace(/::.+$/g,'')};
 
 		aMessageIQ['action']['save_chat'] = {aid: this.__id_chat[0], fid: this.__id_chat[1]};
 		if (this.__id_chat[2]) aMessageIQ['action']['save_chat']['iid'] = WMItems.__serverID(Array.isArray(this.__id_chat[2]) ? this.__id_chat[2][0] : this.__id_chat[2]);
@@ -321,25 +360,40 @@ NewMessage.prototype.send = function(bKeepAttachments, aNotify, aSettingsDefault
     if (aSettingsDefault.sms>0 && this.sSMS)
 		aMessageIQ.sms = this.sSMS;
 
+	if (this.__id && this.__id[2]) {
+		aMessageIQ.action.draft_id = WMItems.__serverID(this.__id[2]);
+	}
+
 	var rst = GWOthers.getItem('RESTRICTIONS', 'disable_gw_types') || '';
 	if (rst.indexOf('c')<0 && window.sPrimaryAccountGW && parseInt(GWOthers.getItem('MAIL_SETTINGS_GENERAL', 'auto_recipient_to_addressbook')))
 		aMessageIQ['action']['auto_addressbook'] = 'true';
 
 	//REMOVE iw-remove marked tags
 	if (aMessageIQ.body){
-		storage.library('purify.wrapper', 'purify');
+		await storage.library('purify.wrapper', 'purify');
 
-		var safeHtml = mkElement('div', {innerHTML: DOMPurify.sanitize(aMessageIQ.body)});
+		var safeHtml = document.createElement('template');
+			safeHtml.innerHTML = DOMPurify.sanitize(aMessageIQ.body);
 
-		[].forEach.call(safeHtml.querySelectorAll('[iw-remove='+ (aMessageIQ.action.html_body === "true"?'html':'text') +']'), function(elm){
+		[].forEach.call(safeHtml.content.querySelectorAll('[iw-remove='+ (aMessageIQ.action.html_body === "true"?'html':'text') +']'), function(elm){
 			elm.parentNode.removeChild(elm);
 		});
 		aMessageIQ.body = safeHtml.innerHTML;
 		aMessageIQ.body = aMessageIQ.body.replace(/class="classification[^"]*?"/g, '');
 	}
 
-	storage.library('wm_messages');
-	return message.add(aMessageIQ, 'dummy', '' , [this, '__messageSent', [aNotify]], aNotify);
+	var recipientHistory = Cookie.get(['recipient_history']) || [];
+	MailAddress.splitEmailsAndNames([this.sTo, this.sCc, this.sBcc].filter(Boolean).join(',')).reverse().forEach(function(recipient) {
+		if (~recipientHistory.indexOf(recipient.email)) {
+			recipientHistory.splice(recipientHistory.indexOf(recipient.email), 1);
+		}
+		recipientHistory.unshift(recipient.email);
+	});
+	recipientHistory = recipientHistory.slice(0, 15);
+	Cookie.set(['recipient_history'], recipientHistory);
+
+	await storage.library('wm_messages');
+	return await message.add(aMessageIQ, '', '' , [this, '__messageSent', [aHandler]]);
 };
 
 NewMessage.prototype.dispose = function() {
@@ -367,11 +421,13 @@ NewMessage.prototype.addSignature = function(bBottom) {
 	var aSign = NewMessage.signature(this.sRcp),
 		sSign = NewMessage.crlf + NewMessage.crlf + (aSign.text ? '<div class="iw-signature">' + aSign.text + '</div>' : '');
 
+	sSign = sSign.replace(/<html.*?>.*?<body.*?>/, '').replace(/<\/body>.*?<\/html>/, '').replace(/<style(.*?)>/g, '<style $1 class="iw-skip">');
+
 	//Add default signature ID
 	this.signatureID = aSign.id;
 	this.bDelegate = aSign.delegate;
 
-	if (GWOthers.getItem('MAIL_SETTINGS_DEFAULT', 'classification')>0) {
+	if (GWOthers.getItem('MAIL_SETTINGS_DEFAULT', 'classification')>0 && !~this.sBody.indexOf('class="classification')) {
 		sSign = '<div class="classification" rel="main"></div>' + sSign + '<div class="classification end" rel="main"></div>'
 	}
 
@@ -382,7 +438,7 @@ NewMessage.prototype.addSignature = function(bBottom) {
 };
 
 NewMessage.prototype.__loadCommonMessageIQ = function(bKeepAttachments, aSettings) {
-	var aMessageIQ = {};
+	var i, aMessageIQ = {};
 
 	if (typeof aSettings != 'object')
 		aSettings = GWOthers.get('MAIL_SETTINGS_DEFAULT', 'storage')['VALUES'];
@@ -393,18 +449,37 @@ NewMessage.prototype.__loadCommonMessageIQ = function(bKeepAttachments, aSetting
 		"html_body": (this.isHtml()) ? 'true' : 'false'
 	};
 
-	if (aSettings.smart_attach>0){
-
-		aMessageIQ['action'].smart_attach = 'true';
-
-		if (aSettings.smart_path){
-			var asap = Path.split(aSettings.smart_path);
-			if (asap && asap[1])
-				aMessageIQ['action'].smart_attach_folder = asap[1];
- 		}
-	}
-
 	aMessageIQ['from'] = aSettings['from'] || getPrimaryAccountFromAddress();
+
+	var smartattach = {
+		process: false
+	};
+	if (aSettings.direct_attach==0){
+
+		smartattach.process = true;
+
+		var asap = Path.split(GWOthers.getItem('DEFAULT_FOLDERS','smartattach'));
+		var from = MailAddress.splitEmailsAndNames(aMessageIQ['from'])[0].email;
+		var aliases = dataSet.get('storage',['ALIASES','ITEMS']);
+
+		for(i in aliases) {
+			if (aliases[i].VALUES.EMAIL.VALUE.toLowerCase() === from && (aliases[i].VALUES.SMARTFOLDER || {}).VALUE) {
+				asap = Path.split(aliases[i].VALUES.SMARTFOLDER.VALUE);
+			}
+		}
+
+		if (asap && asap[1])
+			smartattach.folder = asap[1];
+
+		if (aSettings.webclient_smart_password) {
+			smartattach.password = aSettings.webclient_smart_password;
+		}
+		if (aSettings.webclient_smart_expiration) {
+			smartattach.expire = aSettings.webclient_smart_expiration;
+		}
+	}
+	aMessageIQ.action.smartattach = smartattach;
+
 	aMessageIQ['subject'] = this.sSubject;
 
 	if (aSettings.sms>0){
@@ -418,7 +493,7 @@ NewMessage.prototype.__loadCommonMessageIQ = function(bKeepAttachments, aSetting
 		aMessageIQ['deferred_date'] = this.sDeferred;
 
 	var aHeaders = [];
-	if (parseInt(aSettings['read_confirmation'])) {
+	if (parseInt(aSettings['read_confirmation']) || aSettings['read_confirmation'] === true) {
 		aHeaders.push('X-Confirm-Reading-To: ' + aMessageIQ['from']);
 		aHeaders.push('Disposition-Notification-To: ' + aMessageIQ['from']);
 	}
@@ -435,7 +510,7 @@ NewMessage.prototype.__loadCommonMessageIQ = function(bKeepAttachments, aSetting
 		aHeaders.push('X-IceWarp-WebClient-Edit-Disabled: 1');
 	}
 	// Add any custom headers for this message
-	for(var i in this.oHeaders)
+	for(i in this.oHeaders)
 		aHeaders.push(i+': '+this.oHeaders[i]);
 
 	//RFC 2822 Reply conversations
@@ -474,22 +549,34 @@ NewMessage.prototype.__loadCommonMessageIQ = function(bKeepAttachments, aSetting
 
 		for (var n in aAttachments){
 			aAttachResult[n] = {
-				"class":(aAttachments[n]['class'] ? aAttachments[n]['class'] : 'file'),"type":'normal',
-				"fullpath":(aAttachments[n]['class'] ? aAttachments[n]['fullpath'] : sUploadFolId + '/' + aAttachments[n]['id'])
+				class: aAttachments[n].class || 'file',
+				type: 'normal',
+				fullpath: aAttachments[n].class ? aAttachments[n].fullpath : sUploadFolId + '/' + aAttachments[n].id,
+				description: aAttachments[n].description || aAttachments[n].attachment || aAttachments[n].name
 			};
-			if (aAttachments[n]['description'])
-				aAttachResult[n]['description'] = aAttachments[n]['description'];
+
+			if (aAttachments[n].class === 'item') {
+				aHeaders.push('X-Icewarp-Att-Data:' + this.__encodeUTF8Header(aAttachments[n].name) + '=' + this.__encodeUTF8Header(aAttachResult[n].fullpath));
+			}
 		}
 
-		if (!Is.Empty(aAttachResult))
-			aMessageIQ['attachments'] = aAttachResult;
+		if (!Is.Empty(aAttachResult)) {
+			aMessageIQ.attachments = aAttachResult;
+		}
 	}
 
 	return aMessageIQ;
 };
 
-NewMessage.prototype.__messageSaved = function(bOK, sUId, sError, aNotify, bChat) {
 
+NewMessage.prototype.__encodeUTF8Header = function(string) {
+	return '=?UTF-8?B?' + btoa(encodeURIComponent(string)) + '?=';
+};
+
+/**
+ * @param {object or false} bOK
+ */
+NewMessage.prototype.__messageSaved = function(bOK, sUId, sError, aHandler, bChat) {
 	if (bOK && (sUId.length || (bChat && bOK.TEAMCHAT_ID))) {
 		var bFirstTime = false;
 
@@ -503,12 +590,21 @@ NewMessage.prototype.__messageSaved = function(bOK, sUId, sError, aNotify, bChat
 			this.__id_chat[1] = Path.slash(bOK.TEAMCHAT);
 			this.__id_chat[2] = bOK.TEAMCHAT_ID;
 
-			Item.notify(this.__id_chat, sAction, 'M');
+			Item.notify(this.__id_chat, sAction, {
+				'ITEM-TYPE': 'M'
+			});
 
 			//Always ADD for Y
 			if (bOK.TEAMCHAT_LINK_ID)
-				Item.notify([this.__id_chat[0], this.__id_chat[1], WMItems.__clientID(bOK.TEAMCHAT_LINK_ID)], 'add', 'Y');
-				//NewMessage.notify('add', [this.__id_chat[0],this.__id_chat[1],WMItems.__clientID(bOK.TEAMCHAT_LINK_ID)],'Y');
+				Item.notify([this.__id_chat[0], this.__id_chat[1], WMItems.__clientID(bOK.TEAMCHAT_LINK_ID)], 'add', {
+					'ITEM-TYPE': 'Y'
+				});
+
+			//Call onTChCallback
+			// for(var i in this.onTChCallback){
+			// 	pushParameterToCallback(this.onTChCallback[i], {aid:sPrimaryAccount, fid:this.__id_chat[1], iid:this.__id_chat[2]});
+			// 	executeCallbackFunction(this.onTChCallback[i]);
+			// }
 		}
 		else{
 
@@ -517,9 +613,10 @@ NewMessage.prototype.__messageSaved = function(bOK, sUId, sError, aNotify, bChat
 				this.__id[2] = sUId;
 
 				//Remove saved email from Drafts folder after Sent
-				this.onSentCallback.push([function(id){
+				this.onSentCallback.push([function(id) {
+					var oldDraftCount = dataSet.get('folders', [id[0], id[1], 'COUNT']);
 					Item.remove(makeIDSFromID(id), true);
-					//WMItems.remove({'aid':id[0],'fid':id[1],'iid':[id[2]]},'','','',[function(bOK){ if (bOK) Item.__refreshView(id,true); }]);
+					dataSet.add('folders', [id[0], id[1], 'COUNT'], (oldDraftCount - 1).toString());
 				},[this.__id]]);
 			}
 
@@ -527,23 +624,23 @@ NewMessage.prototype.__messageSaved = function(bOK, sUId, sError, aNotify, bChat
 			Item.__refreshView(this.__id, true);
 		}
 
-  		if (aNotify)
-			executeCallbackFunction(aNotify, true, bFirstTime, this, sError);
+  		if (aHandler)
+			executeCallbackFunction(aHandler, bOK, bFirstTime, this, sError);
 	}
 	else
-	if (aNotify)
-		executeCallbackFunction(aNotify, false, false, sUId, sError);
+	if (aHandler)
+		executeCallbackFunction(aHandler, false, false, sUId, sError);
 };
 
-NewMessage.prototype.__messageSent = function(aOut, sUId, sError, aNotify) {
-	if (aOut && !sError && typeof this.onSentCallback == 'object' && this.onSentCallback.length)
+NewMessage.prototype.__messageSent = function(aOut, sUId, sError, aHandler) {
+	if (aOut && !sError && Is.Array(this.onSentCallback) && this.onSentCallback.length)
 		for(var i in this.onSentCallback){
-            pushParameterToCallback(this.onSentCallback[i],aOut);
+            pushParameterToCallback(this.onSentCallback[i], aOut);
 			executeCallbackFunction(this.onSentCallback[i]);
 		}
 
-	if (aNotify)
-		executeCallbackFunction(aNotify, aOut, sUId, sError);
+	if (aHandler)
+		executeCallbackFunction(aHandler, aOut, sUId, sError);
 
 	if (aOut && !sError) {
 
@@ -556,11 +653,15 @@ NewMessage.prototype.__messageSent = function(aOut, sUId, sError, aNotify) {
 			this.__id_chat[1] = Path.slash(aOut.TEAMCHAT);
 			this.__id_chat[2] = aOut.TEAMCHAT_ID;
 
-			Item.notify(this.__id_chat, sAction, 'M');
+			Item.notify(this.__id_chat, sAction, {
+				'ITEM-TYPE': 'M'
+			});
 
 			//Always ADD for Y
 			if (aOut.TEAMCHAT_LINK_ID)
-				Item.notify(this.__id_chat, 'add', 'Y');
+				Item.notify(this.__id_chat, 'add', {
+					'ITEM-TYPE': 'Y'
+				});
 		}
 
 		//Notify User
@@ -568,32 +669,10 @@ NewMessage.prototype.__messageSent = function(aOut, sUId, sError, aNotify) {
 			if(this.sDeferred) {
 				var d = new IcewarpDate(this.sDeferred);
 				gui.notifier._value({type: 'send_message_deferred', args: [aOut, d]});
-			} else {
-				gui.notifier._value({type: 'message_sent', args: [aOut]});
+			} else if (!this._suppressSuccessNotifications) {
+				gui.notifier._value({type: 'message_sent', css: this.bSMS ? 'sms ' : '', args: [aOut]});
 			}
 		}
 	}
 
-};
-/*
-NewMessage.notify = function(sAction, id, sType){
-	if (gui.socket){
-		var f = dataSet.get('folders',[id[0],id[1]]);
-		if (f){
-			var aOut = {
-				'ACTION':sAction,
-				'TYPE':'item',
-				'ITEM':WMItems.__serverID(id[2]),
-				'FOLDER':f.RELATIVE_PATH,
-				'FOLDER-TYPE':f.TYPE,
-				'EMAIL':f.OWNER,
-				'ITEM-TYPE':sType || 'M'
-			};
-
-			console.log('NewMessage.notify', aOut);
-
-			gui.socket.api._notify(aOut);
-		}
-	}
-};
-*/
+};
\ No newline at end of file
