@@ -1,7 +1,7 @@
 _me = frm_addaddress.prototype;
 function frm_addaddress(){};
 
-_me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPreselect, bDistrList, bSwitch, bNoShared, bIsModal)
+_me.__constructor = async function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPreselect, bDistrList, bSwitch, bNoShared, bIsModal, bNoResources, bNoTeamchat)
 {
 //	this._dockable(false);
 	var me = this;
@@ -10,24 +10,16 @@ _me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPr
 	this._distrList = bDistrList;
 
 	this._title('SELECT_RECIPIENTS::SELECT_EMAILS');
-	this._defaultSize(-1,-1,660,620);
-
-	this._draw('frm_addaddress', 'main',{shared:!this._phoneList});
+	this._defaultSize(660,620);
 
 	//Data for
 	this.__aData = {'__@@ADDRESSBOOK@@__':getLang('ADDRESS_BOOK::ADDRESS_BOOK')};
 
-	if (!this._phoneList && !bNoShared)
-		this.__aData['__@@SHARED@@__'] = getLang('ATTENDEES::SHARED');
-
-	this.__aData['*'] = getLang('COMPOSE::SELECT_C');
-
-	if (sPrimaryAccountCHAT && !this._phoneList)
-		this.__aData['**'] = getLang('COMPOSE::SELECT_I');
-
-	var sResourcesFolderName = dataSet.get('main', ['resources_path']);
-	if (Is.Defined(sResourcesFolderName) && WMFolders.getType({aid:sPrimaryAccount, fid:sResourcesFolderName}) == 'C') {
-		this.__aData[sResourcesFolderName] = sResourcesFolderName;
+	if (!bNoResources) {
+		var sResourcesFolderName = dataSet.get('main', ['resources_path']);
+		if (Is.Defined(sResourcesFolderName) && WMFolders.getType({aid:sPrimaryAccount, fid:sResourcesFolderName}) == 'C') {
+			this.__aData[sResourcesFolderName] = getLang('COMMON_FOLDERS::RESOURCES');
+		}
 	}
 
 	switch(bSwitch){
@@ -58,6 +50,16 @@ _me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPr
 			this._activeFolder = "__@@ADDRESSBOOK@@__";
 	}
 
+	await this._draw('frm_addaddress', 'main',{shared:!this._phoneList});
+
+	if (!this._phoneList && !bNoShared)
+		this.__aData['__@@SHARED@@__'] = getLang('ATTENDEES::SHARED');
+
+	this.__aData['*'] = getLang('COMPOSE::SELECT_C');
+
+	if (sPrimaryAccountCHAT && !this._phoneList && !bNoTeamchat)
+		this.__aData['**'] = getLang('COMPOSE::SELECT_I');
+
 	//RENDER INPUTS
 	this.__selected = sPreselect || '';
 
@@ -65,11 +67,18 @@ _me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPr
 	for (var i in aTabsNames)
 		obj.input.push({name:i,label:aTabsNames[i]});
 
-	this._draw('frm_addaddress_line','right',obj);
+	await this._draw('frm_addaddress_line','right',obj);
 	this.x_btn_ok._tabIndex();
 	this.x_btn_cancel._tabIndex();
 
-	for (var i in aTabsNames){
+	await this._create('search', 'obj_search', 'search', '', {
+		aid: sPrimaryAccount,
+		fid: this._activeFolder
+	}, {
+		preferredPosition: 'top'
+	});
+
+	for (i in aTabsNames){
 
 		if (!this.__selected.toString()) this.__selected = i;
 
@@ -133,7 +142,7 @@ _me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPr
 		return true;
 	};
 
-	var check_inp = function(sName){
+	var check_inp = function(){
 		var b = true;
 		for (var i in aTabsNames)
 			if (me['input_'+i]._value()){
@@ -183,29 +192,30 @@ _me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPr
 			}
 		}
 	};
-	this.grid._prepareBody = function (aItems, sFolType){
+	this.grid._prepareBody = function (aItems) {
 		var aResult = {},
 			sPrefix, sName, sFullName, sCompany, aRow, sEmail;
 
-/*
-		if (me._SQLsearch)
-			var rxp  = new RegExp('('+me._SQLsearch+')', "gi");
-*/
 		for(var sItId in aItems)
 		{
-			if (aItems[sItId]['FLAGS'] == 4){
+			if (aItems[sItId].skeleton) {
+				aResult[sItId] = {
+					id: sItId,
+					css: 'skeleton',
+					data: {
+						LABEL: ['sName', 'sName'],
+						EMAIL: 'LCTEMAIL1',
+						COMPANY: 'ITMCOMPANY',
+						DEPARTMENT: 'ITMDEPARTMENT'
+					}
+				};
+			} else if (aItems[sItId]['FLAGS'] == 4){
 
 				if (me._activeFolder == '__@@SHARED@@__' && aItems[sItId].LCTEMAIL1 == 'anyone' && !aItems[sItId].ITMTITLE)
 					aItems[sItId].ITMTITLE = getLang('SHARING::ANONYMOUS');
-/*
-				//highlight Name
-				if (me._SQLsearch)
-					sName = aItems[sItId].ITMTITLE.replace(rxp,"<b>$1</b>");
-				else
-*/
 					sName = aItems[sItId].ITMTITLE;
 
-				aResult[sItId] = {"id":sItId,"data":{'LABEL':[sName,sName/*aItems[sItId].LCTEMAIL1*/],'EMAIL':aItems[sItId].LCTEMAIL1,'COMPANY':aItems[sItId].ITMCOMPANY,'DEPARTMENT':aItems[sItId].ITMDEPARTMENT},"arg":{'aid':aItems[sItId]['aid'],'fid':aItems[sItId]['fid'],'iid':sItId}};
+				aResult[sItId] = {"id":sItId,"data":{'LABEL':[sName,sName/*aItems[sItId].LCTEMAIL1*/],'EMAIL':aItems[sItId].LCTEMAIL1,'COMPANY':aItems[sItId].ITMCOMPANY,'DEPARTMENT':aItems[sItId].ITMDEPARTMENT},"arg":{'aid':aItems[sItId]['aid'],'fid':aItems[sItId]['fid'],'iid':sItId}, aData: aItems[sItId]};
 			}
 			else
 			if (aItems[sItId]['ITMCLASS'] == 'L'){
@@ -220,7 +230,14 @@ _me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPr
 					if (aItems[sItId].fid != Mapping.getDefaultFolderForGWType('C') && aItems[sItId].fid != "__@@ADDRESSBOOK@@__")
 						sPrefix = aItems[sItId].fid + '::';
 
-					aResult[sItId] = {"id":sItId,"data":{'LABEL':'['+sPrefix+sName+']','EMAIL':'','COMPANY':aItems[sItId].ITMCOMPANY,'DEPARTMENT':aItems[sItId].ITMDEPARTMENT},"arg":{'aid':aItems[sItId]['aid'],'fid':aItems[sItId]['fid'],'iid':sItId}};
+					aResult[sItId] = {"id":sItId,"data":{'LABEL':'['+sPrefix+sName+']','EMAIL':'','COMPANY':aItems[sItId].ITMCOMPANY,'DEPARTMENT':aItems[sItId].ITMDEPARTMENT},"arg":{'aid':aItems[sItId]['aid'],'fid':aItems[sItId]['fid'],'iid':sItId}, aData: aItems[sItId]};
+				} else {
+					this.__totalFilteredIds = this.__totalFilteredIds || [];
+					this.__totalFilteredIds.push(sItId);
+					this.__totalFilteredIds = this.__totalFilteredIds.filter(function(v, i, a) {
+						return a.indexOf(v) === i;
+					});
+					this.__totalFilteredRows = this.__total - this.__totalFilteredIds.length;
 				}
 			}
 			else{
@@ -238,39 +255,24 @@ _me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPr
 				else
 					sFullName = aItems[sItId]['ITMCLASSIFYAS'];
 
-				//highlight Name
-/*
-				if (me._SQLsearch)
-					sName = sFullName.replace(rxp,"<b>$1</b>");
-				else
-*/
 					sName = sFullName;
 
 				aRow = {};
 
 				sCompany = aItems[sItId].ITMCOMPANY;
-/*
-				if ((sCompany = aItems[sItId].ITMCOMPANY) && me._SQLsearch)
-                	sCompany = sCompany.replace(rxp,"<b>$1</b>");
-*/
 				if (me._phoneList){
 					if (Is.Defined(aItems[sItId]['LCTPHNMOBILE'])){
 						aRow = {'LABEL':[sName,sFullName],'PHONE':[aItems[sItId]['LCTPHNMOBILE']],'COMPANY':sCompany,'DEPARTMENT':aItems[sItId].ITMDEPARTMENT};
-						aResult[sItId] = {"id":sItId,"data":aRow,"arg":{'aid':aItems[sItId]['aid'],'fid':aItems[sItId]['fid'],'iid':sItId,'phone':aItems[sItId]['LCTPHNMOBILE']}};
+						aResult[sItId] = {"id":sItId,"data":aRow,"arg":{'aid':aItems[sItId]['aid'],'fid':aItems[sItId]['fid'],'iid':sItId,'phone':aItems[sItId]['LCTPHNMOBILE']}, aData: aItems[sItId]};
 					}
 				}
 				else{
 					for(var i = 1;i<4;i++)
 						if (Is.Defined(aItems[sItId]['LCTEMAIL'+i])){
-/*
-							if (me._SQLsearch)
-								sEmail = aItems[sItId]['LCTEMAIL'+i].replace(rxp,"<b>$1</b>");
-							else
-*/
 								sEmail = aItems[sItId]['LCTEMAIL'+i];
 
 							aRow = {'LABEL':[sName,sFullName],'EMAIL':[sEmail,aItems[sItId]['LCTEMAIL'+i]],'COMPANY':sCompany,'DEPARTMENT':aItems[sItId].ITMDEPARTMENT};
-							aResult[sItId+'#'+i] = {"id":sItId+'#'+i,"data":aRow,"arg":{'aid':aItems[sItId]['aid'],'fid':aItems[sItId]['fid'],'iid':sItId}};
+							aResult[sItId+'#'+i] = {"id":sItId+'#'+i,"data":aRow,"arg":{'aid':aItems[sItId]['aid'],'fid':aItems[sItId]['fid'],'iid':sItId}, aData: aItems[sItId]};
 						}
 				}
 			}
@@ -298,16 +300,16 @@ _me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPr
 	};
 	this.grid._onkeypress = function (e){
 		var s = String.fromCharCode(e.charCode);
-		if (s && (s = s.trim()) && !/[\x00-\x1F]/.test(s)){
+		if (s && (s = s.trim()) && !(/[\x00-\x1F]/).test(s)){
 			me.search._value(s);
 			me.search._focus(true);
 		}
 	};
 
-	this.grid._oncontext = function(e,elm,arg) {
+	this.grid._oncontext = async function(e,elm,arg) {
 		if (arg && arg.fid != '__@@SHARED@@__' && arg.iid) {
 				var id = [arg['aid'], arg['fid'], arg['iid']];
-				var menu = gui._create('context', 'obj_context');
+				var menu = await me._gui._create('context', 'obj_context');
 
 				menu._place(e.clientX,e.clientY);
 				menu._fill([{"title": 'POPUP_ITEMS::EDIT', 'arg': [Item.openwindow,[id, null, null, dataSet.get(this._listener_data,[arg['aid'],arg['fid'],arg['iid'],'ITMCLASS']) == 'L'?'L':'C']]}]);
@@ -331,26 +333,26 @@ _me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPr
 	}
 
     this.select._value(this._activeFolder);
-    this.select._onbeforechange = function(sOld, sNew){
+    this.select._onbeforechange = async function(sOld, sNew){
 		var sFolder;
 		if (sNew == '*'){
 
 			if (me.frm_select_folder && !me.frm_select_folder._destructed)
-				me.frm_select_folder._destruct();
+				await me.frm_select_folder._destruct();
 
 			if (me._activeFolder != '__@@SHARED@@__' && WMFolders.getType([sPrimaryAccount, me._activeFolder]) == 'C')
 				sFolder = me._activeFolder;
 			else
 				sFolder = Mapping.getDefaultFolderForGWType('C');
 
-			me.frm_select_folder = gui._create('frm_select_folder','frm_select_folder','','','POPUP_FOLDERS::SELECT_FOLDER',sPrimaryAccount,sFolder,[me,'__changeFolder'],true,true,'C','',true);
+			me.frm_select_folder = await me._gui._create('frm_select_folder','frm_select_folder','','','POPUP_FOLDERS::SELECT_FOLDER',sPrimaryAccount,sFolder,[me,'__changeFolder'],true,true,'C','',true);
 			return false;
 		}
 		else
 		if (sNew == '**'){
 
 			if (me.frm_select_folder && !me.frm_select_folder._destructed)
-				me.frm_select_folder._destruct();
+			await me.frm_select_folder._destruct();
 
 			if (WMFolders.getType([sPrimaryAccount, me._activeFolder]) == 'I')
 				sFolder = me._activeFolder;
@@ -364,7 +366,7 @@ _me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPr
 				}
 			}
 
-			me.frm_select_folder = gui._create('frm_select_folder','frm_select_folder','','','CHAT::SELECT',sPrimaryAccount,sFolder,[me,'__addRoom'],true,true,['Y','I'],'',true);
+			me.frm_select_folder = await me._gui._create('frm_select_folder','frm_select_folder','','','CHAT::SELECT',sPrimaryAccount,sFolder,[me,'__addRoom'],true,true,['Y','I'],'',true);
 			return false;
 		}
 	};
@@ -383,7 +385,7 @@ _me.__constructor = function(aOnPopupCloseResponse, aTabsNames, aTabsValues, sPr
 			me.grid._SQLfulltext = s;
 
 			// Send search query to server
-			me.grid._serverSort('','','',[me,'_selectFirst']);
+			me.grid._serverSort('','','',[me,'_selectFirst'], false, true);
 		}
 	};
 /*
@@ -414,7 +416,7 @@ _me._selectFirst = function () {
 				var str = obj._value();
 				var v = this.search._value();
 				v = v.indexOf(':') == -1 ? v : v.substr(v.indexOf(':') + 1);
-				if (obj && v && v.length > 2 && v.indexOf('@') && v.indexOf('@') != -1 && v.indexOf('@') != v.length - 1 && /^\S+$/.test(v) && str.indexOf(v) == -1) {
+				if (obj && v && v.length > 2 && v.indexOf('@') && v.indexOf('@') != -1 && v.indexOf('@') != v.length - 1 && (/^\S+$/).test(v) && str.indexOf(v) == -1) {
 					obj._value(str ? str + ', ' + v : v);
 					this.search._select();
 				}
@@ -429,7 +431,7 @@ _me.__add = function(){
 	if (!(obj = this['input_'+this.__selected]))
 		return;
 
-	var v = this.grid._value();
+	var str, v = this.grid._value();
 	if (v.length){
 		var tmp,a = [], s = obj._value() || '',sData;
 		if (s)
@@ -462,15 +464,15 @@ _me.__add = function(){
 		}
 
 		if (a.length){
-			var str = a.join(', ');
+			str = a.join(', ');
 			obj._value(str);
 			obj._setRange(str.length);
 		}
 
 	} else if(this.grid._SQLsearch && typeof this.grid._SQLsearch == "string") {
 		// If no contact is found and selected, add typed in search instead
-		var str = this.grid._SQLsearch;
-		var v = obj._value();
+		str = this.grid._SQLsearch;
+		v = obj._value();
 		str = str.indexOf(':')==-1 ? str : str.substr(str.indexOf(':')+1);
 		obj._value(v?v+', '+str:str);
 		if(str.indexOf('@')!=-1)
@@ -483,20 +485,20 @@ _me.__add = function(){
 _me.__defaultFilter = function(){
 	var filter = '';
 	if (this._activeFolder == '__@@SHARED@@__'){
-		filter = "has:email1";
+		filter = '+has:"email1"';
 
 		if (+GWOthers.getItem('RESTRICTIONS','disable_anyone'))
-			filter += ' AND -is:anyone';
+			filter += ' -is:anyone';
 
 		if (this._sharedList == 3 || +GWOthers.getItem('RESTRICTIONS', 'disable_group_sharing'))
-			filter += ' AND flags:0';
+			filter += ' +flags:0';
     }
 	else
 	if (WMFolders.getType([sPrimaryAccount, this._activeFolder]) == 'C'){
 		if (this._phoneList)
-	        filter = "has:mobile";
+	        filter = '+has:"mobile"';
 		else
-			filter = "has:email";
+			filter = '+has:"email"';
 	}
 
 	return filter;
@@ -510,6 +512,7 @@ _me.__changeFolder = function(sAccount, sFolder) {
 	this._activeFolder = sFolder;
 	this.grid._defaultfilter = this.__defaultFilter();
 
+	this.search._disabled(sFolder === '__@@SHARED@@__');
 	this.search._setFolder({aid:sAccount || sPrimaryAccount,fid:sFolder}, 'C'); //WMFolders.getType([sPrimaryAccount, this._activeFolder])
 	this.search._focus();
 
@@ -529,7 +532,12 @@ _me.__changeFolder = function(sAccount, sFolder) {
 		this.select._fill(aData);
 	}
 
-	this.grid._serverSort({aid:sAccount || sPrimaryAccount,fid:this._activeFolder},this._activeFolder == '__@@SHARED@@__'?'EMAIL':'LABEL','',[this,'_selectFirst'], { groupbyemail: GWOthers.getItem('MAIL_SETTINGS_GENERAL', 'GROUP_CONTACTS_BY_EMAIL') == '1' });
+	this.grid._serverSort({
+		aid: sAccount || sPrimaryAccount,
+		fid: this._activeFolder
+	}, this._activeFolder == '__@@SHARED@@__' ? 'EMAIL' : 'LABEL', '', [this, '_selectFirst'], {
+		groupbyemail: GWOthers.getItem('MAIL_SETTINGS_GENERAL', 'GROUP_CONTACTS_BY_EMAIL') == '1'
+	});
 };
 
 _me.__addRoom = function(sAccount, sFolder){
