@@ -1,10 +1,10 @@
 function frm_addroom_main(){};
 
 frm_addroom_main.prototype = {
-    __constructor:function(view){
+    __constructor: async function(view){
         view.data = {};
         view.title('CHAT::ADD_ROOM_TITLE');
-		view.buttons({'btn_cancel':{value:'FORM_BUTTONS::CANCEL'}});
+		await view.buttons({'btn_cancel':{value:'FORM_BUTTONS::CANCEL'}});
 
 		this._getAnchor('private').onclick = function(){
 			view.view('private');
