@@ -1,11 +1,11 @@
 function frm_addroom_public(){};
 
 frm_addroom_public.prototype = {
-    __constructor:function(view){
+    __constructor: async function(view){
 
 		view.title('CHAT::ADD_ROOM_PUBLIC_CAPTION');
 
-		view.buttons({
+		await view.buttons({
             'btn_next':{value:'FORM_BUTTONS::CREATE', css:'color1', disabled: true, onclick:function(){
 				view.parent._final({
 					type:'public',
@@ -16,29 +16,23 @@ frm_addroom_public.prototype = {
             'btn_cancel':{value:'FORM_BUTTONS::CANCEL'}
 		});
 
-		var groups = view.parent._getGroups(),
-			select = 0;
-
-		// preselect group
-		if (view.data.fid){
-			for (i = groups.length;i--;){
-				if (groups[i][1] === view.data.fid){
-					select = i;
-					break;
-				}
-			}
-		}
+		var groups = view.parent._getGroups();
 
 		this.group._fill(groups);
-		this.group._value(select);
+		this.group._onchange = function() {
+			view.parent.btn_next._disabled(!this.group._getDataValue() || !this.name._value());
+		}.bind(this);
+		if (groups.length === 1) {
+			this.group._value(0);
+		}
 
-		this.name._restrict('![/\\\\:\?\"\<\>\|\~]+','','^.{1,255}$');
+		this.name._restrict('![/\\\\:?"<>|~]+','','^.{1,255}$');
 		this.name.__eIN.setAttribute('maxlength', 255);
 		this.name._onerror = function(b){
-			view.parent.btn_next._disabled(b);
-		};
-		this.name._onsubmit = function(){
-			if (this.__check())
+			view.parent.btn_next._disabled(b || !this.group._getDataValue());
+		}.bind(this);
+		this.name._onsubmit = async function(){
+			if (await this.__check())
 				view.parent.btn_next._onclick();
 		};
 	}
