@@ -9,7 +9,7 @@ function frm_alert_suppress(){};
  * Martin Ekblom 2013
  */
 
-_me.__constructor = function(aResponse, sLabel1, sLabel2, aSubstitution) {
+_me.__constructor = async function(aResponse, sLabel1, sLabel2, aSubstitution) {
 	var aPath;
 
 	// Content of Suppress Alert must be localazition string (not translated)
@@ -21,11 +21,11 @@ _me.__constructor = function(aResponse, sLabel1, sLabel2, aSubstitution) {
 		return false;
 
 	// Create checkbox for remembering suppress state
-	this._create('conceal','obj_checkbox','message','','ALERTS::SUPPRESS');
-	this.conceal._onclick = function(e) {
+	await this._create('conceal','obj_checkbox','message','','ALERTS::SUPPRESS');
+	this.conceal._onclick = function() {
 		Cookie.set(['suppressed',aPath[0],aPath[1]],this._value()?0:1);
 	};
 
 	// Resize to fit message
-	this._size(400,this._getAnchor('message').offsetHeight + 120,true);
+	this._size(400,'auto',true);
 };
\ No newline at end of file
