@@ -1,12 +1,11 @@
 _me = frm_blackwhite.prototype;
 function frm_blackwhite(){};
 
-_me.__constructor = function(sAcc,sFol) {
+_me.__constructor = async function(sAcc,sFol) {
 	var bWhite = true;
 	if (sFol.indexOf('Blacklist') > -1)
 		bWhite = false;
-	this._size(350,150,true);
-	this._draw('frm_blackwhite', 'main');
+	await this._draw('frm_blackwhite', 'main');
 	this._title('BLACKWHITE::' + (bWhite?'WHITE':'BLACK'));
 	this._acc = sAcc;
 	this._fol = sFol;
@@ -17,9 +16,9 @@ _me.__constructor = function(sAcc,sFol) {
 	this.EMAIL._onerror = function(b){
 		me.x_btn_ok._disabled(b);
 	};
-	this.EMAIL._restrict('.+');
+	this.EMAIL._restrict(/(^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$)|(^((?!-))(xn--)?[a-z0-9][a-z0-9\-_]{0,61}[a-z0-9]{0,1}\.(xn--)?([a-z0-9-]{1,61}|[a-z0-9-]{1,30}\.[a-z]{2,})$)/);
 	this.EMAIL._onsubmit = function(){
-		me.x_btn_ok._onclick();
+		!me.x_btn_ok._disabled() && me.x_btn_ok._onclick();
 	};
 	this.x_btn_ok._onclick = function() {
 
@@ -33,6 +32,8 @@ _me.__constructor = function(sAcc,sFol) {
 		else
 			me._destruct();
 	};
+
+	this._size(350,'auto',true);
 };
 
 _me._listFolder = function (){
@@ -44,7 +45,7 @@ _me._listFolder = function (){
 
 		var aSort = Cookie.get(['views',this._acc,this._fol,'sort']);
 
-		gui.frm_main.main.list._serverSort({aid:this._acc,fid:this._fol},aSort['column'],aSort['type']);
+		gui.frm_main.main.list._serverSort({aid:this._acc,fid:this._fol},aSort['column'],aSort['type'],void 0, void 0, true, true);
 	}
 	this._destruct();
 };
\ No newline at end of file
