@@ -7,11 +7,11 @@ function frm_confirm_select(){};
  *	Confirm popup with select dropdown
  */
 
-_me.__constructor = function(aResponse, sLabel1, sLabel2, aSubstitution, aFill, sDefault, bCompulsory) {
+_me.__constructor = async function(aResponse, sLabel1, sLabel2, aSubstitution, aFill, sDefault, bCompulsory) {
 	var me = this;
 
 	// Create and fill dropdown select
-	this._create('select','obj_select','message');
+	await this._create('select','obj_select','message');
 	this.select._fill(aFill);
 	if(sDefault)
 		this.select._value(sDefault);
@@ -20,7 +20,7 @@ _me.__constructor = function(aResponse, sLabel1, sLabel2, aSubstitution, aFill,
 	}
 
 	// Adjust size (height + header&footer + padding)
-	this._size(400,this._getAnchor('message').offsetHeight+82+30,true);
+	this._size(500,'auto',true);
 
 	// Disable Ok button until chosen (if applicable)
 	if(bCompulsory)
