@@ -9,23 +9,23 @@ function frm_confirm_suppress(){};
  * Martin Ekblom 2013
  */
 
-_me.__constructor = function(aResponse, sLabel1, sLabel2, aSubstitution) {
+_me.__constructor = async function(aResponse, sLabel1, sLabel2) {
 	var aPath;
 
 	// Content of Suppress Alert must be localazition string (not translated)
 	if(sLabel2 && sLabel2.indexOf('::')) {
-		this.obj_label._value(getLang(sLabel2, aSubstitution));
+		// this.obj_label._value(getLang(sLabel2, aSubstitution));
 		aPath = sLabel2.toLowerCase().split('::');
 	}
 	else
 		return false;
 
 	// Create checkbox for remembering suppress state, do not ask again
-	this._create('conceal','obj_checkbox','message','','CONFIRMATION::SUPPRESS');
-	this.conceal._onclick = function(e) {
+	await this._create('conceal','obj_checkbox','message','','CONFIRMATION::SUPPRESS');
+	this.conceal._onclick = function() {
 		Cookie.set(['suppressed',aPath[0],aPath[1]],this._value()?0:1);
 	};
 
 	// Resize to fit message
-	this._size(400,this._getAnchor('message').offsetHeight + 120,true);
+	this._size(400,'auto',true);
 };
\ No newline at end of file
