@@ -1,7 +1,7 @@
 _me = frm_confirm_threestates.prototype;
 function frm_confirm_threestates(){};
 
-_me.__constructor = function(aResponse, sLabel1, sLabel2, aSubstitution, sOKLabel, sCancelLabel, sCancel2Label) {
+_me.__constructor = async function(aResponse, sLabel1, sLabel2, aSubstitution, sOKLabel, sCancelLabel, sCancel2Label) {
 	var me = this;
 
 	// aResponse, sLabel1, sLabel2 and aSubstitution is handled in parent
@@ -21,7 +21,7 @@ _me.__constructor = function(aResponse, sLabel1, sLabel2, aSubstitution, sOKLabe
 	};
 
 	// Create 'CANCEL2' button
-	this._create('x_btn_cancel2', 'obj_button', 'footer','noborder simple cancel');
+	await this._create('x_btn_cancel2', 'obj_button', 'footer','noborder simple cancel');
 	this.x_btn_cancel2._tabIndex();
 	this.x_btn_cancel2._value(sCancel2Label || 'FORM_BUTTONS::CANCEL');
 	this.x_btn_cancel2._onclick = function() { me._destruct() };
