@@ -1,35 +1,28 @@
 _me = frm_contact.prototype;
 function frm_contact(){};
 
-_me.__constructor = function() {
-	this._defaultSize(-1,-1,650,620);
-	this._draw(
-		'frm_contact',
-		'main',
-		{
-			sort_as : GWOthers.getItem('RESTRICTIONS','sortstring') == '1',
-			disable_html : !GWOthers.getItemAccess('MAIL_SETTINGS_DEFAULT','html_message') && '0' === GWOthers.getItem('MAIL_SETTINGS_DEFAULT','html_message')
-		}
-	);
+_me.__constructor = async function() {
+	this._defaultSize(650, 620, {
+		goldenRatio: true
+	});
+	await this._draw('frm_contact', 'main', {
+		sort_as : GWOthers.getItem('RESTRICTIONS','sortstring') == '1',
+		disable_html : !GWOthers.getItemAccess('MAIL_SETTINGS_DEFAULT','html_message') && '0' === GWOthers.getItem('MAIL_SETTINGS_DEFAULT','html_message')
+	});
 
 	if (this._sFolderID == '@@mycard@@'){
-		this.__initForm('MAIN_MENU::VCARD');
-		//addcss(this._main,'vcard');
+		await this.__initForm('MAIN_MENU::VCARD');
+
 		if (this.x_btn_delete)
 			this.x_btn_delete._destruct();
 	}
 	else
-		this.__initForm('CONTACT::CONTACT');
-
-	// Refresh list and preview in case tags were changed
-	this.maintab.tab1.ITMCATEGORY.input._onChange = function(){
-		this.__refreshView = true;
-	}.bind(this);
+		await this.__initForm('CONTACT::CONTACT');
 };
 
-_me.__print = function(aValues){
+_me.__print = async function(aValues){
 
-	var atmp,
+	var atmp, j,
 		ainf = ['LCTSTREET','LCTCITY','LCTSTATE','LCTCOUNTRY','LCTZIP','LCTWEBPAGE'];
 	if (aValues.LOCATIONS)
 		for (var i in aValues.LOCATIONS){
@@ -37,13 +30,13 @@ _me.__print = function(aValues){
 				aValues.values = objConcat (aValues.values,aValues.LOCATIONS[i].values);
 
 				aValues.values.PHONES = [];
-				for (var j in aValues.LOCATIONS[i].values)
+				for (j in aValues.LOCATIONS[i].values)
 				    if (j.indexOf('LCTPHN') == 0 && aValues.LOCATIONS[i].values[j])
 						aValues.values.PHONES.push({PHNTYPE:getLang('PHONE::'+j),PHNNUMBER: aValues.LOCATIONS[i].values[j]});
 			}
 
 			atmp = {};
-			for (var j in ainf)
+			for (j in ainf)
 				if (aValues.LOCATIONS[i].values[ainf[j]])
 					atmp[ainf[j]] = aValues.LOCATIONS[i].values[ainf[j]];
 
@@ -54,66 +47,25 @@ _me.__print = function(aValues){
 	aValues = aValues.values;
 
 	if (!gui.print)
-		gui._create('print','frm_print');
+		await gui._create('print','frm_print');
 	gui.print._add('C', aValues);
 };
 
 _me.__showAvatar = function(sType,sPath) {
 	var me = this,
-		sURL = './server/download.php?' + buildURL({'sid':dataSet.get('main',['sid']),'class':sType,'fullpath':sPath,'no':unique_id()}),
+		sURL = sPrimaryAccountClient + 'server/download.php?' + buildURL({'dlsess':dataSet.get('main',['dlsess']),'class':sType,'fullpath':sPath,'no':unique_id()}),
 		img = new Image();
 
 	img.onload = function(){
 		var elm = document.getElementById(me.maintab.tab1._pathName+'#avatar_image');
 		if (elm){
 
-			if (currentBrowser() == 'MSIE7'){
-				addcss(elm,'msie');
-
-				//Remove old
-				var span = elm.getElementsByTagName('SPAN');
-				if (span && span[0])
-					span[0].parentNode.removeChild(span[0]);
-			}
-
 			if (this.height>10 && this.width>10){
-				if (currentBrowser() == 'MSIE7'){
-
-					//Create New One
-					var img = mkElement('img'),
-						span = mkElement('span');
-						span.style.width = elm.clientWidth + 'px';
-						span.style.height = elm.clientHeight + 'px';
-						span.appendChild(img);
-
-					if (this.width>this.height){
-						var r = this.height/elm.clientHeight;
-						img.style.height = elm.clientHeight + 'px';
-
-						if ((this.width/r)>elm.clientWidth)
-							img.style.right = (((this.width/r)-elm.clientWidth)/2) + 'px';
-					}
-					else{
-						var r = this.width/elm.clientWidth;
-						img.style.width = elm.clientWidth + 'px';
-
-						if ((this.height/r)>elm.clientHeight)
-							this.style.bottom = (((this.height/r)-elm.clientHeight)/2) + 'px';
-					}
-
-					img.src = sURL;
-
-					elm.style.backgroundImage = 'none';
-					elm.insertBefore(span,elm.firstChild);
-				}
-				else
-					elm.style.backgroundImage = 'url("'+ sURL +'")';
-
+				elm.style.backgroundImage = 'url("'+ sURL +'")';
 				elm.style.backgroundColor = '#FFFFFF';
 
 				// When avatar is displayed allow remove
 				addcss(elm,'removable');
-				me.maintab.tab1.X_AVATAR._main.style.display = 'none';
 				elm.title = getLang('POPUP_ITEMS::DELETE');
 
 			}
@@ -124,7 +76,6 @@ _me.__showAvatar = function(sType,sPath) {
 
 				// No avatar, do not show remove
 				removecss(elm,'removable');
-				me.maintab.tab1.X_AVATAR._main.style.display = 'block';
 				elm.title = "";
 			}
 		}
@@ -135,17 +86,15 @@ _me.__showAvatar = function(sType,sPath) {
 
 _me.__loadItems = function() {
 	if (!this.maintab) return;
-	var me = this;
-	//var bDisabledHtml = !GWOthers.getItemAccess('MAIL_SETTINGS_DEFAULT','html_message') && '0' === GWOthers.getItem('MAIL_SETTINGS_DEFAULT','html_message');
-
-	var oTabs = this.maintab;
+	var me = this, i, t,
+		oTabs = this.maintab;
 
 	// parsing Locations
 	if (Is.Defined(this._aValues['LOCATIONS'])) {
 		var aLocation;
 		var aLocations = this._aValues['LOCATIONS'];
 
-		for (var i in aLocations) {
+		for (i in aLocations) {
 			aLocation = aLocations[i];
 
 			switch (aLocation['values']['LCTTYPE']) {
@@ -161,6 +110,8 @@ _me.__loadItems = function() {
 	// TAB1
 	loadDataIntoForm(oTabs.tab1,this._aValues);
 
+	this.maintab.tab1.ITMCATEGORY._setId([this._sAccountID, this._sFolderID, this._sItemID]);
+
 	//FullName
 	oTabs.tab1.X_NAME._value(createNameFromLocation(this._aValues));
 	oTabs.tab1.X_NAME._onblur = function(){
@@ -182,19 +133,17 @@ _me.__loadItems = function() {
 			    sTitle = sTitle.substr(0,32)+'...';
 
             if (me._sFolderID != '@@mycard@@')
-				me._title(sTitle + (me._aValues.ITMFOLDER?' - '+me._aValues.ITMFOLDER:''),true);
+				me._title(sTitle + (me._aValues.ITMFOLDER?' - '+ WMFolders.getFolderName({aid: sPrimaryAccount, fid: me._aValues.ITMFOLDER}):''),true);
 		}
 		else
 		if (me._sFolderID != '@@mycard@@')
-			me._title(getLang('CONTACT::CONTACT') + (me._aValues.ITMFOLDER?' - '+me._aValues.ITMFOLDER:''),true);
+			me._title(getLang('CONTACT::CONTACT') + (me._aValues.ITMFOLDER?' - '+WMFolders.getFolderName({aid: sPrimaryAccount, fid: me._aValues.ITMFOLDER}):''),true);
 	};
-	this.__genClassifyAs(oTabs.tab1.ITMCLASSIFYAS._value()?true:false);
+	this.__genClassifyAs(!oTabs.tab1.ITMCLASSIFYAS._value());
 	oTabs.tab1.ITMCLASSIFYAS._onblur();
 
 	//Avatar
 	var av = document.getElementById(this.maintab.tab1._pathName+'#avatar_image');
-	if (currentBrowser() == 'MSIE7')
-		addcss(av,'msie');
 
 	if (!this.__readonly){
 
@@ -207,9 +156,9 @@ _me.__loadItems = function() {
 		oTabs.tab1.X_AVATAR._setPostParam('width',800);
 		oTabs.tab1.X_AVATAR._setPostParam('height',800);
 
-		oTabs.tab1.X_AVATAR._onuploadstart = function(){
+		oTabs.tab1.X_AVATAR._onuploadstart = async function(){
 			me.x_btn_ok._disabled(true);
-			me._create('image_processing', 'obj_loader');
+			await me._create('image_processing', 'obj_loader');
 			me.image_processing._value(getLang('COMMON::PROCESSING'));
 		};
 		oTabs.tab1.X_AVATAR._onuploadend = function(arg){
@@ -223,45 +172,15 @@ _me.__loadItems = function() {
 			}
 		};
 
-		av.onmousemove = function(e){
-			var e = e || window.event,
-				elm = me.maintab.tab1.X_AVATAR._main,
-				pos = getSize(this);
-				x = e.clientX - pos.x - 7,
-				y = e.clientY - pos.y - 7;
-
-			if (x<0)
-				x = 0;
-			else
-			if (x>pos.w)
-				x = pos.w;
-
-			if (y<0)
-				y = 0;
-			else
-			if (y>pos.h)
-				y = pos.h;
-
-
-
-			elm.style.left = x + 'px';
-			elm.style.top = y + 'px';
-		};
-
 		// Clicking on image to remove if uploaded
-		av.onclick = function(e) {
+		av.onclick = async function() {
 			var elm = document.getElementById(me.maintab.tab1._pathName+'#avatar_image');
 			if (hascss(elm,'removable')){
-				var conf = gui._create('delete_avatar_confirm','frm_confirm','','', [me,function() {
-					//Remove IE8 Avatars
-					var img = elm.getElementsByTagName('IMG');
-					for (var i = img.length-1;i>=0;i--)
-						img[i].parentNode.removeChild(img[i]);
+				var conf = await gui._create('delete_avatar_confirm','frm_confirm','','', [me,function() {
 
 					elm.style.backgroundImage = '';
 					elm.style.backgroundColor = '';
 
-					this.maintab.tab1.X_AVATAR._main.style.display = 'block';
 					removecss(elm,'removable');
 					elm.title = "";
 
@@ -273,6 +192,8 @@ _me.__loadItems = function() {
 				}],'POPUP_ITEMS::CONTACTS','CONFIRMATION::DELETE_AVATAR');
 				addcss(conf.x_btn_ok._main, 'color2');
 			}
+			else
+				oTabs.tab1.X_AVATAR._click();
 		};
 
 		oTabs.tab1.X_AVATAR._dropzone(oTabs.tab1._main);
@@ -280,14 +201,14 @@ _me.__loadItems = function() {
 
     //Avatar Image
 	if (Is.Defined(me._aValues['ATTACHMENTS']))
-		for(var i in me._aValues['ATTACHMENTS'])
+		for(i in me._aValues['ATTACHMENTS'])
 			if (me._aValues['ATTACHMENTS'][i].values.ATTTYPE == 'P'){
                 this.__showAvatar('attachment', me._sAccountID+'/'+me._sFolderID+'/'+WMItems.__serverID(me._sItemID)+'/'+i);
                 break;
 			}
 
 	//Default Phone type values
-	var loc = {'LCTPHNWORK1':'X_PHONE1', 'LCTPHNHOME1':'X_PHONE2', 'LCTPHNFAXWORK':'X_PHONE3', 'LCTPHNMOBILE':'X_PHONE4'};
+	var loc = {'LCTPHNWORK1':'X_PHONE1', 'LCTPHNHOME1':'X_PHONE2', 'LCTPHNFAXWORK':'X_PHONE3', 'LCTPHNMOBILE':'X_PHONE4'},
 		loc_used = [];
 
 	for(var name in loc){
@@ -310,7 +231,7 @@ _me.__loadItems = function() {
 			oTabs.tab1.X_IM1._value(this._sHomeLocation['values']['LCTIM']);
 
 		//Phone
-		for(var name in loc){
+		for(name in loc){
 			if (Is.Defined(this._sHomeLocation['values'][name])){
 				delete loc[name];
 				loc_used.push(name);
@@ -402,7 +323,8 @@ _me.__loadItems = function() {
 				me._close(true);
 			};
 
-			loadDataIntoForm(this,me._aValues);
+			this.ITMDESCRIPTION._value(DOMPurify.sanitize(me._aValues.ITMDESCRIPTION || ''));
+			// loadDataIntoForm(this,me._aValues);
 		}
 
 		if (me.__readonly)
@@ -416,38 +338,37 @@ _me.__loadItems = function() {
 	oTabs.tab5._onactive = function (bFirstTime) {
 		if (bFirstTime){
 
+			var out = [], i;
 			if (me._aValues['ATTACHMENTS']){
-				var out = [];
-				for (var i in me._aValues['ATTACHMENTS'])
+				for (i in me._aValues['ATTACHMENTS'])
 					out.push({
 						'name': me._aValues['ATTACHMENTS'][i]['values']['ATTDESC'],
 						'class': me._aValues['ATTACHMENTS'][i]['values']['ATTTYPE'],
 						'id': i,
 						'ticket': me._aValues['ATTACHMENTS'][i]['values']['TICKET'],
 						'fullpath': me._aValues.fullpath,
-						'size': me._aValues['ATTACHMENTS'][i]['values']['ATTSIZE']}
-						);
+						'size': me._aValues['ATTACHMENTS'][i]['values']['ATTSIZE']
+					});
 
 				if (me._aValues.fullpath)
 					this.X_ATTACHMENTS._value({'values': out});
 				else
-					this.X_ATTACHMENTS._value({'path': me._sAccountID+'/'+me._sFolderID+'/'+WMItems.__serverID(me._sItemID), 'values': out});
-			}
-			else
-			if (me._aValues['PUSH_ATTACHMENTS']){
-				var out = [];
-				for (var i in me._aValues['PUSH_ATTACHMENTS'])
-					out.push({
-						'name': me._aValues['PUSH_ATTACHMENTS'][i]['title'],
-						'id': me._aValues['PUSH_ATTACHMENTS'][i]['id'],
-						'size': me._aValues['PUSH_ATTACHMENTS'][i]['size'],
-						'class': me._aValues['PUSH_ATTACHMENTS'][i]['embedded'] ? 'item' : 'itemlink',
-						'fullpath': me._aValues['PUSH_ATTACHMENTS'][i]['fullpath']
+					this.X_ATTACHMENTS._value({'folder': me._sAccountID+'/'+me._sFolderID+'/'+WMItems.__serverID(me._sItemID), 'values': out});
+			} else if (me._aValues.PUSH_ATTACHMENTS) {
+				for (i in me._aValues.PUSH_ATTACHMENTS) {
+					var att = me._aValues.PUSH_ATTACHMENTS[i];
+					out.push(att.values || {
+						name: att.title,
+						id: att.id,
+						size: att.size,
+						'class': att.embedded ? 'item' : 'itemlink',
+						fullpath: att.fullpath
 					});
-
+				}
+				var fullpath = (me._aValues.PUSH_ATTACHMENTS[0] || {}).fullpath;
 				this.X_ATTACHMENTS._value({
-					path:me._aValues['PUSH_ATTACHMENTS'][0]['fullpath'].substr(0,me._aValues['PUSH_ATTACHMENTS'][0]['fullpath'].lastIndexOf('/')),
-					values:out
+					folder: fullpath && fullpath.substr(0, fullpath.lastIndexOf('/')),
+					values: out
 				});
 			}
 
@@ -491,7 +412,7 @@ _me.__loadItems = function() {
 					}
 				}
 
-				this.X_CERT._value({'path':me._sAccountID+'/'+me._sFolderID+'/'+me._sItemID, 'values':aCerts});
+				this.X_CERT._value({'folder':me._sAccountID+'/'+me._sFolderID+'/'+me._sItemID, 'values':aCerts});
 			}
 
 			if (me.__readonly)
@@ -504,14 +425,14 @@ _me.__loadItems = function() {
 	// Force user to fill in required fields in vCard
 	var mandatory = GWOthers.getItem('GLOBAL_SETTINGS', 'mandatory_contact_fields');
 	if (this._sFolderID == '@@mycard@@' && mandatory && GWOthers.getItem('RESTRICTIONS','mandatory_user_info') == '1' && Cookie.get(['suppressmandatory'])!=1) {
-		this.__mandatoryfields = [];
+		this.__mandatoryfields = [], i;
 
 		var checkall = function(error) {
 			// Disable saving until all fields are correct
 			me.x_btn_ok._disabled(true);
 
 			var tabs = {};
-			for(var i=me.__mandatoryfields.length; i; i--) {
+			for(i = me.__mandatoryfields.length; i; i--) {
 				// Check all fields and tabs for errors
 				var tab = me.__mandatoryfields[i-1]._parent._type=='obj_tab' ? me.__mandatoryfields[i-1]._parent : me.__mandatoryfields[i-1]._parent._parent;
 				if(hascss(me.__mandatoryfields[i-1]._main,"error")) {
@@ -519,9 +440,9 @@ _me.__loadItems = function() {
 					error = true;
 				}
 				// Mark tabs as incorrect if necessary
-				for(var t in mandatory)
+				for(t in mandatory)
 					oTabs[t]._incorrect(false);
-				for(var t in tabs)
+				for(t in tabs)
 					oTabs[t]._incorrect(true);
 				// If there are no errors, allow saving
 				if(!error)
@@ -534,7 +455,7 @@ _me.__loadItems = function() {
 		if(mandatory.indexOf('(')!=-1) {
 			var tmp = mandatory.split(';');
 			mandatory = {};
-			for(var i=tmp.length; i; i--) {
+			for(i = tmp.length; i; i--) {
 				var m = tmp[i-1].match(/^(tab[0-9]+)\((.+)\)$/i);
 				mandatory[m[1]] = m[2].toUpperCase().split(',');
 			}
@@ -544,7 +465,7 @@ _me.__loadItems = function() {
 		// Check and mark all required fields
 		for(var tab in mandatory) {
 			this.maintab[tab]._active();
-			for(var i=mandatory[tab].length; i; i--) {
+			for(i = mandatory[tab].length; i; i--) {
 				// Finding field objects from labels
 				var field = mandatory[tab][i-1];
 				if(field.indexOf('_')!=-1) {
@@ -558,12 +479,17 @@ _me.__loadItems = function() {
 					field._onerror = checkall;
 					if(field._restrict)
 						field._restrict(".+");
-					else console.warn("No restrict!",tab,field);
+					else {
+						 console.warn("No restrict!",tab,field);
+					}
 				}
 			}
 		}
 	}
 
+	oTabs.tab2._active(true);
+	oTabs.tab3._active(true);
+	oTabs.tab1._active(true);
 
 	if (me._aValues['PUSH_ATTACHMENTS'] && this.maintab.tab5)
 		this.maintab.tab5._active();
@@ -595,16 +521,21 @@ _me.__genClassifyAs = function(bNoValue){
 			tmp += (aValues.ITMFIRSTNAME?' ':'') + aValues.ITMMIDDLENAME;
 	}
 
+	if (this._aValues.ITMCLASSIFYAS)
+		aFill[this._aValues.ITMCLASSIFYAS] = this._aValues.ITMCLASSIFYAS;
+
+	if (aValues.ITMCLASSIFYAS)
+		aFill[aValues.ITMCLASSIFYAS] = aValues.ITMCLASSIFYAS;
+
 	if (tmp)
 		aFill[tmp] = tmp;
 	else
 	    tmp = '';
 
-	if (aValues.ITMCLASSIFYAS)
-		aFill[aValues.ITMCLASSIFYAS] = aValues.ITMCLASSIFYAS;
-
 	if (tab1.ITMCOMPANY._value()){
-		aFill[tmp+' ('+ tab1.ITMCOMPANY._value() +')'] = tmp + ' ('+ tab1.ITMCOMPANY._value() +')';
+		aFill[aValues.ITMCLASSIFYAS +' ('+ tab1.ITMCOMPANY._value() +')'] = aValues.ITMCLASSIFYAS + ' ('+ tab1.ITMCOMPANY._value() +')';
+		aFill[tmp +' ('+ tab1.ITMCOMPANY._value() +')'] = tmp + ' ('+ tab1.ITMCOMPANY._value() +')';
+		aFill[tab1.ITMCOMPANY._value() + ' (' + aValues.ITMCLASSIFYAS + ')'] = tab1.ITMCOMPANY._value() + ' (' + aValues.ITMCLASSIFYAS + ')';
 		aFill[tab1.ITMCOMPANY._value() + (tmp?' ('+tmp+')':'')] = tab1.ITMCOMPANY._value() + (tmp?' ('+tmp+')':'');
 	}
 
@@ -707,7 +638,7 @@ _me.__saveItems = function(aValues) {
 
 	if (!isFormEmpty(aBusiness)) {
 		if (!Is.Empty(aPhones))
-			for (var s in aPhones)
+			for (s in aPhones)
 				aBusiness['values'][s] = '';//aPhones[s];
 
 		// add location
@@ -717,7 +648,7 @@ _me.__saveItems = function(aValues) {
 
 	if (!isFormEmpty(aOther)) {
 		if (!Is.Empty(aPhones))
-			for (var s in aPhones)
+			for (s in aPhones)
 				aOther['values'][s] = '';//aPhones[s];
 
 		// add location
@@ -735,16 +666,14 @@ _me.__saveItems = function(aValues) {
 	}
 
 	//Attachments
-	var aAttachments;
+	var aAttachments, i;
 	if (tab5 && tab5.X_ATTACHMENTS && !Is.Empty(aAttachments = tab5.X_ATTACHMENTS._value()))
 		aValues['ATTACHMENTS'] = aAttachments;
 
 	//Avatar - add or remove if changed
 	if (tab1.X_AVATAR && (this.__avatarChanged || this.__avatarDeleted)) {
-
-
 		//remove existing avatar
-		for (var i in this._aValues['ATTACHMENTS'])
+		for (i in this._aValues['ATTACHMENTS'])
 			if (this._aValues['ATTACHMENTS'][i].values.ATTTYPE == 'P'){
 
 				if (!aValues['ATTACHMENTS'])
@@ -774,7 +703,7 @@ _me.__saveItems = function(aValues) {
 				aValues['CERTIFICATES'] = [];
 				var tmp;
 
-				for(var i in v)
+				for(i in v)
 		            if (v[i].uid)
 		               aValues['CERTIFICATES'].push(v[i]);
 		            else
@@ -790,7 +719,7 @@ _me.__saveItems = function(aValues) {
 		}
 		else
 		if (this._aValues['CERTIFICATES'])
-			for (var i in this._aValues['CERTIFICATES'])
+			for (i in this._aValues['CERTIFICATES'])
 				if (this._aValues['CERTIFICATES'][i].values['class'] == 'item'){
 					if (!aValues['CERTIFICATES'])
 						aValues['CERTIFICATES'] = [];
@@ -803,3 +732,25 @@ _me.__saveItems = function(aValues) {
 	if(GWOthers.getItem('RESTRICTIONS','mandatory_user_info') == '1' && this._sFolderID == '@@mycard@@')
 		Cookie.set(['suppressmandatory'],1);
 };
+
+_me.__autoreopen = function(data) {
+	if (!data.values.EVN_ID && (data.ATTACHMENTS || []).length) {
+		var i = 0;
+		data.PUSH_ATTACHMENTS = (data.ATTACHMENTS || []).map(function(att) {
+			att = att.values;
+			return {
+				values: {
+					name: att.description,
+					id: i++,
+					size: att.size,
+					class: att.class,
+					fullpath: att.fullpath
+				}
+			}
+		});
+		delete data.ATTACHMENTS;
+	}
+
+	Object.assign(this._aValues, data, data.values);
+	this.__loadItems();
+};
