@@ -1,71 +1,28 @@
 _me = frm_file.prototype;
 function frm_file(){};
 
-_me.__constructor = function() {
+_me.__constructor = async function() {
 	this._skip_teamchat_choose_folder = arguments[6];
 	var me = this;
 
-	this._defaultSize(-1,-1,740,315);
-
 	// Create 'OK' button
-	this._create('x_btn_ok', 'obj_button', 'footer', 'noborder color1');
+	await this._create('x_btn_ok', 'obj_button', 'footer', 'noborder color1');
 	this.x_btn_ok._value('FORM_BUTTONS::SAVE');
 
+	await this._create('X_ATTACHMENTS','obj_upload_item');
+
 	//Prepare Upload Object
 	if (!this._sItemID){
-		this._size(740, this._skip_teamchat_choose_folder ? 290 : 315);
 		this._resizable(false);
-		this._draw('frm_file_main', 'main', {folders:(this._aValues.fullpath)?true:false, teamchat: !!this._skip_teamchat_choose_folder});
+		await this._draw('frm_file_main', 'main', {folders:(this._aValues.fullpath)?true:false, teamchat: !!this._skip_teamchat_choose_folder});
+		this._defaultSize(740, this._skip_teamchat_choose_folder ? 'auto' : 350);
+		this._size(740, this._skip_teamchat_choose_folder ? 'auto' : 350);
+
 		this.__tab1 = this;
-
-		this._create('X_ATTACHMENTS','obj_upload_edit','attach','noborder');
-
-		//Add Blank File
-		this.X_ATTACHMENTS._create('btn_blank','obj_button','controls','simple ico blank');
-		this.X_ATTACHMENTS.btn_blank._value('FILE::BLANK_DOC');
-		this.X_ATTACHMENTS.btn_blank._disabled(this.__readonly);
-		this.X_ATTACHMENTS.btn_blank._main.onmousedown = function(e){
-			var e = e || window.event;
-			if (e.stopPropagation) e.stopPropagation();
-				e.cancelBubble = true;
-		};
-		this.X_ATTACHMENTS.btn_blank._onclick = function(e){
-
-			if (e.preventDefault) e.preventDefault();
-			if (e.stopPropagation) e.stopPropagation();
-				e.cancelBubble = true;
-
-			if (this.cmenu && !this.cmenu._destructed){
-				this.cmenu._destruct();
-				delete this.cmenu;
-				return;
-			}
-
-			var pos = getSize(this._main),
-				aMenu = [
-					{"title": 'MAIN_MENU::NEW_WORD', 'arg':'.docx'},
-					{"title": 'MAIN_MENU::NEW_EXCEL', 'arg': '.xlsx'},
-					{"title": 'MAIN_MENU::NEW_PPOINT', 'arg': '.pptx'},
-					{"title": 'MAIN_MENU::NEW_TEXT', 'arg': '.txt'},
-					{"title": 'MAIN_MENU::NEW_HTML', 'arg': '.html'}
-				];
-
-			this.cmenu = gui._create("cmenu","obj_context",'','',this),
-			this.cmenu._fill(aMenu);
-			this.cmenu._place(pos.x+(pos.w/2),pos.y+pos.h,'',1);
-			this.cmenu._onclick = function(e, elm, id, arg) {
-				if (arg){
-					me.X_ATTACHMENTS._value({'values':[{"name":arg,"class":'document'}]});
-					me.X_ATTACHMENTS._rename(me.X_ATTACHMENTS.list.__idtable.length-1);
-				}
-			};
-		};
-
-		//this.EVNNOTE._focus();
 	}
 	else{
-		this._size(false, 585);
-		this._draw(
+		this._defaultSize(740, 585);
+		await this._draw(
 			'frm_file',
 			'main',
 			{
@@ -74,27 +31,12 @@ _me.__constructor = function() {
 				disable_html : !GWOthers.getItemAccess('MAIL_SETTINGS_DEFAULT','html_message') && '0' === GWOthers.getItem('MAIL_SETTINGS_DEFAULT','html_message')
 			}
 		);
-		this.__tab1 = this.maintab.tab1;
-
-		this._create('X_ATTACHMENTS','obj_upload_edit_single','attach','big noborder');
-		this.X_ATTACHMENTS.list.__hasRemove = false;
+		this.X_ATTACHMENTS.file._setMultipleSelection(false);
 
 		if (this._aValues.EVNLOCKOWN_ID && this._aValues.EVNLOCKOWN_ID != sPrimaryAccountGWID)
 			this.__readonly = true;
 
-		// Create 'New Revision' button
-		/*
-		this._create('x_btn_revision', 'obj_button', 'footer','ico ok noborder simple');
-		this.x_btn_revision._value('FORM_BUTTONS::REVISION');
-		this.x_btn_revision._onclick = function() {
-			if (!me.x_btn_ok._disabled()){
-				me.__revision = true;
-				me.x_btn_ok._onclick();
-			}
-		};
-		*/
-
-		//this.EVNNOTE._focus();
+		this.__tab1 = this.maintab.tab1;
 	}
 
 	if(this.__tab1.EVNNOTE) {
@@ -108,22 +50,14 @@ _me.__constructor = function() {
 		};
 
 		this.__tab1.EVNNOTE._rtl && this.__tab1.EVNNOTE.__eFrame.contentDocument.body.setAttribute('dir', 'rtl');
-		this.__tab1.EVNNOTE._onresize = function() {
-			var frmtbl = me._getAnchor('main').querySelector('.popupmaindialog > .frmtbl');
-			frmtbl && me._size(false, frmtbl.offsetHeight + 88);
-		};
 	} else {
-		this._create('EVNNOTE', 'obj_chat_input', 'text', '', {
-			smiles_enabled: GWOthers.getItem('CHAT', 'smiles') == '1'
+		await this._create('EVNNOTE', 'obj_chat_input', 'text', '', {
+			upload: true
 		});
 		this.EVNNOTE.input._placeholder(getLang('IM::COMMENT_PH'));
 		this.EVNNOTE.input._onsubmit = function(){
 			!me.x_btn_ok._disabled() && me.x_btn_ok._onclick();
 		};
-		this.EVNNOTE.input._onresize = function() {
-			var frmtbl = me._getAnchor('main').querySelector('.popupmaindialog > .frmtbl');
-			frmtbl && me._size(false, frmtbl.offsetHeight + 88);
-		};
 		this.EVNNOTE._folder({
 			aid: this._sAccountID,
 			fid: this._sFolderID
@@ -131,22 +65,23 @@ _me.__constructor = function() {
 	}
 
 	// Create 'CANCEL' button
-	this._create('x_btn_cancel', 'obj_button', 'footer','cancel noborder simple');
+	await this._create('x_btn_cancel', 'obj_button', 'footer','cancel noborder simple');
 	this.x_btn_cancel._value('FORM_BUTTONS::CANCEL');
 	this.x_btn_cancel._onclick = function() { me._destruct() };
 
-	this.__tab1._getAnchor('msiebox') && msiebox(this.__tab1._getAnchor('msiebox'));
+	this.x_btn_cancel._main.insertAdjacentElement('afterend', mkElement('div', {
+		className: 'flex-row-filler'
+	}));
 
 	if (this.x_btn_ok)
-	this.x_btn_ok._disabled(true);
+		this.x_btn_ok._disabled(true);
 
     this.X_ATTACHMENTS._disabled(this.__readonly);
     this.X_ATTACHMENTS._onchange = function(){
-
 		//Title
 		var sName = getLang('FILE::FILE');
 		if (this.__idtable)
-			for (var i = this.__idtable.length-1;i>-1;i--)
+			for (var i = this.__idtable.length;i--;)
 				if (!this.__idtable[i].removed && this.__idtable[i].name){
 					sName = this.__idtable[i].description || this.__idtable[i].name || '';
 
@@ -155,14 +90,10 @@ _me.__constructor = function() {
 
 					break;
 				}
-
-		me._title(sName + (me._aValues.EVNFOLDER ? ' - ' + me._aValues.EVNFOLDER : '') + (me._aValues.EVNEXPIRE > 0 ? ' - ' + getLang('FILE::EXPIRE', [IcewarpDate.unix(me._aValues.EVNEXPIRE).format('L')]) : ''), true);
-
-		//Form Buttons
-		// me.__checkEnableForm();
+		me._title(sName + (me._aValues.EVNFOLDER ? ' - ' + WMFolders.getFolderPath({aid: me._aValues.aid, fid: me._aValues.fid}) : '') + (me._aValues.EVNEXPIRE > 0 ? ' - ' + getLang('FILE::EXPIRE', [IcewarpDate.unix(me._aValues.EVNEXPIRE).format('L')]) : ''), true);
 	};
 
-	this.__initForm('FILE::FILE');
+	await this.__initForm('FILE::FILE');
 
 	if (this.__tab1.x_folders) {
 		if (this._sItemID){
@@ -177,16 +108,16 @@ _me.__constructor = function() {
 
 	//remove contextmenu
 	this.X_ATTACHMENTS._onuploadstart = function(){
-		if (this._parent.x_btn_ok)
-			this._parent.x_btn_ok._disabled(true);
-		if (this._parent.x_btn_cancel)
-			this._parent.x_btn_cancel._disabled(true);
+		if (me.x_btn_ok)
+			me.x_btn_ok._disabled(true);
+		if (me.x_btn_cancel)
+			me.x_btn_cancel._disabled(true);
 	};
 
 	this.X_ATTACHMENTS._onuploadend = function(){
 		me.__checkEnableForm();
-		if (this._parent.x_btn_cancel)
-			this._parent.x_btn_cancel._disabled(false);
+		if (me.x_btn_cancel)
+			me.x_btn_cancel._disabled(false);
 	};
 
 	//Registr DropZone
@@ -194,21 +125,12 @@ _me.__constructor = function() {
 		this.X_ATTACHMENTS._dropzone(this.__eContainer);
 
 	if((((this._aValues['ATTACHMENTS'] || [])[0] || {})['values'] || {})['ATTDESC']) {
-		this.X_ATTACHMENTS.btn_blank._disabled(true);
 		this.X_ATTACHMENTS.file._disabled(true);
 	}
 
-	// disable scrolling in this dialog (focus or pressing the TAB key can do the scrolling at least at Firefox)
-	// ...this is a workaround, not perfect solution, but it keeps current rendering as it is
-	this.__tab1._getAnchor('main').onscroll = function() {
-		me.__tab1._getAnchor('main').scrollTop = 0;
-	}.bind(this);
-
-	// Refresh list and preview in case tags were changed
-	if (this.__tab1.EVNTYPE)
-		this.__tab1.EVNTYPE.input._onChange = function(){
-			this.__refreshView = true;
-		}.bind(this);
+	if (this.__tab1.EVNTYPE) {
+		this.__tab1.EVNTYPE._setId([this._sAccountID, this._sFolderID, this._sItemID]);
+	}
 
 	this.__checkEnableForm(void 0, false);
 };
@@ -242,16 +164,16 @@ _me.__checkEnableForm = function(b, show_error){
 	return !disabled;
 };
 
-_me.__print = function(aValues){
+_me.__print = async function(aValues){
 	aValues = aValues.values;
 
 	if (!gui.print)
-		gui._create('print','frm_print');
+		await gui._create('print','frm_print');
 
 	gui.print._add('F', aValues);
 };
 
-_me.__loadItems = function() {
+_me.__loadItems = async function() {
 	var me = this;
 
 	if (this.__tab1.EVNNOTE.select){
@@ -261,107 +183,101 @@ _me.__loadItems = function() {
 			this.__tab1.EVNNOTE.select._value('disabled');
 	}
 
-	this._title(getLang('FILE::FILE') + (this._aValues.EVNFOLDER?' - '+this._aValues.EVNFOLDER:'') + (this._aValues.EVNEXPIRE>0?' - '+getLang('FILE::EXPIRE',[IcewarpDate.unix(this._aValues.EVNEXPIRE).format('L')]):''),true);
-
 	if (this._aValues['ATTACHMENTS'] || this._aValues['PUSH_ATTACHMENTS']){
-		var out = [], tmp;
-		for (var i in this._aValues['ATTACHMENTS']) {
-			tmp = {
-				'name': (this._aValues['ATTACHMENTS'][i]['values']['ATTDESC'] || '').unescapeHTML(),
-				'class': this._aValues['ATTACHMENTS'][i]['values']['ATTTYPE'],
-				'id': i,
-				'size': this._aValues['ATTACHMENTS'][i]['values']['ATTSIZE']
-			};
+		var out = [], tmp, i;
 
-			if (this._aValues['ATTACHMENTS'][i]['values']['fullpath'])
-				tmp.fullpath = this._aValues['ATTACHMENTS'][i]['values']['fullpath'];
-			else
-			if (this._aValues.fullpath)
-				tmp.fullpath = this._aValues.fullpath;
-			else
-			if (this._sItemID)
-				tmp.folder = this._sAccountID+'/'+this._sFolderID+'/'+WMItems.__serverID(this._sItemID);
+		if (this._aValues['ATTACHMENTS']){
+			for (i in this._aValues['ATTACHMENTS']) {
 
-			out.push(tmp);
+				if (~['thumbnail', 'pdf'].indexOf(this._aValues['ATTACHMENTS'][i]['values']['ATTTYPE']))
+					continue;
+
+				tmp = {
+					'name': (this._aValues['ATTACHMENTS'][i]['values']['ATTDESC'] || '').unescapeHTML(),
+					'class': this._aValues['ATTACHMENTS'][i]['values']['ATTTYPE'],
+					'id': i,
+					'size': this._aValues['ATTACHMENTS'][i]['values']['ATTSIZE'],
+				};
+
+				if (this._aValues['ATTACHMENTS'][i]['values']['ATTPARAMS'])
+					tmp.mime = this._aValues['ATTACHMENTS'][i]['values']['ATTPARAMS'].split('=')[1];
+
+				if (this._aValues['ATTACHMENTS'][i]['values']['fullpath'])
+					tmp.fullpath = this._aValues['ATTACHMENTS'][i]['values']['fullpath'];
+				else
+				if (this._aValues.fullpath)
+					tmp.fullpath = this._aValues.fullpath;
+				else
+				if (this._sItemID)
+					tmp.folder = this._sAccountID+'/'+this._sFolderID+'/'+WMItems.__serverID(this._sItemID);
+
+				out.push(tmp);
+			}
 		}
 
 		if (this._aValues['PUSH_ATTACHMENTS']){
-			this.X_ATTACHMENTS.list._value({
-				path: this._aValues['PUSH_ATTACHMENTS'][0]['fullpath'].substr(0, this._aValues['PUSH_ATTACHMENTS'][0]['fullpath'].lastIndexOf('/')),
-				values: [
-					{
-						'name': this._aValues['PUSH_ATTACHMENTS'][0]['title'],
-						'id': this._aValues['PUSH_ATTACHMENTS'][0]['id'],
-						'size': this._aValues['PUSH_ATTACHMENTS'][0]['size'],
-						'class': this._aValues['PUSH_ATTACHMENTS'][0]['embedded'] ? 'item' : 'itemlink',
-						'fullpath': this._aValues['PUSH_ATTACHMENTS'][0]['fullpath']
-					}
-				]
-			});
-		} else if (this._aValues.fullpath || !this._sItemID)
-			this.X_ATTACHMENTS.list._value({'values': out});
-		else
-			this.X_ATTACHMENTS.list._value({'path': this._sAccountID+'/'+this._sFolderID+'/'+WMItems.__serverID(this._sItemID), 'values': out});
+			//path = this._aValues['PUSH_ATTACHMENTS'][0]['fullpath'].substr(0, this._aValues['PUSH_ATTACHMENTS'][0]['fullpath'].lastIndexOf('/'));
 
-		//Focus to Blank file
-		//if (out[0]['class'] == 'document')
-			//this.X_ATTACHMENTS.list._rename(this.X_ATTACHMENTS.list.__idtable.length-1);
-	//	else
-			//this.__checkEnableForm(false);
+			for (i in this._aValues['PUSH_ATTACHMENTS']) {
+				tmp = {
+					'name': this._aValues['PUSH_ATTACHMENTS'][0]['title'],
+					'id': this._aValues['PUSH_ATTACHMENTS'][0]['id'],
+					'size': this._aValues['PUSH_ATTACHMENTS'][0]['size'],
+					'class': this._aValues['PUSH_ATTACHMENTS'][0]['embedded'] ? 'item' : 'itemlink',
+					'fullpath': this._aValues['PUSH_ATTACHMENTS'][0]['fullpath']
+				};
 
-		var extension;
-		var mime = '';
-		if (this._aValues.PUSH_ATTACHMENTS){
-			extension = (this._aValues.PUSH_ATTACHMENTS[0].title || '').split('.').pop();
-		} else if(Array.isArray(this._aValues['ATTACHMENTS'])) {
-			extension = (this._aValues['ATTACHMENTS'][0]['values']['ATTDESC'] || '').split('.').pop();
-		} else{
-			Object.keys(this._aValues['ATTACHMENTS']).filter(function(key) {
-				return this._aValues.ATTACHMENTS[key].values.ATTTYPE === 'attachment';
-			}, this).forEach(function(key) {
-				extension = (this._aValues.ATTACHMENTS[key].values.ATTDESC || '').split('.').pop();
-				this._aValues.ATTACHMENTS[key].values.ATTPARAMS && (mime = this._aValues.ATTACHMENTS[key].values.ATTPARAMS.split('=')[1]);
-			}, this);
+				out.push(tmp);
+			}
 		}
 
-		this.X_ATTACHMENTS._main.style.display = 'none';
+		if (this._aValues.fullpath || !this._sItemID)
+			this.X_ATTACHMENTS._value({values: out});
+		else
+			this.X_ATTACHMENTS._value({'folder': this._sAccountID+'/'+this._sFolderID+'/'+WMItems.__serverID(this._sItemID), 'values': out});
 
-		var label;
-		if(~mime.indexOf('video')) {
+		var extension = Path.extension(out[0].name),
+			label = '', type, preview,
+			mime = out[0].mime || '';
+
+		if (Item.imageSupport(out[0].name)) {
+			label = getLang('ITEMVIEW::MIME_IMG');
+			type = 'image';
+			preview = true;
+		} else if(~mime.indexOf('video')) {
+			type = 'video';
 			label = getLang('ITEMVIEW::MIME_VIDEO');
 		} else if(~mime.indexOf('audio')) {
+			type = 'audio';
 			label = getLang('ITEMVIEW::MIME_SOUND');
 		} else if(~mime.indexOf('image')) {
+			type = 'image';
 			label = getLang('ITEMVIEW::MIME_IMG');
 		} else if(~mime.indexOf('template')) {
 			label = getLang('ITEMVIEW::MIME_TPL');
-		} else if(~mime.indexOf('spreadsheet') || ~mime.indexOf('ms-excel')) {
-			label = getLang('ITEMVIEW::MIME_SS');
-		} else if(~mime.indexOf('document') || ~mime.indexOf('msword')) {
-			label = getLang('ITEMVIEW::MIME_DOC');
 		} else if(~mime.indexOf('compressed')) {
 			label = getLang('ITEMVIEW::MIME_ARCHIVE');
-		} else {
-			switch(extension.toLowerCase()) {
-				case 'pdf':
-					label = getLang('ITEMVIEW::MIME_PDF');
-				break;
-				case 'txt':
-					label = getLang('ITEMVIEW::MIME_TXT');
-				break;
-				case 'docx':
-					label = getLang('ITEMVIEW::MIME_DOC');
-				break;
-				case 'xlsx':
-					label = getLang('ITEMVIEW::MIME_SS');
-				break;
-				case 'pptx':
-					label = getLang('ITEMVIEW::MIME_PRESENTATION');
-				break;
-				default:
-					label = getLang('ITEMVIEW::MIME_TXT');
-				break;
-			}
+		} else if(extension.toLowerCase() === 'pdf') {
+			type = 'pdf';
+			label = getLang('ITEMVIEW::MIME_PDF');
+		} else if (Item.isDocument(out[0].name)) {
+			label = getLang('ITEMVIEW::MIME_DOC');
+			type = 'doc';
+		} else if (Item.isPresentation(out[0].name)) {
+			label = getLang('ITEMVIEW::MIME_PRESENTATION');
+			type = 'ppt';
+		} else if (Item.isSpreadsheet(out[0].name)) {
+			label = getLang('ITEMVIEW::MIME_SS');
+			type = 'xls';
+		} else if (Item.videoSupport(out[0].name)) {
+			label = getLang('ITEMVIEW::MIME_VIDEO');
+			type = 'video';
+		} else if (Item.audioSupport(out[0].name)) {
+			label = getLang('ITEMVIEW::MIME_SOUND');
+			type = 'audio';
+		} else if (Item.editSupport(out[0].name)) {
+			label = getLang('ITEMVIEW::MIME_TXT');
+			type = 'note';
 		}
 
 		var that = this;
@@ -372,10 +288,17 @@ _me.__loadItems = function() {
 			this.x_btn_ok._main.classList.add('new_document');
 		}
 
-		that._getAnchor('icon').appendChild(mkElement('div', {
-			className: 'icon ico_' + extension,
-			text: label
-		}));
+		if (preview) {
+			that._getAnchor('icon').appendChild(mkElement('img', {
+				className: 'preview',
+				src: await Item.webdavURL([this._sAccountID, this._sFolderID, this._sItemID], this.X_ATTACHMENTS.__idtable[0].name)
+			}))
+		}  else {
+			that._getAnchor('icon').appendChild(mkElement('div', {
+				className: 'icon ' + type,
+				text: label
+			}));
+		}
 
 		that._getAnchor('extension').textContent = '.' + extension;
 
@@ -385,7 +308,7 @@ _me.__loadItems = function() {
 		}
 		this.__changeFolder(this._sAccountID, this._sFolderID, this._sItemID);
 
-		if(this._skip_teamchat_choose_folder && ~['I', 'Y'].indexOf(WMFolders.getType([this._sAccountID, this._sFolderID]))) {
+		if (this._skip_teamchat_choose_folder && ~['I', 'Y'].indexOf(WMFolders.getType([this._sAccountID, this._sFolderID]))) {
 			this.__tab1.x_folders._destruct();
 		} else {
 			this.__tab1.x_folders._value('DOCUMENT::CHANGE_FOLDER');
@@ -395,7 +318,7 @@ _me.__loadItems = function() {
 		that._getAnchor('name').focus();
 		that._getAnchor('name').setAttribute('placeholder', getLang('DOCUMENT::NAME_HELPER'));
 		that._getAnchor('name').addEventListener('keyup', function(evn) {
-			if(evn.keyCode === 13) {
+			if (evn.keyCode === 13) {
 				if(!this.__checkEnableForm()) {
 					return;
 				}
@@ -404,47 +327,36 @@ _me.__loadItems = function() {
 				evn.target.blur();
 				return this.x_btn_ok._onclick();
 			}
-			for(var i in this.X_ATTACHMENTS.__idtable) {
-				this.X_ATTACHMENTS.__idtable[i].description = that._getAnchor('name').value + '.' + extension;
+
+			var files = this.X_ATTACHMENTS._value();
+			for(var i in files) {
+				if (files[i].id)
+					this.X_ATTACHMENTS._rename(files[i].id, that._getAnchor('name').value + '.' + extension);
 			}
 
-			//Title
-			var sName = getLang('FILE::FILE');
-			if (this.X_ATTACHMENTS.__idtable)
-				for (var i = this.X_ATTACHMENTS.__idtable.length-1;i>-1;i--)
-					if (!this.X_ATTACHMENTS.__idtable[i].removed && this.X_ATTACHMENTS.__idtable[i].name){
-						sName = this.X_ATTACHMENTS.__idtable[i].description || this.X_ATTACHMENTS.__idtable[i].name;
-
-						if (sName.length>64)
-							sName = sName.substr(0,50)+'..'+ Path.extension(sName);
-
-						break;
-					}
-
-			this._title(sName + (me._aValues.EVNFOLDER ? ' - ' + me._aValues.EVNFOLDER : '') + (me._aValues.EVNEXPIRE > 0 ? ' - ' + getLang('FILE::EXPIRE', [IcewarpDate.unix(me._aValues.EVNEXPIRE).format('L')]) : ''), true);
-
 			this.__checkEnableForm();
 		}.bind(this));
 
 		this._colapsed = true;
 		that._getAnchor('separator') && that._getAnchor('separator').addEventListener('click', function() {
 			this._onresize();
-			if(this._colapsed) {
+			if (this._colapsed) {
 				that._getAnchor('separator_text').textContent = getLang('DOCUMENT::SHOW_LESS_OPTIONS');
 				that._getAnchor('separator').classList.add('less');
 				var frmtbl = that._getAnchor('main').querySelector('.popupmaindialog > .frmtbl');
-				frmtbl && this._size(false, frmtbl.offsetHeight + 88);
+				frmtbl && this._defaultSize(740, 600);
 				this._resizable(true);
 			} else {
 				that._getAnchor('separator_text').textContent = getLang('DOCUMENT::SHOW_MORE_OPTIONS');
 				that._getAnchor('separator').classList.remove('less');
-				this._size(740, 315);
+				if(this.__position.max) {
+					this._maximize();
+				}
+				this._defaultSize(740, 350);
 				this._resizable(false);
 			}
 			this._colapsed = !this._colapsed;
 		}.bind(this));
-
-		// this.__checkEnableForm(!this._aValues['PUSH_ATTACHMENTS']);
 	}
 
 	//General
@@ -456,11 +368,9 @@ _me.__loadItems = function() {
 
 	//Resources
 	if (this.maintab && this.maintab['revisions'])
-	this.maintab['revisions']._onactive = function (bFirstTime) {
+	this.maintab['revisions']._onactive = async function (bFirstTime) {
 		if (bFirstTime){
 			//Layout
-			msiebox(this._getAnchor('msiebox'));
-
 			var btncheck = function (){
 				var l = me.maintab['revisions'].list._value().length<1,
 					b = me.x_btn_ok._disabled() || l || (me.__readonly?true:false);
@@ -474,7 +384,7 @@ _me.__loadItems = function() {
 			//DataGrid
 			this.list.__sortColumn = 'DATE';
 			this.list.__sortType = 'desc';
-			this.list._addColumns({
+			await this.list._addColumns({
 				DATE:{title:"DATAGRID_ITEMS_VIEW::DATE",width:120,arg:{sort:'asc'}},
 				OWNER:{title:"DATAGRID_ITEMS_VIEW::SNDOWNER",width:200,arg:{sort:'asc'}},
 				COMMENT: {title: "MAIL_VIEW::COMMENT", width: 100, mode:'%', encode: true}
@@ -486,7 +396,7 @@ _me.__loadItems = function() {
 				var aData = [];
 				for (var id in me._aValues['REVISIONS']){
 					aData.push({data:{
-						DATE:[IcewarpDate.unix(me._aValues['REVISIONS'][id].values.REVTIMESTAMP).format('L'),me._aValues['REVISIONS'][id].values.REVTIMESTAMP],
+						DATE:[IcewarpDate.unix(me._aValues['REVISIONS'][id].values.REVTIMESTAMP).format('L LT'),me._aValues['REVISIONS'][id].values.REVTIMESTAMP],
 						OWNER:me._aValues['REVISIONS'][id].values.REVEMAIL,
 						COMMENT:[me._aValues['REVISIONS'][id].values.REVCOMMENT,'',me._aValues['REVISIONS'][id].values.REVCOMMENT]
 					},
@@ -512,14 +422,14 @@ _me.__loadItems = function() {
 			this.list._ondblclick = function(){
 				this._parent.download._onclick();
 			};
-			this.list._oncontext = function(e,elm,arg,row,col){
+			this.list._oncontext = async function(e,elm,arg,row){
 				if (arg && arg.id){
 
 					if (inArray(this._value(),row)<0)
 						this._value([row]);
 
-					var cmenu = gui._create('cmenu','obj_context');
-						cmenu._fill([
+					var cmenu = await gui._create('cmenu','obj_context');
+						await cmenu._fill([
 							{title:'ATTACHMENT::DOWNLOAD',arg:[me.maintab.revisions.download,'_onclick']},
 							{title:'ITEM::EDIT_COMMENT',arg:[me.maintab.revisions.edit,'_onclick'],disabled:me.maintab.revisions.edit._disabled()},
 							{title:'ITEM::REVERT',arg:[me.maintab.revisions.revert,'_onclick'],disabled:me.maintab.revisions.revert._disabled()},
@@ -557,10 +467,6 @@ _me.__loadItems = function() {
 
 									me.maintab.revisions.list._fill2();
 								}
-								//Error
-								else{
-
-								}
 							}]);
 					}
 					else
@@ -568,7 +474,7 @@ _me.__loadItems = function() {
 					if ((v = v[0]) && (v = this._parent.list._aData[v]) && (v = v.arg))
 						switch (this._name)	{
 							case 'download':
-								downloadItem(buildURL({'sid': dataSet.get('main', ['sid']), 'class': 'revision', 'fullpath': me._sAccountID+'/'+me._sFolderID+'/'+WMItems.__serverID(me._sItemID)+'|'+v.id+'/'+attid}));
+								downloadItem(buildURL({'dlsess': dataSet.get('main', ['dlsess']), 'class': 'revision', 'fullpath': me._sAccountID+'/'+me._sFolderID+'/'+WMItems.__serverID(me._sItemID)+'|'+v.id+'/'+attid}));
 								break;
 
 							case 'edit':
@@ -584,7 +490,7 @@ _me.__loadItems = function() {
 								break;
 
 							case 'revert':
-								WMItems.action({aid:me._sAccountID,fid:me._sFolderID,iid:me._sItemID,values:{revision:v.id}},'revert_to_revision',[function(bOK){
+								WMItems.action({aid:me._sAccountID,fid:me._sFolderID,iid:me._sItemID,values:{revision:v.id}},'revert_to_revision',[function(){
 									me._destruct();
 									Item.openwindow([me._sAccountID,me._sFolderID,me._sItemID]);
 								}]);
@@ -600,8 +506,6 @@ _me.__loadItems = function() {
 			this.remove._onclick = click;
 		}
 	};
-	// this.__checkEnableForm();
-
 };
 
 _me.__changeFolder = function (sAccount, sFolder, sItem) {
@@ -620,13 +524,7 @@ _me.__changeFolder = function (sAccount, sFolder, sItem) {
 		sPath = (yf.NAME || yf.RELATIVE_PATH) + '/' + data.NAME;
 	}
 	else{
-		if (sFolder.indexOf(sPrimaryAccountSPREFIX) === 0)
-			sFolder = sFolder.substr(sPrimaryAccountSPREFIX.length);
-
-		if (data.NAME)
-			sPath = Path.basedir(sFolder) + '/' + data.NAME;
-		else
-			sPath = sFolder;
+		sPath = WMFolders.getFolderPath({aid: sAccount, fid: sFolder});
 	}
 
 	(this._getAnchor('folder') ? this : this.maintab.tab1)._getAnchor('folder').textContent = sPath;
@@ -636,7 +534,6 @@ _me.__changeFolder = function (sAccount, sFolder, sItem) {
 };
 
 _me.__saveItems = function(aValues) {
-
 	var is_teamchat_folder = WMFolders.getType({aid: this._sAccountID, fid: this._sFolderID}) === 'I';
 
 	if(this.__tab1.EVNNOTE.select && !is_teamchat_folder) {
@@ -646,32 +543,26 @@ _me.__saveItems = function(aValues) {
 		aValues['values']['EVNDESCFORMAT'] = 'text/plain';
 		this.__tab1.EVNNOTE.__output_format = false;
 	}
-	if(is_teamchat_folder && this.__tab1.EVNNOTE.__doc) {
-		aValues.values.EVNNOTE = this.__tab1.EVNNOTE.__doc.body.textContent.replace(/\n/g, ' ');
-	} else {
-		aValues.values.EVNNOTE = this.__tab1.EVNNOTE._value();
+	aValues.values.EVNNOTE = this.__tab1.EVNNOTE._value();
+	if(is_teamchat_folder) {
+		var turndownService = new TurndownService();
+		turndownService.use([turndownPluginGfm.tables]);
+		aValues.values.EVNNOTE = turndownService.turndown(aValues.values.EVNNOTE).replace(/!\[.*?\]\(.*?\)/g, '').replace(/<\/?img.*?>/g, '');
 	}
 
 	var addon;
 	aValues['values']['EVNCLASS'] = 'F';
-	if (!Is.Empty(addon = this.X_ATTACHMENTS.list._value())) {
-
+	if (!Is.Empty(addon = this.X_ATTACHMENTS._getAddon())) {
 		if (this._sItemID || addon.length == 1){
-			aValues['values']['EVNLOCATION'] = aValues['values']['EVNTITLE'] = addon[addon.length-1]['values']['description'];
+			aValues['values']['EVNLOCATION'] = aValues['values']['EVNTITLE'] = addon[0].values.description;
 			aValues['values']['EVNRID'] = aValues['values']['EVNLOCATION'];
-			aValues['values']['EVNCOMPLETE'] = this.X_ATTACHMENTS.list._getSize();
+			aValues['values']['EVNCOMPLETE'] = this.X_ATTACHMENTS._getSize();
 		}
 
 		var now = new IcewarpDate();
 		aValues['values']['EVNSTARTDATE'] = now.format(IcewarpDate.JULIAN);
 		aValues['values']['EVNSTARTTIME'] = now.format(IcewarpDate.JULIAN_TIME);
 
-		//EDIT since 7.10.2009 13:42:20
-		if (this._sItemID && !Is.Empty(this._aValues) && addon.length>1){
-            addon[1].uid = addon[0].uid;
-            addon.shift();
-		}
-
 		//Rename on Item level
 		if (addon.length == 1 && addon[0].values.description)
 			aValues.values.EVNTITLE = aValues.values.EVNLOCATION = aValues.values.EVNRID = addon[0].values.description;
@@ -695,9 +586,20 @@ _me._openFileAndListFolder = function (bOk,aData,bAuto){
 				iid: WMItems.__clientID(aData.id)
 			};
 
-			Item.officeOpen(arg,[Item.downloadFile,[[arg['aid'], arg['fid'], arg['iid']]]],Path.extension(aData.name).toLowerCase());
+			Item.officeOpen(arg,[Item.downloadFile,[[arg['aid'], arg['fid'], arg['iid']]]], Path.extension(aData.name));
 		} else {
-			Item.editFile([this._id[0], this._id[1], WMItems.__clientID(aData.id)]);
+			Item.previewItem([this._id[0], this._id[1], WMItems.__clientID(aData.id)], aData.name);
 		}
 	}
 };
+
+_me.__autoreopen = async function(data) {
+	// do not reinit attachments
+	var attachments = this._aValues.ATTACHMENTS;
+	delete this._aValues.ATTACHMENTS;
+
+	Object.assign(this._aValues, data.values);
+	await this.__loadItems();
+
+	this._aValues.ATTACHMENTS = attachments;
+};
\ No newline at end of file
