@@ -1,7 +1,7 @@
 _me = frm_gw.prototype;
 function frm_gw(){};
 
-_me.__constructor = function(sAccountID, sFolderID, sItemID, aValues, aReccurenceValues, oResponse, bClone) {
+_me.__constructor = async function(sAccountID, sFolderID, sItemID, aValues, aReccurenceValues, oResponse, bClone) {
 	if (!Is.Defined(sAccountID) || !Is.Defined(sFolderID)){
 		this._destruct();
 		return;
@@ -23,44 +23,6 @@ _me.__constructor = function(sAccountID, sFolderID, sItemID, aValues, aReccurenc
 	if (Is.Object(aValues)){
 		this._aValues = aValues;
 	}
-	// else
-	// if (this._sItemID){
-
-	// 	var aGet = {"aid": this._sAccountID, "fid": this._sFolderID, "iid": this._sItemID};
-	// 	if (aReccurenceValues && Is.Defined(aReccurenceValues.EVNRCR_ID) && Is.Defined(aReccurenceValues.EVNSTARTDATE))
-	// 		aGet.date = aReccurenceValues.EVNSTARTDATE;
-
-	// 	this._aValues = WMItems.list(aGet);
-
-	// 	if (this._aValues && this._aValues[this._sAccountID] && this._aValues[this._sAccountID][this._sFolderID]){
-	// 		if (this._sFolderID == '@@mycard@@'){
-    //             this._sItemID = '';
-
-	// 			this._aValues = this._aValues[this._sAccountID][this._sFolderID];
-	//             delete this._aValues['/'];
-	//             delete this._aValues['$'];
-	//             delete this._aValues['#'];
-	//             delete this._aValues['@'];
-
-	// 			for(var i in this._aValues)
-	// 				if (Is.Object(this._aValues[i])){
-	// 					this._sItemID = i;
-	// 					break;
-	// 				}
-
-	// 			this._aValues = this._aValues[this._sItemID] || {};
-	// 		}
-	// 		else
-	// 			this._aValues = this._aValues[this._sAccountID][this._sFolderID][this._sItemID];
-	// 	}
-	// 	else
-	// 	if (this._sFolderID == '@@mycard@@'){
-	// 		this._sItemID = '';
-	// 		this._aValues = {LOCATIONS:[{values:{LCTEMAIL1:sPrimaryAccount,LCTTYPE:'H'}}]};
-	// 	}
-	// 	else
-	// 		this._aValues = {};
-	// }
 	else
 		this._aValues = {};
 
@@ -84,9 +46,9 @@ _me.__constructor = function(sAccountID, sFolderID, sItemID, aValues, aReccurenc
 	}
 };
 
-_me._onclose = function (b) {
-	if (b && (this._userEdited() || this.__autosaveid)){
-		gui._create('frm_confirm','frm_confirm', '','', [this, '__confirmed'], 'CONFIRMATION::ARE_YOU_SURE','CONFIRMATION::ALL_CHANGES_WILL_BE_LOST');
+_me._onclose = async function (b) {
+	if (b && (await this._userEdited() || this.__autosaveid)){
+		this._gui._create('frm_confirm','frm_confirm', '','', [this, '__confirmed'], 'CONFIRMATION::ARE_YOU_SURE','CONFIRMATION::ALL_CHANGES_WILL_BE_LOST');
 		return false;
 	}
 
@@ -107,125 +69,111 @@ _me.__end_autosave = function() {
 };
 
 // return stringified form data
-_me.__stringifyForm = function(){
+_me.__stringifyForm = async function(){
 	var aValues = {'values':{}};
 	storeDataFromForm(this, aValues['values']);
 
-	aValues.values[this._type == 'frm_contact' || this._type == 'frm_distrib'?'ITMSHARETYPE':'EVNSHARETYPE'] = this.X_ITMSHARETYPE._value();
+	if (this.X_ITMSHARETYPE) {
+		aValues.values[this._type == 'frm_contact' || this._type == 'frm_distrib'?'ITMSHARETYPE':'EVNSHARETYPE'] = this.X_ITMSHARETYPE._value();
+	}
 
-	this.__saveItems(aValues);
+	await this.__saveItems(aValues);
 
 	return JSON.stringify(aValues);
 };
 
-_me.__initForm = function(sTitle) {
+_me.__initForm = async function(sTitle) {
 	this._title(sTitle);
 
 	this.__loadItems();
-	this.__createFooter();
+	await this.__createFooter();
 
 	// Save initial form content
 	if (!this._bClone && !this._aValues.PUSH_ATTACHMENTS && (this._sItemID || Is.Empty(this._aValues)))
-		this.__currentSaveState = this.__stringifyForm();
+		setTimeout(async function() { // allow initialization of async components
+			this.__currentSaveState = await this.__stringifyForm();
+		}.bind(this), 50); // timeout for froala init
 };
 
 // Detect changes from initial state
-_me._userEdited = function() {
-	return (this.__currentSaveState || '').replace(/\xA0|&nbsp;/g, ' ') != (this.__stringifyForm() || '').replace(/\xA0|&nbsp;/g, ' ');
+_me._userEdited = async function() {
+	var oldState = (this.__currentSaveState || '').replace(/\xA0|&nbsp;/g, ' ');
+	var newState = (await this.__stringifyForm() || '').replace(/\xA0|&nbsp;/g, ' ');
+
+	var oldState_parsed, newState_parsed;
+	try {
+		oldState_parsed = JSON.parse(oldState);
+	} catch {
+		oldState_parsed = {values: {}};
+	}
+	try {
+		newState_parsed = JSON.parse(newState);
+	} catch {
+		newState_parsed = {values: {}};
+	}
+
+	for (var i in newState_parsed.values) {
+		if ((~i.indexOf('NOTE') || ~i.indexOf('DESCRIPTION')) && (newState_parsed.values[i] === '' || newState_parsed.values[i] === NewMessage.crlf) && oldState_parsed.values[i] === void 0) {
+			delete newState_parsed.values[i];
+			delete newState_parsed.values.ITMDESCFORMAT;
+			delete newState_parsed.values.EVNDESCFORMAT;
+		} else if (newState_parsed.values[i] === '' && oldState_parsed.values[i] === void 0) {
+			delete newState_parsed.values[i];
+		}
+	}
+
+	oldState = JSON.stringify(oldState_parsed) || '';
+	newState = JSON.stringify(newState_parsed) || '';
+
+	return oldState != newState;
 };
 
 
 // Virtual function
-_me.__saveItems = function(aValues) {};
+_me.__saveItems = async function() {};
 
 // Virtual function
-_me.__loadItems = function(aValues) {};
+_me.__loadItems = function() {};
 
-_me.__createFooter = function() {
+_me.__createFooter = async function() {
 
 	var me = this;
 
-	//Share button
-	if (this._sItemID && !sPrimaryAccountGUEST){
+	//Private checkbox
+	await this._create('X_ITMSHARETYPE', 'obj_sharing', 'footer');
 
-		this._create('x_btn_share', 'obj_button', 'footer', 'color1 simple x_btn_right' + (this._type === 'frm_file' ? ' select' : ''));
+	//Print button
+	if (this._type != 'frm_distrib' && this._type != 'frm_file'){
+
+		await this._create('x_btn_print', 'obj_button', 'footer','noborder transparent ico img print simple x_btn_right');
+		this.x_btn_print._title('MAIN_MENU::PRINT');
+		this.x_btn_print._onclick = async function() {
+			if (me.__print)
+				me.__print(await me.__readItems());
+		};
+	}
+
+	//Share button
+	if (this._sItemID && !sPrimaryAccountGUEST && WMFolders.getType([me._sAccountID, me._sFolderID]) !== 'E') {
+
+		await this._create('x_btn_share', 'obj_button', {
+			position: 'afterend',
+			element: this._getAnchor('title')
+		}, 'color1 simple rounded share');
 		this.x_btn_share._value('MAIN_MENU::SHARE');
 
 		if (!sPrimaryAccountGUEST || me._aValues.TICKET){
-			this.x_btn_share._onclick = function(e) {
+			this.x_btn_share._onclick = function() {
 				if(me._type !== 'frm_file') {
 					return Item.sendAsEmail([me._sAccountID, me._sFolderID, [me._sItemID]]);
 				}
-
-				if (!this.__cmenu || this.__cmenu._destructed){
-					var pos = getSize(this.__eIN),
-						aMenu = [];
-
-					if (!sPrimaryAccountGUEST)
-						aMenu.push({title:'POPUP_ITEMS::SEND_AS_EMAIL', 'arg': [Item.sendAsEmail, [[me._sAccountID, me._sFolderID, [me._sItemID]]]]});
-
-					if (me._aValues.TICKET)
-						aMenu.push({title:'ITEM::PUBLIC', 'arg':[me,'_publicURL']});
-
-					this.__cmenu = gui._create('cmenu','obj_context'),
-					this.__cmenu._fill(aMenu);
-					this.__cmenu._place(pos.x+pos.w/2,pos.y,'',3);
-
-					if (e.stopPropagation) e.stopPropagation();
-					e.cancelBubble = true;
-					return false;
-				}
+				Item.collaborate([me._sAccountID, me._sFolderID, me._sItemID], me._gui);
 			};
 		}
 		else
 			this.x_btn_share._disabled(true);
 	}
 
-	//Print button
-	if (this._type != 'frm_distrib' && this._type != 'frm_file'){
-
-		this._create('x_btn_print', 'obj_button', 'footer','noborder transparent ico img print simple x_btn_right');
-		this.x_btn_print._title('MAIN_MENU::PRINT');
-		this.x_btn_print._onclick = function() {
-			if (me.__print)
-				me.__print(me.__readItems());
-		};
-	}
-
-	//Private checkbox
-	this._create('X_ITMSHARETYPE', 'obj_sharing', 'footer','toright');
-
-	if (this._sItemID){
-		this._create('x_btn_delete', 'obj_button', 'footer','noborder transparent ico img trash simple x_btn_right');
-		this.x_btn_delete._title('MAIN_MENU::DELETE');
-
-		this.x_btn_delete._onclick = function(){
-			if (me._repeating)
-				gui._create('frm_confirm','frm_confirm_repeating','','', [function(state){
-
-					Item.__removeWithRepeating(state, [me._sAccountID, me._sFolderID, [me._sItemID]], me._aReccurenceValues);
-					me._destruct();
-
-					if (me.__oresponse)
-						executeCallbackFunction(me.__oresponse, false);
-
-				}],'REPEATING_CONFIRM::TITLE_DELETE','REPEATING_CONFIRM::TEXT_DELETE');
-			else {
-				var frm = gui._create('frm_confirm','frm_confirm', '', '', [function(){
-					Item.__delete([me._sAccountID, me._sFolderID, [me._sItemID]]);
-					me._destruct();
-
-					if (me.__oresponse)
-						executeCallbackFunction(me.__oresponse, false);
-
-				}],'CONFIRMATION::DELETE_ITEM_CONFIRMATION','CONFIRMATION::DELETE_ITEM');
-				frm._size(400, 180);
-				frm.obj_label.__eIN.innerHTML += '<br>' + getLang('CONFIRMATION::DELETE_ITEM_TRASH');
-			}
-		};
-	}
-
-
 	if (this.__readonly){
 		//Disable Buttons
 		this.x_btn_ok._disabled(1);
@@ -239,7 +187,7 @@ _me.__createFooter = function() {
 	}
 	else{
 		// This function is triggered when 'OK' button is pressed
-		this.x_btn_ok._onclick = function() {
+		this.x_btn_ok._onclick = async function() {
 
 			if (this._disabled()) return;
 
@@ -251,19 +199,55 @@ _me.__createFooter = function() {
 
 			if(me.__onBeforeSave)
 				me.__onBeforeSave();
-			// Perform save
-			if (me.__storeItems())
-				me.__hide();
-			else
-				this._disabled(false);
+
+			if (me.__onBeforeStoreItems) {
+				me.__onBeforeStoreItems();
+			} else {
+				// Perform save
+				if (await me.__storeItems())
+					me.__hide('ATTACHMENT::SAVING');
+				else
+					this._disabled(false);
+			}
 		};
 
+		this.x_btn_cancel._destruct();
 		// Cancel button will remove autosaved item
-		this.x_btn_cancel._onclick = function(e) {
-			if (me.__autosaveid && GWOthers.getItem('DOCUMENTS','autosave') == 1)
-				Item.__delete([me._sAccountID, me._sFolderID, [me._sItemID]]);
+		// this.x_btn_cancel._onclick = function(e) {
+		// 	if (me.__autosaveid && GWOthers.getItem('DOCUMENTS','autosave') == 1)
+		// 		Item.__delete([me._sAccountID, me._sFolderID, [me._sItemID]]);
 
-			me._close(true, e);
+		// 	me._close(true, e);
+		// };
+	}
+
+	if (this._sItemID){
+		await this._create('x_btn_delete', 'obj_button', 'footer', 'color2 x_btn_right');
+		this.x_btn_delete._value('MAIN_MENU::DELETE');
+
+		this.x_btn_delete._onclick = async function(){
+			if (me._repeating)
+				await gui._create('frm_confirm','frm_confirm_repeating','','', [function(state){
+
+					Item.__removeWithRepeating(state, [me._sAccountID, me._sFolderID, [me._sItemID]], me._aReccurenceValues);
+					me._destruct();
+
+					if (me.__oresponse)
+						executeCallbackFunction(me.__oresponse, false);
+
+				}],'REPEATING_CONFIRM::TITLE_DELETE','REPEATING_CONFIRM::TEXT_DELETE');
+			else {
+				var frm = await gui._create('frm_confirm','frm_confirm', '', 'frm_trash', [function(){
+					Item.__delete([me._sAccountID, me._sFolderID, [me._sItemID]]);
+					me._destruct();
+
+					if (me.__oresponse)
+						executeCallbackFunction(me.__oresponse, false);
+
+				}],'CONFIRMATION::DELETE_ITEM_CONFIRMATION','CONFIRMATION::DELETE_ITEM');
+				frm.obj_label.__eIN.innerHTML += '<br>' + getLang('CONFIRMATION::DELETE_ITEM_TRASH');
+				frm._size(400, 'auto');
+			}
 		};
 	}
 
@@ -307,7 +291,6 @@ _me.__sharetype = function(){
 			var types = {
 				frm_contact:'contact_sharing',
 				frm_distrib:'contact_sharing',
-				frm_journal:'journal_sharing',
 				frm_note:'note_sharing',
 				frm_file:'file_sharing',
 				frm_task:'task_sharing'
@@ -319,11 +302,7 @@ _me.__sharetype = function(){
 	}
 };
 
-_me._publicURL = function(){
-	this._sItemID && Item.collaborate([this._sAccountID, this._sFolderID, [this._sItemID]]);
-};
-
-_me.__readItems = function(bFilter){
+_me.__readItems = async function(bFilter){
 	var aValues = {'values':{}};
 
 	for(var i in this._aValues)
@@ -334,7 +313,7 @@ _me.__readItems = function(bFilter){
 
 	if (bFilter){
 		for(var key in aValues.values){
-			if(!!~this.__filterOut.indexOf(key.toLowerCase())){
+			if(~this.__filterOut.indexOf(key.toLowerCase())){
 				delete aValues.values[key];
 			}
 		}
@@ -343,7 +322,7 @@ _me.__readItems = function(bFilter){
 	//store sharetype
 	aValues.values[this._type == 'frm_contact' || this._type == 'frm_distrib'?'ITMSHARETYPE':'EVNSHARETYPE'] = this.X_ITMSHARETYPE._value();
 
-	this.__saveItems(aValues);
+	await this.__saveItems(aValues);
 
 	return aValues;
 };
@@ -359,16 +338,20 @@ _me.__filterOut = [
 	'tzlink'
 ];
 
-_me.__storeItems = function (bAuto){
-	var me = this;
+_me.__storeItems = async function (bAuto){
+	var me = this, i;
 
-	if (this._userEdited() || (!bAuto && this._aValues.PUSH_ATTACHMENTS)){
+	var folder_changed = false;
+	if (this._type === Mapping.getFormNameByGWType('E')) {
+		folder_changed = me._id[2] && (this._aValues.EVNFOLDER === Path.split(this.X_PATH._value())[1] ? false : Path.split(this.X_PATH._value()));
+	}
+	if (await this._userEdited() || (!bAuto && this._aValues.PUSH_ATTACHMENTS) || folder_changed) {
 
 		//PREPARE DATE
 		var aValues = {'values':{}};
 		storeDataFromForm(this, aValues['values']);
 		for(var key in aValues.values){
-			if(!!~this.__filterOut.indexOf(key.toLowerCase())){
+			if(~this.__filterOut.indexOf(key.toLowerCase())){
 				delete aValues.values[key];
 			}
 		}
@@ -386,12 +369,12 @@ _me.__storeItems = function (bAuto){
 		// 	aValues.values.SKIP_INVITATION = this._aValues.SKIP_INVITATION;
 
 		// Parse and add values specific to GW type
-		this.__saveItems(aValues);
+		await this.__saveItems(aValues);
 
 		// For edited calendar events let user decide if participants (if any) should be informed of changes
 		var not_me_attendees = [];
 		if(this._aValues.CONTACTS) {
-			for(var i in this._aValues.CONTACTS) {
+			for(i in this._aValues.CONTACTS) {
 				if(this._aValues.CONTACTS[i].CNTEMAIL !== sPrimaryAccount) {
 					not_me_attendees.push(this._aValues.CONTACTS[i]);
 				}
@@ -400,34 +383,122 @@ _me.__storeItems = function (bAuto){
 		if (!bAuto && this._type == Mapping.getFormNameByGWType('E') && this._sItemID && not_me_attendees.length) {
 
 			// Always notify others when repeated event or removed participants
-			if (this._repeating || this._notify_attendees) {
-				gui._create('frm_confirm','frm_confirm', '', '', [function(){
-					WMItems.add(me._id, aValues, '', '','',[me,'_listFolder']);
-					me.__hide();
-				}],'CONFIRMATION::INFORM_ATTENDEES','CONFIRMATION::NOTIFY_OR_DISCARD');
+			if (this._notify_attendees) {
+				await gui._create('frm_confirm','frm_confirm', '', '', [function(){
+					WMItems.add(me._id, aValues, '', '','',[function(bOk, aData) {
+						if (folder_changed) {
+							WMItems.move({'aid':me._id[0],'fid':me._id[1],'iid':[me._id[2]],'account':folder_changed[0],'folder':folder_changed[1]},'items','','folders',[function() {
+								me._listFolder(bOk, aData);
+							}]);
+						} else {
+							me._listFolder(bOk, aData);
+						}
+					}]);
+					me.__hide('ATTACHMENT::SAVING');
+				}],'CONFIRMATION::INFORM_ATTENDEES','CONFIRMATION::NOTIFY_OR_DISCARD', '', [function() {
+					me.x_btn_ok._disabled(false);
+				}]);
 				return false;
 			}
 
 			// For other changes let user decide if others should be informed
-			gui._create('frm_confirm','frm_confirm', '', '', [function(){
-				WMItems.add(me._id, aValues, '', '','',[me,'_listFolder']);
+			await gui._create('frm_confirm','frm_confirm', '', '', [function(){
+				WMItems.add(me._id, aValues, '', '','',[function(bOk, aData) {
+					if (folder_changed) {
+						WMItems.move({'aid':me._id[0],'fid':me._id[1],'iid':[me._id[2]],'account':folder_changed[0],'folder':folder_changed[1]},'items','','folders',[function() {
+							me._listFolder(bOk, aData);
+						}]);
+					} else {
+						me._listFolder(bOk, aData);
+					}
+				}]);
 			}],'CONFIRMATION::INFORM_ATTENDEES','CONFIRMATION::SEND_EVENT_UPDATE');
 			gui.frm_confirm.x_btn_cancel._value('CONFIRMATION::SUPPRESS_EVENT_UPDATE');
 			gui.frm_confirm.x_btn_cancel._onclick = function() {
 				aValues.values  = aValues.values || {};
 				aValues.values['SKIP_INVITATION'] = 1;
-				WMItems.add(me._id, aValues, '', '','',[me,'_listFolder']);
+				WMItems.add(me._id, aValues, '', '','',[function(bOk, aData) {
+					if (folder_changed) {
+						WMItems.move({'aid':me._id[0],'fid':me._id[1],'iid':[me._id[2]],'account':folder_changed[0],'folder':folder_changed[1]},'items','','folders',[function() {
+							me._listFolder(bOk, aData);
+						}]);
+					} else {
+						me._listFolder(bOk, aData);
+					}
+				}]);
 				gui.frm_confirm._destruct();
 			};
 
+			await gui.frm_confirm._create('cancel', 'obj_button', 'footer', 'color2 simple');
+			gui.frm_confirm.cancel._value('FORM_BUTTONS::CANCEL');
+			gui.frm_confirm.cancel._onclick = function() {
+				gui.frm_confirm._destruct();
+				me.x_btn_ok._disabled(false);
+			};
+
+			var autoSupress = true;
+			for (i in aValues.values) {
+				if (!~['EVNFLAGS', 'EVNTYPE'].indexOf(i) && (aValues.values[i] || '').replace(/<br ?\/>/g, '<br>') != (this._aValues[i] || '').replace(/<br ?\/>/g, '<br>')) {
+					autoSupress = false;
+					break;
+				}
+			}
+			for (i in aValues.CONTACTS) {
+				if (!aValues.CONTACTS[i].uid) {
+					autoSupress = false;
+				}
+				if (autoSupress) {
+					break;
+				}
+				if ((aValues.CONTACTS[i].values && !this._aValues.CONTACTS[aValues.CONTACTS[i].uid].values) || (!aValues.CONTACTS[i].values && this._aValues.CONTACTS[aValues.CONTACTS[i].uid].values)) {
+					autoSupress = false;
+					break;
+				}
+				for (var j in aValues.CONTACTS[i].values) {
+					if (j !== 'CNTSTATUS' && ((aValues.CONTACTS[i].values[j] || false) != (this._aValues.CONTACTS[aValues.CONTACTS[i].uid].values[j] || false))) {
+						autoSupress = false;
+						break;
+					}
+				}
+			}
+			if (autoSupress) {
+				gui.frm_confirm.x_btn_cancel._onclick();
+			}
+
 			return true;
 		}
 
 		// Send server request to save item
-		if (!Is.Empty(aValues))
-			return WMItems.add(this._id, aValues, '', '','',[this,this._type === 'frm_file' ? '_openFileAndListFolder' : '_listFolder',[bAuto]],bAuto);
-		else
-			this.__currentSaveState = this.__stringifyForm();
+		if (!Is.Empty(aValues)) {
+			return WMItems.add(this._id, aValues, '', '','',[function(bOk, aData) {
+				if (folder_changed) {
+					WMItems.move({'aid':me._id[0],'fid':me._id[1],'iid':[me._id[2]],'account':folder_changed[0],'folder':folder_changed[1]},'items','','folders',[function() {
+						if (me._type === 'frm_file') {
+							me._openFileAndListFolder(bOk, aData, bAuto);
+						} else {
+							me._listFolder(bOk, aData, bAuto);
+						}
+					}]);
+				} else {
+					if (me._type === 'frm_file') {
+						aData.name = aData.name || aValues.values.EVNTITLE;
+						aData.folder = aData.folder || me._id[1];
+						me._openFileAndListFolder(bOk, aData, bAuto);
+					} else {
+						if (me._type === 'frm_contact' && me._id[2]) {
+							delete obj_avatar.__localCache[WMItems.__serverID(me._id[2])];
+							obj_avatar.getAvatarElement({
+								contactId: me._id[2]
+							});
+						}
+						me._listFolder(bOk, aData, bAuto);
+					}
+				}
+			}],bAuto, [function() {
+				me.__show();
+			}]);
+		} else
+			this.__currentSaveState = await this.__stringifyForm();
 	}
 	else
 	if (!bAuto){
@@ -441,13 +512,13 @@ _me.__storeItems = function (bAuto){
 	}
 };
 
-_me._listFolder = function (bOk,oData,bAuto){
+_me._listFolder = async function (bOk,oData,bAuto){
 	if (!bOk){
 		switch(oData) {
 			case 'item_create':
 				var me = this;
 
-				var frm = gui._create('create_default', 'frm_confirm', '', '', [
+				var frm = await gui._create('create_default', 'frm_confirm', '', '', [
 					function() {
 						me._id[1] = me._sFolderID = Mapping.getDefaultFolderForGWType(WMFolders.getType([me._sAccountID, me._sFolderID]));
 						me._id[0] = me._sAccountID = sPrimaryAccount;
@@ -468,7 +539,7 @@ _me._listFolder = function (bOk,oData,bAuto){
 	}
 
 	if (!this._destructed){
-		this.__currentSaveState = this.__stringifyForm();
+		this.__currentSaveState = await this.__stringifyForm();
 
 		//Item was not stored but error was handled
 		if (bOk == 2){
@@ -515,19 +586,19 @@ _me._listFolder = function (bOk,oData,bAuto){
 				'ACTION': this._sItemID?'edit':'add',
 				'TYPE':'item',
 				'ITEM': oData.id,
-				'FOLDER':f.RELATIVE_PATH,
+				'ITEM-NAME': oData.name,
+				'FOLDER':this._sFolderID,
 				'FOLDER-TYPE':f.TYPE,
 				'EMAIL':sPrimaryAccount,
-				'NAME':dataSet.get('main',['fullname']) || dataSet.get('main', ['user']),
-				'ITEM-TYPE': this._aValues.EVNCLASS || f.TYPE,
-				'owner': this._pathName
+				'ORIGINATOR-NAME':dataSet.get('main',['fullname']) || dataSet.get('main', ['user']),
+				'ORIGINATOR-EMAIL':sPrimaryAccount,
+				'ORIGINATOR-DEVICEID': dataSet.get('main', ['device_id']),
+				'ITEM-TYPE': this._aValues.EVNCLASS || f.TYPE
 			};
 
 			gui.socket.api._notify(aOut);
 		}
 	}
-
-	return false;
 };
 
 _me._refreshListAndPreview = function() {
@@ -554,7 +625,7 @@ _me._refreshListAndPreview = function() {
 					gui.frm_main.main._serverSort();
 		}
 	}
-	catch(r){gui._REQUEST_VARS.debug && console.log(this._name||false,r)}
+	catch(r){ console.log(this._name||false,r)}
 
 	// Refresh Preview
 	if (this._sItemID && gui.frm_main.main.itemview && dataSet.get('preview',[this._sAccountID,this._sFolderID,this._sItemID])){
@@ -563,3 +634,7 @@ _me._refreshListAndPreview = function() {
 	}
 	*/
 };
+
+_me.__storeAutoreopenData = async function() {
+	return await this.__readItems();
+};
