@@ -6,17 +6,16 @@ function frm_input(){};
  *
  * @see  IM/new group
  */
-_me.__constructor = function(aResponse, sLabel1, sLabel2, sLabel3) {
+_me.__constructor = async function(aResponse, sLabel1, sLabel2, sLabel3) {
 
 	var me = this;
 
-	this._size(300,145,true);
     this._resizable(false);
     this._dockable(false);
 	this._modal(true);
 
-	this._draw('frm_input','main');
-	this._create('obj_input','obj_input','input','obj_input_100');
+	await this._draw('frm_input','main');
+	await this._create('obj_input','obj_input','input','obj_input_100');
 
 	this.obj_input._onsubmit = function (){
 		me.x_btn_ok._onclick();
@@ -36,4 +35,5 @@ _me.__constructor = function(aResponse, sLabel1, sLabel2, sLabel3) {
 	};
 
 	this.obj_input._focus();
+	this._size(300,'auto',true);
 };
\ No newline at end of file
