@@ -4,15 +4,15 @@ function frm_mail_print(){};
 /**
  *
  */
-_me.__constructor = function(aResponse, aOpt) {
+_me.__constructor = async function(aResponse, aOpt) {
 	var me = this;
 
-	this._size(450,300,true);
 	//this._modal(true);
 	this._dockable(false);
 	this._resizable(false);
 
-    this._draw('frm_mail_print', 'main', aOpt);
+    await this._draw('frm_mail_print', 'main', aOpt);
+	this._size(450,'auto',true);
 
 	this._title('PRINT::OPTIONS');
 
@@ -23,7 +23,7 @@ _me.__constructor = function(aResponse, aOpt) {
 			this[sObj]._checked(true);
 
 	// Create 'OK' button
-	this._create('x_btn_ok', 'obj_button', 'footer', 'noborder ok color1');
+	await this._create('x_btn_ok', 'obj_button', 'footer', 'noborder ok color1');
 	this.x_btn_ok._value('MAIN_MENU::PRINT');
 	this.x_btn_ok._onclick = function() {
 
@@ -41,7 +41,7 @@ _me.__constructor = function(aResponse, aOpt) {
 	};
 
 	// Create 'CANCEL' button
-	this._create('x_btn_cancel', 'obj_button', 'footer', 'cancel noborder simple');
+	await this._create('x_btn_cancel', 'obj_button', 'footer', 'cancel noborder simple');
 	this.x_btn_cancel._value('FORM_BUTTONS::CANCEL');
 	this.x_btn_cancel._onclick = function() { me._destruct(); };
 
