@@ -1,14 +1,15 @@
 _me = frm_main_calendar.prototype;
-function frm_main_calendar() {}
-;
+function frm_main_calendar() {};
 
-_me.__constructor = function (sDataset){
+_me.__constructor = async function (sDataset){
 	//Reset Title
 	gui.frm_main._title();
 
 	var me = this;
 
-	cInit.prototype._setupCalendar();
+	this.__lastFolder = {};
+
+	await initPRO.prototype._setupCalendar();
 
 	this._hasWorkView = parseInt(GWOthers.getItem('CALENDAR_SETTINGS', 'workweek_begins')) && parseInt(GWOthers.getItem('CALENDAR_SETTINGS', 'workweek_ends'));
 
@@ -33,190 +34,234 @@ _me.__constructor = function (sDataset){
 			oRepeating = dataSet.get('items', [me.__aid, me.__fid, id]);
 		}
 
-		Item.remove([me.__aid, me.__fid, [me._stripPipe(id)]], shift_key || false, oRepeating, this);
-	};
-
-	this.calendar._ondblclick = function (e, elm, arg) {
-		switch (arg._event) {
-			case 'event':
-				var oRepeating = dataSet.get('items', [me.__aid, me.__fid, arg['id']]);
-				Item.open([me.__aid, me.__fid, me._stripPipe(arg['id'])], oRepeating);
-				break;
-
-			case 'allday':
-			case 'blank':
-				if (!this.__rights.write)
-					return;
-
-				arg['enddate'] = arg['startdate'];
-
-				if (arg._event == 'allday')
-					arg['enddate']++;
-
-				var nStartTime = (arg['starttime'] >= 0) ? arg['starttime'] / 60 : -1;
-				var nEndTime = (arg['starttime'] >= 0) ? (arg['starttime'] / 60) + 30 : -1;
-
-				if (arg._event == 'blank' && nEndTime == 1440){
-					nEndTime = 0;
-					arg['enddate']++;
-				}
-
-				var aInterval = {'EVNSTARTDATE': arg['startdate'], 'EVNSTARTTIME': nStartTime, 'EVNENDDATE': arg['enddate'], 'EVNENDTIME': nEndTime};
-				Item.openwindow([me.__aid, me.__fid], aInterval);
-
-				break;
+		if (shift_key && !Is.Object(oRepeating)) {
+			return gui._create('frm_confirm', 'frm_confirm_suppress', '', 'frm_trash', [Item.remove, [[me.__aid, me.__fid, [WMItems._stripPipe(id)]], true, oRepeating, this]],'CONFIRMATION::DELETE_ITEM_CONFIRMATION','CONFIRMATION::DELETE_ITEM');
 		}
+		Item.remove([me.__aid, me.__fid, [WMItems._stripPipe(id)]], shift_key || false, oRepeating, this);
 	};
 
-	this.calendar._oncontext = function (e, elm, arg) {
-		var aMenu = '',
-			cmenu;
+	this.calendar._ondblclick = this._dblclick.bind(this);
+	this.calendar._oncontext = this._contextmenu.bind(this);
 
-		arg['aid'] = me.__aid;
-		arg['fid'] = me.__fid;
+	gui._obeyEvent('item', [this, '__item_refresh']);
+	this._add_destructor('__destruct_refresh');
+};
 
-		switch (arg._event) {
-			case 'allday':
-			case 'blank':
-				arg['enddate'] = arg['startdate'];
+_me.__destruct_refresh = function(){
+	gui._disobeyEvent('item', [this, '__item_refresh']);
+	clearTimeout(this.__xhrTimeout);
+};
 
-				if (arg._event == 'allday') {
-					arg['enddate']++;
+_me.__item_refresh = function(sMethod, args){
+	if (args.aid == this.__aid && (args.fid == this.__fid || Is.Defined((this.__fid == "__@@VIRTUAL@@__/__@@EVENTS@@__" && dataSet.get('folders',[this.__aid, this.__fid, 'VIRTUAL', 'FOLDERS']) || {})[args.fid]))){
+		switch(sMethod){
+			case 'move':
+				//check if id is in the range
+				if (!(Is.Array(args.iid) && args.iid.some(function(iid){
+					return !!dataSet.get(this.__sDataset,[this.__aid, this.__fid, iid]);
+				}, this))){
+					return;
 				}
+			case 'remove':
+			default:
+				this._serverSort();
+		}
+	}
+};
 
-				var nStartTime = (arg['starttime'] >= 0) ? arg['starttime'] / 60 : -1,
-						nEndTime = (arg['starttime'] >= 0) ? (arg['starttime'] / 60) + 30 : -1,
-						date = IcewarpDate.julian(arg['startdate']);
+_me._dblclick = function (e, elm, arg) {
+	switch (arg._event) {
+		case 'event':
+			var oRepeating = dataSet.get(this.__sDataset || 'items', [this.__aid, this.__fid, arg['id']]);
+			Item.open([this.__aid, this.__fid, WMItems._stripPipe(arg['id'])], oRepeating);
+			break;
 
-				aMenu = [
-					{"title": 'POPUP_ITEMS::NEW', 'arg': [Item.createInFolder, [arg['aid'], arg['fid'], {
-									'EVNSTARTDATE': arg['startdate'],
-									'EVNSTARTTIME': nStartTime,
-									'EVNENDDATE': arg['enddate'],
-									'EVNENDTIME': nEndTime
-								}]], 'disabled': !this.__rights.write}
-				];
-				break;
+		case 'allday':
+		case 'blank':
+			if (!this.calendar._rights().write)
+				return;
 
-			case 'selection':
-				var nStartTime;
+			arg['enddate'] = arg['startdate'];
 
-				if (arg['starttime'] >= 0) {
-					nStartTime = arg['starttime'] / 60;
-				} else {
-					nStartTime = -1;
-					arg['enddate']++;
-				}
+			if (arg._event == 'allday')
+				arg['enddate']++;
 
-				var nEndTime = (arg['endtime'] >= 0) ? arg['endtime'] / 60 : -1;
+			var nStartTime = (arg['starttime'] >= 0) ? arg['starttime'] / 60 : -1;
+			var nEndTime = (arg['starttime'] >= 0) ? (arg['starttime'] / 60) + 30 : -1;
 
-				aMenu = [{"title": 'POPUP_ITEMS::NEW', 'arg': [Item.createInFolder, [arg['aid'], arg['fid'], {
-									'EVNSTARTDATE': arg['startdate'],
-									'EVNSTARTTIME': nStartTime,
-									'EVNENDDATE': arg['enddate'],
-									'EVNENDTIME': nEndTime
-								}]], 'disabled': !this.__rights.write}];
+			if (arg._event == 'blank' && nEndTime == 1440){
+				nEndTime = 0;
+				arg['enddate']++;
+			}
+
+			var aInterval = {'EVNSTARTDATE': arg['startdate'], 'EVNSTARTTIME': nStartTime, 'EVNENDDATE': arg['enddate'], 'EVNENDTIME': nEndTime};
+			Item.openwindow([this.__aid, this.__fid], aInterval);
+
+			break;
+	}
+};
+
+_me._contextmenu = async function (e, elm, arg) {
+	var aMenu = '',
+		cmenu, i,
+		nStartTime, nEndTime;
+
+	arg['aid'] = this.__aid;
+	arg['fid'] = this.__fid;
+
+	var folder = arg.fid;
+	if (!this.calendar._rights().write) {
+		folder = Mapping.getDefaultFolderForGWType('E');
+	}
+
+	switch (arg._event) {
+		case 'allday':
+		case 'blank':
+			arg['enddate'] = arg['startdate'];
+
+			if (arg._event == 'allday') {
+				arg['enddate']++;
+			}
+
+			nStartTime = (arg['starttime'] >= 0) ? arg['starttime'] / 60 : -1;
+			nEndTime = (arg['starttime'] >= 0) ? (arg['starttime'] / 60) + 30 : -1;
+			var date = IcewarpDate.julian(arg['startdate']);
+
+			aMenu = [
+				{"title": 'POPUP_ITEMS::NEW', 'arg': [Item.createInFolder, [arg['aid'], folder, {
+								'EVNSTARTDATE': arg['startdate'],
+								'EVNSTARTTIME': nStartTime,
+								'EVNENDDATE': arg['enddate'],
+								'EVNENDTIME': nEndTime
+							}]]}
+			];
+			break;
+
+		case 'selection':
+			if (arg['starttime'] >= 0) {
+				nStartTime = arg['starttime'] / 60;
+			} else {
+				nStartTime = -1;
+				arg['enddate']++;
+			}
+
+			nEndTime = (arg['endtime'] >= 0) ? arg['endtime'] / 60 : -1;
+
+			aMenu = [{"title": 'POPUP_ITEMS::NEW', 'arg': [Item.createInFolder, [arg['aid'], folder, {
+								'EVNSTARTDATE': arg['startdate'],
+								'EVNSTARTTIME': nStartTime,
+								'EVNENDDATE': arg['enddate'],
+								'EVNENDTIME': nEndTime
+							}]]}];
 
 
-				//DELETE SELECTION 1
-				if (!Is.Empty(arg['contains'])) {
-					for (var i in arg['contains']) {
-						if (arg['contains'][i].indexOf('|') < 0) {
-							aMenu.push({"title": '-'}, {"title": 'POPUP_ITEMS::DELETE_SELECTED', 'arg': [Item.remove, [[arg['aid'], arg['fid'], arg['contains']]]], 'disabled': !this.__rights.remove});
-							break;
-						}
-					}
-				}
-
-				break;
-
-			case 'event':
-
-				var origItem = me._stripPipe(arg['id']),
-					id = [arg['aid'], arg['fid'], origItem],
-					ids = [arg['aid'], arg['fid'], [origItem]],
-					itmData = dataSet.get(this.__sDataset || 'items', [arg['aid'], arg['fid'], arg['id']]);
-
-				if (Item.hasReccurence([arg['aid'], arg['fid'], arg['id']],void 0,itmData)) {
-					var oRepeating = dataSet.get('items', [arg['aid'], arg['fid'], arg['id']]);
-				}
-				var itmRights = {'read': true, 'write': true, 'modify': true, 'remove': true, 'owner': true};
-				// overwrite rights if the event is not user's
-				if (arg.owner != sPrimaryAccountGWID) {
-//					itmRights = WMFolders.getAccess({'aid': sPrimaryAccount, 'fid': arg.evnfolder});
-					itmRights = this.__rights;
-				}
-
-				cmenu = gui._create("cmenu", "obj_context_item");
-				aMenu = cmenu.__createGWMenu(id, ids, 'E', false, oRepeating, itmRights,'','','',itmData);
-
-				var nStartTime = (arg['starttime'] >= 0) ? arg['starttime'] / 60 : -1,
-						nEndTime = (arg['starttime'] >= 0) ? (arg['starttime'] / 60) + 30 : -1;
-
-				for (var i in aMenu) {
-					if (aMenu[i].title == 'MAIN_MENU::NEW') {
-						aMenu[i].arg = [Item.createInFolder, [arg['aid'], arg['fid'], {
-									'EVNSTARTDATE': arg['startdate'],
-									'EVNSTARTTIME': nStartTime,
-									'EVNENDDATE': arg['enddate'],
-									'EVNENDTIME': nEndTime
-								}]];
-
+			//DELETE SELECTION 1
+			if (!Is.Empty(arg['contains'])) {
+				for (i in arg['contains']) {
+					if (arg['contains'][i].indexOf('|') < 0) {
+						aMenu.push({"title": '-'}, {"title": 'POPUP_ITEMS::DELETE_SELECTED', 'arg': [Item.remove, [[arg['aid'], arg['fid'], arg['contains']]]], 'disabled': !this.calendar._rights().remove});
 						break;
 					}
 				}
+			}
 
-				break;
-		}
+			break;
 
-		if (aMenu) {
+		case 'event':
 
-			if (arg._event != 'event') {
-				aMenu.push(
-						{"title": '-'},
-						{"title": 'MAIN_MENU::DAY_VIEW', 'arg': [gui.frm_main, '_selectView', [{aid: me.__aid, fid: me.__fid}, 'day_view', true, date]]},
-						{"title": 'MAIN_MENU::WEEK_VIEW', 'arg': [gui.frm_main, '_selectView', [{aid: me.__aid, fid: me.__fid}, 'week_view', true, date]]}
-				);
+			var origItem = WMItems._stripPipe(arg['id']),
+				id = [arg['aid'], arg['fid'], origItem],
+				ids = [arg['aid'], arg['fid'], [origItem]],
+				itmData = dataSet.get(this.__sDataset || 'items', [arg['aid'], arg['fid'], arg['id']]);
 
-				if (me._hasWorkView) {
-					aMenu.push({"title": 'MAIN_MENU::WORKWEEK_VIEW', 'arg': [gui.frm_main, '_selectView', [{aid: me.__aid, fid: me.__fid}, 'workweek_view', true, date]]});
+			if (Item.hasReccurence([arg['aid'], arg['fid'], arg['id']],void 0,itmData)) {
+				var oRepeating = dataSet.get('items', [arg['aid'], arg['fid'], arg['id']]) || dataSet.get('items', [arg.aid, '__@@VIRTUAL@@__/__@@EVENTS@@__', arg.id]);
+			}
+
+			var itmRights = {'read': true, 'write': true, 'modify': true, 'remove': true, 'owner': true};
+
+			// overwrite rights if the event is not user's
+			if (arg.owner != sPrimaryAccountGWID) {
+				itmRights = this.calendar._rights();
+			}
+
+			cmenu = await gui._create("cmenu", "obj_context_item");
+			aMenu = cmenu.__createGWMenu(id, ids, 'E', false, oRepeating, itmRights,'','','',itmData);
+
+			nStartTime = (arg['starttime'] >= 0) ? arg['starttime'] / 60 : -1;
+			nEndTime = (arg['starttime'] >= 0) ? (arg['starttime'] / 60) + 30 : -1;
+
+			for (i in aMenu) {
+				if (aMenu[i].title == 'MAIN_MENU::NEW') {
+					aMenu[i].arg = [Item.createInFolder, [arg['aid'], arg['fid'], {
+								'EVNSTARTDATE': arg['startdate'],
+								'EVNSTARTTIME': nStartTime,
+								'EVNENDDATE': arg['enddate'],
+								'EVNENDTIME': nEndTime
+							}]];
+
+					break;
 				}
-
-				aMenu.push(
-						{"title": 'MAIN_MENU::MONTH_VIEW', 'arg': [gui.frm_main, '_selectView', [{aid: me.__aid, fid: me.__fid}, 'month_view', true, date]]},
-						{"title": 'MAIN_MENU::EVENTS_LIST', 'arg': [gui.frm_main, '_selectView', [{aid: me.__aid, fid: me.__fid}, 'list_view', true]]}
-				);
 			}
 
-			if (!cmenu) {
-				cmenu = gui._create("cmenu", "obj_context_item");
+			break;
+	}
+
+	if (aMenu) {
+
+		if (arg._event != 'event') {
+			aMenu.push(
+					{"title": '-'},
+					{"title": 'MAIN_MENU::DAY_VIEW', 'arg': [gui.frm_main, '_selectView', [{aid: this.__aid, fid: this.__fid}, 'day_view', true, date]]},
+					{"title": 'MAIN_MENU::WEEK_VIEW', 'arg': [gui.frm_main, '_selectView', [{aid: this.__aid, fid: this.__fid}, 'week_view', true, date]]}
+			);
+
+			if (this._hasWorkView) {
+				aMenu.push({"title": 'MAIN_MENU::WORKWEEK_VIEW', 'arg': [gui.frm_main, '_selectView', [{aid: this.__aid, fid: this.__fid}, 'workweek_view', true, date]]});
 			}
-			cmenu._fill(aMenu);
-			cmenu._place(e.clientX, e.clientY);
-		} else {
-			cmenu._destruct();
+
+			aMenu.push(
+					{"title": 'MAIN_MENU::MONTH_VIEW', 'arg': [gui.frm_main, '_selectView', [{aid: this.__aid, fid: this.__fid}, 'month_view', true, date]]},
+					{"title": 'MAIN_MENU::EVENTS_LIST', 'arg': [gui.frm_main, '_selectView', [{aid: this.__aid, fid: this.__fid}, 'list_view', true]]}
+			);
 		}
-	};
+
+		if (!cmenu) {
+			cmenu = await gui._create("cmenu", "obj_context_item");
+		}
+		await cmenu._fill(aMenu);
+		cmenu._place(e.clientX, e.clientY);
+	} else {
+		cmenu._destruct();
+	}
 };
 
-
-_me._showsearch = function (aFolder, bFocus) {
+_me._showsearch = function (aFolder, sSearch) {
 	if (aFolder) {
+		if (sSearch === '') {
+			gui.frm_main.search.__lastQuery.E = '';
+		}
 		gui.frm_main.search._setFolder(aFolder);
-		gui.frm_main.search._deactivate();
 
 		gui.frm_main.search._onsearch = function (v) {
-			gui.frm_main._selectView(aFolder, 'list_view', true, '', '', this._value());
+			if (v) {
+				gui.frm_main.__currentView = gui.frm_main.__currentView || Cookie.get(['views', aFolder.aid, aFolder.fid, 'view']);
+
+				Cookie.set(['views', aFolder.aid, aFolder.fid, 'sort'], {
+					column: 'EVENT_STARTDATE',
+					type: 'desc'
+				});
+				gui.frm_main._selectView(aFolder, 'list_view', true, '', '', v);
+			}
 		};
-		gui.frm_main.search._focus(bFocus);
 	}
 };
 
 
-_me._print = function (bCalendar) {
+_me._print = async function (bCalendar) {
 
 	this.__getFolder();
-	var range = this.calendar._range();
+	var range = await this.calendar._range();
 
 	//Print Calendar view as it is
 	if (bCalendar && this.calendar) {
@@ -225,22 +270,35 @@ _me._print = function (bCalendar) {
 
 		document.body.appendChild(frame);
 
-		var doc = frame.contentDocument || frame.contentWindow.document,
-				html = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">" +
-				'<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">' +
-				"\n<head>\n" +
-				'<meta http-equiv="x-dns-prefetch-control" content="off" />' +
-				'<base href="' + document.location.protocol + '//' + document.location.hostname + (document.location.port ? ':' + document.location.port : '') + document.location.pathname + '" />' +
-				"\n</head>\n<body></body>\n</html>";
+		var doc = frame.contentDocument || frame.contentWindow.document;
 
-		doc.open('text/html', 'replace');
-		doc.write(html);
+		doc.write('<head/><body/>');
+
+		doc.head.appendChild(mkElement('meta', {
+			'http-equiv': 'x-dns-prefetch-control',
+			content: 'off'
+		}, doc));
+		doc.head.appendChild(mkElement('base', {
+			href: document.location.protocol + '//' + document.location.hostname + (document.location.port ? ':' + document.location.port : '') + document.location.pathname
+		}, doc));
 
 		//STYLES
-		doc.getElementsByTagName('head')[0].appendChild(mkElement('link', {type: "text/css", rel: "stylesheet", href: "client/skins/index.css"}, doc));
-		doc.getElementsByTagName('head')[0].appendChild(mkElement('link', {type: "text/css", rel: "stylesheet", href: 'client/skins/' + GWOthers.getItem('LAYOUT_SETTINGS', 'skin') + '/css/css.php?' + buildURL({skin: GWOthers.getItem('LAYOUT_SETTINGS', 'skin'), palette: GWOthers.getItem('LAYOUT_SETTINGS', 'skin_style'), file: 'style.css'})}, doc));
+		doc.head.appendChild(mkElement('link', {type: "text/css", rel: "stylesheet", href: "client/skins/index.css", onload: onLoad}, doc));
+		doc.head.appendChild(mkElement('link', {type: "text/css", rel: "stylesheet", href: getCssPath('style.css'), onload: onLoad}, doc));
 		//print calendar styles
-		doc.getElementsByTagName('head')[0].appendChild(mkElement('link', {type: "text/css", rel: "stylesheet", href: 'client/skins/' + GWOthers.getItem('LAYOUT_SETTINGS', 'skin') + '/css/css.php?' + buildURL({skin: GWOthers.getItem('LAYOUT_SETTINGS', 'skin'), palette: GWOthers.getItem('LAYOUT_SETTINGS', 'skin_style'), file: 'print_calendar.css'})}, doc));
+		doc.head.appendChild(mkElement('link', {type: "text/css", rel: "stylesheet", href: getCssPath('print_calendar.css'), onload: onLoad}, doc));
+
+		var toLoad = 3;
+		function onLoad() {
+			if(!--toLoad) {
+				frame.contentWindow.focus();
+				if (frame.contentWindow.document.queryCommandSupported('print')) {
+					frame.contentWindow.document.execCommand('print', false, null);
+				} else {
+					frame.contentWindow.print();
+				}
+			}
+		}
 
 		//HEADER
 		var header = mkElement('div', {className: 'print_calendar'}, doc);
@@ -259,12 +317,13 @@ _me._print = function (bCalendar) {
 			var cf = dataSet.get('folders', [this.__aid, this.__fid, 'VIRTUAL', 'FOLDERS']);
 			if (cf) {
 				for (var i in cf) {
-					cal.push({name: i, color: getCalendarColor(i.str_replace('\\', '/'))});
+					var color = getCalendarColor(Path.slash(i));
+					cal.push({name: i, style: '--bgcolor: ' + (calendarPalette[color] || color)});
 				}
 			}
 		}
 
-		header.innerHTML = template.tmp('print_calendar', {date: dstr, calendars: cal});
+		header.innerHTML = await (new cTemplate()).tmp('print_calendar', {date: dstr, calendars: cal});
 
 		//BODY
 		doc.body.appendChild(header);
@@ -279,9 +338,10 @@ _me._print = function (bCalendar) {
 			var times = tree.getElementsByClassName('tline');
 
 			// Calculate beginning of first, and end of last, event
-			var i = events.length;
+			i = events.length;
+			var day;
 			while (i--) {
-				var day = original_events[i];
+				day = original_events[i];
 				if (day) {
 					var y = parseInt(day.offsetTop);
 					var t = Math.floor(y / 52);
@@ -292,9 +352,9 @@ _me._print = function (bCalendar) {
 			}
 			top = Math.floor(top);
 			// Move events up to new first hour
-			var i = events.length;
+			i = events.length;
 			while (i--) {
-				var day = events[i];
+				day = events[i];
 				if (day) {
 					day.style.top = parseInt(day.style.top) - top * 52 + 'px';
 				}
@@ -311,29 +371,13 @@ _me._print = function (bCalendar) {
 			while (times[0] && top--) {
 				times[0].removeChild(times[0].children[0]);
 			}
-			// Remove dimming of work hours
-			times[0].parentNode.removeChild(tree.getElementsByClassName('time1')[0]);
-			times[0].parentNode.removeChild(tree.getElementsByClassName('time2')[0]);
+			var ct = this._main.getElementsByClassName('current_time')[0];
+			ct && ct.parentNode.removeChild(ct);
 		}
 		var add = tree.querySelector('.obj_evnviewblock');
 		add && add.parentNode.removeChild(add);
 		doc.body.appendChild(tree);
 
-		doc.body.onload = function () {
-			frame.contentWindow.focus();
-			if (frame.contentWindow.document.queryCommandSupported('print')) {
-				frame.contentWindow.document.execCommand('print', false, null);
-			} else {
-				frame.contentWindow.print();
-			}
-
-			setTimeout(function () {
-//				frame.parentNode.removeChild(frame);
-			}, 1000);
-		};
-
-		doc.close();
-
 	}
 	//Print list of events
 	else {
@@ -342,10 +386,10 @@ _me._print = function (bCalendar) {
 		WMItems.list({"aid": this.__aid, "fid": this.__fid, "values": aValues, "filter": {'interval': range['start'] + '-' + range['end']}}, '', '', '', [this, '__print']);
 	}
 };
-_me.__print = function (aData) {
+_me.__print = async function (aData) {
 
 	var tmp = [];
-//gui._create('print','frm_print');
+//await gui._create('print','frm_print');
 	for (var i in aData)
 		for (var j in aData[i]) {
 			for (var k in aData[i][j])
@@ -381,8 +425,10 @@ _me.__print = function (aData) {
 				return x;
 			});
 
-			for (var i = 0; i < tmp.length; i++)
-				(gui.print || gui._create('print', 'frm_print'))._add('E', tmp[i]);
+			for (var l = 0; l < tmp.length; l++) {
+				gui.print = gui.print || await gui._create('print', 'frm_print');
+				gui.print._add('E', tmp[l]);
+			}
 
 			return;
 		}
@@ -398,44 +444,51 @@ _me.__update = function ()
 	}
 };
 
-_me._serverSort = function (sView)
+_me._serverSort = async function ()
 {
 	this.__getFolder();
+	var range = await this.calendar._range();
 
-	var range = this.calendar._range();
+	if (this.__lastFolder.aid !== this.__aid || this.__lastFolder.fid !== this.__fid || this.__lastFolder.start !== range.start || this.__lastFolder.end !== range.end) {
+		this.__lastFolder = {
+			aid: this.__aid,
+			fid: this.__fid,
+			start: range.start,
+			end: range.end
+		};
+		await this.calendar._create('loader', 'obj_loader', 'self');
+		dataSet.add('main_calendar', '', [], true);
+		dataSet.update('main_calendar');
+	}
 
-	if (!Is.Defined(range['start'])) {
+	if (!Is.Defined(range.start)) {
 		//set rights for calendar object
 		this.calendar._rights(WMFolders.getAccess({'aid': this.__aid, 'fid': this.__fid}));
 		return;
 	}
 
-	var aValues = WMItems.default_values('EI');
-	WMItems.list({"aid": this.__aid, "fid": this.__fid, "values": aValues, "filter": {'interval': range['start'] + '-' + range['end']}}, 'items');
-
 	if(!this.calendar_view) {
-		this._create("calendar_view", "obj_hmenu_calendar_view", 'change_view');
+		await this._create("calendar_view", "obj_hmenu_calendar_view", 'change_view', 'obj_hmenu_toolbar');
 	}
 	this.calendar_view.__menu({aid: this.__aid, fid: this.__fid});
+
+	var aValues = WMItems.default_values('EI');
+	clearTimeout(this.__xhrTimeout);
+	this.__xhrTimeout = setTimeout(async function() {
+		this.__listUID = unique_id();
+		this.__xhr = await WMItems.list({
+			aid: this.__aid,
+			fid: this.__fid,
+			values: aValues,
+			filter: {
+				interval: range.start + '-' + range.end
+			}
+		}, 'items');
+	}.bind(this), 50);
 };
 
 _me._onItemAdd = function (aValues) {
 
-	/*
-	 aResultValues = {values:{
-	 'EVNTITLE':aValues.title,
-	 'EVNSHARETYPE':GWOthers.getItem('DEFAULT_CALENDAR_SETTINGS','event_sharing'),
-	 'EVNFLAGS':{F:4,T:8,O:16,S:0}[GWOthers.getItem('DEFAULT_CALENDAR_SETTINGS','event_show_as')] || 0,
-
-	 '_TZEVNSTARTTIME':aValues.starttime<0?-1:aValues.starttime/60,
-	 '_TZEVNENDTIME':aValues.endtime<0?-1:aValues.endtime/60,
-	 '_TZEVNSTARTDATE':aValues.startdate,
-	 '_TZEVNENDDATE':aValues.enddate,
-	 '_TZID': GWOthers.getItem('CALENDAR_SETTINGS','timezone'),
-	 'EVNTIMEFORMAT':'Z'
-	 }};
-	 */
-
 	var aResultValues = {
 		'EVNTITLE': aValues.title,
 		'EVNSHARETYPE': GWOthers.getItem('DEFAULT_CALENDAR_SETTINGS', 'event_sharing'),
@@ -453,7 +506,7 @@ _me._onItemAdd = function (aValues) {
 		aResultValues['_TZEVNENDTIME'] = aValues.endtime / 60;
 		aResultValues['_TZEVNSTARTDATE'] = aValues.startdate;
 		aResultValues['_TZEVNENDDATE'] = aValues.enddate;
-		aResultValues['_TZID'] = GWOthers.getItem('CALENDAR_SETTINGS', 'timezone');
+		aResultValues['_TZID'] = normalizeTimezone(GWOthers.getItem('CALENDAR_SETTINGS', 'timezone'));
 		aResultValues['EVNTIMEFORMAT'] = 'Z';
 	}
 	aResultValues = {values: aResultValues};
@@ -489,16 +542,16 @@ _me._onItemAdd = function (aValues) {
 	if (aReminderDefault && (aReminderDefault = aReminderDefault['VALUES']) && aReminderDefault.default_reminder == '1') {
 
 		var time = aReminderDefault.time,
-				days = 0,
-				hours = 0,
-				minutes = 0;
+			days = 0,
+			hours = 0,
+			minutes = 0;
 
 		if (time > 0) {
-			var days = Math.floor(time / 86400000);
+			days = Math.floor(time / 86400000);
 			time %= 86400000;
-			var hours = Math.floor(time / 3600000);
+			hours = Math.floor(time / 3600000);
 			time %= 3600000;
-			var minutes = Math.floor(time / 60000);
+			minutes = Math.floor(time / 60000);
 		}
 
 		aResultValues.REMINDERS = [{values: {
@@ -508,7 +561,7 @@ _me._onItemAdd = function (aValues) {
 				}}];
 	}
 
-	WMItems.add([this.__aid, this.__fid], aResultValues, 'dummy', '', '', [this, '__addEventHandler', [this.__aid, this.__fid, true]]);
+	WMItems.add([this.__aid, this.__fid], aResultValues, '', '', '', [this, '__addEventHandler', [this.__aid, this.__fid, true]]);
 };
 /*
  nEditType:
@@ -520,7 +573,7 @@ _me._onItemChange = function (nEditType, aValues, bIgnoreRepeating, bRefresh) {
 
 	var sAccId = this.__aid;
 	var sFolId = this.__fid;
-	var sItId = aValues['id'] ? this._stripPipe(aValues['id']) : '';
+	var sItId = aValues['id'] ? WMItems._stripPipe(aValues['id']) : '';
 	var bRepeats = sItId ? Item.hasReccurence([sAccId, sFolId, aValues['id']]) : false;
 
 	// New - Do not ask, just apply "Only this instance" option
@@ -528,19 +581,6 @@ _me._onItemChange = function (nEditType, aValues, bIgnoreRepeating, bRefresh) {
 		nEditType = 0;
 	}
 
-	/* Old
-	 if (bRepeats && !bIgnoreRepeating){
-	 // invoke dialog to confirm type of the repeating (only this item or all items) and callback this function again
-	 var cfm = gui._create('frm_confirm','frm_confirm_repeating', '','', [this, '_onItemChange', [aValues, true]],'REPEATING_CONFIRM::TITLE_EDIT','REPEATING_CONFIRM::TEXT_EDIT_2');
-	 var me = this;
-	 cfm._onclose = function(){
-	 me.__refresh();
-	 return true;
-	 };
-	 return;
-	 }
-	 */
-
 	var nStartTime = (aValues['starttime'] >= 0) ? aValues['starttime'] / 60 : -1;
 	var nEndTime = (aValues['endtime'] >= 0) ? aValues['endtime'] / 60 : -1;
 
@@ -563,7 +603,7 @@ _me._onItemChange = function (nEditType, aValues, bIgnoreRepeating, bRefresh) {
 			//this instance
 		case '0':
 			if (bRepeats) {
-				var oRepeating = dataSet.get('items', [sAccId, sFolId, aValues['id']]);
+				var oRepeating = dataSet.get(this.__sDataset, [sAccId, sFolId, aValues['id']]);
 				aResultValues['EXPDATE'] = oRepeating['EVNSTARTDATE'];
 				aResultValues['EVNSTARTDATE'] = aValues['startdate'];
 				aResultValues['EVNENDDATE'] = (nStartTime >= 0) ? aValues['enddate'] : aValues['enddate'] + 1;
@@ -581,7 +621,7 @@ _me._onItemChange = function (nEditType, aValues, bIgnoreRepeating, bRefresh) {
 				aResultValues['EVNSTARTDATE'] = aValues['startdate'];
 				aResultValues['EVNENDDATE'] = (nStartTime >= 0) ? aValues['enddate'] : aValues['enddate'] + 1;
 			} else {
-				var oRepeating1 = dataSet.get('items', [sAccId, sFolId, aValues['id']]);
+				var oRepeating1 = dataSet.get(this.__sDataset, [sAccId, sFolId, aValues['id']]);
 
 				aResultValues['EVNSTARTDATE'] = parseInt(aValues.osd) + parseInt(aValues.startdate) - parseInt(oRepeating1.EVNSTARTDATE);
 				aResultValues['EVNENDDATE'] = parseInt(aValues.oed) + parseInt(aValues.enddate) - parseInt(oRepeating1.EVNENDDATE);
@@ -592,10 +632,6 @@ _me._onItemChange = function (nEditType, aValues, bIgnoreRepeating, bRefresh) {
 	WMItems.add([sAccId, sFolId, sItId], {"values": aResultValues}, 'dummy', '', '', [this, '__addEventHandler', [sAccId, sFolId, bRepeats, bRefresh]]);
 };
 
-_me._stripPipe = function (s) {
-	return s.split('|')[0];
-};
-
 _me._createEvent = function ()
 {
 	var aSelection = clone(this.calendar._selection);
@@ -614,7 +650,7 @@ _me._createEvent = function ()
 		var aInterval = {'EVNSTARTDATE': aSelection['startdate'], 'EVNSTARTTIME': nStartTime, 'EVNENDDATE': aSelection['enddate'], 'EVNENDTIME': nEndTime};
 		Item.openwindow([this.__aid, this.__fid], aInterval);
 	} else {
-		Item.openwindow([this.__aid, this.__fid], getActualEventTime());
+		Item.openwindow([this.__aid, this.__fid], getCurrentEventTime());
 	}
 };
 
@@ -628,26 +664,17 @@ _me._getDate = function () {
 	}
 };
 
-
-_me.__onServerResponse = function (sAccId, sFolId, nStart, nEnd)
-{
-	if (!Is.Defined(this.calendar))
-		return;
-	var aRange = this.calendar._range();
-
-	if (this.__aid == sAccId && this.__fid == sFolId && aRange['start'] == nStart && aRange['end'] == nEnd)
-		this.__refresh();
-};
-
 _me.__refresh = function () {
 	var aValues = dataSet.get('items', [this.__aid, this.__fid]),
-			aFillData = [];
+		aFillData = [],
+		sDefaultFolder = Mapping.getDefaultFolderForGWType('E');
 
 	for (var i in aValues) {
 		if (i === '/' || i === '#' || i === '$' || i === '@') {
 			continue;
 		}
 		try {
+			var fcolor = (this.__aid == sPrimaryAccount && (this.__fid == '__@@VIRTUAL@@__/__@@EVENTS@@__' || this.__fid == sDefaultFolder) && aValues[i]['EVNFOLDER'] ? getCalendarColor(Path.slash(aValues[i]['EVNFOLDER'])) : '');
 			aFillData.push({
 				'starttime': (aValues[i]['EVNSTARTTIME'] >= 0) ? aValues[i]['EVNSTARTTIME'] * 60 : -1,
 				'endtime': (aValues[i]['EVNENDTIME'] >= 0) ? aValues[i]['EVNENDTIME'] * 60 : -1,
@@ -668,47 +695,26 @@ _me.__refresh = function () {
 				'owner': aValues[i]['EVNOWN_ID'],
 				'evnfolder': aValues[i]['EVNFOLDER'],
 				'evnowneremail': aValues[i]['EVNOWNEREMAIL'],
-				'fcolor': (this.__aid == sPrimaryAccount && this.__fid == '__@@VIRTUAL@@__/__@@EVENTS@@__' && aValues[i]['EVNFOLDER'] ? getCalendarColor(Path.slash(aValues[i]['EVNFOLDER'])) : ''),
+				'fcolor': calendarPalette[fcolor] || fcolor,
 				'id': i
 			});
 		} catch (e) {
-			console.log(e);
+			 console.log(e);
 		}
 	}
 
 	dataSet.add('main_calendar', '', aFillData, true);
 	dataSet.update('main_calendar');
+	this.calendar.loader && this.calendar.loader._destruct();
 };
 
-_me.__addEventHandler = function (bOk, sAccId, sFolId, bRec, bRefresh) {
-
+_me.__addEventHandler = function (bOk) {
 	if (bOk == false) {
 		this.__update();
 	}
-	if (bRec || bRefresh) {
-		this._serverSort();
-	}
-};
-
-_me.__getStartDay = function () {
-	storage.library('gw_others');
-	var aCalendarSettings = (new gw_others).get('CALENDAR_SETTINGS', 'storage')['VALUES'];
-
-	if (aCalendarSettings['begin_on_today'] != '0') {
-		return (new IcewarpDate()).day();
-	} else {
-		if (Is.Defined(aCalendarSettings['week_begins'])) {
-			var aDays = {'sunday': 0, 'monday': 1, 'tuesday': 2, 'wednesday': 3, 'thursday': 4, 'friday': 5, 'saturday': 6};
-			return aDays[aCalendarSettings['week_begins']];
-		} else {
-			return 0;
-		}
-	}
 };
 
 _me.__getWorkingHours = function () {
-	storage.library('gw_others');
-
 	var sTime = {};
 	sTime['start'] = GWOthers.getItem('CALENDAR_SETTINGS', 'day_begins');
 	sTime['end'] = GWOthers.getItem('CALENDAR_SETTINGS', 'day_ends');
@@ -717,15 +723,6 @@ _me.__getWorkingHours = function () {
 };
 
 _me.__getFolder = function () {
-	// var aItems = dataSet.get(this.__sDataset);
-
-	// for (var sAccId in aItems) {
-	// 	for (var sFolId in aItems[sAccId]) {
-	// 		this.__aid = sAccId;
-	// 		this.__fid = sFolId;
-	// 		return;
-	// 	}
-	// }
 	var path = Path.split(dataSet.get('current_folder'), true);
 	this.__aid = path.aid;
 	this.__fid = path.fid;
