@@ -1,11 +1,10 @@
 _me = frm_main_calendar_month.prototype;
 function frm_main_calendar_month() {};
 
-_me.__constructor = function (sDataset, _date) {
+_me.__constructor = async function (sDataset, _date) {
 	this.__date = new IcewarpDate(_date);
 
 	var me = this;
-	this._main.querySelector('.frmtbl tr').appendChild(mkElement('td', {class:"alternative_calendar"}));
 
 	this.inp_month._ondateselect = function () {
 		if (this._value() != me.__date.format(IcewarpDate.JULIAN)) {
@@ -31,12 +30,24 @@ _me.__constructor = function (sDataset, _date) {
 		me._updateLabel();
 	};
 
-	this._updateLabel(true);
+	await this._updateLabel(true);
+
+	this.refresh._onclick = function() {
+		gui.frm_main._getNew(function() {
+			if (this._destructed) {
+				return;
+			}
+			this.refresh._disabled(false);
+			this.refresh._main.classList.remove('loading');
+		}.bind(this));
+		this.refresh._disabled(true);
+		this.refresh._main.classList.add('loading');
+	}.bind(this);
 };
 
-_me._updateLabel = function (bSkip) {
+_me._updateLabel = async function (bSkip) {
 	this.inp_month._value(this.__date, true);
-	this.calendar._range(this.__date.format(IcewarpDate.JULIAN));
+	await this.calendar._range(this.__date.format(IcewarpDate.JULIAN));
 
 	if (!bSkip)
 		this._serverSort();
