@@ -1,17 +1,21 @@
 _me = frm_ok_cancel.prototype;
 function frm_ok_cancel(){};
 
-_me.__constructor = function(){
+_me.__constructor = async function(){
 
 	var me = this;
 
     // Create 'OK' button
-	this._create('x_btn_ok', 'obj_button', 'footer', 'ok noborder color1');
+	await this._create('x_btn_ok', 'obj_button', 'footer', 'ok noborder color1');
 	this.x_btn_ok._value('FORM_BUTTONS::OK');
 	this.x_btn_ok._onclick = function() { me._destruct(); };
 
 	// Create 'CANCEL' button
-	this._create('x_btn_cancel', 'obj_button', 'footer', 'cancel noborder');
+	await this._create('x_btn_cancel', 'obj_button', 'footer', 'cancel noborder');
 	this.x_btn_cancel._value('FORM_BUTTONS::CANCEL');
 	this.x_btn_cancel._onclick = function() { me._destruct(); };
+
+	this.x_btn_cancel._main.insertAdjacentElement('afterend', mkElement('div', {
+		className: 'flex-row-filler'
+	}));
 };
