@@ -6,19 +6,20 @@ function frm_password(){};
  * 
  * @see  Options/security password
  */
-_me.__constructor = function(aResponse, sLabel1, sLabel2, sPlaceholder) {
+_me.__constructor = async function(aResponse, sLabel1, sLabel2, sPlaceholder) {
 
 	var me = this;
 
-	this._size(250,145,true);
     this._resizable(false);
 	this._dockable(false);
 	this._modal(true);
     
-    this._draw('frm_input','main');
-	this._create('obj_input','obj_password','input','obj_input_100');
+    await this._draw('frm_input','main');
+	await this._create('obj_input','obj_password','input','obj_input_100');
 
-    this.obj_input._focus();
+    setTimeout(function() {
+		me.obj_input._focus();
+	}, 5);
 	this.obj_input._onsubmit = function (){
 		me.x_btn_ok._onclick();
 	};
@@ -39,4 +40,5 @@ _me.__constructor = function(aResponse, sLabel1, sLabel2, sPlaceholder) {
 		executeCallbackFunction(aResponse,me.obj_input._value());
 		me._destruct();
 	};
+	this._size(300,'auto',true);
 };
