@@ -1,11 +1,11 @@
 _me = frm_revision.prototype;
 function frm_revision(){};
 
-_me.__constructor = function(aItemInfo,bFrm){
+_me.__constructor = async function(aItemInfo,bFrm){
 	var me = this;
 
 	this._title('ITEM::NEW_REVISION');
-	this._defaultSize(-1,-1,400,200);
+	this._defaultSize(500,400);
 	this._modal(true);
 
 	if (bFrm){
@@ -13,7 +13,7 @@ _me.__constructor = function(aItemInfo,bFrm){
 		me.x_btn_cancel._value('FORM_BUTTONS::SKIP');
 	}
 
-	this._create('text','obj_text','main','obj_text100 noborder');
+	await this._create('text','obj_text','main','obj_text100 noborder');
 	this.text._placeholder(getLang('COMMON::COMMENT'));
 	this.text._onsubmit = function(){
 		me.x_btn_ok._onclick();
