@@ -1,7 +1,7 @@
 _me = frm_select_account.prototype;
 function frm_select_account(){};
 
-_me.__constructor = function(aResponse){
+_me.__constructor = async function(aResponse){
 
 	this._modal(true);
 
@@ -10,7 +10,7 @@ _me.__constructor = function(aResponse){
 	this._title('SETTINGS::ACCOUNTS');
 	this._size(300,500,true);
 
-	this._create('tree_folder','obj_tree','main', 'folder_icons');
+	await this._create('tree_folder','obj_tree','main', 'folder_icons');
 
 	var aFolders = dataSet.get('folders',[sPrimaryAccount]),
 		aData = {};
