@@ -1,13 +1,13 @@
 _me = frm_send_message.prototype;
 function frm_send_message(){};
 
-_me.__constructor = function(aValues) {
+_me.__constructor = async function(aValues) {
 	var me = this;
 	this._size(400,330,true);
 	this._modal(true);
 
 	this._title('FILTERS::MESSAGE');
-	this._draw('frm_send_message', 'main');
+	await this._draw('frm_send_message', 'main');
 
 	var aTransformedValues = {};
 	for (var i in aValues)
