@@ -1,11 +1,14 @@
 _me = frm_signature_preview.prototype;
 function frm_signature_preview(){};
 
-_me.__constructor = function(sSignature) {
+_me.__constructor = async function(sSignature) {
 	var signatureContainer, me = this;
 
 
-	this._size(450,300,true);
+	this._defaultSize(900, 620, {
+		goldenRatio: true,
+		minHeight: 400
+	});
 	this._modal(true);
 	this._dockable(false);
 	this._resizable(true);
@@ -18,7 +21,7 @@ _me.__constructor = function(sSignature) {
 
 	this.__eMain.appendChild(signatureContainer);
 
-	this._create('x_btn_ok', 'obj_button', 'footer', 'noborder ok');
+	await this._create('x_btn_ok', 'obj_button', 'footer', 'noborder ok');
 	this.x_btn_ok._value('FORM_BUTTONS::CLOSE');
 	this.x_btn_ok._onclick = function() {
 		me._destruct();
