@@ -4,9 +4,9 @@
  @used:	frm_alert_suppress
 		frm_confirm_suppress
 */			
-function frm_suppress_check(sLabel2){};
-frm_suppress_check.prototype.__constructor = function(aResponse, sLabel1, sLabel2, aSubstitution){
-	return false;
+function frm_suppress_check(){};
+frm_suppress_check.prototype.__constructor = async function(aResponse, sLabel1, sLabel2){
+	// return false;
 
 	if (typeof sLabel2=="string" && sLabel2.indexOf('::')) {
 		var aPath = sLabel2.toLowerCase().split('::');
