@@ -6,7 +6,7 @@ function frm_text(){};
  * 
  * @see  IM/new group
  */
-_me.__constructor = function(aResponse, sLabel1, sLabel2) {
+_me.__constructor = async function(aResponse, sLabel1, sLabel2) {
 
 	var me = this;
 
@@ -15,8 +15,8 @@ _me.__constructor = function(aResponse, sLabel1, sLabel2) {
     this._dockable(false);
 	this._modal(true);
     
-	this._draw('frm_input','main');
-	this._create('obj_input','obj_text','input','obj_text100');	
+	await this._draw('frm_input','main');
+	await this._create('obj_input','obj_text','input','obj_text100');	
 
 	this.obj_input._onsubmit = function (){
 		me.x_btn_ok._onclick();
