@@ -1,24 +1,22 @@
 _me = frm_timezone.prototype;
 function frm_timezone(){};
 
-_me.__constructor = function(aHandler) {
+_me.__constructor = async function(aHandler) {
 	var me = this;
 	this._dockable(false);
 	this._resizable(false);
 	this._closable(false);
-	this._size(430,290,true);
+	this._modal(true);
 	this._title('SETTINGS::TIMEZONE');
 
-	this._draw ('frm_timezone','main');
-	this.tz._value(GWOthers.getItem('CALENDAR_SETTINGS', 'timezone'));
-
-	this.x_btn_ok._onclick = function() {
-
-		Cookie.set(['tzoffset'],(new IcewarpDate()).utcOffset() - (60 * new IcewarpDate().getMoment().isDST()));
+	await this._draw('frm_timezone','main');
+	this._size(430,'auto',true);
+	this.tz._value(normalizeTimezone(GWOthers.getItem('CALENDAR_SETTINGS', 'timezone')));
 
+	this.x_btn_ok._onclick = async function() {
 		dataSet.add('tmp_storage','',dataSet.get('storage','',true));
 		GWOthers.set('CALENDAR_SETTINGS',{timezone:me.tz._value()},'storage');
-		if (WMStorage.set({'resources':dataSet.get('storage')},'storage','',[me,'__saveItemsHandler']) == 2){
+		if (await WMStorage.set({'resources':dataSet.get('storage')},'storage','',[me,'__saveItemsHandler']) == 2){
 			dataSet.remove('tmp_storage');
 			me.x_btn_cancel._onclick();
 		}
@@ -28,6 +26,13 @@ _me.__constructor = function(aHandler) {
 		me._destruct();
 		executeCallbackFunction(aHandler);
 	};
+
+	this.tz._obeyEvent('show', [function(_, data) {
+		var zIndex = +getComputedStyle(me._main).zIndex;
+		if (zIndex) {
+			data.owner.block._zIndex(zIndex + 1);
+		}
+	}]);
 };
 
 _me.__saveItemsHandler = function(bOK)
