@@ -1,19 +1,19 @@
 function frm_verify_app(){};
 
 frm_verify_app.prototype = {
-    __constructor:function(view){
+    __constructor: async function(view){
         var next = 'ios',
             me = this;
 
         view.title('VERIFICATION::APP');
-        view.buttons({
+        await view.buttons({
             'btn_next':{value:'COMMON::NEXT', css:'color1', onclick:function(){
                 view.view('auth', next);
             }},
             'btn_cancel':{value:'FORM_BUTTONS::CANCEL'}
         });
 
-        function toggle (e){
+        function toggle (){
 
 			next = this.getAttribute('rel');
 
