@@ -1,11 +1,11 @@
 function frm_verify_code(){};
 
 frm_verify_code.prototype = {
-    __constructor:function(view, arg){
+    __constructor: async function(view){
 
         view.title('VERIFICATION::TWOSTEP');
 
-        view.buttons({
+        await view.buttons({
             'btn_next':{value:'COMMON::NEXT', css:'color1', onclick:function(){
                 view.view('auth');
             }},
@@ -28,6 +28,6 @@ frm_verify_code.prototype = {
 
 		//Ask for SMS
 		var img = new Image();
-			img.src = 'server/download.php?' + buildURL({'sid':dataSet.get('main',['sid']), 'class':'two_factor_qr', 'fullpath':'', 'rand':unique_id()});
+			img.src = sPrimaryAccountClient + 'server/download.php?' + buildURL({'dlsess':dataSet.get('main',['dlsess']), 'class':'two_factor_qr', 'fullpath':'', 'rand':unique_id()});
     }
 };
\ No newline at end of file
