@@ -1,10 +1,10 @@
 function frm_verify_main(){};
 
 frm_verify_main.prototype = {
-    __constructor:function(view){
+    __constructor: async function(view){
         view.data = {};
         view.title('VERIFICATION::TWOSTEP');
-        view.buttons({'btn_cancel':{value:'FORM_BUTTONS::CANCEL'}});
+        await view.buttons({'btn_cancel':{value:'FORM_BUTTONS::CANCEL'}});
 
         this._getAnchor('app').onclick = function(){
             WMAccounts.action({TWO_FACTOR_TYPE:0},'setup_two_factor', [response, ['app']]);
@@ -13,7 +13,7 @@ frm_verify_main.prototype = {
         if (sPrimaryAccount2F<2){
             addcss(this._getAnchor('sms'), 'disabled');
             var elm = this._getAnchor('sms').querySelector('span.info');
-            gui.tooltip._add(elm,getLang('VERIFICATION::SMS_DISABLED'), {hide:false, css:'dark pad'});
+            gui.tooltip._add(elm,getLang('VERIFICATION::SMS_DISABLED'), {hide:false, css:'dark pad wrap'});
         }
         else{
             this._getAnchor('sms').onclick = function(){
@@ -26,7 +26,7 @@ frm_verify_main.prototype = {
                 view.view(sType);
             }
             else{
-                console.warn('Unable to activate two-factor' + sType);
+                 console.warn('Unable to activate two-factor' + sType);
             }
         }
     }
