@@ -1,11 +1,11 @@
 function frm_verify_success(){};
 
 frm_verify_success.prototype = {
-    __constructor:function(view, arg){
+    __constructor: async function(view){
 
 		view.parent._enable2FE(true);
 
-        view.buttons({
+        await view.buttons({
             'btn_next':{value:'COMMON::DONE', css:'color1', onclick:function(){
                 view.parent._destruct();
             }}
@@ -15,6 +15,6 @@ frm_verify_success.prototype = {
             lbl: getLang('VERIFICATION::SUCCESS', ['<span class="bold">' + sPrimaryAccount + '</span>'])
         };
 
-        this._draw('frm_verify_success', 'main', aData);
+        await this._draw('frm_verify_success', 'main', aData);
     }
 };
\ No newline at end of file
