@@ -15,7 +15,7 @@
  * - PRINT as invisible
  * - exe JS
  *
- * gui.__focus = boolen; window focus
+ * gui.__focus = boolean; window focus
  *
  */
 
@@ -25,7 +25,7 @@
 
 */
 
-function Gui(sName,sType,oParent){
+function Gui(sName,sType,oParent,aDocument){
 
 	// PUBLIC
 	this._name		= sName;
@@ -60,7 +60,17 @@ function Gui(sName,sType,oParent){
 
 	//Call __root for main gui wrapper init
 	if (!sType && !oParent)
-		Gui.__root.call(this);
+		Gui.__root.call(this, aDocument);
+
+	Object.defineProperty(this, '_gui', {
+		get: function() {
+			var that = this;
+			while (!that._main && that._parent) {
+				that = that._parent;
+			}
+			return (that._main && that._main.parentNode) ? (that._main.ownerDocument.defaultView || {}).gui || gui : gui;
+		}
+	});
 };
 
 /**
@@ -69,26 +79,29 @@ function Gui(sName,sType,oParent){
  *
  *		@target: 	can be string or object	{target:<string>, first:<bool>}
  */
-Gui.prototype._create = function (sName,sType,target,sClass){
+Gui.prototype._create = async function (sName,sType,target,sClass){
+	sClass = sClass || '';
 
 	//Do not create child objects in destructed or removed object
 	if (this._destructed || (this._main && !this._main.parentNode))
 		return false;
 
 	// load object XML descriptor from storage cache
-	var sTarget, aObj = storage.object(sType);
+	var sTarget = 'main', i,
+		aObj = await storage.object(sType),
+		sObj;
 
 	// check parent type for allowed type
 	if (aObj.PARENTS){
 		var er = true;
-		for (var i in aObj.PARENTS[0].OBJ)
+		for (i in aObj.PARENTS[0].OBJ)
 			if (this._type == aObj.PARENTS[0].OBJ[i].VALUE){
 				er = false;
 				break;
 			}
 
 		if (er)
-			throw new Error("gui._create() -  OBJ "+ sType +"\n disallowed parent "+ this._type);
+			throw new Error("await gui._create() -  OBJ "+ sType +"\n disallowed parent "+ this._type);
 	}
 
 	// create name for new Gui instnce
@@ -97,13 +110,13 @@ Gui.prototype._create = function (sName,sType,target,sClass){
 			if (aObj.UNIQUE[0].VALUE == 'keep')
 				return this[sName];
 			else
-				this[sName]._destruct();
+				await this[sName]._destruct();
 		}
 	}
 	else{
 		//generate name
 		var sApx = '';
-		for(var i=0;;i++){
+		for(i=0;;i++){
 			if (this[sName+sApx] === undefined){
 				sName += sApx;
 				break;
@@ -114,28 +127,27 @@ Gui.prototype._create = function (sName,sType,target,sClass){
 	}
 
 	// create _main htmlobj in parent's anchor
-	if (Is.Object(target))
-		sTarget = target.target || 'main';
-	else
-	if (Is.Defined(target) && (target = target.toString()).length)
-		sTarget = target;
-	else
-		sTarget = 'main';
+	if (Is.Defined(target)){
+		if (Is.Object(target))
+			sTarget = target.target || 'main';
+		else
+			sTarget = target.toString() || 'main';
+	}
 
 	// create instance of Gui
-	this[sName] = new Gui(sName,sType,this);
+	var newObj = this[sName] = new Gui(sName,sType,this);
 
 	// add anchors pointers (as text representations)
 	if (aObj.ANCHORS && typeof(aObj.ANCHORS[0].ELM) != 'undefined'){
 		var sAnchor = '', aAnchors = {};
-		for (var i in aObj.ANCHORS[0].ELM){
+		for (i in aObj.ANCHORS[0].ELM){
 			if ((sAnchor = aObj.ANCHORS[0].ELM[i].VALUE))
 				aAnchors[sAnchor] = this._pathName + "." + sName + "#" + sAnchor;
 		}
-		this[sName]._anchors = aAnchors;
+		newObj._anchors = aAnchors;
 	}
 
-	this[sName]._anchor = sTarget;
+	newObj._anchor = sTarget;
 
 	// create wrapper HTML element
 	var chld = false;
@@ -154,58 +166,64 @@ Gui.prototype._create = function (sName,sType,target,sClass){
 	case 'form'	:
 		chld = mkElement('form');
 		chld.onsubmit = function(){return false};
-		chld.name = this[sName]._pathName;
+		chld.name = newObj._pathName;
 		break;
 	default:
 		chld = mkElement('div');
 	}
 
 	if (chld){
-
-		//must be before className because of MSIE 6 slowdown
 		var main;
-		if (!(main = this._getAnchor(sTarget)))
-			throw 'Anchor "'+ sTarget +'" doesn\'t exists in "'+ this._type +'" object';
+		if (!(main = this._getAnchor(sTarget))) {
+			main = this._getAnchor('main');
+			 console.info('Anchor "'+ sTarget +'" doesn\'t exists in "'+ this._type +'" object. Creating in component main.')
+		}
 
-		chld.setAttribute('id', this[sName]._pathName);
+		chld.setAttribute('id', newObj._pathName);
 
 		// when target = {first:true}
-		if (Is.Object(target) && target.first && main.firstChild)
-			main.insertBefore(chld, main.firstChild);
-		else
+		if (Is.Object(target)) {
+			if (target.first) {
+				main.insertAdjacentElement('afterbegin', chld);
+			} else if (target.position && target.element) {
+				target.element.insertAdjacentElement(target.position, chld);
+			} else {
+				throw 'Wrong target';
+			}
+		} else {
 			main.appendChild(chld);
+		}
 
 		// Container's CSS can be set as name attr. of <object> tag "<type>" is always used as class descriptor
-		this[sName]._css = sClass;
-		this[sName]._allcss = chld.className = (aObj.ATTRIBUTES && aObj.ATTRIBUTES.CSS?aObj.ATTRIBUTES.CSS+' ':'') + sType + (sClass?' '+sClass:'');
-		this[sName]._main = chld;
+		newObj._css = sClass;
+		newObj._allcss = chld.className = (aObj.ATTRIBUTES && aObj.ATTRIBUTES.CSS?aObj.ATTRIBUTES.CSS+' ':'') + sType + (sClass?' '+sClass:'');
+		newObj._main = chld;
 
 		main = null;
 		chld = null;
 	}
 
-	// Prepare Arguments for __construcor
+	// Prepare Arguments for __constructor
 	var aArg = [].slice.call(arguments).splice(4);
 
 	// inherit defined methods from extensions and execute their __constructor methods for BEFORE
 	if (aObj.BEFORE){
-		var sobj;
-		for (var i in aObj.BEFORE){
+		for (i in aObj.BEFORE){
 
-			if (this[sName] && this[sName]._destructed) return;
+			if (newObj && newObj._destructed) return;
 
 			if (aObj.BEFORE[i].ATTRIBUTES && aObj.BEFORE[i].ATTRIBUTES.CLASS)
-				sobj = aObj.BEFORE[i].ATTRIBUTES.CLASS;
+				sObj = aObj.BEFORE[i].ATTRIBUTES.CLASS;
 			else
-				sobj = aObj.BEFORE[i].VALUE;
+				sObj = aObj.BEFORE[i].VALUE;
 
-			if (typeof window[sobj] == 'function'){
-				inherits(this[sName], window[sobj]);
+			if (typeof window[sObj] == 'function'){
+				inherits(newObj, window[sObj]);
 
 				// do not create object if before.__constructor returns FALSE!
-				if (window[sobj].prototype.__constructor && this[sName].__constructor.apply(this[sName],aArg) === false){
-					this[sName]._destruct();
-					return false;
+				if (window[sObj].prototype.__constructor && await newObj.__constructor.apply(newObj,aArg) === false){
+					await newObj._destruct();
+					return;
 				}
 			}
 		}
@@ -214,29 +232,30 @@ Gui.prototype._create = function (sName,sType,target,sClass){
 	// add template name (just template name from XML)
 	if (aObj.TEMPLATE && typeof aObj.TEMPLATE[0].VALUE != 'undefined'){
 
-		this[sName]._template = aObj.TEMPLATE[0].VALUE;
+		newObj._template = aObj.TEMPLATE[0].VALUE;
 
 		// DRAW
 		if (Is.Defined(this._parent) && Is.Defined(this._parent._aTemplateData))
-			this[sName]._draw(null,null,this._parent._aTemplateData);
+			await newObj._draw(null,null,this._parent._aTemplateData);
 		else
 		if (Is.Defined(this._aTemplateData))
-			this[sName]._draw(null,null,this._aTemplateData);
+			await newObj._draw(null,null,this._aTemplateData);
 		else
-			this[sName]._draw();
+			await newObj._draw();
 	}
 
 	// add destructors
 	// may be created before obj. inicialization...!?
 	if (aObj.ONUNLOAD)
-		for (var i in aObj.ONUNLOAD)
-			this[sName]._add_destructor(aObj.ONUNLOAD[i].VALUE,aObj.ONUNLOAD[i].ATTRIBUTES);
+		for (i in aObj.ONUNLOAD)
+			newObj._add_destructor(aObj.ONUNLOAD[i].VALUE,aObj.ONUNLOAD[i].ATTRIBUTES);
 
 	 // inherit defined methods from extensions and execute their __constructor methods for LIRBARY
-	if (aObj.LIBRARY)
-		for (var sObj = '', i = 0, j = aObj.LIBRARY.length; i<j; i++){
+	if (aObj.LIBRARY) {
+		var j;
+		for (sObj = '', i = 0, j = aObj.LIBRARY.length; i<j; i++){
 
-			if (this[sName] && this[sName]._destructed) return;
+			if (newObj && newObj._destructed) return;
 
 			sObj = aObj.LIBRARY[i].VALUE;
 
@@ -251,30 +270,41 @@ Gui.prototype._create = function (sName,sType,target,sClass){
 			}
 
 			if (typeof window[sObj] == 'function'){
-				inherits(this[sName],window[sObj]);
+				inherits(newObj,window[sObj]);
 
 				if (window[sObj].prototype.__constructor)
-					this[sName].__constructor.apply(this[sName],aArg);
+					await newObj.__constructor.apply(newObj,aArg);
 			}
 		}
+	}
 
 	// Object was fully constructed
-	if (this[sName] && this[sName]._finished)
-		this[sName]._finished();
+	if (newObj && newObj._finished)
+		newObj._finished();
 
 	//Call oncreate Event
 	if (this.__onCreateChild)
 		this.__onCreateChild(sName,sType,sTarget,sClass);
 
-	return this[sName];
-};
+	newObj && newObj.__setAutoreopenArgs.call(newObj, {
+		context: 'gui',
+		fun: ['_create'],
+		args: [].slice.call(arguments)
+	});
 
+	return newObj;
+};
 
 /**
  * @param: sTmpName  - optional, name of template
  * 	    Anchor    - optional, can by string or directly DOM element
  **/
-Gui.prototype._draw = function (sTmpName,sTarget,aData,append){
+Gui.prototype._draw = async function (sTmpName,sTarget,aData,append){
+
+	function rand(n){
+		n = n || 10000000000000000;
+		return (Math.floor ( Math.random ( ) * n + 1 ));
+	};
 
 	if (aData && typeof aData === 'object')
 		aData._ins = this._pathName;
@@ -285,7 +315,7 @@ Gui.prototype._draw = function (sTmpName,sTarget,aData,append){
 		aData._skin = 'client/skins/'+GWOthers.getItem('LAYOUT_SETTINGS', 'skin');
 
 	// retrieve template
-	var sHtml = template.tmp((sTmpName?sTmpName:this._template),aData) || '';
+	var sHtml = await (new cTemplate()).tmp((sTmpName?sTmpName:this._template),aData) || '';
 
 	// parse <OBJ>
 	// <obj> can be parsed from gui objects only because of _ins var requirement */
@@ -295,10 +325,6 @@ Gui.prototype._draw = function (sTmpName,sTarget,aData,append){
 
 	if (sHtml.indexOf('<obj ')>-1){
 
-		//Safari title tag hack
-		// if ((currentBrowser()=='Safari' || currentBrowser()=='Chrome') && sHtml.indexOf('<title')>-1)
-		// 	sHtml = sHtml.replace(/<title/ig,'<safari_title').replace(/<\/title/ig,'</safari_title');
-
 		//OBJECTS
 		var aList = xTpl.getElementsByTagName("obj");
 		if (aList.length){
@@ -346,7 +372,7 @@ Gui.prototype._draw = function (sTmpName,sTarget,aData,append){
 						sAnchor = inArray(this._anchors,id);
 						if (sAnchor == -1){
 							do{
-								sAnchor = Math.rand();
+								sAnchor = rand();
 							}
 							while(this._anchors[sAnchor]);
 
@@ -355,7 +381,7 @@ Gui.prototype._draw = function (sTmpName,sTarget,aData,append){
 					}
 					else{
 						do{
-							sAnchor = Math.rand();
+							sAnchor = rand();
 						}
 						while(this._anchors[sAnchor]);
 
@@ -436,7 +462,17 @@ Gui.prototype._draw = function (sTmpName,sTarget,aData,append){
 	else
 		sAnchor = sTarget;
 
+	[].forEach.call(xTpl.querySelectorAll('[title]'), function(el) {
+		gui.tooltip._add(el, el.getAttribute('title'), {
+			y: function() {
+				return getSize(this).y - 28
+			}.bind(el)
+		});
+		el.removeAttribute('title');
+	});
+
 	var eTarget = sTmpName && sAnchor?this._getAnchor(sTarget):this._main;
+	var result = xTpl.firstElementChild;
 	if (eTarget){
 		//Prepend
 		if (append === 2){
@@ -448,9 +484,16 @@ Gui.prototype._draw = function (sTmpName,sTarget,aData,append){
 		//Append & Replace
 		else{
 			//Replace
-			if (!append)
+			if (!append) {
+				var children = this._getChildObjects();
+				for (var child of children) {
+					if (eTarget.contains(child._main)) {
+						await child._destruct();
+					}
+				};
 				while(eTarget.firstChild)
 					eTarget.removeChild(eTarget.firstChild);
+			}
 
 			while(xTpl.firstChild) {
 				eTarget.appendChild(xTpl.firstChild);
@@ -465,18 +508,21 @@ Gui.prototype._draw = function (sTmpName,sTarget,aData,append){
 	delete aData._ins;
 
 	if (Is.Empty(aData))
-		this.__addObjects(obj);
+		await this.__addObjects(obj);
 	else
-		this.__addObjects(obj,null,aData);
+		await this.__addObjects(obj,null,aData);
 
 	if (this.__onCreateChild)
 		this.__onCreateChild('','',sTarget);
+
+	return result;
 };
 
 
-Gui.prototype.__addObjects = function(obj,oParent,aData){
+Gui.prototype.__addObjects = async function(obj,oParent,aData){
 
-	var newObj, aInit, oParent = oParent || this;
+	var newObj, aInit, j;
+	oParent = oParent || this;
 
 	if (aData)
 		this._aTemplateData = aData;
@@ -489,7 +535,7 @@ Gui.prototype.__addObjects = function(obj,oParent,aData){
 		if (obj[i].init){
 			//pole
 			if (typeof obj[i].init == 'object'){
-				for (var j in obj[i].init)
+				for (j in obj[i].init)
 					aInit.push(obj[i].init[j]);
 			}
 			//string
@@ -500,7 +546,7 @@ Gui.prototype.__addObjects = function(obj,oParent,aData){
 		if (this._aTemplateData && oParent)
 			oParent._aTemplateData = aData;
 
-		newObj = oParent._create.apply(oParent,aInit);
+		newObj = await oParent._create.apply(oParent,aInit);
 
 		// _title
 		if (obj[i].title && Is.Function(newObj._title))
@@ -512,7 +558,7 @@ Gui.prototype.__addObjects = function(obj,oParent,aData){
 
 		// _fill
 		if (obj[i].fill && Is.Function(newObj._fill))
-			newObj._fill(obj[i].fill);
+			await newObj._fill(obj[i].fill);
 
 		// _value
 		if (obj[i].value && Is.Function(newObj._value))
@@ -552,7 +598,7 @@ Gui.prototype.__addObjects = function(obj,oParent,aData){
 			var atmp = [];
 
 			if (typeof obj[i].restrictions == 'object'){
-				for(var j in obj[i].restrictions)
+				for(j in obj[i].restrictions)
 					atmp.push(obj[i].restrictions[j],j);
 			}
 			else
@@ -563,7 +609,7 @@ Gui.prototype.__addObjects = function(obj,oParent,aData){
 				if (atmp.length)
 					newObj._restrict.apply(newObj,atmp);
 			}
-			catch(er){
+			catch {
 				throw "invalid input array for restrictions in:\n"+oParent._pathName+'.'+obj[i].name;
 			}
 		}
@@ -576,7 +622,7 @@ Gui.prototype.__addObjects = function(obj,oParent,aData){
 			}
 			else{
 				aData = arrConcat(aData,obj[i].draw[2]);
-				newObj._draw(obj[i].draw[0],obj[i].draw[1],aData);
+				await newObj._draw(obj[i].draw[0],obj[i].draw[1],aData);
 				if (newObj._isActive && newObj._active) newObj._active(true);
 			}
 
@@ -588,7 +634,7 @@ Gui.prototype.__addObjects = function(obj,oParent,aData){
 				newObj.__drawData = aData;
 			}
 			else{
-				this.__addObjects(obj[i].objects,newObj,aData);
+				await this.__addObjects(obj[i].objects,newObj,aData);
 
 				if (newObj._isActive && newObj._active)
 					newObj._active(true);
@@ -644,12 +690,12 @@ Gui.prototype._disobeyEvent = function(sType, oEvn){
 	return false;
 };
 
-Gui.prototype.__exeEvent = function(sType,e,arg){
+Gui.prototype.__exeEvent = async function(sType,e,arg){
 
 	if (this._events[sType])
 		for (var j in this._events[sType])
 			try{
-				if (typeof this._events[sType][j] == 'undefined' || !Is.Object(this._events[sType][j][0]) || executeCallbackFunction(this._events[sType][j][0], e, arg) === false)
+				if (typeof this._events[sType][j] == 'undefined' || !Is.Object(this._events[sType][j][0]) || await executeCallbackFunction(this._events[sType][j][0], e, arg) === false)
 					delete this._events[sType][j];
 			}
 			catch(r){
@@ -686,26 +732,30 @@ Gui.prototype._getChildObjects = function(sAnchor,sType){
  * @brief: destruct all child objects
  * @param: sAnchor
  **/
-Gui.prototype._clean = function(sAnchor,sType){
+Gui.prototype._clean = async function(sAnchor,sType){
 	var aObj = this._getChildObjects(sAnchor,sType);
 	for (var i in aObj)
-		aObj[i]._destruct();
+		await aObj[i]._destruct();
 
 	return true;
 };
 
 Gui.prototype._getAnchor = function(sAnchor){
-
+	var doc = (this._main || {}).ownerDocument || document;
 	if (Is.Defined(sAnchor))
 		sAnchor = sAnchor.toString();
 
 	if (this._anchors[sAnchor])
-		return document.getElementById(this._anchors[sAnchor]);
+		return doc.getElementById(this._anchors[sAnchor]);
 	else
 	if (sAnchor == 'main' || sAnchor == 'self')
-		return this._main;
+		if (this._name === 'gui' && doc !== document) {
+			return doc.getElementById('gui');
+		} else {
+			return this._main;
+		}
 	else
-		return document.getElementById(this._pathName + (sAnchor?'#' + sAnchor:''));
+		return doc.getElementById(this._pathName + (sAnchor?'#' + sAnchor:''));
 };
 
 /**
@@ -756,7 +806,7 @@ Gui.prototype._remove_destructor = function (sMethod){
  * @breif: destruct object
  * @date : 30.6.2006 13:58:51
  **/
-Gui.prototype._destruct = function(){
+Gui.prototype._destruct = async function(){
 
 	if (this._destructed) return;
 	this._destructed = true;
@@ -771,7 +821,7 @@ Gui.prototype._destruct = function(){
 	*/
 	for (var val in this._destructors)
 		if (Is.Function(this[val]))
-			this[val].apply(this, Is.Array(this._destructors[val])?this._destructors[val]:arguments);
+			await this[val].apply(this, Is.Array(this._destructors[val])?this._destructors[val]:arguments);
 
 	this.__exeEvent('ondestruct', null, {"owner":this});
 
@@ -781,7 +831,7 @@ Gui.prototype._destruct = function(){
 	*/
 	for (var i in this){
 		if (i.indexOf('_')==0 || typeof this[i] != 'object' || this[i]==null || typeof this[i]._destruct != 'function') continue;
-		this[i]._destruct();
+		await this[i]._destruct();
 		delete this[i];
 	}
 
@@ -795,7 +845,7 @@ Gui.prototype._destruct = function(){
 	try{
 		this._main && this._main.parentNode && this._main.parentNode.removeChild(this._main);
 	}
-	catch(er){gui._REQUEST_VARS.debug && console.log(this._name||false,er)}
+	catch(er){ console.log(this._name||false,er)}
 
 	// remove instance
 	this._parent[this._name] = null;
@@ -808,31 +858,40 @@ Gui.prototype._destruct = function(){
 	this.__exeEvent('destructed', null, this);
 };
 
-// root is not inherit into gui descendents
-Gui.__root = function() {
+Gui.prototype.__setAutoreopenArgs = function(aAutoreopenArgs, bForce) {
+	if (this.__autoreopen && (!this.__autoreopenArgs || bForce)) {
+		this.__autoreopenArgs = aAutoreopenArgs;
+		gui.__exeEvent('autoreopenArgs_updated');
+	}
+};
 
+// root is not inherit into gui descendents
+Gui.__root = function(aDocument) {
 	var me = this;
+	aDocument = aDocument || document;
 
 	this._main = mkElement('div',{
 		id: this._name,
+		className: GWOthers.getItem('LAYOUT_SETTINGS', 'blurred_background') == 1 ? 'allow-blurred-background' : '',
 		style: {
 			width:"100%",
 			height:"100%",
 			overflow:"hidden",
 			//Auto-Scroll to Focus hack
-			position:"absolute"
+			position:"absolute",
+			display: 'flex',
+			'flex-direction': 'column',
+			backgroundPosition: 'center'
 		}});
 
 	//Auto-Scroll to Focus hack
 	this._main.onscroll = function(e){
-		var e = e || window.event;
 		this.scrollTop = 0;
 		this.scrollLeft = 0;
-		e.preventDefault && e.preventDefault();
-		return false;
+		e.preventDefault();
 	};
 
-	document.getElementsByTagName('body')[0].appendChild(this._main);
+	aDocument.body.appendChild(this._main);
 
 	this.__X = 0;
 	this.__Y = 0;
@@ -853,28 +912,6 @@ Gui.__root = function() {
 		}()
 	};
 
-	// Ctrl+F11 Fullscreen
-	AttachEvent(document, "onkeydown", function(e) {
-	  if ((e.ctrlKey || e.metaKey) && e.keyCode == 122)
-			if (!document.fullscreenElement && !document.mozFullScreenElement && !document.webkitFullscreenElement) {
-				if (document.documentElement.requestFullscreen) {
-					document.documentElement.requestFullscreen();
-				} else if (document.documentElement.mozRequestFullScreen) {
-					document.documentElement.mozRequestFullScreen();
-				} else if (document.documentElement.webkitRequestFullscreen) {
-					document.documentElement.webkitRequestFullscreen(Element.ALLOW_KEYBOARD_INPUT);
-				}
-			} else {
-				if (document.cancelFullScreen) {
-					document.cancelFullScreen();
-				} else if (document.mozCancelFullScreen) {
-					document.mozCancelFullScreen();
-				} else if (document.webkitCancelFullScreen) {
-					document.webkitCancelFullScreen();
-				}
-			}
-	});
-
 	// Touch prefix for touch enabled device
 	if (('ontouchstart' in window) || (navigator.maxTouchPoints > 0) || (navigator.msMaxTouchPoints > 0)) {
 		addcss(this._main, 'touch');
@@ -883,9 +920,8 @@ Gui.__root = function() {
 
 
 	//DOCUMENT EVENT Handlers
-	function evn (e){
-		var e = e || window.event;
-		me.__exeEvent(e.type,e);
+	function evn (e, sForcedType){
+		me.__exeEvent(sForcedType || e.type, e);
 	};
 
 	window.onresize = function(e){
@@ -905,27 +941,88 @@ Gui.__root = function() {
 		evn(e);
 	};
 
-	//window.navigator && window.navigator.onLine === false
+	this._isVisible = function() {
+		if (this.__visibilityState) {
+			return true;
+		}
+
+		for(var i in Gui.__detachedWindows) {
+			if (Gui.__detachedWindows[i].gui.__visibilityState) {
+				return true;
+			}
+		}
+	}
+	this.__visibilityState = aDocument.visibilityState === 'visible';
+	aDocument.addEventListener('visibilitychange', function(e) {
+		// visible if browser tab is active, browser is not minimized and OS session is not locked
+		me.__visibilityState = aDocument.visibilityState === 'visible';
+		if (gui.__lastVisibleDocument !== e.target) {
+			gui.__lastVisibleDocument = e.target;
+			gui.__exeEvent('activeWindowChange', e.target.defaultView);
+		}
+		evn(e);
+	}, false);
+	window.addEventListener('pagehide', function(e) {
+		// safari does not fire visibilitychange event when visibility changed to 'hidden'
+		if (!e.persisted) {
+			if (!~document.cookie.indexOf('permanentLogin=')) {
+				auth.logout(true);
+				if (navigator.sendBeacon) {
+					var sURL = GWOthers.getItem('LAYOUT_SETTINGS', 'logout_url') || (location.protocol + '//' + location.hostname + (location.port ? ':' + location.port : '') + location.pathname);
+
+					if (window.TeamChatAPI && TeamChatAPI.teamChatOnly()) {
+						sURL += '?' + buildURL({tconly:1, token:TeamChatAPI.getToken(), notifyuri:TeamChatAPI.getNotifyURI()});
+					}
+
+					navigator.sendBeacon(sURL);
+				}
+			}
+			return;
+		}
+		me.__visibilityState = false;
+		evn(e, 'visibilitychange');
+	}, false);
 
 	this.__online = true;
-	document.body.ononline = function(e){
+	var offlineNotification;
+	var offlineNotificationTimeout;
+	aDocument.body.ononline = function(e){
+		clearTimeout(offlineNotificationTimeout);
 		me.__online = true;
+		if (offlineNotification) {
+			gui.notifier._closeNotification(offlineNotification);
+			offlineNotification = void 0;
+		}
 		evn(e);
 	};
-	document.body.onoffline  = function(e){
+	aDocument.body.onoffline = function(e){
 		me.__online = false;
+		clearTimeout(offlineNotificationTimeout);
+		offlineNotificationTimeout = setTimeout(function() {
+			offlineNotification = offlineNotification || gui.notifier._value({
+				type: 'offline',
+				browserOnly: true
+			});
+		}, 10000);
 		evn(e);
 	};
 
-	document.onclick = function(e){
+	aDocument.onclick = function(e){
 		//Chrome fix...
 		if (!me.__focus)
 			me.__focus = true;
 
 		evn(e);
 	};
-	document.onmousedown = evn;
-	document.onmouseup = function(e){
+	aDocument.oncontextmenu = function(e){
+		//Chrome fix...
+		if (!me.__focus)
+			me.__focus = true;
+
+		evn(e);
+	};
+	aDocument.onmousedown = evn;
+	aDocument.onmouseup = function(e){
 		//Chrome fix...
 		if (!me.__focus)
 			me.__focus = true;
@@ -934,9 +1031,8 @@ Gui.__root = function() {
 	};
 
 	//WheelSpin
-	AttachEvent(document, "onmousewheel", function(e){
-		var e = e || window.event,
-			delta;
+	AttachEvent(document, "onwheel", function(e){
+		var delta;
 
 		if (Is.Defined(e.deltaY))
 			delta = e.deltaY;
@@ -947,69 +1043,37 @@ Gui.__root = function() {
 			delta = Is.Defined(e.detail)? e.detail : e.wheelDelta/-120;
 		}
 
-		evn({type:'mousewheel', delta:delta});
+		evn({type:'wheel', delta:delta});
 	});
 
-	document.onmousemove = function(e){
-		var e = e || window.event;
-
+	aDocument.onmousemove = function(e){
 		me.__X = e.clientX;
 		me.__Y = e.clientY;
 
 		evn(e);
 	};
-	document.onkeydown = function (e){
-		var e = e || window.event;
+	aDocument.onkeydown = function (e){
 		evn(e);
 
 		//stops F5 (refresh) propagation to browser
 		//it caused unwanted refresh(es) because of placement in NB keyboard
 		//stops Esc because it cancel actual open http connection
 		if (e.keyCode == 116 || e.keyCode == 27){
-			e.cancelBubble = true;
-			e.preventDefault && e.preventDefault();
-			e.stopPropagation && e.stopPropagation();
-			return false;
-		}
-		else
-		//Ctrl+F11 for Full Screen Mode
-		if (e.keyCode == 122 && (e.ctrlKey || e.metaKey)){
-			if (!document.fullscreenElement && !document.mozFullScreenElement && !document.webkitFullscreenElement) {
-				if (document.documentElement.requestFullscreen)
-					document.documentElement.requestFullscreen();
-				else
-				if (document.documentElement.mozRequestFullScreen)
-					document.documentElement.mozRequestFullScreen();
-				else
-				if (document.documentElement.webkitRequestFullscreen)
-					document.documentElement.webkitRequestFullscreen(Element.ALLOW_KEYBOARD_INPUT);
-			}
-			else
-			if (document.cancelFullScreen)
-				document.cancelFullScreen();
-			else
-			if (document.mozCancelFullScreen)
-				document.mozCancelFullScreen();
-			else
-			if (document.webkitCancelFullScreen)
-				document.webkitCancelFullScreen();
+			e.preventDefault();
+			e.stopPropagation();
 		}
 	};
-	document.onkeyup = function (e){
-		var e = e || window.event;
+	aDocument.onkeyup = function (e){
 		evn(e);
 
 		//stops Esc because it cancel actual open http connection
 		if (e.keyCode == 116 || e.keyCode == 27){
-			e.cancelBubble = true;
-			e.preventDefault && e.preventDefault();
-			e.stopPropagation && e.stopPropagation();
-			e.keyCode == 116 && gui.frm_main._getNew();
-			return false;
+			e.preventDefault();
+			e.stopPropagation();
+			e.keyCode == 116 && gui.frm_main._getNew && gui.frm_main._getNew();
 		}
 	};
-	document.onpaste = function (e){
-		var e = e || window.event;
+	aDocument.onpaste = function (e){
 		evn(e);
 	};
 
@@ -1027,16 +1091,282 @@ Gui.__root = function() {
 			try{
 				this.__loading_obj._loading(this.__loading_stack.length);
 			}
-			catch(r){gui._REQUEST_VARS.debug && console.log(this._name||false,r)}
+			catch(r){ console.log(this._name||false,r)}
 		}
 		else
 			this.__loading_stack = [];
 	};
 
-	//MSIE page scrolling fix
-	if (currentBrowser().indexOf('MSIE')===0)
-		setInterval(function(){
-			if (me._main.scrollTop>0)
-				me._main.scrollTop = 0;
-		},1000);
+	window.addEventListener('beforeunload', function() {
+		for(var i in Gui.__detachedWindows) {
+			var win = Gui.__detachedWindows[i];
+			win && !win.closed && win.close();
+		}
+	});
 };
+
+Gui.__detachedWindows = {};
+
+Gui.prototype._detach = function(wWindow) {
+	if (this._onbeforedetach) {
+		this._onbeforedetach(this.__detach.bind(this, wWindow));
+	} else {
+		this.__detach(wWindow);
+	}
+};
+
+Gui.prototype.__detach = function(wWindow) {
+	this.__detachedId = unique_id();
+
+	// create new window
+	this.__detached = wWindow || window.open(this.__url || '', this.__detachedId);
+	if (!this.__detached) {
+		return;
+	}
+
+	var doc = this.__detached.document;
+	focus();
+	Gui.__detachedWindows[this.__detachedId] = this.__detached;
+
+	// copy head
+	var toLoad = 0;
+	[].forEach.call(document.head.children, function(child) {
+		var clone = child.cloneNode(true);
+		if (clone.tagName === 'LINK' && clone.getAttribute('rel') === 'stylesheet') {
+			toLoad++;
+			clone.addEventListener('load', function() {
+				if (!--toLoad) {
+					var NM = NightMode(doc.defaultView);
+					if (window.IW_NM && IW_NM.init) {
+						NM.activate();
+					}
+				}
+			});
+		} else if (clone.tagName === 'STYLE') {
+			doc.head.appendChild(clone);
+		}
+		doc.head.appendChild(clone);
+	}, this);
+
+	// move popup to new window
+	var new_gui = new Gui('gui', void 0, void 0, doc);
+	this.__detached.component = this;
+	this.__detached.gui = new_gui;
+	this.__detached.opener = this._gui;
+	new_gui._main.appendChild(this._main);
+	doc.body.appendChild(new_gui._main);
+	doc.title = this._title();
+	addcss(this._main, 'detached');
+
+	//Drop File, registr Events
+	if (window.FormData){
+		var frm_main = gui.frm_main;
+		doc.addEventListener("dragover", function(e){
+			if (frm_main.__filedrag){
+				e.preventDefault();
+
+				if (frm_main.__dragtimer == null)
+					frm_main.__file_dragover(e);
+		        else
+		            clearTimeout(frm_main.__dragtimer);
+
+				frm_main.__dragtimer = setTimeout(function(){
+					frm_main.__dragtimer = null;
+					frm_main.__file_dragover(false);
+				},500);
+			}
+		}, false);
+
+		doc.addEventListener("drop", function(e){ if (frm_main.__filedrag) e.preventDefault(); }, false);
+
+		doc.addEventListener("dragstart", function(){
+			frm_main.__filedrag = false;
+		}, true);
+		doc.addEventListener("dragend", function(){
+			frm_main.__filedrag = true;
+		}, true);
+	}
+
+	this.__detached.onbeforeunload = function(e) {
+		switch(GWOthers.getItem('LAYOUT_SETTINGS','confirm_exit').toString()){
+			case '1':
+				if (this._type !== 'frm_compose')
+					break;
+
+			case '2':
+				e.preventDefault();
+				e.returnValue = getLang('CONFIRMATION::UNLOAD');
+				return getLang('CONFIRMATION::UNLOAD');
+		}
+	}.bind(this);
+
+	this.__detached.onunload = function() {
+		this.__detached.onbeforeunload = false;
+		this.__detached.opener._disobeyEvent('resize',[this,'__checkPosition']);
+		new_gui._disobeyEvent('resize',[this,'__checkPosition']);
+		this._destruct();
+		for (var i in Gui.__detachedWindows) {
+			if (Gui.__detachedWindows[i].component === this) {
+				delete Gui.__detachedWindows[i];
+			}
+		}
+	}.bind(this);
+
+	this.__detached.focus();
+	gui.__exeEvent('ondetach', new_gui);
+	gui.__exeEvent('activeWindowChange', doc.defaultView);
+	this._add_destructor('__closeWindowOnDestruct');
+	new_gui._obeyEvent('resize',[this,'__checkPosition']);
+
+	addEventListener('NightMode', function(event) {
+		if (!this.__detached || this.__detached.closed) {
+			return;
+		}
+		var NM = NightMode(doc.defaultView);
+		if (event.detail.enabled) {
+			NM.activate();
+		} else {
+			NM.reset();
+		}
+	}.bind(this));
+};
+
+Gui.prototype.__closeWindowOnDestruct = function() {
+	this.__detached && !this.__detached.closed && this.__detached.close();
+};
+
+Gui.prototype._setTheme = async function(sColor, sImagePath, bCrossOrigin) {
+	await storage.library('extract-colors', 'extract-colors');
+
+	if (GWOthers.getItem('LAYOUT_SETTINGS', 'daily_wallpaper') == '1') {
+		var response = await fetch(atob('aHR0cHM6Ly9yYXcuZ2l0aHVidXNlcmNvbnRlbnQuY29tL25pdW1vby9iaW5nLXdhbGxwYXBlci9tYWluL2Jpbmctd2FsbHBhcGVyLm1k')).then((response) => {
+			if (response.ok) {
+				return response.text();
+			}
+		});
+		sImagePath = ((response || '').match(new RegExp((new Date()).toISOString().split('T')[0] + ' \\| \\[.*?\\]\\((.*?)\\)')) || [])[1] || sImagePath;
+
+		this.__dailyBackgroundCheck = setTimeout(function() {
+			gui._setTheme();
+		}, 3600000);
+	} else {
+		clearTimeout(this.__dailyBackgroundCheck);
+	}
+
+	var eThemeStyle = document.getElementById('iw-theme-background') || mkElement('style', { id: 'iw-theme-background' });
+
+	var sTheme = ':root {';
+	if (sColor) {
+		sTheme += '--backgroundColor: ' + sColor + ';';
+	}
+	if (sImagePath) {
+		sTheme += '--backgroundImageURL: url(' + sImagePath + ');';
+	}
+	sTheme += '}';
+	
+	eThemeStyle.textContent = sTheme;
+	document.head.appendChild(eThemeStyle);
+
+	var login_interface_color = GWOthers.getItem('LAYOUT_SETTINGS', 'login_interface_color');
+	if (sImagePath) {
+		extractColors.extractColorsFromSrc(sImagePath, {
+			crossOrigin: bCrossOrigin ? 'anonymous' : false
+		}).then(function(swatches) {
+			var mostPopulous = swatches[0];
+			for (var i in swatches) {
+				if (swatches[i].area * swatches[i].intensity * swatches[i].saturation * swatches[i].lightness > mostPopulous.area * mostPopulous.intensity * mostPopulous.saturation * mostPopulous.lightness) {
+					mostPopulous = swatches[i];
+				}
+			}
+
+			var rgb = [mostPopulous.red, mostPopulous.green, mostPopulous.blue];
+			var hsl = colors.rgb2hsl.apply(colors, rgb);
+			this.__backgroundColor = colors.rgb2hex.apply(colors, colors.hsl2rgb(hsl[0], hsl[1], 0.45));
+
+			if (login_interface_color === 'background') {
+				this._setInterfaceColors(this.__backgroundColor);
+			} else {
+				this._setInterfaceColors(login_interface_color);
+			}
+
+		}.bind(this)).catch(function() {
+			this.__backgroundColor = sColor || '#702EB7';
+			if (login_interface_color === 'background') {
+				this._setInterfaceColors(this.__backgroundColor);
+			} else {
+				this._setInterfaceColors(login_interface_color);
+			}
+		}.bind(this));
+
+		if (login_interface_color !== 'background') {
+			this._setInterfaceColors(login_interface_color, !Gui.__skin_colors.bg_custom);
+		}
+	} else {
+		var hsl = colors.rgb2hsl.apply(colors, colors.hex2rgb(sColor));
+		this.__backgroundColor = colors.rgb2hex.apply(colors, colors.hsl2rgb(hsl[0], hsl[1], 0.45));
+
+		if (login_interface_color === 'background') {
+			this._setInterfaceColors(this.__backgroundColor);
+		} else {
+			this._setInterfaceColors(login_interface_color);
+		}
+	}
+};
+
+Gui.__skin_colors = {
+	bg_green: "#4FC980",
+	bg_important: '#d51414',
+	bg_important_darker: '#A44A3E'
+};
+
+Gui.prototype._setInterfaceColors = function(hex, bCustom) {
+	var rgb = colors.hex2rgb(hex);
+	var hsl = colors.rgb2hsl.apply(colors, rgb);
+
+	rgb = colors.hsl2rgb(hsl[0], hsl[1], 0.45);
+	hex = colors.rgb2hex(rgb[0], rgb[1], rgb[2]);
+
+	rgb = colors.hsl2rgb(hsl[0], hsl[1], 0.3);
+	var hex_dark = colors.rgb2hex(rgb[0], rgb[1], rgb[2]);
+
+	rgb = colors.hsl2rgb(hsl[0], hsl[1], 0.95);
+	var hex_light = colors.rgb2hex(rgb[0], rgb[1], rgb[2]);
+
+	Object.assign(Gui.__skin_colors, {
+		bg_main: hex,
+		bg_main_darker: hex_dark,
+		bg_main_lighter: hex_light,
+		bg_ok_button: hex,
+		bg_ok_button_darker: hex_dark
+	});
+
+	if (bCustom) {
+		Gui.__skin_colors.bg_custom = hex;
+	}
+
+	dataSet.add('main', ['skin_colors'], Gui.__skin_colors);
+	dataSet.add('storage', ['LAYOUT_SETTINGS', 'ITEMS', 0, 'VALUES', 'CUSTOM_SKIN_COLORS'], {
+		ATTRIBUTES: {
+			DONT_SEND: true
+		}
+	});
+	GWOthers.setItem('LAYOUT_SETTINGS', 'custom_skin_colors', JSON.stringify(Gui.__skin_colors));
+
+	var style = ':root{';
+	for (var i in Gui.__skin_colors) {
+		style += '--' + i + ':' + Gui.__skin_colors[i] + ';';
+	}
+	if (this.__backgroundColor) {
+		style += '--bg_color:' + this.__backgroundColor + ';';
+	}
+	style += '}';
+
+	if(!document.getElementById('iw-skin-colors')) {
+		document.head.appendChild(mkElement('style', { id: 'iw-skin-colors' }));
+	}
+	document.getElementById('iw-skin-colors').textContent = style;
+
+	if (dataSet.get('main', ['night_mode_enabled'])) {
+		NightMode().activate(false, true);
+	}
+};
\ No newline at end of file
