@@ -1,7 +1,7 @@
 function hash(){}
 
 hash.prototype = {
-	__constructor: function(){		
+	__constructor: async function(){		
 		var me = this;
 
 		window.addEventListener("hashchange", function(e){
