@@ -1,7 +1,7 @@
 _me = obj_allow_settings.prototype;
 function obj_allow_settings(){};
 
-_me.__constructor = function(bUser) {
+_me.__constructor = async function(bUser) {
 	var me = this;
 	
 	if (this.domadmin)
