@@ -19,8 +19,9 @@ _me._place = function(x,y,w,h){
 			css.left = (x<0?10:x)+'px';
 	}
 	setTimeout(function () {
-		var max_x = (window.innerWidth - 10 - this._main.clientWidth);
-		css.left = (x > max_x ? max_x : x)+'px';;
+		var win = this._main.ownerDocument.defaultView;
+		var max_x = (win.innerWidth - 10 - this._main.clientWidth);
+		css.left = Math.max(x > max_x ? max_x : x, 0)+'px';;
 	}.bind(this), 5);
 	if (y)
 	    css.top = (y.toString().indexOf('%')>-1?y:y+'px');
