@@ -15,18 +15,15 @@ function obj_button_check(){};
 * @brief:
 * @date : 30.4.2013 17:24:21
 **/
-_me.__constructor = function(){
+_me.__constructor = async function(){
 	var me = this;
 	this.__checked = false;
 
 	/*** ADD EVENTS ***/
 	this.__eIN.onclick = this._main.onclick = function (e){
-		var e = e || window.event,
-			elm = e.target || e.srcElement;
-
-		if (this == elm){
+		if (this == e.target){
 			if (!me._disabled()){
-				if (elm == me._main)
+				if (e.target == me._main)
 					me._focus();
 
 				me._checked(!me._checked());
@@ -39,10 +36,7 @@ _me.__constructor = function(){
 	};
 
 	AttachEvent(this.__eIN, 'onkeydown', function(e){
-		var e = e || window.event,
-			elm = e.target || e.srcElement;
-
-		if (this == elm && !me._disabled()){
+		if (this == e.target && !me._disabled()){
 			if (me._onkeypress)
 				me._onkeypress(e);
 		}
