@@ -1,13 +1,13 @@
 _me = obj_button_map.prototype;
 function obj_button_map(){};
 
-_me.__constructor = function (){
+_me.__constructor = async function (){
 	this._title(getLang('FORM_BUTTONS::MAP_IT'));
 };
 
 _me._onclick = function(){
 	var v = this._value();
-	if (v)
+	if (v && GWOthers.getItem('EXTERNAL_SETTINGS', 'google_maps_api_key'))
 		this._showmap(v);
 };
 
