@@ -1,7 +1,7 @@
 _me = obj_charset.prototype;
 function obj_charset(){};
 
-_me.__constructor = function(){
+_me.__constructor = async function(){
 	this._fill({
 	"UTF-8":'Auto Unicode (UTF8)',
 	"ISO-8859-1":'Western European (ISO-8859-1)',
