@@ -6,7 +6,7 @@
 _me = obj_checkbox_inverse.prototype;
 function obj_checkbox_inverse(){};
 
-	_me.__constructor = function(){
+	_me.__constructor = async function(){
 		this.__eIN.checked = true;
 	};	
 	/**
