@@ -2,7 +2,7 @@ _me = obj_conference_banner.prototype;
 
 function obj_conference_banner() {};
 
-_me.__constructor = function () {
+_me.__constructor = async function () {
 	if (GWOthers.getItem('BANNER_OPTIONS', 'norecordingbanner') > 0) {
 		this._destruct();
 		return;
