@@ -2,19 +2,19 @@ _me = obj_conference_home.prototype;
 
 function obj_conference_home() {};
 
-_me.__constructor = function () {
+_me.__constructor = async function () {
 
 	this.__opt = {
 		schedule: ~['jitsi', 'jitsi-full'].indexOf(dataSet.get('accounts',[sPrimaryAccount, 'MEETING_PROVIDER']))
 	};
 
-	this._draw('obj_conference_home','cell', {
+	await this._draw('obj_conference_home','cell', {
 		schedule:this.__opt.schedule,
 		recording: !this.__opt.schedule || sPrimaryAccountRECORDING
 	});
 
-	this.btn_start._onclick = function(){
-		storage.library('wm_conference');
+	this.btn_start._onclick = async function(){
+		await storage.library('wm_conference');
 		if(dataSet.get('main', ['conference'])) {
 			wm_conference.get(dataSet.get('main', ['conference'])).join();
 			this._view('progress');
@@ -28,5 +28,6 @@ _me.__constructor = function () {
 			this._view('schedule');
 		}.bind(this._parent);
 
-	this._create('scrollbar','obj_scrollbar')._scrollbar(this._main.querySelector('div.scroll'), this._main);
+	await this._create('scrollbar','obj_scrollbar');
+	this.scrollbar._scrollbar(this._main.querySelector('div.scroll'), this._main);
 };
