@@ -1,7 +1,7 @@
 _me = obj_datagrid.prototype;
 function obj_datagrid(){};
 
-_me.__constructor = function(bDontListen){
+_me.__constructor = async function(){
 	//to disable neverending functionality
 	this.__offset = 0;
 	this.__reizeInterval = 1;
@@ -54,7 +54,7 @@ _me._serverSort = function (aData, sColumn, sType){
 	    aTmp.reverse();
 
 	var aSortedData = {};
-	for (var i in aTmp)
+	for (i in aTmp)
 		aSortedData['*' + aTmp[i][0]] = this._aData[aTmp[i][0]];
 
 	this._fill(aSortedData);
