@@ -1,7 +1,7 @@
 _me = obj_day_of_week.prototype;
 function obj_day_of_week(){};
 
-_me.__constructor = function() {
+_me.__constructor = async function() {
 	var sOptName;
 
 	this._aDays = {
@@ -22,13 +22,13 @@ _me.__constructor = function() {
 		'friday': "DAYS::FRIDAY",
 		'saturday': "DAYS::SATURDAY"
 	};
-	storage.library('gw_others');
+	await storage.library('gw_others');
 	// this is a bit tricky, don't try to understand it :-) (sets the start day of the week)
 	var which = (7 - this._aDays[GWOthers.getItem('CALENDAR_SETTINGS','week_begins')]) % 7;
 
 	for (var day in this._aDays) {
 		sOptName = 'day_' + which;
-		this._create(day, 'obj_checkbox', sOptName,'', aLabels[day]);
+		await this._create(day, 'obj_checkbox', sOptName,'', aLabels[day]);
 		which = (which + 1) % 7;
 	}
 
@@ -36,16 +36,16 @@ _me.__constructor = function() {
 };
 
 _me._value = function(v) {
-	var n;
+	var n, i;
 
 	if (Is.Defined(v)) {
-		for (var i in this._objects) {
+		for (i in this._objects) {
 			n = Math.pow(2, this._aDays[i]);
 			this._objects[i]._value(v&n);
 		}
 	} else {
 		var nResult = 0;
-		for (var i in this._objects) {
+		for (i in this._objects) {
 			if (this._objects[i]._value()) nResult |= Math.pow(2, this._aDays[i]);
 		}
 		return nResult;
