@@ -1,10 +1,10 @@
 _me = obj_dayview.prototype;
 function obj_dayview(){};
 
-_me.__constructor = function(owner){
+_me.__constructor = async function(owner){
 	this._closeOnContext = false;
 
-	this._create('day', 'obj_evnview');
+	await this._create('day', 'obj_evnview');
 
 	//hardlink to frm_main_calendar
 	if (owner){
