@@ -4,9 +4,9 @@ function obj_dtmf(){};
 /**
  * DTMF Keypad
  */         
-_me.__constructor = function() {
+_me.__constructor = async function() {
 
-    this._draw('obj_dtmf', 'main');
+    await this._draw('obj_dtmf', 'main');
 
 	// Add and handle dial pad
 	this.btn_0._onclick = function(){
