@@ -1,11 +1,11 @@
 _me = obj_edit_header.prototype;
 function obj_edit_header() {};
 
-_me.__constructor = function() {
+_me.__constructor = async function() {
 	var me = this;
 	this.__aValues = '';
 
-	this._create('button','obj_button');
+	await this._create('button','obj_button');
 	this.button._value('FILTERS::THREE_DOTS');
 	this.button._onclick = function(){
 		gui._create('edit_header', 'frm_edit_header', '', '', me);
