@@ -1,7 +1,7 @@
 _me = obj_evnmonth.prototype;
 function obj_evnmonth() {};
 
-_me.__constructor = function () {
+_me.__constructor = async function () {
 	var me = this;
 
 	this._active = new IcewarpDate();
@@ -13,8 +13,6 @@ _me.__constructor = function () {
 	this.__activeEvent;
 	this.__iToday;
 
-	this._row_height = 24; // Height + 2 (margin)
-
 	//access rights
 	this.__rights = {'read': true, 'write': true, 'modify': true, 'remove': true};
 
@@ -22,9 +20,10 @@ _me.__constructor = function () {
 
 	this._selection = {startdate: 0, enddate: 0};
 	this.__noRefresh = false;
+	this._row_height = 25;
 
 	//Get offset & Create calendar header
-	storage.library('gw_others');
+	await storage.library('gw_others');
 //	var aDays = {'monday': 1, 'tuesday': 2, 'wednesday': 3, 'thursday': 4, 'friday': 5, 'saturday': 6, 'sunday': 0};
 //	try {
 //		this.__offset = aDays[GWOthers.get('CALENDAR_SETTINGS', 'storage')['VALUES']['week_begins']];
@@ -34,7 +33,7 @@ _me.__constructor = function () {
 //	IcewarpDate.setCustomWeekStart(this.__offset);
 	var aData = {"days": IcewarpDate.weekdaysShort()};
 
-	this._getAnchor('header').innerHTML = template.tmp('obj_evnmonth_header', aData);
+	this._getAnchor('header').innerHTML = await (new cTemplate()).tmp('obj_evnmonth_header', aData);
 
 	//Quick Edit/Add
 	this.__edit = false;
@@ -45,8 +44,7 @@ _me.__constructor = function () {
 
 		if (!me.__edit) {
 
-			var e = e || window.event,
-					newEvn = {};
+			var newEvn = {};
 
 			if (!me.__rights.modify) {
 				if (!me.__activeEvent || !(newEvn = me._useItem(me.__activeEvent)) || newEvn.owner != sPrimaryAccountGWID)
@@ -126,23 +124,19 @@ _me.__constructor = function () {
 	};
 
 	this._main.onmousedown = function (e) {
-		var e = e || window.event,
-				elm = e.target || e.srcElement;
+		var elm = e.target;
 
 		if (elm.tagName !== 'INPUT') {
 			me._focus();
 		}
 	};
-	//MSIE needs onmouseup too...
-	this._main.onmouseup = this._main.onmousedown;
 
 	this.__eMain = this._getAnchor('main');
 	this._scrollbar(this.__eMain, this.__eMain.parentNode);
 
 	// Events handling
 	this.__eMain.ondblclick = function (e) {
-		var e = e || window.event,
-				elm = e.target || e.srcElement;
+		var elm = e.target;
 		if (elm === this) {
 			return;
 		}
@@ -151,7 +145,8 @@ _me.__constructor = function () {
 		switch (info['type']) {
 			case 'TD':
 			case 'TH':
-				var d = new Date(), e = new Date(d);
+				var d = new Date();
+				e = new Date(d);
 				var msSinceMidnight = e - d.setHours(0,0,0,0);
 				arg = {"startdate": info.pos, "starttime": msSinceMidnight / 1000, "_event": 'blank'};
 				break;
@@ -182,8 +177,7 @@ _me.__constructor = function () {
 
 	// CONTEXT MENU HANDLER
 	this.__eMain.oncontextmenu = function (e) {
-		var e = e || window.event,
-				elm = e.target || e.srcElement;
+		var elm = e.target, i;
 		if (elm === this) {
 			return;
 		}
@@ -201,7 +195,7 @@ _me.__constructor = function () {
 
 				//Obsazene Events
 				arg.contains = [];
-				for (var i in me.__value) {
+				for (i in me.__value) {
 					if (me.__value[i].evnclass == 'E' && me.__value[i].startdate >= me._selection.startdate && me.__value[i].enddate <= me._selection.enddate) {
 						arg.contains.push(me.__value[i].id);
 					}
@@ -212,7 +206,7 @@ _me.__constructor = function () {
 			case 'B':
 			case 'I':
 			case 'P':
-				for (var i = 0; i < me.__idTable.length; i++) {
+				for (i = 0; i < me.__idTable.length; i++) {
 					if (me.__idTable[i].id == info.id) {
 						arg = clone(me.__idTable[i]);
 						arg._event = 'event';
@@ -240,9 +234,8 @@ _me.__constructor = function () {
 		return false;
 	};
 
-	this.__eMain.onclick = function (e) {
-		var e = e || window.event,
-				elm = e.target || e.srcElement;
+	this.__eMain.onclick = async function (e) {
+		var elm = e.target;
 
 		if (elm.tagName == 'EM') {
 
@@ -252,7 +245,7 @@ _me.__constructor = function () {
 					view = getSize(me._main),
 					w = cell.w > 250 ? cell.w : 250;
 
-			this.preview = gui._create('preview', 'obj_dayview', '', '', me);
+			this.preview = await gui._create('preview', 'obj_dayview', '', '', me);
 			this.preview._place(view.x + view.w < cell.x + w + 20 ? view.x + view.w - w + 20 : cell.x, view.y, w, view.h);
 
 			var info = me.__getInfo(e, elm);
@@ -263,8 +256,7 @@ _me.__constructor = function () {
 
 	this.__eMain.onmousedown = function (e) {
 
-		var e = e || window.event,
-				elm = e.target || e.srcElement;
+		var elm = e.target;
 
 		if (elm.tagName == 'INPUT' || elm === this || e.button == 2)
 			return;
@@ -283,12 +275,10 @@ _me.__constructor = function () {
 				//Create selection
 				gui._obeyEvent('mouseup', [me, '__mouseup']);
 				this.onmousemove = function (e) {
-					var e = e || window.event,
-							elm = e.target || e.srcElement;
-					if (elm == this)
+					if (e.target == this)
 						return;
 
-					var info2 = me.__getInfo(e, elm);
+					var info2 = me.__getInfo(e, e.target);
 					if (info2.pos >= info.pos)
 						me.__draw_selection(info.pos, info2.pos);
 					else
@@ -322,8 +312,6 @@ _me.__constructor = function () {
 					if (!me.__rights.modify && (!itm || itm.owner != sPrimaryAccountGWID))
 						return false;
 
-					//if (e.preventDefault) e.preventDefault();
-
 					var cursor_offset = info.pos - arg['start'];
 					if (!Is.Number(cursor_offset))
 						return false;
@@ -331,12 +319,10 @@ _me.__constructor = function () {
 					//stop
 					gui._obeyEvent('mouseup', [me, '__mouseup', [arg, itm]]);
 					this.onmousemove = function (evn) {
-						var evn = evn || window.event,
-								elm = evn.target || evn.srcElement;
-						if (elm == this)
+						if (evn.target == this)
 							return;
 
-						var info2 = me.__getInfo(evn, elm);
+						var info2 = me.__getInfo(evn, evn.target);
 
 						//Do some magic
 						if (info2.pos > 0) {
@@ -346,11 +332,11 @@ _me.__constructor = function () {
 								var tmp = parseInt(itm.enddate, 10) - parseInt(itm.startdate, 10);
 								itm.startdate = parseInt(info2.pos, 10);
 								itm.enddate = parseInt(info2.pos, 10) + tmp;
-								me._fill(me.__value);
+								me._fill(me.__value, true);
 							} else
 							if (sAction == 'start' && itm.startdate != info2.pos && info2.pos <= itm.enddate) {
 								itm.startdate = info2.pos;
-								me._fill(me.__value);
+								me._fill(me.__value, true);
 							} else
 							if (sAction == 'end' && info2.pos >= itm.startdate) {
 
@@ -360,7 +346,7 @@ _me.__constructor = function () {
 								else
 									itm.enddate = info2.pos;
 
-								me._fill(me.__value);
+								me._fill(me.__value, true);
 							} else
 								return;
 
@@ -406,7 +392,7 @@ _me.__getIDs = function (id) {
 	return out;
 };
 _me.__getInfo = function (evn, elm) {
-	var pos, elm = elm, sType, id, tmp_id;
+	var pos, sType, id, tmp_id;
 
 	switch (elm.tagName) {
 		case 'EM':
@@ -433,9 +419,6 @@ _me.__getInfo = function (evn, elm) {
 		case 'I':
 			sType = 'SPAN';
 
-			if (elm.parentNode.tagName == 'SPAN')
-				offset = 8;
-
 			elm = Is.Child(elm, 'P');
 		case 'P':
 
@@ -460,13 +443,14 @@ _me.__getInfo = function (evn, elm) {
 	return {pos: pos, elm: elm, 'type': sType, tmp_id: tmp_id, id: id};
 };
 
-_me._fill = function (aDataIn) {
+_me._fill = async function (aDataIn, bNoScroll) {
 	//*** Draw GRID ***
+	var i, j, t, aLoc, elm;
 	var today = new IcewarpDate();
 	var aData = {"days": IcewarpDate.weekdaysShort()};
 	var aTags = dataSet.get('tags') || {};
 
-	this._getAnchor('header').innerHTML = template.tmp('obj_evnmonth_header', aData);
+	this._getAnchor('header').innerHTML = await (new cTemplate()).tmp('obj_evnmonth_header', aData);
 	if (gui._rtl) {
 		this._getAnchor('header').querySelector('.table_header').setAttribute('dir', 'rtl');
 	}
@@ -485,6 +469,7 @@ _me._fill = function (aDataIn) {
 			week.push({
 				julian: start_day.format(IcewarpDate.JULIAN),
 				value: start_day.date(),
+				date: start_day.clone(),
 				_date: +GWOthers.getItem('CALENDAR_SETTINGS', 'alternative_calendar') >= 0 ? start_day.clone({calendar: +GWOthers.getItem('CALENDAR_SETTINGS', 'alternative_calendar')}) : false,
 				class: (start_day.isSame(today, 'day') ? 'today ' : '') + (current_month === month ? 'month ' : '')
 			});
@@ -497,10 +482,13 @@ _me._fill = function (aDataIn) {
 		var end = start_day.clone().subtract(1, 'days');
 		this.__range['end'] = start_day.format(IcewarpDate.JULIAN);
 
-		if (+GWOthers.getItem('CALENDAR_SETTINGS', 'alternative_calendar') >= 0) {
+		if (+GWOthers.getItem('CALENDAR_SETTINGS', 'alternative_calendar') >= 0 && gui.frm_main.main) {
 			start = start.clone({calendar: +GWOthers.getItem('CALENDAR_SETTINGS', 'alternative_calendar')});
 			end = end.clone({calendar: +GWOthers.getItem('CALENDAR_SETTINGS', 'alternative_calendar')});
-			gui.frm_main.main._main.querySelector('.alternative_calendar').textContent = start.format('LL') + ' - ' + end.format('LL');
+			var alternative_calendar = this._parent._main.querySelector('.alternative_calendar');
+			if (alternative_calendar) {
+				alternative_calendar.textContent = start.format('LL') + ' - ' + end.format('LL');
+			}
 		}
 
 		//Draw Month Grid
@@ -510,14 +498,14 @@ _me._fill = function (aDataIn) {
 		} else {
 			str = ['<table border="0" class="evnmonth_calendar"><tbody>'];
 		}
-		for (var i = 0; i < aWeek.length; i++) {
+		for (i = 0; i < aWeek.length; i++) {
 			str.push('<tr class="day_header">');
-			for (var j = 0; j < aWeek[i].length; j++)
-				str.push('<th id="' + this._pathName + '#' + aWeek[i][j].julian + '"' + (aWeek[i][j]['class'] ? ' class="' + aWeek[i][j]['class'] + '"' : '') + '><h3 unselectable="on"><em></em>' + aWeek[i][j].value + (aWeek[i][j]._date ? ('<span style="font-weight: normal;"> / ' + aWeek[i][j]._date.date()+ '</span>') : '') + '</h3></th>');
+			for (j = 0; j < aWeek[i].length; j++)
+				str.push('<th id="' + this._pathName + '#' + aWeek[i][j].julian + '"' + (aWeek[i][j]['class'] ? ' class="' + aWeek[i][j]['class'] + '"' : '') + '><h3 unselectable="on">' + (aWeek[i][j].date.day() === IcewarpDate.firstDayOfWeek() ? '<span>' + aWeek[i][j].date.week() + '</span>' : '') + '<em></em>' + aWeek[i][j].value + (aWeek[i][j].value == 1 ? ' ' + aWeek[i][j].date.format('MMM') : '') + (aWeek[i][j]._date ? ('<i style="font-weight: normal;"> / ' + aWeek[i][j]._date.date()+ '</i>') : '') + '</h3></th>');
 
 			str.push('</tr><tr class="days">');
 
-			for (var j = 0; j < aWeek[i].length; j++) {
+			for (j = 0; j < aWeek[i].length; j++) {
 				str.push('<td id="' + this._pathName + '/' + aWeek[i][j].julian + '"' + (aWeek[i][j]['class'] ? ' class="' + aWeek[i][j]['class'] + '"' : '') + ' unselectable="on">');
 				str.push('</td>');
 			}
@@ -550,17 +538,16 @@ _me._fill = function (aDataIn) {
 		this.__value = aDataIn; //clone?
 		this.__idTable = [];
 
-		var bCelsius = GWOthers.getItem('CALENDAR_SETTINGS', 'temperature') == 'C',
-				aLoc;
+		var bCelsius = GWOthers.getItem('CALENDAR_SETTINGS', 'temperature') == 'C';
 
-		for (var i in this.__value) {
+		for (i in this.__value) {
 			this.__value[i].startdate = parseInt(this.__value[i].startdate, 10);
 			this.__value[i].enddate = parseInt(this.__value[i].enddate, 10);
 
 			//holidays
 			if (this.__value[i].evnclass == 'H') {
 				if (!this.__holidays)
-					for (var j = this.__value[i]['startdate']; j <= this.__value[i]['enddate']; j++) {
+					for (j = this.__value[i]['startdate']; j <= this.__value[i]['enddate']; j++) {
 
 						if (!holidays[j])
 							holidays[j] = [this.__value[i]];
@@ -613,7 +600,7 @@ _me._fill = function (aDataIn) {
 					var arr = tmpEvn.evntype.split(','),
 							at = {}, sColor;
 
-					for (var t in arr)
+					for (t in arr)
 						if ((arr[t] = arr[t].trim()).length) {
 							sColor = aTags[arr[t]] && aTags[arr[t]].TAGCOLOR ? aTags[arr[t]].TAGCOLOR : '#FFFFFF';
 
@@ -623,7 +610,7 @@ _me._fill = function (aDataIn) {
 							at[sColor].push(arr[t]);
 						}
 
-					for (var t in at) {
+					for (t in at) {
 						tmpEvn.color += '<ins title="' + at[t].join(', ').escapeHTML().replace(/"/g, '&quot;') + '" style="background-color:' + t + '"></ins>';
 						tmpEvn.taglist += '<li><ins style="background-color:' + t + '"></ins>' + at[t].join(', ').escapeHTML() + '</li>';
 					}
@@ -635,7 +622,7 @@ _me._fill = function (aDataIn) {
 				if (tmpEvn['start'] != tmpEvn['end']) {
 					var no = 0;
 					//deli se v tydnu?
-					for (var j = tmpEvn['start'] + 1; j <= tmpEvn['end']; j++) {
+					for (j = tmpEvn['start'] + 1; j <= tmpEvn['end']; j++) {
 						if (!((j - this.__range['start']) % 7)) {
 							bBreak = true;
 
@@ -663,24 +650,19 @@ _me._fill = function (aDataIn) {
 				if (!bBreak)
 					this.__idTable.push(tmpEvn);
 			}
-
-
-
-
 		}
 
 		//**** Draw holidays
 		if (!this.__holidays && !Is.Empty(holidays)) {
 			this.__holidays = true;
 
-			var aLoc;
-			for (var i in holidays)
+			for (i in holidays)
 				if ((elm = document.getElementById(this._pathName + '#' + i))) {
 
 					//Sort
 					holidays[i].sort(this.__holSort);
 
-					for (var j = 0; j < holidays[i].length; j++) {
+					for (j = 0; j < holidays[i].length; j++) {
 
 						var div = document.createElement('div');
 						div.unselectable = "on";
@@ -714,34 +696,34 @@ _me._fill = function (aDataIn) {
 
 		//**** Draw Events
 		//Rozradit do Weeku
-		var aOut = {}, iWeek, elm;
+		var aOut = {}, iWeek;
 
-		for (var i = parseInt(this.__range['start'], 10); i < this.__range['end']; i += 7)
+		for (i = parseInt(this.__range['start'], 10); i < this.__range['end']; i += 7)
 			aOut[i] = [];
 
-		for (var i = 0; i < this.__idTable.length; i++) {
+		for (i = 0; i < this.__idTable.length; i++) {
 			iWeek = Math.floor((this.__idTable[i]['start'] - this.__range['start']) / 7) * 7 + this.__range['start'];
 			if (aOut[iWeek])
 				aOut[iWeek].push(this.__idTable[i]);
 		}
 
 		//Seradit weeky
-		var elm, sHTML, infos, skip, ymax, css, noTitle = getLang('EVENT_VIEW::NO_TITLE'), privateTitle = getLang('EVENT_VIEW::PRIVATE'), t1;
-		for (var iWeek in aOut) {
+		var sHTML, infos, skip, ymax, css, noTitle = getLang('EVENT_VIEW::NO_TITLE'), privateTitle = getLang('EVENT_VIEW::PRIVATE'), t1;
+		for (iWeek in aOut) {
 			aOut[iWeek].sort(this.__evnSort);
 
 			//RENDER
 			sHTML = '';
-			y = 0;
 			ymax = 0, infos = [];
 			if ((elm = document.getElementById(this._pathName + '/' + iWeek))) {
-				for (var i in aOut[iWeek]) {
+				for (i in aOut[iWeek]) {
+					var style = '';
 					infos[i] = {};
 					aOut[iWeek][i].y = 0;
 					skip = {};
 					infos[i].title = aOut[iWeek][i].title ? aOut[iWeek][i].title : (aOut[iWeek][i].evnsharetype === 'U' ? noTitle : privateTitle);
 
-					for (var j in aOut[iWeek]) {
+					for (j in aOut[iWeek]) {
 						if (j == i)
 							break;
 						if (aOut[iWeek][j]['start'] <= aOut[iWeek][i]['start'] && aOut[iWeek][j]['end'] >= aOut[iWeek][i]['start']) {
@@ -776,8 +758,11 @@ _me._fill = function (aDataIn) {
 					if (aOut[iWeek][i].color)
 						css.push('color');
 
-					if (aOut[iWeek][i].fcolor)
-						css.push(aOut[iWeek][i].fcolor);
+					if (aOut[iWeek][i].fcolor) {
+						style = '--bgcolor: ' + aOut[iWeek][i].fcolor + '; --color: ' + colors.fast_contrast(aOut[iWeek][i].fcolor);
+					} else {
+						css.push('no_color');
+					}
 
 					if (aOut[iWeek][i].evnflags & 32)
 						css.push('unresponded');
@@ -790,7 +775,7 @@ _me._fill = function (aDataIn) {
 						css.push('O');
 
 					// Preparing date and time information for display
-					t1 = t2 = false;
+					t1 = false;
 					infos[i].start = IcewarpDate.julian(aOut[iWeek][i].startdate).setTime(aOut[iWeek][i].starttime);
 					infos[i].end = IcewarpDate.julian(aOut[iWeek][i].enddate).setTime(aOut[iWeek][i].endtime);
 					if(aOut[iWeek][i].endtime === -1){
@@ -802,8 +787,6 @@ _me._fill = function (aDataIn) {
 						infos[i].endtime = infos[i].end.format('LT');
 						if (iWeek <= aOut[iWeek][i].startdate) // Starting this week (not before)
 							t1 = infos[i].starttime;
-						if (parseInt(iWeek, 10) + 6 >= aOut[iWeek][i].enddate) // Ending this week (not later)
-							t2 = infos[i].endtime;
 					} else {
 						infos[i].starttime = '';
 						infos[i].endtime = '';
@@ -811,7 +794,7 @@ _me._fill = function (aDataIn) {
 					infos[i].duration = aOut[iWeek][i].enddate - aOut[iWeek][i].startdate;
 
 					if (infos[i].duration) { // Longer than one day
-						infos[i].duration = new IcewarpNumber(infos[i].duration + 1).localize() + ' ' + getLang(infos[i].duration + 1 ? 'TIME::MOREDAYS' : 'TIME::ONEDAY');
+						infos[i].duration = (infos[i].duration + 1).toLocaleString(document.documentElement.lang || 'en') + ' ' + getLang(infos[i].duration + 1 ? 'TIME::MOREDAYS' : 'TIME::ONEDAY');
 					} else {
 						if (infos[i].starttime && infos[i].endtime) { // During day with time
 							infos[i].duration = IcewarpDate.duration(infos[i].end.diff(infos[i].start)).humanize();
@@ -839,15 +822,12 @@ _me._fill = function (aDataIn) {
 							'width:' + ((aOut[iWeek][i]['end'] - aOut[iWeek][i]['start'] + 1) * 100) + '%;' +
 							(gui._rtl ? 'right' : 'left') + ':' + ((aOut[iWeek][i]['start'] - iWeek) * 100) + '%;' +
 							'top:' + (aOut[iWeek][i].y * this._row_height + 10) + 'px;' +
-							'" ' + (css.length ? 'class="' + css.join(' ') + '"' : '') + //<B> for OPERA
+							style + '" ' + (css.length ? 'class="' + css.join(' ') + '"' : '') +
 							'>' +
 							'<span unselectable="on"><span unselectable="on">' +
 							aOut[iWeek][i].color +
 							'<b unselectable="on">' +
 							(t1 ? '<i unselectable="on">' + t1 + '</i>' : '') +
-							//(t1?'<i unselectable="on"'+ (t1&&t2?' class="nopadding"':'') +'>'+ t1 +'</i>':'') +
-							//(t1&&t2?'<i class="middot">&middot;</i>':'') +
-							//(t2?'<i unselectable="on">'+ t2 +'</i>':'') +
 							infos[i].title.entityify() + '</b></span></span></p>';
 				}
 
@@ -856,7 +836,7 @@ _me._fill = function (aDataIn) {
 
 				// Adding tooltip with more information
 				var p = elm.getElementsByTagName('P');
-				for (var i = 0; i < p.length; i++){
+				for (i = 0; i < p.length; i++){
 					AttachEvent(p[i], 'onmouseover', this._showTooltip({template: 'obj_event_tooltip', values: infos[i]}));
 				}
 			}
@@ -868,12 +848,12 @@ _me._fill = function (aDataIn) {
 		this.__createEdit();
 	else
 	//Auto Scroll to Today
-	if (this.__autoscroll && Is.Defined(aDataIn)) {
+	if (this.__autoscroll && Is.Defined(aDataIn) && !bNoScroll) {
 		this.__autoscroll = false;
 		var id = 'gui.frm_main.main.calendar/'+(new IcewarpDate().format(IcewarpDate.JULIAN));
 		var element = document.getElementById(id);
 		if(element && element.parentElement && element.parentElement.previousElementSibling){
-			element.parentElement.previousElementSibling.scrollIntoView();
+			this.__eMain.scrollTop = getSize(element.parentElement.previousElementSibling).y;
 		}
 		var go;
 		if (this._selection.startdate && this._selection.startdate > this.__range['start'] && this._selection.startdate < this.__range['end'])
@@ -884,7 +864,7 @@ _me._fill = function (aDataIn) {
 		else
 			return;
 
-		var elm = document.getElementById(this._pathName + '/' + go);
+		elm = document.getElementById(this._pathName + '/' + go);
 		if (elm) {
 			var p1 = getSize(this.__eMain),
 					p2 = getSize(elm);
@@ -902,10 +882,9 @@ _me._fill = function (aDataIn) {
 
 
 _me.__createEdit = function () {
-
 	if (this.__edit && this.__editEvent) {
-		var me = this, elm, ids, inp,
-				itm = this._useItem(this.__editEvent);
+		var me = this, elm, ids, inp, i,
+			itm = this._useItem(this.__editEvent);
 
 		if (!itm || (!(elm = document.getElementById(this._pathName + '/' + this.__editEvent)) && (!(ids = this.__getIDs(this.__editEvent)) || !(elm = document.getElementById(this._pathName + '/' + ids[0]))))) {
 			this.__noRefresh = false;
@@ -919,7 +898,7 @@ _me.__createEdit = function () {
 		//clear TITLE from remaining parts
 		if (ids) {
 			var tmp;
-			for (var i = 1; i < ids.length; i++)
+			for (i = 1; i < ids.length; i++)
 				if ((tmp = document.getElementById(this._pathName + '/' + ids[i])))
 					tmp.innerHTML = '<span><span></span></span>';
 			tmp = null;
@@ -928,7 +907,6 @@ _me.__createEdit = function () {
 		//Bind input actions
 		if ((inp = elm.getElementsByTagName('INPUT')) && (inp = inp[0])) {
 			inp.onkeydown = function (e) {
-				var e = e || window.event;
 				switch (e.keyCode) {
 					case 27:
 						this.value = '';
@@ -939,19 +917,15 @@ _me.__createEdit = function () {
 			};
 
 			inp.ondblclick = function (e) {
-				var e = e || window.event;
-				e.cancelBubble = true;
-				if (e.preventDefault)
-					e.preventDefault();
-				return false;
+				e.preventDefault();
+				e.stopPropagation();
 			};
 
-			inp.onblur = function (e) {
-
+			inp.onblur = function () {
 				// save
 				if (this.value.length) {
 					//update in __value
-					for (var i = me.__value.length - 1; i > - 1; i--)
+					for (i = me.__value.length - 1; i > - 1; i--)
 						if (me.__value[i].id == me.__editEvent) {
 
 							//save only different Title
@@ -980,7 +954,7 @@ _me.__createEdit = function () {
 				} else
 				// destroy on Esc
 				if (me.__editEvent == 'edit')
-					for (var i = me.__value.length - 1; i > - 1; i--)
+					for (i = me.__value.length - 1; i > - 1; i--)
 						if (me.__value[i].id == me.__activeEvent) {
 							me.__value.splice(i, 1)[0];
 							break;
@@ -1101,7 +1075,7 @@ _me._value = function (v) {
 	else
 		return this.__value;
 };
-_me._range = function (aData) {
+_me._range = async function (aData) {
 	if (aData) {
 		this._active = new IcewarpDate();
 		this._dDate = new IcewarpDate();
@@ -1114,7 +1088,7 @@ _me._range = function (aData) {
 			this._dDate = new IcewarpDate(aData, {format: IcewarpDate.JULIAN}).date(1);
 		}
 
-		this._fill();
+		await this._fill();
 	} else
 		return this.__range;
 };
@@ -1155,14 +1129,14 @@ _me._listen_data = function (sDataSet, aDataPath) {
 
 _me._activate = function (info) {
 	if (Is.Defined(info) && (info.id || '') != this.__activeEvent) {
-		var elm;
+		var elm, i, tmp;
 		//deactivate
 		if (this.__activeEvent) {
 			if ((elm = document.getElementById(this._pathName + '/' + this.__activeEvent)))
 				removecss(elm, 'active');
 			else {
 				var ids = this.__getIDs(this.__activeEvent);
-				for (var i = 0; i < ids.length; i++)
+				for (i = 0; i < ids.length; i++)
 					if ((tmp = document.getElementById(this._pathName + '/' + ids[i])))
 						removecss(tmp, 'active');
 			}
@@ -1176,7 +1150,7 @@ _me._activate = function (info) {
 
 			if (info.tmp_id != info.id) {
 				ids = this.__getIDs(info.id);
-				for (var i = 0; i < ids.length; i++)
+				for (i = 0; i < ids.length; i++)
 					if ((tmp = document.getElementById(this._pathName + '/' + ids[i])))
 						addcss(tmp, 'active');
 			} else {
@@ -1192,7 +1166,7 @@ _me._activate = function (info) {
 			this.__activeEvent = '';
 
 			// Update top menu bar
-			gui.__exeEvent('folderSelected', [this._parent.__aid, this._parent.__fid]);
+			gui.__exeEvent('folderSelected', { aid: this._parent.__aid, fid: this._parent.__fid });
 		}
 
 		if (this._onactivate)
@@ -1215,7 +1189,9 @@ _me._useItem = function (id, aValue) {
 
 _me._focus = function () {
 	this.__eIN.value = '';
-	this.__eIN.focus();
+	this.__eIN.focus({
+		preventScroll: true
+	});
 };
 /**
  * Returns active Item or Item(s) inside of selection
