@@ -1,16 +1,13 @@
 _me = obj_frame_control.prototype;
 function obj_frame_control(){};
 
-_me.__constructor = function(){
+_me.__constructor = async function(){
 	var me = this;
 	this._getAnchor('header').onclick = function(e){
-		var e = e || window.event;
-		var elm = e.target || e.srcElement;
-
-		if (elm.tagName != 'A' || !elm.id) return;
+		if (e.target.tagName != 'A' || !e.target.id) return;
 
 		//Nak to nefunguje, ovlada to main window
-		switch(elm.id.substr(me._pathName.length+1)){
+		switch(e.target.id.substr(me._pathName.length+1)){
 		case 'home':
 		    if (me._homepage)
 		        me._src(me._homepage);
