@@ -1,9 +1,9 @@
 /*
 	(function(){
 
-		var frm = gui._create('frm','obj_popup');
+		var frm = await gui._create('frm','obj_popup');
 		frm._size(400,800,true);
-		frm._create('fb','obj_freebusy');
+		await frm._create('fb','obj_freebusy');
 
 		frm.fb._init({
 			users: [
@@ -31,7 +31,7 @@ obj_freebusy.__PREFETCH = 3;		// number of days
 obj_freebusy.__EVNTYPE = {UNKNOWN:'unknown', BUSY:'busy'}; //Allowed event types
 
 _me = obj_freebusy.prototype;
-_me.__constructor = function () {
+_me.__constructor = async function () {
 	this.__eBody = this._getAnchor('body');
 	this.__scrollTop = 0;
 
@@ -59,7 +59,8 @@ _me.__constructor = function () {
 		this.__timeline.time.push(iwd.setTime(i, true).format('LT'));
 	}
 
-	this._create('scrollbar', 'obj_scrollbar')._scrollbar(this.__eBody, this._getAnchor('container'));
+	await this._create('scrollbar', 'obj_scrollbar');
+	this.scrollbar._scrollbar(this.__eBody, this._getAnchor('container'));
 
 	this.__eBody.onscroll = function (e) {
 		if (this.__scrollTop != this.__eBody.scrollTop) {
@@ -73,9 +74,7 @@ _me.__constructor = function () {
 	}.bind(this);
 
 	this.__eBody.ondblclick = function(elm, e){
-
-		var e = e || window.event;
-		if (this.__plan_div && this.__plan_div.contains(e.target || e.srcElement))
+		if (this.__plan_div && this.__plan_div.contains(e.target))
 			return;
 
 		var y = gui.__Y - getSize(elm).y + elm.scrollTop,
@@ -89,10 +88,7 @@ _me.__constructor = function () {
 
 	var me = this;
 	this.__eBody.onmousedown = function(e) {
-		var e = e || window.event,
-			elm = e.target || e.srcElement;
-
-		if (this.__readonly || elm.parentNode.classList.contains('plan')) {
+		if (this.__readonly || e.target.parentNode.classList.contains('plan')) {
 			return;
 		}
 
@@ -137,7 +133,7 @@ _me.__constructor = function () {
 
 			new_event.style.top = top + 'px';
 			new_event.style.height = Math.abs(diff) + 'px';
-			difference = Math.round((end - start) / 360000) / 10;
+			var difference = Math.round((end - start) / 360000) / 10;
 			duration.textContent = getLang('TIME_INTERVAL::DURATION') + ': ' + Math.floor(difference) + ':' + ('0' + (difference % 1 * 60)).slice(-2) + 'h';
 		};
 
@@ -183,7 +179,7 @@ _me.__constructor = function () {
 	this._main.appendChild(eFrame);
 
 	var height = eFrame.offsetHeight;
-	eFrame.contentWindow.onresize = function (e) {
+	eFrame.contentWindow.onresize = function () {
 		if (height != eFrame.offsetHeight) {
 			height = eFrame.offsetHeight;
 
@@ -193,7 +189,7 @@ _me.__constructor = function () {
 	}.bind(this);
 
 	//Add PLAN plug-in object
-	this._create('plan', 'obj_freebusy_plan', 'block', '', this);
+	await this._create('plan', 'obj_freebusy_plan', 'block', '', this);
 
 	this._add_destructor('__destructor');
 };
@@ -250,6 +246,9 @@ _me._onscroll = function(v){
 };
 
 _me._dateLabel = function(){
+	if (this._destructed) {
+		return;
+	}
 	var iwd = IcewarpDate.julian(this.__range[0] + Math.floor((this.__eBody.scrollTop + obj_freebusy.__SLOTHEIGHT)/obj_freebusy.__DAYHEIGHT)),
 		elm = this._getAnchor('date');
 
@@ -316,6 +315,7 @@ _me._value = function(aValue, bNoUpdate, bNoScroll, aOpt){
 
 			if (!bNoScroll){
 				this.__scroll(this.__position((aValue.startdate - this.__range[0])*1440 + aValue.start_t) - Math.ceil(this.__eBody.offsetHeight/8), true);
+				this._dateLabel();
 			}
 		}
 		//outside
@@ -441,7 +441,7 @@ _me._users = function(aUsers){
 		this.__opt.users = aUsers;
 };
 	_me.__removeUser = function(sUser){
-		var sUser = sUser.toLowerCase();
+		sUser = sUser.toLowerCase();
 		for (var id in this.__aData){
 			if (this.__aData[id].ACCOUNT.toLowerCase() == sUser)
 				delete this.__aData[id];
@@ -495,7 +495,7 @@ _me.__request = function(aRange, aUsers, aOpt){
 
 _me.__clean = function(){
 	var tmp = this.__eBody.querySelectorAll('div.event');
-	for(i = tmp.length; i--;){
+	for(var i = tmp.length; i--;){
 		tmp[i].parentNode && tmp[i].parentNode.removeChild(tmp[i]);
 	}
 };
@@ -507,7 +507,7 @@ _me.__clean = function(){
  * @param {object} aData
  * @param {array} aRange
  */
-_me.__response = function(bOK, aData, aRange, aUsers, rqid, aOpt){
+_me.__response = async function(bOK, aData, aRange, aUsers, rqid, aOpt){
 	if (!bOK || this._destructed || this.__lastRq != rqid){
 		return;
 	}
@@ -575,7 +575,7 @@ _me.__response = function(bOK, aData, aRange, aUsers, rqid, aOpt){
 	var aEvents = this.__prepare(this.__aData);
 
 	//print days
-	this.__days(days);
+	await this.__days(days);
 
 	//set scroll
 	if (aOpt.scroll){
@@ -605,13 +605,13 @@ _me.__response = function(bOK, aData, aRange, aUsers, rqid, aOpt){
  *
  * @param {array} days range
  */
-_me.__days = function(days){
+_me.__days = async function(days){
 
-	var iwd, tplData = this.__timeline;
+	var iwd, tplData = this.__timeline, jul;
 
 	if (days.prepend){
 		tplData.day = [];
-		for(var jul = days.prepend[0]; jul <= days.prepend[1]; jul++){
+		for(jul = days.prepend[0]; jul <= days.prepend[1]; jul++){
 			iwd = IcewarpDate.julian(jul);
 			tplData.day.push({
 				date: iwd.format('MMM D'),
@@ -620,12 +620,12 @@ _me.__days = function(days){
 			});
 		}
 
-		this._draw('obj_freebusy_day', 'body', tplData, 2);
+		await this._draw('obj_freebusy_day', 'body', tplData, 2);
 	}
 
 	if (days.append){
 		tplData.day = [];
-		for(var jul = days.append[0]; jul <= days.append[1]; jul++){
+		for(jul = days.append[0]; jul <= days.append[1]; jul++){
 			iwd = IcewarpDate.julian(jul);
 			tplData.day.push({
 				date: iwd.format('MMM D'),
@@ -634,7 +634,7 @@ _me.__days = function(days){
 			});
 		}
 
-		this._draw('obj_freebusy_day', 'body', tplData, !days.clean);
+		await this._draw('obj_freebusy_day', 'body', tplData, !days.clean);
 	}
 };
 
@@ -652,10 +652,7 @@ _me.__replan = function(bCollision){
 		var me = this;
 
 		this.__plan_div.onmousedown = function(e){
-
-			var e = e || window.event,
-				elm = e.target || e.srcElement;
-
+			var elm = e.target;
 			if (elm === this || me.__readonly) {
 				return;
 			} else if (elm === eControls[2] || elm === eControls[3]) {
@@ -666,11 +663,11 @@ _me.__replan = function(bCollision){
 				r = me.__range[0],
 				old = clone(me.__value);
 
-			function move (e){
+			function move (){
 
 				var dif = gui.__Y - x + me.__eBody.scrollTop - (r-me.__range[0])*obj_freebusy.__DAYHEIGHT,
 					dif_time = Math.round(dif/obj_freebusy.__SLOTHEIGHT)*30,
-					v;
+					v, iwtS, iwtE;
 
 				if (dif<=0 && !dif_time){
 					v = old;
@@ -692,8 +689,8 @@ _me.__replan = function(bCollision){
 					else
 					//RESIZE Start
 					if (elm === eControls[2]){
-						var iwtS = IcewarpDate.julian(old.startdate,old.starttime),
-							iwtE = IcewarpDate.julian(old.enddate,old.endtime);
+						iwtS = IcewarpDate.julian(old.startdate,old.starttime);
+						iwtE = IcewarpDate.julian(old.enddate,old.endtime);
 
 						iwtS.add(dif_time,'m');
 
@@ -706,8 +703,8 @@ _me.__replan = function(bCollision){
 					else
 					//RESIZE End
 					if (elm === eControls[3]){
-						var iwtS = IcewarpDate.julian(old.startdate,old.starttime),
-							iwtE = IcewarpDate.julian(old.enddate,old.endtime);
+						iwtS = IcewarpDate.julian(old.startdate,old.starttime);
+						iwtE = IcewarpDate.julian(old.enddate,old.endtime);
 
 						iwtE.add(dif_time,'m');
 
@@ -772,8 +769,8 @@ _me.__replan = function(bCollision){
  * @returns {object}
  */
 _me.__prepare = function(aData, aRange){
-	var aOut = [], evn,
-		aRange = aRange || this.__range;
+	var aOut = [], evn;
+	aRange = aRange || this.__range;
 
 	for(var id in aData){
 		evn = aData[id];
@@ -818,7 +815,7 @@ _me.__prepare = function(aData, aRange){
 
 	// move block into holes :)
 	var old_j,
-		j = 0,
+		j = 0, k, l,
 		block,
 		iend = 0,
 		relPosition = 0;
@@ -831,7 +828,7 @@ _me.__prepare = function(aData, aRange){
 		relPosition = 0;
 
 		//najit celou posloupnost bloku az do urovne 0
-		for (var k = j + 1, l = aOut.length; k < l; k++) {
+		for (k = j + 1, l = aOut.length; k < l; k++) {
 			if (iend < block.end)
 				iend = block.end;
 
@@ -862,13 +859,13 @@ _me.__prepare = function(aData, aRange){
 
 	// delimiter
 	var pos, del;
-	for (var j = 0, l = aOut.length; j < l; j++) {
+	for (j = 0, l = aOut.length; j < l; j++) {
 
 		if (aOut[j]['position'] == 0) {
 
 			//zjistit jestli ho neco prekrejva, pokud ne tak continue
 			var prekrejva = -1;
-			for (var k = j; k < l; k++) {
+			for (k = j; k < l; k++) {
 
 				if (aOut[k]['position'] > aOut[j]['position']) {
 					if (aOut[k]['start'] < aOut[j]['end'])
@@ -883,7 +880,7 @@ _me.__prepare = function(aData, aRange){
 			// zjistit pocet columns z potomku
 			del = 1;
 			pos = 0;
-			for (var k = prekrejva; k < l; k++) {
+			for (k = prekrejva; k < l; k++) {
 				if (aOut[k]['position'] < pos)
 					break;
 				if (aOut[k]['position'] == pos)
@@ -898,7 +895,7 @@ _me.__prepare = function(aData, aRange){
 
 				var dif = 0;
 
-				for (var l = j + 1; l < k; l++) {
+				for (l = j + 1; l < k; l++) {
 					if (aOut[j]['position'] < aOut[l]['position'] && aOut[j]['start'] < aOut[l]['end'] && aOut[j]['end'] > aOut[l]['start']) {
 						dif = aOut[l]['position'] - aOut[j]['position'];
 						break;
@@ -1125,7 +1122,7 @@ _me._find = function(aHandler){
 			}
 			else{
 				//error
-				console.warn('freebusy', 'get data error');
+				 console.warn('freebusy', 'get data error');
 				out = [];
 			}
 
@@ -1136,7 +1133,9 @@ _me._find = function(aHandler){
 					try{
 						removecss(this.plan._main, 'notfound');
 					}
-					catch(r){}
+					catch {
+						//
+					}
 				}.bind(this), 3600);
 			}
 
@@ -1152,7 +1151,8 @@ _me._find = function(aHandler){
 		var workstart = GWOthers.getItem('CALENDAR_SETTINGS','day_begins')*60,
 			workend = GWOthers.getItem('CALENDAR_SETTINGS','day_ends')*60,
 			bgn = GWOthers.getItem('CALENDAR_SETTINGS','workweek_begins')*1,
-			end = GWOthers.getItem('CALENDAR_SETTINGS','workweek_ends')*1;
+			end = GWOthers.getItem('CALENDAR_SETTINGS','workweek_ends')*1,
+			slot, slots = [], evn, i;
 
 		if (bgn<1 || bgn>7) bgn = 1;
 		if (end<1 || end>7) bgn = 5;
@@ -1180,7 +1180,7 @@ _me._find = function(aHandler){
 		}
 
 		//mask work-hours, work-days and weekends
-		for(var i = range[0]-1;i<=range[1]; i++){
+		for(i = range[0]-1;i<=range[1]; i++){
 
 			if (i == jul){
 				var tim = iwt.format('julian_time');
@@ -1218,7 +1218,7 @@ _me._find = function(aHandler){
 		});
 
 		//Find free slots between events
-		for (var slot, slots = [], evn, i = 0; (evn = events[i]);){
+		for (slots = [], i = 0; (evn = events[i]);){
 
 			//free slot
 			while(true){
@@ -1260,7 +1260,7 @@ _me._find = function(aHandler){
 		}
 
 		//DEBUG
-		if (gui._REQUEST_VARS.xml == '1'){
+		if (gui._REQUEST_VARS.debug){
 			console.warn('range', range);
 			console.warn('events', events);
 			console.warn('slots', slots);
