@@ -1,18 +1,18 @@
 _me = obj_groupchat_conference.prototype;
 function obj_groupchat_conference(){};
 
-_me.__constructor = function() {
+_me.__constructor = async function() {
 
 	var linkextras = {};
 
 	if (this.__aData.EVNLINKEXTRAS)
 		linkextras = parseURL(this.__aData.EVNLINKEXTRAS);
 
-	this._draw('obj_groupchat_conference', 'addon');
+	await this._draw('obj_groupchat_conference', 'addon', '', 2);
 
 	if (linkextras.started == '1' && linkextras.id){
-		this.btn_accept._onclick = function(){
-			storage.library('wm_conference');
+		this.btn_accept._onclick = async function(){
+			await storage.library('wm_conference');
 			wm_conference.get(linkextras.id).join();
 		};
 
@@ -43,7 +43,7 @@ _me.__update = function(){
 			// 	this._init_body(aData);
 
 			//Reactions
-			this._init_reactions(aData);
+			this._init_toolbar(aData);
 		}
 
 	}.bind(this)]);
