@@ -3,9 +3,9 @@ function obj_input_email(){
 	this.__disabled = false;
 };
 
-_me.__constructor = function(){
-	this._create('inp','obj_input','','obj_input_100');
-	this._create('btn','obj_button','','simple ico img');
+_me.__constructor = async function(){
+	await this._create('inp','obj_input','','obj_input_100');
+	await this._create('btn','obj_button','','simple ico img');
 	this.btn._onclick = function (){
 		if(this.__disabled) return;
 
