@@ -1,7 +1,7 @@
 _me = obj_input_number.prototype;
 function obj_input_number(){};
 
-_me.__constructor = function(){
+_me.__constructor = async function(){
 	var me = this;
 
 	this.__range = [];
@@ -10,7 +10,7 @@ _me.__constructor = function(){
 			'sub':['&#xe043;','',2],
 			'add':['&#xe044;','',2]
 		},
-		[function(id){
+		[async function(id){
 			if (!me._disabled()){
 
 				var i = parseInt(me._value(),10) || 0;
@@ -25,7 +25,7 @@ _me.__constructor = function(){
 					if (!Is.Number(me.__range[0]) || i>me.__range[0])
 						me._value(i - 1);
 				}
-				if(!me.__check()) {
+				if(!await me.__check()) {
 					me._value(i);
 				}
 			}
@@ -33,7 +33,7 @@ _me.__constructor = function(){
 		true
 	);
 
-	this._obeyEvent('onkeyup',[function(e){
+	this._obeyEvent('onkeyup',[function(){
 		var v = me.__eIN.value.replace(/[^0-9]/g,'') || 0;
 
 		if (Is.Number(me.__range[0]))
