@@ -5,7 +5,7 @@ function obj_input_search(){};
  * @brief: CONSTRUCTOR,
  * @date : 21.3.2007 13:19:41
  **/
-_me.__constructor = function(){
+_me.__constructor = async function(){
 	var me = this;
 
 	this.__setMask({'clear':['&#xe036;',getLang('COMMON::CLEAR')]},
@@ -23,7 +23,7 @@ _me.__constructor = function(){
 			}
 		}]);
 
-	this._main.appendChild(mkElement('a',{className:'input_ico',onclick:function(e){
+	this._main.appendChild(mkElement('a',{className:'input_ico',onclick:function(){
 		if (!me._onsubmit || !me._onsubmit())
 			me._focus();
 	}}));
