@@ -5,8 +5,8 @@ function obj_input_search_auto(){};
  * @brief: CONSTRUCTOR,
  * @date : 21.3.2007 13:19:41
  **/
-_me.__constructor = function(){
-	this.__timeout; //global variable for timeout session
+_me.__constructor = async function(){
+	this.__timeout;
     this._obeyEvent('onkeyup',[this,'__starttimer']);
     
 	var me = this;
@@ -26,11 +26,14 @@ _me.__constructor = function(){
 };
 
 _me.__starttimer = function(){
+	var me = this;
 	if (this.__timeout) clearTimeout(this.__timeout);
-	this.__timeout = setTimeout("try{"+this._pathName+"._search();}catch(er){gui._REQUEST_VARS.debug && console.log(this._name||false,er);}",1000);
+	this.__timeout = setTimeout(function() {
+		!me._destructed && me._search();
+	}, 1000);
 };
 
 _me._search = function(){
 	if (this._onsearch) this._onsearch(this._value());
 	this.__exeEvent('onsearch',null,{"owner":this,"value":this._value()});
-};
\ No newline at end of file
+};
