@@ -5,7 +5,7 @@ function obj_input_search_next(){};
  * @brief: CONSTRUCTOR,
  * @date : 21.3.2007 13:19:41
  **/
-_me.__constructor = function(){
+_me.__constructor = async function(){
 	var me = this;
 
 	this.__setMask({
