@@ -3,21 +3,21 @@ function obj_input_url(){
 	this.__disabled = false;
 };
 
-_me.__constructor = function(bText){
+_me.__constructor = async function(bText){
 	var me = this;
 	this._features = 'toolbar=1,menubar=1,status=1,location=1,scrollbars=1';
 
 	if (bText)
-		this._create('inp','obj_text');
+		await this._create('inp','obj_text');
 	else
-		this._create('inp','obj_input','','obj_input_100');
+		await this._create('inp','obj_input','','obj_input_100');
 		
 	this.inp._onclick = function(){
 		if (me._onclick && !me.__disabled)
 			me._onclick();
 	};
 
-	this._create('btn','obj_button','','simple ico img');
+	await this._create('btn','obj_button','','simple ico img');
 	this.btn._onclick = function (){
 		if(me.__disabled) return;
 
