@@ -2,7 +2,7 @@ _me = obj_itemview_notify.prototype;
 function obj_itemview_notify(){};
 
 //// Notifications ////
-_me.__constructor = function() {
+_me.__constructor = async function() {
 	// PUSH Notifications
 	if (gui.socket){
 		this._add_destructor('__notify_off');
