@@ -1,11 +1,11 @@
 _me = obj_label_phone.prototype;
 function obj_label_phone(){};
 
-_me.__constructor = function(){
+_me.__constructor = async function(){
 	this.__showSMS = false;
 };
 
-_me._onclick = function(e){
+_me._onclick = async function(e){
 	var v = this._value(),
 		aMenu = [];
 
@@ -27,17 +27,14 @@ _me._onclick = function(e){
 		}
 
 		if (aMenu.length){
-			// e.__source = {obj:this, skip:true, type:this._type, path:this._pathName};
-
 			var pos = getSize(this._main);
 
-			this.__cmenu = gui._create('cmenu','obj_context'),
-			this.__cmenu._fill(aMenu);
+			this.__cmenu = await gui._create('cmenu','obj_context'),
+			await this.__cmenu._fill(aMenu);
 			this.__cmenu._place(pos.x+pos.w/2,pos.y,'',3);
 
-			if (e.stopPropagation) e.stopPropagation();
-			e.cancelBubble = true;
-			return false;
+			e.stopPropagation();
+			e.preventDefault();
 		}
 	}
 };
