@@ -1,7 +1,7 @@
 _me = obj_labeled_select.prototype;
 function obj_labeled_select(){};
 
-_me.__constructor = function(sTitle) {
+_me.__constructor = async function(sTitle) {
 	if (Is.Defined(sTitle))
 	    this.label._value(sTitle);
 };
