@@ -8,7 +8,7 @@ _me.__bindobj = function(){
 			obj = obj[part];
 		});
 	}
-	catch(er){
+	catch {
 		obj = null;
 	}
 	return obj;
@@ -22,7 +22,7 @@ _me._bind = function(obj){
 		    this.__bind.__bindlist = null;
 		    this.__bind = null;
 		}
-		catch(er){gui._REQUEST_VARS.debug && console.log(this._name||false,er);}
+		catch(er){ console.log(this._name||false,er);}
 	}
 
 	//bind new object
