@@ -6,5 +6,9 @@ function obj_loader(){};
  * @date : 4.1.2007 12:21:09
  **/
 _me._value = function(s){
+	if (typeof s === 'undefined') {
+		return this._getAnchor('content').innerHTML;
+	}
 	this._getAnchor('content').innerHTML = s;
+	return this;
 };
\ No newline at end of file
