@@ -9,8 +9,6 @@ function obj_popup(){
  *		_onresize
  **/
 
-obj_popup._oldPositioning = currentBrowser() == 'MSIE7' || (navigator && navigator.browser && navigator.browser.engine == 'WebKit');
-
 obj_popup.activeStack = {
 	//PRIVATE
 	__stack: [],
@@ -32,7 +30,7 @@ obj_popup.activeStack = {
 	// Focus next window below the docked one
 	focusNext: function(){
 		var obj = this.get();
-		return obj && obj._focus && obj._focus();
+		return obj && obj._focus && obj._focus(true);
 	},
 	get: function(id){
 		return this.__stack[id || 0];
@@ -40,7 +38,7 @@ obj_popup.activeStack = {
 	//Re-order modal popups
 	order: function(){
 		for (var i = 0, j = this.__stack.length; i<j; i++)
-			if (this.__stack[i] && this.__stack[i]._isModal() && this.__stack[i]._focus){
+			if (this.__stack[i] && !this.__stack[i]._destructed && this.__stack[i]._isModal() && this.__stack[i]._focus){
 				this.__stack[i]._focus();
 				break;
 			}
@@ -53,29 +51,32 @@ AttachEvent(document, "onkeydown", function(e) {
 		var p = obj_popup.activeStack.get();
 		if (p){
 			if ((e.target.nodeName === 'INPUT' || e.target.nodeName === 'TEXTAREA')){
-				if (!p._docked && !p.__hidden && p.__closable && p._main.contains(e.target)){
+				if (!p.__detached && !p._docked && !p.__hidden && p.__closable && p._main.contains(e.target)){
 					p._close(true, e);
 				}
 			}
 			else
-			if (!p._docked && !p.__hidden && p.__closable){
+			if (!p.__detached && !p._docked && !p.__hidden && p.__closable){
 				p._close(true, e);
 			}
 		}
 	}
 });
 
-_me.__constructor = function(owner){
+_me.__constructor = async function(owner){
 	this._owner = owner;
 
-	this.__resizable= true;
-	this.__closable	= true;
-	this.__dockable	= true;
-	this.__moveble	= true;
+	this.__resizable	= true;
+	this.__closable		= true;
+	this.__dockable		= true;
+	this.__moveble		= true;
+	this.__detachable	= false;
 
 	this.__zindex;
 	this.__position = {}; // position memory for fullscrean
 	this.__positionShadow = {}; //real possition memory (when shifted)
+	this.__positionOffset = {}; //Shift offset
+
 	this.__hidden = false;
 
 	var me = this;
@@ -100,9 +101,15 @@ _me.__constructor = function(owner){
 
 	var sResize;
 
+	this.__defaultSizeOptions = {
+		maximized: false,
+		goldenRatio: false,
+		minWidth: 300,
+		minHeight: 150
+	};
+
 	this.__eContainer.onmousemove = function(e){
-		var e = e || window.event,
-			elm = e.target || e.srcElement;
+		var elm = e.target;
 
 		if (this !== elm || !me.__resizable || me.__position.max) {
 			this.style.cursor = 'default';
@@ -166,15 +173,11 @@ _me.__constructor = function(owner){
 				sResize = 'b';
 		}
 	};
-	//msie9 fix, cursor is hanging
-	this.__eContainer.onmouseout = function(e){
-		this.style.cursor = 'default';
-	};
 
 	/* REMOVE EVENT */
-	function dispatch(e){
+	function dispatch(){
 		me.__hideBlock();
-		gui._disobeyEvent('mousemove',[me,'__move_handler']);
+		me._gui._disobeyEvent('mousemove',[me,'__move_handler']);
 
 		if (me._onresize_end)
 			me._onresize_end();
@@ -182,32 +185,12 @@ _me.__constructor = function(owner){
 		return false;
 	};
 
-
-	//DblClick on Title and Ico for MSIE
-	if (currentBrowser() == 'MSIE7')
-		this.__eContainer.ondblclick = function(e){
-			var e = e || window.event;
-			var elm = e.target || e.srcElement;
-			switch (elm.id){
-			case me._anchors['top']:
-			case me._anchors['title']:
-				me._maximize();
-				if (me._onresize) me._onresize(e,'max');
-				me.__exeEvent('onresize',e,{"owner":me,"main":me._main,"body":me.__eContainer,"type":'max'});
-
-				return;
-
-			case me._pathName + '#ico':
-			    me._close(true, e);
-			}
-		};
-
 	this.__eContainer.onmousedown = function(e){
 
-		var e = e || window.event,
-			elm = e.target || e.srcElement,
+		var elm = e.target,
 			disp = false,
-			bStop = false;
+			bStop = false,
+			t = +new IcewarpDate();
 
 		// set max zindex
 		me._focus();
@@ -227,9 +210,7 @@ _me.__constructor = function(owner){
 			case '#title_text':
 			case '#title':
 
-				//DblClick on title for other then MSIE browsers
 				if (!this.ondblclick)
-					var t = +new IcewarpDate();
 					if (this.__lastClick && this.__lastClick+600>t){
 						this.__lastClick = 0;
 
@@ -247,19 +228,13 @@ _me.__constructor = function(owner){
 
 				me.__showBlock('m');
 
-				// var difX = e.clientX - me.__eContainer.offsetLeft;
-				// var difY = e.clientY - me.__eContainer.offsetTop;
-
 				var difX = e.clientX - me.__positionShadow.x,
 					difY = e.clientY - me.__positionShadow.y;
 
-				if (me.__move_handler)
-					gui._disobeyEvent('mousemove',[me,'__move_handler']);
-
 				me.__move_handler = function(e){
-					me._place(e.clientX - difX,e.clientY - difY);
+					me._place(Math.max(e.clientX - difX, 0), Math.max(e.clientY - difY, 0));
 				};
-				gui._obeyEvent('mousemove',[me,'__move_handler']);
+				me._gui._obeyEvent('mousemove',[me,'__move_handler']);
 				if (me._onmove)
 					me._onmove(e);
 
@@ -267,6 +242,35 @@ _me.__constructor = function(owner){
 
 				break;
 
+			case '#ico':
+                if (this.ondblclick)
+                    break;
+				else
+				{
+					if (!this.__lastClick || this.__lastClick+600<t){
+						this.__lastClick = t;
+						break;
+					}
+				}
+		}
+
+		if (disp || bStop){
+			//propagate click to close context-menus, etc...
+			me._main.ownerDocument.onclick && me._main.ownerDocument.onclick({type:'click', 'clientY':e.clientY, 'clientX':e.clientX,'target':elm});
+
+			if (disp)
+				me._gui._obeyEvent('mouseup',[dispatch,[e]]);
+
+			if (bStop){
+				e.stopPropagation();
+			}
+			e.preventDefault();
+		}
+	};
+
+	this.__eContainer.onclick = function(e) {
+		var bStop;
+		switch(e.target.id.substr(me._pathName.length)) {
 			// maximalize
 			case '#max':
 				if (e.button<2){
@@ -284,40 +288,24 @@ _me.__constructor = function(owner){
 
 				bStop = true;
 				break;
-			// close by ico for other then MSIE
-			case '#ico':
-                if (this.ondblclick)
-                    break;
-				else
-				{
-					var t = +new IcewarpDate();
-					if (!this.__lastClick || this.__lastClick+600<t){
-						this.__lastClick = t;
-						break;
-					}
-				}
+			// detach
+			case '#detach':
+				// time to allow cleanup
+				setTimeout(me._detach.bind(me), 5);
+				bStop = true;
+				break;
 			// close
 			case '#rem':
 				if (me.__closable && e.button<2)
 					me._close(true, e);
 
-				bStop = true;
+				// bStop = true;
 				break;
 		}
 
-		if (disp || bStop){
-			//propagate click to close context-menus, etc...
-			document.onclick({type:'click', 'clientY':e.clientY, 'clientX':e.clientX,'target':elm});
-
-			if (disp)
-				gui._obeyEvent('mouseup',[dispatch,[e]]);
-
-			if (bStop){
-				e.cancelBubble = true;
-				if (e.preventDefault) e.preventDefault();
-				if (e.stopPropagation) e.stopPropagation();
-			}
-			return false;
+		if (bStop){
+			e.preventDefault();
+			e.stopPropagation();
 		}
 	};
 
@@ -325,11 +313,13 @@ _me.__constructor = function(owner){
 	this.__eContainer.onmouseup = function(e){
 		if (!Is.Child(e.target, me.__eMain))
 			if (!me._preventfocus && me._activeElement) {
-				if (document.activeElement != me._activeElement)
-					me._activeElement.focus();
+				if (me._main.ownerDocument.activeElement != me._activeElement)
+					me._activeElement.focus({
+						preventScroll: true
+					});
 			}
 			else
-				document.activeElement && document.activeElement.blur && document.activeElement.blur();
+			me._main.ownerDocument.activeElement && me._main.ownerDocument.activeElement.blur && me._main.ownerDocument.activeElement.blur();
 	};
 
 	this.__eMain.scrollTop = 0;
@@ -358,19 +348,38 @@ _me.__constructor = function(owner){
 	obj_popup.activeStack.order();
 
 	//window resize
-	gui._obeyEvent('resize',[this,'__checkPosition']);
+	me._gui._obeyEvent('resize',[this,'__checkPosition']);
 
-	AttachEvent(this._main, "onfocusin", function(e) {
+	AttachEvent(this._main, "onfocusin", function() {
 		me._activate();
 	});
 
+	gui.tooltip._add(this._getAnchor('detach'), getLang('POPUP_ITEMS::OPEN_WINDOW'));
+	gui.tooltip._add(this._getAnchor('min'), getLang('POPUP_ITEMS::MINIMIZE'));
+	gui.tooltip._add(this._getAnchor('max'), getLang('POPUP_ITEMS::MAXIMIZE'));
+	gui.tooltip._add(this._getAnchor('rem'), getLang('FORM_BUTTONS::CLOSE'));
 };
 
+_me._destruct = async function(bForce) {
+	if (bForce && this.__detached) {
+		this.__detached.onbeforeunload = false;
+	}
+	await Gui.prototype._destruct.call(this);
+}
+
+_me._detachable = function(bDetachable) {
+	if (bDetachable === void 0 || ~currentBrowser().indexOf('MSIE')) {
+		return this.__detachable;
+	}
+	this.__detachable = bDetachable;
+	(bDetachable ? removecss : addcss)(this._getAnchor('detach'), 'hidden');
+}
+
 _me._dockable = function(b){
-	this._getAnchor('min').style.display = (this.__dockable = (b?true:false))?'':'none';
+	this._getAnchor('min').style.display = ((this.__dockable = (b?true:false)))?'':'none';
 };
 _me._resizable = function(b){
-	this._getAnchor('max').style.display = (this.__resizable	= (b?true:false))?'':'none';
+	this._getAnchor('max').style.display = ((this.__resizable = (b?true:false)))?'':'none';
 };
 _me._closable = function(b){
 	if ((this.__closable = b?true:false))
@@ -385,54 +394,21 @@ _me._autofocusfields = function(b) {
 	this._preventfocus = !b;
 };
 
-// Show in full screen mode (elm to show, by default popup content)
-_me._fullscreen = function(elm) {
-	if(!elm)
-		elm = this._getAnchor('box') || this.__eMain;
-
-	var fullelm = this._isfullscreen();
-	var fullscreen = elm.requestFullscreen || elm.msRequestFullscreen || elm.mozRequestFullScreen || elm.webkitRequestFullscreen;
-	if(fullscreen && !fullelm) {
-		// Request fullscreen for element
-		fullscreen.apply(elm);
-		// Prevent key press events from propagating to normal interface
-		if(fullelm = this._isfullscreen())
-			fullelm.onkeydown = function(e) {
-				e.stopPropagation();
-			};
-	} else if(fullelm) {
-		// Exit full screen mode
-		var exit = document.exitFullscreen || document.msExitFullscreen || document.mozCancelFullScreen || document.webkitExitFullscreen;
-		exit.apply(document);
+_me.__hide = async function(message, bSkipFocusNext) {
+	if (this.__detached) {
+		this.loader = await this._gui._create('loader', 'obj_loader');
+		message && this.loader._value(getLang(message));
+		return;
 	}
-};
-// Check if currently in full screen mode
-_me._isfullscreen = function() {
-	return document.fullscreenElement || document.msFullscreenElement || document.mozFullScreenElement || document.webkitFullscreenElement;
-};
 
-_me.__onCreateChild = function(sName,sType,sTarget){
-	if (sTarget == 'footer' || sTarget == 'header'){
-		var elm = this._getAnchor(sTarget);
-			elm.style.display = 'block';
-
-		this._getAnchor('box').style[sTarget=='header'?'paddingTop':'paddingBottom'] = elm.offsetHeight + 'px';
-		elm = null;
+	if (this._docked) {
+		this.__dock.classList.add('hidden');
+		gui.frm_main.dock.__checkSize();
 	}
-};
-_me.__onDestroyChild = function(sName,sType,sTarget){
-	if (sTarget == 'footer' || sTarget == 'header'){
-		if (this._getChildObjects(sTarget).length>0)
-			this._getAnchor('box').style[sTarget=='header'?'paddingTop':'paddingBottom'] = this._getAnchor(sTarget).offsetHeight + 'px';
-		else{
-			this._getAnchor(sTarget).style.display = 'none';
-			this._getAnchor('box').style[sTarget=='header'?'paddingTop':'paddingBottom'] = 0;
-		}
-	}
-};
 
-_me.__hide = function(){
-	this.__eContainer.style.zIndex = -1000;
+	this.__eContainer.style.zIndex = -1;
+	this.__eContainer.style.visibility = 'hidden';
+	this._main.setAttribute('aria-hidden', '');
 
 	this.__hidden = true;
 
@@ -442,13 +418,28 @@ _me.__hide = function(){
 
 	// Remove docked windows from active windows stack
 	obj_popup.activeStack.remove(this);
-	// Focus next window below the docked one
-	obj_popup.activeStack.focusNext();
+	if (!bSkipFocusNext) {
+		// Focus next window below the docked one
+		obj_popup.activeStack.focusNext();
+	}
 
 	this.__exeEvent('onhide','',{"owner":this});
 };
+
 _me.__show = function(){
+	if (this.__detached) {
+		this.loader && this.loader._destruct();
+		return;
+	}
+
+	if (this._docked) {
+		this.__dock.classList.remove('hidden');
+		gui.frm_main.dock.__checkSize();
+	}
+
 	this.__eContainer.style.zIndex = this.__zindex;
+	this.__eContainer.style.visibility = 'visible';
+	this._main.removeAttribute('aria-hidden');
 
 	this.__hidden = false;
 
@@ -470,24 +461,21 @@ _me.__showBlock = function(sResize){
 
 	if (!this.__maskDIV){
 		this.__maskDIV = mkElement('div',{className:'obj_popup_mask'});
-
-		var me = this;
 		this.__maskDIV.onclick = function(){
-			me.__hideBlock();
-		};
+			this.__hideBlock();
+		}.bind(this);
 
-		document.getElementsByTagName('body')[0].appendChild(this.__maskDIV);
+		document.body.appendChild(this.__maskDIV);
 	}
 
 	if (sResize)
-		this.__maskDIV.style.cursor = {m:'Move',tl:'se-resize',lb:'sw-resize',l:'w-resize',rt:'sw-resize',rb:'se-resize',r:'w-resize',t:'s-resize',b:'s-resize'}[sResize];
-
+		this.__maskDIV.style.cursor = {m:'Move',lt:'se-resize',lb:'sw-resize',l:'w-resize',rt:'sw-resize',rb:'se-resize',r:'w-resize',t:'s-resize',b:'s-resize'}[sResize];
 };
 _me.__hideBlock = function(){
 	if (this.__maskDIV){
 		try{
 			this.__maskDIV.parentNode.removeChild(this.__maskDIV);
-		}catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+		}catch(e){ console.log(this._name||false,e)}
 		this.__maskDIV = null;
 
 		return true;
@@ -496,16 +484,19 @@ _me.__hideBlock = function(){
 };
 
 
-_me._dock = function (skip){
-	var dock = this._mydock || gui._dock,
-		sTitle = false,
-		sCSS = '';
+_me._dock = function (skip, sCSS, bUnshift){
+	if (this._docked) {
+		return;
+	}
 
-	if (!this.__dockable || !dock) return;
+	var sTitle = false;
+	sCSS = sCSS || '';
+
+	if (!this.__dockable) return;
 
 	// If implemented, fire ondock
 	if (this._ondock){
-		var out = this._ondock(dock);
+		var out = this._ondock();
 		// If returns falsy value, skip docking
 		if (!out)
 			return;
@@ -543,30 +534,31 @@ _me._dock = function (skip){
 		}
 	}
 
+	this._main.setAttribute('aria-hidden', '');
+
 	this._docked = true;
 
-	if (!skip) dock._add(this,sTitle,sCSS);
+	if (!skip) gui.frm_main.dock._add(this,sTitle,sCSS,bUnshift);
 
 	return true;
 };
 
 
 _me._undock = function (skip){
-	var dock = this._mydock || gui._dock;
-	if (dock && this._docked){
-
+	if (this._docked){
+		this._main.removeAttribute('aria-hidden');
 		this.__eContainer.style.marginLeft = 'auto';
 		this.__eContainer.style.visibility = 'visible';
 
 		this._docked = false;
 
 		if (!skip)
-			dock._remove(this);
+			gui.frm_main.dock._remove(this);
 
 		this._focus();
 
 		if (this._onundock)
-			this._onundock(dock);
+			this._onundock();
 
 		return true;
 	}
@@ -582,7 +574,7 @@ _me.__resize = function(e,sResize){
 
 	//remove old handler
 	if (this.__move_handler){
-		gui._disobeyEvent('mousemove',[this,'__move_handler']);
+		me._gui._disobeyEvent('mousemove',[this,'__move_handler']);
 		this.__move_handler = null;
 	}
 
@@ -592,21 +584,20 @@ _me.__resize = function(e,sResize){
 
 	switch(sResize){
 	case 't' :
-		difY = e.clientY;
+		difY = e.clientY - position.y;
 		dif1 = position.h;
 		dif2 = position.y;
 
 		me.__move_handler = function(evn){
-			var evn = evn || window.event,
-				Y = evn.clientY;
+			var y = Math.max(evn.clientY - difY, 0),
+				h = dif1 - (y - dif2);
 
-		    Y = Y<0?0:Y;
-			Y -= difY;
+			if (h < me.__defaultSizeOptions.minHeight){
+				y -= me.__defaultSizeOptions.minHeight - h;
+				h = me.__defaultSizeOptions.minHeight;
+			}
 
-			if (dif1 - 101<Y)
-				Y = dif1 - 100;
-
-			me._place('',(dif2 + Y),'',(dif1 - Y));
+			me._place('',y,'',h);
 
 			/*** resize event ***/
 			if (me._onresize) me._onresize(e,'t');
@@ -616,37 +607,30 @@ _me.__resize = function(e,sResize){
 		break;
 
 	case 'lt':
-		difY = e.clientY;
-		difX = e.clientX;
+		difX = e.clientX - position.x;
+		difY = e.clientY - position.y;
 		dif1 = position.h;
 		dif2 = position.y;
 		dif3 = position.w;
 		dif4 = position.x;
 
 		me.__move_handler = function(evn){
-			var evn = evn || window.event,
-				X = evn.clientX,
-				Y = evn.clientY;
+			var x = Math.max(evn.clientX - difX, 0),
+				y = Math.max(evn.clientY - difY, 0),
+				w = dif3 + dif4 - x,
+				h = dif1 - (y - dif2);
 
-			X = X<0?0:X;
-			Y = Y<0?0:Y;
+				if (w < me.__defaultSizeOptions.minWidth){
+					x -= me.__defaultSizeOptions.minWidth - w;
+					w = me.__defaultSizeOptions.minWidth;
+				}
 
-			var size1 = dif1 - Y + difY;
-			if (size1<101){
-				  Y = dif1 + difY - 100;
-				  size1 = 100;
+			if (h < me.__defaultSizeOptions.minHeight){
+				y -= me.__defaultSizeOptions.minHeight - h;
+				h = me.__defaultSizeOptions.minHeight;
 			}
 
-			var size3 = dif3 - X + difX;
-			if (size3<201){
-				X = dif3 + difX - 200;
-				size3 = 200;
-			}
-
-			var size2 = dif2 + Y - difY;
-			var size4 = dif4 - difX + X;
-
-			me._place(size4,size2,size3,size1);
+			me._place(x,y,w,h);
 
 			/*** resize event ***/
 			if (me._onresize) me._onresize(e,'lt');
@@ -656,25 +640,20 @@ _me.__resize = function(e,sResize){
 		break;
 
 	case 'l' :
-		difX = e.clientX;
+		difX = e.clientX - position.x;
 		dif1 = position.w;
 		dif2 = position.x;
 
 		me.__move_handler = function(evn){
-			var evn = evn || window.event,
-				X = evn.clientX;
+			var x = Math.max(evn.clientX - difX, 0),
+				w = dif1 + dif2 - x;
 
-			X = X<0?0:X;
-
-			var size1 = dif1 - X + difX;
-			if (size1<201){
-				X = dif1 - 200 + difX;
-				size1 = 200;
+			if (w < me.__defaultSizeOptions.minWidth){
+				x -= me.__defaultSizeOptions.minWidth - w;
+				w = me.__defaultSizeOptions.minWidth;
 			}
 
-			var size2 = dif2 - difX + X;
-
-			me._place(size2,'',size1);
+			me._place(x,'',w);
 
 			/*** resize event ***/
 			if (me._onresize) me._onresize(e,'l');
@@ -684,33 +663,26 @@ _me.__resize = function(e,sResize){
 		break;
 
 	case 'lb':
-		difX = e.clientX;
-		difY = e.clientY;
+		difX = e.clientX - position.x;
+		difY = e.clientY - (position.y + position.h);
 		dif1 = position.w;
 		dif2 = position.x;
-		dif3 = position.h;
+		dif3 = position.y;
 
 		me.__move_handler = function(evn){
-			var evn = evn || window.event,
-				X = evn.clientX,
-				Y = evn.clientY;
+			var x = Math.max(evn.clientX - difX, 0),
+				w = dif1 + dif2 - x,
+				h = evn.clientY - dif3 - difY;
 
-			X = X<0?0:X;
-			Y = Y<0?0:Y;
-
-			var size1 = dif1 - X + difX;
-			if (size1<200){
-                X = dif1 - 200 + difX;
-                size1 = 200;
+			if (w < me.__defaultSizeOptions.minWidth){
+				x -= me.__defaultSizeOptions.minWidth - w;
+				w = me.__defaultSizeOptions.minWidth;
 			}
 
-			var size2 = dif2 - difX + X;
+			if (h < me.__defaultSizeOptions.minHeight)
+				h = me.__defaultSizeOptions.minHeight;
 
-			var size3 = dif3 + Y - difY;
-			if (size3<100)
-				size3 = 100;
-
-			me._place(size2,'',size1,size3);
+			me._place(x,'',w,h);
 
 			/*** resize event ***/
 			if (me._onresize) me._onresize(e,'lb');
@@ -719,18 +691,15 @@ _me.__resize = function(e,sResize){
 
 		break;
 	case 'b' :
-		difY = e.clientY;
-		dif1 = position.h;
+		difY = position.h - e.clientY;
 
 		me.__move_handler = function(evn){
-			var evn = evn || window.event,
-				Y = evn.clientY;
-				Y = Y<0?0:Y;
+			var y = Math.max(evn.clientY,0);
 
-			var size = dif1 + Y - difY;
-				size = size<100?100:size;
+			var size = y + difY;
+			size = size<me.__defaultSizeOptions.minHeight?me.__defaultSizeOptions.minHeight:size;
 
-			me._size('',size);
+			me._size('',size, false, true);
 
 			/*** resize event ***/
 			if (me._onresize) me._onresize(e,'b');
@@ -739,25 +708,20 @@ _me.__resize = function(e,sResize){
 
 		break;
 	case 'rb':
-		difX = e.clientX;
-		difY = e.clientY;
-		dif1 = position.w;
-		dif2 = position.h;
+		difX = position.w - e.clientX;
+		difY = position.h - e.clientY;
 
 		me.__move_handler = function(evn){
-			var evn = evn || window.event,
-				X = evn.clientX,
-				Y = evn.clientY;
+			var x = Math.max(evn.clientX, 0),
+				y = Math.max(evn.clientY, 0);
 
-			X = X<0?0:X;
-			Y = Y<0?0:Y;
+			var size1 = x + difX;
+				size1 = size1<me.__defaultSizeOptions.minWidth?me.__defaultSizeOptions.minWidth:size1;
 
-			var size1 = dif1 + X - difX;
-				size1 = size1<200?200:size1;
-			var size2 = dif2 + Y - difY;
-			    size2 = size2<100?100:size2;
+			var size2 = y + difY;
+			    size2 = size2<me.__defaultSizeOptions.minHeight?me.__defaultSizeOptions.minHeight:size2;
 
-			me._size(size1,size2);
+			me._size(size1,size2, false, true);
 
 			/*** resize event ***/
 			if (me._onresize) me._onresize(e,'rb');
@@ -770,14 +734,13 @@ _me.__resize = function(e,sResize){
 		dif1 = position.w;
 
 		me.__move_handler = function(evn){
-			var evn = evn || window.event,
-				X = evn.clientX;
+			var X = evn.clientX;
 				X = X<0?0:X;
 
 			var size1 = dif1 + X - difX;
-   				size1 = size1<200?200:size1;
+   				size1 = size1<me.__defaultSizeOptions.minWidth?me.__defaultSizeOptions.minWidth:size1;
 
-			me._size(size1);
+			me._size(size1, '', false, true);
 
 			/*** resize event ***/
 			if (me._onresize) me._onresize(e,'r');
@@ -793,23 +756,22 @@ _me.__resize = function(e,sResize){
 		dif3 = position.y;
 
 		me.__move_handler = function(evn){
-			var evn = evn || window.event,
-				Y = evn.clientY,
+			var Y = evn.clientY,
 				X = evn.clientX;
 
 			Y = Y<0?0:Y;
 			X = X<0?0:X;
 
 			var size2 = dif2 - Y + difY;
-			if (size2<100){
-				Y = dif2 - 100 + difY;
-                size2 = 100;
+			if (size2<me.__defaultSizeOptions.minHeight){
+				Y = dif2 - me.__defaultSizeOptions.minHeight + difY;
+                size2 = me.__defaultSizeOptions.minHeight;
 			}
 
 			var size3 = dif3 + Y - difY;
 
 			var size1 = dif1 + X - difX;
-            	size1 = size1<200?200:size1;
+            	size1 = size1<me.__defaultSizeOptions.minWidth?me.__defaultSizeOptions.minWidth:size1;
 
 			me._place('',size3,size1,size2);
 
@@ -822,33 +784,37 @@ _me.__resize = function(e,sResize){
 	}
 
 	if (this.__move_handler && this.__move_handler!= null)
-		gui._obeyEvent('mousemove',[this,'__move_handler']);
+		me._gui._obeyEvent('mousemove',[this,'__move_handler']);
 };
 
 _me._title = function(sVal,bText){
 	if (typeof sVal == 'undefined')
 		return this.__eTitle.innerHTML;
-	else
-	if (bText)
-		this.__eTitle.innerHTML = sVal?sVal.escapeHTML():'';
-	else{
-		var tmp = getLang(sVal);
-		this.__eTitle.innerHTML = tmp?tmp.escapeHTML():'';
+	else {
+		if (bText)
+			this.__eTitle.innerHTML = sVal?sVal.escapeHTML():'';
+		else{
+			var tmp = getLang(sVal);
+			this.__eTitle.innerHTML = tmp?tmp.escapeHTML():'';
+		}
+		if (this.__detached) {
+			this.__detached.document.title = this.__eTitle.innerHTML;
+		}
+		if (this._docked) {
+			gui.frm_main.dock._setTitle(this, this.__eTitle.innerHTML);
+		}
 	}
 };
 
-_me._close = function(b, e){
-	if (!this.__closable || (this._onclose && !this._onclose(b, e))) return false;
+_me._close = async function(b, e) {
+	if (!this.__closable || (this._onclose && !await this._onclose(b, e))) return false;
     this._remove_destructor('_onclose');
 	this._destruct();
 	return true;
 };
 
-/**
- * e - optional event parameter
- **/
-_me._focus = function (){
-	if (this._docked && this._undock())
+_me._focus = function (bSoft){
+	if (this.__detached || (this._docked && this._undock()))
 		return;
 
 	// Remove previous z-index and assign new top value
@@ -880,13 +846,16 @@ _me._focus = function (){
 		obj_popup.activeStack.order();
 
 	this._activate();
+	if (!bSoft) {
+		(this._main.ownerDocument.defaultView || window).focus();
+	}
 };
 
 _me._activate = function(){
 	if (!hascss(this._main, 'active')){
 		addcss(this._main, 'active');
 
-		var pops = gui._main.querySelectorAll('div.obj_popup');
+		var pops = this._gui._main.querySelectorAll('div.obj_popup');
 		for(var i = pops.length; i--;){
 			if (this._main !== pops[i])
 				removecss(pops[i], 'active');
@@ -929,70 +898,83 @@ _me._modal = function(b){
 	}
 };
 
-_me._maximize = function(bWait){
+_me._maximize = function(bForcedIsMaximized){
 
 	if (this.__resizable==false) return;
 
-	if (this.__position.max){
+	if (bForcedIsMaximized === false || (bForcedIsMaximized === void 0 && this.__position.max)){
 		this.__moveble	= true;
 		this.__position.max = 0;
 		removecss(this.__eContainer,'maximized');
-		this._place(this.__position.x,this.__position.y,this.__position.w,this.__position.h);
+		gui.tooltip._add(this._getAnchor('max'), getLang('POPUP_ITEMS::MAXIMIZE'))
+
+		this._place(this.__position.x,this.__position.y,this.__position.w,this.__position.h, true);
 	}
-	else{
+	else if(bForcedIsMaximized === true || bForcedIsMaximized === void 0) {
 
 		this.__position.max = 1;
 		this.__moveble	= false;
 
-		var tmp = clone(this.__position,true);
+		addcss(this.__eContainer,'maximized');
+		gui.tooltip._add(this._getAnchor('max'), getLang('POPUP_ITEMS::RESTORE'))
 
-		if(bWait){
-			var me = this;
+		// if (obj_popup._oldPositioning){
+			this.__eContainer.style.left = '0';
+			this.__eContainer.style.top = '0';
+		// }
+		// else
+		// 	this.__eContainer.style.transform = 'translate(0,0)';
 
-			this.__position_tmp = tmp;
+		this.__eContainer.style.height = '100%';
+		this.__eContainer.style.width = '100%';
 
-			window.setTimeout(function(){
-				try{
-					addcss(me.__eContainer,'maximized');
-					me._place(0,0,document.body.offsetWidth ,document.body.offsetHeight + (me.__eMain.offsetHeight - me.__eContainer.offsetHeight));
-					me.__position = me.__position_tmp;
-				}
-				catch(er){gui._REQUEST_VARS.debug && console.log(this._name||false,er);}
-			},500);
-		}
-		else{
-			addcss(this.__eContainer,'maximized');
-			this._place(0,0,document.body.offsetWidth ,document.body.offsetHeight);
-			this.__position = tmp;
-		}
+		if (!this.__eContainer.style.visibility)
+			this.__eContainer.style.visibility = 'visible';
 	}
 };
 
-_me._defaultSize = function(x,y,w,h,bMax) {
+_me._defaultSize = function(w, h, aOptions) {
+	var x = -1;
+	var y = -1;
 
-	//any change will be stored into cookie
-	this.__defaultSize = {x:x,y:y,w:w,h:h,max:bMax};
+	this.__defaultSizeOptions = aOptions = Object.assign(this.__defaultSizeOptions, aOptions || {});
 
-	//get proper cookie
-	var aSize;
-	if (Cookie)
-		aSize = Cookie.get(['popup_'+this._type,screen.width+'x'+screen.height]);
+	if (aOptions.goldenRatio) {
+		var goldenRatio = 1.618;
+		var newW = this._main.ownerDocument.body.clientWidth / goldenRatio;
+		if(newW > w) {
+			w = newW;
+		}
+		var newH = this._main.ownerDocument.body.clientHeight / goldenRatio;
+		if(newH > h) {
+			h = newH;
+		}
+	}
 
-	if (Is.Defined(aSize)) {
-		if (Is.Defined(aSize['x']) && (x = parseInt(aSize['x']) || 0))
-			x = Math.max(0,x);
+	if (GWOthers.getItem('LAYOUT_SETTINGS', 'remember_popup_position') == 1) {
+		this.__defaultSize = {x:x,y:y,w:w,h:h,max:aOptions.maximized};
 
-		if (Is.Defined(aSize['y']) && (y = parseInt(aSize['y']) || 0))
-			y = Math.max(0,y);
+		var aSize = Cookie.get(['popup_' + this._type, screen.width + 'x' + screen.height]);
 
-		if (aSize['w'] && (w = parseInt(aSize['w'])))
-			w = Math.max(200, w);
+		if (aSize) {
+			if (Is.Defined(aSize.x) && (x = parseInt(aSize.x) || 0))
+				x = Math.max(0, x);
 
-		if (aSize['h'] && (h = parseInt(aSize['h'])))
-			h = Math.max(88, h);
+			if (Is.Defined(aSize.y) && (y = parseInt(aSize.y) || 0))
+				y = Math.max(0, y);
 
-		if (aSize['max'])
-			bMax = !!aSize['max'];
+			if (aSize.w && (w = parseInt(aSize.w)))
+				w = Math.max(200, w);
+
+			if (aSize.h && (h = parseInt(aSize.h)))
+				h = Math.max(88, h);
+
+			if (aSize.max)
+				aOptions.maximized = !!aSize.max;
+		}
+
+		// register function that will store current size
+		this._add_destructor('__rememberSize');
 	}
 
 	this.__position.x = x;
@@ -1001,81 +983,80 @@ _me._defaultSize = function(x,y,w,h,bMax) {
 	this.__position.h = h;
 	this.__positionShadow = clone(this.__position);
 
-	if (bMax){
+	if (aOptions.maximized) {
 		this._maximize();
 	}
 	else
-	if (Is.Number(x) && Is.Number(y) && x >= 0 && y >= 0) {
+	if ((Is.Number(x) && Is.Number(y) && x >= 0 && y >= 0) || aOptions.goldenRatio) {
 		// Shift non-modal popups, set place
 		this._placeShift();
 	}
 	else
 	if (Is.Number(w) && w > 0) {
 		if (Is.Number(h) && h > 0)
-			this._size(w,h,1);
+			this._size(w,h,true);
 		else
-			this._size(w,'',1);
+			this._size(w,'',true);
 
 		// Shift non-modal popups
-		if (!this._isModal())
-			this._placeShift(true);
+		this._placeShift(true);
 	}
-
-	// register function that will store current size
-	this._add_destructor('__rememberSize');
 };
 
-	_me.__rememberSize = function(){
-		if (Cookie){
-			var tmp = clone(this.__position);
+_me.__rememberSize = function(){
+	var tmp = clone(this.__position);
 
-			for (var i in tmp)
-				if (tmp[i] == this.__defaultSize[i])
-					delete tmp[i];
-
-			Cookie.set(['popup_'+this._type,screen.width+'x'+screen.height],tmp);
+	for (var i in tmp) {
+		if (tmp[i] == this.__defaultSize[i]) {
+			delete tmp[i];
 		}
-	};
+	}
+
+	Cookie.set(['popup_' + this._type, screen.width + 'x' + screen.height], tmp);
+};
 
 /**
  * @param {bool} bShiftOnly		do not set new position when no shift detected
  */
 _me._placeShift	= function(bShiftOnly){
 
-	var bForced = false;
-	var x = this.__positionShadow.x,
-		y = this.__positionShadow.y,
-		w = this.__positionShadow.w,
-		h = this.__positionShadow.h;
+	var bForced = true;
+	var x = this.__position.x,
+		y = this.__position.y,
+		w = this.__position.w,
+		h = this.__position.h;
 
 	if (!this._isModal()){
 		//Get all instances
-		var aPop = gui._getChildObjects('main', this._type),
-			iH = gui._main.offsetHeight,
-			iW = gui._main.offsetWidth,
+		var aPop = this._gui._getChildObjects('main', this._type),
+			iH = this._gui._main.offsetHeight,
+			iW = this._gui._main.offsetWidth,
 			bShifted = false;
 
 		for (var i = aPop.length; i--;){
 			if (aPop[i] !== this && !aPop[i]._destructed && !aPop[i]._docked){
-				if (x == aPop[i].__positionShadow.x && y == aPop[i].__positionShadow.y){
-					bForced = true;
 
-					x += 30;
-					y += 30;
+				if (x == aPop[i].__position.x && y == aPop[i].__position.y){
 
-					if (!bShifted && (iH<y+h || iW<x+w)){
+					x = aPop[i].__positionShadow.x + 30;
+					y = aPop[i].__positionShadow.y + 30;
+
+					if (iH<y+h || iW<x+w){
 						y = 30;
 						x = 30;
-						bShifted = true;
 					}
 
-					i = aPop.length;
+					this.__positionOffset.x = x - this.__position.x;
+					this.__positionOffset.y = y - this.__position.y;
+
+					bShifted = true;
+					break;
 				}
 			}
 		}
 	}
 
-	if (!bShiftOnly || bForced){
+	if (!bShiftOnly || bShifted){
 		if (Is.Number(w) && w > 0) {
 			if (Is.Number(h) && h > 0)
 				this._place(x,y,w,h,bForced);
@@ -1087,157 +1068,128 @@ _me._placeShift	= function(bShiftOnly){
 	}
 };
 
-_me._place = function(x,y,w,h, bForced){
+_me._place = function(x,y,w,h, bForce){
 
-	if (!Is.Defined(x) && !Is.Defined(y) && !Is.Defined(w) && !Is.Defined(h))
+	if (this.__position.max || !Is.Defined(x) && !Is.Defined(y) && !Is.Defined(w) && !Is.Defined(h))
 		return this.__position;
 
-	var iH = gui._main.offsetHeight,
-		iW = gui._main.offsetWidth,
-		bRender = this.__eContainer.style.transform == "";
+	//User initialized, will be saved
+	if (!bForce){
 
-	//check position (move window fully into view if part of it is outside of viewable area)
-	if (!this.__position.max && Is.Defined(x) && Is.Defined(y) && Is.Defined(w) && Is.Defined(h)){
-		if(h > window.innerHeight) {
-			h = window.innerHeight;
-		}
-		//HEIGHT
-		if (iH<y+h){
-			y = iH-h;
+		//reset offset when user change position
+		this.__positionOffset = {};
 
-			// if less space than popup size is available
-			if (y<0){
-				if (this.__resizable)
-					h = Math.max(88, h+y);
-
-				y = 0;
+		if (Is.Number(x)){
+			if (x>=0){
+				this.__positionShadow.x = x;
 			}
 
-			bForced = true;
+			this.__position.x = x;
 		}
 
-		//WIDTH
-		if (iW<x+w){
-			x = iW-w;
-
-			// if less space than popup size is available
-			if (x<0){
-				if (this.__resizable)
-					w = Math.max(200, w+x);
-
-				x = 0;
+		if (Is.Number(y)){
+			if (y>=0){
+				this.__positionShadow.y = y;
 			}
 
-			bForced = true;
+			this.__position.y = y;
+		}
+
+		if (Is.Number(w)){
+			this.__positionShadow.w = w;
+			this.__position.w = w;
+		}
+
+		if (Is.Number(h)){
+			this.__positionShadow.h = h;
+			this.__position.h = h;
 		}
 	}
+	else{
 
-	if (Is.Number(x)){
-		x = Math.max(0, (x>iW-100?iW-100:x));
+		var iH = this._gui._main.offsetHeight,
+			iW = this._gui._main.offsetWidth,
+			tmp_w = w, tmp_h = h;
 
-		if (this.__positionShadow.x != x)
-			bRender = true;
+		//check Width
+		if (w == 'auto'){
+			this.__eContainer.style.width = 'auto';
+			tmp_w = this.__eContainer.offsetWidth;
+		}
+		//center X
+		if (x<0){
+			x = Math.round(Math.max(0, (this._main.ownerDocument.body.clientWidth - tmp_w)/2));
+		}
+		if (x+tmp_w>iW){
+			if (tmp_w>iW){
+				tmp_w = w = Math.max(200, iW);
+			}
+
+			x = Math.max(0, iW-tmp_w);
+		}
 
 		this.__positionShadow.x = x;
-		if (!bForced)
-			this.__position.x = x;
-	}
+		this.__positionShadow.w = w;
 
-	if (Is.Number(y)){
-		y = Math.max(0, (y>iH-100?iH-100:y));
+		//check Height
+		if (h == 'auto'){
+			this.__eContainer.style.height = 'auto';
+			tmp_h = this.__eContainer.offsetHeight;
+		}
+		//center Y
+		if (y<0){
+			y = Math.round(Math.max(0, (this._main.ownerDocument.body.clientHeight - tmp_h)/2));
+		}
+		if (y+tmp_h>iH){
+			if (tmp_h>iH){
+				tmp_h = h = Math.max(88, iH);
+			}
 
-		if (this.__positionShadow.y != y)
-			bRender = true;
+			y = Math.max(0, iH-tmp_h);
+		}
 
 		this.__positionShadow.y = y;
-		if (!bForced)
-			this.__position.y = y;
-	}
-
-	if (bRender){
-		if (obj_popup._oldPositioning){
-			this.__eContainer.style.left = this.__positionShadow.x +'px';
-			this.__eContainer.style.top = this.__positionShadow.y +'px';
-		}
-		else
-			this.__eContainer.style.transform = 'translate('+ Math.floor(this.__positionShadow.x) +'px,'+ Math.floor(this.__positionShadow.y) +'px)';
-	}
-
-	if (Is.Number(w) || Is.Number(h))
-		this._size(w,h,false, bForced);
-};
-
-_me._size = function (w, h, center, bForced){
-	var tmp, bRender = false;
-
-	if (w>=200 || w === 'auto'){
-		if (this.__eContainer.offsetWidth != w) {
-			this.__eContainer.style.width = w !== 'auto' ? w + 'px' : w;
-			w = this.__eContainer.offsetWidth;
-		}
-
-		if (center){
-			tmp = document.body.clientWidth - w;
-			tmp = tmp<0?0:tmp;
-			tmp = Math.round(tmp/2);
-
-			this.__positionShadow.x = tmp;
-			bRender = true;
-		}
-
-		this.__positionShadow.w = w;
-		if (!bForced)
-			this.__position.w = w;
-	}
-
-	if (h>=88 || h === 'auto'){
-		if (h === 'auto' && this.__eContainer.clientHeight > window.innerHeight) {
-			h = '100%';
-		}
-
-		if (this.__eContainer.offsetHeight != h) {
-			this.__eContainer.style.height = Is.Number(h) ? h + 'px' : h;
-			h = this.__eContainer.offsetHeight;
-		}
-
-		if (center){
-			tmp = (document.body.clientHeight - h)/2;
-			tmp = tmp<0?0:tmp;
-			tmp = Math.round(tmp/2);
-
-			this.__positionShadow.y = tmp;
-			bRender = true;
-		}
-
 		this.__positionShadow.h = h;
-
-		if (!bForced)
-			this.__position.h = h;
 	}
 
-	if (bRender){
-		if (obj_popup._oldPositioning){
-			this.__eContainer.style.left = this.__positionShadow.x +'px';
-			this.__eContainer.style.top = this.__positionShadow.y +'px';
-		}
-		else
-			this.__eContainer.style.transform = 'translate('+ Math.floor(this.__positionShadow.x) +'px,'+ Math.floor(this.__positionShadow.y) +'px)';
-	}
+	if (Is.Empty(this.__position) || bForce)
+		this.__position = clone(this.__positionShadow);
+
+	// if (obj_popup._oldPositioning){
+		this.__eContainer.style.left = this.__positionShadow.x +'px';
+		this.__eContainer.style.top = this.__positionShadow.y +'px';
+	// }
+	// else
+	// 	this.__eContainer.style.transform = 'translate('+ Math.floor(this.__positionShadow.x) +'px,'+ Math.floor(this.__positionShadow.y) +'px)';
+
+	this.__eContainer.style.height = Is.Number(this.__positionShadow.h) ? this.__positionShadow.h + 'px' : this.__positionShadow.h;
+	this.__eContainer.style.width = Is.Number(this.__positionShadow.w) ? this.__positionShadow.w + 'px' : this.__positionShadow.w;
+
 
 	// Container is initially hidden, until size is defined so is clear what to show
-	this.__eContainer.style.visibility = 'visible';
+	if (!this.__eContainer.style.visibility)
+		this.__eContainer.style.visibility = 'visible';
+};
+
+
+/**
+ * @brief: destructor
+ * bDirect - do no conversion
+ **/
+_me._size = function (w, h, center, bDirect){
+
+	w = Is.Defined(w)?w:this.__positionShadow.w;
+	h = Is.Defined(h)?h:this.__positionShadow.h;
+
+	if (center)
+		this._place(-1, -1, w, h, !bDirect);
+	else
+		this._place(this.__positionShadow.x, this.__positionShadow.y, w, h, !bDirect);
 };
 
 _me.__checkPosition = function(){
-	if (!this.__position.max){
-
-		if (!Is.Defined(this.__position.x) || !Is.Defined(this.__position.y))
-			var position = getSize(this.__eContainer);
-		else
-			var position = this.__position;
-
-		this._place(position.x, position.y, position.w, position.h, true);
-	}
+	if (!this.__position.max)
+		this._place(this.__position.x + (this.__positionOffset.x || 0), this.__position.y + (this.__positionOffset.y || 0), this.__position.w, this.__position.h, true);
 };
 
 
@@ -1250,8 +1202,8 @@ _me._isActive = function() {
  * @brief: destructor
  * @date : 4.7.2006 14:38:46
  **/
-_me.__destructor = function(){
-	this._undock();
+_me.__destructor = async function(){
+	await this._undock();
 	this.__hideBlock();
 
 	// remove object's zindex
@@ -1262,7 +1214,7 @@ _me.__destructor = function(){
 	// Focus next window below the docked one
 	obj_popup.activeStack.focusNext();
 
-	gui._disobeyEvent('resize',[this,'__checkPosition']);
+	this._gui._disobeyEvent('resize',[this,'__checkPosition']);
 };
 
 /**
