@@ -5,10 +5,10 @@ function obj_print(){};
  * @brief:
  * @date : 21.7.2014
  **/
-_me.__constructor = function(s){
+_me.__constructor = async function(){
 	var me = this;
 
-	this._create('frame','obj_frame');
+	await this._create('frame','obj_frame');
 
 	this.frame._onresize = function(b){
 		if (b){
@@ -21,6 +21,6 @@ _me.__constructor = function(s){
 _me._print = function(s, bText){
 	if (Is.String(s)){
 		this.frame._resize = true;
-		this.frame._write(bText?s.escapeHTML().wrap():s);
+		this.frame._write(bText?s.escapeHTML().replace(/(?:\r\n|\r|\n)/gm, '<br>'):s);
 	}
 };
\ No newline at end of file
