@@ -2,13 +2,13 @@ _me = obj_reminder_item.prototype;
 function obj_reminder_item() {};
 
 _me._value = function(aValues) {
-	
+	var aSubValues;
 	if (this._parent._type != 'obj_reminder')
 		this._getAnchor('label').innerHTML = getLang('REMINDER::REMIND1');	
 
 	if (Is.Defined(aValues)) {
 		if (Is.Defined(aValues['values'])) {
-			var aSubValues = aValues['values'];
+			aSubValues = aValues['values'];
 
 			this.time._value(((parseInt(aSubValues['RMNDAYSBEFORE'])*24+parseInt(aSubValues['RMNHOURSBEFORE']))*60+parseInt(aSubValues['RMNMINUTESBEFORE']))*60000);
 
@@ -16,7 +16,7 @@ _me._value = function(aValues) {
 		}
 	} else {
 		var aResult = {'values': {}};
-		var aSubValues = aResult['values'];
+		aSubValues = aResult['values'];
 		
 		aSubValues['RMNDAYSBEFORE'] = 0;
 		aSubValues['RMNHOURSBEFORE'] = 0;
