@@ -1,11 +1,11 @@
 _me = obj_send_message.prototype;
 function obj_send_message() {};
 
-_me.__constructor = function() {
+_me.__constructor = async function() {
 	var me = this;
 	this.__aValues = {};
 
-    this._create('button','obj_button');
+    await this._create('button','obj_button');
 	this.button._value('FILTERS::THREE_DOTS');
 	this.button._onclick = function(){
 		gui._create('send_message', 'frm_send_message', '', '', me.__aValues);
