@@ -1,9 +1,9 @@
 _me = obj_sharing.prototype;
 function obj_sharing() {};
 
-_me.__constructor = function(sPrivate) {
+_me.__constructor = async function(sPrivate) {
 	this.__privateValue = sPrivate || 'P';
-	this._create('btn','obj_button_check','main','ico img private simple noborder transparent');
+	await this._create('btn','obj_button_check','main','ico img private simple noborder transparent');
 	this.btn._title('SHARING::PRIVATE');
 };
 
