@@ -4,43 +4,55 @@ function obj_sip(){};
 /*
 	aHandler is executed once when sip goes online, used for direct Call when offline
 */
-_me.__constructor = function(aHandler){
+_me.__constructor = async function(aHandler){
 
 	this.__aHandler = aHandler;
 	this.__playSounds = true;
 
 	// Determine if SIP via WebRTC and js should be used
-	storage.library('sip_bridge');
+	await storage.library('sip_bridge');
 	this._use_webrtc = window.JsSIP && IceSIP && IceSIP.supported();
 
 	// Check if connection is secure, otherwise abort
 	if(document.location.protocol!='https:') {
-		var nortc = gui._create('no_webrtc','frm_confirm','','',[function(){
-			window.open('https://www.icewarp.com/support/troubleshoot_webrtc/?' + buildURL({ishttps:(location.protocol.indexOf("https")===0?'1':'0'),lang:GWOthers.getItem('LAYOUT_SETTINGS','language')}));
-		}],'SIP::SIP','SIP::NOTSECURE');
-		nortc.x_btn_ok._value('ERROR::TROUBLESHOOT');
-		addcss(nortc.x_btn_ok._main, 'help');
-		this._destruct();
+		var notification = gui.notifier._value({
+			type: 'sip_not_reachable',
+			args: {
+				text: 'SIP::NOTSECURE'
+			},
+			buttons: [{
+				text: getLang('FORM_BUTTONS::CLOSE'),
+				onclick: function() {
+					notification.close();
+				}
+			}, {
+				text: getLang('ERROR::TROUBLESHOOT'),
+				className: 'decline',
+				onclick: function() {
+					window.open('https://www.icewarp.com/support/troubleshoot_webrtc/?' + buildURL({ishttps:(location.protocol.indexOf("https")===0?'1':'0'),lang:GWOthers.getItem('LAYOUT_SETTINGS','language')}));
+					notification.close();
+				}
+			}]
+		});
 		return false;
 	}
 
 	// Add ringtones
-	var ext = navigator.browser && navigator.browser.application=='Opera' ? 'ogg' : 'mp3';
 	var loc = GWOthers.getItem('LAYOUT_SETTINGS','language')=='en' ? 'uk' : 'eu';
 
 	this._ringtone = new Audio();
-	this._ringtone.src = 'client/inc/sip/tones/phone.'+ext;
+	this._ringtone.src = 'client/inc/sip/tones/phone.mp3';
 	this._ringtone.loop = true;
 
 	this._busytone = new Audio();
-	this._busytone.src = 'client/inc/sip/tones/busy.'+ext;
+	this._busytone.src = 'client/inc/sip/tones/busy.mp3';
 
 	this._calltone = new Audio();
-	this._calltone.src = 'client/inc/sip/tones/'+loc+'/call.'+ext;
+	this._calltone.src = 'client/inc/sip/tones/'+loc+'/call.mp3';
 	this._calltone.loop = true;
 
 	this._failtone = new Audio();
-	this._failtone.src = 'client/inc/sip/tones/fail.'+ext;
+	this._failtone.src = 'client/inc/sip/tones/fail.mp3';
 
 	// Call history
 	this.__callBuffer = [];
@@ -65,10 +77,10 @@ _me.__init = function (){
 		this.__started = true;
 
 		// Set SIP parameters
-		var c = {display_name: dataSet.get('main',['fullname']),trace_sip: true};
+		var c = {display_name: dataSet.get('main',['fullname']),trace_sip: true}, user;
 		if (GWOthers.getItem('SIP','external')>0 && (GWOthers.getItem('RESTRICTIONS', 'disable_esip') || 0)<1 && GWOthers.getItem('SIP','user') && GWOthers.getItem('SIP','pass') && (GWOthers.getItem('SIP','server') || GWOthers.getItem('SIP','user').indexOf('@'))) {
 			var tmp = GWOthers.getItem('SIP','user').split('@');
-			var user = tmp[0];
+			user = tmp[0];
 			var server = tmp.length==1 ? GWOthers.getItem('SIP','server') : tmp[1];
 
 			// If applicable, add user name used for authentication (used by Cisco)
@@ -83,7 +95,7 @@ _me.__init = function (){
 			c.password = GWOthers.getItem('SIP','pass');
 
 		} else {
-			var user = sPrimaryAccount.split('@');
+			user = sPrimaryAccount.split('@');
 			var port = dataSet.get('accounts',[sPrimaryAccount,'SIP_PORT']);
 			c.uri = 'sip:' + (dataSet.get('accounts',[sPrimaryAccount,'SIP_EXTENSION']) || dataSet.get('accounts',[sPrimaryAccount,'SIP_USERNAME']) || user[0]) + '@' + ((dataSet.get('accounts',[sPrimaryAccount,'SIP_HOST'])+(port ? ':'+port : '')) || user[1]);
 			c.ha1 = dataSet.get('accounts',[sPrimaryAccount,'SIP_HASH']);
@@ -95,7 +107,7 @@ _me.__init = function (){
 		this.phone = new IceSIP(c);
 
 		// Handle connection status
-		this.phone.onconnect = function(e) {
+		this.phone.onconnect = function() {
 			dataSet.add('sip',['state'],'online');	// e.host
 
 			if (me.__aHandler){
@@ -135,7 +147,7 @@ _me.__init = function (){
 			var domain = sPrimaryAccount.split('@')[1];
 			alias += '@' + domain;
 			if(domain && e.user==alias) {
-				me.call.onhangup = function(e) {};
+				me.call.onhangup = function() {};
 				me.call.hangup(488);
 				return;
 			}
@@ -189,9 +201,9 @@ _me._add_callbeck = function (aHandler){
 
 	this.__callBuffer.push(aHandler);
 };
-_me._execute = function (aData){
+_me._execute = async function (aData){
 	for(var i = this.__callBuffer.length-1; i>=0; i--)
-		if (!getCallbackFunction(this.__callBuffer[i]) || executeCallbackFunction(this.__callBuffer[i],aData) === false)
+		if (!getCallbackFunction(this.__callBuffer[i]) || await executeCallbackFunction(this.__callBuffer[i],aData) === false)
 			this.__callBuffer.splice(i,1);
 };
 
@@ -224,9 +236,9 @@ _me._execute = function (aData){
  *
  *   Error             | Primarily legacy handling
  */
-_me.__process = function (sType, sParam1, sParam2, sParam3){
+_me.__process = async function (sType, sParam1, sParam2, sParam3){
 	var me = this;
-	var sValue = '';
+	var sValue = '', name, tmp;
 
 	// Stop ringtone for incoming call
 	if (this._ringtone)
@@ -262,20 +274,17 @@ _me.__process = function (sType, sParam1, sParam2, sParam3){
 	switch (sType.toString()){
 	case 'UserMediaRequested':
 		me.__mrfrm = true;
-		setTimeout(function(){
+		setTimeout(async function(){
 			if(me.__mrfrm) {
-				me.__mrfrm = gui._create('sip_alert','frm_alert','','sip',[function(){}],'SIP::WEBPHONE_SETUP','SIP::MEDIAREQUEST' + (me.__bVideo ? '_VIDEO' : ''));
+				me.__mrfrm = await gui._create('sip_alert','frm_alert','','sip',[function(){}],'SIP::WEBPHONE_SETUP','SIP::MEDIAREQUEST' + (me.__bVideo ? '_VIDEO' : ''));
 				me.__mrfrm._size(620);
 
 				var browser = false;
 				switch(currentBrowser()) {
 					case 'Chrome':
 					case 'Safari':
-					case 'Opera':
 						browser = currentBrowser().toLowerCase();
 						break;
-					case 'MSIE7':
-					case 'MSIE9':
 					case 'MSIE11':
 						browser = 'IE';
 					case 'Mozilla':
@@ -290,8 +299,8 @@ _me.__process = function (sType, sParam1, sParam2, sParam3){
 
 				if (browser) {
 					var image = mkElement('img', {
-						className: 'image ' + browser + (currentBrowser.HighDensity ? ' x2' : ''),
-						src: './client/skins/default/images/allow_dialogs_cross_bws/' + browser + '/' + (me.__bVideo ? 'camera' : 'microphone') + '@' + (currentBrowser.HighDensity ? '2' : '1') + 'x.png'
+						className: 'image ' + browser + (devicePixelRatio > 1.3 ? ' x2' : ''),
+						src: './client/skins/default/images/allow_dialogs_cross_bws/' + browser + '/' + (me.__bVideo ? 'camera' : 'microphone') + '@' + (devicePixelRatio > 1.3 ? '2' : '1') + 'x.png'
 					});
 					me.__mrfrm.obj_label._main.insertBefore(image, me.__mrfrm.obj_label._main.firstChild);
 					image.onload = function() {
@@ -349,7 +358,7 @@ _me.__process = function (sType, sParam1, sParam2, sParam3){
 				m = "SIP::MEDIADENIED";
 		}
 	//	gui.notifier._value({type: 'alert', args: {header: 'SIP::ERROR', text: m}});
-		gui._create('alert', 'frm_alert', '', '', '', 'SIP::ERROR', m);
+		await gui._create('alert', 'frm_alert', '', '', '', 'SIP::ERROR', m);
 		// Remember that user denied device (will remain block for this login session)
 		dataSet.add('sip',['media'],'denied',false);
 		return;
@@ -360,8 +369,8 @@ _me.__process = function (sType, sParam1, sParam2, sParam3){
 
 	case 'Calling':
 		// Get name and identity of person calling
-		var tmp = MailAddress.splitEmailsAndNames(sParam1)[0] || {};
-		var name = tmp.name || tmp.email.split('@')[0];
+		tmp = MailAddress.splitEmailsAndNames(sParam1)[0] || {};
+		name = tmp.name || tmp.email.split('@')[0];
 
 		dataSet.add('sip',['remote'],{name: name, id: sParam1},false);
 
@@ -375,8 +384,8 @@ _me.__process = function (sType, sParam1, sParam2, sParam3){
 		// sParam2: (boolean) incoming video call
 		// sParam3: (boolean) incoming presentation
 
-		var tmp = MailAddress.splitEmailsAndNames(sParam1)[0] || {};
-		var name = tmp.name || tmp.email.split('@')[0];
+		tmp = MailAddress.splitEmailsAndNames(sParam1)[0] || {};
+		name = tmp.name || tmp.email.split('@')[0];
 
 		sValue = getLang('SIP::INCOMINGCALL', [name]);
 		gui.notifier._value({type: 'sip_incoming', args: {header: 'SIP::SIP', text_plain: sValue}, group: 'sip'});
@@ -389,19 +398,6 @@ _me.__process = function (sType, sParam1, sParam2, sParam3){
 
 		break;
 
-	case 'Joining':
-		// Conference joining is handled in frm_conference
-		break;
-
-	case 'ConferenceStarted':
-		// Conference start is handled in frm_conference
-		break;
-
-	case 'ConferenceCanceled':
-	case 'ConferenceFinished':
-		// Conference end is handled in frm_conference
-		break;
-
 	case 'CallEstablished':
 
 		if(!dataSet.get('sip',['remote','id']))
@@ -465,8 +461,8 @@ _me.__process = function (sType, sParam1, sParam2, sParam3){
 
 				break;
 			default:
-				var tmp = MailAddress.splitEmailsAndNames(sParam1)[0] || {};
-				var name = tmp.name || tmp.email.split('@')[0];
+				tmp = MailAddress.splitEmailsAndNames(sParam1)[0] || {};
+				name = tmp.name || tmp.email.split('@')[0];
 				sValue = getLang('SIP::CALLFINISHED');
 				gui.notifier._value({type: 'sip_canceled', args: {header: 'SIP::SIP', text_plain: sValue, data: { NAME: name, EMAIL: tmp.email }}, group: 'sip', save: true});
 			}
@@ -557,7 +553,7 @@ _me._call = function(address,bVideo,bScreen) {
 
 		this.__process('PreparingCall',address,bVideo);
 
-		this.call.oncalling = function(e) {
+		this.call.oncalling = function() {
 			obj_sip._activity("Calling",true);
 
 			me.__process('Calling',address,'true');
@@ -598,8 +594,8 @@ _me._call = function(address,bVideo,bScreen) {
 };
 
 // Open popup for video for webcam calls
-_me.__video = function(remoteStream,localStream) {
-	return gui._create('video','frm_video','','',remoteStream,localStream);
+_me.__video = async function(remoteStream,localStream) {
+	gui._create('video','frm_video','','',remoteStream,localStream);
 };
 
 _me._dtmf = function(sChar){
@@ -665,78 +661,6 @@ _me._addVideo = function() {
 	}
 };
 
-// Join conference using WebRTC
-_me._join = function(conferenceNumber) {
-	var me = this;
-
-	IceSIP.devices(function(d){
-		me.__join(conferenceNumber,d.microphone===false);
-	});
-};
-_me.__join = function(conferenceNumber,bSkipMic) {
-	var me = this;
-
-	this.conference = new IceSIP.Conference(this.phone);
-	this._conference_id = conferenceNumber;
-
-	if(bSkipMic)
-		this.conference.microphone = false;
-
-	obj_sip._activity("Joining",true);
-
-	this.__process('Joining',conferenceNumber,'true');
-
-	this.conference.onjoined = function(e) {
-		me.callid = e.callid;
-
-		obj_sip._activity("Conference",true);
-
-		me.__process('ConferenceStarted',e.user,{
-			stream: e.video && !bOrganise ? true : false,
-			callid: e.callid
-		});
-
-		if(e.video)
-			me.__video(e.video);
-
-	};
-	this.conference.onfailed = function(e) {
-		// Failed, try again without microphone
-		if(me.conference.microphone) {
-			me.conference.microphone = false;
-			me.conference.join(conferenceNumber);
-		} else {
-			obj_sip._activity("",true);
-			me.__process('ConferenceCanceled','',e.reason);
-		}
-	};
-	this.conference.onended = function(e) {
-		obj_sip._activity("",true);
-		me.__process('ConferenceFinished','',e.reason);
-	};
-	this.conference.oninfo = function(e) {
-		me.__process('ConferenceInformation',e.mimetype,e.message);
-	};
-	this.phone.onmessage = function(e) {
-		if(e.group)
-			me.__process('IncomingConferenceMessage',e.from_id,e.content);
-		else
-			me.__process('IncomingParticipantMessage',e.from_id,e.content);
-	};
-	this.conference.onsent = function(e) {
-	//	console.log('message SENT!',e.failed);
-	};
-
-	this.conference.tracking = 'data=client%3Dinternal';
-
-	this.conference.join(conferenceNumber);
-};
-
-// Check microphone connected
-_me._passive = function() {
-	return !this.conference.microphone;
-};
-
 // Turn off microphone
 _me._mute = function(mute) {
 	if (Is.Boolean(mute)){
@@ -744,8 +668,6 @@ _me._mute = function(mute) {
 
 		if(this.call)
 			muted = this.call.mute(mute);
-		else if(this.conference)
-			muted = this.conference.mute(mute);
 
 		dataSet.add('sip',['muted'],mute&&muted);
 	}
@@ -767,68 +689,6 @@ _me._hold = function(hold) {
 		return dataSet.get('sip',['onhold'])?true:false;
 };
 
-// Turn off sound for loudspeakers
-_me._silence = function(quiet) {
-	if(this.conference)
-		this.conference.silence(!!quiet);
-};
-
-// Share screen
-_me._share = function(bSend,callback) {
-	if(this.conference) {
-		this.conference.share = new IceSIP.Sharing(this.conference);
-
-		this.conference.share.onstarted = function(e) {
-			if(e.video)
-				callback(e.video);
-			else if(e.sending)
-				callback(true);
-		};
-
-		this.conference.share.onfailed = function(e) {
-			callback(false);
-		};
-
-		this.conference.share.onended = function(e) {
-			callback(false);
-		};
-
-		if(bSend)
-			this.conference.share.share();
-		else
-			this.conference.share.join();
-	}
-
-};
-
-// Stop screen sharing in conference
-_me._unshare = function() {
-	if (this.conference && this.conference.share) {
-		this.conference.share.stop();
-		this.conference.share = null;
-	}
-};
-
-// Leave conference started using WebRTC
-_me._leave = function() {
-	obj_sip._activity('');
-
-	if (this.conference) {
-		// Stop sharing if running
-		this._unshare();
-		// Leave conference
-		if(this.conference.state!='Ended')
-			this.conference.leave();
-	}
-	// Restore speaker volume
-	this._silence(false);
-};
-
-// Send chat message
-_me._send = function(from,message,to) {
-	this.conference.send(from,message,to);
-};
-
 _me._alive = function() {
 	return this.__started ? true : false;
 };
