@@ -1,8 +1,8 @@
 _me = obj_storage_usage.prototype;
 function obj_storage_usage() {}
 
-_me.__constructor = function (quota, usage, id) {
-	this._draw('obj_storage_usage');
+_me.__constructor = async function (quota, usage, id) {
+	await this._draw('obj_storage_usage');
 
 	var label = '';
 	quota = (quota || 0) * 1024;
@@ -18,8 +18,8 @@ _me.__constructor = function (quota, usage, id) {
 	}
 	this._getAnchor('label').textContent = label;
 
-	this._getAnchor('manage').onclick = function () {
-		gui._create('storage', 'frm_storage', '', '', id, quota, usage);
+	this._getAnchor('manage').onclick = async function () {
+		await gui._create('storage', 'frm_storage', '', '', id, quota, usage);
 	};
 };
 
