@@ -1,7 +1,7 @@
 function obj_subscriber(){};
 
 obj_subscriber.prototype = {
-	__constructor:function(){
+	__constructor: async function(){
 
 		//Auto re-subscribe on disconnect
 		var relogin = false;
