@@ -4,17 +4,15 @@
 _me = obj_tab.prototype;
 function obj_tab(){};
 
-_me.__constructor = function(bFirst){
+_me.__constructor = async function(bFirst){
 
 	var me = this;
 
-	this.__eLi = mkElement('li',{id:this._parent._pathName+"/"+this._name, className: this._css}); //ID is for automatic testing
+	this.__eLi = mkElement('li',{id:this._parent._pathName+"/"+this._name, className: this._css || ''}); //ID is for automatic testing
 
 	this.__eLi.onmousedown = function(e){
-		var e = e || window.event;
-			e.cancelBubble = true;
-		try{e.preventDefault();}catch(r){gui._REQUEST_VARS.debug && console.log(this._name||false,r);}
-		try{e.stopPropagation();}catch(r){gui._REQUEST_VARS.debug && console.log(this._name||false,r);}
+		e.preventDefault();
+		e.stopPropagation();
 
 		if (!me._isActive){
 			me._active(null, true);
@@ -27,12 +25,8 @@ _me.__constructor = function(bFirst){
 			if (me._ondeactive) me._ondeactive();
 			me.__exeEvent('ondeactive',null,{"owner":me});
 		}
-
-		return false;
 	};
 	this.__eLi.oncontextmenu = function(e){
-		var e = e || window.event;
-
 		if (me._oncontext)
 			me._oncontext(e);
 
@@ -45,6 +39,12 @@ _me.__constructor = function(bFirst){
 		elm.insertBefore(this.__eLi, elm.firstChild);
 	else
 		elm.appendChild(this.__eLi);
+
+	if (elm.childElementCount === 1) {
+		this._parent._main.classList.add('noheader');
+	} else {
+		this._parent._main.classList.remove('noheader');
+	}
 };
 
 /**
