@@ -1,26 +1,21 @@
 _me = obj_tab_close.prototype;
 function obj_tab_close(){};
 
-_me.__constructor = function(){
+_me.__constructor = async function(){
 
 	var me = this;
 
     addcss(this.__eLi,'obj_tab_close_link');
 
 	this.__eLi.onmousedown = function(e){
-    	var e = e || window.event;
-			e.cancelBubble = true;
-		if (e.preventDefault) e.preventDefault();
-		if (e.stopPropagation) e.stopPropagation();
+		e.preventDefault();
+		e.stopPropagation();
 
-		var elm = e.target || e.srcElement;
-		if (elm.parentElement.tagName == 'B' && !e.button){
+		if (e.target.parentElement.tagName == 'B' && !e.button){
 			me._close();
 		}
 		else
 			me._active(null, true);
-
-		return false;
 	};
 };
 
