@@ -5,7 +5,7 @@ function obj_tag_color(){};
  * @brief: CONSTRUCTOR
  * @date : 29.1.2013
  **/
-_me.__constructor = function(){
+_me.__constructor = async function(){
 	this._listen_data('tags');	
 };
 
@@ -15,7 +15,7 @@ _me._listen_data = function(sDataSet,aDataPath){
 	dataSet.obey(this,'_listener_data',sDataSet);
 };
 
-_me.__update = function(sDataSet,sDataPath){
+_me.__update = function(sDataSet){
 
 	if (!sDataSet || this._destructed) return;
 
@@ -34,12 +34,28 @@ _me._decode = function(v){
 		aIN = v.split(','),
 		aOut = [];
 
+	var NM = NightMode();
+	var invertStyleProperty = NM.invertStyleProperty.bind(NM);
+
 	for (var i in aIN)
 		if ((aIN[i] = aIN[i].trim())){
+			var tagColor = (aTags[aIN[i]] || {}).TAGCOLOR;
+			var textColor = (aTags[aIN[i]] || {}).TEXTCOLOR || '#000000';
+			var id;
+			if (dataSet.get('main', ['night_mode_enabled'])) {
+				var inverted = invertStyleProperty('color', textColor);
+				id = inverted.id;
+				textColor = inverted.color;
+				inverted = tagColor ? invertStyleProperty('background-color', tagColor, id) : { id: id };
+				id = inverted.id;
+				tagColor = tagColor ? inverted.color : '';
+			}
 			aOut.push({
 				tag:aIN[i],
-				bgcolor:(aTags[aIN[i]] && aTags[aIN[i]].TAGCOLOR?aTags[aIN[i]].TAGCOLOR:''),
-				color:(aTags[aIN[i]] && aTags[aIN[i]].TEXTCOLOR?aTags[aIN[i]].TEXTCOLOR:'')
+				id: id,
+				ID: (aTags[aIN[i]] || {}).ID,
+				bgcolor: (aTags[aIN[i]] && tagColor || ''),
+				color: (aTags[aIN[i]] && textColor || '')
 			});
 		}	
 
