@@ -1,7 +1,7 @@
 _me = obj_timetable.prototype;
 function obj_timetable(){};
 
-_me.__constructor = function(owner,evnid,bAttendee){
+_me.__constructor = async function(owner,evnid,bAttendee){
 
 	var me = this;
 
@@ -11,7 +11,7 @@ _me.__constructor = function(owner,evnid,bAttendee){
 	this.__attendee = bAttendee;
 
 	//WORKING HOURS
-	storage.library('gw_others');
+	await storage.library('gw_others');
 	this.__workstart = GWOthers.getItem('CALENDAR_SETTINGS','day_begins')*60;
 	this.__workend = GWOthers.getItem('CALENDAR_SETTINGS','day_ends')*60;
 
@@ -39,7 +39,7 @@ _me.__constructor = function(owner,evnid,bAttendee){
 	this._list = this._getAnchor('list');
 	this._heading = this._getAnchor('heading');
 
-	storage.library('wm_freebusy');
+	await storage.library('wm_freebusy');
 	this._freeBusy = new wm_freebusy();
 
 	//prepare timeline
@@ -55,7 +55,7 @@ _me.__constructor = function(owner,evnid,bAttendee){
 	this._getAnchor('body').unselectable = "on";
 
 	//Scroll handler
-	this._body.onscroll = function(e){
+	this._body.onscroll = function(){
 		var scpos = this.scrollLeft;
 
 		me._list.style.marginTop = (this.scrollTop*-1) + 'px';
@@ -73,24 +73,19 @@ _me.__constructor = function(owner,evnid,bAttendee){
 		this.__timer = setTimeout(me._pathName+'.__query('+ (me.__base_date + pozice) +','+ (me.__base_date + pozice + zobrazeno - 1) +')',500);
 	};
 
-	this._body.onmousedown = function(e){
+	this._body.onmousedown = function(){
 		this.__lastScrollLeft = this.scrollLeft;
 	};
 
-	//Other Browsers onmouseup handler
-	if (currentBrowser().indexOf('MSIE')!==0)
-		this._body.onmouseup = function(e){
-			var e = e || window.event;
-			var elm = e.target || e.srcElement;
-
-			if (elm==this && this.__lastScrollLeft != this.scrollLeft){
-				if (this.scrollLeft<me.__hopzone)
-					me._scrollPrev();
-				else
-				if (me._container.clientWidth-this.scrollLeft<me.__hopzone)
-					me._scrollNext();
-			}
-		};
+	this._body.onmouseup = function(e){
+		if (e.target==this && this.__lastScrollLeft != this.scrollLeft){
+			if (this.scrollLeft<me.__hopzone)
+				me._scrollPrev();
+			else
+			if (me._container.clientWidth-this.scrollLeft<me.__hopzone)
+				me._scrollNext();
+		}
+	};
 
 	//Prev Button
 	this.rf._onclick = function(){
@@ -119,10 +114,9 @@ _me.__constructor = function(owner,evnid,bAttendee){
 	};
 
 	/* SELECTION */
-	this._container.ondragstart = function(e){ return false; };
+	this._container.ondragstart = function(){ return false; };
 	this._container.onmousedown = function(e){
-		var e = e || window.event;
-		var elm = e.target || e.srcElement;
+		var elm = e.target, posX, div;
 
 		var posC = getSize(me._body);
 
@@ -131,8 +125,8 @@ _me.__constructor = function(owner,evnid,bAttendee){
 
 			gui._obeyEvent('mouseup',[dispatch]);
 
-			var div  = document.getElementById(me._pathName+'/selection');
-			var posX = div.offsetLeft;
+			div  = document.getElementById(me._pathName+'/selection');
+			posX = div.offsetLeft;
 			var posD = div.offsetWidth;
 
 			if (elm.className == 'LSide'){
@@ -205,15 +199,15 @@ _me.__constructor = function(owner,evnid,bAttendee){
     		return;
 		}
 
-		var posX = e.clientX - posC.x + me._body.scrollLeft;
-			posX = Math.floor(posX/30)*30;
+		posX = e.clientX - posC.x + me._body.scrollLeft;
+		posX = Math.floor(posX/30)*30;
 
 		var a = me.__base_date - (me.__base_position/1440);
 		var b = Math.floor(posX/1440);
 
 		me._value({STARTDATE:a + b,STARTTIME: posX-(b*1440),ENDDATE:a + b,ENDTIME:posX-(b*1440)+30},1);
 
-		var div = document.getElementById(me._pathName+'/selection');
+		div = document.getElementById(me._pathName+'/selection');
 
 		//RESIZE 2
 		gui._obeyEvent('mouseup',[dispatch]);
@@ -243,7 +237,7 @@ _me.__constructor = function(owner,evnid,bAttendee){
 
 			//SET VALUE
 			var b,a = me.__base_date - (me.__base_position/1440);
-			if (!Is.Undefined(left)){
+			if (Is.Defined(left)){
 				b = Math.floor(left/1440);
                 me.__value.STARTDATE = a + b;
                 me.__value.STARTTIME = left - (b*1440);
@@ -251,8 +245,8 @@ _me.__constructor = function(owner,evnid,bAttendee){
 				div.style.left = left + 'px';
 			}
 
-			if (!Is.Undefined(width)){
-				var b = Math.floor((left+width)/1440);
+			if (Is.Defined(width)){
+				b = Math.floor((left+width)/1440);
 				me.__value.ENDDATE = a + b;
                 me.__value.ENDTIME = (left+width) - (b*1440);
 
@@ -260,12 +254,6 @@ _me.__constructor = function(owner,evnid,bAttendee){
 			}
 		};
 		gui._obeyEvent('mousemove',[me,'__move_handler']);
-
-		/*
-		//MSIE stupid scroll fix
-		if (currentBrowser().indexOf('MSIE') === 0)
-			setTimeout(me._pathName+'._body.scrollLeft = '+me._body.scrollLeft,0);
-		*/
 	};
 
 
@@ -281,8 +269,8 @@ _me.__constructor = function(owner,evnid,bAttendee){
 		};
 		if (dataSet.get('main',['resources_path'])){
 			this.addr_button._disabled(false);
-			this.addr_button._onclick = function() {
-				var frm = gui._create('address_book', 'frm_addaddress', '', '',[me, '__onAddNewFromAddressbook',['S']], ['ADDRESS_BOOK::SELECTED_ADDRESSES'],'','',true,4);
+			this.addr_button._onclick = async function() {
+				var frm = await gui._create('address_book', 'frm_addaddress', '', '',[me, '__onAddNewFromAddressbook',['S']], ['ADDRESS_BOOK::SELECTED_ADDRESSES'],'','',true,4);
 					frm.select._disabled(true);
 			};
 		}
@@ -340,8 +328,7 @@ _me.__constructor = function(owner,evnid,bAttendee){
 	};
 
 	this._list.onclick = function(e){
-		var e = e || window.event,
-			elm = e.target || e.srcElement,
+		var elm = e.target,
 			bRemove = false;
 
 		if (elm.tagName == 'SPAN') {
@@ -367,7 +354,7 @@ _me.__constructor = function(owner,evnid,bAttendee){
 							removecss(tmp,'active');
 							this.__users[active[i]].active = false;
 						}
-						catch(r){gui._REQUEST_VARS.debug && console.log(this._name||false,r);}
+						catch(r){ console.log(this._name||false,r);}
 					}
 				}
 
@@ -383,21 +370,15 @@ _me.__constructor = function(owner,evnid,bAttendee){
 
 					this.oncontextmenu(e);
 
-					e.cancelBubble=true;
-					if (e.preventDefault)
-						e.preventDefault();
-					if (e.stopPropagation)
-						e.stopPropagation();
-
-					return false;
+					e.preventDefault();
+					e.stopPropagation();
 				}
 			}
 		}
 	};
 
-	this._list.oncontextmenu = function(e){
-		var e = e || window.event,
-			elm = e.target || e.srcElement;
+	this._list.oncontextmenu = async function(e){
+		var elm = e.target;
 
 		if (elm.tagName == 'SPAN')
 			elm = elm.parentNode;
@@ -411,7 +392,7 @@ _me.__constructor = function(owner,evnid,bAttendee){
 			if (e.clientX-pos.x<20 && me.__attendee)
 				return;
 
-			me.cmenu = gui._create('cmenu','obj_context','','obj_timetable_context');
+			me.cmenu = await gui._create('cmenu','obj_context','','obj_timetable_context');
 
 			me.cmenu._onclick = function(e,elm,id,arg){
 				var usr;
@@ -428,7 +409,7 @@ _me.__constructor = function(owner,evnid,bAttendee){
 						{'title':'ATTENDEES::STATUS_A',css:'bg_status_A','arg':{'id':id,'key':'status',value:'A'}},
 						{'title':'ATTENDEES::STATUS_D',css:'bg_status_D','arg':{'id':id,'key':'status',value:'D'}}];
 
-				me.cmenu._fill(aMenu);
+				await me.cmenu._fill(aMenu);
 				me.cmenu._place(gui._rtl?pos.x+218:pos.x+10,pos.y+pos.h,150,2);
 			}
 			//Role
@@ -437,17 +418,16 @@ _me.__constructor = function(owner,evnid,bAttendee){
 						{'title':'ATTENDEES::ROLE_S',css:'bg_role_S','arg':{'id':id,'key':'role',value:'S'}},
 						{'title':'ATTENDEES::ROLE_T',css:'bg_role_T','arg':{'id':id,'key':'role',value:'T'}}];
 
-				me.cmenu._fill(aMenu);
+				await me.cmenu._fill(aMenu);
 				me.cmenu._place(gui._rtl?pos.x+236:pos.x+30,pos.y+pos.h,150,2);
 			}
 		}
 
-		return false;
+		e.preventDefault();
 	};
 
 	this._list.ondblclick = function(e){
-		var e = e || window.event,
-			elm = e.target || e.srcElement;
+		var elm = e.target;
 
 		if (elm.id && elm.id.indexOf((me._pathName+'.list/'))==0){
 			var active = this._getActive();
@@ -568,7 +548,7 @@ _me.__constructor = function(owner,evnid,bAttendee){
                 doUpd = true;
 
 			//merge values
-			for (var i in aValues)
+			for (i in aValues)
 				this.__users[iid][i] = aValues[i];
 
 			this.__users[iid].action = 'edit';
@@ -603,8 +583,8 @@ _me.__constructor = function(owner,evnid,bAttendee){
 		}
 	};
 
-	//list.onmousewheel -> body
-	AttachEvent(this._list, "onmousewheel", function(e){
+	//list.onwheel -> body
+	AttachEvent(this._list, "onwheel", function(e){
 		me._body.scrollTop += e.deltaY*10;
 	});
 
@@ -645,7 +625,7 @@ _me._scrollPrev = function(){
 	}
 
 	//POSUNOUT ZBYTEK DOPRAVA
-	for (var i in this.__rendered){
+	for (i in this.__rendered){
 
 		this.__rendered[i][0] += this.__hop;
 
@@ -672,7 +652,7 @@ _me._scrollPrev = function(){
 	this._value(this.__value,1);
 };
 
-_me._scrollNext = function(e){
+_me._scrollNext = function(){
 	//SMAZAT POSLEDNI DNY ZLEVA
 	var base = this.__base_date - (((this.__range/1440)-1)/2);
 
@@ -693,7 +673,7 @@ _me._scrollNext = function(e){
 	}
 
 	//POSUNOUT ZBYTEK DOLEVA
-	for (var i in this.__rendered){
+	for (i in this.__rendered){
 		this.__rendered[i][0] -= this.__hop;
 
         if((elm = document.getElementById(this._pathName+'/time/'+i))){
@@ -756,7 +736,7 @@ _me.__query = function(iStart, iEnd, bClean){
 
 	//REQUEST DATA
     var aData = {users:[]};
-	for (var i in this._list.__users)
+	for (i in this._list.__users)
 		if (this._list.__users[i].action != 'remove')
         	aData.users.push(this._list.__users[i].email || this._list.__users[i].name);
 
@@ -781,10 +761,10 @@ _me._response = function(aData,bClean,iStart,iEnd){
 				if (day<iStart || day>iEnd)
 					this._removeDay(day);
 
-		for(var i = iStart;i<=iEnd;i++)
+		for(i = iStart;i<=iEnd;i++)
 			this._addEvents(i);
 	}
-	catch(r){gui._REQUEST_VARS.debug && console.log(this._name||false,r);}
+	catch(r){ console.log(this._name||false,r);}
 };
 
 
@@ -839,7 +819,7 @@ _me._addEvents = function(day){
 		    return a.STARTTIME-b.STARTTIME;
 		};
 		sum.sort(sorter);
-		for(var i = 1;i<sum.length;i++){
+		for(i = 1;i<sum.length;i++){
 			if ((sum[i-1].ENDTIME) >= (sum[i].STARTTIME)){
 				if (sum[i-1].ENDTIME<sum[i].ENDTIME)
 				    sum[i-1].ENDTIME=sum[i].ENDTIME;
@@ -849,7 +829,7 @@ _me._addEvents = function(day){
 		}
 
 		str = '';
-		for(var i = 0;i<sum.length;i++)
+		for(i = 0;i<sum.length;i++)
 			str += '<div class="busy" style="left: '+ sum[i].STARTTIME +'px;width:'+(sum[i].ENDTIME-sum[i].STARTTIME)+'px"></div>';
 
 		if (str){
@@ -916,11 +896,11 @@ _me._removeDay = function(day){
 };
 
 
-_me._value = function(aValue,bNoScroll,bForce){
+_me._value = function(aValue,bNoScroll){
 
 	if (aValue){
 
-		if (Is.Undefined(aValue.STARTDATE) || Is.Undefined(aValue.STARTTIME) || Is.Undefined(aValue.ENDDATE) || Is.Undefined(aValue.ENDTIME))
+		if (!Is.Defined(aValue.STARTDATE) || !Is.Defined(aValue.STARTTIME) || !Is.Defined(aValue.ENDDATE) || !Is.Defined(aValue.ENDTIME))
 		    return;
 
 		if (!bNoScroll)
