@@ -1,11 +1,11 @@
 _me = obj_timezones.prototype;
 function obj_timezones() {};
 
-_me.__constructor = function(sDefault) {
+_me.__constructor = async function(sDefault) {
 
 	var aTZ = dataSet.get('storage',['TIMEZONES','ITEMS']);
 	for(var i in aTZ)
-		this.__idTable[aTZ[i].VALUES.TZID.VALUE] = [aTZ[i].VALUES.NAME.VALUE,'',aTZ[i].VALUES.NAME.VALUE];
+		this.__idTable[normalizeTimezone(aTZ[i].VALUES.TZID.VALUE)] = [aTZ[i].VALUES.NAME.VALUE,'',aTZ[i].VALUES.NAME.VALUE];
 
 	//move floating time to the end
 	if (this.__idTable['F']){
@@ -22,4 +22,8 @@ _me.__constructor = function(sDefault) {
 	}
 	else
 		this._value('F');
-};
\ No newline at end of file
+};
+
+_me._value = function(v, bNoEvn) {
+	return obj_select.prototype._value.call(this, normalizeTimezone(v), bNoEvn);
+}
\ No newline at end of file
