@@ -17,7 +17,7 @@
 _me = obj_tree.prototype;
 function obj_tree(){};
 
-_me.__constructor = function(){
+_me.__constructor = async function(){
 
 	this._telemetry = 'id'; //telemetry log
 
@@ -41,9 +41,9 @@ _me.__constructor = function(){
 
 		//Esc
 		if (e.keyCode == 27){
-			if (v === '')
-				me._focus();
-			else{
+			if (v === '') {
+				// !bSkipFocus && me._focus();
+			}else{
 				this._value('');
 				v = '';
 			}
@@ -65,76 +65,55 @@ _me.__constructor = function(){
 
 
 	//IM focus
-	var hasFocus = false;
-	this._main.setAttribute('tabindex', -1);
-	//IE trick
-	if (~currentBrowser().indexOf('MSIE')){
-		AttachEvent(this._main, 'onclick', function () {
-			hasFocus = true;
-		});
-	}
-	else{
-		AttachEvent(this._main, 'onfocus', function () {
-			hasFocus = true;
-		});
-	}
-	AttachEvent(this._main, 'onblur', function () {
-		hasFocus = false;
-	});
 	AttachEvent(this._main, 'onkeydown', function (e) {
-		if (hasFocus && !e.isComposing && e.keyCode !== 229 && (e.key || '').length === 1)
-			me.inp_search._focus();
+		switch(e.key) {
+			case 'ArrowUp':
+				if (me._setPreviousActive()) {
+					e.preventDefault();
+					e.stopPropagation();
+				}
+			break;
+			case 'ArrowDown':
+				if (me._setNextActive()) {
+					e.preventDefault();
+					e.stopPropagation();
+				}
+			break;
+			case 'Enter':
+				me._onsubmit && me._onsubmit(e);
+		}
 	});
 
 
 	this.__eBody.onclick = function(e){
-		var e = e || window.event,
-			elm = e.target || e.srcElement,
+		var elm = e.target,
 			eli,id;
 
 		switch(elm.tagName){
+			case 'P':
+				e.stopPropagation();
+
+				var p = getSize(elm);
+				e = {
+					target: elm,
+					clientX: gui._rtl ? p.x : (p.x + p.w),
+					clientY: p.y + (p.h / 2),
+					type: 'contextmenu',
+					originalType: e.type
+				};
+
 			case 'B':
 				elm = Is.Child(elm,'DIV');
 
 			case 'DIV':
-				if (elm.parentNode.tagName == 'LI'){
-
-					if (hascss(elm.parentNode,'active') || hascss(elm.parentNode,'menu')){
-						var p = getSize(me._main),
-							ctx;
-
-						if (gui._rtl){
-							ctx = e.clientX - p.x < 33;
-						}
-						else{
-							ctx = p.x + p.w - e.clientX < 33;
-						}
-
-						if (ctx){
-
-							e.cancelBubble = true;
-							if (e.stopPropagation)
-								e.stopPropagation();
-
-							var p2 = getSize(elm);
-							e = {
-								srcElement: elm,
-								clientX: gui._rtl ? p.x + 22 : p.x + p.w,
-								clientY: p2.y + (p2.h / 2),
-								type: 'contextmenu',
-								originalType: e.type
-							};
-						}
-					}
-				}
-				else
-					break;
-
 			case 'SPAN':
 			case 'I'   :
 			case 'U'   :
 
 				eli = Is.Child(elm,'LI');
+				if (!eli) {
+					return;
+				}
 				id = eli.id.substr(me._pathName.length+1);
 
 				if (elm.tagName == 'SPAN' && e.type != 'contextmenu'){
@@ -164,25 +143,15 @@ _me.__constructor = function(){
 		return false;
 	};
 
-	//For MSIE10 Only
-	// if (window.navigator.msPointerEnabled)
-	// 	this.__eBody.onmspointerup = function(e){
-	// 		var e = e || window.event;
-	// 		if (me._autofocus && e.button == 0 && (!e.pointerType || (e.pointerType == e.MSPOINTER_TYPE_MOUSE) || (e.pointerType == 'mouse')))
-	// 			me._focus();
-	// 	};
-
-
 	this.__eBody.ondblclick = function(e){
-		var e = e || window.event;
-		var elm = e.target || e.srcElement;
+		var elm = e.target, eli;
 
 		switch(elm.tagName){
 			case 'U':
 			case 'B':
 				if (me._ondblclick){
-					var eli = Is.Child(elm,'LI'),
-						id = eli.id.substr(me._pathName.length+1),
+					eli = Is.Child(elm,'LI');
+					var id = eli.id.substr(me._pathName.length+1),
 						idt = me.__idtable[id]?clone(me.__idtable[id],true):{};
 
 					var realId = me._getRealId(id);
@@ -194,7 +163,7 @@ _me.__constructor = function(){
 				}
 
 			case 'I':
-				var eli = Is.Child(elm,'LI');
+				eli = Is.Child(elm,'LI');
 				me.__open(eli.id.substr(me._pathName.length+1), hascss(eli,'root')?'plus':'');
 		}
 	};
@@ -202,6 +171,27 @@ _me.__constructor = function(){
 	this.__eBody.oncontextmenu = this.__eBody.onclick;
 };
 
+_me._setPreviousActive = function() {
+	var ids = Object.keys(this.__idtable);
+	var i = ids.indexOf(this._getTreeId(this.__activeNode));
+	while(--i > -1) {
+		if (this.__idtable[ids[i]] && !this.__idtable[ids[i]].disabled) {
+			this._setActive(this._getRealId(ids[i]));
+			return true;
+		}
+	}
+};
+
+_me._setNextActive = function() {
+	var ids = Object.keys(this.__idtable);
+	var i = ids.indexOf(this._getTreeId(this.__activeNode));
+	while(++i < ids.length) {
+		if (this.__idtable[ids[i]] && !this.__idtable[ids[i]].disabled) {
+			this._setActive(this._getRealId(ids[i]));
+			return true;
+		}
+	}
+};
 
 _me._disabled = function(b){
 	if (Is.Defined(b))
@@ -216,7 +206,7 @@ _me._getFocusElement = function(){
 };
 _me._focus = function(){
 	// return this.inp_search._focus();
-	return this._main.focus();
+	// return this._main.focus();
 };
 
 /**
@@ -234,7 +224,7 @@ _me._focus = function(){
 	.. [itmclass] =
  *
  **/
-_me._fill = function (aData){
+_me._fill = async function (aData){
 
 	if (aData){
 		this.__aData = aData;
@@ -260,7 +250,7 @@ _me._fill = function (aData){
 		if (this._filterRawData)
 			aData = this._filterRawData(aData);
 
-		this.__aFillData = this.__prepare_data?this.__prepare_data(aData):aData;
+		this.__aFillData = this.__prepare_data?await this.__prepare_data(aData):aData;
 
 		this.__row(this.__aFillData);
 
@@ -293,6 +283,8 @@ _me._fill = function (aData){
 	} else {
 		addcss(this._main, 'noitems');
 	}
+
+	this._onfill && this._onfill(Object.keys(this.__idTableReal));
 };
 
 _me._filter = function (aData){
@@ -300,13 +292,14 @@ _me._filter = function (aData){
 		aOpen = [];
 
 	var	aKey = [],
-		sName;
+		sName,
+		sLast = '';
 
 	for(var i in aData)
 		aKey.push(i);
 	aKey.sort();
 
-	for(var sLast = '',i = aKey.length-1;i>=0;i--){
+	for(i = aKey.length-1;i>=0;i--){
 
 		if (aKey[i] == sLast)
 			sLast = aKey[i].indexOf('/')>-1?aKey[i].substr(0,aKey[i].lastIndexOf('/')):aKey[i];
@@ -325,7 +318,7 @@ _me._filter = function (aData){
 
 	if (aKey.length){
 		aData2 = {};
-		for(var i in aData)
+		for(i in aData)
 			if (inArray(aKey,i)>-1)
 				aData2[i] = aData[i];
 	}
@@ -336,31 +329,31 @@ _me._filter = function (aData){
 
 _me._invalidateActive = function() {
 	try{
-		removecss(document.getElementById(this._getElmId(this._getTreeId(this.__activeNode))),'active');
+		removecss(this._main.ownerDocument.getElementById(this._getElmId(this._getTreeId(this.__activeNode))),'active');
 	}
-	catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+	catch(e){ console.log(this._name||false,e);}
 
 	delete this.__activeNode;
 };
 
 
 _me._getActive = function(bNamed) {
-	return Path.split(this.__activeNode,bNamed);
+	return Path.split(this.__activeNode || '', bNamed);
 };
 
 /**
  * @brief: set active node and open thread
  **/
-_me._setActive = function(id, bNoOpen){
-
+_me._setActive = async function(id, bNoOpen){
+	id = id.replace(/__@@VIRTUAL@@__\/@@SUBTREE@@\//, '');
 	var elm, treeId = this._getTreeId(id);
 	if (this.__activeNode && this.__activeNode != id){
-		elm = document.getElementById(this._getElmId(this._getTreeId(this.__activeNode)));
+		elm = this._main.ownerDocument.getElementById(this._getElmId(this._getTreeId(this.__activeNode)));
 		if (elm)
 			removecss(elm, 'active');
 	}
 
-	elm = document.getElementById(this._getElmId(treeId));
+	elm = this._main.ownerDocument.getElementById(this._getElmId(treeId));
 	if (elm && elm.parentNode){
 
 		if (!bNoOpen){
@@ -378,7 +371,7 @@ _me._setActive = function(id, bNoOpen){
 	}
 
 	if (this._onactivate)
-		this._onactivate(id, this.__activeNode != id);
+		await this._onactivate(id, this.__activeNode != id);
 
 	this.__activeNode = id;
 	this.__exeEvent('activate',null,{"id":id,"owner":this});
@@ -404,7 +397,7 @@ _me._parseElmId = function (id){
  * @date : 12.5.2006 11:50:37
  **/
 _me.__row = function(aData,child){
-	var path='',elm1,elm2,elm3,elm4,liclass;
+	var path='',elm1,elm2,elm3,elm4,elm5,liclass;
 
 	/* create main UL element */
 	elm1 = mkElement("ul");
@@ -420,8 +413,15 @@ _me.__row = function(aData,child){
 
 	for(var i in aData){
 		elm2 = mkElement("li",{"className":liclass + (aData[i]["liclass"]?' '+aData[i]["liclass"]:''),"id":this._pathName + '/' + path + (aData[i].id || i)});
-		elm3 = mkElement("div",{"className":(aData[i]['ico']?'ico ' + aData[i]['ico']:'')});
-		elm3.innerHTML = '<span class="plussign"></span><i></i>';
+		elm5 = mkElement('span', {
+			className: 'plussign'
+		});
+		elm3 = mkElement("div",{"className":(aData[i]['ico']?'ico ' + aData[i]['ico']:'')}, false, [
+			elm5, mkElement('i')
+		]);
+		if (aData[i].style) {
+			elm3.setAttribute('style', aData[i].style);
+		}
 
 		if (aData[i]['arg'])
 			this.__idtable[path + i] = aData[i]['arg'];
@@ -435,13 +435,18 @@ _me.__row = function(aData,child){
 		if (aData[i]["itmclass"])
 			addcss(elm4,aData[i]["itmclass"]);
 
-		elm4.innerHTML = (aData[i]['title']?getLang(aData[i]['title'],null,true):(aData[i]['text']?aData[i]['text']:i.toString().entityify()))/* + (aData[i]['title2']?'<u>'+aData[i]['title2']+'</u>':'')*/;  // Quote STRING !!!
+		elm4.innerHTML = '<span>' + (aData[i]['title']?getLang(aData[i]['title'],null,true):(aData[i]['text']?aData[i]['text']:i.toString().entityify())) + '</span>';
 
 		elm3.appendChild(elm4);
-		aData[i].group && elm3.appendChild(mkElement('div', {text:aData[i].group,class:'unread-group'}));
-
-		if (aData[i]['title2'])
-			elm3.appendChild(mkElement('u',{innerHTML:aData[i]['title2']}));
+		if (aData[i].group && (typeof aData[i].group !== 'function')) {
+			elm4.appendChild(mkElement('span', {
+				text: (aData[i]['title2'] || 0) + aData[i].group,
+				class:'unread-group'
+			}));
+		}
+		if (aData[i]['title2']) {
+			elm4.appendChild(mkElement('u',{innerHTML:aData[i]['title2']}));
+		}
 
 		elm2.appendChild(elm3);
 
@@ -455,6 +460,7 @@ _me.__row = function(aData,child){
 		}
 		// n++;
 		elm1.appendChild(elm2);
+		elm3.appendChild(mkElement('p'));
 	}
 
 	/* set "end" className for last node with childs */
@@ -481,7 +487,8 @@ _me.__row = function(aData,child){
 
 		// active
 		if (this.__activeNode){
-			var bDoNotOpen = this._value().indexOf(Path.basedir(this.__activeNode)) == -1;
+			// never expand parent nodes
+			var bDoNotOpen = true;//this._value().indexOf(Path.basedir(this.__activeNode)) == -1;
 			this._setActive(this.__activeNode, bDoNotOpen);
 		}
 
@@ -509,15 +516,17 @@ _me.__openup = function(css,id){
 	if (id)
 		a = [id];
 	else
-		a = this.__value.map(this._getTreeId, this);
+		a = this.__value.map(function(id) {
+			return this._getTreeId(id, true);
+		}, this);
 
 	for (var elm, newPath, i = 0; a[i]; i++){
 
-		var newPath = Path.basedir(a[i]);
+		newPath = Path.basedir(a[i]);
 		if (newPath && a.indexOf(newPath) == -1)
 			a.push(newPath);
 
-		elm = document.getElementById(this._getElmId(a[i]));
+		elm = this._main.ownerDocument.getElementById(this._getElmId(a[i]));
 		if (elm && elm.tagName == 'LI' && !hascss(elm, css) && hascss(elm, inv)){
 			addcss(elm,css);
 			removecss(elm, inv);
@@ -570,7 +579,7 @@ _me._open = function(id,style){
 					try {
 
 						this._getTreeId(this.__value[i], true).forEach(function(treeId){
-							var tmp = document.getElementById(this._getElmId(treeId));
+							var tmp = this._main.ownerDocument.getElementById(this._getElmId(treeId));
 							if (tmp && hascss(tmp, 'minus')) {
 								removecss(tmp, 'minus');
 								addcss(tmp, 'plus');
@@ -578,27 +587,19 @@ _me._open = function(id,style){
 						}, this);
 
 					} catch (e) {
-						gui._REQUEST_VARS.debug && console.log(this._name || false, e);
+						 console.log(this._name || false, e);
 					}
 					this.__value.splice(i, 1);
 					break;
 				}
 			}
 		}
-		// WC-8623, should be handled by _getTreeId
-		// else {
-		// 	elm = null;
-		// 	var a = treeId.split("/");
-		// 	if (a.length > 1)
-		// 		this.__open(a.slice(0, -1).join("/"), 'minus');
-		// 	return;
-		// }
+
+		/* destruct elm var */
+		elm = null;
 
 	}.bind(this));
 
-	/* destruct elm var */
-	elm = null;
-
 	/* save opened threads into dataset */
 	if (this._saveState && this._listener)
 		this._saveme();
@@ -640,11 +641,11 @@ _me.__update = function(sDataSet,aDataPath){
 
 		if (aDataPath && aDataPath[2] == 'RECENT'){
 			var elm,elm2;
-			if ((elm = document.getElementById(this._pathName+'/'+aDataPath[0]+'/'+aDataPath[1]))){
+			if ((elm = this._main.ownerDocument.getElementById(this._pathName+'/'+aDataPath[0]+'/'+aDataPath[1]))){
 
-				var tmp = dataSet.get(sDataSet,[aDataPath[0], aDataPath[1]]);
-					iRec = tmp.RECENT,
-					elm2 = (elm2 = elm.getElementsByTagName('DIV')) && (elm2 = elm2[0].getElementsByTagName('U'))?elm2[0]:false;
+				var tmp = dataSet.get(sDataSet,[aDataPath[0], aDataPath[1]]),
+					iRec = tmp.RECENT;
+				elm2 = (elm2 = elm.getElementsByTagName('DIV')) && (elm2 = elm2[0].getElementsByTagName('U'))?elm2[0]:false;
 
 				if ((parseInt(iRec) || parseInt(tmp.COUNT))>0){
 
@@ -655,12 +656,17 @@ _me.__update = function(sDataSet,aDataPath){
 
 					elm2.innerHTML = tmp.COUNT || ('&#8234;' + iRec.toString().escapeHTML());
 					elm.classList.add('unread');
+
+					var folders = gui.frm_main.folderPicker;
+					if (folders && !folders._main.classList.contains('active')) {
+						folders._main.classList.add('unread');
+					}
 				}
 				else{
 					elm.classList.remove('unread');
 
 					if (elm2) {
-						var parent = document.getElementById(this._pathName+'/'+aDataPath[0]+'/'+aDataPath[1].split('/').slice(0, -1).join('/'));
+						var parent = this._main.ownerDocument.getElementById(this._pathName+'/'+aDataPath[0]+'/'+aDataPath[1].split('/').slice(0, -1).join('/'));
 						if(parent) {
 							var count = parent.querySelector('div div.unread-group');
 							if(count && parseInt(count.innerText)) {
@@ -696,7 +702,6 @@ _me.__update = function(sDataSet,aDataPath){
  * @date : 14.5.2006 17:33:56
  **/
 _me._value = function (v){
-
 	if (v && typeof(v) == 'object') {
 		// close all nodes
 		if (this.__value.length) this.__openup('plus');
@@ -707,22 +712,24 @@ _me._value = function (v){
 			ds = this._listener_data?dataSet.get(this._listener_data,this._listenerPath_data):null;
 
 		for(var i = this.__value.length; i--;){
-
 			if (!Is.String(this.__value[i]))
 				continue;
 
-			aPath = this.__value[i].split('/');
-			fPath = Path.split(this.__value[i]);
+			var value = this.__value[i];
+			var prefix = '';
+
+			aPath = value.split('/');
+			fPath = Path.split(value);
 
 			do{
 				sPath = aPath.join('/');
-				treeId = this._getTreeId(sPath, true);
+				treeId = this._getTreeId(prefix + sPath, true);
 
 				//Open folder node
 				if (treeId.length){
 
 					//add new path to the values (if folder doesn't exit in DS)
-					if (sPath != this.__value[i] && (!this._listener_data || !ds[fPath[0]] || (fPath[1] && !ds[fPath[0]][fPath[1]]))){
+					if (sPath != value && (!this._listener_data || !ds[fPath[0]] || (fPath[1] && !ds[fPath[0]][fPath[1]]))){
 						this.__value.splice(i,1,sPath);
 					}
 
@@ -739,17 +746,17 @@ _me._value = function (v){
 
 			if (treeId.length){
 				treeId.forEach(function(treeId){
-					if ((elm = document.getElementById(this._getElmId(treeId)))){
+					if ((elm = this._main.ownerDocument.getElementById(this._getElmId(treeId)))){
 						if (elm.tagName == 'LI' && elm.getElementsByTagName('UL').length>0){
 							removecss(elm,'plus');
 							addcss(elm,'minus');
 						}
 					}
-				}.bind(this));
+				}, this);
 			}
 			else
 			//do not remove folders existing in DS
-			if (!this._listener_data || !ds[fPath[0]] || (fPath[1] && !ds[fPath[0]][fPath[1]])){
+			if ((!this._listener_data || !ds[fPath[0]] || (fPath[1] && !ds[fPath[0]][fPath[1]])) && (fPath[0] !== 'root::other') && (!fPath[0] === 'other' || !ds[fPath[1]])){
 				this.__value.splice(i,1);
 			}
 		}
@@ -789,14 +796,4 @@ _me._getTreeId = function(id, bArray){
  */
 _me._getRealId = function(id){
 	return id;
-};
-
-
-/**
- * @brief: optional onclick method
- * @param: e    - Event Handler (onclick event object)
- *         elm  - srcElement (link)
- *         id   - objects internal id (without pathName)
- *         arg  - item's arg section
- */
-//_me._onclick = function(e,elm,id,arg){};
+};
\ No newline at end of file
