@@ -5,7 +5,7 @@ function obj_tree_folder(){};
  * @date: 6.7.2006 16:28:30
  **/
 
-_me.__constructor = function(sFilterAccountId, sFilterFolderType, sFilterRights, bSkipVirtual)
+_me.__constructor = async function(sFilterAccountId, sFilterFolderType, sFilterRights, bSkipVirtual)
 {
 	var me = this;
 
@@ -31,7 +31,7 @@ _me.__constructor = function(sFilterAccountId, sFilterFolderType, sFilterRights,
 	if (!dataSet.get('folders')){
 		var aFolders = {};
 		for (var sAccId in aAccounts) {
-			aFolders[sAccId] = WMFolders.list({"aid": sAccId})[sAccId];
+			aFolders[sAccId] = (await WMFolders.list({"aid": sAccId}))[sAccId];
 		}
 		dataSet.add('folders', null, aFolders);
 	}
@@ -49,9 +49,8 @@ _me.__constructor = function(sFilterAccountId, sFilterFolderType, sFilterRights,
 		//Edit mode
 		if (me.rename) return;
 
-		var e = e || window.event;
-		if (e.button>1 || e.ctrlKey || !me.__initdrag) return;
-		var elm = e.target || e.srcElement;
+		if (e.button>1 || e.ctrlKey || e.metaKey || !me.__initdrag) return;
+		var elm = e.target;
 
 		if (elm == this) return;
 		if (elm.tagName != 'LI')
@@ -59,20 +58,19 @@ _me.__constructor = function(sFilterAccountId, sFilterFolderType, sFilterRights,
 
 		if (!elm || !elm.id) return;
 
-		var tmp = (elm.id.substring(me._pathName.length+1)).split('/');
+		var tmp = Path.split(elm.id.substring(me._pathName.length+1), true);
 
 		//Account
-		if (tmp.length<2) return true;	//Check type, (QL...)
+		if (!tmp.fid) return true;	//Check type, (QL...)
 
-		var id = [{aid:tmp.splice(0,1)[0],fid:tmp.join('/')}],
-			sType = WMFolders.getType(id[0]),
+		var sType = WMFolders.getType(tmp),
 			sDragType = 'folder';
 
 		//Forbidden folders
 		if (!sType || sType == 'A')
 			return true;
 		else
-		if (id[0].aid == sPrimaryAccount && id[0].fid.indexOf('__@@VIRTUAL@@__/')==0)
+		if (tmp.aid == sPrimaryAccount && tmp.fid.indexOf('__@@VIRTUAL@@__/')==0)
 			sDragType = 'virtual_folder';
 
 		//fire the event :)
@@ -80,13 +78,10 @@ _me.__constructor = function(sFilterAccountId, sFilterFolderType, sFilterRights,
 
 		gui._obeyEvent('mouseup',[me,'__dndDispatch']);
 		me.__dndtimer = setTimeout(function(){
-			me.__initdrag(id,sType,x,y,sDragType);
-		},500);
+			me.__initdrag([tmp],sType,x,y,sDragType);
+		}, 250);
 
-		if (e.stopPropagation) e.stopPropagation();
-		e.cancelBubble = true;
-		//if (e.preventDefault) e.preventDefault();
-		//return false;
+		e.stopPropagation();
 	};
 
 	//Registr Drop
@@ -115,7 +110,7 @@ _me._filter = function (aData){
 
 	for (sDomain in aData) {
 		aTmp = aData[sDomain];
-		aKey = [];
+		var aKey = [];
 
 		for (i in aTmp) {
 			aKey.push(i);
@@ -151,7 +146,7 @@ _me._filter = function (aData){
 
 		if (aKey.length){
 			aData2[sDomain] = {};
-			for(var i in aTmp)
+			for(i in aTmp)
 				if (inArray(aKey,i)>-1)
 					aData2[sDomain][i] = aTmp[i];
 		}
@@ -221,15 +216,17 @@ _me.__initdrag = function(id,sType,x,y,sDragType){
 	if ((!sDragType || sDragType == 'folder') && WMFolders.getType(id[0]) == 'Y')
 		return false;
 
-	//prepare data
-	var aName = id[0].fid.split('/'),
-		sBody = '<div class="drag_folder drag_folder_'+sType+'">'+ aName[aName.length-1] +'</div>';
-
 	//create Drag box
-	gui.frm_main.dnd.create_drag(sBody, {type:sDragType || 'folder',value:id}, x, y);
+	gui.frm_main.dnd.create_drag({
+		type: sDragType || 'folder',
+		value: id,
+		x: x,
+		y: y,
+		obj: this
+	});
 };
 
-_me.__mousewheel = function(e){
+_me.__wheel = function(e){
 	this.__eBody.scrollTop += e.delta*20;
 };
 
@@ -240,13 +237,13 @@ _me._active_dropzone = function(v){
 	if(v){
 
 		//Opened Nodes
-		var val = {} ,tmp = this._value();
+		var val = {} ,tmp = this._value(), getTargets;
 		for (var i in tmp)
 			val[tmp[i]] = 1;
 
 		//Opened Nodes by Search
 		if (this.__filter && this.__filterOpen)
-			for (var i in this.__filterOpen)
+			for (i in this.__filterOpen)
 				val[this.__filterOpen[i]] = 1;
 
 		switch(v['type']){
@@ -254,7 +251,7 @@ _me._active_dropzone = function(v){
 
 			var itmtype = WMFolders.getType(v.value[0]);
 
-			var getTargets = function (data, out){
+			getTargets = function (data, out){
 				out = out || [];
 				if (typeof data != 'object') return out;
 
@@ -289,7 +286,7 @@ _me._active_dropzone = function(v){
 			if (!this.__aFillData[v.value[0].aid] || !this.__aFillData[v.value[0].aid].nodes ||	!WMFolders.getRights(v.value[0],'remove') || sType == 'QL' || sType == 'I' || sType == 'Y')
 				break;
 
-			var getTargets = function (data, out){
+			getTargets = function (data, out){
 
                 if (typeof data != 'object') return out;
 
@@ -330,14 +327,6 @@ _me._active_dropzone = function(v){
 	}
 };
 
-_me.__dragscroll = function(){
-	if (this.__objPos){
-		var delta;
-		if (((delta = gui.__Y - this.__objPos.y)<51 && (delta -= 50)) || ((delta = gui.__Y - this.__objPos.y - this.__objPos.h)>-51 && (delta += 50)))
-			this.__eBody.scrollTop += Math.floor(delta/10);
-	}
-};
-
 _me._ondragover = function(v){
 	var me = this;
 
@@ -345,7 +334,7 @@ _me._ondragover = function(v){
 	if (!this.__dragON){
 		this.__dragON = true;
 
-		gui._obeyEvent('mousewheel',[this,'__mousewheel']);
+		gui._obeyEvent('wheel',[this,'__wheel']);
 
 		this.__dragscrollloop = window.setInterval(function(){
 			if (me.__objPos){
@@ -420,16 +409,13 @@ _me._ondragover = function(v){
 		}
 	}
 
-	a = null;
-	elm = null;
-
 	return bOK;
 };
 
-_me._ondragout = function(v){
+_me._ondragout = function(){
 
 	//disobey scroll
-	gui._disobeyEvent('mousewheel',[this,'__mousewheel']);
+	gui._disobeyEvent('wheel',[this,'__wheel']);
 	this.__dragON = false;
 
 	if (this.__dragscrollloop){
@@ -448,10 +434,10 @@ _me._ondragout = function(v){
 	}
 };
 
-_me._ondrop = function(v){
+_me._ondrop = async function(v){
 
 	//disobey scroll
-	gui._disobeyEvent('mousewheel',[this,'__mousewheel']);
+	gui._disobeyEvent('wheel',[this,'__wheel']);
 	this.__dragON = false;
 
 	var size,elm,id = '';
@@ -489,17 +475,17 @@ _me._ondrop = function(v){
 		gui.frm_main.dnd.remove_drag(true);
 
 		if (fol && dataSet.get('folders',[v.value[0].aid, v.value[0].fid,'DEFAULT'])){
-			gui._create('move','frm_confirm','','',[function(){
+			gui._create('move','frm_confirm','','',[async function(){
 				//WMFolders.add({'aid':v.value[0].aid,'fid':v.value[0].fid,'name':(fol?fol + '/':'') + Path.basename(v.value[0].fid)},'folders');
-				storage.library('obj_context_folder');
-				obj_context_folder.__moveFolder(v.value[0].aid, v.value[0].fid, (fol?fol + '/':'') + Path.basename(v.value[0].fid));
+				await storage.library('obj_context_folder');
+				obj_context_folder.__moveFolder(v.value[0].aid, dataSet.get('folders', [v.value[0].aid, v.value[0].fid, 'RELATIVE_PATH']) || v.value[0].fid, (fol?fol + '/':'') + (dataSet.get('folders', [v.value[0].aid, v.value[0].fid, 'NAME']) || Path.basename(v.value[0].fid)));
 
 			}], 'POPUP_FOLDERS::MOVE_FOLDER', 'POPUP_FOLDERS::MOVE_DEFAULT');
 		}
 		else{
 			//WMFolders.add({'aid':v.value[0].aid,'fid':v.value[0].fid,'name':(fol?fol + '/':'') + Path.basename(v.value[0].fid)},'folders');
-			storage.library('obj_context_folder');
-			obj_context_folder.__moveFolder(v.value[0].aid, v.value[0].fid, (fol?fol + '/':'') + Path.basename(v.value[0].fid));
+			await storage.library('obj_context_folder');
+			obj_context_folder.__moveFolder(v.value[0].aid, dataSet.get('folders', [v.value[0].aid, v.value[0].fid, 'RELATIVE_PATH']) || v.value[0].fid, (fol?fol + '/':'') + (dataSet.get('folders', [v.value[0].aid, v.value[0].fid, 'NAME']) || Path.basename(v.value[0].fid)));
 		}
 
 		break;
@@ -508,7 +494,11 @@ _me._ondrop = function(v){
 		//Drag & Drop doesnt work with popups because of event handlers.
 		gui.frm_main.dnd.remove_drag(true);
 		if (!fol) return;
-        Item.__convertToFolder(acc,fol,v);
+		var iids = [];
+		for (i in v.value){
+			iids.push(v.value[i].iid)
+		}
+		Item.__convertToFolder(acc,fol,v, gui.frm_main.main.list, gui.frm_main.main.list && gui.frm_main.main.list.__getNext(iids));
 	}
 };
 
@@ -520,7 +510,7 @@ _me.__getGroup = function (group) {
 	});
 };
 
-_me.__prepare_data = function(aData)
+_me.__prepare_data = async function(aData)
 {
 	var aResult = {},
 		aFrame,aSplitFolder,aParcSplitFolder,sFolder,sFolderType,
@@ -533,7 +523,7 @@ _me.__prepare_data = function(aData)
 
 	//SORT
 	var srt = [];
-	for(var i in aAccounts_tmp)
+	for(i in aAccounts_tmp)
 		if (i!=sPrimaryAccount)
 			srt.push(i);
 
@@ -542,12 +532,11 @@ _me.__prepare_data = function(aData)
 	if (!this.sFilterAccountId || this.sFilterAccountId == sPrimaryAccount)
 		aAccounts[sPrimaryAccount] = aAccounts_tmp[sPrimaryAccount];
 
-	for(var i = 0;i<srt.length;i++)
+	for(i = 0;i<srt.length;i++)
 		aAccounts[srt[i]] = aAccounts_tmp[srt[i]];
 	aAccounts_tmp = null;
 
 	//get common folders
-	storage.library('wm_folders');
 	var sSpamPath = dataSet.get('main',['spam_path']),
 		sArchivePath = dataSet.get('main',['archive_path']),
 		aDFolders = GWOthers.get('DEFAULT_FOLDERS','storage')['VALUES'],
@@ -556,7 +545,7 @@ _me.__prepare_data = function(aData)
 
 	if (aActive[0] == sPrimaryAccount && aActive[1] == '__@@VIRTUAL@@__/__@@EVENTS@@__'){
 		aCalendars = dataSet.get('folders',[aActive[0],aActive[1],'VIRTUAL','FOLDERS'],true) || {};
-		for (var i in aCalendars) {
+		for (i in aCalendars) {
 			aCalendars[i] = [aCalendars[i],getCalendarColor(i)];
 			if (aCalendars[i][0]){
 				this.__currentVirtualFolder = i;
@@ -614,26 +603,27 @@ _me.__prepare_data = function(aData)
 			}
 
 			//TYPE filter
-			if (this._sFilterFolderType){
+			if (!Is.Empty(this._sFilterFolderType)){
 				if (aData[sAccId][sFolId]['TYPE'] == 'M' && aData[sAccId][sFolId]['RSS'] && !this._sFilterFolderType['R'])
 					continue;
 				else if (this._sFilterFolderType['B'] && aData[sAccId][sFolId]['TYPE'] == 'M' && aData[sAccId][sFolId]['DEFAULT']=='H'){
-
+					//
 				}else if (!this._sFilterFolderType[aData[sAccId][sFolId]['TYPE']]){
 					continue;
 				}
 			}
 			//Hide Group Chat folders in all filters except Y
-			if ((aData[sAccId][sFolId]['TYPE'] == 'Y' ||  aData[sAccId][sFolId]['TYPE'] == 'I') && this._sFilterFolderType && !this._sFilterFolderType['Y'])
+			if ((aData[sAccId][sFolId]['TYPE'] == 'Y' ||  aData[sAccId][sFolId]['TYPE'] == 'I') && !Is.Empty(this._sFilterFolderType) && !this._sFilterFolderType['Y'])
 				continue;
 
+			var bSkip, sRights, r;
 			//RIGHTS filter (regExp.test() doesnt work properly in loop)
 			if (this._sFilterRights){
-				var bSkip = false,
-					sRights = Cookie.get(['rights',sAccId,sFolId]).join('');
+				bSkip = false,
+				sRights = Cookie.get(['rights',sAccId,sFolId]).join('');
 
 				if (sRights.length){
-					for (var r in this._sFilterRights)
+					for (r in this._sFilterRights)
 						if (sRights.indexOf(this._sFilterRights[r])<0){
 							bSkip = true;
 							break;
@@ -645,10 +635,10 @@ _me.__prepare_data = function(aData)
 			}
 
 			if (this._sFilterRightsOr){
-				var bSkip = true,
-					sRights = Cookie.get(['rights',sAccId,sFolId]).join('');
+				bSkip = true;
+				sRights = Cookie.get(['rights',sAccId,sFolId]).join('');
 
-				for (var r in this._sFilterRightsOr)
+				for (r in this._sFilterRightsOr)
 					if (sRights.indexOf(this._sFilterRightsOr[r])>-1){
 						bSkip = false;
 						break;
@@ -663,7 +653,7 @@ _me.__prepare_data = function(aData)
 			aFrame = aResult[sAccId]["nodes"];
 
 			//Procházíme cestu folderu
-			for(var i in aSplitFolder)
+			for(i in aSplitFolder)
 			{
 				aParcSplitFolder.push(aSplitFolder[i]);
 				sFolder = aParcSplitFolder.join('/');
@@ -681,7 +671,7 @@ _me.__prepare_data = function(aData)
 						}
 
 						//hide all groupchat parent folders
-						if (this._sFilterFolderType && this._sFilterFolderType['Y'] && this._sFilterFolderType['I'] && count(this._sFilterFolderType) == 2) {
+						if (!Is.Empty(this._sFilterFolderType) && this._sFilterFolderType['Y'] && this._sFilterFolderType['I'] && count(this._sFilterFolderType) == 2) {
 							if (!this._sFilterFolderType[sFolderType])
 								sLiClass = 'skip';
 
@@ -750,6 +740,7 @@ _me.__prepare_data = function(aData)
 											case aDFolders.trash: sIconClass = 'ico_trash'; break;
 											case aDFolders.sent: sIconClass = 'ico_sent'; break;
 											case aDFolders.drafts: sIconClass = 'ico_drafts'; break;
+											case aDFolders.smartattach: sIconClass = 'ico_upload'; break;
 										}
                                     }
                                     else
@@ -759,6 +750,7 @@ _me.__prepare_data = function(aData)
 										case aDFolders.notes:
 										case aDFolders.journal:
 										case aDFolders.files:
+										case aDFolders.smartattach:
 											sMainClass+=' default';
 									}
 
@@ -811,7 +803,7 @@ _me.__prepare_data = function(aData)
 			}
 		}
 
-		if (this._sFilterFolderType && Is.Empty(aResult[sAccId]['nodes']) && sAccId != sPrimaryAccount) delete aResult[sAccId];
+		if (!Is.Empty(this._sFilterFolderType) && Is.Empty(aResult[sAccId]['nodes']) && sAccId != sPrimaryAccount) delete aResult[sAccId];
 	}
 
 //console.warn(aResult);
