@@ -1,9 +1,13 @@
 _me = obj_tree_folder2.prototype;
 function obj_tree_folder2(){};
 
-_me.__constructor = function(sFilterAccountId, sFilterFolderType, sFilterRights, bSkipVirtual)
+_me.__constructor = async function(sFilterAccountId, sFilterFolderType, sFilterRights, bSkipVirtual, aOptions)
 {
-	this._opt = {privateRootActive:true};
+	this._opt = Object.assign({
+		privateRootActive: true,
+		rootFolderDroppable: false,
+		listenCookie: 'tree'
+	}, this._opt || {}, aOptions);
 
 	if (sFilterAccountId)
 		this._filter_account(sFilterAccountId);
@@ -28,7 +32,7 @@ _me.__prepare_data = function(aData)
 		sTrashPath = GWOthers.getItem('DEFAULT_FOLDERS', 'trash');
 
 	//Get aCalendars for Calendar view
-	if (this._sFilterFolderType && this._sFilterFolderType['E'] && count(this._sFilterFolderType) == 1){
+	if (!Is.Empty(this._sFilterFolderType) && this._sFilterFolderType['E'] && count(this._sFilterFolderType) == 1){
 		aCalendars = dataSet.get('folders',[sPrimaryAccount,'__@@VIRTUAL@@__/__@@EVENTS@@__','VIRTUAL','FOLDERS'],true) || {};
 		for (var i in aCalendars) {
 			aCalendars[i] = [aCalendars[i],getCalendarColor(i)];
@@ -95,7 +99,7 @@ _me.__prepare_data = function(aData)
 			nodes = {'private':[], 'public':[], 'shared':[], resources:[]};
 
 		for (var fid in oFolders){
-			if (oFolders.hasOwnProperty(fid)){
+			if (fid && Object.prototype.hasOwnProperty.call(oFolders, fid)){
 
 				//VIRTUAL filter
 				if ((aCalendars || this._sFilterVirtual) && sAccId == sPrimaryAccount && fid.indexOf('__@@VIRTUAL@@__/') == 0)
@@ -106,7 +110,7 @@ _me.__prepare_data = function(aData)
 					continue;
 
 				//TYPE filter
-				if (this._sFilterFolderType){
+				if (!Is.Empty(this._sFilterFolderType)){
 
 					if (!this._sFilterFolderType['R'] && oFolders[fid].TYPE == 'M' && oFolders[fid].RSS){
 						continue;
@@ -120,16 +124,17 @@ _me.__prepare_data = function(aData)
 				}
 
 				// TEAMCHAT filter, hide Chat folders in all filters except Y
-				if ((oFolders[fid].TYPE == 'Y' ||  oFolders[fid].TYPE == 'I' || oFolders[fid].PRIVATE_ROOT) && (!this._sFilterFolderType || !this._sFilterFolderType['Y']))
+				if ((oFolders[fid].TYPE == 'Y' ||  oFolders[fid].TYPE == 'I' || oFolders[fid].PRIVATE_ROOT) && (Is.Empty(this._sFilterFolderType) || !this._sFilterFolderType['Y']))
 				 	continue;
 
 				//RIGHTS filter (regExp.test() doesnt work properly in loop)
+				var bSkip, sRights, r;
 				if (this._sFilterRights){
-					var bSkip = false,
-						sRights = Cookie.get(['rights',sAccId,fid]).join('');
+					bSkip = false,
+					sRights = Cookie.get(['rights',sAccId,fid]).join('');
 
 					if (sRights.length){
-						for (var r in this._sFilterRights)
+						for (r in this._sFilterRights)
 							if (sRights.indexOf(this._sFilterRights[r])<0){
 								bSkip = true;
 								break;
@@ -141,10 +146,10 @@ _me.__prepare_data = function(aData)
 				}
 
 				if (this._sFilterRightsOr){
-					var bSkip = true,
-						sRights = Cookie.get(['rights',sAccId,fid]).join('');
+					bSkip = true;
+					sRights = Cookie.get(['rights',sAccId,fid]).join('');
 
-					for (var r in this._sFilterRightsOr)
+					for (r in this._sFilterRightsOr)
 						if (sRights.indexOf(this._sFilterRightsOr[r])>-1){
 							bSkip = false;
 							break;
@@ -217,12 +222,15 @@ _me.__prepare_data = function(aData)
 				if (this._filterData && this._filterData({aid:sAccId,fid:fid,data:oFolders[fid]}))
 					continue;
 
+				if ((this._opt.publicOnly && node_type !== 'public') || (this._opt.sharedOnly && node_type !== 'shared') || (this._opt.privateOnly && (node_type !== 'private' && node_type !== 'resources'))) {
+					continue;
+				}
 				nodes[node_type].push({aid:sAccId,fid:fid,data:oFolders[fid]});
 			}
 		}
 
 		for (var t in nodes){
-			if (nodes.hasOwnProperty(t)){
+			if (Object.prototype.hasOwnProperty.call(nodes ,t)){
 				//fill missing paths by X type folders
 				nodes[t] = this.__cleanup(this.__fillIn(nodes[t], sAccId));
 
@@ -237,36 +245,39 @@ _me.__prepare_data = function(aData)
 		//Construct ROOT node structure
 
 		if (nodes.private.length){
-			var private = {
+			var privat = {
 				nodes: this.__privateSort(nodes.private),
 				liclass: 'root_private',
 				ico: 'root_private',
 				group: nodes.private.group,
 				arg: {
 					disabled: !this._opt.privateRootActive || (this._sFilterFolderType && (this._sFilterFolderType['Y'] || this._sFilterFolderType['E'])),
-					aid:sAccId
+					aid:sAccId,
+					folderDroppable: this._opt.rootFolderDroppable
 				}
 			};
 
 			if (aAccounts[sAccId].PRIMARY){
-				private.text = dataSet.get('accounts',[sPrimaryAccount,'FULLNAME']) ||  dataSet.get('accounts',[sPrimaryAccount,'USERNAME']);
-				aResult[sAccId].nodes.push(private);
+				var name = dataSet.get('accounts',[sPrimaryAccount,'FULLNAME']) ||  dataSet.get('accounts',[sPrimaryAccount,'USERNAME']);
+				privat.text = obj_avatar.getAvatarHTML({ email: sPrimaryAccount, name: name, size: 20 }) + name;
+				aResult[sAccId].nodes.push(privat);
 			}
 			//Other Accounts
 			else{
-				aResult['other'].nodes[sAccId].nodes = private.nodes;
+				aResult['other'].nodes[sAccId].nodes = privat.nodes;
+				aResult['other'].nodes[sAccId].text = obj_avatar.getAvatarHTML({ email: sAccId, size: 20 }) + sAccId;
 			}
 		}
 
 		if (aAccounts[sAccId].PRIMARY){
 			if (nodes.public.length){
-				aResult[sAccId].nodes.push({nodes:nodes.public, title:'COMMON_FOLDERS::PUBLIC_ROOT', liclass:'root_public', ico:'root_public', arg:{disabled:true}});
+				aResult[sAccId].nodes.push({nodes:nodes.public, title:'COMMON_FOLDERS::PUBLIC_ROOT', liclass:'root_public skip', ico:'root_public', arg:{disabled:true, folderDroppable: this._opt.rootFolderDroppable}});
 			}
 			if (nodes.resources.length){
-				aResult[sAccId].nodes.push({nodes:nodes.resources, title:'COMMON_FOLDERS::RESOURCES', liclass:'root_resources', ico:'root_resources', arg:{disabled:true}});
+				aResult[sAccId].nodes.push({nodes:nodes.resources, title:'COMMON_FOLDERS::RESOURCES', liclass:'root_resources', ico:'root_resources', arg:{disabled:true, folderDroppable: this._opt.rootFolderDroppable, resources: true}});
 			}
 			if (nodes.shared.length){
-				aResult[sAccId].nodes.push({nodes:nodes.shared, title:'COMMON_FOLDERS::SHARED_ROOT', liclass:'root_shared', ico:'root_shared', arg:{disabled:true, aid: sAccId}});
+				aResult[sAccId].nodes.push({nodes:nodes.shared, title:'COMMON_FOLDERS::SHARED_ROOT', liclass:'root_shared skip', ico:'root_shared', arg:{disabled:true, aid: sAccId, folderDroppable: this._opt.rootFolderDroppable}});
 			}
 		}
 		else
@@ -285,19 +296,6 @@ _me.__prepare_data = function(aData)
 	//Map ID to folder paths
 	this.__mapTree(aResult);
 
-	//Always expand default calendar
-	if (aCalendars){
-		var treeID;
-		for (var i in aCalendars) {
-			treeID = this._getTreeId(sPrimaryAccount + '/' + i);
-			if (aCalendars[i][0] && treeID){
-				this.__value = this.__cleanup(this.__value.concat(treeID.split('/').map(function (v, i, a) {
-					return a.slice(0, i).concat(v).join('/');
-				})));
-			}
-		}
-	}
-
 	return aResult;
 };
 
@@ -307,7 +305,7 @@ _me.__prepare_data = function(aData)
  * @param {*} aData tree data
  * @param {string} sPath optional
  */
-_me.__mapTree = function(aData, sPath){
+_me.__mapTree = function(aData, sPath, bShared){
 	if (!sPath){
 		this.__idTableReal = {};
 		this.__idTableTree = {};
@@ -315,7 +313,7 @@ _me.__mapTree = function(aData, sPath){
 	}
 
 	for (var id in aData){
-		if (aData.hasOwnProperty(id)){
+		if (Object.prototype.hasOwnProperty.call(aData, id)){
 			//Folders
 			if (aData[id].arg.aid){
 				var tmp = aData[id].arg.aid + (Is.Defined(aData[id].arg.fid)?'/' + aData[id].arg.fid:'');
@@ -330,14 +328,14 @@ _me.__mapTree = function(aData, sPath){
 				this.__idTableReal[pathId] = [pathId];
 			}
 			if (aData[id].nodes){
-				this.__mapTree(aData[id].nodes, sPath + id + '/');
+				this.__mapTree(aData[id].nodes, sPath + id + '/', bShared);
 			}
 		}
 	}
 };
 
 _me._getTreeId = function(id, bArray){
-	return bArray?(this.__idTableReal[id] || []):(this.__idTableReal[id]?this.__idTableReal[id][0]:null);
+	return bArray?(this.__idTableReal[id] || []):(this.__idTableReal[id] || [])[0];
 };
 
 _me._getRealId = function(id){
@@ -350,29 +348,55 @@ _me._getRealId = function(id){
  * @returns {object}
  */
 _me._filter = function (aData){
-
 	var aData2 = {},
 		filter = this.__filter,
-		sName, aFolders, aKey, bFound, sLast, oOpen = {};
+		sName, aFolders, aKey, bFound, sLast, oOpen = {},
+		bShowSubfolders = this.__filter_show_subfolders === void 0 ? GWOthers.getItem('LAYOUT_SETTINGS', 'filter_show_subfolders') == 1 : this.__filter_show_subfolders;
 
 	for(var sDomain in aData){
+		if (bShowSubfolders && ~sDomain.indexOf(filter)) {
+			aData2[sDomain] = aData[sDomain];
+			continue;
+		}
+
 		sLast = '';
 
 		aFolders = aData[sDomain];
+		var aKeyExact = [];
 		aKey = Object.keys(aFolders).sort().reverse().filter(function(v){
+			if (~filter.indexOf('/')) {
+				var paths = [];
+				var parts = v.split('/');
+				for (var i = 1; i <= parts.length; i++) {
+					var path = parts.slice(0, i).join('/');
+					paths.push((aFolders[path] || {}).NAME || Path.basename(path));
+				}
+				sName = paths.join('/').toLowerCase();
+			} else {
+				sName = (aFolders[v].NAME || Path.basename(v)).toLowerCase();
+			}
 
-			sName = (aFolders[v].NAME || Path.basename(v)).toLowerCase();
 			bFound = ~sName.indexOf(filter);
 
+			if (bFound) {
+				aKeyExact.push(v);
+			}
 			if (v == sLast || bFound){
 				sLast = Path.basedir(v);
 				oOpen[sDomain + (sLast?'/' + sLast:'')] = true;
-				return !!bFound;
+				return true;
 			}
 		});
 
 		if (aKey.length){
 			aData2[sDomain] = {};
+			if (bShowSubfolders) {
+				aKey = aKey.concat(Object.keys(aFolders).filter(function(v) {
+					return aKeyExact.some(function(k) {
+						return v.indexOf(k) === 0;
+					});
+				}));
+			}
 			aKey.reverse().forEach(function(v){
 				aData2[sDomain][v] = aFolders[v];
 			});
@@ -395,19 +419,35 @@ _me.__fillIn = function(arr, sAccId){
 
 	var aFolders = dataSet.get('folders', [sAccId]);
 
-	//Quick sort by path
-	if (Intl && 'Collator' in Intl){
-		var coll = new Intl.Collator();
-		arr.sort(function(a,b){
-			return coll.compare(a.fid, b.fid);
-		});
-	}
-	else {
-		//Quick sort by path
-		arr.sort(function(a,b){
-			return a.fid.localeCompare(b.fid);
-		});
-	}
+	var coll = (Intl && 'Collator' in Intl) ? new Intl.Collator(void 0, {
+		numeric: true,
+		sensitivity: 'base'
+	}) : {
+		compare: function(a, b){
+			return a.localeCompare(b);
+		}
+	};
+
+	arr.sort(function(a,b){
+		/* archive reverse sort */
+		var base;
+		if (a.data.ARCHIVE === void 0) {
+			base = aFolders[a.fid.split('/')[0]];
+			a.data.ARCHIVE = base && !!base.ARCHIVE;
+		}
+		if (b.data.ARCHIVE === void 0) {
+			base = aFolders[b.fid.split('/')[0]];
+			b.data.ARCHIVE = base && !!base.ARCHIVE;
+		}
+		if (a.data.ARCHIVE && b.data.ARCHIVE && a.fid.match(/\d/) && b.fid.match(/\d/) && a.fid.split('/').slice(0, -1).join('/') === b.fid.split('/').slice(0, -1).join('/')) {
+			return -coll.compare(a.fid, b.fid);
+		}
+		/* archive reverse sort */
+
+		var aName = WMFolders.getFolderPath(a);
+		var bName = WMFolders.getFolderPath(b);
+		return coll.compare(aName, bName);
+	});
 
 	//fill missing paths
 	for(var i = 0, j = arr.length; i<j; i++){
@@ -493,23 +533,23 @@ _me.__toTree = function(arr, sNodeType, aCalendars){
 		tmp = {},
 		group = 0;
 
-	arr.forEach(function(f, i){
+	arr.forEach(function(f){
 		var path = f.fid.split('/');
 
 
 		var	item = {
-				arg: {
-					aid: f.aid,
-					fid: f.fid,
-					data: f.data,
-					ftype: f.data.TYPE || 'X'
-				},
-				text: f.data.NAME || Path.basename(f.fid),
-				ico: this.__getIcon(f)
-			};
+			arg: {
+				aid: f.aid,
+				fid: f.fid,
+				data: f.data,
+				ftype: f.data.TYPE || 'X'
+			},
+			text: f.data.NAME || Path.basename(f.fid),
+			ico: this.__getIcon(f)
+		};
 
 		if (sNodeType == 'shared' && ~item.text.indexOf('@')){
-			item.text =  '<img src="'+ getAvatarURL(item.text) +'" class="avatar"/>' + item.text;
+			item.text =  obj_avatar.getAvatarHTML({ email: item.text, name: item.text, size: 20 }) + item.text;
 		}
 
 		//do not draw RESOURCE root in Calendar view
@@ -534,7 +574,10 @@ _me.__toTree = function(arr, sNodeType, aCalendars){
 				if (aCalendars[f.fid][0]){
 					item.liclass = 'active';
 				}
-				item.ico += ' checked' + (aCalendars[f.fid][1]?' '+aCalendars[f.fid][1]:'');
+				item.ico += ' checked';
+				if (aCalendars[f.fid][1]) {
+					item.style = '--color: ' + (calendarPalette[aCalendars[f.fid][1]] || aCalendars[f.fid][1]);
+				}
 			}
 		}
 		else
@@ -548,48 +591,42 @@ _me.__toTree = function(arr, sNodeType, aCalendars){
 		}
 
 		//Write right
-		if (!WMFolders.getRights([f.aid,f.fid],'write')){
+		if (!WMFolders.getRights([f.aid,f.fid],'write') && !WMFolders.getRights([f.aid,f.fid],'owner')){
 			item.ico += ' no-write';
 		}
 
 		//to the tree...
+		var parentPath = path.slice(0,-1).join('/');
+
 		if (path.length === 1){
 			aOut.push(item);
+		} else if (tmp[parentPath]){
+			if (!tmp[parentPath].nodes)
+				tmp[parentPath].nodes = [];
+
+			tmp[parentPath].nodes.push(item);
+
+			//Teamchat, Primary
+			if (sNodeType == 'private'){
+				if (f.data.TYPE == 'Y')
+					tmp[parentPath].liclass = 'skip';
+			}
 		}
-		else{
-			var parentPath = path.slice(0,-1).join('/');
-			if (tmp[parentPath]){
-
-				if (!tmp[parentPath].nodes)
-					tmp[parentPath].nodes = [];
-
-				tmp[parentPath].nodes.push(item);
-
-				//Teamchat, Primary
-				if (sNodeType == 'private'){
-					if (f.data.TYPE == 'Y')
-						tmp[parentPath].liclass = 'skip';
-				}
-
-				//Teamchat, group unread counts
-				if (f.data.TYPE == 'I' && f.data.RECENT>'0'){
-					if (sNodeType == 'private'){
-						group = group + (+f.data.RECENT);
-						if(group > 999) {
-							group = '999+';
-						}
-					}
-					else{
-						parentPath = path.slice(0,-2).join('/');
-						if (parentPath) {
-							tmp[parentPath].group = +(tmp[parentPath].group || 0) + (+f.data.RECENT);
-							if(tmp[parentPath].group > 999) {
-								tmp[parentPath].group = '999+';
-							}
-						}
-					}
-				}
 
+		if (f.data.RECENT>'0'){
+			if (sNodeType == 'private'){
+				group = group + (+f.data.RECENT);
+				// if(group > 999) {
+				// 	group = '999+';
+				// }
+			}
+			if (tmp[parentPath]) {
+				do {
+					tmp[parentPath].group = +(tmp[parentPath].group || 0) + (+f.data.RECENT);
+					// if(tmp[parentPath].group > 999) {
+					// 	tmp[parentPath].group = '999+';
+					// }
+				} while (parentPath = parentPath.split('/').slice(0,-1).join('/'))
 			}
 		}
 
@@ -641,6 +678,15 @@ _me.__getIcon = function(folder){
 			case 'QL':
 				ico = folder.fid === 'SPAM_QUEUE/Blacklist'?'black':'white';
 				break;
+			case 'F':
+				ico = 'f';
+				if (folder.fid === Path.split(GWOthers.getItem('DEFAULT_FOLDERS', 'SMARTATTACH'))[1]) {
+					ico = 'upload';
+				}
+				if (folder.fid === Path.split(GWOthers.getItem('DEFAULT_FOLDERS', 'DASHBOARD'))[1]) {
+					ico = 'dashboard';
+				}
+				break;
 			default:
 				if (folder.fid == 'SPAM_QUEUE')
 					ico = 'spamqueue';
@@ -675,7 +721,7 @@ _me.__privateSort = function(aFolders){
 		aDefaultMail = [],
 		aDefaultGW = [];
 
-	var aFilter = aFolders.filter(function(folder, i){
+	var aFilter = aFolders.filter(function(folder){
 		if (folder.arg.ftype != 'X'){
 			if (folder.arg.fid == 'INBOX'){
 				aInbox.push(folder);
@@ -743,7 +789,7 @@ _me._filter_folder = function (filterFolderType){
 	}
 	else{
 		for(var i in filterFolderType){
-			if (filterFolderType.hasOwnProperty(i)){
+			if (Object.prototype.hasOwnProperty.call(filterFolderType, i)){
 				this._sFilterFolderType[filterFolderType[i]] = true;
 			}
 		}
@@ -768,7 +814,7 @@ _me._filter_rights_or = function (sFilterRightsOr){
 	this._sFilterRightsOr = (sFilterRightsOr || '').split('');
 };
 
-_me._rename = function(arg){
+_me._rename = async function(arg){
 	if (this.rename) this.rename._onclose();
 	this._norefresh = true;
 
@@ -789,35 +835,35 @@ _me._rename = function(arg){
 		sFolderName = dataSet.get("folders", [arg['aid'],arg['fid'], 'NAME']) || arg.fid.substring(nPos + 1);
 
 	// create obj_input in this' editDiv anchor
-	var rename = this._create('rename','obj_input','editDiv');
-		rename._restrict('![/\\\\:\?\"\<\>\|\~]+','','^.{1,255}$');
+	var rename = await this._create('rename','obj_input','editDiv');
+		rename._restrict('![/\\\\:?"<>|~]+','','^.{1,255}$');
 		rename.__eIN.setAttribute('maxlength', 255);
 
 		// cancel "onclick" propagation to obj_tree
 		rename.__eIN.onclick = function(e){
-			var e = e || window.event;
-			if (e.preventDefault) e.preventDefault();
-			if (e.stopPropagation) e.stopPropagation();
-			e.cancelBubble = true;
-			return false;
+			e.preventDefault();
+			e.stopPropagation();
 		};
 
 		rename._value(sFolderName);
 		rename._setRange(0,sFolderName.length);
 
-		rename._onsubmit = function(e){
+		rename._onsubmit = function(){
 			if (this._checkError && this._checkError.length) return;
 
 			var snew = (this._value() || '').trim();
-			var sold = sFolderName;
 
-			if (sold == snew){
+			if (sFolderName == snew){
 				this._onclose();
 				return;
 			}
 
 			var sNewFolderID = (sParentName ? sParentName + '/' : '') + snew;
-			obj_context_folder.__moveFolder(arg.aid, arg.fid, sNewFolderID);
+			var fid = dataSet.get('folders', [arg.aid, arg.fid, 'RELATIVE_PATH']) || arg.fid;
+			if (WMFolders.getType([arg.aid, arg.fid]) === 'I' || dataSet.get('folders', [arg.aid, arg.fid, 'OWNER']) !== sPrimaryAccount) {
+				fid = arg.fid;
+			}
+			obj_context_folder.__moveFolder(arg.aid, fid, sNewFolderID);
 
 			this._onclose();
 		};
@@ -835,11 +881,8 @@ _me._rename = function(arg){
 				this.__update(this._listener_data);
 		}.bind(this);
 
-		//MSIE11, prevent search input focus
 		rename._onkeydown = function(e){
-			var e = e || window.event;
-			if (e.stopPropagation) e.stopPropagation();
-			e.cancelBubble = true;
+			e.stopPropagation();
 		};
 
 		rename._onblur = rename._onclose;
