@@ -1,25 +1,28 @@
 _me = obj_tree_folder_context.prototype;
 function obj_tree_folder_context(){};
 
-_me.__constructor = function(){
+_me.__constructor = async function(){
 	//this._autofocus = false;
-	this._listen_cookie('tree');
+	this._listen_cookie(this._opt.listenCookie || 'tree');
 
 	if (sPrimaryAccountGUEST)
 		addcss(this._main,'guest');
 };
 obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
-	var aMenu = [];
+	var aMenu = [], i;
 	if (arg){
 		if (arg['fid']){
 
+			var aRights = WMFolders.getRights(arg);
 			var sFolType = WMFolders.getType(arg);
+			var bIsFavorite = (dataSet.get('cookies', ['favorites']) || []).some(function(fav) {
+				return fav.arg.aid === arg.aid && fav.arg.fid === arg.fid;
+			}.bind(this));
 
 			if (sFolType == 'Y'){
 
-				var aRights = WMFolders.getRights(arg),
-					folder_write = ~(dataSet.get('folders', [arg.aid, arg.fid, 'RIGHTS']) || '').indexOf('k'),
-					bPrivate = dataSet.get('folders',[arg.aid, Path.basedir(arg.fid),'PRIVATE_ROOT']) === "true";
+				var folder_write = ~(dataSet.get('folders', [arg.aid, arg.fid, 'RIGHTS']) || '').indexOf('k'),
+					bPrivate = dataSet.get('folders',[arg.aid, Path.basedir(arg.fid) || arg.fid,'PRIVATE_ROOT']) === "true";
 
 				aMenu.push(
 					{'title':'POPUP_FOLDERS::ADD_ROOM','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'add_room'}, css:'ico2 add_folder', 'disabled':!(folder_write || bPrivate)}
@@ -27,7 +30,7 @@ obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
 
 				if (!bPrivate && aRights.owner)
 					aMenu.push(
-						{'title':'POPUP_FOLDERS::MEMBERS','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'share'}, css:'color1 ico2 members'}
+						{'title':'POPUP_FOLDERS::MEMBERS','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'manage_members'}, css:'color1 ico2 members'}
 					);
 
 				if (~(dataSet.get('folders', [arg.aid, Path.basedir(arg.fid), 'RIGHTS']) || '').indexOf('a'))
@@ -38,21 +41,27 @@ obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
 			}
 			else
 			if (sFolType == 'I'){
-				var aRights = WMFolders.getRights(arg);
+				if (!bMoreMenu) {
+					if (bIsFavorite) {
+						aMenu.push({title:'POPUP_FOLDERS::REMOVE_FAVORITES', arg: {aid: arg.aid, fid: arg.fid, method: 'remove_favorite'}, css: 'ico2 fav_folder'});
+					} else {
+						aMenu.push({title:'POPUP_FOLDERS::ADD_FAVORITES', arg: {aid: arg.aid, fid: arg.fid, method: 'add_favorite'}, css: 'ico2 fav_folder'});
+					}
+				}
 
 				aMenu.push(
 					{'title':'POPUP_FOLDERS::RENAME_ROOM','arg':{'aid':arg['aid'],'fid':arg['fid'],'ftype':arg['ftype'],'method':'rename_folder'}, css:'ico2 edit_folder', 'disabled':!aRights.edit_folder},
 					{'title':'FORM_BUTTONS::UNSUBSCRIBE', css:'ico2 archive_folder','arg':{'aid':arg['aid'],'fid':arg['fid'],'ftype':arg['ftype'], method:'unsubscribe'
 					// function(folder_info){
 					// 	//'method':'unsubscribe',
-					// 	var elm = document.getElementById(gui.frm_main.bar.tree.folders._getElmId(arg.aid+'/'+arg.fid));
+					// 	var elm = document.getElementById(gui.frm_main.bar.folders._getElmId(arg.aid+'/'+arg.fid));
 					// 	if (elm)
 					// 		gui.frm_main.bar.top.switch.__unsubscribeRoom(elm, arg, folder_info);
 					// }.bind(this, arg)
 					}}
 				);
 
-				aRights.owner && aMenu.push({'title':'POPUP_FOLDERS::MEMBERS','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'share'}, css:'color1 ico2 members'});
+				aRights.owner && aMenu.push({'title':'POPUP_FOLDERS::MEMBERS','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'manage_members'}, css:'color1 ico2 members'});
 
 				!sPrimaryAccountGUEST && aMenu.push(
 					{'title':'POPUP_FOLDERS::CLONE_ROOM','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'clone'}, css:'ico2 clone_room', 'disabled':!WMFolders.getRights([arg.aid, Path.basedir(arg.fid)]).write},
@@ -63,6 +72,8 @@ obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
 					{'title': '-'},
 					{'title':'POPUP_FOLDERS::DELETE_ROOM','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'delete_folder'}, css:'ico2 color2 delete_folder','disabled':!aRights.remove}
 				);
+			} else if (this._sFilterFolderType && this._sFilterFolderType['Y']) {
+				aMenu.push({title: 'POPUP_FOLDERS::ADD_ROOM', arg: {aid: arg.aid, fid: arg.fid, method: 'add_room'}, css: 'ico2 add_folder', disabled: !frm_addroom.prototype._getGroups().length});
 			}
 			else
 			if (sFolType == 'G'){
@@ -75,29 +86,37 @@ obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
 				var bVirtual = arg['fid'].indexOf('__@@VIRTUAL@@__/') == 0,
 					bArchive = dataSet.get('folders',[arg['aid'],arg['fid'].split('/')[0],'ARCHIVE'])?true:false,
 					//get folder access rights
-					aRights = WMFolders.getRights(arg),
 					aAccess = WMFolders.getAccess(arg);
 
 				if (sFolType == 'M') {
-					var title;
+					var snoozed = arg.fid === '__@@SNOOZED@@__';
+					var title,
+						aFolder = dataSet.get('folders', [arg.aid, arg.fid]) || {},
+						empty_folder_enabled = (aFolder.DEFAULT === 'H' || aFolder.SPAM === 'true' || GWOthers.getItem('RESTRICTIONS', 'DISABLE_EMPTY_FOLDER') != 1) && !snoozed;
 
-					if (gui.frm_main.main && gui.frm_main.main.list && gui.frm_main.main.list._SQLsearch) {
+					var filter = gui.frm_main.main && gui.frm_main.main.list && (gui.frm_main.main.list._SQLsearch || '').replace(/\+folders:".*?"/, '').trim();
+					if ((dataSet.get('current_folder') === (arg.aid + '/' + arg.fid)) && filter) {
 						title = 'POPUP_FOLDERS::ITEM_ACTIONS_FILTERED';
 					} else {
 						title = 'POPUP_FOLDERS::ITEM_ACTIONS';
 					}
 					// Manage content items
-					aMenu.push(
-						{'title':title,css:'ico2 manage',nodes:[
-							{'title':'POPUP_FOLDERS::MARK_ALL_AS_READ','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'mark_all_as_read'}, css:'ico2 markread', 'disabled':!aAccess.modify},
-							{'title':'-'},
-							{'title':'POPUP_ITEMS::COPY_MAIL_TO','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'copyall'}, css:'ico2 copy', 'disabled':bVirtual},
-							{'title':'POPUP_ITEMS::MOVE_MAIL_TO','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'moveall'}, css:'ico2 move', 'disabled':!aAccess.remove || bVirtual},
-							{'title':'-'},
-							{'title':'MAIN_MENU::EMPTY_FOLDER','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'empty_folder'}, 'disabled':!aAccess.remove, 'css':'ico2 color2 empty_folder'}
-						]},
-						{'title':'-'}
-					);
+					if (!bVirtual) {
+						aMenu.push(
+							{'title':title,css:'ico2 manage',nodes:[
+								{'title':'POPUP_FOLDERS::MARK_ALL_AS_READ','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'mark_all_as_read'}, css:'ico2 markread', 'disabled':!aAccess.modify},
+								{'title':'-'},
+								{'title':'POPUP_ITEMS::COPY_MAIL_TO','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'copyall'}, css:'ico2 copy', 'disabled':bVirtual},
+								{'title':'POPUP_ITEMS::MOVE_MAIL_TO','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'moveall'}, css:'ico2 move', 'disabled':!aAccess.remove || bVirtual},
+								empty_folder_enabled && {'title':'-'},
+								empty_folder_enabled && {'title': GWOthers.getItem('DEFAULT_FOLDERS','trash') === arg.aid + '/' + arg.fid ? 'MAIN_MENU::EMPTY_TRASH' : 'MAIN_MENU::EMPTY_FOLDER','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'empty_folder'}, 'disabled':!aAccess.remove, 'css':'ico2 color2 empty_folder'}
+							].filter(Boolean)},
+						);
+					}
+
+					if (!snoozed) {
+						aMenu.push({title:'-'});
+					}
 
 					if(bMoreMenu)
 						aMenu.push({title: 'MAIN_MENU::NEW_MESSAGE', css: 'ico2 new_template', arg: {method: 'new_template'}});
@@ -105,12 +124,15 @@ obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
 
 				if (sFolType == 'VA'){
 					if (dataSet.get('folders',[arg['aid'],arg['fid'],'SUBSCRIPTION_TYPE']) == 'account')
-						aMenu.push(
-							{'title':'POPUP_FOLDERS::ADD_FOLDER','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'add_folder'}, css:'ico2 add_folder', 'disabled':!aRights.write},
-							{'title':'POPUP_FOLDERS::SHARING','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'share'}, css:'ico2 color1 share_folder', 'disabled':!aRights.owner},
-							{'title':'-'},
-							{'title':'POPUP_FOLDERS::REMOVE_ACCOUNT','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'delete_folder'}, css:'ico2 color2 delete_folder', 'disabled':!aRights.remove}
-						);
+						if (dataSet.get('folders',[arg['aid'],arg['fid'],'TYPE']) !== 'X') {
+							aMenu.push({'title':'POPUP_FOLDERS::ADD_FOLDER','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'add_folder'}, css:'ico2 add_folder', 'disabled':!aRights.write});
+							if (GWOthers.getItem('RESTRICTIONS', 'disable_folder_sharing') == 0) {
+								aMenu.push({'title':'POPUP_FOLDERS::SHARING','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'share'}, css:'ico2 color1 share_folder', 'disabled':!aRights.owner});
+							}
+							this.__selectAllCalendarsMenu(aMenu, arg);
+							aMenu.push({'title':'-'});
+						}
+						aMenu.push({title: 'POPUP_FOLDERS::REMOVE_ACCOUNT', arg: {aid: arg.aid, fid: arg.fid, method: 'delete_folder'}, css: 'ico2 color2 delete_folder', disabled: !aRights.remove});
 				}
 				else
 				if (sFolType == 'X' && arg['fid']=='__@@VIRTUAL@@__'){
@@ -123,35 +145,45 @@ obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
 					if (sFolType == 'E' && dataSet.get('active_folder') == sPrimaryAccount+'/__@@VIRTUAL@@__/__@@EVENTS@@__'){
 						aMenu.push(
 							{'title':'POPUP_FOLDERS::COLOR', nodes:
-								[
-									{'title':'COLORS::BLUE','arg':{'fid':arg['fid'], color:'blue', 'method':'cal_color'}, css:'ico2 color blue'},
-									{'title':'COLORS::GREEN','arg':{'fid':arg['fid'], color:'green', 'method':'cal_color'}, css:'ico2 color green'},
-									{'title':'COLORS::BROWN','arg':{'fid':arg['fid'], color:'brown', 'method':'cal_color'}, css:'ico2 color brown'},
-									{'title':'COLORS::PURPLE','arg':{'fid':arg['fid'], color:'purple', 'method':'cal_color'}, css:'ico2 color purple'},
-									{'title':'COLORS::YELLOW','arg':{'fid':arg['fid'], color:'yellow', 'method':'cal_color'}, css:'ico2 color yellow'},
-									{'title':'COLORS::TEAL','arg':{'fid':arg['fid'], color:'teal', 'method':'cal_color'}, css:'ico2 color teal'},
-									{'title':'COLORS::PLUM','arg':{'fid':arg['fid'], color:'plum', 'method':'cal_color'}, css:'ico2 color plum'},
-									{'title':'COLORS::RED','arg':{'fid':arg['fid'], color:'red', 'method':'cal_color'}, css:'ico2 color red'},
-
-									{'title':'COLORS::AQUA','arg':{'fid':arg['fid'], color:'aqua', 'method':'cal_color'}, css:'ico2 color aqua'},
-									{'title':'COLORS::AQUAMARINE','arg':{'fid':arg['fid'], color:'aquamarine', 'method':'cal_color'}, css:'ico2 color aquamarine'},
-									{'title':'COLORS::ORCHID','arg':{'fid':arg['fid'], color:'orchid', 'method':'cal_color'}, css:'ico2 color orchid'},
-									{'title':'COLORS::PINK','arg':{'fid':arg['fid'], color:'pink', 'method':'cal_color'}, css:'ico2 color pink'},
-									{'title':'COLORS::SILVER','arg':{'fid':arg['fid'], color:'silver', 'method':'cal_color'}, css:'ico2 color silver'},
-									{'title':'COLORS::WHITE','arg':{'fid':arg['fid'], color:'white', 'method':'cal_color'}, css:'ico2 color white'},
-									{'title':'COLORS::GRAY','arg':{'fid':arg['fid'], color:'gray', 'method':'cal_color'}, css:'ico2 color gray'},
-									{'title':'COLORS::LEMON','arg':{'fid':arg['fid'], color:'lemon', 'method':'cal_color'}, css:'ico2 color lemon'}
-								],
-								css: 'ico2 color '+ getCalendarColor(arg['fid'])
-							},
-							{'title':'-'}
+								Object.keys(window.calendarPalette).map(function(color) {
+									return {
+										title: 'COLORS::' + color,
+										arg: {
+											fid: arg.fid,
+											color: window.calendarPalette[color] || color,
+											method: 'cal_color'
+										},
+										css: 'ico2 color',
+										style: '--color: ' + (window.calendarPalette[color] || color)
+									};
+								}).concat({
+									title: 'TAGS::CUSTOM_COLOR',
+									arg: {
+										fid: arg.fid,
+										currentColor: window.calendarPalette[getCalendarColor(arg.fid)] || getCalendarColor(arg.fid),
+										method: 'cal_color'
+									},
+									css: 'ico2 colorpicker'
+								}),
+								css: 'ico2 color',
+								style: '--color: ' + (window.calendarPalette[getCalendarColor(arg.fid)] || getCalendarColor(arg.fid))
+							}
 						);
-					}
 
+						var aFolders = dataSet.get('folders', [sPrimaryAccount]);
+						for (i in aFolders) {
+							if (!i.indexOf(arg.fid + '/')) {
+								this.__selectAllCalendarsMenu(aMenu, arg);
+								break;
+							}
+						}
+
+						aMenu.push({title: '-'});
+					}
 
 					if (bVirtual)
 						aMenu.push(	{'title':'POPUP_FOLDERS::EDIT_VIRTUAL', css:'ico2 add_vfolder', 'arg':{'fid':arg['fid'], 'method':'add_virtual'}});
-					else{
+					else if (!snoozed) {
 						aMenu.push(	{'title':'POPUP_FOLDERS::ADD_FOLDER','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'add_folder'}, css:'ico2 add_folder', 'disabled':!aRights.write});
 
 						if (arg['aid']==sPrimaryAccount && GWOthers.getItem('RESTRICTIONS','disable_virtual') != '1' && sFolType!='X'){
@@ -170,12 +202,32 @@ obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
 					if(sFolType === 'E') {
 						rename_title = 'POPUP_FOLDERS::RENAME_CALENDAR';
 					}
-					aMenu.push({'title':rename_title,'arg':{'aid':arg['aid'],'fid':arg['fid'],'ftype':arg['ftype'],'method':'rename_folder'}, css:'ico2 edit_folder', 'disabled':bArchive || !aRights.modify || sFolType == 'A' || sFolType == 'X'/* || arg['fid'].indexOf(sPrimaryAccountSPREFIX)==0*/});
-					if (!bVirtual)
+					if (arg.fid !== Mapping.getDefaultFolderForGWType('D') && !snoozed) {
+						aMenu.push({'title':rename_title,'arg':{'aid':arg['aid'],'fid':arg['fid'],'ftype':arg['ftype'],'method':'rename_folder'}, 
+						css:'ico2 edit_folder', 'disabled':bArchive || !aRights.modify || sFolType == 'A' || sFolType == 'X' || (arg.aid === sPrimaryAccount && Mapping.getDefaultFolderForGWType('C') === arg.fid)});
+					}
+					if (!bVirtual && !snoozed)
 						aMenu.push({'title':'POPUP_FOLDERS::MOVE_FOLDER','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'move_folder'}, css:'ico2 move_folder', 'disabled':bArchive || !aRights.modify || sFolType == 'A' || sFolType == 'X'});
 
-					if (GWOthers.getItem('LAYOUT_SETTINGS','favorites')>0 && !bMoreMenu)
-						aMenu.push({'title':'POPUP_FOLDERS::ADD_FAVORITES','arg':{aid:arg['aid'], fid:arg['fid'], 'method':'add_favorite'}, css:'ico2 fav_folder'});
+					if (sFolType === 'F' && gui.frm_main._dashboardLoaded() && arg.fid !== Mapping.getDefaultFolderForGWType('D')) {
+						if (gui.frm_main.dashboard._isPinnedItem({
+							aid: arg.aid,
+							fid: arg.fid,
+							type: 'folder'
+						})) {
+							aMenu.push({title: 'POPUP_ITEMS::UNPIN_FROM_DASHBOARD', arg: { aid: arg.aid, fid: arg.fid, method: 'unpin_folder' }, css: 'ico2 dashboard_pin'});
+						} else {
+							aMenu.push({title: 'POPUP_ITEMS::PIN_TO_DASHBOARD', arg: { aid: arg.aid, fid: arg.fid, method: 'pin_folder' }, css: 'ico2 dashboard_pin'});
+						}
+					}
+
+					if (!bMoreMenu && sFolType !== 'X' && !snoozed) {
+						if (bIsFavorite) {
+							aMenu.push({'title':'POPUP_FOLDERS::REMOVE_FAVORITES','arg':{aid:arg['aid'], fid:arg['fid'], 'method':'remove_favorite'}, css:'ico2 fav_folder'});
+						} else {
+							aMenu.push({'title':'POPUP_FOLDERS::ADD_FAVORITES','arg':{aid:arg['aid'], fid:arg['fid'], 'method':'add_favorite'}, css:'ico2 fav_folder'});
+						}
+					}
 
 					var aOtherMenu = {'title':'COMMON::MORE_OPTIONS', css:'ico2 more', nodes:[]};
 
@@ -183,11 +235,10 @@ obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
 						aMenu.push(	{'title':'POPUP_FOLDERS::CHANGE_CHANNEL','arg':{'aid':arg['aid'],'fid':arg['fid'],'ftype':arg['ftype'],'method':'change_channel'}, css:'ico2 manage_rss', 'disabled':!aRights.modify});
 					else
 					if (arg['aid']==sPrimaryAccount && !bVirtual){
-
 						//Default folders
-						if (arg['fid'].indexOf(sPrimaryAccountSPREFIX) < 0)
+						// if (arg['fid'].indexOf(sPrimaryAccountSPREFIX) < 0)
 							if (sFolType == 'M'){
-								if (!bArchive  && arg['fid'].indexOf('/')<0 && arg['fid']!='INBOX' && !dataSet.get('folders',[arg['aid'],arg['fid'],'SPAM']) && !dataSet.get('folders',[arg['aid'],arg['fid'],'PUBLIC'])){
+								if (!bArchive  && arg['fid'].replace(/^~\w*?@\w*?.\w*?\//, '').indexOf('/')<0 && arg['fid']!='INBOX' && !dataSet.get('folders',[arg['aid'],arg['fid'],'SPAM']) && !dataSet.get('folders',[arg['aid'],arg['fid'],'PUBLIC'])){
 									aOtherMenu.nodes.push(
 										{'title':'POPUP_FOLDERS::DEFAULT', css:'ico2 default_folder', nodes:[
 											{'title':'COMMON_FOLDERS::DRAFTS','arg':{'aid':arg['aid'],'fid':arg['fid'],'default':'D','method':'default'}, css:'draft_folder',disabled:GWOthers.getItem('DEFAULT_FOLDERS','drafts') == arg['aid']+'/'+arg['fid']},
@@ -199,7 +250,7 @@ obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
 								}
 							}
 							else
-							if (sFolType != 'A' && sFolType != 'X'){
+							if (sFolType != 'A' && sFolType != 'X' && arg.fid !== Mapping.getDefaultFolderForGWType('D') && arg.fid !== Path.split(GWOthers.getItem('DEFAULT_FOLDERS', 'SMARTATTACH'))[1]){
 								if (arg['fid'].indexOf('/')<0)
 									aOtherMenu.nodes.push({'title':'POPUP_FOLDERS::DEFAULT', css:'ico2 default_folder','arg':{'aid':arg['aid'],'fid':arg['fid'],'default':sFolType,'method':'default'},disabled:GWOthers.getItem('DEFAULT_FOLDERS', {C:'CONTACTS',E:'EVENTS',T:'TASKS',N:'NOTES',J:'JOURNAL',F:'FILES'}[sFolType]) == arg['aid']+'/'+arg['fid']});
 							}
@@ -207,48 +258,67 @@ obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
 					}
 
 					if (sFolType != 'A' && sFolType != 'X') {
-						aOtherMenu.nodes.push({'title': 'POPUP_FOLDERS::EXPORT', 'arg': {'aid': arg['aid'], 'fid': arg['fid'], 'method': 'save_folder'}, css:'ico2 export_folder', disabled: arg['aid'] != sPrimaryAccount});
+						if (GWOthers.getItem('RESTRICTIONS', 'disable_folder_export') == 0) {
+							aOtherMenu.nodes.push({'title': 'POPUP_FOLDERS::EXPORT', 'arg': {'aid': arg['aid'], 'fid': arg['fid'], 'method': 'save_folder'}, css:'ico2 export_folder', disabled: arg['aid'] != sPrimaryAccount});
+						}
 
 						if (sFolType != 'M' && dataSet.get('folders',[sPrimaryAccount,'__@@GWTRASH@@__']))
-							aOtherMenu.nodes.push({'title':'POPUP_FOLDERS::RECOVERY','arg':{'aid':sPrimaryAccount,'fid':'__@@GWTRASH@@__','search':'folder:"'+ Path.backslash(arg['fid']).replace(/\"/g,'\\\"') +'"','method':'view'}, css:'ico2 rec_folder', disabled:arg['aid'] != sPrimaryAccount});
+							aOtherMenu.nodes.push({'title':'POPUP_FOLDERS::RECOVERY','arg':{'aid':sPrimaryAccount,'fid':'__@@GWTRASH@@__','search':'folder:"'+ Path.backslash(arg['fid']).replace(/"/g,'\\"') +'"','method':'view'}, css:'ico2 rec_folder', disabled:arg['aid'] != sPrimaryAccount});
 					}
 
-					if (sFolType == 'M' && arg['fid']!='INBOX' && !bArchive && !bVirtual){
+					if (sFolType == 'M' && arg['fid']!='INBOX' && !bArchive){
 						if (dataSet.get("folders",[arg['aid'],arg['fid'],'SYNC']))
 							aOtherMenu.nodes.push({'title':'POPUP_FOLDERS::SUBSCRIBED','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'unsubscribe'}, css:'ico2 check'});
 						else
 							aOtherMenu.nodes.push({'title':'POPUP_FOLDERS::SUBSCRIBE','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'subscribe'}, css:'ico2 sync_folder'});
 					}
 
-					if (aOtherMenu.nodes.length)
+					if (aOtherMenu.nodes.length && !snoozed)
 						aMenu.push(aOtherMenu);
 
-					if (!bVirtual)
-						aMenu.push({'title':'POPUP_FOLDERS::SHARING','arg':{'aid':arg['aid'],'fid':arg['fid'], 'method':'share'}, css:'ico2 color1 share_folder', disabled:(sFolType == 'M' && sPrimaryAccountProtocol!='imap') || !aRights.owner || bArchive || !sPrimaryAccountSHARING});
-					else
-						aMenu.push({'title':'-'});
+					if (!snoozed) {
+						if (!bVirtual && (GWOthers.getItem('RESTRICTIONS', 'disable_folder_sharing') == 0)) {
+							var title = 'POPUP_FOLDERS::SHARING';
+							if (sFolType === 'E') {
+								title = 'POPUP_FOLDERS::SHARING_CALENDAR';
+							}
+							aMenu.push({'title':title,'arg':{'aid':arg['aid'],'fid':arg['fid'], 'method':'share'}, css:'ico2 color1 share_folder', disabled:(sFolType == 'M' && sPrimaryAccountProtocol!='imap') || !aRights.owner || bArchive || !sPrimaryAccountSHARING});
+						} else
+							aMenu.push({'title':'-'});
+					}
 
 					//check for default (sub)folders
-					var bProtected = false;
+					var bProtected = arg.fid === Mapping.getDefaultFolderForGWType('D');
 					if (aRights.remove){
 						var aSubFol = dataSet.get('folders',[arg['aid']]);
-						for (var i in aSubFol)
+						for (i in aSubFol)
 							if ((arg['fid'] == i || i.indexOf(arg['fid']+'/')==0) && aSubFol[i].DEFAULT){
 								bProtected = true;
 								break;
 							}
-					}
-					else
+
+						if (arg.aid === sPrimaryAccount && !~arg.fid.indexOf('/') && (aSubFol[arg.fid] || {}).NAME === 'Upload') {
+							bProtected = true;
+						}
+					} else {
 						bProtected = true;
+					}
 
 					if (arg['aid'] == sPrimaryAccount && arg['fid'].indexOf(sPrimaryAccountSPREFIX)==0)
 						aMenu.push({'title':'POPUP_FOLDERS::UNSUBSCRIBE_FOLDER','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'unsubscribe_folder'},'css':'ico2 color2 delete_folder','disabled':dataSet.get('folders', [arg.aid, arg.fid, 'SUBSCRIPTION_TYPE']) != 'folder'});
 
-					aMenu.push({'title':'POPUP_FOLDERS::DELETE_FOLDER','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'delete_folder'},'css':'ico2 color2 delete_folder','disabled':bProtected || sFolType == 'X'});
+					if (!snoozed) {
+						aMenu.push({'title':'POPUP_FOLDERS::DELETE_FOLDER','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'delete_folder'},'css':'ico2 color2 delete_folder','disabled':bProtected || sFolType == 'X'});
+					}
 				}
 				else{
-					if (GWOthers.getItem('LAYOUT_SETTINGS','favorites')>0 && !bMoreMenu)
-						aMenu.push({'title':'POPUP_FOLDERS::ADD_FAVORITES','arg':{aid:arg['aid'], fid:arg['fid'], 'method':'add_favorite'}, css:'ico2 fav_folder'});
+					if (!bMoreMenu && sFolType !== 'X') {
+						if (bIsFavorite) {
+							aMenu.push({'title':'POPUP_FOLDERS::REMOVE_FAVORITES','arg':{aid:arg['aid'], fid:arg['fid'], 'method':'remove_favorite'}, css:'ico2 fav_folder'});
+						} else {
+							aMenu.push({'title':'POPUP_FOLDERS::ADD_FAVORITES','arg':{aid:arg['aid'], fid:arg['fid'], 'method':'add_favorite'}, css:'ico2 fav_folder'});
+						}
+					}
 
 					if (sFolType == 'Q')
 						aMenu = [{'title':'MAIN_MENU::EMPTY_FOLDER','css':'color2','arg':{'aid':arg['aid'],'fid':arg['fid'],'method':'empty_folder', css:'ico2 color2 empty_folder'}}];
@@ -258,33 +328,52 @@ obj_tree_folder_context.__createFolderMenu = function(arg, bMoreMenu){
 		else
 		if (arg['aid']){
 
-			if (!this._sFilterFolderType || !this._sFilterFolderType['Y']){
+			if (Is.Empty(this._sFilterFolderType) || !this._sFilterFolderType['Y']){
 				aMenu.push({'title':'POPUP_FOLDERS::ADD_FOLDER','arg':{'aid':arg['aid'],'method':'add_folder'}, css:'ico2 add_folder'});
 
 				if (GWOthers.getItem('RESTRICTIONS','disable_virtual') != '1')
 					aMenu.push({'title':'POPUP_FOLDERS::ADD_VIRTUAL','arg':{'method':'add_virtual'}, css:'ico2 add_vfolder', 'disabled': arg['aid']!=sPrimaryAccount});
 
+				this.__selectAllCalendarsMenu(aMenu, arg);
+
 				aMenu.push(
 					{'title':'-'},
-					{'title':'POPUP_FOLDERS::ADD_SHARED_ACCOUNT', 'arg':{'aid':arg['aid'],'method':'add_account'}, css:'ico2 share_account',disabled:arg['aid']!=sPrimaryAccount || !sPrimaryAccountSHARING},
-					{'title':'POPUP_FOLDERS::SHARING','arg':{'aid':arg['aid'], 'method':'share'}, css:'ico2 color1 share_folder', disabled:arg['aid']!=sPrimaryAccount || !sPrimaryAccountSHARING},
-					{'title':'-'});
+					{'title':'POPUP_FOLDERS::ADD_SHARED_ACCOUNT', 'arg':{'aid':arg['aid'],'method':'add_account'}, css:'ico2 share_account',disabled:arg['aid']!=sPrimaryAccount || !sPrimaryAccountSHARING}
+				);
+				if (GWOthers.getItem('RESTRICTIONS', 'disable_folder_sharing') == 0) {
+					aMenu.push({'title':'POPUP_FOLDERS::SHARING_ACCOUNT','arg':{'aid':arg['aid'], 'method':'share'}, css:'ico2 color1 share_folder', disabled:arg['aid']!=sPrimaryAccount || !sPrimaryAccountSHARING});
+				}
 			}
-
-			aMenu.push(
-				{'title':'COMMON::APPEARANCE',
-					nodes:[
-						{'title':'COMMON::EXPAND','arg':{method:'tree_mode',v:2}, css:'ico2' + (Cookie.get(['hide_tree']) == 2?' check':'')},
-						{'title':'COMMON::COLLAPSED','arg':{method:'tree_mode',v:1}, css:'ico2' + (Cookie.get(['hide_tree']) == 1?' check':'')},
-						{'title':'COMMON::AUTOCOLLAPSED','arg':{method:'tree_mode'}, css:'ico2' + (Cookie.get(['hide_tree'])?'':' check')}
-					]
-				});
+		}
+		else
+		if (arg.resources) {
+			this.__selectAllCalendarsMenu(aMenu, arg);
 		}
 	}
 
 	return aMenu;
 };
-_me._onclick = function(e,elm,id,arg){
+
+_me.__selectAllCalendarsMenu = function(aMenu, arg) {
+	if (dataSet.get('active_folder') !== sPrimaryAccount + '/__@@VIRTUAL@@__/__@@EVENTS@@__') {
+		return;
+	}
+
+	if (aMenu.length) {
+		aMenu.push({ title: '-' });
+	}
+	aMenu.push({
+		title: 'POPUP_FOLDERS::SELECT_ALL_CALENDARS',
+		arg: { aid: arg.aid, fid: arg.fid, resources: arg.resources, method: 'selectAllCalendars', select: true },
+		css: 'ico2 select_all'
+	}, {
+		title: 'POPUP_FOLDERS::DESELECT_ALL_CALENDARS',
+		arg: { aid: arg.aid, fid: arg.fid, resources: arg.resources, method: 'selectAllCalendars' },
+		css: 'ico2 deselect_all'
+	});
+};
+
+_me._onclick = async function(e,elm,id,arg){
 
 	if (arg.fid && arg.fid == '~')
 		delete arg.fid;
@@ -296,9 +385,17 @@ _me._onclick = function(e,elm,id,arg){
 			sFolType = WMFolders.getType(arg);
 
 		if (aMenu.length){
-			var cmenu = gui._create("cmenu","obj_context_folder",'','',this);
-				cmenu._fill(aMenu);
+			elm.closest('li').classList.add('active2');
+			var cmenu = await gui._create("cmenu","obj_context_folder",'','',this);
+				await cmenu._fill(aMenu);
 				cmenu._place(e.clientX,e.clientY);
+				cmenu._add_destructor('__onclose');
+				cmenu.__onclose = function() {
+					elm.closest('li').classList.remove('active2');
+				};
+				cmenu.__close = function() {
+					cmenu._destruct();
+				}
 
 			if (sFolType === 'Y') {
 				var args = {
@@ -326,16 +423,20 @@ _me._onclick = function(e,elm,id,arg){
 		else{
 			arg = clone(arg);
 			arg.elm = elm;
-			return this._handleNode(arg);
+			var result = await this._handleNode(arg, true);
+			if (!result && e) {
+				e.stopPropagation();
+			}
+			return result;
 		}
 	}
 };
 
-_me._handleNode = function(arg,bUpdate)
+_me._handleNode = async function(arg,bUpdate)
 {
 
 	if (arg['aid']){
-		var t = arg.ftype || dataSet.get("folders",[arg['aid'],arg['fid'],'TYPE']);
+		var t = arg.ftype || WMFolders.getType(arg), i;
 
 		//DO NOT handle Shared accounts
 		if (arg['fid'] && (t=='A' || t=='VA' || t=='X'))
@@ -355,15 +456,15 @@ _me._handleNode = function(arg,bUpdate)
 		//MERGED CALENDARS
 		if (aFolder == sPrimaryAccount+'/__@@VIRTUAL@@__/__@@EVENTS@@__' && t == 'E' && arg['fid']!='__@@VIRTUAL@@__/__@@EVENTS@@__'){
 
-			var aVirtual = dataSet.get('folders',[sPrimaryAccount,'__@@VIRTUAL@@__/__@@EVENTS@@__','VIRTUAL','FOLDERS'],true) || {},
-				bUpdate = false;
+			var aVirtual = dataSet.get('folders',[sPrimaryAccount,'__@@VIRTUAL@@__/__@@EVENTS@@__','VIRTUAL','FOLDERS'],true) || {};
+			bUpdate = false;
 
 			//Add
 			if (!Is.Defined(aVirtual[arg['fid']])){
 				if (arg.elm && arg.elm.tagName == 'I')
 					aVirtual[arg['fid']] = false;
 				else{
-					for (var i in aVirtual)
+					for (i in aVirtual)
 						aVirtual[i] = false;
 
 					aVirtual[arg['fid']] = true;
@@ -391,7 +492,7 @@ _me._handleNode = function(arg,bUpdate)
 			else
 			if (!aVirtual[arg['fid']]){
 
-				for (var i in aVirtual)
+				for (i in aVirtual)
 					aVirtual[i] = false;
 
 				aVirtual[arg['fid']] = true;
@@ -399,17 +500,34 @@ _me._handleNode = function(arg,bUpdate)
 			}
 
 			//Edit __@@EVENTS@@__
-			this._setActive(sPrimaryAccount+'/__@@VIRTUAL@@__/__@@EVENTS@@__');
+			await this._setActive(sPrimaryAccount+'/__@@VIRTUAL@@__/__@@EVENTS@@__');
 			this._fill();
 
-			if (bUpdate)
+			if (bUpdate) {
+				dataSet.add('folders',[sPrimaryAccount,'__@@VIRTUAL@@__/__@@EVENTS@@__','VIRTUAL','FOLDERS'],aVirtual);
 				WMFolders.add({'aid':sPrimaryAccount,'fid':'__@@VIRTUAL@@__/__@@EVENTS@@__','name':'__@@VIRTUAL@@__/__@@EVENTS@@__','type':'E','virtual':{folders:aVirtual}},'folders','',[this,'__refresh',[arg]]);
-			else
+			} else {
 				gui.__exeEvent('folderSelected',arg);
+			}
 
 			return false;
 		}
 		else{
+			var filter = gui.frm_main.search.__lastQueryClient[t] || '';
+			var subtree = filter.match(/\+folders:"subtree"/);
+			if (subtree) {
+				delete gui.frm_main.main.list.__lastSearchFolder;
+				delete gui.frm_main.main.list.__lastSearchAccount;
+			} else if (~filter.indexOf('+folders:"')) {
+				filter = filter.replace(/\+folders:"all"/, '').trim();
+				if (filter.replace(/is:unread/, '').trim()) {
+					if (aFolder !== arg['aid'] +'/'+ (arg['fid'] || '~')) {
+						delete gui.frm_main.main.list.__lastSearchFolder;
+						delete gui.frm_main.main.list.__lastSearchAccount;
+					}
+					gui.frm_main.search.__lastQueryClient[t] = filter;
+				}
+			}
 			//Neotvíráme již otevřený folder?
 			if (aFolder != arg['aid'] +'/'+ (arg['fid'] || '~') || bUpdate)
 				gui.frm_main._selectView(arg);
@@ -421,11 +539,6 @@ _me._handleNode = function(arg,bUpdate)
 	}
 };
 
-_me.__refresh = function(aData){
-	if (aData && aData.virtual){
-		//write
-		dataSet.add('folders',[sPrimaryAccount,'__@@VIRTUAL@@__/__@@EVENTS@@__','VIRTUAL','FOLDERS'],aData.virtual.folders);
-	}
-
+_me.__refresh = function(){
 	gui.frm_main._selectView({aid:sPrimaryAccount,fid:'__@@VIRTUAL@@__/__@@EVENTS@@__'},'',true);
 };
