@@ -1,7 +1,7 @@
 _me = obj_upload_info.prototype;
 function obj_upload_info(){};
 
-_me.__constructor = function(aAbortHandler){
+_me.__constructor = async function(){
 	this.__eSize = this._getAnchor('size');
 	this.__eName = this._getAnchor('name');
 
