@@ -1,7 +1,7 @@
 _me = obj_vtabs.prototype;
 function obj_vtabs(){};
 
-_me.__constructor = function(){
+_me.__constructor = async function(){
 	this._scrollbar(this._getAnchor('header2'));
 };
 
