@@ -2,7 +2,7 @@
 _me = obj_websocket_xmpp.prototype;
 function obj_websocket_xmpp(){};
 
-_me.__constructor = function(){
+_me.__constructor = async function(){
 	this._idPrefix = 'xmpp-';
 	this._add_destructor('_close');
 
@@ -25,7 +25,7 @@ _me._init = function(){
 	}
 };
 
-_me._close = function(e){
+_me._close = function(){
 	if (this._state()){
 		this._state(0);
 		this._parent._sendArray({'CLOSE':[{ATTRIBUTES:{XMLNS:'urn:ietf:params:xml:ns:xmpp-framing'}}]});
@@ -35,7 +35,11 @@ _me._close = function(e){
 //// SOCKET EVENTS ////
 _me._onstatechange = function(state){
 
-	console.log('xmpp.state', state);
+	 console.log('xmpp.state', state);
+	this.__exeEvent('status',{
+		status:this._opt.status,
+		state:state
+	});
 
 	//Offline
 	if (state === 0){
@@ -53,7 +57,7 @@ _me._onstatechange = function(state){
 				bUpdate = false;
 
 			for(var jid in ds){
-				if (ds.hasOwnProperty(jid)){
+				if (Object.prototype.hasOwnProperty.call(ds, jid)){
 					//set offline
 					if (ds[jid].show && ds[jid].show != 'offline'){
 						ds[jid].show = 'offline';
@@ -81,17 +85,16 @@ _me._onstatechange = function(state){
 			}
 			bUpdate && dataSet.update('xmpp', ['roster']);
 
-			//global activity monitor
 			dataSet.update('main', ['im']);
 		}
 	}
 };
 
-_me._onclose = function(e){
+_me._onclose = function(){
 	this._state(0);
 };
 
-_me._onopen = function(e){
+_me._onopen = function(){
 	if (this._state() === 0 && this._opt.status && this._opt.status != 'offline'){
 		this._state(1);
 		this._parent._sendArray({
@@ -143,27 +146,19 @@ _me._getResourceHash = function() {
 	return hash;
 };
 
-_me._onmessage = function(e, aResponse){
+_me._onmessage = async function(e, aResponse){
 	//Skip API calls
 	if (aResponse.IQ && aResponse.IQ[0].ATTRIBUTES && aResponse.IQ[0].ATTRIBUTES.XMLNS === 'http://icewarp.com/api/')
 		return;
 
+	var i;
 	if (aResponse.OPEN){
 		this._state(2);
 	}
 	else
 	if (aResponse['STREAM:FEATURES']){
 		//<stream:features><starttls xmlns="urn:ietf:params:xml:ns:xmpp-tls"/><mechanisms xmlns="urn:ietf:params:xml:ns:xmpp-sasl"><mechanism>DIGEST-MD5</mechanism><mechanism>SCRAM-SHA-1</mechanism></mechanisms><feature xmlns="http://icewarp.com/ns/"/></stream:features>
-		// if (aResponse['STREAM:FEATURES'][0].MECHANISMS){
-			// if ((aResponse['STREAM:FEATURES'][0].MECHANISMS[0].MECHANISM || []).some(function(m){
-			// 	return m.VALUE === 'DIGEST-MD5';
-			// })){
-				this._login();
-				return;
-			// }
-		// }
-
-		this._onerror('STREAM:FEATURES', true);
+		return this._login();
 	}
 	else
 	if (aResponse['STREAM:ERROR']){
@@ -186,7 +181,7 @@ _me._onmessage = function(e, aResponse){
 
 		//<iq type="set"><bind xmlns="urn:ietf:params:xml:ns:xmpp-bind"/></iq>
 		this._create_iq({BIND:[{ATTRIBUTES:{XMLNS:"urn:ietf:params:xml:ns:xmpp-bind"}, RESOURCE:[{VALUE:this._opt.resource + '-' + this._opt.resourceHash}]}]}, {type:'set'},
-			[function(aData){
+			[function(){
 
 				this._state(4);
 
@@ -255,7 +250,7 @@ _me._onmessage = function(e, aResponse){
 	//QUERY
 	if (aResponse.IQ){
 
-		for(var i in aResponse.IQ){
+		for(i in aResponse.IQ){
 
 			//HANDLERS
 			var id = aResponse.IQ[i].ATTRIBUTES.ID;
@@ -267,6 +262,7 @@ _me._onmessage = function(e, aResponse){
 					bError = true;
 				}
 
+				var aHandler;
 				if (bError && this.__requests[id].error)
 					aHandler = this.__requests[id].error;
 				else
@@ -329,7 +325,7 @@ _me._onmessage = function(e, aResponse){
 						if (query.STREAMHOST && query.ATTRIBUTES && query.ATTRIBUTES.SID && this.__streamBuffer[query.ATTRIBUTES.SID]){
 
 							//Check connection by wm_upload
-							storage.library('sha1');
+							await storage.library('sha1');
 							var out = [];
 
 							//SHA1(SID + Initiator JID + Target JID)
@@ -363,13 +359,13 @@ _me._onmessage = function(e, aResponse){
 
 	//PRESENCE
 	if (aResponse.PRESENCE){
-		var attr,
+		var attr, sFrom,
 			presenceBuffer = {};
 
 		//Move unavailable presences to the top
 		aResponse.PRESENCE.sort(function(a, b){
-			var a = a.ATTRIBUTES || {},
-				b = b.ATTRIBUTES || {};
+			a = a.ATTRIBUTES || {},
+			b = b.ATTRIBUTES || {};
 
 			if (a.TYPE == b.TYPE)
 				return 0;
@@ -381,7 +377,7 @@ _me._onmessage = function(e, aResponse){
 				return 1;
 		});
 
-		for(var i in aResponse.PRESENCE){
+		for(i in aResponse.PRESENCE){
 
 			if (aResponse.PRESENCE[i].ERROR){
 				this.__exeEvent('response', 'error', aResponse.PRESENCE[i].ERROR[0]);
@@ -392,7 +388,7 @@ _me._onmessage = function(e, aResponse){
 
 			//Probe presence of all remaining resources when one resource goes offline
 			if (attr.FROM){
-				var sFrom = attr.FROM.split('/').shift().toLowerCase();
+				sFrom = Path.split(attr.FROM)[0].toLowerCase();
 				if (sFrom !== sPrimaryAccount){
 
 					if (!presenceBuffer[sFrom])
@@ -410,20 +406,30 @@ _me._onmessage = function(e, aResponse){
 		}
 
 		//Probe presence of all remaining resources when one resource goes offline
-		for(var sFrom in presenceBuffer){
-			if (presenceBuffer.hasOwnProperty(sFrom) && presenceBuffer[sFrom].probe)
+		for(sFrom in presenceBuffer){
+			if (Object.prototype.hasOwnProperty.call(presenceBuffer, sFrom) && presenceBuffer[sFrom].probe)
 				this._request(this._create_presence('','probe', sFrom));
 		}
 	}
 
 	//MESSAGE
-	(aResponse.MESSAGE || []).forEach(function(message) {
+	for (var message of aResponse.MESSAGE || []) {
 		var ampRule,
 			amp = message.AMP || ((((((message.RECEIVED || message.SENT || [])[0] || {}).FORWARDED || [])[0] || {}).MESSAGE || [])[0] || {}).AMP,
+			displayed = message.DISPLAYED || ((((((message.RECEIVED || message.SENT || [])[0] || {}).FORWARDED || [])[0] || {}).MESSAGE || [])[0] || {}).DISPLAYED,
 			request = this.__requests[message.ATTRIBUTES.ID],
 			sType = '',
 			data = '';
 
+		// skip displayed messages
+		if (displayed) {
+			var sJID = (((((((message.RECEIVED || message.SENT || [])[0] || {}).FORWARDED || [])[0] || {}).MESSAGE || [])[0] || {}).ATTRIBUTES || {}).TO;
+			if (sJID) {
+				dataSet.add('xmpp', ['roster', sJID, 'action'], '');
+				return;
+			}
+		}
+
 		if (amp && (ampRule = (((amp[0] || {}).RULE || [])[0] || {}).ATTRIBUTES) && ampRule.ACTION === 'notify' && ampRule.CONDITION === 'deliver' && (!ampRule.VALUE || ampRule.VALUE === 'direct')) {
 			// send delivered confirmation
 			this._parent._sendArray({
@@ -450,7 +456,7 @@ _me._onmessage = function(e, aResponse){
 		}
 
 		if (request && (!~(message.ATTRIBUTES.FROM || '').indexOf('@')) && amp[0].ATTRIBUTES.TO === request.request.MESSAGE[0].ATTRIBUTES.TO) {
-			executeCallbackFunction(request.response, aResponse);
+			await executeCallbackFunction(request.response, aResponse);
 		}
 
 		if (message.ERROR){
@@ -498,8 +504,21 @@ _me._onmessage = function(e, aResponse){
 			data = message.RECEIVED[0].FORWARDED[0];
 		}
 
-		this.__exeEvent('response', sType, data);
-	}, this);
+		await this.__exeEvent('response', sType, data);
+	};
+};
+
+_me._markAsDisplayed = function(sRecipient, sMessageId, aHandler) {
+	var aData = {
+		DISPLAYED: [{
+			ATTRIBUTES:{
+				XMLNS: "urn:xmpp:chat-markers:0",
+				ID: sMessageId
+			}
+		}]
+	};
+
+	this._request(this._create_message(aData, 'chat', sRecipient), aHandler);
 };
 
 //// LOGIN SEQUENCE ////
@@ -547,7 +566,7 @@ _me._create_iq = function(aData, opt, aHandler, aErrHandler, bNoBuffer){
 	if (!Is.Defined(id))
 		do { id = this._idPrefix + unique_id() } while (this.__requests[id]);
 
-	var aData = aData || {};
+	aData = aData || {};
 
 	if (!aData.ATTRIBUTES)
 		aData.ATTRIBUTES = {};
@@ -584,15 +603,18 @@ _me._create_query = function(aData,sXMLns){
 	return aRequest;
 };
 
-_me._create_message = function(aData, sType, sTo, aHandler, message){
+_me._getUniqueID = function() {
+	return this._idPrefix + unique_id();
+}
+
+_me._create_message = function(aData, sType, sTo, aHandler, message, id){
 	var aRequest = {MESSAGE:[]},
-		tmp = {},
-		id;
+		tmp = {};
 
 	do {
-		id = this._idPrefix + unique_id()
-		if (message) {
-			message.id = id;
+		id = id || this._getUniqueID();
+		if (message && message.data) {
+			message.data.id = id;
 		}
 	} while (this.__requests[id]);
 
@@ -659,13 +681,13 @@ _me.__carbons_support = function(){
 				for(var id in aFeatures){
 					switch(((aFeatures[id] || {}).ATTRIBUTES || {}).VAR) {
 						case "urn:xmpp:carbons:2":
-							var aData = {ENABLE:[{ATTRIBUTES:{XMLNS:"urn:xmpp:carbons:2"}}]};
+							aData = {ENABLE:[{ATTRIBUTES:{XMLNS:"urn:xmpp:carbons:2"}}]};
 
 							this._create_iq(aData, {type:'set'}, [function(aData){
 								if (aData.IQ){
 									aData = aData.IQ[0];
 									if (aData && aData.ATTRIBUTES && aData.ATTRIBUTES.TYPE === "result"){
-										console.log('xmpp.carbons', 'active');
+										 console.log('xmpp.carbons', 'active');
 										this._opt.carbons = true;
 									}
 								}
@@ -674,6 +696,9 @@ _me.__carbons_support = function(){
 						case 'http://jabber.org/protocol/amp':
 							this._opt.amp = true;
 							break;
+						case 'urn:xmpp:chat-markers:0':
+							this._opt.markable = true;
+							break;
 					}
 				}
 			}
@@ -699,7 +724,6 @@ _me._presence = function(sStatus, sStatusText, aHandler){
 					state:state
 				});
 
-				//global activity monitor
 				dataSet.update('main', ['im']);
 			}
 
@@ -852,13 +876,17 @@ _me._vcard = function(sTo, aValue, aHandler, aErrHandler){
 	 * @param {xml array} aData
 	 * @param {handler} aHandler
 	 */
-	_me.__photo_hash = function(aData, aHandler){
+	_me.__photo_hash = async function(aData, aHandler){
 
 		//Avatar Photo
 		if (aData && aData.IQ && (aData = aData.IQ[0]) && aData.VCARD && aData.VCARD[0] && (aData = aData.VCARD[0])){
 			if (aData.PHOTO && aData.PHOTO[0] && aData.PHOTO[0].TYPE && aData.PHOTO[0].BINVAL){
-				storage.library('sha1');
-				this._opt.imageHash = SHA1(base64.decode(aData.PHOTO[0].BINVAL[0].VALUE));
+				await storage.library('sha1');
+				try {
+					this._opt.imageHash = SHA1(atob(aData.PHOTO[0].BINVAL[0].VALUE));
+				} catch {
+					this._opt.imageHash = '';
+				}
 			}
 			else
 				this._opt.imageHash = '';
@@ -965,11 +993,11 @@ _me._socks_connect = function (aData,sSID,sFrom){
 };
 // UPLOAD
 
-_me._stream_send = function (sFolder, sFile, sClass, iSize, sName, sTo, sDesc) {
+_me._stream_send = async function (sFolder, sFile, sClass, iSize, sName, sTo, sDesc) {
 
 	//Do Upload.Extract when size is 0 (GW Object)
 	if (!iSize){
-	    storage.library('wm_upload');
+	    await storage.library('wm_upload');
 		var up = new wm_upload();
 			up.extract(sFolder,sFile,sClass,[this, '_stream_send', [sTo, sDesc]]);
 
@@ -1022,7 +1050,7 @@ _me._stream_send = function (sFolder, sFile, sClass, iSize, sName, sTo, sDesc) {
 		</si>
 	</iq>
 	*/
-	storage.library('sha1');
+	await storage.library('sha1');
 	var sHash = SHA1(unique_id()+sPrimaryAccount+'/WebClient'+sTo);
 	var q = {SI:[{
 		ATTRIBUTES:{XMLNS:"http://jabber.org/protocol/si",PROFILE:"http://jabber.org/protocol/si/profile/file-transfer",ID:sHash},
@@ -1129,7 +1157,7 @@ _me._stream_support = function(aData, sHash, sFolder, sFile, sClass, sDesc) {
 		try{
 			bSupport = aData.SI[0].FEATURE[0].X[0].FIELD[0].VALUE[0].VALUE == 'http://jabber.org/protocol/bytestreams' && aData.SI[0].FEATURE[0].X[0].FIELD[0].ATTRIBUTES.VAR == 'stream-method';
 		}
-		catch(r){gui._REQUEST_VARS.debug && console.log(this._name||false,r)}
+		catch(r){ console.log(this._name||false,r)}
 
 		if (bSupport){
 			var q = this._create_query('','http://jabber.org/protocol/bytestreams');
@@ -1150,13 +1178,13 @@ _me._stream_support = function(aData, sHash, sFolder, sFile, sClass, sDesc) {
 	}
 };
 
-_me._stream_upload = function(aData, sHash, sFolder, sFile, sClass){
+_me._stream_upload = async function(aData, sHash, sFolder, sFile, sClass){
 
 	if (aData && aData.IQ && (aData = aData.IQ[0]) && aData.ATTRIBUTES && aData.ATTRIBUTES.TYPE == 'result' && aData.QUERY && aData.QUERY[0]['STREAMHOST-USED'] && aData.QUERY[0]['STREAMHOST-USED'][0].ATTRIBUTES && aData.QUERY[0]['STREAMHOST-USED'][0].ATTRIBUTES.JID == GWOthers.getItem('STREAMHOST', 'jid')){
-		storage.library('wm_upload');
-		storage.library('sha1');
+		await storage.library('wm_upload');
+		await storage.library('sha1');
 
-		var sHash = SHA1(sHash+sPrimaryAccount+'/WebClient'+aData.ATTRIBUTES.FROM);
+		sHash = SHA1(sHash+sPrimaryAccount+'/WebClient'+aData.ATTRIBUTES.FROM);
 		var up = new wm_upload();
 			up.socket(sFolder, sFile, sClass, sHash, [function(){
 				this.__exeEvent('response', 'bytestream', arguments);
@@ -1166,8 +1194,8 @@ _me._stream_upload = function(aData, sHash, sFolder, sFile, sClass){
 	else
 		gui.notifier._value({type: 'alert', args: {header: 'IM::SEND_FILE', text: 'IM::ERROR_BYTESTREAM'}});
 };
-_me._oob_send = function (sFolder, sFile, sClass, sTo, sDesc) {
-	storage.library('wm_upload');
+_me._oob_send = async function (sFolder, sFile, sClass, sTo, sDesc) {
+	await storage.library('wm_upload');
 	var up = new wm_upload();
 		up.ticket(sFolder, sFile, sClass, [this, '_oob_support', [sTo, sDesc]]);
 };
@@ -1219,15 +1247,15 @@ _me._oob_finish = function (aData, aHandler){
 };
 
 //Initiate File upload
-_me._file_upload = function (sTo, aArg, aHandler){
-	var sStatus = Is.Array(sTo)?'offline':this._user_status(sTo);
+_me._file_upload = function (sTo, aArg, aHandler, sId){
+	// var sStatus = Is.Array(sTo)?'offline':this._user_status(sTo);
 
-	if (sStatus == 'offline')
-		this._xlink(sTo, aArg, aHandler);
-	else{
-		//check support
-		this._create_iq(this._create_query('','http://jabber.org/protocol/disco#info'), {to:sTo}, [this,'_file_upload_support',[sTo, aArg, aHandler]]);
-	}
+	// if (sStatus == 'offline')
+		this._xlink(sTo, aArg, aHandler, sId);
+	// else{
+	// 	//check support
+	// 	this._create_iq(this._create_query('','http://jabber.org/protocol/disco#info'), {to:sTo}, [this,'_file_upload_support',[sTo, aArg, aHandler]]);
+	// }
 };
 
 //Choose proper upload method
@@ -1280,20 +1308,29 @@ _me._ooblink = function(sTo, aArg, aHandler){
 	}
 };
 
-_me._xlink = function(sTo, aArg, aHandler){
+_me._xlink = function(sTo, aArg, aHandler, sId){
 	if (sTo){
 
 		var aData = {
-			DISPLAYED:[{ATTRIBUTES:{XMLNS:"rn:xmpp:chat-markers:0"}}],
+			DISPLAYED:[{ATTRIBUTES:{XMLNS:"urn:xmpp:chat-markers:0"}}],
 			X:[{
 				ATTRIBUTES:{XMLNS:"jabber:x:oob"},
 				URL:[{VALUE:aArg.link}],
 				DESC:[{VALUE:aArg.desc || ''}],
 				SIZE:[{VALUE:aArg.size || 0}]
+			}],
+			ACTIVE: [{
+				ATTRIBUTES: { XMLNS: "http://jabber.org/protocol/chatstates" }
 			}]
 		};
 
-		this._request(this._create_message(aData,'chat',sTo), aHandler);
+		if (this._opt.markable) {
+			aData.MARKABLE = [{
+				ATTRIBUTES: { XMLNS: 'urn:xmpp:chat-markers:0' }
+			}];
+		}
+
+		this._request(this._create_message(aData,'chat',sTo, false, false, sId), aHandler);
 	}
 };
 
@@ -1316,7 +1353,7 @@ _me._roster_get = function(aHandler, aErrorHandler, bNoBuffer){
 		if (aData && aData.IQ && aData.IQ[0].QUERY && (aData = aData.IQ[0].QUERY[0])){
 
 			var sSub, sJID,	sName, sGroup, oldShow,
-				bRefresh, bRefreshJid,
+				bRefreshJid,
 				aGateways = dataSet.get('xmpp', ['gateways']) || {};
 
 			for(var i in aData.ITEM){
@@ -1344,11 +1381,9 @@ _me._roster_get = function(aHandler, aErrorHandler, bNoBuffer){
 					if (tmp || ((tmp = sJID.indexOf('@'))>-1 && (tmp = aGateways[sJID.substr(tmp+1)])))
 						dataSet.add('xmpp', ['roster', sJID, 'type'], tmp.TYPE, true);
 				}
-				bRefresh = true;
 			}
 
-			if (bRefresh)
-				dataSet.update('xmpp', ['roster']);
+			dataSet.update('xmpp', ['roster']);
 		}
 
 		aHandler && executeCallbackFunction(aHandler);
@@ -1371,7 +1406,7 @@ _me._history_get = function(aData, aHandler){
 	var q = {RETRIEVE:[{ATTRIBUTES:{XMLNS:'urn:xmpp:archive',WITH:aData.from},SET:[{ATTRIBUTES:{XMLNS:'http://jabber.org/protocol/rsm'}}]}]};
 
 	if (aData['start'])
-		q.RETRIEVE[0].ATTRIBUTES.START = typeof aData['start'] == 'date'?aData['start'].format('imDateTime'):aData['start'];
+		q.RETRIEVE[0].ATTRIBUTES.START = typeof aData['start'] == 'object'?aData['start'].format('imDateTime'):aData['start'];
 
 	if (aData['search'])
 		q.RETRIEVE[0].BODY = [{VALUE:aData['search']}];
@@ -1506,7 +1541,7 @@ _me._user_unsubscribed = function(sJID, aHandler){
 };
 
 // Conference invitation - send
-_me._invitation = function (sTo, sBody, aHandler, aHandler2) {
+_me._invitation = function (sTo, sBody, aHandler, aHandler2, message, id) {
 	if (sTo) {
 		var aData = {
 			BODY: [{ VALUE: sBody }],
@@ -1531,13 +1566,13 @@ _me._invitation = function (sTo, sBody, aHandler, aHandler2) {
 				}]
 			}];
 		}
-		this._request(this._create_message(aData, 'chat', sTo, aHandler2), aHandler);
+		this._request(this._create_message(aData, 'chat', sTo, aHandler2, false, id), aHandler);
 	}
 };
 
 // MESSAGE
 
-_me._message = function (sTo, sBody, aHandler, aHandler2, message) {
+_me._message = function (sTo, sBody, aHandler, aHandler2, message, id) {
 	var aData = {
 		BODY: [{ VALUE: sBody }],
 		ACTIVE: [{
@@ -1555,7 +1590,12 @@ _me._message = function (sTo, sBody, aHandler, aHandler2, message) {
 			}]
 		}];
 	}
-	this._request(this._create_message(aData, 'chat', sTo, aHandler2, message), aHandler);
+	if (this._opt.markable) {
+		aData.MARKABLE = [{
+			ATTRIBUTES: { XMLNS: 'urn:xmpp:chat-markers:0' }
+		}];
+	}
+	this._request(this._create_message(aData, 'chat', sTo, aHandler2, message, id), aHandler);
 };
 
 _me._msg_status = function(sTo, sType, aHandler){
@@ -1578,7 +1618,7 @@ _me._geo = function(sTo, aArg, aHandler){
 					ATTRIBUTES:{NODE:"http://jabber.org/protocol/geoloc"},
 					ITEM:[{
 						GEOLOC:[{
- 							ATTRIBUTES:{XMLNS:"http://jabber.org/protocol/geoloc", 'XML\:LANG':"en"},
+ 							ATTRIBUTES:{XMLNS:"http://jabber.org/protocol/geoloc", 'XML:LANG':"en"},
  							LAT:[{VALUE:aArg.lat}],
  							LON:[{VALUE:aArg.lon}]
 						}]
