@@ -52,12 +52,12 @@ var IceSIP = function(config) {
 			me.onconnect({host: host});
 		}
 	});
-	this.sip.on('unregistered',function(e){
+	this.sip.on('unregistered',function(){
 		me.state = 'offline';
 		me.activity = null;
 		me.ondisconnect({reason: 'offline'});
 	});
-	this.sip.on('registrationFailed',function(e){
+	this.sip.on('registrationFailed',function(){
 		me.state = 'failed';
 		me.activity = null;
 		me.ondisconnect({reason: 'failed'});
@@ -111,7 +111,7 @@ var IceSIP = function(config) {
 
 				// Handling started call
 				session.on('confirmed',(function(call){
-					return function(e) {
+					return function() {
 						me.activity = 'Talking';
 
 						call.onanswer({
@@ -124,7 +124,7 @@ var IceSIP = function(config) {
 				// Determine if it's a audio, video or presentation call
 				var sdp = e.request.body;
 				var presentation = sdp && sdp.indexOf("m=video")!=-1 && sdp.indexOf("a=sendonly")!=-1;
-				var video = presentation || sdp && sdp.indexOf("m=video")!=-1 && sdp.indexOf("a=sendrecv")!=-1;
+				video = presentation || sdp && sdp.indexOf("m=video")!=-1 && sdp.indexOf("a=sendrecv")!=-1;
 
 				// Only remote stream is ready in this stage
 				me.oncall({
@@ -156,7 +156,7 @@ var IceSIP = function(config) {
 						message.group = tmp=="is_private=0" ? true : false;
 					else if(tmp.indexOf("from_id=")===0)
 						message.from_id = tmp.substr(8);
-				} while(tmp=data.pop());
+				} while((tmp=data.pop()));
 			}
 			me.onmessage(message);
 		}
@@ -174,7 +174,9 @@ IceSIP.supported = function() {
 		try {
 			if(new ActiveXObject('Tem.TemWebRTCPlugin'))
 				supported = true;
-		} catch(e) {}
+		} catch {
+			//
+		}
 		return supported;
 	} else if(window.webkitAudioContext) { // WebKit engine without native support (Safari), check for plugin
 		if(navigator.plugins && navigator.plugins.namedItem && navigator.plugins.namedItem('TemWebRTCPlugin'))
@@ -204,7 +206,7 @@ IceSIP.devices = function(callback) {
 			});
 			callback(media);
 		})
-		.catch(function(err) {
+		.catch(function() {
 			callback({});
 		});
 	} else callback({});
@@ -252,19 +254,19 @@ IceSIP.prototype.busy = function() {
 
 IceSIP.prototype.onconnect = function(e) {
 	// Abstract method
-	console.warn('No handler assigned: Connected to '+e.host);
+	 console.warn('No handler assigned: Connected to '+e.host);
 	// You are connected to host
 };
 
-IceSIP.prototype.ondisconnect = function(e) {
+IceSIP.prototype.ondisconnect = function() {
 	// Abstract method
-	console.warn('No handler assigned: Disconnected');
+	 console.warn('No handler assigned: Disconnected');
 	// Connection was lost
 };
 
-IceSIP.prototype.onmedia = function(e) {
+IceSIP.prototype.onmedia = function() {
 	// Abstract method
-	console.warn('No handler assigned: Media');
+	 console.warn('No handler assigned: Media');
 	// Asking for or granted access to media
 };
 
@@ -287,22 +289,8 @@ IceSIP.attachVideoStream = function(stream) {
 		video = elm.firstChild;
 	} else {
 		video = document.createElement('video');
-		//video.setAttribute('muted', true);
 		video.setAttribute('autoplay', true);
 
-		// video.onloadeddata = function(e){
-		// 	console.warn('loadeddata', e);
-		// };
-		// video.oncanplay = function(e){
-		// 	console.warn('canplay', e);
-		// };
-		// video.oncanplaythrough = function(e){
-		// 	setTimeout(function(){
-		// 		this.play();
-		// 		console.warn('oncanplaythrough',e);
-		// 	}.bind(this), 1000);
-		// };
-
 		video.srcObject = stream;
 
 	}
@@ -325,8 +313,8 @@ IceSIP.Call.prototype.getLocalStreams = function(){
 
 	senders.forEach(function(sender) {
 		stream.addTrack(sender.track);
-		if (sender.track.kind === 'video')
-			camera = true;
+		// if (sender.track.kind === 'video')
+		// 	camera = true;
 	});
 
 	return stream;
@@ -357,10 +345,10 @@ IceSIP.Call.prototype.make = function(address,bVideo,bScreen) {
 	var audio = false;
 
 	this.session = this.sip.call('sip:'+address,{eventHandlers: {
-		progress: function(e) {
+		progress: function() {
 			me.state = 'Calling';
 		},
-		connecting: function(e) {
+		connecting: function() {
 			me.oncalling({
 				call: me.session,
 				name: me.session._remote_identity._display_name,
@@ -372,7 +360,7 @@ IceSIP.Call.prototype.make = function(address,bVideo,bScreen) {
 			me.state = 'Failed';
 			me.onhangup({reason: e.cause ? e.cause : 'Unknown', remote: e.originator=='remote'});
 		},
-		confirmed: function(e) {
+		confirmed: function() {
 			if(me.session && me.session._remote_identity) {
 				me.state = 'Talking';
 
@@ -414,27 +402,7 @@ IceSIP.Call.prototype.make = function(address,bVideo,bScreen) {
 		},
 		replaces: function(e) {
 			e.accept();
-		}/*,
-		reinvite: function(e) {
-			e.callback = function() {
-				if(!gui.video || gui.video._destructed) {
-					var stream = pc.getRemoteStreams()[0];
-					if(stream.getVideoTracks().length) {
-						var localstream = pc.getLocalStreams();
-						var camera = localstream.length && localstream[0].getVideoTracks().length;
-						var video = {
-							remote: IceSIP.attachVideoStream(stream),
-							local: camera ? IceSIP.attachVideoStream(localstream[0]) : false
-						};
-						me.onstreaming({
-							camera: camera ? true : false,
-							video: video,
-							audio: false
-						});
-					}
-				}
-			}
-		}*/
+		}
 	},
 	mediaConstraints: mc,
 	rtcOfferConstraints: oc
@@ -513,16 +481,10 @@ IceSIP.Call.prototype.answer = function(bShow) {
 			mc.video = false;
 
 		this.session.answer({
-			mediaConstraints: mc /*,
-			rtcAnswerConstraints: {
-				mandatory: {
-					OfferToReceiveAudio: true,
-					OfferToReceiveVideo: true
-				}
-			}	*/
+			mediaConstraints: mc
 		});
 
-		this.session._connection.addEventListener('addstream', function(e) {
+		this.session._connection.addEventListener('addstream', function() {
 			var remote, local;
 
 			var ext_stream = me.getRemoteStreams(),
@@ -542,7 +504,7 @@ IceSIP.Call.prototype.answer = function(bShow) {
 			else
 			// Adding pure audio stream
 			if (ext_stream.getAudioTracks().length) {
-				me.session.audio = audio = new Audio();
+				me.session.audio = new Audio();
 				me.session.audio.srcObject = ext_stream;
 				me.session.audio.play();
 			}
@@ -572,7 +534,7 @@ IceSIP.Call.prototype.decline = function() {
 
 // Dial numbers via keypad
 IceSIP.Call.prototype.dial = function(n,bInband) {
-	if(this.sip.isConnected() && this.sip.isRegistered() && this.session && /^[*0-9#]+$/.test(n)) {
+	if(this.sip.isConnected() && this.sip.isRegistered() && this.session && (/^[*0-9#]+$/).test(n)) {
 		var stream = this.session._localMediaStream;
 		var audio = stream.getAudioTracks() || null;
 		var peer = this.session._connection.pc;
@@ -640,7 +602,7 @@ IceSIP.Call.prototype.hangup = function(code) {
 	}
 };
 
-IceSIP.Call.prototype.addvideo = function(code) {
+IceSIP.Call.prototype.addvideo = function() {
 	if(this.sip.isConnected() && this.sip.isRegistered() && this.session) {
 		var session = this.session;
 		var connection = this.session._connection;
@@ -658,7 +620,7 @@ IceSIP.Call.prototype.addvideo = function(code) {
 			// connection.addStream(stream);
 
 			session.renegotiate({useUpdate:false},function(){
-				console.log("Renegotiate", arguments);
+				 console.log("Renegotiate", arguments);
 			});
 
 
@@ -667,7 +629,7 @@ IceSIP.Call.prototype.addvideo = function(code) {
 };
 
 // Add video to an ongoing call
-IceSIP.Call.prototype.camera = function(conference_id,data) {
+IceSIP.Call.prototype.camera = function() {
 	var me = this;
 
 	var constraints = {
@@ -684,11 +646,11 @@ IceSIP.Call.prototype.camera = function(conference_id,data) {
 		navigator.webkitGetUserMedia(constraints,
 			addStream,
 			function(){
-				console.log("No stream!");
+				 console.log("No stream!");
 			}
 		);
 	} catch(e){
-		console.log('Failed',e);
+		 console.log('Failed',e);
 	}
 
 };
@@ -696,366 +658,34 @@ IceSIP.Call.prototype.camera = function(conference_id,data) {
 
 IceSIP.Call.prototype.ondial = function(e) {
 	// Abstract method
-	console.warn('No handler assigned: '+e.number+' was dialed');
+	 console.warn('No handler assigned: '+e.number+' was dialed');
 	// You are connected to host
 };
 
 IceSIP.Call.prototype.onanswer = function(e) {
 	// Abstract method
-	console.warn('No handler assigned: '+e.user+' has picked up');
+	 console.warn('No handler assigned: '+e.user+' has picked up');
 	// Outgoing call was answered
 };
 
 IceSIP.Call.prototype.ondecline = function(e) {
 	// Abstract method
-	console.warn('No handler assigned: '+e.user+' has declined call');
+	 console.warn('No handler assigned: '+e.user+' has declined call');
 	// Outgoing call was declined
 };
 
 IceSIP.Call.prototype.oncalling = function(e) {
 	// Abstract method
-	console.warn('No handler assigned: you are calling '+e.user);
+	 console.warn('No handler assigned: you are calling '+e.user);
 	// Outgoing call was terminated
 };
 
 IceSIP.Call.prototype.onhangup = function(e) {
 	// Abstract method
-	console.warn('No handler assigned: '+e.user+' has hanged up');
+	 console.warn('No handler assigned: '+e.user+' has hanged up');
 	// Outgoing call was terminated
 };
 
-/* Conference */
-
-IceSIP.Conference = function(gear,session) {
-	// Internal data
-	this.sip = gear.sip;
-	this.state = 'Calling';
-	this.session = session;
-	this.sip.type = 'Conference';
-	// External reference
-	IceSIP.Conference.current = this;
-	// Configuration parameters
-	this.microphone = true;
-	this.tracking = '';
-};
-
-IceSIP.Conference.current = null;
-
-IceSIP.Conference.prototype.join = function(conference_id) {
-	var me = this;
-
-	var id = conference_id.match(/^conference\*([0-9]+)/);
-	id = id.length ? id[1] : 'null';
-
-	this.full_id = conference_id;
-	this.numeric_id = id;
-
-	var mc = {audio: this.microphone || false, video: false};
-	var oc = {offerToReceiveAudio: true,offerToReceiveVideo: false};
-
-	var guest = window.sPrimaryAccountGUEST ? {
-		email: dataSet.get('main',['user']),
-		name: dataSet.get('main',['fullname'])
-	} : false;
-
-	this.session = this.sip.call('sip:'+conference_id,{
-		eventHandlers: {
-			progress: function(e) {
-				me.state = 'Ringing';
-			},
-			failed: function(e) {
-				me.state = 'Failed';
-				me.onfailed({reason: e.data && e.data.cause ? e.data.cause : 'Unknown'});
-			},
-			confirmed: function(e) {
-				if(me.session && me.session._request) {
-					me.state = 'Joining';
-
-					me.onjoined({
-						audio: true,
-						callid: me.session._request.call_id
-					});
-				}
-			},
-			ended: function(e) {
-				me.state = 'Ended';
-
-				if(me.session.audio)
-					me.session.audio.pause();
-
-				me.onended({reason: 'Ended'});
-			},
-			newInfo: function(e) {
-				var speakers = e.info.body;
-				if(speakers)
-					me.oninfo({
-						message: speakers,
-						mimetype: e.info.contentType
-					});
-			}
-		},
-		mediaConstraints: mc,
-		RTCOfferConstraints: oc,
-		extraHeaders: [
-			'X-IceWarp-Conference: id='+id,
-			'X-IceWarp-ParticipantData: '+this.tracking
-		],
-		guest: guest
-	});
-
-	var pc = this.session._connection;
-	pc.addEventListener('addstream', function(e) {
-		if(me.session && me.session._remote_identity) {
-			var audiostream = e.stream;
-			if(audiostream) {
-				me.session.audio = audio = new Audio();
-				me.session.audio.srcObject = audiostream;
-				var promise = me.session.audio.play();
-				// Safari as default does not allow to play sound without user interaction
-				if(promise) {
-					promise.catch(function(){
-						gui._create('alert', 'frm_alert', '', '', [function(){
-							me.session.audio.play();
-						}], 'SIP::ERROR', 'SIP::PLAYBLOCKED');
-					}).then(function(){
-					//	me.session.audio.play();
-					});
-				}
-			}
-		}
-	},false);
-
-};
-
-// Screen sharing functionality for Conference
-
-IceSIP.Sharing = function(conference) {
-	this.sip = conference.sip;
-	this.conference_id = conference.full_id;
-	conference.share = this;
-	//this.reference = conference._dialog.id.call_id;
-	this.reference = conference.session._dialog._id.call_id;
-	this.live = false;
-};
-
-// Join an ongoing screen sharing session
-IceSIP.Sharing.prototype.join = function() {
-	this.screen();
-};
-
-// Prepare for sharing screen to other users (Chrome only)
-IceSIP.Sharing.prototype.share = function() {
-	var me = this;
-
-	if(navigator.browser && navigator.browser.application=="Chrome") {
-		WebRTCSharing.getScreenId(function(error, screen_capture_id) {
-			// Set screen constraints for screen capturing
-			var screen_constraints = {audio: false,
-				video: {
-					mandatory: {
-						chromeMediaSource: screen_capture_id ? 'desktop' : 'screen',
-						maxWidth: screen.width > 1920 ? 1920 : screen.width,
-						maxHeight: screen.height > 1080 ? 1080 : screen.height,
-						maxFrameRate: 30
-					},
-					optional: [
-						{minFrameRate: 10}
-					]
-				}
-			};
-			// Add capture id if available
-			if(screen_capture_id)
-				screen_constraints.video.mandatory.chromeMediaSourceId = screen_capture_id;
-			// Send constrains and start shaing
-			me.screen(screen_constraints);
-		});
-	} else if(navigator.browser && navigator.browser.application=="Firefox") {
-		var screen_constraints = {
-			video: {
-				mediaSource: "window", // "screen",
-				width: {max: '1920'},
-				height: {max: '1080'},
-				frameRate: {max: '10'}
-			}
-		};
-		this.screen(screen_constraints);
-	}
-
-};
-
-// Start sharing screen
-IceSIP.Sharing.prototype.screen = function(mc) {
-	var me = this;
-
-	mc = mc || {audio: false, video: false};
-
-	var conference_id = this.conference_id;
-	var id = conference_id.match(/^conference\*([0-9]+)/);
-	id = id.length ? id[1] : 'null';
-
-	var oc = {offerToReceiveAudio: false,offerToReceiveVideo: !mc.video};
-
-	var guest = window.sPrimaryAccountGUEST ? {
-		email: dataSet.get('main',['user']),
-		name: dataSet.get('main',['fullname'])
-	} : false;
-
-	var videostream = null;
-	this.session = this.sip.call('sip:'+this.conference_id,{
-		eventHandlers: {
-		//	progress: function(e) {},
-		//	sdp: function (e) {},
-			failed: function(e) {
-				// Screen shaing failed
-				me.onfailed({reason: e && e.cause ? e.cause : 'Unknown'});
-			},
-			confirmed: function(e) {
-				if(me.session && me.session._remote_identity) {
-					me.onstarted({sending: !!mc.video});
-				}
-			},
-			ended: function(e) {
-				// Screen sharing ended
-				me.onended({reason: 'Ended'});
-			}
-		},
-		mediaConstraints: mc,
-		rtcOfferConstraints: oc,
-		extraHeaders: [
-			'X-IceWarp-Conference: id='+id,
-			'X-IceWarp-Reference: callid='+me.reference
-		],
-		guest: guest
-	});
-
-	var pc = this.session._connection;
-	pc.addEventListener('addstream', function(e) {
-		if(me.session && me.session._remote_identity) {
-
-			if(e.stream.getVideoTracks()[0])
-				videostream = e.stream;
-
-			var video = false;
-			if(videostream)
-				video = IceSIP.attachVideoStream(videostream);
-
-			// Screen sharing started
-			me.onstarted({
-				receiving: !mc.video,
-				video: video
-			});
-		}
-	},false);
-
-};
-
-// End screen sharing
-IceSIP.Sharing.prototype.stop = function() {
-	this.live = false;
-
-	if (this.session)
-		this.session.terminate();
-};
-
-// Conference mute is identical to call mute
-IceSIP.Conference.prototype.mute = IceSIP.Call.prototype.mute;
-
-// Conference silence loudspeakers
-IceSIP.Conference.prototype.silence = function(quiet) {
-	if(this.session && this.session.audio)
-		this.session.audio.muted = quiet;
-};
-
-// Leave conference
-IceSIP.Conference.prototype.leave = function() {
-	if(this.sip.isConnected() && this.session) {
-		this.state = 'Ended';
-		this.sip.activity = 'Ready';
-		if(this.session.audio)
-			this.session.audio.pause();
-
-		// Terminate conference, will throw exception if already cancelled or terminated
-		try {
-			this.session.terminate();
-		} catch(e) {
-		//	console.log("Termination failed :", this.session && this.session.status);
-		}
-	}
-	IceSIP.Conference.current = null;
-};
-
-// Send text message to another participant
-IceSIP.Conference.prototype.send = function(from,message,to) {
-	var me = this;
-
-	var guest = window.sPrimaryAccountGUEST ? {
-		email: dataSet.get('main',['user']),
-		name: dataSet.get('main',['fullname'])
-	} : false;
-
-	if(this.sip.isConnected() /*&& this.sip.isRegistered()*/) {
-		this.sip.sendMessage('sip:'+this.full_id,message,{
-			eventHandlers: {
-				succeeded: function(e) {
-					me.onsent({failed: false});
-				},
-				failed: function(e) {
-					me.onsent({failed: true});
-				}
-			},
-			extraHeaders: [
-				'Content-Encoding: utf-8',
-				'X-IceWarp-Conference: id='+this.numeric_id,
-				'X-IceWarp-Message: from_id=sender,to_id='+(to||'*')+',is_private='+(!to?0:1)
-			],
-			guest: guest
-		});
-	} else
-		this.onsent({failed: true});
-};
-
-// Incoming message, called by sip when message has icewarp header (private)
-IceSIP.Conference.prototype.message = function(m) {
-	if(IceSIP.Conference.current)
-		IceSIP.Conference.current.onmessage(m);
-};
-
-IceSIP.Conference.prototype.onsent = function(e) {
-	// Abstract method
-	console.warn('No handler assigned: '+e.successful+' received');
-	// User received a message
-};
-
-IceSIP.Conference.prototype.onmessage = function(e) {
-	// Abstract method
-	console.warn('No handler assigned: '+e.text+' received');
-	// User received a message
-};
-
-IceSIP.Conference.prototype.oninfo = function(e) {
-	// Abstract method
-	console.warn('No handler assigned: '+e.message+' received');
-	// Conference information sent
-};
-
-IceSIP.Conference.prototype.onjoined = function(e) {
-	// Abstract method
-	console.warn('No handler assigned: '+e.conference+' started');
-	// User joined conference
-};
-
-IceSIP.Conference.prototype.onfailed = function(e) {
-	// Abstract method
-	console.warn('No handler assigned: '+e.conference+' failed');
-	// User could not join conference
-};
-
-IceSIP.Conference.prototype.onended = function(e) {
-	// Abstract method
-	console.warn('No handler assigned: '+e.conference+' has ended');
-	// Conference ended for this user
-};
-
 /* SIP Messages */
 
 IceSIP.Message = function(connection,content) {
@@ -1065,22 +695,17 @@ IceSIP.Message = function(connection,content) {
 
 // Send message
 IceSIP.Message.prototype.send = function(to) {
+	var me = this;
 	this.to = to;
 	this.sip.sendMessage('sip:'+to,this.content,{
 		eventHandlers: {
-			succeeded: function(e) {
+			succeeded: function() {
 				me.onsent({failed: false});
 			},
-			failed: function(e) {
+			failed: function() {
 				me.onsent({failed: true});
 			}
 		},
 		extraHeaders: []
 	});
-};
-
-IceSIP.Conference.prototype.onsent = function(e) {
-	// Abstract method
-	console.warn('No handler assigned: Message sent '+e.failed);
-	// Message sending success
 };
\ No newline at end of file
