@@ -1,6 +1,6 @@
 function telemetry(){}
 
-telemetry.prototype.__constructor = function(){
+telemetry.prototype.__constructor = async function(){
 
 	var me = this;
 
@@ -8,8 +8,7 @@ telemetry.prototype.__constructor = function(){
 	this.__pending = false;
 
 	AttachEvent(document.body, 'onmouseup', function(e){
-		var e = e || window.event,
-			elm = e.target || e.srcElement;
+		var elm = e.target;
 
 		if (elm){
 
@@ -46,7 +45,7 @@ telemetry.prototype.__constructor = function(){
 
 	
 	// store() loop executing
-	storage.library('wm_tools');
+	await storage.library('wm_tools');
 	this.tools = new wm_tools();
 	setInterval(function(){
 		me.__store();
@@ -58,7 +57,7 @@ telemetry.prototype._add = function(aIn){
 	try{
 		this.__storage[id] = this.__storage[id]?this.__storage[id]+1:1;
 	}
-	catch(r){gui._REQUEST_VARS.debug && console.log(this._name||false,r);}
+	catch(r){ console.log(this._name||false,r);}
 };
 
 	telemetry.prototype.__response = function(){
