@@ -3,7 +3,7 @@ function wm_accounts(){
 };
 
 wm_accounts.inherit(wm_generic);
-var _me = wm_accounts.prototype;
+_me = wm_accounts.prototype;
 
 //********************************************************************************
 //Základní SET funkce pro pro přidávání a update accountu
@@ -42,7 +42,7 @@ _me.add = function(aAccountInfo,sDataSet,aDataPath,aHandler){
 	}
 	else
 	//ADD
-	if (aAccountInfo['SERVER'] && aAccountInfo['USERNAME'] && aAccountInfo['PASSWORD'] && aAccountInfo['EMAIL']){
+	if (aAccountInfo['SERVER'] && aAccountInfo['USERNAME'] && (aAccountInfo['PASSWORD'] || aAccountInfo.DIGEST) && aAccountInfo['EMAIL']){
 		switch(aAccountInfo['PROTOCOL']){
 			case 'imap':
 			case 'pop3':
@@ -63,6 +63,7 @@ _me.add = function(aAccountInfo,sDataSet,aDataPath,aHandler){
 					"SERVER":[{"VALUE":aAccountInfo['SERVER']}],
 					"USERNAME":[{"VALUE":aAccountInfo['USERNAME']}],
 					"PASSWORD":[{"VALUE":aAccountInfo['PASSWORD']}],
+					"DIGEST":[{"VALUE":aAccountInfo['DIGEST']}],
 					"PORT":[{"VALUE":aAccountInfo['PORT']}],
 					"EMAIL":[{"VALUE":aAccountInfo['EMAIL']}],
 					"DESCRIPTION":[{"VALUE":aAccountInfo['DESCRIPTION']}]}]};
@@ -70,21 +71,7 @@ _me.add = function(aAccountInfo,sDataSet,aDataPath,aHandler){
 	else
 		return false;
 
-	//Pracujeme synchronně či asynchronně?
-	if (sDataSet || aHandler){
-		//AJAX...
-		this.create_iq(aRequest,[this,'response',['add',sDataSet,aDataPath,aHandler]],'','set', uid);
-		return true;
-	}
-	else{
-		var aResponse = this.create_iq(aRequest,'','','set');
-		try{
-			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE']=='result')
-				return true;
-		}
-		catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
-		return false;
-	}
+	this.create_iq(aRequest,[this,'response',['add',sDataSet,aDataPath,aHandler]],'','set', uid);
 };
 
 /*
@@ -131,6 +118,24 @@ _me.unsubscribe = function(aAccountInfo,aHandler){
 	}
 };
 
+_me.invite = function(aAccountInfo, aHandler) {
+	var aRequest = {
+		ACCOUNT: [{
+			ATTRIBUTES: {
+				ACTION: 'invite',
+				UID: sPrimaryAccount
+			},
+			INVITE: aAccountInfo.map(function(account) {
+				return {
+					VALUE: account.email
+				};
+			})
+		}]
+	};
+
+	this.create_iq(aRequest, [this, 'response', ['invite', '', '', aHandler]], '', 'set');
+}
+
 /*
 
 <--
@@ -154,15 +159,35 @@ _me.unsubscribe = function(aAccountInfo,aHandler){
 </iq>
 
 */
-_me.test = function(aAccountInfo,aHandler){
+_me.test = function(aAccountInfo, aHandler) {
+	if (!sPrimaryAccountRSA) {
+		return this._test(aAccountInfo, aHandler);
+	}
+
+	this.__authRSA(function(digest) {
+		aAccountInfo.DIGEST = digest;
+		delete aAccountInfo.PASSWORD;
+		this._test(aAccountInfo, aHandler);
+	}.bind(this), aAccountInfo);
+};
+
+_me._test = function(aAccountInfo,aHandler){
 
 	var aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"ACTION":"test"}}]};
 	var aAccount = aRequest.ACCOUNT[0];
 		aAccount.EMAIL = [{VALUE:aAccountInfo.EMAIL}];
 		aAccount.USERNAME = [{VALUE:aAccountInfo.USERNAME}];
 
-		if (aAccountInfo.PASSWORD)
+		if (aAccountInfo.DIGEST) {
+			aAccount.DIGEST = [{
+				VALUE: aAccountInfo.DIGEST
+			}];
+			aAccount.AUTHTYPE = [{
+				VALUE: 1
+			}];
+		} else if (aAccountInfo.PASSWORD) {
 			aAccount.PASSWORD = [{VALUE:aAccountInfo.PASSWORD}];
+		}
 
 		aAccount.SERVER = [{VALUE:aAccountInfo.SERVER}];
 		aAccount.PORT = [{VALUE:aAccountInfo.PORT}];
@@ -173,59 +198,81 @@ _me.test = function(aAccountInfo,aHandler){
 
 		aAccount.DESCRIPTION = [{VALUE:aAccountInfo.DESCRIPTION}];
 
-	if (!aHandler){
-		var aResponse = this.create_iq(aRequest,'','','set');
-		try{
-			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE']=='result')
-				return true;
-		}
-		catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
-		return false;
-	}
-	else{
-		//AJAX...
-		this.create_iq(aRequest,[this,'response',['test','','',aHandler]],'','set');
-		return true;
-	}
+	this.create_iq(aRequest,[this,'response',['test','','',aHandler]],'','set');
 };
 
-/*
-Self sign up
+_me.__authRSA = async function(callback, aAccountInfo) {
+	await storage.library('prng4', 'rsa');
+	await storage.library('rng', 'rsa');
+	await storage.library('jsbn', 'rsa');
+	await storage.library('rsa', 'rsa');
 
-<iq type="set">
-<query xmlns="webmail:iq:accounts">
-<account action="signup">
-<username>testik</username>
-<password>testik</password>
-[<fullname>]
-[<alternative>]
-<email>testik@icewarpdemo.com</email>
-<captcha uid="{captchaUID}">TRLH BH78</captcha>
-</account>
-</query>
-</iq>
-*/
-_me._signup = function(sUser,sPass,sFull,sDomain,sAlt,csid,captcha){
+	icewarpapi.send({
+		commandname: 'getauthchallenge',
+		commandparams: {
+			authtype: 1
+		}
+	}, {
+		success: function (auth) {
+			var RSA = new RSAKey();
+			RSA.setPublic(auth.hashid, '10001');
+			var digest = RSA.encrypt('p=' + encodeURIComponent(aAccountInfo.PASSWORD).replace(/!/g, '%21').replace(/'/g, '%27').replace(/\(/g, '%28').replace(/\)/g, '%29').replace(/\*/g, '%2A').replace(/%20/g, '+') + '&t=' + auth.timestamp);
 
-	aRequest = {"ACCOUNT":[{
-	ATTRIBUTES:{ACTION:'signup'},
-	USERNAME:[{VALUE:sUser}],
-	PASSWORD:[{VALUE:sPass}],
-	FULLNAME:[{VALUE:sFull}],
-	ALTERNATIVE:[{VALUE:sAlt}],
-	EMAIL:[{VALUE:sUser+'@'+sDomain}],
-	CAPTCHA:[{ATTRIBUTES:{UID:csid},VALUE:captcha}]
-	}]};
+			callback(digest);
+		}
+	});
+};
 
-	var aResponse = (this.create_iq(aRequest,'','','set')).IQ[0];
+_me.__getLicense = function(callback) {
+	if (dataSet.get('main', ['license']) !== void 0) {
+		return callback();
+	}
 
-	if (aResponse.ATTRIBUTES.TYPE == 'result')
-		return {'uid':true};
-	else
-	if (aResponse.ATTRIBUTES.TYPE == 'error')
-		return {'uid':aResponse.ERROR[0].ATTRIBUTES.UID,'value':aResponse.ERROR[0].VALUE};
-	else
-		return false;
+	if (dataSet.get('main', ['license_type']) !== void 0) {
+		return callback();
+	}
+
+	icewarpapi.authenticate({
+		success: function() {
+			icewarpapi.send({
+				commandname: 'GetServerProperties',
+				commandparams: [{
+					serverpropertylist: [{
+						item: [{ propname: 'c_license_type' }]
+					}, {
+						item: [{ propname: 'c_license' }]
+					}]
+				}]
+			}, {
+				success: function (response) {
+					for (var i in response.item) {
+						var val = (response.item[i].propertyval || {}).val;
+						var name = (response.item[i].apiproperty || {}).propname;
+						switch (name) {
+							case 'c_license':
+								var license = {};
+								try {
+									license = XMLTools.Str2Arr(val.unentityify()).LICENSE[0];
+								} catch {
+									//
+								}
+								dataSet.add('main', ['license'], license);
+								break;
+
+							case 'c_license_type':
+								dataSet.add('main', ['license_type'], val);
+						}
+					}
+					
+					callback();
+				},
+				error: this.__errorHandler,
+				context: this
+			});
+		},
+		error: this.__errorHandler,
+		context: this
+	})
 };
 
 //********************************************************************************
@@ -238,7 +285,7 @@ _me.license = function(aHandler){
 	return true;
 };
 
-_me.list = function(aAccountInfo,sDataSet,aDataPath,sParam,aHandler){
+_me.list = async function(aAccountInfo,sDataSet,aDataPath,sParam,aHandler){
 	var aRequest = {};
 
 	if (aAccountInfo && aAccountInfo.aid)
@@ -253,7 +300,7 @@ _me.list = function(aAccountInfo,sDataSet,aDataPath,sParam,aHandler){
 		return true;
 	}
 	else
-		return this.account_sort(this.parse(this.create_iq(aRequest)));
+		return this.account_sort(this.parse(await this.create_iq(aRequest)));
 };
 	// sort accounts (main on top and rss on bottom)
 	_me.account_sort = function(arr){
@@ -277,14 +324,14 @@ _me.list = function(aAccountInfo,sDataSet,aDataPath,sParam,aHandler){
 		return main;
 	};
 
-_me.refresh = function(aAccountInfo,sDataSet,aDataPath,aHandler){
+_me.refresh = async function(aAccountInfo,sDataSet,aDataPath,aHandler){
 	//Máme definované account_id?
 	if (!aAccountInfo['aid']) return false;
 
 	//Vytvoříme XML dotaz pro refresh accountů
 	// <ACCOUNT UID='aid' ACTION='refresh'/>
 	var sType = 'refresh',
-		aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"ACTION":"refresh","UID":aAccountInfo['aid']}}]};
+		aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"ACTION":"refresh","UID":aAccountInfo['aid'], FORCE: !!aAccountInfo.force}}]};
 
 	if (aHandler || sDataSet){
 		this.create_iq(aRequest,[this,'response',[sType,sDataSet,aDataPath,aHandler]],'','set',aAccountInfo['aid'],'',true);
@@ -292,11 +339,11 @@ _me.refresh = function(aAccountInfo,sDataSet,aDataPath,aHandler){
 	}
 	//Pracujeme synchronně či asynchronně?
 	else{
-		var result = WMFolders.parse(this.create_iq(aRequest,'','','set'));
+		var result = WMFolders.parse(await this.create_iq(aRequest,'','','set'));
 
 		//map folders
 		if (result[sPrimaryAccount])
-			this.__mapFolders(result[sPrimaryAccount]);
+			await this.__mapFolders(result[sPrimaryAccount]);
 
 		return result;
 	}
@@ -309,7 +356,7 @@ _me.refresh = function(aAccountInfo,sDataSet,aDataPath,aHandler){
 //  povinné:   'aid':account_id
 //********************************************************************************
 
-_me.remove = function(aAccountInfo,sDataSet,aDataPath,sParam){
+_me.remove = async function(aAccountInfo,sDataSet,aDataPath,sParam){
 	//Máme definované account_id?
 	if (!aAccountInfo['aid'])
 		return false;
@@ -320,12 +367,12 @@ _me.remove = function(aAccountInfo,sDataSet,aDataPath,sParam){
 
 	//Pracujeme synchronně či asynchronně?
 	if (!sDataSet){
-		var aResponse = this.create_iq(aRequest,'','','set');
+		var aResponse = await this.create_iq(aRequest,'','','set');
 		try{
 			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE']=='result')
 			return true;
 		}
-		catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+		catch(e){ console.log(this._name||false,e);}
 		return false;
 	}
 	else{
@@ -339,7 +386,7 @@ _me.remove = function(aAccountInfo,sDataSet,aDataPath,sParam){
 //Základní SET funkce pro vyvolání synchronizace accountů na serveru
 //********************************************************************************
 
-_me.synchronize = function(aFoldersInfo,sDataSet,aDataPath){
+_me.synchronize = async function(aFoldersInfo,sDataSet,aDataPath){
 	//Máme definované account_id?
 	if (!aFoldersInfo['aid'])
 		return false;
@@ -350,12 +397,13 @@ _me.synchronize = function(aFoldersInfo,sDataSet,aDataPath){
 
 	//Pracujeme synchronně či asynchronně?
 	if (!sDataSet){
-		var aResponse = this.create_iq(aRequest,'','','set');
+		var aResponse = await this.create_iq(aRequest,'','','set');
 		try{
 			if (aResponse['Array']['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result')
 			return true;
+		} catch {
+			//
 		}
-		catch(e) {}
 
 		return false;
 	}
@@ -371,7 +419,7 @@ _me.synchronize = function(aFoldersInfo,sDataSet,aDataPath){
 // následovaná GET funkcí listující accounty
 //********************************************************************************
 
-_me.synclist = function(aFoldersInfo,sDataSet,aDataPath){
+_me.synclist = async function(aFoldersInfo,sDataSet,aDataPath){
 	//Máme definované account_id?
 	if (!aFoldersInfo['aid']) return false;
 
@@ -381,13 +429,14 @@ _me.synclist = function(aFoldersInfo,sDataSet,aDataPath){
 
 	//Pracujeme synchronně či asynchronně?
 	if (!sDataSet){
-		var aResponse = this.create_iq(aRequest,'','','set');
+		var aResponse = await this.create_iq(aRequest,'','','set');
 		try{
 			//Proběhla-li synchronizace úspěšně, budeme foldery v accountu listovat
 			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result')
-				(new wm_folders).list({'aid':aFoldersInfo['aid']});
+				await WMFolders.list({'aid':aFoldersInfo['aid']});
+		} catch {
+			//
 		}
-		catch(e) {}
 
 		return false;
 	}
@@ -416,7 +465,7 @@ _me.action = function(aInfo, sType, aHandler){
 //Pomocná funkce realizující asynchronní obsluhu odpovědi
 //********************************************************************************
 
-_me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aHandler){
+_me.response = async function(aResponse,sMethodName,sDataSet,aDataPath,aHandler){
 	var aXMLResponse = aResponse['Array'],
 		aIQAttribute = aXMLResponse['IQ'][0]['ATTRIBUTES'];
 
@@ -443,7 +492,7 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aHandler){
 			if (aXMLResponse['IQ'][0]['ATTRIBUTES']['TYPE']=='result')
 				out = true;
 		}
-		catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+		catch(e){ console.log(this._name||false,e);}
 
 		executeCallbackFunction(aHandler,out);
 		return;
@@ -463,7 +512,7 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aHandler){
 			if (aIQAttribute['UID']){
 				var aAccounts = dataSet.get(sDataSet,aDataPath);
 				dataSet.remove(aIQAttribute['UID'],'',true);
-				for(sAccId in aAccounts)
+				for(var sAccId in aAccounts)
 					WMFolders.list({'aid':sAccId},aIQAttribute['UID']);
 			}
 		}
@@ -476,8 +525,7 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aHandler){
 			try{
 				att = aXMLResponse.IQ[0].ERROR[0].ATTRIBUTES.UID;
 				str = aXMLResponse.IQ[0].ERROR[0].VALUE;
-			}
-			catch(e){
+			} catch {
 				str = att = '';
 			}
 
@@ -508,18 +556,22 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aHandler){
 					dataSet.add('main',['SMS_SENT'],aAccountAttribute.SMS_SENT);
 				}
 
-				//Inbox Recent
-				var	iRecent_old = +(dataSet.get(sDataSet,[aIQAttribute['UID'],'INBOX','RECENT']) || 0);
+				// Recents
+				var oRecent = {}, i;
+				var aFolders = dataSet.get(sDataSet, [aIQAttribute['UID']]);
+				for (i in aFolders) {
+					oRecent[i] = aFolders[i].RECENT;
+				}
 
 				if (aIQAttribute['UID'] == sPrimaryAccount)
-					this.__mapFolders(aParsedData[aIQAttribute['UID']]);
+					await this.__mapFolders(aParsedData[aIQAttribute['UID']]);
 
 				this.__mergeDataset(aIQAttribute['UID'], aParsedData[aIQAttribute['UID']]);
 
 				// single folder refresh (full update)
 				if (sMethodName === 'synchronize') { // && Object.keys(aParsedData[aIQAttribute['UID']]).length === 1
-					for(var i in aParsedData[aIQAttribute['UID']]) {
-						if (aParsedData[aIQAttribute['UID']].hasOwnProperty(i)){
+					for(i in aParsedData[aIQAttribute['UID']]) {
+						if (Object.prototype.hasOwnProperty.call(aParsedData[aIQAttribute['UID']], i)){
 							dataSet.add(sDataSet, [aIQAttribute['UID'], i], aParsedData[aIQAttribute['UID']][i]);
 						}
 					}
@@ -529,27 +581,41 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aHandler){
 					dataSet.add(sDataSet, [aIQAttribute['UID']], aParsedData[aIQAttribute['UID']]);
 				}
 
-				if (iRecent_old < dataSet.get(sDataSet,[aIQAttribute['UID'],'INBOX','RECENT'])){
-
-					//change title
-					if (gui.frm_main && gui.frm_main.title)
-						gui.frm_main.title._add(getLang('TITLE::NEW_EMAIL'),10);
-
-					//show notifier
-					if (gui.notifier)
-						gui.notifier._value({type: 'new_email', args: [sPrimaryAccount]});
-
-					//new email sound
-					storage.library('gw_others');
-					if (gui.frm_main && gui.frm_main.sound && parseInt(GWOthers.getItem('MAIL_SETTINGS_GENERAL','sound_notify'))>0)
-						gui.frm_main.sound._play('mail');
-				}
-				else
-				if (iRecent_old > 0 && (dataSet.get(sDataSet,[aIQAttribute['UID'],'INBOX','RECENT']) || 0) == 0) {
-					//reset title
-					if (gui.frm_main && gui.frm_main.title)
-						gui.frm_main.title._reset();
+				// get new data
+				aFolders = dataSet.get(sDataSet, [aIQAttribute['UID']]);
+				for (i in aFolders) {
+					if (aFolders[i].TYPE !== 'M' || ((i !== 'INBOX') && ((GWOthers.getItem('MAIL_SETTINGS_GENERAL', 'secondary_notify') == 0) || !aFolders[i].SYNC || aFolders[i].DEFAULT || aFolders[i].SPAM || (aFolders[i.split('/')[0]] || {}).ARCHIVE))) {
+						continue;
+					}
+					if (oRecent[i] < aFolders[i].RECENT) {
+						if (!window.skipNewMailNotification) {
+							// change title
+							if (gui.frm_main && gui.frm_main.title) {
+								gui.frm_main.title._add(getLang('TITLE::NEW_EMAIL'), 10);
+							}
+		
+							//show notifier
+							var emails = dataSet.get('temp', ['new_mail', i]) || [];
+							if (gui.notifier && emails.length) {
+								gui.notifier._value({type: 'new_email', args: {aid: aIQAttribute['UID'], fid: i, emails: emails}});
+								dataSet.add('temp', ['new_mail', i], [], true);
+							}
+		
+							//new email sound
+							await storage.library('gw_others');
+							if (gui.frm_main && gui.frm_main.sound && parseInt(GWOthers.getItem('MAIL_SETTINGS_GENERAL','sound_notify'))>0) {
+								gui.frm_main.sound._play('mail');
+							}
+						}
+					} else if (i === 'INBOX' && oRecent[i] > 0 && !aFolders[i].RECENT) {
+						//reset title
+						if (gui.frm_main && gui.frm_main.title) {
+							gui.frm_main.title._reset();
+						}
+					}
 				}
+
+				delete window.skipNewMailNotification;
 			}
 			else
 				dataSet.add(sDataSet, [aIQAttribute['UID']], aParsedData[aIQAttribute['UID']]);
@@ -565,7 +631,7 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aHandler){
 				this.list(sDataSet,aDataPath,aIQAttribute['UID']);
 			return true;
 		}
-		catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+		catch(e){ console.log(this._name||false,e);}
 
 		return false;
 
@@ -579,14 +645,14 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aHandler){
 			}
 			else
 			if(aIQAttribute['TYPE'] == 'result' && typeof aHandler == 'object'){
-				executeCallbackFunction(aHandler);
+				executeCallbackFunction(aHandler, aIQAttribute['TYPE'] == 'result');
 				return true;
 			}
 			else
 			if (aIQAttribute['TYPE'] == 'error' && typeof aHandler == 'object')
 				executeCallbackFunction(aHandler,aXMLResponse['IQ'][0]['ERROR'][0]['ATTRIBUTES']['UID'],aXMLResponse['IQ'][0]['ERROR'][0]['VALUE']);
 		}
-		catch(r){gui._REQUEST_VARS.debug && console.log(this._name||false,r);}
+		catch(r){ console.log(this._name||false,r);}
 		return false;
 
 	case 'action':
@@ -594,13 +660,13 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aHandler){
 	}
 };
 
-_me.__mapFolders = function(aFolders){
+_me.__mapFolders = async function(aFolders){
 
 	if (this.__mapped == sPrimaryAccount) return;
 
 	var test = {},out = {};
 	aFolders = aFolders || {};
-	storage.library('gw_others');
+	await storage.library('gw_others');
 
 	var sTrashFolder = Path.split(GWOthers.getItem('DEFAULT_FOLDERS', 'trash'))[1];
 	if (!aFolders[sTrashFolder])
@@ -684,7 +750,11 @@ _me.parse = function(aData){
 						if (aFrame[nAccNum][sTag][0] && (aRights = aFrame[nAccNum][sTag][0].RIGHT)){
 							aResultFrame.ACL = {};
 							for(var i in aRights)
-								aResultFrame.ACL[aRights[i].ATTRIBUTES.EMAIL] = aRights[i].VALUE?aRights[i].VALUE.split(''):[];
+								aResultFrame.ACL[aRights[i].ATTRIBUTES.EMAIL] = {
+									name: aRights[i].ATTRIBUTES.NAME,
+									external: aRights[i].ATTRIBUTES.EXTERNAL,
+									rights: aRights[i].VALUE?aRights[i].VALUE.split(''):[]
+								};
 						}
 						aRights = null;
 				}
@@ -704,6 +774,19 @@ _me.parse = function(aData){
 				dataSet.add('main',['keep_deleted_items_force_expiration'],aResultFrame['KEEPDELETEDITEMSFORCEEXPIRATION'],true);
 				dataSet.add('main',['device_id'],aResultFrame['DEVICE_ID'],true);
 				dataSet.add('main',['server_id'],aResultFrame['ICEWARP_SERVER_ID'],true);
+				dataSet.add('main',['MBOX_QUOTA'],aResultFrame.MBOX_QUOTA,true);
+				dataSet.add('main',['MBOX_USAGE'],aResultFrame.MBOX_USAGE);
+				dataSet.add('main',['version'],aResultFrame['VERSION'],true);
+				dataSet.add('main',['nice_version'],aResultFrame['NICE_VERSION'],true);
+
+				var iwma = aResultFrame.ICEWARPMOBILEAPP_URL;
+				if (iwma) {
+					iwma = iwma.replace('1252689833', '1591648135');
+					iwma = iwma.replace(/teamchatMobile/g, 'servicesHub');
+					iwma = iwma.replace(/teamchatmobile.page.link/g, 'icewarpserviceshub.page.link');
+					iwma = iwma.replace(/com.icewarp.teamchatmobile/g, 'com.icewarp.serviceshub');
+					dataSet.add('main',['ICEWARPMOBILEAPP_URL'], iwma, true);
+				}
 
 				window.sPrimaryAccount = aFrame[nAccNum]['ATTRIBUTES']['UID'];
 				window.sPrimaryAccountTeamchatToken = window.sPrimaryAccountTeamchatToken || false;
@@ -714,13 +797,16 @@ _me.parse = function(aData){
 				window.sPrimaryAccountWebAdmin = aFrame[nAccNum]['ATTRIBUTES']['WEBADMIN_URL'];
 				window.sPrimaryAccountClient = aFrame[nAccNum]['ATTRIBUTES']['CLIENT_URL'];
 				window.sPrimaryAccountGW = aResultFrame['GW'];
+				window.sPrimaryAccountGWSUPPORT = aFrame[nAccNum]['ATTRIBUTES']['GW_SUPPORT'] === 'true';
 				window.sPrimaryAccountGWID = aFrame[nAccNum]['ATTRIBUTES']['GW_OWNERID'];
 				window.sPrimaryAccountGWTRASH = aFrame[nAccNum]['ATTRIBUTES']['GWTRASH_SUPPORT'];
 				window.sPrimaryAccountIM = aResultFrame['IM_SUPPORT'];
 				window.sPrimaryAccountIMHISTORY = aResultFrame['IM_HISTORY_SUPPORT'];
 				window.sPrimaryAccountSIP = aResultFrame['SIP_SUPPORT'];
+				window.sPrimaryAccountRSA = true;//aResultFrame['RSA_SUPPORT'];
 				window.sPrimaryAccount2F = aResultFrame['TWO_FACTOR_SUPPORT'];
 				window.sPrimaryAccount2FE = aResultFrame['TWO_FACTOR_ENABLED'];
+				window.sPrimaryAccountRECALL = aResultFrame['RECALL_MESSAGES_ENABLED'];
 				window.sPrimaryAccountCHAT = aResultFrame['GROUPCHAT_SUPPORT'];
 				window.sPrimaryAccountGUEST = aResultFrame['GUEST_ACCOUNT'];
 				window.sPrimaryAccountACTIVESYNC = aResultFrame['ACTIVESYNC_SUPPORT'];
@@ -746,14 +832,15 @@ _me.parse = function(aData){
 				window.sPrimaryAccountOLDWC = aResultFrame['OLD_SUPPORT'];
 
 				window.sPrimaryTelemetry = aResultFrame['TELEMETRY_SUPPORT'];
+
+				window.aPrimaryAttributes = Object.assign({}, aResultFrame, aFrame[nAccNum]['ATTRIBUTES']);
 			}
 
 			aResult[aResultFrame['UID']] = aResultFrame;
 		}
 
 		return aResult;
-	}
-	catch(e){
+	} catch {
 		return false;
 	}
 };
