@@ -1,6 +1,6 @@
 function wm_domains(){};
 
-var _me = wm_domains.prototype;
+_me = wm_domains.prototype;
 
 _me.get = function(sDataSet,bSet){
 	if (!sDataSet) return;
