@@ -3,7 +3,7 @@ function wm_folders(){
 };
 
 wm_folders.inherit(wm_generic);
-var _me = wm_folders.prototype;
+_me = wm_folders.prototype;
 
 
 wm_folders.aux = {
@@ -40,14 +40,14 @@ wm_folders.aux = {
 //  nepovinné: 'fid':folder_id, 'type':typ folderu
 //********************************************************************************
 
-_me.add = function(aFolderInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
+_me.add = async function(aFolderInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
 {
 
 	//Máme definované account_id a jméno folderu?
 	if (!aFolderInfo['aid'])
 		return false;
 
-	var aRequest;
+	var aRequest, i;
 
 	//Budeme složku editovat či přidávat?
 	if (aFolderInfo['fid']){
@@ -74,8 +74,11 @@ _me.add = function(aFolderInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
 		aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"ACTION":"add"},"TYPE":[{"VALUE":aFolderInfo['type']}]}]}]};
 	}
 
-	if (aFolderInfo['name'] && aFolderInfo['fid']!=aFolderInfo['name'])
-		aRequest.ACCOUNT[0].FOLDER[0].NAME = [{'VALUE':aFolderInfo['name']}];
+	if (aFolderInfo.name)
+		aRequest.ACCOUNT[0].FOLDER[0].NAME = [{VALUE: aFolderInfo.name}];
+
+	if (aFolderInfo.folderName)
+		aRequest.ACCOUNT[0].FOLDER[0].FOLDERNAME = [{VALUE: aFolderInfo.folderName}];
 
 	if (aFolderInfo['default'])
 		aRequest.ACCOUNT[0].FOLDER[0].DEFAULT = [{'VALUE':aFolderInfo['default']}];
@@ -93,12 +96,12 @@ _me.add = function(aFolderInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
 
 		if (aFolderInfo.virtual.sharetype)
 			aRequest.ACCOUNT[0].FOLDER[0].VIRTUAL = [{SHARETYPE:[{'VALUE':aFolderInfo.virtual.sharetype}]}];
-		else
+		// else
 		if (aFolderInfo.virtual.folders){
 			aRequest.ACCOUNT[0].FOLDER[0].VIRTUAL = [{FOLDERS:[{'FOLDER':[]}]}];
 
 			var tmp;
-			for(var i in aFolderInfo.virtual.folders){
+			for(i in aFolderInfo.virtual.folders){
 				tmp = {'VALUE':i};
 				if (aFolderInfo.virtual.folders[i])
 					tmp.ATTRIBUTES = {'PRIMARY':"true"};
@@ -115,7 +118,7 @@ _me.add = function(aFolderInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
 		//<right email="{group_email}">{rights}</right>
 
 		var aGrp = [];
-		for (var i in aFolderInfo['acl'])
+		for (i in aFolderInfo['acl'])
 			aGrp.push({VALUE:aFolderInfo['acl'][i].join(''),ATTRIBUTES:{EMAIL:i}});
 
 		if (aGrp.length)
@@ -126,7 +129,7 @@ _me.add = function(aFolderInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
 
 	if (Is.Array(aFolderInfo['channel'])){
 		aRequest.ACCOUNT[0].FOLDER[0].CHANNELS = [{CHANNEL:[]}];
-		for(var i in aFolderInfo['channel'])
+		for(i in aFolderInfo['channel'])
 			if (aFolderInfo['channel'][i])
 				aRequest.ACCOUNT[0].FOLDER[0].CHANNELS[0].CHANNEL.push({"VALUE":aFolderInfo['channel'][i]});
 	}
@@ -138,13 +141,13 @@ _me.add = function(aFolderInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
 		return true;
 	}
 	else{
-		var aResponse = this.create_iq(aRequest,'','','set');
+		var aResponse = await this.create_iq(aRequest,'','','set');
 
 		try{
 			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result')
 				return true;
 		}
-		catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+		catch(e){ console.log(this._name||false,e);}
 
 		return false;
 	}
@@ -157,7 +160,7 @@ _me.add = function(aFolderInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
 //  povinné:   'aid':account_id
 //********************************************************************************
 
-_me.list = function(aFoldersInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
+_me.list = async function(aFoldersInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
 {
 	//Máme definované account_id?
 	if (!aFoldersInfo.aid)
@@ -165,7 +168,7 @@ _me.list = function(aFoldersInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
 
 	//Vytvoříme XML dotaz pro listování folderu v accountu
 	// <ACCOUNT UID='aid'/>
-	var aRequest = { ACCOUNT:[{ ATTRIBUTES: { UID: aFoldersInfo.aid }}]};
+	var aRequest = { ACCOUNT:[{ ATTRIBUTES: { UID: aFoldersInfo.aid, FORCE: !!aFoldersInfo.force}}]};
 	var folder;
 
 	if (aFoldersInfo.fid) {
@@ -184,7 +187,7 @@ _me.list = function(aFoldersInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
 
 	//Pracujeme synchronně či asynchronně?
 	if (!sDataSet && !aHandler)
-		return this.parse(this.create_iq(aRequest));
+		return this.parse(await this.create_iq(aRequest));
 	else{
 		//AJAX...
 		this.create_iq(aRequest,[this,'response',['list',sDataSet,aDataPath,'',aHandler, aErrorHandler]],'','get',aFoldersInfo['aid']);
@@ -199,15 +202,59 @@ _me.list = function(aFoldersInfo,sDataSet,aDataPath,aHandler,aErrorHandler)
  **/
 _me.getType = function (aFolderInfo){
     var aid = aFolderInfo.aid || aFolderInfo[0],
-    	fid = aFolderInfo.fid || aFolderInfo[1];
+    	fid = Path.slash(aFolderInfo.fid || aFolderInfo[1] || '');
 
-	if (aid === sPrimaryAccount && fid === '__@@VIRTUAL@@__/__@@MEETINGS@@__')
-		return 'W';
+	if (!Is.Defined(fid))
+		return 'X';
+
+	if (aid === sPrimaryAccount){
+		if (fid.match(/__@@VIRTUAL@@__\/.*#\w$/)) {
+			return fid.slice(-1);
+		}
+		var subtree;
+		if (subtree = (fid.match(/__@@VIRTUAL@@__\/@@SUBTREE@@\/(.*)/) || [])[1]) {
+			return dataSet.get('folders', [sPrimaryAccount, subtree, 'TYPE']);
+		}
+
+		switch(fid){
+			case '__@@VIRTUAL@@__/__@@PRIVATE_ROOMS@@__':
+				return 'Y';
+			case '__@@VIRTUAL@@__/__@@MEETINGS@@__':
+				return 'W';
+			case '__@@VIRTUAL@@__/__@@DASHBOARD@@__':
+				return 'D';
+			case '__@@VIRTUAL@@__/__@@NOTES@@__':
+				return 'N';
+			case '__@@VIRTUAL@@__/__@@EVENTS@@__':
+				return 'E';
+			case '__@@THREADS@@__':
+				return 'I';
+			case '__@@ARCHIVE@@__':
+			case '__@@SNOOZED@@__':
+				return 'M';
+			case '__@@UPLOAD@@__':
+				return 'F';
+			case '__@@ADDRESSBOOK@@__':
+			case '__@@mycard@@__':
+			case '@@mycard@@':
+				return 'C';
+		}
+	}
 	else
 	if (aid === '@@alfresco@@')
 		return 'K';
 
-	return dataSet.get('folders',[aid,fid,'TYPE']) || 'X';
+	if (aFolderInfo.fid && aFolderInfo.fid.indexOf(sPrimaryAccountSPREFIX) === 0 && !~aFolderInfo.fid.indexOf('/')) {
+		return 'VA';
+	}
+
+	var sType = dataSet.get('folders',[aid,fid,'TYPE']) || 'X';
+
+	//TeamChat thread path
+	if (sType == 'X' && ~fid.indexOf('/TeamChat/') && dataSet.get('folders',[aid,Path.basedir(fid),'TYPE']) == 'I')
+		return 'I';
+
+	return sType;
 };
 
 /**
@@ -380,7 +427,7 @@ _me.empty = function(aFolderInfo,sDataSet,aDataPath,aDestination,aHandler)
 	}
 };
 
-_me.action = function(aItemInfo,sDataSet,aDataPath,sAction,aDestination,aHandler)
+_me.action = async function(aItemInfo,sDataSet,aDataPath,sAction,aDestination,aHandler)
 {
 	//Máme definovaná account_id a folder_id?
 	if (!sAction || !aItemInfo['aid'] || !aItemInfo['fid'])
@@ -401,9 +448,15 @@ _me.action = function(aItemInfo,sDataSet,aDataPath,sAction,aDestination,aHandler
 		aRequest.ACCOUNT[0].FOLDER[0].FOLDER = [{"VALUE":aDestination['fid']}];
 	}
 
+	if (aItemInfo.filter) {
+		aRequest.ACCOUNT[0].FOLDER[0].SEARCH = [{
+			VALUE: aItemInfo.filter
+		}];
+	}
+
 	//Pracujeme synchronně či asynchronně?
 	if (!sDataSet && !aHandler){
-		var aResponse = this.create_iq(aRequest,'','','set');
+		var aResponse = await this.create_iq(aRequest,'','','set');
 		if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result')
 			return true;
 
@@ -425,7 +478,7 @@ _me.save_folder = function(aItemsInfo,aHandler)
 
 	var aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aItemsInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aItemsInfo['fid'],"ACTION":'save_items'}}]}]};
 
-	this.create_iq(aRequest,[this,'response',['save_folder','','','','',aHandler]],'','set');
+	this.create_iq(aRequest,[this,'response',['save_folder','','','',aHandler]],'','set');
 	return true;
 };
 
@@ -434,7 +487,7 @@ _me.save_folder = function(aItemsInfo,aHandler)
  * @IQ Example: <folder uid="{uid}" action="edit"><subscribed>{0|1}</subscribed></folder>
  **/
 _me.subscribe = function(aFolderInfo,sDataSet,aDataPath,bSync, aHandler){
-	var sDataSet = sDataSet || 'folders';
+	sDataSet = sDataSet || 'folders';
 	var aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":"edit"},"SUBSCRIBED":[{"VALUE":bSync?1:0}]}]}]};
 
 	//AJAX...
@@ -471,7 +524,7 @@ _me.sync = function(aFoldersInfo, sDataSet, aDataPath, aHandler){
 	this.create_iq(aRequest, [WMAccounts, 'response', ['synchronize', sDataSet, aDataPath, aHandler]], '', 'set',aFoldersInfo.aid);
 };
 
-_me.markItemsRead = function(aFolderInfo,sDataSet,aDataPath,bRead)
+_me.markItemsRead = async function(aFolderInfo,sDataSet,aDataPath,bRead)
 {
 	//Máme definovaná account_id a folder_id?
 	if (!aFolderInfo['aid'] || !aFolderInfo['fid'])
@@ -483,15 +536,21 @@ _me.markItemsRead = function(aFolderInfo,sDataSet,aDataPath,bRead)
 	var aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":sType}}]}]};
 	aFolderInfo['bRead'] = bRead;
 
+	if (aFolderInfo.filter) {
+		aRequest.ACCOUNT[0].FOLDER[0].SEARCH = [{
+			VALUE: aFolderInfo.filter
+		}];
+	}
+
 	//Pracujeme synchronně či asynchronně?
 	if (!sDataSet){
-		var aResponse = this.create_iq(aRequest,'','','set');
+		var aResponse = await this.create_iq(aRequest,'','','set');
 
 		try{
 			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result')
 			return true;
 		}
-		catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+		catch(e){ console.log(this._name||false,e);}
 
 		return false;
 	}
@@ -506,10 +565,11 @@ _me.markItemsRead = function(aFolderInfo,sDataSet,aDataPath,bRead)
 //Pomocná funkce realizující asynchronní obsluhu odpovědi
 //********************************************************************************
 
-_me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHandler,aErrorHandler)
+_me.response = async function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHandler,aErrorHandler)
 {
 	var aXMLResponse = aResponse['Array'];
 	var aIQAttribute = aXMLResponse['IQ'][0]['ATTRIBUTES'];
+	var tmp, aFolder, srt, aDataSet, i, aItems, sAccId, sFolId;
 
 	//pri IMAP erroru
 	try{
@@ -520,7 +580,7 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 				att = aXMLResponse.IQ[0].ERROR[0].ATTRIBUTES.UID;
 				str = aXMLResponse.IQ[0].ERROR[0].VALUE;
 			}
-			catch(e){
+			catch {
 				str = att = '';
 			}
 
@@ -552,7 +612,20 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 			}
 		}
 	}
-	catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+	catch(e){ console.log(this._name||false,e);}
+
+	try {
+		aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0].ATTRIBUTES.UID = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0].ATTRIBUTES.UID.replace(/^\//, '');
+	} catch {
+		//
+	}
+	try {
+		if (aFolderInfo) {
+			aFolderInfo.fid = (aFolderInfo.fid || '').replace(/^\//, '');
+		}
+	} catch {
+		//
+	}
 
 	switch(sMethodName){
     case 'save_folder':
@@ -564,14 +637,14 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 					'fullpath':aXMLResponse.IQ[0].RESULT[0].ACCOUNT[0].FOLDER[0].ITEM[0].VALUES[0].FULLPATH[0].VALUE
 				};
 			}
-			catch(e){
+			catch {
 				break;
 			}
 
 			if (Is.Object(aHandler))
 			    executeCallbackFunction(aHandler,aOut);
 			else{
-				aOut['sid'] = dataSet.get('main', ['sid']);
+				aOut['dlsess'] = dataSet.get('main', ['dlsess']);
 				downloadItem(buildURL(aOut));
 			}
 		}
@@ -596,16 +669,15 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 				if (aIQAttribute['UID'])
 					this.list({'aid':aIQAttribute['UID']},sDataSet,aDataPath);
 			}
-			catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+			catch(e){ console.log(this._name||false,e);}
 			return true;
 		}
         //acl cookie update
 		else{
-
 			//TeamChat folder has different "fid" to name, must be mapped
 			if (aFolderInfo.type == 'I' && Is.String(aFolderInfo.name)){
-				var tmp = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0];
-				if (Path.basename(aFolderInfo.name) == tmp.FOLDER[0].NAME[0].VALUE)
+				tmp = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0];
+				if (!tmp.FOLDER[0].NAME || (Path.basename(aFolderInfo.name) == tmp.FOLDER[0].NAME[0].VALUE))
 					aFolderInfo.name = tmp.FOLDER[0].ATTRIBUTES.UID;
 			}
 
@@ -622,27 +694,36 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 				*/
 
 				try{
-	                var tmp = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0];
-	                if (tmp.ATTRIBUTES.UID == aFolderInfo.aid && tmp.FOLDER[0].ATTRIBUTES.UID == aFolderInfo.name)
+	                tmp = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0];
+	                if (tmp.ATTRIBUTES.UID == aFolderInfo.aid && tmp.FOLDER[0].ATTRIBUTES.UID == aFolderInfo.name) {
 						Cookie.set(['rights',aFolderInfo.aid,aFolderInfo.name], tmp.FOLDER[0].ATTRIBUTES.RIGHTS?tmp.FOLDER[0].ATTRIBUTES.RIGHTS.split(''):'');
+						if (aFolderInfo.name === '__@@VIRTUAL@@__/__@@EVENTS@@__') {
+							for (i in (aFolderInfo.virtual || {}).folders) {
+								if (aFolderInfo.virtual.folders[i]) {
+									Cookie.set(['rights', aFolderInfo.aid, i], tmp.FOLDER[0].ATTRIBUTES.RIGHTS ? tmp.FOLDER[0].ATTRIBUTES.RIGHTS.split('') : '');
+									break;
+								}
+							}
+						}
+					}
 				}
-				catch(r){gui._REQUEST_VARS.debug && console.log(this._name||false,r);}
+				catch(r){ console.log(this._name||false,r);}
 			}
 		}
 
 		//edit
 		if	(aFolderInfo['fid'] && ((aFolderInfo['name'] && aFolderInfo['fid'] != aFolderInfo['name']) || typeof aFolderInfo['channel'] != 'undefined') || aFolderInfo['default'])
 		{
-			var aFolder = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0],
-				sName = aFolderInfo['name'] || aFolderInfo['fid'],
-				aDataSet = dataSet.get(sDataSet,[aFolderInfo['aid']],true) || {};
+			aFolder = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0];
+			var sName = aFolderInfo['name'] || aFolderInfo['fid'], ds;
+			aDataSet = dataSet.get(sDataSet,[aFolderInfo['aid']],true) || {}
 
-			for(var i in aDataSet){
+			for(i in aDataSet){
 				//folder
 				if (i == aFolderInfo['fid']){
 
 					if (aFolder.RELATIVE_PATH)
-						aDataSet[i].RELATIVE_PATH = aFolder.RELATIVE_PATH[0].VALUE;
+						aDataSet[i].RELATIVE_PATH = Path.slash(aFolder.RELATIVE_PATH[0].VALUE);
 
 					if (aFolder.OWNER)
 						aDataSet[i].OWNER = aFolder.OWNER[0].VALUE;
@@ -655,50 +736,64 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 					//Teamchat has no subfolders
 					if (aDataSet[i].TYPE != 'I'){
 
-						var ds = aDataSet[i];
+						ds = aDataSet[i];
 						delete aDataSet[i];
 
 						//RSS
 						if (typeof aFolderInfo['channel']!='undefined')
 							ds.CHANNEL = aFolderInfo['channel'];
 
-						var id = aFolder.ATTRIBUTES.UID || sName;
-						aDataSet[id] = ds;
+						aDataSet[aFolder.ATTRIBUTES.UID] = ds;
 					}
 				}
 				else
 				//subfolder
 				if (i.indexOf(aFolderInfo['fid']+'/') === 0){
-					var ds = aDataSet[i];
+					ds = aDataSet[i];
 
 					delete aDataSet[i];
-					aDataSet[sName + i.substr(aFolderInfo['fid'].length)] = ds;
+					ds.RELATIVE_PATH = aFolder.ATTRIBUTES.UID + i.substr(aFolderInfo['fid'].length);
+					aDataSet[aFolder.ATTRIBUTES.UID + i.substr(aFolderInfo['fid'].length)] = ds;
 				}
 			}
 
-			if (aFolderInfo['aid'] == sPrimaryAccount && aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0].DEFAULT){
+			if (aFolderInfo['aid'] == sPrimaryAccount && ((aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0].DEFAULT || [])[0] || {}).VALUE){
 				var tmp_type = {C:'CONTACTS',E:'EVENTS',T:'TASKS',N:'NOTES',J:'JOURNAL',F:'FILES',D:'DRAFTS',H:'TRASH',S:'SENT',P:'TEMPLATES'}[aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0].DEFAULT[0].VALUE];
 
-				//remove old default
-				var def = GWOthers.getItem('DEFAULT_FOLDERS',tmp_type);
-				if (def && (def = Path.split(def)) && aDataSet[def[1]])
-					delete aDataSet[def[1]].DEFAULT;
+				if (tmp_type) {
+					//remove old default
+					var def = GWOthers.getItem('DEFAULT_FOLDERS',tmp_type);
+					if (def && (def = Path.split(def)) && aDataSet[def[1]])
+						delete aDataSet[def[1]].DEFAULT;
 
-				//set new default
-				aDataSet[sName].DEFAULT = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0].DEFAULT[0].VALUE;
-				GWOthers.setItem('DEFAULT_FOLDERS',tmp_type,aFolderInfo['aid']+'/'+sName);
+					//set new default
+					aDataSet[sName].DEFAULT = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0].DEFAULT[0].VALUE;
+					GWOthers.setItem('DEFAULT_FOLDERS',tmp_type,aFolderInfo['aid']+'/'+sName);
+				}
 			}
 
-			var srt = {};
-				srt[aFolderInfo['aid']] = aDataSet;
+			srt = {};
+			srt[aFolderInfo['aid']] = aDataSet;
 
 			dataSet.add(sDataSet,[aFolderInfo['aid']], this.sort(srt)[aFolderInfo['aid']], true);
 			dataSet.update(sDataSet,[aFolderInfo['aid']]);
+
+			if (!aFolderInfo.bSkipNotify) {
+				if (dataSet.get('folders', [aFolderInfo.aid, aFolderInfo.fid])) { // rename
+					Item.notify([aFolderInfo.aid, aFolderInfo.fid], 'UPDATE');
+				} else { // move
+					Item.notify([aFolderInfo.aid, aFolder.ATTRIBUTES.UID], 'UPDATE', {
+						FOLDER: aFolderInfo.fid,
+						'FOLDER-ID': aFolder.ATTRIBUTES.UID
+					});
+				}
+			}
 		}
 		//add
 		else
 		if (!aFolderInfo['fid'] && aFolderInfo['name']){
-			var out = this.parse(aXMLResponse)[aFolderInfo['aid']][aFolderInfo['name']];
+			aFolder = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0];
+			var out = this.parse(aXMLResponse)[aFolderInfo['aid']][aFolder.ATTRIBUTES.UID];
 
 			//Remove after fix WC-288
 			//out.TYPE = aFolderInfo['type'];
@@ -706,15 +801,15 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 			if (typeof aFolderInfo['channel'] != 'undefined')
 	            out.RSS = 'true';
 
-			var srt = {},
-				tmp = dataSet.get(sDataSet,[aFolderInfo['aid']],true) || {};
-			    tmp[aFolderInfo['name']] = out;
-				srt[aFolderInfo['aid']] = tmp;
+			srt = {};
+			tmp = dataSet.get(sDataSet,[aFolderInfo['aid']],true) || {};
+			tmp[aFolder.ATTRIBUTES.UID] = out;
+			srt[aFolderInfo['aid']] = tmp;
 
 			//Check for Root of Private I folder
 			if (aFolderInfo.type == 'I' && aFolderInfo.private){
-				if (!srt[aFolderInfo['aid']][Path.split(aFolderInfo['name']).shift()]){
-					srt[aFolderInfo['aid']][Path.split(aFolderInfo['name']).shift()] = {
+				if (!srt[aFolderInfo['aid']][Path.split(aFolder.ATTRIBUTES.UID).shift()]){
+					srt[aFolderInfo['aid']][Path.split(aFolder.ATTRIBUTES.UID).shift()] = {
 						SHARED: "true",
 						OWNER: wm_folders.aux.private_owner,
 						PRIVATE_ROOT: "true",
@@ -723,8 +818,8 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 					};
 				}
 
-				if (!srt[aFolderInfo['aid']][Path.basedir(aFolderInfo['name'])]){
-					srt[aFolderInfo['aid']][Path.basedir(aFolderInfo['name'])] = {
+				if (!srt[aFolderInfo['aid']][Path.basedir(aFolder.ATTRIBUTES.UID)]){
+					srt[aFolderInfo['aid']][Path.basedir(aFolder.ATTRIBUTES.UID)] = {
 						NAME: getLang('COMMON_FOLDERS::PRIVATE_ROOT'),
 						TYPE: 'Y'
 					};
@@ -732,6 +827,10 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 			}
 
 			dataSet.add(sDataSet,[aFolderInfo['aid']],this.sort(srt)[aFolderInfo['aid']]);
+
+			if (!aFolderInfo.bSkipNotify) {
+				Item.notify([aFolderInfo.aid, aFolder.ATTRIBUTES.UID], 'ADD');
+			}
 		}
 
 		if (typeof aHandler == 'object')
@@ -746,7 +845,7 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 				var aData = this.parse(aXMLResponse,true);
 
 				if (aData[sPrimaryAccount])
-					WMAccounts.__mapFolders(aData[sPrimaryAccount]);
+					await WMAccounts.__mapFolders(aData[sPrimaryAccount]);
 
 				WMAccounts.__mergeDataset(aIQAttribute['UID'], aData[aIQAttribute['UID']]);
 
@@ -759,7 +858,7 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 				return true;
 			}
 		}
-		catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+		catch(e){ console.log(this._name||false,e);}
 
 		return false;
 
@@ -770,14 +869,14 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 				return true;
 			}
 		}
-		catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+		catch(e){ console.log(this._name||false,e);}
 
 		//ASYNC DataSet change
 		var blank = true;
 		var bPerfm = false;
-		var aDataSet = dataSet.get(sDataSet,[aFolderInfo['aid']]);
+		aDataSet = dataSet.get(sDataSet,[aFolderInfo['aid']]);
 
-		for(var i in aDataSet){
+		for(i in aDataSet){
 			if (i == aFolderInfo['fid'] || i.indexOf(aFolderInfo['fid']+'/') === 0){
 				bPerfm = true;
 				delete aDataSet[i];
@@ -786,6 +885,10 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 				blank = false;
 		}
 
+		if (!aFolderInfo.bSkipNotify) {
+			Item.notify([aFolderInfo.aid, aFolderInfo.fid], 'DELETE');
+		}
+
 		//	bNoRefresh = true protoze pracujeme primo s datasetem, proto po add neni zmena dat (add je jen pro sychr)
 		//	Nutne vyvolat refresh manualne
 		if (blank && aFolderInfo['aid'] == sPrimaryAccount+'_rss'){
@@ -803,31 +906,31 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 	case 'empty':
 		try{
 			if (aIQAttribute['TYPE'] != 'result' && aIQAttribute['UID']) {
-				var aFolder = Path.split(aIQAttribute['UID']);
+				aFolder = Path.split(aIQAttribute['UID']);
 				this.list({'aid':aFolder[0]},sDataSet,aDataPath);
 
 				// je potreba znovu vylistovat otevreny folder?
-				var aItems = dataSet.get('items');
-				for(var sAccId in aItems)
-					for(var sFolId in aItems[sAccId]);
+				aItems = dataSet.get('items');
+				for(sAccId in aItems)
+					for(sFolId in aItems[sAccId]);
 						if (sAccId == aFolder[0] && sFolId == aFolder[1])
-							WMItems.list({'aid':sAccId,'fid':sFolId,'values':items.default_values('M')},'items');
+							WMItems.list({'aid':sAccId,'fid':sFolId,'values':WMItems.default_values('M')},'items');
 
 				return true;
 			}
 			else
 			if (aIQAttribute['UID']){
-				var aFolder = Path.split(aIQAttribute['UID']);
+				aFolder = Path.split(aIQAttribute['UID']);
 
 				// je potreba promazat Items?
-				var aItems = dataSet.get('items');
-				for(var sAccId in aItems)
-					for(var sFolId in aItems[sAccId]);
+				aItems = dataSet.get('items');
+				for(sAccId in aItems)
+					for(sFolId in aItems[sAccId]);
 						if (sAccId == aFolder[0] && sFolId == aFolder[1])
 							dataSet.add('items',[aFolder[0],aFolder[1]],{});
 			}
 		}
-		catch(e){gui._REQUEST_VARS.debug && console.log(this._name||false,e);}
+		catch(e){ console.log(this._name||false,e);}
 
 		//ASYNC DataSet change
 		if (dataSet.get(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'])>0)
@@ -845,8 +948,9 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 			this.list({'aid':aFolderInfo['aid']},sDataSet,aDataPath);
 
 			// aktualizuj Items
-			if (gui.frm_main.main && gui.frm_main.main.list)
-				gui.frm_main.main.list._serverSort();
+			gui.frm_main._getNew();
+			// if (gui.frm_main.main && gui.frm_main.main.list)
+			// 	gui.frm_main.main.list._serverSort();
 		}
 		break;
 
@@ -867,22 +971,31 @@ _me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHa
 					nCount = 0;
 			}
 
-			var i = dataSet.get(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT']);
-				i = i>0?i:0;
+			i = dataSet.get(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT']);
+			i = i>0?i:0;
 
 			if (i != nCount)
 				dataSet.add(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'],nCount.toString());
 		}
 
 		// je potreba aktualizovat Items?
-		var aItems = dataSet.get('items');
-		for(var sAccId in aItems)
-			for(var sFolId in aItems[sAccId]);
-				if (sAccId == aFolderInfo['aid'] && sFolId == aFolderInfo['fid'])
-					gui.frm_main.main.list._serverSort();
+		// skip new email notification
+
+		window.skipNewMailNotification = true;
+		gui.frm_main._getNew();
+		// var aItems = dataSet.get('items');
+		// for(var sAccId in aItems)
+		// 	for(var sFolId in aItems[sAccId]);
+		// 		if (sAccId == aFolderInfo['aid'] && sFolId == aFolderInfo['fid'])
+		// 			gui.frm_main.main.list._serverSort();
 
 		return true;
 
+	// case 'add_member':
+	// case 'remove_member':
+	// 	if (aIQAttribute['TYPE'] == 'result')
+	// 		this._notify(sMethodName, aFolderInfo);
+
 	default:
 		if (typeof aHandler == 'object')
 			executeCallbackFunction(aHandler, aIQAttribute['TYPE'] == 'result', aFolderInfo);
@@ -963,7 +1076,7 @@ _me.sort = function(aFolders)
 				if (aFolders[sAccId][sFolId].PRIVATE_ROOT) {
 					aFolders[sAccId][sFolId].TYPE = 'Y';
 				}
-				
+
 				var sort = '';
 				if (aFolders[sAccId][sFolId].TYPE == 'I'){
 
@@ -1033,7 +1146,7 @@ _me.sort = function(aFolders)
 		aSortShared.sort(wm_folders.aux.sort);
 
 		//Add missing folders (in case of shared subfolders)
-		var fld,tmp;
+		var tmp,k;
 		for (var i = 0; i<aSortShared.length; i++){
 			if (aSortShared[i].FOLDER.indexOf('/')>-1){
 				fld = aSortShared[i].FOLDER.split('/'),
@@ -1060,41 +1173,42 @@ _me.sort = function(aFolders)
 		aResultFolFrame = {};
 
 		// INBOX
-		for(var n in aSortInbox){
+		var n, sItem;
+		for(n in aSortInbox){
 			aFolder = aFolders[sAccId][aSortInbox[n]['FOLDER']];
 			aResultFolFrame[aSortInbox[n]['FOLDER']] = {};
-			for(var sItem in aFolder)
+			for(sItem in aFolder)
 				aResultFolFrame[aSortInbox[n]['FOLDER']][sItem] = aFolder[sItem];
 		}
 		// Default Mail
-		for(var n in aSortDefault){
+		for(n in aSortDefault){
 			aFolder = aFolders[sAccId][aSortDefault[n]['FOLDER']];
 			aResultFolFrame[aSortDefault[n]['FOLDER']] = {};
-			for(var sItem in aFolder)
+			for(sItem in aFolder)
 				aResultFolFrame[aSortDefault[n]['FOLDER']][sItem] = aFolder[sItem];
 		}
 		// Default GW
-		for(var n in aSortGWDefault){
+		for(n in aSortGWDefault){
 			aFolder = aFolders[sAccId][aSortGWDefault[n]['FOLDER']];
 			aResultFolFrame[aSortGWDefault[n]['FOLDER']] = {};
-			for(var sItem in aFolder)
+			for(sItem in aFolder)
 				aResultFolFrame[aSortGWDefault[n]['FOLDER']][sItem] = aFolder[sItem];
 		}
 
 		// Rest...
-		for(var n in aSortOthers){
+		for(n in aSortOthers){
 			aFolder = aFolders[sAccId][aSortOthers[n]['FOLDER']];
 
 			if (!aFolder && aSortOthers[n]['VALUES'])
 				aResultFolFrame[aSortOthers[n]['FOLDER']] = aSortOthers[n]['VALUES'];
 			else{
 				aResultFolFrame[aSortOthers[n]['FOLDER']] = {};
-				for(var sItem in aFolder)
+				for(sItem in aFolder)
 					aResultFolFrame[aSortOthers[n]['FOLDER']][sItem] = aFolder[sItem];
 			}
 		}
 
-		for(var n in aSortShared){
+		for(n in aSortShared){
 
 			aFolder = aFolders[sAccId][aSortShared[n]['FOLDER']];
 
@@ -1110,7 +1224,7 @@ _me.sort = function(aFolders)
 			else
 				aResultFolFrame[aSortShared[n]['FOLDER']] = {};
 
-			for(var sItem in aFolder)
+			for(sItem in aFolder)
 				if (!aResultFolFrame[aSortShared[n]['FOLDER']][sItem])
 					aResultFolFrame[aSortShared[n]['FOLDER']][sItem] = aFolder[sItem];
 		}
@@ -1125,7 +1239,7 @@ _me.sort = function(aFolders)
 //Pomocná funkce realizující převod z "XML" pole do Dataset struktur        * OK *
 //********************************************************************************
 
-_me.parse = function(aData, bNoChatSync)
+_me.parse = function(aData)
 {
 	try
 	{
@@ -1136,7 +1250,7 @@ _me.parse = function(aData, bNoChatSync)
 			aResult = {},
 			aResultAccFrame = {},
 			aResultFolFrame,
-			aRights;
+			aRights, i;
 
 		//Map GW folders
 		var aTmp = {C:'CONTACTS',E:'EVENTS',T:'TASKS',N:'NOTES',J:'JOURNAL',F:'FILES',
@@ -1150,14 +1264,14 @@ _me.parse = function(aData, bNoChatSync)
 		*/
 		for(var nFolNum in aFolFrame){
 			aResultFolFrame = {};
-            sFolId = aFolFrame[nFolNum]['ATTRIBUTES']['UID'];
+            sFolId = aFolFrame[nFolNum]['ATTRIBUTES']['UID'].replace(/^\//, '');
 
 			for(var sItem in aFolFrame[nFolNum])
                 if (sItem == 'CHANNELS'){
 					if (aFolFrame[nFolNum][sItem][0] && aFolFrame[nFolNum][sItem][0].CHANNEL){
 						aResultFolFrame[sItem] = [];
                         aRights = aFolFrame[nFolNum][sItem][0].CHANNEL;
-						for(var i in aRights)
+						for(i in aRights)
 							if (aRights[i].VALUE)
 								aResultFolFrame[sItem].push(aRights[i].VALUE);
 					}
@@ -1167,11 +1281,13 @@ _me.parse = function(aData, bNoChatSync)
 					if (aFolFrame[nFolNum][sItem][0] && aFolFrame[nFolNum][sItem][0].RIGHT){
 						aResultFolFrame[sItem] = {};
                         aRights = aFolFrame[nFolNum][sItem][0].RIGHT;
-						for(var i in aRights)
-							aResultFolFrame[sItem][aRights[i].ATTRIBUTES.EMAIL] = aRights[i].VALUE?aRights[i].VALUE.split(''):[];
+						for(i in aRights)
+							aResultFolFrame[sItem][aRights[i].ATTRIBUTES.EMAIL] = {
+								name: aRights[i].ATTRIBUTES.NAME,
+								isGuest: aRights[i].ATTRIBUTES.ISGUEST,
+								rights: aRights[i].VALUE?aRights[i].VALUE.split(''):[]
+							};
 					}
-
-					acl = true;
                 }
 				else
 				/*
@@ -1191,6 +1307,9 @@ _me.parse = function(aData, bNoChatSync)
 					if (sFolderType == 'M')
                         sFolderType = aFolFrame[nFolNum][sItem][0].VALUE;
 
+					if (!sFolderType || !aTmp[sFolderType]) {
+						continue;
+					}
 					GWOthers.setItem('DEFAULT_FOLDERS',aTmp[sFolderType],sPrimaryAccount+'/'+aFolFrame[nFolNum].ATTRIBUTES.UID);
 
 					aResultFolFrame[sItem] = aFolFrame[nFolNum][sItem][0]['VALUE'];
@@ -1202,11 +1321,10 @@ _me.parse = function(aData, bNoChatSync)
 					if (aFolFrame[nFolNum][sItem][0].FOLDERS && aFolFrame[nFolNum][sItem][0].FOLDERS[0].FOLDER){
 	                    aResultFolFrame[sItem].FOLDERS = {};
 						var aFolers = aFolFrame[nFolNum][sItem][0].FOLDERS[0].FOLDER;
-						for(var i in aFolers)
+						for(i in aFolers)
 							if (!aFolers[i].ATTRIBUTES || !aFolers[i].ATTRIBUTES.NOEXIST)
 								aResultFolFrame[sItem].FOLDERS[aFolers[i].VALUE] = aFolers[i].ATTRIBUTES && aFolers[i].ATTRIBUTES.PRIMARY?true:false;
 					}
-					else
 					if (aFolFrame[nFolNum][sItem][0].SHARETYPE)
 						aResultFolFrame[sItem].SHARETYPE = aFolFrame[nFolNum][sItem][0].SHARETYPE[0].VALUE;
 
@@ -1223,19 +1341,19 @@ _me.parse = function(aData, bNoChatSync)
 
 		return aFolFrame ? this.sort(aResult) : {};
 	}
-	catch(e){
+	catch {
 		return false;
 	}
 };
 
 // TODO move this function somewhere else
-_me.__emptyFolder = function(sAccId, sFolId, bMoveToTrash, sSQLsearch)
+_me.__emptyFolder = async function(sAccId, sFolId, bMoveToTrash, sSQLsearch)
 {
 	var aTrashFolder, aHandler;
 
 	if (this.getType([sAccId,sFolId]) === 'M'){
 
-		storage.library('gw_others');
+		await storage.library('gw_others');
 
 		var aFolData = clone(dataSet.get('folders', [sAccId,sFolId]), true),
 			aTrash = Path.split(GWOthers.getItem('DEFAULT_FOLDERS', 'trash'));
@@ -1255,13 +1373,21 @@ _me.__emptyFolder = function(sAccId, sFolId, bMoveToTrash, sSQLsearch)
 			}
 
 			aTrashFolder = aTrash;
+
+			aHandler = [function(bOK){
+				if (bOK) {
+					dataSet.add('folders',[sAccId, sFolId, 'COUNT'], 0);
+				}
+			}]
 		}
 		else
 		// refreh folders on empty Trash
 		if (aTrash.aid === sAccId && aTrash.fid === sFolId)
 			aHandler = [function(bOK){
-				if (bOK)
+				if (bOK) {
+					dataSet.add('folders',[sAccId, sFolId, 'COUNT'], 0);
 					WMAccounts.refresh({aid:aTrash.aid}, 'folders');
+				}
 			}];
 
 		// clear recent
@@ -1323,5 +1449,112 @@ _me.getUnsubscribedChatFolders = function () {
 	return unsubscribed;
 };
 
+_me._notify = function(sAction, args){
+	gui.__exeEvent('folder', sAction, args);
+};
+
+_me._getFolderByRelativePath = function(aid, fid) {
+	var folders = dataSet.get('folders', [sPrimaryAccount]);
+	for(var i in folders) {
+		if (folders[i].OWNER === aid && folders[i].RELATIVE_PATH === Path.slash(fid)) {
+			return folders[i];
+		}
+	}
+};
+
+_me.getFolderName = function(aPath) {
+	var fid = Path.slash(aPath.fid);
+	var ds = dataSet.get('folders', [aPath.aid, fid]) || {};
+	var name = ds.NAME || Path.basename(Path.slash(ds.RELATIVE_PATH || fid));
+	
+	if (~fid.indexOf('__@@VIRTUAL@@__/@@SUBTREE@@/')) {
+		ds = dataSet.get('folders', [aPath.aid, fid.replace(/__@@VIRTUAL@@__\/@@SUBTREE@@\//, '')]) || {};
+		name = ds.NAME || Path.basename(Path.slash(ds.RELATIVE_PATH || fid.replace(/__@@VIRTUAL@@__\/@@SUBTREE@@\//, '')));
+		return getLang('COMMON_FOLDERS::SUBTREE', [name]);
+	}
+	if (~fid.indexOf('__@@VIRTUAL@@__/@@PUBLIC@@/')) {
+		return getLang('COMMON_FOLDERS::ALL_FOLDERS', [fid.replace('__@@VIRTUAL@@__/@@PUBLIC@@/', '').replace(/\/#\w/, '')]);
+	}
+	if (~fid.indexOf('__@@VIRTUAL@@__/@@PERSONAL@@/#')) {
+		return getLang('COMMON_FOLDERS::ALL_PERSONAL');
+	}
+	if (~fid.indexOf('__@@VIRTUAL@@__/@@SHARED@@/~')) {
+		var uid = fid.replace('__@@VIRTUAL@@__/@@SHARED@@/~', '').replace(/\/#\w/, '');
+		return getLang('COMMON_FOLDERS::ALL_FOLDERS', [dataSet.get('accounts', [uid, 'USERNAME']) || uid]);
+	}
+
+	switch(fid) {
+		case '@@UPLOAD@@':
+		case '__@@UPLOAD@@__':
+			return getLang('COMMON_FOLDERS::UPLOAD');
+		case '__@@ARCHIVE@@__':
+			return getLang('COMMON_FOLDERS::ALL_ARCHIVES');
+		case '__@@SNOOZED@@__':
+			return getLang('COMMON_FOLDERS::SNOOZED');
+	}
+
+	return name;
+};
+
+_me.getFolderPath = function(aPath, delimiter) {
+	
+	var account = aPath.aid;
+
+	var path = Path.slash(aPath.fid).split('/');
+	var folders = [];
+	var sType = WMFolders.getType([account, path.join('/')]);
+
+	if (Mapping.isGlobalSearchFolder(path.join('/'))) {
+		folders.push(WMFolders.getFolderName({aid: account, fid: path.join('/')}));
+	} else {
+		for (var i = 0; i < path.length; i++) {
+			var name = (dataSet.get('folders', [account, path.slice(0, i + 1).join('/')]) || {}).NAME || path.slice(i, i + 1)[0];
+			if (!(sType === 'E' && name === '__@@VIRTUAL@@__') && (name !== 'SPAM_QUEUE')) {
+				name = name.replace('__@@VIRTUAL@@__', getLang('COMMON_FOLDERS::VIRTUAL-FOLDERS'));
+				folders.push(WMFolders.getFolderName({fid: name}));
+			}
+		}
+		if (account !== sPrimaryAccount) {
+			folders.unshift(account);
+		}
+	}
+	return folders.filter(Boolean).join(delimiter || ' / ');
+};
+
+_me.addToFavorites = function(aFolder, bSkipDashboardNotification) {
+	var favs = Cookie.get(['favorites']) || [];
+	if (!favs.some(function(fav) {
+		return fav.arg.aid === aFolder.aid && fav.arg.fid === aFolder.fid;
+	})) {
+		favs.push({ title: WMFolders.getFolderName(aFolder), arg: aFolder });
+	}
+	Cookie.set(['favorites'], favs, true);
+
+	if (!bSkipDashboardNotification && gui.frm_main._dashboardLoaded()) {
+		gui.frm_main.dashboard._addFolderToFavorites({
+			aid: aFolder.aid,
+			fid: aFolder.fid
+		});
+	}
+};
+
+_me.removeFromFavorites = function(aFolder, bSkipDashboardNotification) {
+	var aData = Cookie.get(['favorites']) || [];
+	for(var i = 0; i < aData.length; i++) {
+		if (aData[i].arg && aData[i].arg.aid == aFolder.aid && aData[i].arg.fid == aFolder.fid){
+			aData.splice(i, 1);
+			Cookie.set(['favorites'], aData, true);
+
+			if (!bSkipDashboardNotification && gui.frm_main._dashboardLoaded()) {
+				gui.frm_main.dashboard._removeFolderFromFavorites({
+					aid: aFolder.aid,
+					fid: aFolder.fid
+				});
+			}
+			break;
+		}
+	}
+};
+
 ///////////////////////////////
-WMFolders = new wm_folders();
+window.WMFolders = new wm_folders();
