@@ -1,7 +1,4 @@
 function obj_datagrid_skin(){};
 obj_datagrid_skin.prototype.__constructor = function(){
-	this._row_height = 33;
-	this._original_height = 33;
-	
 	this._col_width = 15;
 };
\ No newline at end of file
