@@ -2,6 +2,8 @@
 
 class slControllerAccount extends slControllerDefault
 {
+	public $param;
+	public $oUser;
 	public function check(&$action, &$data)
 	{
 		parent::check($action,$data);
