@@ -1,6 +1,8 @@
 <?php
 slSystem::import('tools/php');
 class slHelperItem extends slHelper {
+	public $param;
+	public $folderHelper;
 	private $DTF=array();
 	
 	public function bytesToSize($bytes) {
@@ -248,7 +250,7 @@ class slHelperItem extends slHelper {
 		
 		 		require_once('inc/webmailiqitems.php');
 		if (method_exists($this,"convert".$container->type)) {
-			$result['data'] = WebmailIQItems::cnvItems(
+			$result['data'] = WebmailIqItems::cnvItems(
 				$oItems,
 				$filter,
 				array(
@@ -257,7 +259,7 @@ class slHelperItem extends slHelper {
 				)
 			);
 		} else {
-			$result['data'] = WebmailIQItems::cnvItems( $oItems , $filter );
+			$result['data'] = WebmailIqItems::cnvItems( $oItems , $filter );
 		}
 		 		if (method_exists($this,$container->type.'finishHandler')){
 			$finish=$container->type.'finishHandler';
@@ -315,6 +317,7 @@ class slHelperItem extends slHelper {
 				case 'continue':
 				default:
 					try{
+						$result = [];
 						$container = $this->getContainer( $param, $result['info'] );
 						$result['container'] = $result['info'];
 						 						$oItem = $container->getItem( $this->data['item'] );
@@ -328,7 +331,7 @@ class slHelperItem extends slHelper {
 						$filter = array();
 						$filter['tag']="*";
 						$items[0] = $oItem;
-						$items = WebmailIQItems::cnvItems(
+						$items = WebmailIqItems::cnvItems(
 							$items,	 							$filter,	 							array(
 								'class'=>$this,
 								'method'=>'convertM'
@@ -426,7 +429,8 @@ class slHelperItem extends slHelper {
 				case 'replytoall':
 					$result['data']['mail_action'] = 'reply';
 
-					 					 					 					$result['data']['date'] = @date($DTFormats['date'].self::convertTimeFormat(" H:i:s",$DTFormats['time']),$result['data']['date']);
+					 					 					 					$DTFormats=self::getDTFormats();
+					$result['data']['date'] = @date($DTFormats['date'].self::convertTimeFormat(" H:i:s",$DTFormats['time']),$result['data']['date']);
 					
 					$result['data']['sourceid'] = $result['data']['id'];
 					$result['data']['sourcefolder'] = $this->data['id'];
@@ -807,7 +811,7 @@ class slHelperItem extends slHelper {
 				}else{
 					$method = false;
 				}
-				$items = WebmailIQItems::cnvItems(
+				$items = WebmailIqItems::cnvItems(
 					$items,	 					$filter,	 					$method,	 					false,		
 					1,			 					false		 				);
 				$item = reset($items);
@@ -831,14 +835,15 @@ class slHelperItem extends slHelper {
 		$request = slRequest::instance();
 		$data = $this->formWidget($param);
 		$link = $request->getPath().'?_c=folder&amp;_a[sync]=1&amp;_s[id]='.rawurlencode($data['container']->name)."&amp;_s[type]=".$data['container']->getType();
+		$application = false;
 		if($sort = $request->get('get._s.sort')){
-			$link = slControllerGrid::replaceURLVariable($link,'_s[sort]',$sort,'&amp;');
+			$link = (new slControllerGrid($application))->replaceURLVariable($link,'_s[sort]',$sort,'&amp;');
 		}
 		if($search = $request->get('get._s.search')){
-			$link = slControllerGrid::replaceURLVariable($link,'_s[search]',$search,'&amp;');
+			$link = (new slControllerGrid($application))->replaceURLVariable($link,'_s[search]',$search,'&amp;');
 		}
 		if($page = $request->get('get._s.page')){
-			$link = slControllerGrid::replaceURLVariable($link,'_s[page]',$page,'&amp;');
+			$link = (new slControllerGrid($application))->replaceURLVariable($link,'_s[page]',$page,'&amp;');
 		}
 		$data['buttons']['refresh'] = $link;
 		$data['container'] = $data['info'];
@@ -983,7 +988,7 @@ class slHelperItem extends slHelper {
 		$aItem['unread'] = !($oItem->flags & Item::FLAG_SEEN);
 		$aItem['aditional']['fromshow'] = htmlspecialchars(self::getDisplayAddress($aItem['from']));
 		$aItem['aditional']['toshow'] = htmlspecialchars(self::getDisplayAddress($aItem['to']));
-		$toArray = Mailparse::parseAddresses($aItem['to']);
+		$toArray = MailParse::parseAddresses($aItem['to']);
 		if(count($toArray)>1 || 
 			strtolower($toArray[0]['address'])!=strtolower($_SESSION['EMAIL'])
 		){
