@@ -1,7 +1,7 @@
 <?php
 class slWidgetsettings extends slWidget
 {
-	public function create()
+	public function create($sID)
 	{
 		die('s');
 	}
