@@ -9,25 +9,73 @@ require_once('inc/include.php');
 require_once('inc/exception.php');
 require_once('inc/defines.php');
 
+function normalizeSize(&$width, &$height) {
+	$sizes = [
+		[640, 480],
+		[1280, 720],
+		[1920, 1080],
+		[2560, 1440],
+		[3840, 2160],
+		[7680, 4320]
+	];
+
+	$ratio = $height / $width;
+	if ($width >= $height) {
+		foreach ($sizes as $size) {
+			if ($width <= $size[0] && $height <= $size[1]) {
+				$width = $size[0];
+				$height = $width * $ratio;
+				return;
+			}
+		}
+
+		$width = end($sizes)[0];
+		$height = $width * $ratio;
+	} else {
+		foreach ($sizes as $size) {
+			if ($height <= $size[0] && $width <= $size[1]) {
+				$height = $size[0];
+				$width = $height / $ratio;
+				return;
+			}
+		}
+		$height = end($sizes)[0];
+		$width = $height / $ratio;
+	}
+}
+
+function get_content($URL){
+	$ch = curl_init();
+	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
+	curl_setopt($ch, CURLOPT_URL, $URL);
+	$data = curl_exec($ch);
+	curl_close($ch);
+	return $data;
+}
+
 function main() {
 	if(defined('SECURE_COOKIE') && SECURE_COOKIE){
 		header('Strict-Transport-Security: max-age=31536000; includeSubDomains; preload');
 	}	
-	if (($fullpath = Get()->fullpath) === null) {
-		exit('Invalid fullpath');
-	}
 	if (($class = Get()->class) === null) {
 		exit('Invalid class');
 	}
+	if (($fullpath = Get()->fullpath) === null && $class != 'raw_message') {
+		exit('Invalid fullpath');
+	}
 	$fallBackUrl = Get()->fallback ?? null;
 	 
-	if (!($sid = Get()->sid)) {
-		 		if($class!='captcha' 
-		&& $class!='ticket' 
-		&& $class!='background' 
-		&& $class!='jitsi_logo'
-		&& $class!='icechat_background'
-		&& $class!='logo') {
+	if (!($sid = Get()->dlsess ? slToolsCrypt::decryptSymmetric(Get()->dlsess) : Get()->sid)) {
+		 		if(
+			$class != 'captcha' &&
+			$class != 'ticket' &&
+			$class != 'background' &&
+			$class != 'jitsi_logo' &&
+			$class != 'icechat_background' &&
+			$class != 'favicon' &&
+			$class != 'logo' &&
+			$class != 'raw_message'
+		) {
 			log_buffer("Download session [Invalid sid]","EXTENDED");
 			exit('Invalid sid');
 		}
@@ -35,16 +83,16 @@ function main() {
 	header('Accept-Ranges: bytes');
 
 	$passphrase_rsa = Get()->passphrase;
-	WebmailIqAuth::checkServerKeys();
-	$privateData = file_get_contents(WM_CONFIGPATH.'private.key');
-	$privateKey = openssl_pkey_get_private($privateData);
-	openssl_private_decrypt(WebmailIqAuth::hexStringToBinary($passphrase_rsa), $passphrase, $privateKey);
+	$passphrase = WebmailRSA::instance()->rsaToPlain($passphrase_rsa);
 	
 	$resize = Get()->resize;
-	$quality = Get()->quality;
-	$crop = Get()->crop;
-	$width = Get()->width;
-	$height = Get()->height;
+	$quality = intval(Get()->quality);
+	$crop = intval(Get()->crop);
+	$width = intval(Get()->width);
+	$height = intval(Get()->height);
+	$disposition = Get()->disposition;
+	$uid = Get()->uid;
+	$default = Get()->default;
 	$type = $start_part_id = $filename = null;
 
 	try {
@@ -65,7 +113,9 @@ function main() {
 		}
 		
 		log_buffer("Download session [".($sid?$sid:'Unauthorized')."] STARTED","DEBUG");
-		$pathData = Tools::parseFullPath($fullpath,$class);
+		if ($fullpath) {
+			$pathData = Tools::parseFullPath($fullpath,$class);
+		}
 		log_buffer("Download session [".($sid?$sid:'Unauthorized')."] PARAMETERS : [class=".$class.",fullpath=".$fullpath."]","DEBUG");
 		if($class == 'item' && $pathData['account'][0] == '@') $class = 'special';
 		switch($class) {
@@ -107,14 +157,16 @@ function main() {
 								'item_id'=>$item_id,
 								'sid'=>$sid
 							),
-							$item->getSubjectFileName('.eml'),
+							$item->getSubjectFileName(messageext),
 							$resize,
 							'',
 							$_REQUEST['skin'],
-							$passphrase
+							$passphrase,
+							false,
+							$disposition
 						);
 					}else{
-						$item->sendData();
+						$item->sendData($disposition);
 					}
 				}else{
 					if($item->wmclass=='GW'){
@@ -194,7 +246,6 @@ function main() {
 					$account = $user->getAccount($account_id);
 					$folder = $account->getFolder($folder_id);
 				}
-
 				$item = $folder->getItem($item_id);
 				if($resize){
 					$resize = array();
@@ -204,6 +255,22 @@ function main() {
 					$resize['type'] = $type;
 					$resize['quality'] = $quality;
 				}		
+
+				if(strpos($pathData['part'],'draft_sa_')!==false){
+					$fullpath = base64_decode(str_replace('draft_sa_','',$pathData['part']));
+					$fullpath = Item::fixSmartAttachFullPath($fullpath);
+					$fullpath = Tools::parseFullPath($fullpath, 'item');
+					$folder = $account->getFolder($fullpath['folder']);
+					$item = $folder->getItem($fullpath['item']);
+					
+					if($item->item['EVNCLASS']=='F'){
+						$account_id = $fullpath['account'];
+						$folder_id = $fullpath['folder'];
+						$item_id = $fullpath['item'];
+						$part_id = $item->getFileAttachmentID();
+					}
+				}
+
 				$item->sendAttachment(
 					$part_id,
 					$start_part_id,
@@ -219,7 +286,7 @@ function main() {
 					$_REQUEST['skin'],
 					$passphrase,
 					false,
-					$_REQUEST['disposition'] ?? 'attachment'
+					('inline' === $_REQUEST['disposition'] ?? false) ? 'inline' : 'attachment'
 				);
 				 				break;
 			case 'message':
@@ -328,6 +395,7 @@ function main() {
 				$item = $folder->getItem($item_id);
 				 				if ($item->wmclass=="M" || $item->wmclass=="Q") {
 					$fname = $item->getSubjectFileName($start_part_id);
+					$fname = preg_replace('/\s-\s\d+$/', '', preg_replace('/^(\b(fw|re):?\s)*/i', '', $fname));
 				}
 				if(empty($fname)) $fname = date('YmdGis');
 
@@ -391,7 +459,7 @@ function main() {
 				@$size = filesize($filename);
 				$attachment['name'] = str_replace("+"," ",$attachment['name']);
 				$user->closeSession();
-				slToolsFilesystem::sendFileHeaders($attachment['name'], $size, $attachment['type']);
+				slToolsFilesystem::sendFileHeaders($attachment['name'], $size, $attachment['type'], false, ('inline' === $_REQUEST['disposition'] ?? false) ? 'inline' : 'attachment');
 				slToolsFilesystem::downloadFile($filename,$attachment['delete']);
 				
 				 
@@ -438,11 +506,11 @@ function main() {
 				echo $oCaptcha->showrandomimage($sWord);
 				break;
 			case 'ticket':
-				 				$sPrivateKey = WebmailIqAuth::getServerData('private');
+				 				$sPrivateKey = WebmailRSA::instance()->getServerData('private');
 				 				$sHash = $pathData['hash'];
 				unset($pathData['hash']);
-				 				$sNewHash = MerakGWAPI::createURLLine($pathData);
-				 				$sNewHash = rawurlencode(base64_encode(slToolsPHP::gzcompress($sNewHash)));
+				 				$sNewHash = MerakGWAPI::CreateURLLine($pathData);
+				$sNewHash = rawurlencode(base64_encode(gzcompress($sNewHash)));
 				$sNewHash = $sNewHash.md5($sPrivateKey);
 				 				if($sNewHash!==$sHash){
 					exit('Server Private Key missmatch');
@@ -624,44 +692,64 @@ function main() {
 				$resource_name = 'conferencing_options';
 				$fileSettings = 'jitsi_logo_name';
 			case 'icechat_background':
-			case 'logo':
 			case 'background':
+				if($resize && $width && $height) {
+					normalizeSize($width, $height);
+				}
+			case 'logo':
+			case 'favicon':
 				header("Expires: Mon, 1 Jan 2099 00:00:00 GMT");
 				$dom = Tools::getHostDomain($pathData['host']);
 				$suffix = ($class=='background' || $class=='icechat_background')?'background_name':'logo';
-				$resource_name = $resource_name ?? (($class=='icechat_background')?'icechat_settings':'layout_settings');
+				$resource_name = $resource_name ?? 'layout_settings';
 				$global = Storage::getDefaults($resource_name);
 				$domain = Storage::getDomainDefaults($resource_name,$dom);
 				$settings = WebmailIqPublic::get($resource_name,$global,$domain,false);
 				$fileSettings = $fileSettings ?? 'login_'.$suffix;
+				if ($class === 'favicon') {
+					$fileSettings = 'login_favicon_name';
+				}
 
 				$file = $settings['@childnodes']['item'][0]['@childnodes'][$fileSettings][0]['@value'];
+				$file = str_replace(['../', '..\\'], '', $file);
 				$forced = $settings['@childnodes']['item'][0]['@childnodes'][$fileSettings][0]['@attributes']['access'] ?? false;
 
+				$type = '';
+				if ($class=='background') {
+					$type = 'jpg';
+				}
 				if($class=='background' || $class=='icechat_background'){
-					$defaultfile = 'background--default.jpg';
-				}else{
+					$defaultfile = 'background--default.png';
+				} elseif ($class === 'favicon') {
+					$defaultfile = 'favicon.png';
+				} else {
 					$defaultfile = 'logo--w.svg';
 				}
-				 				if(!$file){
+				
+				 				$use_custom = true;
+				if(!$file || $default) {
+					$use_custom = false;
 					$file = $defaultfile;
 				}
 				 				$subdir = $dom?($dom.'/'):'';
 
-				if ( file_exists(WM_CONFIGPATH.'images/'.$class.'/custom/'.$subdir.$file) && !$forced){
-					$filename = WM_CONFIGPATH.'images/'.$class.'/custom/'.$subdir.$file;	
-				}else if ( file_exists(WM_CONFIGPATH.'images/'.$class.'/custom/'.$file)){
-					$filename = WM_CONFIGPATH.'images/'.$class.'/custom/'.$file;	
-				}else if(file_exists(WM_CONFIGPATH.'images/'.$class.'/'.$file)){
+				$used_class = $class;
+				if ($class === 'icechat_background') {
+					$used_class = 'background';
+				}
+
+				if ($use_custom && file_exists(WM_CONFIGPATH.'images/'.$used_class.'/custom/'.$subdir.$file) && !$forced){
+					$filename = WM_CONFIGPATH.'images/'.$used_class.'/custom/'.$subdir.$file;
+				} elseif ($use_custom && file_exists(WM_CONFIGPATH.'images/'.$used_class.'/custom/'.$file)){
+					$filename = WM_CONFIGPATH.'images/'.$used_class.'/custom/'.$file;
+				} elseif (file_exists(WM_CONFIGPATH.'images/'.$class.'/'.$file)){
 					$filename = WM_CONFIGPATH.'images/'.$class.'/'.$file;
-				}else {
+				} elseif (file_exists(WM_CONFIGPATH.'images/'.$used_class.'/'.$file)){
+					$filename = WM_CONFIGPATH.'images/'.$used_class.'/'.$file;
+				} else {
 					$filename = WM_CONFIGPATH.'images/'.$class.'/'.$defaultfile;
 				}
 				if(!file_exists($filename)){
-					if($fallBackUrl){
-						header('Location: '. $fallBackUrl);
-						exit();
-					}
 					http_response_code(404);
 					die();
 				}
@@ -675,20 +763,49 @@ function main() {
 					slSystem::import('tools/image');
 					$image = new slToolsImage();
 					$api = createobject('api');
-					$tempdir = $api->getProperty('C_System_Storage_Dir_TempPath').'webmail/'.slSystem::uniqueID().'/';
+					$tempdir = $api->getProperty('C_System_Storage_Dir_TempPath') . 'webmail/' .
+						$class .
+						($width ? 'w_' . $width : '') .
+						($height ? 'h_' . $height : '') .
+						($crop ? 'c_' . $crop : '') .
+						($quality ? 'q_' . $quality : '') . '/';
 
-					if(!is_dir($tempdir)){
-						slToolsFilesystem::mkdir_r($tempdir);
+					$tempFile = $tempdir.$uid.$type.basename($filename);
+					if($type && (file_exists($tempFileTyped = preg_replace('/\\.\\w+$/', '.' . $type, $tempFile)))) {
+						$resized_name = $tempFileTyped;
+					} elseif(file_exists($tempFile)){
+						$resized_name = $tempFile;
+					} else {
+						if(!is_dir($tempdir)){
+							slToolsFilesystem::mkdir_r($tempdir);
+						}
+						copy($filename,$tempFile);
+						$image->load($tempFile);
+						$image->edit($resize['width'],$resize['height'],$resize['crop'],false, $resize['type']);
+						$resized_name = $tempdir.$image->save('', '', -1);
 					}
-					$tempFile = $tempdir.basename($filename);
-					copy($filename,$tempFile);
-					$image->load($tempFile);
-					$image->edit($resize['width'],$resize['height'],$resize['crop']);
-					$resized_name = $tempdir.$image->save();	
 				}
 				
-				slToolsFilesystem::sendFileHeaders(basename($filename),filesize($resized_name?$resized_name:$filename),Tools::getMimeType($filename),false,'inline',true);
-				slToolsFilesystem::downloadFile($resized_name?$resized_name:$filename,$resized_name?true:false);
+				 				$mtime = @filemtime($filename);
+				$gmt_mtime = gmdate('D, d M Y H:i:s', $mtime) . ' GMT';
+				$etag = sprintf('%08x-%08x', crc32($filename.$width.$height.$crop.$type.$quality), $mtime);
+				header('ETag: "' . $etag . '"');
+				header('Last-Modified: '.gmdate(DATE_RFC1123,filemtime($filename)));
+				header('Cache-Control: private');
+				header('Content-Type: '.slMail::getMimeType($filename));
+				if(isset($_SERVER['HTTP_IF_NONE_MATCH']) && !empty($_SERVER['HTTP_IF_NONE_MATCH'])) {
+					$tmp = explode(';', $_SERVER['HTTP_IF_NONE_MATCH']);  					if(!empty($tmp[0]) && strtotime($tmp[0]) == strtotime($gmt_mtime)) {
+						header('HTTP/1.1 304 Not Modified');
+						return;
+					}
+				}
+				if(isset($_SERVER['HTTP_IF_NONE_MATCH'])) {
+					if(str_replace(array('\"', '"'), '', $_SERVER['HTTP_IF_NONE_MATCH']) == $etag) {
+						header('HTTP/1.1 304 Not Modified');
+						return;
+					}
+				}
+				slToolsFilesystem::downloadFile($resized_name?$resized_name:$filename);
 
 				break;
 			case 'two_factor_qr':
@@ -700,9 +817,24 @@ function main() {
 				die(403);
 			  }
 			  $filename = 'qr_code.png';
-			  slToolsFilesystem::sendFileHeaders($filename,0,Tools::getMimeType($filename),false,'inline',true);
+			  slToolsFilesystem::sendFileHeaders($filename,0,slMail::getMimeType($filename),false,'inline',true);
 			  echo base64_decode($qrCode);
-			break;
+				break;
+
+			case 'raw_message':
+				$_GET['_l'] = 'item';
+				$_GET['p0'] = 'html';
+				$_GET['p1'] = 'html';
+				$_GET['p2'] = 'mail.view.html';
+				$_GET['p3'] = Get()->aid;
+				$_GET['p4'] = Get()->fid;
+				$_GET['p5'] = 'M';
+				$_GET['p6'] = Get()->iid;
+				$_GET['PHPSESSID_BASIC'] = $sid;
+				$_GET['raw'] = '1tem';
+
+				include_once('../basic/index.html');
+				break;
 		}
 	} catch (Exc $e) {
 		echo $e->wmcode;
@@ -718,4 +850,4 @@ function main() {
 	log_buffer("Download session [".($sid?$sid:'Unauthorized')."] ENDED","DEBUG");
 }
 main();
-?>
+?>
\ No newline at end of file
