@@ -10,6 +10,8 @@ abstract class Account extends slObserver
 
 	 	public $folders;
 
+	public $allFolders = [];
+
 	 	public $server;
 
 	 	public $port;
@@ -30,7 +32,11 @@ abstract class Account extends slObserver
 	public $sizeAccount;
 	public $aSyncedFolders;
 	public $meetingsFolder;
-	
+	public $contactsFolder;
+	public $snoozedFolder;
+
+
+
 	   private static $aAutoCreateFolderTypes = array
 	(
 		'trash'=>'M',
@@ -57,7 +63,8 @@ abstract class Account extends slObserver
 			'__@@ADDRESSBOOK@@__',
 			'__@@DEVICES@@__',
 			'__@@GWTRASH@@__',
-			'__@@CRO_FOLDER@@__'
+			'__@@CRO_FOLDER@@__',
+			'__@@SNOOZED@@__'
 		);
 
 	
@@ -66,8 +73,13 @@ abstract class Account extends slObserver
   abstract public function sync($force_gw = false, $folders = array());
 
 	 
-	protected function __construct($server, $port, $username, $password,$properties)
-	{
+	protected function __construct(
+		$server,
+		$port,
+		$username,
+		 		$password,
+		$properties
+	) {
 		$this->folders	= array(
 			'main' => array(),
 			'gw'	=> array(),
@@ -210,7 +222,7 @@ abstract class Account extends slObserver
 				$this->folders["gw"] = array();
 			}
 		}
-		
+
 		 		if ($this->virtualAccount){
 			
 			try{
@@ -271,9 +283,9 @@ abstract class Account extends slObserver
 				}
 			}
 		}
-		
+
 		if ($this->primary){
-			 			if(is_array($this->folders['quarantine']) && !empty($this->folders['quarantine'])){
+			 			if(is_array($this->folders['quarantine'])){
 				foreach($this->folders['quarantine'] as $folder){
 					$return[$folder->name] = $folder;
 				}
@@ -287,16 +299,141 @@ abstract class Account extends slObserver
 				foreach($this->folders['search'] as $folder){
 					$return[$folder->name] = $folder;
 				}
+			}
+
+			 			if(($snoozedFolder = $this->getFolder('__@@SNOOZED@@__')) && !Folder::isRestrictedType('M')){
+				$return[$snoozedFolder->name] = $snoozedFolder;
 			}
 		}
 		return $return;
 	}
 
+	protected function getVirtualFolder($folderName)
+	{
+		if($this->gwAccount && $this->gwAccount->getIsExternal()){
+			return false;
+		}
+		$allowedFolders = [
+			'__@@VIRTUAL@@__/__@@TASKS@@__' => 'T',
+			'__@@VIRTUAL@@__/__@@NOTES@@__' => 'N',
+			'__@@VIRTUAL@@__/__@@FILES@@__' => 'F',
+			'__@@VIRTUAL@@__/__@@JOURNALS@@__' => 'J',
+			'__@@VIRTUAL@@__/__@@CONTACTS@@__' => 'C',
+			'__@@VIRTUAL@@__/__@@MYUPLOAD@@__' => 'U',
+		];
+		if(!isset($allowedFolders[$folderName])) return false;
+		$type = $allowedFolders[$folderName];
+		$primary = User::getDefaultFolder($type);
+		$varName = lcfirst($primary) . 'Folder';
+		if(isset($this->{$varName}) && !empty($this->{$varName})) return $this->{$varName};
+		return $this->{$varName} = new VirtualFolder($this->virtualAccount, $folderName, $type, $folders, $primary,false,false,false,false,'personal');
+	}
+
+	public function getAllVirtualFolder($name)
+	{
+		if(Folder::isAllVirtualFolderName($name, $folderInfo)){
+			if(!isset($this->allFolders[$name])){
+				if(!$this->virtualAccount){
+					$virtualAccount = $_SESSION['user']->getAccount($_SESSION['EMAIL'])->virtualAccount;
+				}else{
+					$virtualAccount = $this->virtualAccount;
+				}
+				$this->allFolders[$name] = VirtualFolder::createFromFolderInfo($virtualAccount, $name, $folderInfo);	
+			}
+			return $this->allFolders[$name];
+		}
+		 
+		$nameUc = strtoupper($name);
+
+		 		if (strpos($nameUc,'__@@VIRTUAL@@__/##PUBLIC FOLDERS##/') === 0){
+			$publicFolder = str_replace('__@@VIRTUAL@@__/##PUBLIC FOLDERS##/', '', $nameUc);
+			if(!$this->allFolders['public'][$publicFolder]){
+				$type = 'M';
+				foreach($this->folders['main'] as $id => $folder){
+					if($folder->isPublic() && $folder->getType()=='M' && stripos(strtoupper($folder->name), $publicFolder)===0){
+						$ids[$id] = $id;
+					}
+				}
+				$this->allFolders['public'][$publicFolder] = new VirtualFolder(
+					$this->virtualAccount,
+					$nameUc,
+					$type,
+					$ids,
+					reset($ids),
+					false,
+					false,
+					false,
+					false,
+					'public',
+					$publicFolder
+				);
+			}
+			return $this->allFolders['public'][$publicFolder];
+		}
+
+		 		if(substr($nameUc,0,strlen('__@@VIRTUAL@@__/#'))=='__@@VIRTUAL@@__/#'){
+			$type = substr($nameUc,strlen('__@@VIRTUAL@@__/#'),1);
+			if(!isset($this->allFolders['all'][$type])){
+				$ids  = [];
+				if($type == 'M'){
+					foreach($this->folders['main'] as $id => $folder){
+						if(($folder->getType()==$type) 
+						&& !$folder->isShared() 
+						&& !$folder->isPublic() 
+						&& !$folder->isArchiveSubfolder()
+						&& !$this->isDualFolder($folder)){
+							$ids[$id] = $id;
+						}
+					}
+				}
+				$primary = reset($ids);
+				$this->allFolders['all'][$type] = new VirtualFolder($this->virtualAccount, $nameUc, $type, $ids, $primary,false,false,false,false,'all');
+			}		
+			return $this->allFolders['all'][$type];
+		}
+
+		$subfolder = substr($nameUc,strlen('__@@VIRTUAL@@__/'),strlen($nameUc)-strlen('__@@VIRTUAL@@__/'));
+
+		 		if($subfolder && $subfolder[0] == $_SESSION['SHARED_PREFIX']){
+			$sharedAccount = substr($subfolder, 1, strlen($subfolder)-1);
+			if(!$this->allFolders['shared'][$sharedAccount]){
+				$ids = [];
+				foreach($this->folders['main'] as $id => $folder){
+					if($folder->isShared($sharedAccount) && $folder->getType()=='M'){
+						$ids[$id] = $id;
+					}
+				}
+				$this->allFolders['shared'][$sharedAccount] = new VirtualFolder(
+					$this->virtualAccount,
+					$nameUc,
+					'M',
+					$ids,
+					reset($ids),
+					false,
+					false,
+					false,
+					false,
+					'shared',
+					$sharedAccount
+				);
+			}
+			return $this->allFolders['shared'][$sharedAccount];
+		}
+
+		return false;
+	}
+
+
 	 
-	public function &getFolder($name, &$type = '', $logError = true, $allowDualHiddenFolder = false)
+	public function getFolder($name, &$type = '', $logError = true, $allowDualHiddenFolder = false)
 	{
 		$oUser = &$_SESSION['user'];
 		$nameUc = strtoupper($name);
+		$result = $this->getVirtualFolder($nameUc);
+		if($result !== false) return $result;
+		$result = $this->getAllVirtualFolder($name);
+		if($result !== false) return $result;
+
 		switch($nameUc){
 			case '__@@VIRTUAL@@__/__@@MEETINGS@@__':
 				if(!($this->meetingsFolder ?? false)) {
@@ -306,12 +443,22 @@ abstract class Account extends slObserver
 				}
 				return $this->meetingsFolder;
 				break;
+			case '__@@THREADS@@__':
+				if(!$this->folders['tcthreads']){
+					$oAccount = &$this->gwAccount;
+					$this->folders['tcthreads'] = new GroupWareFolder(
+						$oAccount,
+						'__@@THREADS@@__',
+						'I'
+					);
+				}
+				return $this->folders['tcthreads'];
 			case '__@@ARCHIVE@@__':
 				if(!$this->archiveFolder){
 					$ids = array();
 					if(isset($this->folders['main']) && !empty($this->folders['main'])){
 						foreach($this->folders['main'] as $id => $folder){
-							if($folder->isArchiveSubfolder()){
+							if($folder->isArchiveSubfolder() && $folder->getType()=='M'){
 								$ids[$id] = $id;
 							}
 						}
@@ -321,8 +468,14 @@ abstract class Account extends slObserver
 								'__@@ARCHIVE@@__',
 								'M',
 								$ids,
-								reset($ids)
+								reset($ids),
+								false,
+								false,
+								false,
+								false,
+								'archive'
 						);
+						$this->archiveFolder->setScope('archive');
 				}
 				return $this->archiveFolder;
 				break;
@@ -344,7 +497,9 @@ abstract class Account extends slObserver
 				if(!$this->folders['addressbook']){
 					 					$primary = User::getDefaultFolder('C');
 					 					$folders = array($primary,'@@GAL@@');
-					if($_SESSION['CUSTOM_ADDRESS_BOOK']!=User::getDefaultFolder('C') && $_SESSION['CUSTOM_ADDRESS_BOOK']!='__@@ADDRESSBOOK@@__'){
+					if(($_SESSION['CUSTOM_ADDRESS_BOOK']!='') 
+					&& ($_SESSION['CUSTOM_ADDRESS_BOOK']!=User::getDefaultFolder('C'))
+					&& ($_SESSION['CUSTOM_ADDRESS_BOOK']!='__@@ADDRESSBOOK@@__')){
 						$folders[] = $_SESSION['CUSTOM_ADDRESS_BOOK'];
 					}
 
@@ -413,6 +568,7 @@ abstract class Account extends slObserver
 				$folder = &$this->hipaaAccount->folders['icewarp']['__@@HIPAA@@__'];
 				return $folder;
 			break;
+			case '@@UPLOAD@@':
 			case '__@@UPLOAD@@__':
 				$oAccount = &$this->gwAccount;
 				foreach($this->folders['gw'] as $fdr){
@@ -441,6 +597,22 @@ abstract class Account extends slObserver
 				$this->sync();
 				return $folder;
 			break;
+			case '__@@MYUPLOAD@@__':
+				$oAccount = &$this->gwAccount;
+				foreach($this->folders['gw'] as $fdr){
+					if($fdr->getType() == 'U' && $fdr->groupOwner == $_SESSION['EMAIL']) return $fdr;
+				}
+				$folder = new GroupWareFolder($oAccount, '@@MYUPLOAD@@', 'U', Folder::DEFAULT_RIGHTS, false, false, false, false, 0, '', '', '', '', '', 'U');
+				$folder->groupOwner = $_SESSION['EMAIL'];
+				$this->sync();
+				return $folder;
+				break;
+			case '__@@SNOOZED@@__':
+				if(!$this->snoozedFolder){
+					$this->snoozedFolder = new SnoozedFolder($oUser, $this);
+				}
+				return $this->snoozedFolder;
+				break;
 			default:
 				 				if(strpos($name,'__@@GROUP@@__')===0){
 					if(!$this->groupAccount){
@@ -482,13 +654,7 @@ abstract class Account extends slObserver
 		 		if ($this->gwAccount && (!$type || $type=='gw')){
 			try {
 				$oFolder = null;
-				if(!$this->gwAccount->foldersLoaded){
-					$oFolder = $this->gwAccount->getFolder(MerakGWAPI::encode($name), $type);
-				}else{
-					if(isset($this->folders['gw'][$name])){
-						$oFolder = &$this->folders['gw'][$name];
-					}
-				}
+				$oFolder = $this->gwAccount->getFolder(MerakGWAPI::encode($name), $type, $type == 'gw');
 				if(is_object($oFolder) && $oFolder->getType()=='P' && isset($this->folders['main'][$name])){
 					$type = 'main';
 					return $this->folders['main'][$name];
@@ -504,7 +670,7 @@ abstract class Account extends slObserver
 			$type = 'main';
 			return $this->folders[$type][$name];
 			 			 		} else {
-			throw new Exc('folder_does_not_exist',$name,false,$logError);
+			 			throw new Exc('folder_does_not_exist',htmlentities($name),false,$logError);
 		}
 	}
 	
@@ -548,7 +714,7 @@ abstract class Account extends slObserver
 					}
 				}
 				if(!$isAllowed){
-					throw new Exc('folder_does_not_exist', $name);
+					throw new Exc('folder_does_not_exist', htmlentities($name));
 				}
 				 
 				$oAccount->createFolder($param);
@@ -651,7 +817,8 @@ abstract class Account extends slObserver
 	
 	public function syncFolders()
 	{
-		 		$this->aSyncedFolders = array();
+		 		$this->allFolders = [];
+		$this->aSyncedFolders = array();
 		 		foreach($this->folders["main"] as $oFolder){
 			if ($oFolder->sync){
 				log_buffer("Account::syncFolders()->".$oFolder->name."->sync == true => AUTOSYNC START ","EXTENDED");
@@ -707,15 +874,14 @@ abstract class Account extends slObserver
 		return true;
 	}
 
-	public function twoFactorConfirm($totp_code, $password)
-	{
+	public function twoFactorConfirm(
+		$totp_code,
+		 		$password
+	) {
 		 		if(!$totp_code){
 			throw new Exc('auth_two_factor_required');
 		}
-		 		WebmailIqAuth::checkServerKeys();
-		$privateData = file_get_contents(WM_CONFIGPATH.'private.key');
-		$privateKey = openssl_pkey_get_private($privateData);
-		openssl_private_decrypt(WebmailIqAuth::hexStringToBinary($password), $password, $privateKey);
+		 		$password = WebmailRSA::instance()->rsaToPlain($password);
 		parse_str($password,$pwd);
 		$password = $pwd['p'];
 
@@ -747,10 +913,7 @@ abstract class Account extends slObserver
 		 		$oAccount = new IceWarpAccount();
 		$oAccount->Open($_SESSION['EMAIL']);
 		
-		 		WebmailIqAuth::checkServerKeys();
-		$privateData = file_get_contents(WM_CONFIGPATH.'private.key');
-		$privateKey = openssl_pkey_get_private($privateData);
-		openssl_private_decrypt(WebmailIqAuth::hexStringToBinary($key), $key, $privateKey);
+		 		$key = WebmailRSA::instance()->rsaToPlain($key);
 		
 		 		$kInfo = Tools::parseURL($key);
 		$aeskey = $kInfo['key'];
@@ -762,6 +925,11 @@ abstract class Account extends slObserver
 		return $totpSecret;
 	}
 
+	public function isDualFolder($folder)
+	{
+		return isset($this->folders['main'][$folder->name]) && isset($this->folders['gw'][$folder->name]);
+	}
+
 }
 
 ?>
