@@ -18,6 +18,8 @@ abstract class CacheFolder extends Folder
 	public $delayedSync;
 	public $messages;
 	public $path;
+	public $publicRoot;
+	public $groupOwner;
 
 	 
 	protected function __construct(&$account, $folderID, $name, $rights, $sync, $attributes,$path = false,$validity = false,$messages = 0, $unseen = 0,$sync_update = '')
@@ -139,10 +141,10 @@ abstract class CacheFolder extends Folder
 		}
 		if(is_array($oItems)) foreach($oItems as $item) {
 			$ifilename = $item->getSubjectFileName();
-			$ifilename.='.eml';
+			$ifilename.= messageext;
 			$ifile = $item->getMessageFile();
 			if(!file_exists($ifile)){
-				$item->autoCreateMessage($ifile);
+				$ifile = $item->autoCreateMessage($ifile);
 			}
 			$zip->addFile($ifile,$ifilename);
 		}
@@ -220,6 +222,12 @@ abstract class CacheFolder extends Folder
 		 		return $cache->getItems($this, $aFilterTag, $this->itemClassName, $createObject, $ignoreHidden );
 	}
 
+	public function getItemsByIds(array $ids)
+	{
+		$cache = Cache::instance($_SESSION['user']);
+		return $cache->getItemsByIds($this, $ids, $this->itemClassName);
+	}
+
 
 	 
 	public function getItem( $itemID, $cache = array(), $rid = false)
@@ -488,17 +496,17 @@ abstract class CacheFolder extends Folder
 					  $isOpenedSequence = true;	
 					}else{
 					  if($isOpenedSequence){
-					    $sequence.=':'.$lastID.','.ltrim($oItem->uid,'0');	
+					    $sequence.=':'.$lastID.','.ltrim($oItem->uid,'0');
 					  }else{
 					    $sequence.=($sequence?',':'').ltrim($oItem->uid,'0');
 					  }
 					  $isOpenedSequence = false;
-					  $reset_sequence = false;	
+					  $reset_sequence = false;
 					}
-					$lastID = ltrim($oItem->uid,'0');
+					$lastID = (int) $oItem->uid;
 					if ($s++ >= CHUNK_SIZE*(sizeof($sequences_chunk)+1)){
 						if ($isOpenedSequence){
-						  $sequence.=':'.$lastID;	
+						  $sequence.=':'.$lastID;
 						}else{
 						  $sequence.=','.$lastID;
 						}
@@ -900,25 +908,33 @@ abstract class CacheFolder extends Folder
 	 	public function setDefault($type,$updateSettings = true)
 	{
 		log_buffer("(".$this->name.")->setDefault() ".$type,"EXTENDED");
-		$oldDefault = User::getDefaultFolder($type);
 		User::setDefaultFolder($this->name,$type);
-		$old = &$this->account->folders['main'][$oldDefault];
-		$new = &$this->account->folders['main'][$this->name];
-		$old->isDefault = false;
-		$oldID = $old->folderID;
-		$oldAttributes = $old->attributes;
-		$newID = $new->folderID;
-		$newAttributes = $new->attributes;
+
+		 		$oldDefault = User::getDefaultFolder($type);
+		if ($oldDefault) {
+			try{
+				$old = &$this->account->getFolder($oldDefault);
+			}catch(Exc $e){
+				 				 				$oldDefault = ''; 
+			}
+			if($old){
+				$old->isDefault = false;
+				$oldID = $old->folderID;
+				$oldAttributes = $old->attributes;
+				$oldAttributes &= ~(User::$mailFlags[$type] ?? 0);
+				unset($old->defaultType);
+			}
+		}
+
 		if($this->isDefault()){
 			unset($_SESSION['DEFAULT_FOLDERS'][$this->defaultType]);
 			$unsetOld = true;
 			$unsetType = $this->defaultType;
 		}
-		unset($old->defaultType);
-		$new->isDefault = true;
-		$new->defaultType = $type;
+
 		$this->isDefault = true;
 		$this->defaultType = $type;
+
 		 		if($updateSettings){
 			$data = Storage::getUserData();
 			$default_folders = &$data['@childnodes']['default_folders'][0]['@childnodes']['item'][0]['@childnodes'];
@@ -930,12 +946,12 @@ abstract class CacheFolder extends Folder
 			log_buffer("(".$this->name.")->setDefault() update settings: ".$str,"EXTENDED");
 			Storage::setUserDataStr($str,'default_folders');
 		}
-		$oldAttributes &= ~(User::$mailFlags[$type] ?? 0);
-		$newAttributes |= User::$mailFlags[$type];
-		$cache = Cache::instance($_SESSION['user']);
-		$cache->updateFolder($oldID,array('attributes'=>$oldAttributes));
-		$cache->updateFolder($newID,array('attributes'=>$newAttributes));
-		$new->attributes = $newAttributes;
+		if ($oldDefault){
+			$cache = Cache::instance($_SESSION['user']);
+			$cache->updateFolder($oldID,array('attributes'=>$oldAttributes));
+			$this->attributes |= User::$mailFlags[$type];
+			$cache->updateFolder($this->folderID,array('attributes'=>$this->attributes));
+		}
 	}
 	
 	public function getPath()
