@@ -10,7 +10,9 @@ abstract class CacheItem extends Item
 
 	 
 	public $rid;
-
+	public $uid;
+	public $fileid;
+	public $timestamp;
 	public $size;
 	public $body;
 	public $cc;
@@ -30,7 +32,11 @@ abstract class CacheItem extends Item
 	public $taglist;
 	public $item_moved;
 	public $x_icewarp_server_request;
+	public $ctz;
+	public $timezone;
 	protected $parsedMailTemp = [];
+	public $original_date;
+	public $original_folder_id;
 
 	abstract public function getLocalMessage($file = null);
 	abstract public function autoCreateMessage($filename = null);
@@ -126,7 +132,7 @@ abstract class CacheItem extends Item
 	}
 	
 	 
-	protected function edit($file, $message = false, $item = array(),$cache = array())
+	protected function edit($file, $message = false, $item = array(),$cache = array(), $bothIdMatch = false)
 	{
 		$folder  = $this->folder;
 		$account = $folder->account;
@@ -134,14 +140,14 @@ abstract class CacheItem extends Item
 		if (!$cache['cache'])
 			$cache['cache'] = Cache::instance($_SESSION['user']);
 
-		$this->editProperties($item, $cache);
+		$this->editProperties($item, $cache, $bothIdMatch);
 		if(!$folder->isRemote() && ($file || $message)){
 			$cache['cache']->editMessage($account->accountID, $folder->folderID, $this->itemID, $message, $file);
 		}
 	}
 	
 	 
-	private function editProperties($item, $cache = array()) {
+	private function editProperties($item, $cache = array(), $bothIdMatch = false) {
 		if (!isset($cache['cache'])) {
 			$account = $this->folder->account;
 			$cache['cache'] = Cache::instance($_SESSION['user']);
@@ -172,7 +178,8 @@ abstract class CacheItem extends Item
 			'is_hidden'=>'is_hidden',
 			'dummy_id'=>'dummy_id',
 			'taglist'=>'taglist',
-			'item_moved'=>'item_moved'
+			'item_moved'=>'item_moved',
+			'msg_file'=>'fileid'
 		);
 		$setItems = [
 			'header_to' => '',
@@ -189,7 +196,7 @@ abstract class CacheItem extends Item
 		 		if (isset($setItems['has_attachment']))
 			$setItems['has_attachment'] = $setItems['has_attachment'] ? 'T' : 'F';
 
-		$cache['cache']->updateItem($this->itemID, $setItems, $cache);
+		$cache['cache']->updateItem($this->itemID, $setItems, $cache, 'none', false, false, false, $bothIdMatch);
 	}
 	
 	public function updateItem($fields)
@@ -320,7 +327,7 @@ abstract class CacheItem extends Item
 	}
 
 	 
-	protected function setFlags($flags, $field = 'flags')
+	public function setFlags($flags, $field = 'flags')
 	{
 
 		$folder  = $this->folder;
@@ -344,11 +351,11 @@ abstract class CacheItem extends Item
 	}
 
 	 
-	protected function unsetFlags($flags, $field = 'flags')
+	public function unsetFlags($flags, $field = 'flags')
 	{
 		$folder  = $this->folder;
 		$account = $folder->account;
-		
+
 		switch (strtolower($field)) {
 		case 'flags':
 			$params = array('flags' => $this->flags & ~$flags);
@@ -416,7 +423,7 @@ abstract class CacheItem extends Item
 		$account = $folder->account;
 
 		$cache = Cache::instance($_SESSION['user']);
-		 		if($this->folder->isLocal()){
+		 		if($this->folder->isLocal() || explode('@', $account->accountID)[1] === explode('@', $_SESSION['user']->email)[1]) {
 			$messageFile = $this->getLocalMessage();
 			if(!file_exists($messageFile)){
 				 				log_buffer("Invalid message file : ".$messageFile,"EXTENDED");
@@ -453,8 +460,18 @@ abstract class CacheItem extends Item
 	}
 	
 	 
-	public function parseMessage($decodeMime = true,$partID = 1,&$newMessage = false,$block_external = false,&$blocked = false,$download = false, $source = false, $rawhtml = false, $passphrase =  '', $purify = true)
-	{
+	public function parseMessage(
+		$decodeMime = true,
+		$partID = 1,
+		&$newMessage = false,
+		$block_external = false,
+		&$blocked = false,
+		$download = false,
+		$source = false,
+		$rawhtml = false,
+		 		$passphrase =  '',
+		$purify = true
+	) {
 		$ids = array();
 		$ids['account_id'] = $this->folder->account->accountID;
 		$ids['folder_id'] = $this->folder->name;
@@ -464,7 +481,7 @@ abstract class CacheItem extends Item
 		$cache = Cache::instance($_SESSION['user']);
 		$filename =  $this->getMessageFile();
 		if(!file_exists($filename)){
-			$this->autoCreateMessage($filename);
+			$filename = $this->autoCreateMessage($filename);
 		}
 		 		log_buffer("MailParse ( ItemID ) : ".$this->itemID,"EXTENDED");
 		log_buffer("MailParse ( ParseFile ) : ".$filename,"EXTENDED");
@@ -473,7 +490,7 @@ abstract class CacheItem extends Item
 		$parser->PassPhrase = $passphrase;
 		$parser->block_external = $block_external;
 		$parser->download = $download;
-		$structure = $parser->parse(false, $rawhtml, true);
+		$structure = $parser->parse(false, $rawhtml, $this->folder->addHtmlLinks ?? true);
 		$blocked = $parser->blocked;
 
 		$structure['headers_plain'] = '';
@@ -547,8 +564,13 @@ abstract class CacheItem extends Item
 
 	
 	 
-	protected static function getFlagsFromStructure(&$message, &$structure, $cached,$file = false,$passphrase = '')
-	{
+	protected static function getFlagsFromStructure(
+		&$message,
+		&$structure,
+		$cached,
+		$file = false,
+		 		$passphrase = ''
+	) {
 		$result = array();
 		$result['flags'] = 0;
 		$parser = new MailParse('',array(),false,'','1','Dummy');
@@ -583,6 +605,7 @@ abstract class CacheItem extends Item
 				$result['flags'] |= Item::ST_FLAG_EDIT_DISABLED;
 			}
 		}
+		$cert = [];
 		$result['smime_status'] = $parser->sMimeStatus($structure,$message,$file,$cert);
 		if($cert && !isset($structure['certificate'])){
 			$structure['certificate'] = $cert;
@@ -591,8 +614,12 @@ abstract class CacheItem extends Item
 	}
 	
 	 
-	static protected function getStaticFlags(&$message, $cached = true,$file = false, $passphrase ='')
-	{
+	static protected function getStaticFlags(
+		&$message,
+		$cached = true,
+		$file = false,
+		 		$passphrase =''
+	) {
 
 		if($file){
 			$parser = new MailParse($file);
@@ -608,9 +635,9 @@ abstract class CacheItem extends Item
 	{
 		$file = $this->getMessageFile();
 		if(!file_exists($file)){
-			$this->autoCreateMessage($file);
+			$file = $this->autoCreateMessage($file);
 		}
-		slToolsFilesystem::sendFileHeaders($this->getSubjectFileName('.eml'), filesize($file), 'message/rfc822');
+		slToolsFilesystem::sendFileHeaders($this->getSubjectFileName(messageext), filesize($file), 'application/octet-stream');
 		$_SESSION['user']->closeSession();
 		$fp = fopen($file,'rb');
 		while ($buffer = fread($fp,65536)){
@@ -623,21 +650,31 @@ abstract class CacheItem extends Item
 	{
 		$file = $this->getMessageFile();
 		if(!file_exists($file)){
-			$this->autoCreateMessage($file);
+			$file = $this->autoCreateMessage($file);
 		}
-		$info['name'] = self::getSubjectFileName('.eml');
+		$info['name'] = self::getSubjectFileName(messageext);
 		$info['size'] = filesize($file);
-		$info['type'] = 'message/rfc822';
+		$info['type'] = 'application/octet-stream';
 		$info['file'] = $file;
 		return $file;
 	}
 	
 	 
-	public function sendAttachment($partID,$startID,$ids = array(),$sForceName = false,$resize = false,$atttype='',$skin = false,$passphrase = '', $return = false, $disposition = null)
-	{
+	public function sendAttachment(
+		$partID,
+		$startID,
+		$ids = array(),
+		$sForceName = false,
+		$resize = false,
+		$atttype='',
+		$skin = false,
+		 		$passphrase = '',
+		$return = false,
+		$disposition = null
+	) {
 		$file = $this->getMessageFile();
 		if(!file_exists($file)){
-			$this->autoCreateMessage($file);
+			$file = $this->autoCreateMessage($file);
 		}
 		if(empty($ids)){
 			$ids = array(
@@ -649,7 +686,8 @@ abstract class CacheItem extends Item
 		}
 		$mailparse = new MailParse($file,$ids,true,'',$startID);
 		$mailparse->PassPhrase = $passphrase;
-		$_SESSION['user']->closeSession();	
+		$_SESSION['user']->closeSession();
+		$params = [];
 		if($return) return $mailparse->getPart($partID,$params);
 		$mailparse->sendPart($partID,$sForceName,$resize, $disposition);
 	}
@@ -658,7 +696,7 @@ abstract class CacheItem extends Item
 	{
 		$file = $this->getMessageFile();
 		if(!file_exists($file)){
-			$this->autoCreateMessage($file);
+			$file = $this->autoCreateMessage($file);
 		}
 		if(empty($ids)){
 			$ids = array(
@@ -685,6 +723,7 @@ abstract class CacheItem extends Item
 				'sid'=>$_SESSION['SID']
 		);
 		$mailparse = new MailParse($file,$ids,true);
+		$params = [];
 		$attachmentFile = $mailparse->getPartFileCID($cid,$params);
 		
 		$info['mimetype'] = $params['info']['content-type'];
@@ -705,12 +744,14 @@ abstract class CacheItem extends Item
 		if(!isset($this->parsedMailTemp[$file])){
 			$this->parsedMailTemp[$file] = new MailParse($file,$ids,true);
 		}
+		$params = [];
 		$content = $this->parsedMailTemp[$file]->getPartCID($cid,$params);
 		$info['mimetype'] = $params['info']['content-type'];
 		$info['name'] = $params['info']['disposition-filename'];
 		$info['encoding'] = $params['info']['headers']['content-transfer-encoding'];
 		return $content;
 	}
+	
 
   	public function getSubjectFileName($ext='',$part_id = '') 
   	{
@@ -746,8 +787,9 @@ abstract class CacheItem extends Item
 	    $structure = $parser->parse();
 	    $result = $structure['attachments'];
 	    if($result) foreach($result as $key => $attachment){
-	      $content = $parser->getPart($attachment['part_id'],$params);	
-	      if(preg_match('/(?P<url>https?\:\/\/.*\/teamchatapi\/.*)/', $content, $matches) && filter_var($matches['url'], FILTER_VALIDATE_URL) && ($urlContent = slToolsIcewarp::get_curl_file_contents($matches['url']))){
+	      $params = [];
+	      $content = $parser->getPart($attachment['part_id'],$params);
+	      if(preg_match('/(?P<url>https?\:\/\/.*\/teamchatapi\/.*)/', $content, $matches) && filter_var($matches['url'], FILTER_VALIDATE_URL) && ($urlContent = icewarp_file_get_contents($matches['url']))){
 		 $content = $urlContent;
 	      }
 	      $result[$key]['file_content'] = $content;
@@ -779,6 +821,7 @@ abstract class CacheItem extends Item
     $structure = $parser->parse();
     $result = $structure['attachments'];
     if($result) foreach($result as $key => $attachment){
+      $params = [];
       $file = $parser->getPartFile($attachment['part_id'],$params);
       $result[$key]['file'] = $file;
       if(($pos = strpos($attachment['part_id'],'|IMIP'))!==false){
@@ -796,11 +839,13 @@ abstract class CacheItem extends Item
   }
 
  
-	public function getItemData(&$info = array(),$passphrase = '')
-	{
+	public function getItemData(
+		&$info = array(),
+		 		$passphrase = ''
+	) {
 		static $attachCount;
 		$info['mimetype'] = 'message/rfc822';
-		$info['name'] = 'message_'.((string) ++$attachCount).".eml";
+		$info['name'] = 'message_'.((string) ++$attachCount).messageext;
 		$message = $this->getMessage();
 		if($this->sMimeStatus > 1){
 			$crt = '';
@@ -808,11 +853,39 @@ abstract class CacheItem extends Item
 			$smime = new slSMime();
 			$smime->setCertificateList(Storage::getPersonalCertificates());
 			$smime->setTempDir(User::getDir());
+			$isOutlook = false;
+			$expired = false;
 			$smime->decodeMessage($message,false,$crt,$isOutlook,$expired,$this->date,$passphrase);
 		}
 		return $message;
 	}
 
+	public function getItemFile(
+		&$info = array(),
+		 		$passphrase = ''
+	) {
+		static $attachCount;
+		$info['mimetype'] = 'message/rfc822';
+		$info['name'] = 'message_'.((string) ++$attachCount).messageext;
+		if($this->sMimeStatus > 1){
+			$message = $this->getMessage();
+			$crt = '';
+			slSystem::import('mail/smime');
+			$smime = new slSMime();
+			$smime->setCertificateList(Storage::getPersonalCertificates());
+			$smime->setTempDir(User::getDir());
+			$isOutlook = false;
+			$expired = false;
+			$smime->decodeMessage($message,false,$crt,$isOutlook,$expired,$this->date,$passphrase);
+			$file = \Tools::randomFilename();
+			file_put_contents($file, $message);			
+		}else{
+			$file = $this->getMessageFile();
+		}
+		return $file;
+	}
+
+
 	public function getAttachmentData($partID,&$info = array())
 	{
 		$file =  $this->getMessageFile();
@@ -867,6 +940,7 @@ abstract class CacheItem extends Item
 			$att_id = $partID;
 			$part_id = 1;
 		}
+
 		$ids = array(
 			'account_id'=>$this->folder->account->accountID,
 			'folder_id'=>$this->folder->folderID,
@@ -875,11 +949,15 @@ abstract class CacheItem extends Item
 		);
 		$mailparse = new MailParse($file,$ids,true,(!$part_id || $part_id==1)?'':$part_id,$part_id);
 		$mailparse->startPart = $part_id;
-
-		$attachmentFile = $mailparse->getPartFile($att_id,$params);
+		$params = [];
+		$attachmentFile = $mailparse->getPartFile($att_id, $params);
+		
 		$info['mimetype'] = $params['info']['content-type'];
 		$info['name'] = MailParse::getAttachmentName($params['info'],$att_id);
 		$info['encoding'] = $params['info']['transfer-encoding'];
+		$info['is_smarturl'] = $params['is_smarturl'];
+		$info['size'] = $params['size'];
+
 		return $attachmentFile;
 	}
 
@@ -981,8 +1059,10 @@ abstract class CacheItem extends Item
 		return $oInvitation->process();
 	}
 
-	public function getCertificate($structure = false, $passphrase = '')
-	{
+	public function getCertificate(
+		$structure = false,
+		 		$passphrase = ''
+	) {
 		$file = $this->getMessageFile();
 		if($structure === false){
 			$structure = $this->parseMessage(true,'1',$newMessage,false,$blocked,false,false,false,$passphrase);
@@ -1138,8 +1218,12 @@ abstract class CacheItem extends Item
 				}
 			}
 		}
+		
+		if(empty($vcard)){
+			throw new Exc('import_attachment_vcard_empty');
+		}
 
-		if(empty($vcard) || empty($gwAPI) || !$gwAPI->IsConnected()){
+		if(empty($gwAPI) || !$gwAPI->IsConnected()){
 			throw new Exc('groupware_init_failed');
 		}
 
@@ -1201,6 +1285,48 @@ abstract class CacheItem extends Item
 		}
 		return $result;
 	}
+
+	public function setTimezone($minutes)
+	{
+		$this->ctz = $minutes;
+	}
+	
+	public function setTimestamp($timestamp)
+	{
+		$this->timestamp = $timestamp;
+	}
+	
+	public function snooze($minutes = 0)
+	{
+		$cache = ['cache' => Cache::instance($_SESSION['user'])];
+		 		if(!$minutes){
+			if(!$this->timestamp){
+				throw new Exc('snooze_minutes_or_timestamp_missing');
+			}else{
+				$snoozedDate = $this->timestamp;
+			}
+		}else{
+			$snoozedDate = time() + $minutes*60;
+		}
+
+		 		$snoozedItem = $cache['cache']->getSnoozedItem($this->itemID);
+		if (!$snoozedItem){
+			 			$snoozedItem = $cache['cache']->createSnoozedItem($this->itemID, $this->date);
+		}
+		 		return $cache['cache']->updateItem($this->itemID, ['date' => $snoozedDate], $cache, 'flags');
+	}
+
+	public function unsnooze()
+	{
+		$cache = Cache::instance($_SESSION['user']);
+		$snoozedItem = $cache->getSnoozedItem($this->itemID);
+		if(!$snoozedItem){
+			throw new Exc('item_not_snoozed');
+		}
+		return $snoozedItem->unsnooze();
+	}
+
+
 }
 
  
