@@ -20,7 +20,7 @@ class Paths
 	 
 	public function getMessageFile($dir, $itemID)
 	{
-		return $dir . '/' . $itemID . '.msg';
+		return $dir . '/' . $itemID . messageext;
 	}
 
 	 
