@@ -26,6 +26,8 @@ abstract class Folder
 	const FULL_RIGHTS = 959;
 	 
 
+	public $search;
+
 	 	public $name;
 
 	 	public $rights;
@@ -41,6 +43,10 @@ abstract class Folder
   	public $subscription_type;
   	public $attributes;
   	public $account;
+	public $addHtmlLinks = true;
+	public $publicRoot;
+	public $groupOwner;
+	public $aFilterTag = [];
 
 	 
 	protected function __construct($name, $rights)
@@ -55,9 +61,7 @@ abstract class Folder
 	}
 
 	 
-	protected static function create(&$account, $name, $param = '', $createDual = true)
-	{
-	}
+	protected static function create(&$account, $name, $param = '', $createDual = true) {}
 
 	 
 	abstract protected function delete();
@@ -72,18 +76,20 @@ abstract class Folder
 	abstract public function setSubscription($state);
 	abstract public function setNotify($bValue);
 	abstract public function setChannels($mailbox,$channels,$encoded = false);
-
-
-	 
-	public function getItems(&$aFilterTag = array(), $createObject = true, $ignoreHidden = true)
-	{
+	function getItemClass() {
+		return Folder::class;
+	}
+	public function isForceRename(&$aTreeItem) {
+		return false;
 	}
 
 
 	 
-	public function getItem($itemID, $cache = array())
-	{
-	}
+	abstract public function getItems(&$aFilterTag = array(), $createObject = true, $ignoreHidden = true);
+
+
+	 
+	abstract public function getItem($itemID, $cache = array());
 
 
  
@@ -109,7 +115,7 @@ abstract class Folder
 		 		foreach($oItems as $item){
 			$fileName = $item->getMessageFile();
 			if(!file_exists($fileName)){
-				$item->autoCreateMessage($fileName);
+				$fileName = $item->autoCreateMessage($fileName);
 			}
 			if(($oFolder->account->accountID[0] ?? null) == '@'){
 				$newID = $oFolder->createItem($item, $fileName);
@@ -274,7 +280,7 @@ abstract class Folder
 			return 'riwtlka';
 		}
 
-		 		if ('' === $rights) {
+		 		if ('' === $rights || is_array($rights)) {
 			$rights = null;
 		}
 
@@ -395,14 +401,7 @@ abstract class Folder
 		}
 	}
 	
-	public function markItems($flag, $oItems = false)
-	{
-		throw new Exc('folder_mark_non_mail',$this->name);
-	}
-	public function unmarkItems($flag, $oItems = false)
-	{
-		throw new Exc('folder_mark_non_mail',$this->name);
-	}
+	 
 
 	public function getType()
 	{
@@ -473,9 +472,13 @@ abstract class Folder
 	}
 	
 
-	public function isShared()
+	public function isShared($account = '')
 	{
-		return ($this->attributes & 0x2000);
+		$result =  ($this->attributes & 0x2000);
+		if($result && $account!=''){
+			$result = (stripos($this->name, $_SESSION['SHARED_PREFIX'].$account) === 0);
+		}
+		return $result;
 	}
 	
 	public function isArchive()
@@ -493,7 +496,7 @@ abstract class Folder
 		}
 		return false;
 	}
-	
+
 	static public function isRestrictedType($type)
 	{
 		if(isset($_SESSION['RESTRICTIONS']) && isset($_SESSION['RESTRICTIONS'][$type])){
@@ -521,12 +524,51 @@ abstract class Folder
 		return false;
 	}
 	
+	static public function isAllVirtualFolderName($name, &$info = [])
+	{
+		$subfolder = '';
+		$name = str_replace("\\",'/',$name);
+		$virtual = false;
+		if(preg_match('/(?<virtual>__@@VIRTUAL@@__)(\/(?<scope>(@@SUBTREE@@|@@SHARED@@|@@PUBLIC@@|@@PERSONAL@@|@@ALL@@)))(\/?)((?<value>(.+)?)(#(?<type>[A-Z]))|(?<value>(.+)))/Jsi', $name, $matches)){
+			$virtual = true;
+			$matches['scope'] = ($matches['scope']=='')?'personal':$matches['scope'];
+			$matches['scope'] = strtolower(trim($matches['scope'],'@'));
+			$info = $matches;
+			$info['type'] = $matches['type'] ?? 'M';	
+			$info['group'] = $info['value'];
+			$info['folderID'] = (strtolower($info['scope'])=='all'?'@@ALL@@':'@@PERSONAL@@').'#'.$info['type'];
+			if($info['scope']=='public'){
+				$info['public'] = true;
+				$info['publicFolder'] = $matches['value'];
+			}elseif ($info['scope']=='shared'){
+				$info['shared'] = true;
+				$info['sharedAccount'] = iconv_substr($matches['value'],strlen($_SESSION['SHARED_PREFIX']),iconv_strlen($matches['value'],'utf-8'),'utf-8');
+			}elseif ($info['scope']=='personal'){
+				 $info['personal'] = true;
+			}elseif($info['scope']=='subtree'){
+				$info['subtree'] = true;
+			}else{
+				$info['scope'] = 'all';
+				$info['sharetype'] = 'all';
+			}
+		}
+		return $virtual;
+	}
+	
 	static public function checkACLConsistence($acl_visible,$acl_invisible,&$acl)
 	{
 		if($acl_visible) foreach($acl_visible as $key=>$val){
+			 			if ($key=='@') {
+				unset($acl_visible['@']);
+				continue;
+			}
 			$match1[$key] = Folder::rightsToString($val);
 		}
 		if($acl_invisible) foreach($acl_invisible as $key=>$val){
+			 			if ($key=='@') {
+				unset($acl_invisible['@']);
+				continue;
+			}
 			$match2[$key] = Folder::rightsToString($val);
 		}
 		if($match1!=$match2){
@@ -615,6 +657,7 @@ abstract class Folder
 			$class = $attachment['@childnodes']['values'][0]['@childnodes']['class'][0]['@value'];
 			$path = $attachment['@childnodes']['values'][0]['@childnodes']['fullpath'][0]['@value'];
 			$desc = $attachment['@childnodes']['values'][0]['@childnodes']['description'][0]['@value'];
+			$share = ($attachment['@childnodes']['values'][0]['@childnodes']['share'][0]['@value']=='true') || ($attachment['@childnodes']['values'][0]['@childnodes']['share'][0]['@value']=='1');
 			$att = array();
 			$evnuid = '';
 			switch(strtolower($class)){
@@ -696,12 +739,15 @@ abstract class Folder
 
 			$newTreeItem['@childnodes']['attachments'][0]['@childnodes']['attachment'][0] = $attachment;
 			try{
-				 				if($this->getType()=='F' || ($this->getType()=='I' && $aItem['evnclass']=='F')){
+				 				if(($this->getType()=='F' || $this->getType()=='U') || ($this->getType()=='I' && $aItem['evnclass']=='F')){
 					if(!$itemClass::checkAttachmentName($name)){
 						throw new Exc('attachment_name');
 					}
 				}
 				$item = $itemClass::create($this,$aItem,$newTreeItem, $att);
+				if($share){
+					$item->documentInvite('','','');
+				}
 				$result[] = $item;
 			}catch(Exc $e){
 				if($e->wmcode == 'item_duplicity'){
@@ -744,28 +790,52 @@ abstract class Folder
 	}
 
 	 
-	protected function getFilteredItems(string $filter) : array
+	public function getFilteredItems(string $filter) : array
 	{
-		$search = new SearchTool();
-		$search->setType($this->getType());
-		$search->setAccount($this->account);
-		$search->setFolder($this);
-		$sql = $search->parse($filter, $aFilterTag);
-		$aFilterTag = ['sql' => $sql];
+		if($this->getType() == 'M'){
+			$search = new \server\inc\imap\Search($this, $filter);
+		}else{
+			$search = new \server\inc\gw\Search($this, $filter);
+		}
+		if(!$search->isSearchEnabled() || ($condition = $search->getSearchSql()) === false) {
+			$aFilterTag = [ 'search' => $this->search ];
+			$condition = $search->sqlFallback($aFilterTag, $this->account->account);
+		}
+		$aFilterTag = [ 'sql' => $condition ];
 		return $this->getItems($aFilterTag) ?? [];
 	}
 
 	 
 	public function moveFiltered(string $filter, Folder $destinationFolder)
 	{
-		$this->moveItems($destinationFolder, $this->getFilteredItems($filter));
+		$oItems = $this->getFilteredItems($filter);
+		 		if(empty($oItems)){
+			return true;
+		}
+		 		$this->moveItems($destinationFolder, $oItems);
+	}
+
+	 
+	public function copyFiltered(string $filter, Folder $destinationFolder)
+	{
+		$oItems = $this->getFilteredItems($filter);
+		 		if(empty($oItems)){
+			return true;
+		}		
+		 		$this->copyItems($destinationFolder, $oItems);
 	}
 
 	 
 	public function deleteFiltered(string $filter)
 	{
-		$this->deleteItems($this->getFilteredItems($filter));
+		$oItems = $this->getFilteredItems($filter);
+		 		if(empty($oItems)){
+			return true;
+		}	
+		 		$this->deleteItems($oItems);
 	}
+
+
 }
 
  
