@@ -5,9 +5,12 @@ class GroupWareAddon
 	public $sParentID;
 	public $aFolderList;
 	public $ticket;
+	public $inviteTicket;
 	public $sAddonType;
 	public $AddonTypeID;
 	public $item;
+	public $oldNoteFormat;
+	public $data;
 
 	protected static $aAddonTypes = array(
 		'recurrence' => 0, 
@@ -102,11 +105,16 @@ class GroupWareAddon
 		$this->AddonTypeID = self::$aAddonTypes[$sAddonType];
 		$this->item = &$oItem;
 		$this->convertNote = false;
+		$this->loaded = false;
+		$this->oldNoteFormat = isset($this->item->item[$this->item->noteIndex])?($this->item->item[$this->item->noteIndex] == ''):true;
 	}
 
 	public function getData($param = null,$ctz = 0)
 	{
-		
+		 		if($this->sAddonType == 'note' && !$this->oldNoteFormat){
+			return false;
+		}
+
 		$oAccount = &$this->item->folder->account;
 		
 		$sFID = $this->item->folder->openAccess();
@@ -119,31 +127,32 @@ class GroupWareAddon
 		if($this->sAddonType=='revision'){
 			$result = $oAccount->gwAPI->FunctionCall(self::$aGetFunctions[$this->AddonTypeID], $sFID, $itemID, true);	
 		}else{
-			$result = $oAccount->gwAPI->FunctionCall(self::$aGetFunctions[$this->AddonTypeID], $sFID, $itemID, $param);
-			
+			$result = $oAccount->gwAPI->FunctionCall(self::$aGetFunctions[$this->AddonTypeID], $sFID, $itemID, $param);	
 		}
 
+		$this->loaded = true;
+		$this->data = false;
+
 		switch ($this->AddonTypeID) {
 			case 3:
-				if ($result)
-					return array(array('note_text' => $result));
+				if ($result){
+					$this->data = array(array('note_text' => $result));					
+				}					
 				break;
 			case 6:
 				if ($result) {
-					$array = Storage::parseCertificates($result);
-					return $array;
+					$this->data = Storage::parseContactCertificates($result);
 				}
 				break;
 			default:
 				$result = $oAccount->gwAPI->ParseParamLine($result);
 				if (isset($result[0])){
-					return $result;
+					$this->data = $result;
 				}
 				break;
-
 		}
 
-		return false;
+		return $this->data;
 	}
 
 	 
@@ -153,7 +162,7 @@ class GroupWareAddon
 			$sParentID = $this->item->itemID;
 
 		$oAccount = &$this->item->folder->account;
-
+		$this->loaded = false;
 		 		switch ($this->sAddonType) {
 			case 'revision':
 				$this->createRevision($aData);
@@ -167,31 +176,24 @@ class GroupWareAddon
 				$result = $oAccount->gwAPI->FunctionCall(self::$aSetFunctions[$this->AddonTypeID], $sFID, $sParentID, $sParam);
 				break;
 			case 'certificate':
-				$data = true;
 				if($currentData===false){
-					$data = false;
 					$currentData = $this->getData(false,$ctz);
 				}
 				$cert = Storage::getCertificateData( $oAccount->account->user, $aData );
 				unset($cert['pkey']);
 				unset($cert['extracerts']);
 				$currentData[] = $cert;
-				if(!$data){
-					$sParam = Storage::certificateListToString($currentData);
-					$sFID = $this->item->openAccess();
-					if(!($this->item->folder->rights & Folder::RIGHT_WRITE)){
-						throw new Exc('E_RIGHTS', MerakGWAPI::getGwErrorMessage(MerakGWAPI::E_RIGHTS));
-					}
-
-					$result = $oAccount->gwAPI->FunctionCall(
-						self::$aSetFunctions[$this->AddonTypeID], 
-						$sFID, 
-						$sParentID, 
-						$sParam
-					);
-				}else{
-                    $result = false;
-                }
+				$sParam = Storage::certificateListToString($currentData);
+				$sFID = $this->item->openAccess();
+				if(!($this->item->folder->rights & Folder::RIGHT_WRITE)){
+					throw new Exc('E_RIGHTS', MerakGWAPI::getGwErrorMessage(MerakGWAPI::E_RIGHTS));
+				}
+				$result = $oAccount->gwAPI->FunctionCall(
+					self::$aSetFunctions[$this->AddonTypeID], 
+					$sFID, 
+					$sParentID, 
+					$sParam
+				);
 				if(empty($result)){
 					$error = $oAccount->gwAPI->FunctionCall("GetLastError", $oAccount->sGWSessionID);
 					throw new Exc(MerakGWAPI::getGwError($error), MerakGWAPI::getGwErrorMessage($error));
@@ -269,9 +271,37 @@ class GroupWareAddon
 		$attData = Tools::parseFullPath($aData['fullpath'], $aData['class']);
 		$attID = null;
 		 		switch ($aData['class']) {
+			case 'link':
+				$fileToDownload = Tools::randomFileName(User::getUploadDir('url').'/');
+				User::closeSession();
+				$download = new slToolsDownload($attData['url']);
+				$location = $download->getLocation();
+				if($location){
+					$location = str_replace('/collaboration/?','/teamchatapi/'.$attData['name'].'?override_method=files.download&',$location); 
+					$location = preg_replace('/&url=.*/u','',$location);
+					$download = new slToolsDownload($location);
+				}
+				$download->saveLocal($fileToDownload);
+				$fileName = $aData['description']?$aData['description']:$download->getFileName();
+				$fileType = $download->getContentType();
+				User::restoreSession();
+				$attID = $this->attachFile(
+					$fileToDownload, 
+					$fileName, 
+					$fileType,
+					'',
+					isset($aData['AttType'])?$aData['AttType']:'F',
+					$aData['description']
+				);
+				$result['size'] = filesize($fileToDownload);
+				$result['type'] = $fileType;
+				break;
 			case 'file':
 				$attachItems = $_SESSION['user']->getAttachments($attData['folder']);
 				$attachItem = $attachItems[$attData['item']];
+				if('application/octet-stream' == ($attachItem['type'] ?? null)){
+					$attachItem['type'] = 'text/plain';
+				}
 				$attID = $this->attachFile(
 					$attachItem['file'], 
 					$attachItem['name'], 
@@ -294,7 +324,7 @@ class GroupWareAddon
 			case 'attachment':
 				 
 				$attID = $this->attachAttachment($attData['account'], $attData['folder'], $attData['item'],
-					$attData['part'], $aData['description'], $result['type'],$result['size']);
+					$attData['part'], $aData['description'], $result['type'], $result['size']);
 				break;
 			case 'itemlink':
 				$attID = $this->attachItemLink($attData['account'], $attData['folder'], $attData['item'],
@@ -333,13 +363,13 @@ class GroupWareAddon
 						$size = filesize($template);
 						$attachItem['AttSize'] = $size;
 						$attID = $this->addAttachment($attachItem, false,false,'',$template);
-						$this->item->edit(array('evncomplete'=>$size),array(),false,false,false);
+						$this->item->edit(array('evncomplete'=>$size),array(),false);
 						break;
 					default:
 						$attachItem['AttSize'] = 0;
 						
 						$attID = $this->addAttachment($attachItem,'');
-						$this->item->edit(array('evncomplete'=>0),array(),false,false,false);
+						$this->item->edit(array('evncomplete'=>0),array(),false);
 						break;
 				}
 				$result['size'] = $attachItem['AttSize'];
@@ -350,7 +380,7 @@ class GroupWareAddon
 				break;
 		}
 		if(isset($result['size']) && ($this->item->folder->getType()=='I' || $this->item->folder->getType()=='F')) {
-			$this->item->edit(array('evncomplete'=>$result['size']),array(),false,false,false);
+			$this->item->edit(array('evncomplete'=>$result['size']),array(),false);
 		}
 		$result['id'] = $attID;
 		return $result;
@@ -374,6 +404,7 @@ class GroupWareAddon
 			$sParentID = $this->item->itemID;
 		}
 		$oAccount = &$this->item->folder->account;
+		$this->loaded = false;
 		switch ($this->sAddonType) {
 			case 'certificate':
 				$data = true;
@@ -528,11 +559,12 @@ class GroupWareAddon
 				}
 			break;
 		}
+		$this->loaded = false;
 		return $result;
 	}
 
 
-	public function process($aTreeItem)
+	public function process($aTreeItem, $create = false)
 	{
 		$fileDataSet = false;
 		$fileData = '';
@@ -542,16 +574,15 @@ class GroupWareAddon
 			$fileDataSet = true;
 		}
 		 		$this->aFolderList = array();
-
 		$aSentData = $this->getSentData($aTreeItem);
-		
 		 		$dl = $this->sAddonType == 'location' && ($this->item->item['ITMCLASS'] == 'L');
 		$counter = 0;
 		$ctz = $aTreeItem['@childnodes']['values'][0]['@childnodes']['ctz'][0]['@value'];
         $result = false;
+		
 		if ($aSentData){
-			if($this->sAddonType=='certificate'){
-				$currentData = $this->getData();
+			if($this->sAddonType=='certificate' && !$create){
+				$currentData = $this->loaded ? $this->data : $this->getData();
 			}else{
 				$currentData = false;
 			}
@@ -571,8 +602,10 @@ class GroupWareAddon
 						break;
 
 					 					case 'delete':
-						if($this->item->folder->name=='@@mycard@@'){
+						if($this->item->folder->name=='@@mycard@@' && !$create){
 							$currentData = $this->getData();							
+						}else{
+							$currentData = false;
 						}
 						$result = $this->delete( $aData['id'],false, $currentData );
 						break;
@@ -582,7 +615,6 @@ class GroupWareAddon
 		}else{
             $currentData = false;
         }
-
 		 		if ($dl) {
 			 			$aLocations = $this->getData('LCTTYPE=\'O\'');
 			 			if (!$aLocations)
@@ -612,6 +644,7 @@ class GroupWareAddon
                 $result = true;
                 switch(strtolower($ext)){
                     case 'txt':
+                    case 'md':
                     case 'htm':
                     case 'html':
                     	$result = $this->editAttachment($attachment['ATTNAME'],$newData);
@@ -636,9 +669,9 @@ class GroupWareAddon
 			$sParentID = $this->item->itemID;
 			$oAccount = &$this->item->folder->account;
 			$sFID = $this->item->openAccess();
-			$data = $this->getData();
-			if(!$data) return $result;
-			$contacts = array_combine(array_column($this->getData(), 'CNTEMAIL'), $this->getData());
+			$this->data = $this->getData();
+			if(!$this->data) return $result;
+			$contacts = array_combine(array_column($this->data, 'CNTEMAIL'), $this->data);
 			if($contacts) foreach ($aTreeItem['@childnodes']['contacts'] ?? [] as $contact) {
 				$item = $contacts[current($contact['@childnodes']['contact'] ?? [])['EMAIL']] ?? false;
 				if($item){
@@ -690,32 +723,30 @@ class GroupWareAddon
 	 
 	public function getXML($ctz = 0)
 	{
-		$data = $this->getData(null,$ctz);
-
-		 		if (($data[0]['note_text'] ?? false) && true === $this->convertNote) {
-			if ($this->isFormatHtml($this->item->item['EVNDESCFORMAT'])) {
-				$data[0]['note_text'] = slToolsString::purifyHTML($data[0]['note_text'], Tools::externalResourcesEnabled());
-			}
-			elseif ($this->isFormatPlainText($this->item->item['EVNDESCFORMAT'])) {
-				$data[0]['note_text'] = slToolsString::textLinksToHTML($data[0]['note_text']);
-			}
+		if(!$this->loaded){
+			$this->getData(null,$ctz);
 		}
-
-		 		if (false === $data && true === $this->convertNote) {
-			if ($this->isFormatHtml($this->item->item['EVNDESCFORMAT'])) {
-				$this->item->item['EVNNOTE'] = slToolsString::purifyHTML($this->item->item['EVNNOTE'], Tools::externalResourcesEnabled());
-			}
-		}
-
-		 		if (false === $data && true === $this->convertNote) {
-			if ($this->isFormatHtml($this->item->item['ITMDESCFORMAT'])) {
-				$this->item->item['ITMDESCRIPTION'] = slToolsString::purifyHTML($this->item->item['ITMDESCRIPTION'], Tools::externalResourcesEnabled());
-			}
+		if($this->convertNote){
+			if($this->data === false){
+				 				if ($this->isFormatHtml($this->item->item['EVNDESCFORMAT'])) {
+					$this->item->item['EVNNOTE'] = slToolsString::purifyHTML($this->item->item['EVNNOTE'], Tools::externalResourcesEnabled());
+				}
+				 				if ($this->isFormatHtml($this->item->item['ITMDESCFORMAT'])) {
+					$this->item->item['ITMDESCRIPTION'] = slToolsString::purifyHTML($this->item->item['ITMDESCRIPTION'], Tools::externalResourcesEnabled());
+				}
+			}else{
+				if ($this->isFormatHtml($this->item->item['EVNDESCFORMAT'])) {
+					$this->data[0]['note_text'] = slToolsString::purifyHTML($this->data[0]['note_text'], Tools::externalResourcesEnabled());
+				}
+				elseif ($this->isFormatPlainText($this->item->item['EVNDESCFORMAT'])) {
+					$this->data[0]['note_text'] = slToolsString::textLinksToHTML($this->data[0]['note_text']);
+				}
+			} 
 		}
 
 		$sReturn = '';
-		if ($data)
-			foreach ($data as $item) {
+		if ($this->data)
+			foreach ($this->data as $item) {
 				 				if ($this->sAddonType == 'attachment') {
 					switch ($item['ATTTYPE']) {
 						case 'T':
@@ -738,26 +769,26 @@ class GroupWareAddon
 							$item['ATTTYPE'] = 'attachment';
 							break;
 					}
-					$item['TICKET'] = $this->getAttachmentFile($item['ATTNAME'],'TICKET');
+					$item['TICKET'] = User::addTokenToTicket($this->getAttachmentFile($item['ATTNAME'],'READONLYTICKET'));
 					if(!$this->ticket){
 						$this->ticket = $item['TICKET'];
 					}
 					if(!$this->inviteTicket){
-						$item['INVITETICKET'] = $this->getAttachmentFile($item['ATTNAME'],'INVITETICKET');
+						$item['INVITETICKET'] = User::addTokenToTicket($this->getAttachmentFile($item['ATTNAME'],'INVITETICKET'));
 						$this->inviteTicket = $item['INVITETICKET'];
 					}	
 					if($this->item->folder->getType()=='F' && $this->item->item['EVNCOMPLETE']==0){
-						$this->item->edit(array('evncomplete'=>$item['ATTSIZE']),array(),false,false,false);
+						$this->item->edit(array('evncomplete'=>$item['ATTSIZE']),array(),false);
 					}
 					@$sID = slToolsPHP::htmlspecialchars($item['ATTNAME']);
 				} else if ($this->sAddonType=='revision'){
 					$sID = $item['REVID'];
+				} else if($this->sAddonType == 'certificate'){
+					$sID = $item['id'];
 				} else {
 					$sID = $item[self::$aPrefixes[$this->AddonTypeID] . '_ID'];
 				}
-				if($this->sAddonType == 'certificate'){
-					$sID = $item['id'];
-				}
+				
 				$sStart = '<' . $this->sAddonType . '' . ($sID ? (' uid="' . $sID . '"') : ('')) .'>';
 				if($this->sAddonType == 'certificate'){
 					$sValue = '<info>'.slToolsPHP::htmlspecialchars(Storage::certInfo2XML($item['ainfo'])).'</info>';
@@ -792,7 +823,7 @@ class GroupWareAddon
 
 	 
 	protected function isFormatHtml($evnDescFormat) {
-		return strtolower($evnDescFormat) === 'text/html';
+		return strtolower($evnDescFormat ?? '') === 'text/html';
 	}
 
 	 
@@ -869,8 +900,7 @@ class GroupWareAddon
 		}
 		$param = $oAccount->gwAPI->CreateParamLine($param);
 		
-		if ($sNewId = $oAccount->gwAPI->FunctionCall("AddAttachment", $sFID, $objectID,
-			$param, $value ? $value : '')) {
+		if ($sNewId = $oAccount->gwAPI->FunctionCall("AddAttachment", $sFID, $objectID, $param, $value ? $value : '')) {
 			if ($file) {
 				$path = $oAccount->gwAPI->FunctionCall("GetAttachmentPathLocal", $sFID, $objectID, $sNewId, 'LOCK');
 				slSystem::import('tools/icewarp');
@@ -879,12 +909,15 @@ class GroupWareAddon
 				if(!$oAccount->gwAPI->FunctionCall("GetAttachmentPathLocal", $sFID, $objectID, $sNewId, 'UNLOCK')){
 					$lastError = $oAccount->gwAPI->FunctionCall("GetLastError",$oAccount->gwAPI->sessid);
 					$oAccount->gwAPI->FunctionCall("DeleteAttachments",$sFID, $objectID, $sNewId);
-					throw new Exc('attachment_add', $lastError.'|'.$objectID.'|'.$attName);
+					throw new Exc('attachment_add', MerakGWAPI::getGwErrorMessage($lastError).'|'.$objectID.'|'.$attName);
+				}
+				if((in_array($oFolder->getType(), ['F', 'I'])) && (strpos($file, DIRECTORY_SEPARATOR . '~webmail/~upload/') !== false)){
+					$oAccount->gwAPI->FunctionCall('addEventReference', $sFID, $objectID);
 				}
 			}
 		} else {
 			$lastError = $oAccount->gwAPI->FunctionCall("GetLastError",$oAccount->gwAPI->sessid);
-			throw new Exc('attachment_add',$lastError.'|'.$objectID.'|'.$attName);
+			throw new Exc('attachment_add', MerakGWAPI::getGwErrorMessage($lastError).'|'.$objectID.'|'.$attName);
 		}
         return $sNewId;
 	}
@@ -933,7 +966,7 @@ class GroupWareAddon
 							$size = 0;
 							break;
 					}
-					$result = $this->item->edit(array('evncomplete'=>$size),array(),false,false,false);
+					$result = $this->item->edit(array('evncomplete'=>$size),array(),false);
 					break;
 				case 'raw':
 					$param['AttName'] = $addonID;
@@ -991,8 +1024,7 @@ class GroupWareAddon
 		return $result;
 	}
 
-	public function attachFile($path, $filename, $mimeType, $fcontent = '', $attType =
-		'F',$description = '')
+	public function attachFile($path, $filename, $mimeType, $fcontent = '', $attType = 'F',$description = '')
 	{
 		$oAccount = &$this->item->folder->account;
 		if ($fcontent) {
@@ -1014,7 +1046,12 @@ class GroupWareAddon
 		$param['AttName'] = $description?$description:$attName;
 		$param['AttType'] = $attType;
 		$param['AttDesc'] = $description?$description:$filename;
-		$param['AttSize'] = filesize($path);
+		if($fContent){
+			$param['AttSize'] = strlen($fContent);
+		}else{
+			$param['AttSize'] = filesize($path);
+		}
+		
 		$param['AttTime'] = $now;
 		$param['AttParams'] = Tools::createURL(array('mimetype' => $mimeType));
 
@@ -1074,16 +1111,16 @@ class GroupWareAddon
 		 		} else {
 			$file = $attItem->getMessageFile();
 			if(!file_exists($file)){
-				$attItem->autoCreateMessage($file);
+				$file = $attItem->autoCreateMessage($file);
 			}
 			$size = filesize($file);
-			$ext = '.eml';
+			$ext = messageext;
 			$mimeType =  'message/rfc822';
 		}
 		$param['AttName'] = $description ? $description . $ext : md5($attItem->itemID . $now);
 		$param['AttType'] = 'I';
 		$param['AttDesc'] = $description;
-		$param['AttSize'] = 0;
+		$param['AttSize'] = $size;
 		$param['AttTime'] = $now;
 		$param['AttParams'] = Tools::createURL(array('mimetype' => $mimeType));
 
@@ -1120,10 +1157,10 @@ class GroupWareAddon
 		 		} else {
 			$file = $attItem->getMessageFile();
 			if(!file_exists($file)){
-				$attItem->autoCreateMessage($file);
+				$file = $attItem->autoCreateMessage($file);
 			}
 			$size = filesize($file);
-			$ext = '.eml';
+			$ext = messageext;
 			$mimeType =  'message/rfc822';
 		}
 
@@ -1154,11 +1191,10 @@ class GroupWareAddon
 		$param = array();
 		$param['AttName'] = !empty($description) ? $description : $info['name'];
 		$param['AttType'] = 'F';
-		$param['AttSize'] = filesize($file);
+		$size = $param['AttSize'] = $info['size'] ?? filesize($file);
 		$param['AttTime'] = $now;
 		$param['AttParams'] = Tools::createURL(array('mimetype' => $info['mimetype']));
 		$mimeType = $info['mimetype'];
-		$size = $param['AttSize'];
 
 		if ($description)
 			$param['AttDesc'] = $description;
@@ -1179,7 +1215,7 @@ class GroupWareAddon
 		$param = array();
 		$param['AttName'] = $info['name'];
 		$param['AttType'] = 'F';
-		$size = $param['AttSize'] = filesize($file);
+		$size = $param['AttSize'] = $info['size'] ?? filesize($file);
 		$param['AttTime'] = $now;
 		$param['AttParams'] = Tools::createURL(array('mimetype' => $info['mimetype']));
 		$mimeType = $info['mimetype'];
@@ -1197,7 +1233,7 @@ class GroupWareAddon
 		$sFID = $this->item->openAccess();
 		$oGWAPI = &$this->item->folder->account->gwAPI;
 		
-		$params = $oGWAPI->CreateURLLIne($params);
+		$params = $oGWAPI->CreateURLLine($params);
 		if(($revisionId = $oGWAPI->FunctionCall('CreateRevision', $sFID, $this->item->itemID, $params)) == ''){
 			throw new Exc('item_add_revision');
 		}
@@ -1209,7 +1245,7 @@ class GroupWareAddon
 		$sFID = $this->item->openAccess();
 		$oGWAPI = &$this->item->folder->account->gwAPI;
 		
-		$params = $oGWAPI->CreateURLLIne($params);
+		$params = $oGWAPI->CreateURLLine($params);
 		if(!$result = $oGWAPI->FunctionCall(
 				'EditItemRevision',
 				$sFID,
