@@ -2,6 +2,9 @@
  
 class GroupWareFolder extends Folder
 {
+
+	 
+
 	private static $aAliasTags = array('osd','oed','ost','oet');
 	private static $contactQuery = '((LCTTYPE IS NULL) OR (ITMCLASS=\'C\' AND LCTTYPE=\'H\') OR (ITMCLASS=\'L\' AND(( LCTTYPE=\'O\'))))';
 	public $folderID;  	public $name;	  	public $type;	  	public $folder;	 	public $account;  	public $subscription_type;
@@ -29,9 +32,13 @@ class GroupWareFolder extends Folder
 	public $sFID;
 	public $groupOwner;
 	public $shared;
+	public $fdrGroupChatAutoFollow;
+	public $fdrCreated;
+	public $count;
+	public $publicRoot;
 
 	 
-	public function __construct(&$oAccount,$folder,$type,$rights = Folder::DEFAULT_RIGHTS,$default = false,$gal  = false,$display = false, $subscriptionFolder = false, $groupChatUnread = 0, $groupOwner = '', $groupRelativeFolder = '', $groupChatSubscribed = '', $groupChatLastReadID = '',$groupChatLastActivity = '',$subtype = '',$teamChatNotify = false)
+	public function __construct(&$oAccount,$folder,$type,$rights = Folder::DEFAULT_RIGHTS,$default = false,$gal  = false,$display = false, $subscriptionFolder = false, $groupChatUnread = 0, $groupOwner = '', $groupRelativeFolder = '', $groupChatSubscribed = '', $groupChatLastReadID = '',$groupChatLastActivity = '',$subtype = '',$teamChatNotify = false, $fdrGroupChatAutoFollow = false, $fdrCreated = '')
 	{
 		 		$sName = MerakGWAPI::decode($folder);
 		$this->name= $sName;
@@ -58,6 +65,8 @@ class GroupWareFolder extends Folder
 		$this->subscription_type = $subscriptionFolder;
 		$this->subtype = $subtype;
 		$this->teamChatNotify = $teamChatNotify;
+		$this->fdrGroupChatAutoFollow = $fdrGroupChatAutoFollow;
+		$this->fdrCreated = $fdrCreated;
 	}
 
 	protected function handleIdIntervalFilter(array & $aFilterTag)
@@ -130,6 +139,7 @@ class GroupWareFolder extends Folder
 		$reset_unread = $aFilterTag['reset_unread'] ?? null;
 		
 		$sAttributes = $this->account->gwAPI->filterToAttributes($aFilterTag);
+		$sAttributes .= ($sAttributes?';':'') . 'ignore_dl=1';
 		$aItems = array();
 		 
 		$this->openAccess();
@@ -145,10 +155,11 @@ class GroupWareFolder extends Folder
 				$fields .= ',LCTTYPE';
 			}
 		}
-		if($this->type == 'J' && $aFilterTag['meeting'] ?? false){
+		if($this->type == 'J' && ($aFilterTag['meeting'] ?? false) && strpos($aFilterTag['sql'] ?? '', 'EvnMeetingId IS NOT NULL AND EvnMeetingId <>') === false){
 			if(!empty($aFilterTag['sql'] ?? null)) $aFilterTag['sql'] = '('.$aFilterTag['sql'].') AND ';
 			$aFilterTag['sql'] = ($aFilterTag['sql'] ?? '') . 'EvnMeetingId IS NOT NULL AND EvnMeetingId <> \'\'';
 		}
+
 		 		if($this->type=='G'){
 			$eventsline = $this->account->gwAPI->FunctionCall(
 					'GetItemList',
@@ -205,13 +216,35 @@ class GroupWareFolder extends Folder
 						$sIntervalFields
 				);
 				 			} else {
-				$this->function = 'GetEventDetailList';
+				$this->function = 'GetEventList';
 				if($this->getType()!='T'){
 					if(!isset($aFilterTag['groupchat_type']) || $aFilterTag['groupchat_type']!='events'){
 						$sFilter = ($sFilter?'('.$sFilter.') AND ':'');
 						$sFilter.= "( evnclass <> 'O' )";
 					}
 				}
+				if ($this->getType()=='N'){
+					$noteFielfs = array (
+						'evndocpass', 
+						'evndocinvite', 
+						'evndocexpire',
+						'evnurl',
+						'evnclass',
+						'\'\' as evnthumbnailid',
+						'\'\' as evnthumbnailticket',
+						'\'\' as evnthumbnailtime',
+						'\'\' as evnsizeinfo',
+						'\'\' as evnprocessingqueued',
+						'\'\' as evnticket'
+					);
+					$aNewFields = array();
+					foreach($noteFielfs as $field){
+						if(!in_array(strtolower($field),$aFields)){
+							$aFields[] = $field;
+						}
+					}
+					$fields = implode(',',$aFields);	
+				}
 				if($this->getType()=='F' || $this->getType()=='I'){
 					$docFields = array(
 						'evnlockown_email','evn_documenteditinginfo'					);
@@ -312,13 +345,6 @@ class GroupWareFolder extends Folder
 					}
 					$fields = $fields?($fields.','.$virtualFields):$virtualFields; 
 				}
-				if($this->getType() == 'J'){
-					foreach (['EvnFlags', 'EvnComplete', 'EvnUrl', 'EvnNote', 'Evn_Metadata'] as $fieldAddition) {
-						if(stripos($fields, $fieldAddition) !== false) continue;
-						$fields .= ',' . $fieldAddition;
-					}
-					$fields = trim($fields, ',');
-				}
 				if($sAttributes){
 					$sAttributes .=';use_tzid=1';
 				}else{
@@ -394,7 +420,7 @@ class GroupWareFolder extends Folder
 			$aEvents = array_merge($aOtherEvents,$aEvents);
 		}
 		foreach ($aEvents as $key => $aEvent) {
-			$aEvents[$key]['EVNMEETINGID'] = str_replace('@', '_', $aEvents[$key]['EVNMEETINGID'] ?? null);
+			$aEvents[$key]['EVNMEETINGID'] = str_replace('@', '_', $aEvents[$key]['EVNMEETINGID'] ?? '');
 		}
 		if(isset($aFilterTag['groupchat_type']) && $aFilterTag['groupchat_type'] == 'events'){
 			$virtualItem = $aEvents[0];
@@ -450,6 +476,28 @@ class GroupWareFolder extends Folder
 		return $aItems;
 	}
 
+	public function getItemsByIds(array $uids)
+	{
+		$result = [];
+		foreach ($uids as $uid) {
+			$result[] = $this->getItem($uid);
+		}
+		return $result;
+	}
+
+	 
+	protected function throwDuplicateItemsError(array $duplicateItems)
+	{
+		$error = '<duplicate>';
+		foreach ($duplicateItems as $duplicateItem) {
+			$error .= '<item><class>' . $duplicateItem['class'] . '</class>' .
+				'<fullpath>' . $duplicateItem['fullpath'] . '</fullpath>' .
+				'<name>' . $duplicateItem['name'] . '</name>' .
+				'<freename>' . $duplicateItem['freename'] . '</freename></item>';
+		}
+		$error .= '</duplicate>';
+		throw new XMLExc('items_duplicity', $error);
+	}
 
 	private function prepareItems($aNoJoin,&$sAttributes,&$sFilter)
 	{
@@ -457,7 +505,8 @@ class GroupWareFolder extends Folder
 				'GetContactDetailList',
 				$this->sFID,
 				$sFilter,
-				'ITM_ID'
+				'ITM_ID',
+				'ignore_dl=1'
 		);
 		$statement = '';
 		if ($aNoJoin){
@@ -569,7 +618,7 @@ class GroupWareFolder extends Folder
 			} else {
 				$bWithAddons = true;
 			}
-			$aEvent[0]['EVNMEETINGID'] = str_replace('@', '_', $aEvent[0]['EVNMEETINGID'] ?? null);
+			$aEvent[0]['EVNMEETINGID'] = str_replace('@', '_', $aEvent[0]['EVNMEETINGID'] ?? '');
 			return new GroupWareItem($this,$aEvent[0],$aAddonSelect,$fields,$bWithAddons,$ctz);
 		}else{
 			$error = $this->account->gwAPI->FunctionCall("GetLastError", $this->account->sGWSessionID);
@@ -589,10 +638,12 @@ class GroupWareFolder extends Folder
                     $partResult = false;
 					$code = '';
 					try{
-					   $partResult = $oItem->delete(false,false,$reason,$ignore_reason,$skip_trash);
+					   $partResult = (bool) $oItem->delete($reason,$ignore_reason,$skip_trash,$oItem->folder->folderID === '@@TRASH@@');
 					}catch(Exc $e){
+						 						if($e->wmcode=='imip_obsolete')	continue;
 						$code = $e->wmcode;
 					}
+
 					$result = $result && $partResult;
 					if (!$partResult && $code=='item_decline_failed_id') {
 						$failed[] = $oItem->itemID;
@@ -602,6 +653,7 @@ class GroupWareFolder extends Folder
 			if(!empty($failed) && !$ignore_reason){
 				throw new Exc('item_decline_failed_id',join('|',$failed));	
 			}
+
 		}else{
 			$sFID = $this->openAccess();
 			$result = $this->account->gwAPI->FunctionCall(
@@ -616,29 +668,7 @@ class GroupWareFolder extends Folder
 	}
 
 	public function moveItems(&$oFolder,$oItems = false,$cache = false){
-		if($oFolder->type=='V'){
-			if($oFolder->isEmpty()){
-				$sFID = '@@ALL@@#'.$oFolder->getType();
-				$folder = new GroupWareFolder(
-						$oFolder->account->account->gwAccount,
-						$sFID,
-						$oFolder->contentType
-				);
-			}else{
-                $ids = array();
-				foreach($oFolder->folders as $containFolder){
-					$ids[] = $containFolder->folderID;
-				}
-				$sFID = join( CRLF, $ids );
-				$folder = new GroupWareFolder(
-						$oFolder->account->account->gwAccount,
-						$sFID,
-						$oFolder->contentType
-				);
-		
-			}
-			$oFolder = $folder;
-		}
+		if($oFolder->type=='V') $this->getFolderFromVirtual($oFolder);
         $duplicateItems = array();
 		if($oItems){
 			$result = true;
@@ -662,52 +692,14 @@ class GroupWareFolder extends Folder
 			}
 		}else{
 			$sFID = $this->openAccess();
-			$result = $this->account->gwAPI->FunctionCall(
-					'MoveItem',
-					$sFID,
-					'',
-					MerakGWAPI::encode($oFolder->name)
-			);
+			$result = $this->account->gwAPI->FunctionCall('MoveItem', $sFID, '', MerakGWAPI::encode($oFolder->name));
 		}
-		if($duplicateItems){
-			$error = '<duplicate>';
-			foreach($duplicateItems as $duplicateItem){
-				$error.='<item><class>'.$duplicateItem['class'].'</class>'.
-								'<fullpath>'.$duplicateItem['fullpath'].'</fullpath>'.
-								'<name>'.$duplicateItem['name'].'</name>'.
-								'<freename>'.$duplicateItem['freename'].'</freename></item>';
-			}
-			$error .= '</duplicate>';
-			throw new XMLExc('items_duplicity',$error);
-		}
-			
+		if ($duplicateItems) $this->throwDuplicateItemsError($duplicateItems);
 		return $result;
 	}
 
 	public function copyItems(&$oFolder,$oItems = false,$cache = false){
-		if($oFolder->type=='V'){
-			if($oFolder->isEmpty()){
-				$sFID = '@@ALL@@#'.$oFolder->getType();
-				$folder = new GroupWareFolder(
-						$oFolder->account->account->gwAccount,
-						$sFID,
-						$oFolder->contentType
-				);
-			}else{
-				$ids = array();
-				foreach($oFolder->folders as $containFolder){
-					$ids[] = $containFolder->folderID;
-				}
-				$sFID = join( CRLF, $ids );
-				$folder = new GroupWareFolder(
-						$oFolder->account->account->gwAccount,
-						$sFID,
-						$oFolder->contentType
-				);
-		
-			}
-			$oFolder = $folder;
-		}
+		if($oFolder->type=='V') $this->getFolderFromVirtual($oFolder);
 		
 		$duplicateItems = array();
 		if($oItems){
@@ -733,24 +725,9 @@ class GroupWareFolder extends Folder
 			}
 		}else{
 			$sFID = $this->openAccess();
-			$result = $this->account->gwAPI->FunctionCall(
-					'CopyItem',
-					$sFID,
-					'',
-					MerakGWAPI::encode($oFolder->name)
-			);
-		}
-		if($duplicateItems){
-			$error = '<duplicate>';
-			foreach($duplicateItems as $duplicateItem){
-				$error.='<item><class>'.$duplicateItem['class'].'</class>'.
-								'<fullpath>'.$duplicateItem['fullpath'].'</fullpath>'.
-								'<name>'.$duplicateItem['name'].'</name>'.
-								'<freename>'.$duplicateItem['freename'].'</freename></item>';
-			}
-			$error .= '</duplicate>';
-			throw new XMLExc('items_duplicity',$error);
+			$result = $this->account->gwAPI->FunctionCall('CopyItem', $sFID, '', MerakGWAPI::encode($oFolder->name));
 		}
+		if ($duplicateItems) $this->throwDuplicateItemsError($duplicateItems);
 		if(!empty($result_id)){
 			return $result_id;
 		}else{
@@ -758,6 +735,89 @@ class GroupWareFolder extends Folder
 		}
 	}
 
+	protected function getFolderFromVirtual($oFolder)
+	{
+		return $this->account->gwAccount->getFdr($oFolder);
+	}
+
+	public function moveUids(&$oFolder, array $uids, array $duplicities, array $renames)
+	{
+		return $this->transferUids($oFolder, $uids, $duplicities, $renames, 'move');
+	}
+
+	public function copyUids(&$oFolder, array $uids, array $duplicities, array $renames)
+	{
+		return $this->transferUids($oFolder, $uids, $duplicities, $renames, 'copy');
+	}
+
+	private function transferUids(&$oFolder, array $uids, array $duplicities, array $renames, $action = 'copy' )
+	{
+		switch(strtoupper($action)){
+			case 'COPY':
+				$gwFunction = 'CopyItem';
+				break;
+			case 'MOVE':
+				$gwFunction = 'MoveItem';
+		}
+		if($oFolder->type=='V')	$oFolder = $this->getFolderFromVirtual($oFolder);
+		$this->openAccess();
+		if(!$uids) {
+			return $this->account->gwAPI->FunctionCall($gwFunction, $this->sFID, '', MerakGWAPI::encode($oFolder->name));
+		}
+		$duplicateItems = array();
+		$result = true;
+		$account = $this->account;
+		foreach($uids as $uid){
+			$partResult = false;
+			try{
+				$duplicity = '';
+				 				if($duplicities[$uid] == 'rename') {
+					if($renames[$uid]==''){
+						$item = $this->getItem($uid);
+						$renames[$uid] = GroupWareItem::proposeFreeFileName($this, $item->item['EVNLOCATION']);
+					}
+					$duplicity = ';COPYRENAMETO=' . urlencode($renames[$uid]);
+				}elseif($duplicities[$uid] == 'replace'){
+					$duplicity = ';FORCEREPLACE=1';
+				}
+				if(is_callable([$oFolder, 'handleCopyItemFrom'])){
+					if (!$item){
+						$item = $this->getItem($uid);
+					}
+					return $oFolder->handleCopyItemFrom($this, $item);
+				}
+				 				if(!($newID = $account->gwAPI->FunctionCall($gwFunction, $this->sFID, $uid, MerakGWAPI::encode($oFolder->folderID), $duplicity))){
+					$lastError = GroupWareItem::getError($account->gwAPI);
+					if($lastError == 13){
+						if (!$item){
+							$item = $this->getItem($uid);
+						}
+						throw new Exc('item_duplicity',$item->item['EVNTITLE']);
+					}
+					throw new Exc('item_copy',$uid);
+				}
+				$result_id[$uid] = $newID;
+				$partResult = true;
+			}catch(Exc $e){
+				if($e->wmcode == 'item_duplicity'){
+					$duplicate = array();
+					$duplicate['class'] = 'item';
+					$duplicate['fullpath'] = $oFolder->account->account->accountID.'/'.$this->folderID.'/'.$uid;
+					$duplicate['name'] = $e->getMessage();
+					$duplicate['freename'] = GroupWareItem::proposeFreeFileName($oFolder, $duplicate['name']);
+					$duplicateItems[] = $duplicate;
+				}else{
+					throw $e;
+				}
+			}
+			$result = $result && $partResult;
+		}
+		if ($duplicateItems) $this->throwDuplicateItemsError($duplicateItems);
+		if($action=='copy' &&!empty($result_id)) return $result_id;
+		return true;
+	}
+
+
 	public function saveItems($oItems = false)
 	{
 		return $this->account->gwAPI->saveItems($this,$oItems);
@@ -779,12 +839,13 @@ class GroupWareFolder extends Folder
 					$this->sFID,
 					$filter
 			);
-		}else if ($this->getType()=="C"){
-			$countFunction = 'GetContactDetailCount';
+		}else if ($this->getType()=="C"){			
 			if(empty($filter)){
 				$countFunction = 'getContactCount';
 			}else{
+				$countFunction = 'GetContactDetailCount';
 				$filter = ($filter?$filter." AND ":"").self::$contactQuery;
+				$sAttributes .= ($sAttributes?';':'') . 'ignore_dl=1';
 			}
 			$count = $this->account->gwAPI->FunctionCall($countFunction, $this->sFID, $filter, $sAttributes);
 		}else{
@@ -853,9 +914,12 @@ class GroupWareFolder extends Folder
 		 		if (!$account->sGWSessionID = $account->gwAPI->OpenGroup("*")){
 			throw new Exc('group_open','*');
 		}
-		
+
+		if($_SESSION['GUEST_ACCOUNT']){
+			throw new Exc('guest_folder_create');
+		}
 		 		$folder = MerakGWAPI::encode($folder);
-		if($type=='I'){
+		if(($type == 'I') || ($type == 'F')){
 			$pos = strrpos($folder,'\\');
 			if($pos!==false){
 				$displayName = substr($folder,$pos+1);
@@ -871,9 +935,13 @@ class GroupWareFolder extends Folder
 		
 		$parameters = $private?'&FdrPrivate=1':'';
 
-		if($type=='I'){
+		if(($type == 'I') || ($type == 'F')){
 			$folderName = $account->gwAPI->FunctionCall("GenerateFolderId",$account->sGWSessionID);
-			$folder = $root.'\\'.$folderName;
+			if ($root){
+				$folder = $root.'\\'.$folderName;
+			}else{
+				$folder = $folderName;
+			}
 			$parameters.= '&FdrDisplay='.urlencode($displayName);
 		}
 		 		if (!$account->gwAPI->FunctionCall
@@ -885,8 +953,14 @@ class GroupWareFolder extends Folder
 						$parameters
 				)
 		){
-			if($account->gwAPI->FunctionCall("GetLastError",$account->sGWSessionID)==27){
-				throw new Exc('folder_already_exists', $folder);
+			$error_id = intval($account->gwAPI->FunctionCall("GetLastError",$account->sGWSessionID));
+			switch($error_id)
+			{
+				case 27:
+					throw new Exc('folder_already_exists', $folder);
+				case 2:
+				case 3:
+					throw new Exc('folder_create_rights', $folder);
 			}
 			throw new Exc('folder_create',$folder);
 		}else{
@@ -921,7 +995,8 @@ class GroupWareFolder extends Folder
 		$newName = MerakGWAPI::encode($newName);
 		if($checkExistance){
 			try{
-				 				$this->account->account->getFolder($newName);
+				 				$type = 'main';
+				$this->account->account->getFolder($newName, $type);
 				$exist = true;
 			}catch(Exc $e){
 				$exist = false;
@@ -936,86 +1011,28 @@ class GroupWareFolder extends Folder
 		 		$oldParent = substr($sName,0,strrpos($sName,'\\'));
 		 		$parent = substr($newName,0,strrpos($newName,'\\'));
 		$parentDecoded = MerakGWAPI::decode($parent);
-		$oldFolderID = $this->folderID;
 
 		if($parentDecoded && !isset($this->account->account->folders['gw'][$parentDecoded]) && $this->getType()!='I'){
 			$this->createParent($parentDecoded);
 		}
-		if($this->display || $this->getType()=='I'){
-			 			if(strrpos($sName,'\\')!=0){
-				$folderID = substr($sName,strrpos($sName,'\\')+1);
-			}else{
-				$folderID = $this->name;
+		if (!$this->account->gwAPI->FunctionCall
+				(
+						"RenameFolder",
+						$this->account->sGWSessionID,
+						$sName,
+						$newName
+				)
+		){
+			if($this->account->gwAPI->FunctionCall("GetLastError",$this->account->sGWSessionID)==27){
+				throw new Exc('folder_already_exists', $newName);
 			}
-			 			if(strrpos($newName,'\\')!=0){
-				$displayName = substr($newName,strrpos($newName,'\\')+1);
-			}else{
-				$displayName = $newName;
-			}
-			 			if($oldParent!=$parent){
-				$sNewName = $parent.'\\'.$folderID;
-				if (!$this->account->gwAPI->FunctionCall
-						(
-								"RenameFolder",
-								$this->account->sGWSessionID,
-								$sName,
-								IceWarpGWAPI::encode($sNewName)
-						)
-				){
-					throw new Exc('folder_rename');
-				}
-				if (!$this->account->gwAPI->FunctionCall
-						(
-								"AddFolder",
-								$this->sFID,
-								IceWarpGWAPI::encode($sNewName),
-								$this->type,
-								'&FdrDisplay='.urlencode($displayName)
-						)
-				){
-					throw new Exc('folder_rename');
-				}		
-				$this->folderID = IceWarpGWAPI::decode($sNewName);	
-			}else{
-				if (!$this->account->gwAPI->FunctionCall
-						(
-								"AddFolder",
-								$this->sFID,
-								$sName,
-								$this->type,
-								'&FdrDisplay='.urlencode($displayName)
-						)
-				){
-					throw new Exc('folder_rename');
-				}
-				$sNewName = $sName;
-				$this->folderID = IceWarpGWAPI::decode($sName);
-			}
-			if($this->display){
-				$this->display = $displayName;
-			}
-			$this->name = IceWarpGWAPI::decode($sNewName);		
-		}else{
-			if (!$this->account->gwAPI->FunctionCall
-					(
-							"RenameFolder",
-							$this->account->sGWSessionID,
-							$sName,
-							$newName
-					)
-			){
-				if($this->account->gwAPI->FunctionCall("GetLastError",$this->account->sGWSessionID)==27){
-					throw new Exc('folder_already_exists', $newName);
-				}
-				throw new Exc('folder_rename');
-			}
-			$this->folderID =IceWarpGWAPI::decode( $newName );
-			$this->name =IceWarpGWAPI::decode( $newName );
+			throw new Exc('folder_rename');
 		}
-
-		 		$this->account->gwAPI->CloseFolder($oldFolderID);
-		$folderInfo = $this->account->gwAPI->OpenFolder(MerakGWAPI::encode($this->folderID));
-		$sFolderInfo = $this->account->gwAPI->FunctionCall("GetFolderInfo",$folderInfo['fid']);
+		
+		 		$oldFolderID = $this->folderID;
+		$this->folderID = $newName;
+		$this->openAccess();
+		$sFolderInfo = $this->account->gwAPI->FunctionCall("GetFolderInfo",$this->sFID,'1');
 		@$aFolderInfo = reset($this->account->gwAPI->ParseParamLine($sFolderInfo));
 
 		$folder = $aFolderInfo['FDR_ID'];
@@ -1024,15 +1041,18 @@ class GroupWareFolder extends Folder
 		$displayName = $aFolderInfo['FDRDISPLAY'];
 		$groupChatUnread = $aFolderInfo['FDRGROUPCHATUNREAD'];
 		$groupRelativeFolder = $aFolderInfo['GROUPFOLDER'];
-		$rights = self::decodeRights($folderInfo['rights']);
+		$rights = self::decodeRights($this->folderInfo['rights']);
 			
 		$this->groupChatUnread = $groupChatUnread;
 		$this->groupRelativeFolder = $groupRelativeFolder;
 		$this->rigths = $rights;
+		$this->display = $displayName;
+		$this->name = MerakGWAPI::decode($folder);
+		$this->folderID = $folder;
 
 		 		$this->account->account = $_SESSION['user']->getAccount($this->account->account->accountID);
 
-		 		unset($this->account->account->folders['gw'][$oldFolderID]);	
+		 		unset($this->account->account->folders['gw'][$oldFolderID]);
 		$this->account->account->folders['gw'][MerakGWAPI::decode($this->folderID)] = $this;
 
 		 		if(($oMainFolder = $this->getMainFolder($sMainName)) && $bRenameDual){
@@ -1083,11 +1103,132 @@ class GroupWareFolder extends Folder
 		}
 	}
 
+	public function createItemFromModel(\server\model\AGwEventItem $item)
+	{
+		$isRecurrent = ($item instanceof \server\model\THasRecurrences) && !empty($item->getRecurrences());
+		if($isRecurrent){
+			$item->setCtz(null);
+			$item->setEvntimeformat('Z');
+		}
+		$itemClass = $this->getItemClass();
+		if($this->getType() == 'F' && !$itemClass::checkAttachmentName($item->getEvntitle())){
+			throw new Exc('attachment_name', $item->getEvntitle());
+		}
+		if(($this->getType() == 'I' && ($item->getEvnclass() == 'F' || $item->getEvnclass() == 'M')) && !$itemClass::checkAttachmentName($item->getEvntitle())){
+			throw new Exc('attachment_name', $item->getEvntitle());
+		}
+		if(method_exists($item, 'setCtz')) $item->setCtz(null);
+
+		$item->setEvnCreated();
+		$item->save();
+
+		 
+		return $item->getGroupWareItem();
+
+		 		if(!$sEvnId = $gwApi->FunctionCall("Add".$sFuncName."Info", $folderSessionId, $sParametersLine, "")){
+			$lastError = $gwApi->FunctionCall("GetLastError",$this->account->sGWSessionID);
+			if($lastError != 13) throw new Exc('item_create',$sEvnId);
+			if($this->subtype != 'U') throw new Exc('item_duplicity',$item->getEvntitle());
+
+			$freename = GroupWareItem::proposeFreeFileName($this, $item->getEvnlocation());
+			$originalFilename = $item->getEvnlocation();
+
+			$item->setEvntitle($freename);
+			$item->setEvnLocation($freename);
+			$item->setEvnrid($freename);
+
+			$sParameters = $item->getParamLine();
+			$sEvnId = $gwApi->FunctionCall("Add".$sFuncName."Info", $folderSessionId, $sParameters);
+		}
+		$item->setId($sEvnId);
+		$item->setEvnCreated();
+		$gwApi->FunctionCall('SetEventNoteText', $folderSessionId, $item->getId(), $item->getNote());
+
+		 		 		$aItem = $item->getAsArray();
+		$item = new GroupWareItem($this, $aItem);
+
+		 		 		$item->processTZID($aItem,false, $isRecurrent);
+
+		 		$item->getAddons();
+		 		if ($item->aAddons) {
+			foreach ($item->aAddons as $addon) {
+				$addonResult = $addon->process($aTreeItem);
+				if ($addon->sAddonType == 'recurrence' && isset($aTreeItem['@childnodes']['recurrences'])) {
+					$data = $addon->getData();
+					$data = $data[0];
+					$rcrID = $data['RCR_ID'];
+				}
+				if ($addon->sAddonType == 'reaction') {
+					parse_str($addonResult, $data);
+					$item->reactions_metadata = $data['reactions'];
+				}
+				if ($addon->sAddonType == 'attachment') {
+					if ($item->folder->getType() == 'I' || $item->folder->getType() == 'F') {
+						$item->att_webdav_link = User::addTokenToTicket($addon->GetAttachmentFile($addonResult['id'], 'READONLYTICKET'));
+						if ($originalFilename) {
+							$item->att_webdav_link .= '&filename=' . urlencode($originalFilename);
+						}
+						$item->att_size = $addonResult['size'];
+						$contentType = $addonResult['type'];
+					} else {
+						$data = $addon->getData();
+						$item->att_webdav_link = User::addTokenToTicket($addon->GetAttachmentFile($data['ATTNAME'], 'TICKET'));
+						$item->att_size = $item->item['EVNCOMPLETE'];
+					}
+					$noteReplace = 'attachment_id_'.md5($addonResult['id']);
+					if(strpos($addonNote, $noteReplace) !== false){
+						$attachmentPath = User::addTokenToTicket($folder->account->gwAPI->FunctionCall("getAttachmentPath", $sFID, $sEvnId, $addonResult['id'], 'READONLYTICKET'));
+						$addonNote = str_replace('attachment_id_'.md5($addonResult['id']), $attachmentPath, $addonNote);
+						$aItem[$sNote] = $addonNote;
+					}
+				}
+
+				 				if($folder->getType() == 'I' && ($aItem['EVNCLASS'] == 'M' || $contentType == 'message/rfc822') && $addon->sAddonType == 'attachment' && (!isset($aItem['EVNCLASS']) || $aItem['EVNCLASS'] == 'F')){
+					if($contentType == 'message/rfc822'){
+						$aItem['EVNCLASS'] = 'M';
+						 						unset($item->aAddons);
+						$item->getAddons('M');
+					}
+					 					$mailID = $item->getFileAttachmentID();
+					$file = $addon->getAttachmentFile($mailID);
+					$item = GroupWareItem::createFromMessage($folder, $file, ($aItem['EVNCLASS'] == 'M' ? ($aItem['EVNNOTE'] ? $aItem['EVNNOTE'] : $aItem['EVNTITLE']) : ''), $item, $aItem);
+				}
+			}
+		}
+
+		try{
+			 			$item->processTags($aItem);
+		}catch(Exc $e){
+			throw new Exc('item_create');
+		}
+		 		$status = $aItem['EVNSTATUS'];
+		$item->occurrenceID = $item->processStatus($aItem,$aTreeItem,true,$rcrID);
+		if($status=='M' && $status <> $aItem['EVNSTATUS']){
+			 			$folder->account->gwAPI->FunctionCall("Add".$sFuncName."Info", $sFID, 'evnstatus=B', $sEvnId);
+		}
+		if(isset($aItem[$sNote])){
+			if($folder->getType() == 'N'){
+				$folder->account->gwAPI->FunctionCall("setEventNoteText", $sFID, $sEvnId, $aItem[$sNote]);
+			}else{
+				$folder->account->gwAPI->FunctionCall("Add".$sFuncName."Info", $sFID, '&' . $folder->account->gwAPI->CreateParamLine([$sNote => $aItem[$sNote]]), $sEvnId);
+			}
+		}
+		 		$finalizeParams = $item->folder->getType() == 'I' ? '&return_link_id=1' : '';
+		$id = $folder->account->gwAPI->FunctionCall("Add".$sFuncName."Info", $sFID, $finalizeParams, $sEvnId);
+		if($item->folder->getType() == 'I'){
+			parse_str($id,$result);
+			if($result['linkid']){
+				$item->linkID = $result['linkid'];
+			}
+		}
+		return $item;
+	}
+
+
 	 
 	public function createItem($aItem,$aTreeItem)
 	{
 		if(($this->getType()=='F' || ($aItem['evnclass']=='F' || $aItem['evnclass']=='M') ) && !isset($aItem['evntitle'])){
-			 
 		  	return $this->createItemFile($aItem,$aTreeItem);
 		}else{
 			try{
@@ -1132,18 +1273,13 @@ class GroupWareFolder extends Folder
 
 	public function setAcl($aList,$bSetDual = true)
 	{
-		$this->account->setAcl($aList,$this->name,$bSetDual);
+		return $this->account->setAcl($aList,$this->name,$bSetDual);
 	}
 
-	public function getAcl()
+	public function getAcl($extended = false)
 	{
 		$this->openAccess();
-		if($this->getType()=='I' || $this->getType()=='Y'){
-			$expand_guests = true;
-		}else{
-			$expand_guests = false;
-		}
-		return $this->acl = $this->account->getAcl($this->name,$expand_guests);
+		return $this->acl = $this->account->getAcl($this->name, true, $extended);
 	}
 
 	public function getMyRights()
@@ -1174,6 +1310,7 @@ class GroupWareFolder extends Folder
 		}
 		return $folder;
 	}
+
 	 
 	public function openAccess()
 	{
@@ -1208,7 +1345,7 @@ class GroupWareFolder extends Folder
 	public static function decodeRights($gwRights)
 	{
 		$rights = 0;
-		 		if(abs($gwRights) > 1024){
+		 		if(abs((int)$gwRights) > 1024){
 			if($gwRights & MerakGWAPI::RIGHT_READ ) $rights |= Folder::RIGHT_READ;
 			if($gwRights & MerakGWAPI::RIGHT_WRITE ) $rights |= Folder::RIGHT_WRITE;
 			if($gwRights & MerakGWAPI::RIGHT_MODIFY ) $rights |= Folder::RIGHT_MODIFY;
@@ -1228,25 +1365,27 @@ class GroupWareFolder extends Folder
 			if($gwRights & MerakGWAPI::RIGHT_FOLDER_EDIT_DOCUMENT) $rights |= Folder::RIGHT_FOLDER_EDIT_DOCUMENT;
 		 		}else{
 			$gwRights = MerakGWAPI::$aRightsToString[$gwRights];
-			foreach (str_split($gwRights) as $right) {
-				switch ($right) {
-					case 'r':
-						$rights |= Folder::RIGHT_FOLDER_READ;
-						$rights |= Folder::RIGHT_READ;
-						break;
-					case 'w':
-						$rights |= Folder::RIGHT_FOLDER_WRITE;
-						$rights |= Folder::RIGHT_WRITE;
-						break;
-					case 'm':
-						$rights |= Folder::RIGHT_FOLDER_MODIFY;
-						$rights |= Folder::RIGHT_MODIFY;
-						break;
-					case 'd':
-						$rights |= Folder::RIGHT_FOLDER_DELETE;
-						$rights |= Folder::RIGHT_DELETE;
-						break;
-					case 'o':  $rights |= Folder::RIGHT_ADMIN; break;
+			if ($gwRights) {
+				foreach (str_split($gwRights) as $right) {
+					switch ($right) {
+						case 'r':
+							$rights |= Folder::RIGHT_FOLDER_READ;
+							$rights |= Folder::RIGHT_READ;
+							break;
+						case 'w':
+							$rights |= Folder::RIGHT_FOLDER_WRITE;
+							$rights |= Folder::RIGHT_WRITE;
+							break;
+						case 'm':
+							$rights |= Folder::RIGHT_FOLDER_MODIFY;
+							$rights |= Folder::RIGHT_MODIFY;
+							break;
+						case 'd':
+							$rights |= Folder::RIGHT_FOLDER_DELETE;
+							$rights |= Folder::RIGHT_DELETE;
+							break;
+						case 'o':  $rights |= Folder::RIGHT_ADMIN; break;
+					}
 				}
 			}
 		}
@@ -1310,7 +1449,8 @@ class GroupWareFolder extends Folder
 							"GetContactDetailList",
 							$this->sFID,
 							'((ITMCLASS IS Null OR ITMCLASS=\'C\' OR ITMCLASS=\'\'))',
-							"ITMGRP_ID,$select"
+							"ITMGRP_ID,$select",
+							'ignore_dl=1'
 					)
 			);
 			             $list = array();
@@ -1321,69 +1461,73 @@ class GroupWareFolder extends Folder
 			}
 			if(is_array($list)){
 				foreach ($list as $val){
-					$address["ID"]=$val['H'][ITM_ID];
-					 					$address["CONTACTNAME"]=$val['H'][ITMCLASSIFYAS];
-					$address["FIRSTNAME"] = $val['H'][ITMFIRSTNAME];
-					$address["MIDDLENAME"] = $val['H'][ITMMIDDLENAME];
-					$address["LASTNAME"] =$val['H'][ITMSURNAME];
-					$address["NICKNAME"] = $val['H'][ITMNICKNAME];
-					$address["TITLE"] = $val['H'][ITMTITLE];
-					$address["SUFFIX"] = $val['H'][ITMSUFFIX];
-					$address["CATEGORY"]= $val['H'][ITMCATEGORY];
-					 					$address["EMAIL"] =  $val['H'][LCTEMAIL1];
-					$address["EMAIL2"] = $val['H'][LCTEMAIL2];
-					$address["EMAIL3"] = $val['H'][LCTEMAIL3];
-					 					$address["PHONE_WORK"] = $val['H'][LCTPHNWORK1];
-					$address["PHONE_HOME"] = $val['H'][LCTPHNHOME1];
-					$address["PHONE_HOME2"] = $val['H'][LCTPHNHOME2];
-					$address["PHONE_MOBILE"] = $val['H'][LCTPHNMOBILE];
-					$address["PHONE_FAX_WORK"] = $val['H'][LCTPHNFAXWORK];
-					$address["PHONE_FAX_HOME"] = $val['H'][LCTPHNFAXHOME];
-					$address["PHONE_FAX_OTHER"] = $val['H'][LCTPHNOTHERFAX];
+					$address["ID"]=$val['H']['ITM_ID'];
+					 					$address["CONTACTNAME"]=$val['H']['ITMCLASSIFYAS'];
+					$address["FIRSTNAME"] = $val['H']['ITMFIRSTNAME'];
+					$address["MIDDLENAME"] = $val['H']['ITMMIDDLENAME'];
+					$address["LASTNAME"] =$val['H']['ITMSURNAME'];
+					$address["NICKNAME"] = $val['H']['ITMNICKNAME'];
+					$address["TITLE"] = $val['H']['ITMTITLE'];
+					$address["SUFFIX"] = $val['H']['ITMSUFFIX'];
+					$address["CATEGORY"]= $val['H']['ITMCATEGORY'];
+					 					$address["EMAIL"] =  $val['H']['LCTEMAIL1'];
+					$address["EMAIL2"] = $val['H']['LCTEMAIL2'];
+					$address["EMAIL3"] = $val['H']['LCTEMAIL3'];
+					 					$address["PHONE_WORK"] = $val['H']['LCTPHNWORK1'];
+					$address["PHONE_HOME"] = $val['H']['LCTPHNHOME1'];
+					$address["PHONE_HOME2"] = $val['H']['LCTPHNHOME2'];
+					$address["PHONE_MOBILE"] = $val['H']['LCTPHNMOBILE'];
+					$address["PHONE_FAX_WORK"] = $val['H']['LCTPHNFAXWORK'];
+					$address["PHONE_FAX_HOME"] = $val['H']['LCTPHNFAXHOME'];
+					$address["PHONE_FAX_OTHER"] = $val['H']['LCTPHNOTHERFAX'];
 					
-					 					$bday = $val['H'][ITMBDATE];
-					$anniversary = $val['H'][ITMANNIVERSARY];
+					 					$bday = $val['H']['ITMBDATE'];
+					$anniversary = $val['H']['ITMANNIVERSARY'];
 					if($format){
 					  $php_format = slToolsDate::ClassicToPHPFormat($format);	
-					  $bday = date($php_format,MerakGWAPI::calendar2unixTime($bday, 0));
-					  $anniversary = date($php_format,MerakGWAPI::calendar2unixTime($anniversary, 0));     	
+					  if ($bday) {
+					  	$bday = date($php_format,MerakGWAPI::calendar2unixTime($bday, 0));
+					  }
+					  if ($anniversary) {
+					  	$anniversary = date($php_format,MerakGWAPI::calendar2unixTime($anniversary, 0));     	
+					  }
 					}
 					$address["BIRTHDAY".$formatstr] = $bday;
 					$address["ANNIVERSARY".$formatstr] = $anniversary;
 					
-					$address["GENDER"] =$val['H'][ITMGENDER];
-					$address["SPOUSE"] = $val['H'][ITMSPOUSE];
-					 					$address["STREET"] = $val['H'][LCTSTREET];
-					$address["CITY"] = $val['H'][LCTCITY];
-					$address["ZIP"] = $val['H'][LCTZIP];
-					$address["STATE"] = $val['H'][LCTSTATE];
-					$address["COUNTRY"] = $val['H'][LCTCOUNTRY];
-					 					$address["HOMEPAGE"] = $val['H'][LCTWEBPAGE];
-					$address["FREEBUSYURL"] = $val['H'][ITMINTERNETFREEBUSY];
-					$address["IM"] = $val['H'][LCTIM];
+					$address["GENDER"] =$val['H']['ITMGENDER'];
+					$address["SPOUSE"] = $val['H']['ITMSPOUSE'];
+					 					$address["STREET"] = $val['H']['LCTSTREET'];
+					$address["CITY"] = $val['H']['LCTCITY'];
+					$address["ZIP"] = $val['H']['LCTZIP'];
+					$address["STATE"] = $val['H']['LCTSTATE'];
+					$address["COUNTRY"] = $val['H']['LCTCOUNTRY'];
+					 					$address["HOMEPAGE"] = $val['H']['LCTWEBPAGE'];
+					$address["FREEBUSYURL"] = $val['H']['ITMINTERNETFREEBUSY'];
+					$address["IM"] = $val['H']['LCTIM'];
 
-					 					$address["COMPANY"]=$val['H'][ITMCOMPANY];
-					$address["JOB"] = $val['H'][ITMJOBTITLE];
-					$address["PROFESSION"] = $val['H'][ITMPROFESSION];
-					$address["DEPARTMENT"] = $val['H'][ITMDEPARTMENT];
-					$address["ASSISTANT"] = $val['H'][ITMASSISTANTNAME];
-					$address["MANAGER"] = $val['H'][ITMMANAGERNAME];
-					$address["OFFICELOCATION"] = $val['H'][ITMOFFICELOCATION];
-					 					$address["NOTES"] =$val['H'][ITMDESCRIPTION];
-					$address["SHARETYPE"] = $val['H'][ITMSHARETYPE];
+					 					$address["COMPANY"]=$val['H']['ITMCOMPANY'];
+					$address["JOB"] = $val['H']['ITMJOBTITLE'];
+					$address["PROFESSION"] = $val['H']['ITMPROFESSION'];
+					$address["DEPARTMENT"] = $val['H']['ITMDEPARTMENT'];
+					$address["ASSISTANT"] = $val['H']['ITMASSISTANTNAME'];
+					$address["MANAGER"] = $val['H']['ITMMANAGERNAME'];
+					$address["OFFICELOCATION"] = $val['H']['ITMOFFICELOCATION'];
+					 					$address["NOTES"] =$val['H']['ITMDESCRIPTION'];
+					$address["SHARETYPE"] = $val['H']['ITMSHARETYPE'];
 					
-					 					$address["STREETB"] = $val['B'][LCTSTREET];
-					$address["CITYB"] = $val['B'][LCTCITY];
-					$address["ZIPB"] = $val['B'][LCTZIP];
-					$address["STATEB"] = $val['B'][LCTSTATE];
-					$address["COUNTRYB"] = $val['B'][LCTCOUNTRY];
-					$address["WEB"] = $val['B'][LCTWEBPAGE];
-					$address["HOMEPAGE"] = $val['H'][LCTWEBPAGE];
-					 					$address["STREETO"] = $val['O'][LCTSTREET];
-					$address["CITYO"] = $val['O'][LCTCITY];
-					$address["ZIPO"] = $val['O'][LCTZIP];
-					$address["STATEO"] = $val['O'][LCTSTATE];
-					$address["COUNTRYO"] = $val['O'][LCTCOUNTRY];
+					 					$address["STREETB"] = $val['B']['LCTSTREET'];
+					$address["CITYB"] = $val['B']['LCTCITY'];
+					$address["ZIPB"] = $val['B']['LCTZIP'];
+					$address["STATEB"] = $val['B']['LCTSTATE'];
+					$address["COUNTRYB"] = $val['B']['LCTCOUNTRY'];
+					$address["WEB"] = $val['B']['LCTWEBPAGE'];
+					$address["HOMEPAGE"] = $val['H']['LCTWEBPAGE'];
+					 					$address["STREETO"] = $val['O']['LCTSTREET'];
+					$address["CITYO"] = $val['O']['LCTCITY'];
+					$address["ZIPO"] = $val['O']['LCTZIP'];
+					$address["STATEO"] = $val['O']['LCTSTATE'];
+					$address["COUNTRYO"] = $val['O']['LCTCOUNTRY'];
 					
 					$addresses[$address["ID"]]= $address;
 					unset($address);
@@ -1395,13 +1539,11 @@ class GroupWareFolder extends Folder
 	static public function saveAddressesToFile(&$file, $addresses, $separator = delimiterchar, $private = true)
 	{
 		slSystem::import('tools/charset');
-		 		function ARcmp($a, $b)
-		{
+		 		usort($addresses, function($a, $b) {
 			if ($a["ID"] == $b["ID"])
 				return 0;
 			return ($a["ID"] > $b["ID"]) ? 1 : -1;
-		}
-		usort($addresses, "ARcmp");
+		});
 		if ($addresses) {
 			 			$headerAddr = reset($addresses);
 			unset($headerAddr["ID"]);
@@ -1489,7 +1631,7 @@ class GroupWareFolder extends Folder
 		return $this->isGAL;
 	}
 	
-	public function isShared()
+	public function isShared($account = '')
 	{
 		return ($this->shared || Folder::hasSharedRoot($this->name));
 	}
@@ -1516,7 +1658,19 @@ class GroupWareFolder extends Folder
 			}
 		}
 	}
+
+	public function makeOwner($sOwner)
+	{
+		$this->openAccess();
+		$this->account->gwAPI->FunctionCall('MakeRoomOwner', $this->sFID, $sOwner);	
+	}
 	
+	public function resetOwner($sOwner)
+	{
+		$this->openAccess();
+		$this->account->gwAPI->FunctionCall('ResetRoomOwner', $this->sFID, $sOwner);	
+	}
+
 	 
 	public function setSubscription($state)
 	{
@@ -1574,5 +1728,14 @@ class GroupWareFolder extends Folder
 		}
 		return implode(",",$array);
 	}
+
+	public function getInfo()
+	{
+		$this->openAccess();
+		$sFolderInfo = $this->account->gwAPI->FunctionCall("GetFolderInfo",$this->sFID,'0','1');
+		$aFolderInfo = reset($this->account->gwAPI->ParseParamLine($sFolderInfo));
+		return $aFolderInfo;
+	}
+
 }
 ?>
