@@ -4,7 +4,7 @@ class Hipaa
 	public static function setValue($var,$val,$email=false)
 	{
 		if(!$email){$email=$_SESSION['EMAIL'];}
-		
+
 		if($var=='name')
 		{
 			return self::setAccountProperty('U_Name',$val,$email);
@@ -27,11 +27,11 @@ class Hipaa
 		}
 		return false;
 	}
-	
+
 	public static function getValue($var,$email=false)
 	{
 		if(!$email){$email=$_SESSION['EMAIL'];}
-		
+
 		if($var=='name')
 		{
 			return self::getAccountProperty('U_Name',$email);
@@ -51,7 +51,7 @@ class Hipaa
 		}
 		return false;
 	}
-	
+
 	static public function createItem($email,$parameters)
 	{
 		 		$category = $parameters['category'];
@@ -59,17 +59,17 @@ class Hipaa
 		$zip = $parameters['zip'];
 		$city = $parameters['city'];
 		$state = $parameters['state'];
-		
+
 		$notify_email = $parameters['notify_email'];
 		$notify_sms = $parameters['notify_sms'];
-		$smsuser = '';  		$smspass = '';  		
+		$smsuser = '';  		$smspass = '';  
 		$result = true;
 		$parts = self::splitEmailAddress($email);
 		$username = $parts['username'];
 		$domain = $parts['domain'];
 		$notification_alias = $username.'.notification@'.$domain;
 		$sms_notification_alias = $username.'.sms@'.$domain;
-		
+
 		$account = new IceWarpAccount();
 		 		$account->New_($notification_alias);
 		$account->SetProperty('u_type',3);
@@ -86,13 +86,13 @@ class Hipaa
 			$result = $result && $account->SetProperty('u_name',$name);
 		}
 		 		$result = $result && $account->SetProperty('u_authmodevalue',$category);
-		
+
 		 		$pin = rand(0,9).rand(0,9).rand(0,9).rand(0,9);
 		$result = $result && $account->SetProperty('u_password',$pin);
 		$result = $result && $account->Save();
-		
+
 		$result = $result && self::sendCredentials($notify_email,$username,$pin);
-		
+
 		$account->Open($email);
 		 		 
 		 		$mailbox = $account->GetProperty('u_fullmailboxpath');
@@ -116,7 +116,7 @@ class Hipaa
 		 		$result = $result && self::linkPublicFolders($email,$category);
 		return $result;
 	}
-	
+
 	public static function editItem($email,$parameters)
 	{
 		 		$category = $parameters['category'];
@@ -126,14 +126,14 @@ class Hipaa
 		$state = $parameters['state'];
 		$notify_email = $parameters['notify_email'];
 		$notify_sms = $parameters['notify_sms'];
-		$smsuser = '';  		$smspass = '';  		
+		$smsuser = '';  		$smspass = '';  
 		$result = true;
 		$parts = self::splitEmailAddress($email);
 		$username = $parts['username'];
 		$domain = $parts['domain'];
 		$notification_alias = $username.'.notification@'.$domain;
 		$sms_notification_alias = $username.'.sms@'.$domain;
-		
+
 		$account = new IceWarpAccount();
 		 		$account->Open($notification_alias);
 		$account->SetProperty('u_type',3);
@@ -149,11 +149,11 @@ class Hipaa
 		 		if($parameters['name']){
 			$result = $result && $account->SetProperty('u_name',$name);
 		}
-		$result = $result && $account->Save();		
+		$result = $result && $account->Save();
 		$result = $result && self::setAccountCardProperties($email, $parameters);
 		return $result;
 	}
-	
+
 	public static function deleteItem($email, $category = 'patients')
 	{
 		$result = true;
@@ -162,7 +162,7 @@ class Hipaa
 		$domain = $parts['domain'];
 		$notification_alias = $username.'.notification@'.$domain;
 		$sms_notification_alias = $username.'.sms@'.$domain;
-		
+
 		$account = new IceWarpAccount();
 		$result = $result && $account->Open($notification_alias);
 		$result = $result && $account->Delete();
@@ -170,18 +170,17 @@ class Hipaa
 		$result = $result && $account->Delete();
 		$result = $result && $account->Open($email);
 		$result = $result && $account->Delete();
-		
-		$result = $result && self::deleteChessCard($email,$category);
-		$result = $result && self::detachFromGroup($email,$category);
+
+		 		 		$result = $result && self::detachFromGroup($email,$category);
 		return $result;
 	}
-	
+
 	private static function getNotificationAccountProperty($prop,$notifytype='notification',$email=false)
 	{
 		if(!$email){$email=$_SESSION['EMAIL'];}
 		$parts = self::splitEmailAddress($email);
 		$notification_account=$parts['username'].($notifytype?'.'.$notifytype:'').'@'.$parts['domain'];
-		
+
 		$account=new IceWarpAccount();
 		if(!$account->open($notification_account))
 		{
@@ -189,13 +188,13 @@ class Hipaa
 		}
 		return $account->GetProperty($prop);
 	}
-	
+
 	private static function setNotificationAccountProperty($prop,$val,$notifytype='notification',$email=false)
 	{
 		if(!$email){$email=$_SESSION['EMAIL'];}
 		$parts = self::splitEmailAddress($email);
 		$notification_account=$parts['username'].($notifytype?'.'.$notifytype:'').'@'.$parts['domain'];
-		
+
 		$account=new IceWarpAccount();
 		if(!$account->open($notification_account))
 		{
@@ -207,7 +206,7 @@ class Hipaa
 		}
 		return $account->save();
 	}
-	
+
 	private static function getAccountProperty($prop,$email=false)
 	{
 		if(!$email){$email=$_SESSION['EMAIL'];}
@@ -218,7 +217,7 @@ class Hipaa
 		}
 		return $account->GetProperty($prop);
 	}
-	
+
 	private static function setAccountProperty($prop,$val,$email=false)
 	{
 		if(!$email){$email=$_SESSION['EMAIL'];}
@@ -292,7 +291,7 @@ class Hipaa
 		}
 		return false;
 	}
-	
+
 	private static function setDateOfBirth($val,$email=false)
 	{
 		if(!$email){$email=$_SESSION['EMAIL'];}
@@ -339,7 +338,7 @@ class Hipaa
 		}
 		return false;
 	}
-	
+
 	private static function getDateOfBirth($email=false)
 	{
 		if(!$email){$email=$_SESSION['EMAIL'];}
@@ -369,17 +368,17 @@ class Hipaa
 				{
 					return false;
 				}
-				
+
 				$ret['day']=$xml->birth->day;
 				$ret['month']=$xml->birth->month;
 				$ret['year']=$xml->birth->year;
-				
+
 				return $ret;
 			}
 		}
 		return false;
 	}
-	
+
 	private static function setAddress($val,$email=false)
 	{
 		if(!$email){$email=$_SESSION['EMAIL'];}
@@ -407,7 +406,7 @@ class Hipaa
 		}
 		return false;
 	}
-	
+
 	private static function getAddress($email=false)
 	{
 		if(!$email){$email=$_SESSION['EMAIL'];}
@@ -439,7 +438,7 @@ class Hipaa
 				'domain'=>$domain
 		);
 	}
-	
+
 	private static function autoCreateMyCard($email,$parameters = array(),$gw = false)
 	{
 		if($gw === false){
@@ -463,7 +462,7 @@ class Hipaa
 		$gw->Logout();
 		return true;
 	}
-	
+
 	public static function getAccountCardProperties($email,$gw = false)
 	{
 		if(!$gw){
@@ -503,7 +502,7 @@ class Hipaa
 		$result['zip'] = $item['LCTZIP'];
 		return $result;
 	}
-	
+
 	private static function setAccountCardProperties($email,$properties,$certificate = false)
 	{
 		$gw = self::superLogin();
@@ -532,7 +531,7 @@ class Hipaa
 				break;
 			}
 		}
-		
+
 		$id = $item['ITM_ID'];
 		$lctid = $item['LCT_ID'];
 		foreach($properties as $property =>$value){
@@ -567,17 +566,20 @@ class Hipaa
 			$path = $account->GetProperty('u_fullmailboxpath');
 			copy($certificate,$path.'cert.pem');
 		}
+
+		 		$result = $gw->FunctionCall("AddContactInfo",$fid,'',$id);
+
 		$gw->Logout();
 		return $result;
 	}
-	
+
 	private static function generateUserCertificate($email,$parameters)
 	{
 		$parts = self::splitEmailAddress($email);
 		$certificate = icewarp_create_certificate(2048,365,$parameters['country'],'',$parameters['street'],$parameters['zip'],'Sending and Receiving emails',$parameters['name'],$email);
 		return $certificate;
 	}
-	
+
 	private static function linkPublicFolders($email,$category = 'patients')
 	{
 		$account = new IceWarpAccount();
@@ -596,7 +598,7 @@ class Hipaa
 							</contacts>
 						</folders>
 					</personal>');
-			
+
 		}else{
 			$xml = new SimpleXMLElement('
 				<personal>
@@ -612,7 +614,7 @@ class Hipaa
 				</personal>');
 		}
 		return $xml->saveXML($wmDir.'folders.xml');
-		
+
 	}
 
 	private static function assignToGroup($email,$type = 'patients')
@@ -620,7 +622,7 @@ class Hipaa
 		$parts = self::splitEmailAddress($email);
 		$username = $parts['username'];
 		$domain = $parts['domain'];
-		
+
 		$group_name = $type.'group@'.$domain;
 		$group = new IceWarpAccount();
 		$group->Open($group_name);
@@ -645,13 +647,13 @@ class Hipaa
 		}
 		return true;
 	}
-	
+
 	private static function detachFromGroup($email, $type = 'patients')
 	{
 		$parts = self::splitEmailAddress($email);
 		$username = $parts['username'];
 		$domain = $parts['domain'];
-		
+
 		$group_name = $type.'group@'.$domain;
 		$group = new IceWarpAccount();
 		$group->Open($group_name);
@@ -677,27 +679,27 @@ class Hipaa
 		}
 		return true;
 	}
-	
+
 	private static function superLogin()
 	{
 		$api = IceWarpAPI::instance();
 		unset($_SESSION['GW']);
-		$gw = new MerakGWAPI('com.icewarp.webclient');
+		$gw = createobject('gwapi', 'com.icewarp.webclient');
 		$gw->user = $api->GetProperty('c_gw_superuser');
 		$gw->setPassword($api->GetProperty('c_gw_superpass'));
-		
+
 		if($gw->Login()){
 			return $gw;
 		}else{
 			return false;
 		}
-	
+
 	}
-	
+
 	private static function sendCredentials($to,$username,$pin)
 	{
 		$info = self::splitEmailAddress($to);
-			
+
 		$mailer = new Mail();
 		$mailer->XMailer.='-HIPPA';
 		$mailer->addTo($to);
