@@ -16,6 +16,8 @@ class IMAPFolder extends CacheFolder {
 	public $messages;
 	public $toSync;
 	public $fullSync;
+	public $publicRoot;
+	public $groupOwner;
 
 	 	 
 	public function __construct(&$account, $folderID, $name, $rights, $sync = false,$attributes = 0, $path = '',$validity = false,$messages = 0,$unseen = 0,$sync_update = '',$subscription_type = '0')
@@ -222,13 +224,13 @@ class IMAPFolder extends CacheFolder {
 	
 	public function setAcl($aList,$bSetDual = true)
 	{
-		$this->account->setAcl($aList,$this->name,$bSetDual);
+		return $this->account->setAcl($aList,$this->name,$bSetDual);
 	}
 	
-	public function getAcl()
+	public function getAcl($extended = false)
 	{
 		 
-		$this->acl = $this->account->getAcl( $this->name );
+		$this->acl = $this->account->getAcl( $this->name, $extended );
 		return $this->acl;
 	}
 	
@@ -794,7 +796,7 @@ class IMAPFolder extends CacheFolder {
 	
 	public function isLocal()
 	{
-		if($this->isLocal){
+		if($this->account->primary){
 			return true;
 		}
 		return false;
@@ -804,25 +806,29 @@ class IMAPFolder extends CacheFolder {
 	{
 		$account = &$this->account;
 		$cache = Cache::instance($_SESSION['user']);
+		$cache->aFilterTag = &$this->aFilterTag;
 		$count = $cache->countItems($this, $flags, $positive, $filter, $search);
 
 		if(!$filter && !$search && $flags==0){
 			if($this->messages!=$count){
 				if($this->account->isDelayed()){
-					 					$condition = '( flag_update > 1 OR date = -1 )';
-					$delayedItemsID =  $cache->getItemsIDByCondition($this,$condition,'LIMIT 0,1');
-					if(count($delayedItemsID)){
-						log_buffer($this->name."->countItems : count messages in cache(".$count.") and imap(".$this->messages.") are inconsistent >> skip fullsync due to delayed actions","EXTENDED");
+					$countDelayed = $cache->countDelayedItems($this);
+					if($countDelayed > 0){
+						log_buffer($this->name."->countItems : count messages in cache(".$count.") and imap(".$this->messages.") are inconsistent >> skip fullsync due to ".$countDelayed." delayed actions","EXTENDED");
 						$this->fullSync = false;
 					}else{
-						 						$imap = IMAP::instance($this->account);
-						$state = $imap->getMailboxStatus($this->name);
-						$modseq = $state->highestmodseq;
-						$this->toSync = true;
-						 						if($modseq <= $this->last_sync_update){
-							$this->fullSync = true;
+						 						try{
+							$imap = IMAP::instance($this->account);
+							$state = $imap->getMailboxStatus($this->name);
+							$modseq = $state->highestmodseq;
+							$this->toSync = true;
+							 							if($modseq <= $this->last_sync_update){
+								$this->fullSync = true;
+							}
+							log_buffer($this->name."->countItems : count messages in cache(".$count.") and imap(".$this->messages.") are inconsistent >> set ".$this->name."->fullSync = ".$this->fullSync.";".$this->name."->toSync = true;","EXTENDED");
+						}catch(Exc $e){
+							log_buffer($this->name."->countItems : count messages in cache(".$count.") and imap(".$this->messages.") are inconsistent >> skip actions due to IMAP servic not running","EXTENDED");
 						}
-						log_buffer($this->name."->countItems : count messages in cache(".$count.") and imap(".$this->messages.") are inconsistent >> set ".$this->name."->fullSync = ".$this->fullSync.";".$this->name."->toSync = true;","EXTENDED");
 					}
 				}else{
 					$this->toSync = true;
@@ -842,24 +848,6 @@ class IMAPFolder extends CacheFolder {
 
 	public function setNotify($bValue){}
 
-	 
-	protected function getFilteredItems(string $filter) : array
-	{
-		$aFilterTag = [];
-		$imapSearch = null;
-		if($_SESSION['FULLTEXT_SUPPORT']){
-			$imapSearch = new \server\inc\imap\Search($this, $filter);
-		}
-		if(!$imapSearch instanceof \server\inc\imap\Search || !$imapSearch->isSearchEnabled() || ($aFilterTag['sql'] = $imapSearch->getSearchSql()) === false) {
-			require_once(SHAREDLIB_PATH . 'tools/search.php');
-			$search = new SearchTool();
-			$search->setType($this->getType());
-			$search->setAccount($this->account);
-			$search->setFolder($this);
-			$aFilterTag = ['sql' => $search->parse($filter, $aFilterTag)];
-		}
-		return $this->getItems($aFilterTag) ?? [];
-	}
 
 	public function getTypeFromAttributes()
 	{
