@@ -74,8 +74,16 @@ class IMAP extends slObservable{
   
 	
 	 
-	private function __construct($server, $port = 143, $username, $password, $accountID, $secondary = array(),$charset = 'UTF-8', $current_ip = '127.2.0.1')
-	{
+	private function __construct(
+		$server,
+		$port = 143,
+		$username = '',
+		 		$password = '',
+		$accountID = '',
+		$secondary = array(),
+		$charset = 'UTF-8',
+		$current_ip = '127.2.0.1'
+	) {
 		try{
 			$this->connect($server, $port, $username, $password, $accountID);
 		}catch(Exc $e){
@@ -127,8 +135,13 @@ class IMAP extends slObservable{
 		}
 	}
 	
-	private function connect($server, $port, $username, $password, $accountID)
-	{
+	private function connect(
+		$server,
+		$port,
+		$username,
+		 		$password,
+		$accountID
+	) {
 		 		if ($port == 993) $attr = '/ssl/novalidate-cert'; else
 		 		if ($port == '110') $attr ='/pop3/notls'; else
 		 		if ($port == '995') $attr ='/pop3/ssl/novalidate-cert'; else
@@ -348,7 +361,7 @@ class IMAP extends slObservable{
 	 
 
 
-	 	public function getAcl($mailbox,$encoded = true,$decodeRights = true)
+	 	public function getAcl($mailbox,$encoded = true,$decodeRights = true, $extended = false)
 	{
 		if(!$this->getCapability('ACL')){
 			return array($this->accountID=>IMAPFolder::decodeRights(self::$fullRights));
@@ -567,19 +580,24 @@ class IMAP extends slObservable{
     	if(!$file){
     		 
   			 
-  			if (!(@$result = imap_append($this->imap, $encoded_mailbox, $message, $options, $date))) self::error('imap_create_message');
-  			
+  			if (!(@$result = imap_append($this->imap, $encoded_mailbox, $message, $options, $date))) {
+				$quota_error = $_SESSION['user']->checkMailboxQuota(strlen($message));
+				self::error($quota_error?'imap_quota_exceeded':'imap_message_create');
+			}
 		  	$parser = new MailParse(false,false,false,false,'1',$message);
 		  	$headers = $parser->getHeaders(true);
     	}else{
 			 
-			if (!(@$result = imap_append_file($this->imap, $encoded_mailbox, $file, $options, $date))) self::error('imap_message_create');
+			if (!(@$result = imap_append_file($this->imap, $encoded_mailbox, $file, $options, $date))){
+				$quota_error = $_SESSION['user']->checkMailboxQuota(filesize($file));
+				self::error($quota_error?'imap_quota_exceeded':'imap_message_create');
+			}
 		
 			$parser = new MailParse($file);
 			$headers = $parser->getHeaders(true);  		
 
 		}
-		if ($mailbox!="INBOX" && !@imap_reopen($this->imap, $encoded_mailbox)) self::error('imap_mailbox_open',$encoded_mailbox);
+		if (!@imap_reopen($this->imap, $encoded_mailbox)) self::error('imap_mailbox_open',$encoded_mailbox);
 		$data = $this->fetchOverview($result,FT_UID);
 		$result = @reset($data);
 		$newID = IMAP::fixID($result->uid);
@@ -588,18 +606,26 @@ class IMAP extends slObservable{
 	}
 
 	 
-	public function deleteMessage($mailbox, $uid)
+	public function deleteMessage($mailbox, $uid, $expunge = true)
 	{
 		log_buffer("Opening Mailbox:".$mailbox,"DEBUG");
 		$this->openMailbox($mailbox);
 
 		$deleteResult  = imap_delete($this->imap, $uid, FT_UID);
-		if($deleteResult){
+		if($deleteResult && $expunge){
 			$expungeResult = imap_expunge($this->imap);
 		}
 	}
 
 	 
+	public function expunge($mailbox)
+	{
+		log_buffer("Opening Mailbox:".$mailbox,"DEBUG");
+		$this->openMailbox($mailbox);
+		return imap_expunge($this->imap);
+	}
+
+	 
 	public function moveMessage($srcMailbox, $uid, $dstMailbox, $copy = false)
 	{
 		$this->openMailbox($srcMailbox);
@@ -917,12 +943,16 @@ class IMAP extends slObservable{
 		if($full_tags){
 			imap_clearflag_full($this->imap,$uid,$full_tags,FT_UID);
 		}
-		if(is_array($tags) && !empty($tags)){
-			foreach($tags as $key => $val){
-				$tags[$key] = slToolsString::urlenquote($val);
+		if(is_array($tags)) {
+			if (!empty($tags)) {
+				foreach($tags as $key => $val){
+					$tags[$key] = SearchTool::urlEnquoteEscapeTag($val);
+				}
 			}
+		} else {
+			$tags = (array)$tags;
 		}
-		@$tags = join(' ',$tags);
+		@$tags = join(' ', $tags);
 		return imap_setflag_full($this->imap,$uid,$tags,FT_UID);	
 	}
 	
@@ -943,13 +973,13 @@ class IMAP extends slObservable{
 	}
 
      
-	public function getMailboxStatus($mailbox,$encoded = false,$flags = SA_ALL)
+	public function getMailboxStatus($mailbox,$encoded = false)
 	{
 		if(!$encoded){
 			$mailbox = $this->encode($mailbox);
 		}
 		log_buffer("IMAP::getMailBoxStatus(".$this->server.$mailbox.")","EXTENDED");
-		$status = imap_status($this->imap,$this->server.$mailbox,$flags);
+		$status = imap_status($this->imap, $this->server.$mailbox, SA_MESSAGES | SA_RECENT | SA_UNSEEN | SA_UIDNEXT | SA_UIDVALIDITY);
 		return $status;
 	}
 	
@@ -997,7 +1027,7 @@ class IMAP extends slObservable{
 				
 				$aFilterTag['sql'] = str_replace(
 					$replace,
-					!empty($condition)?' ( '.join(' OR ',$condition).' ) ':'0 = 1',
+					!empty($condition)?' ( '.join(' OR ',$condition).' ) ':'(0 = 1)',
 					$aFilterTag['sql']
 				);
 				unset($ids);
@@ -1023,7 +1053,7 @@ class IMAP extends slObservable{
 				}
 				$aFilterTag['sql'] = str_replace(
 					$replace,
-					!empty($condition)?' ( '.join(' OR ',$condition).' ) ':'0 = 1',
+					!empty($condition)?' ( '.join(' OR ',$condition).' ) ':'(0 = 1)',
 					$aFilterTag['sql']
 				);
 				unset($ids);
@@ -1047,7 +1077,7 @@ class IMAP extends slObservable{
 		log_buffer("IMAP::onFolderOpen(".$mailbox.") start","EXTENDED");
 		foreach($this->aObservers as &$observer){
 			if(!isset($observer->changed[$mailbox]) || !$observer->changed[$mailbox]){
-				$status = $this->getMailboxStatus($mailbox,false);
+				$status = $this->getMailboxStatus($mailbox, false);
 				$log = '';
 				if($status) foreach($status as $property => $val){
 					$log.= $property.'=>'.$val.','; 
