@@ -23,7 +23,6 @@ class InputStream
 			
 			 			if (!$sBuffer){
 				throw new Exc('inputstream_no_xml_input');  			}
-			libxml_disable_entity_loader(true);
 			slSystem::import('tools/dom');
 			
 				
@@ -33,10 +32,8 @@ class InputStream
 				
 				$sBuffer = slToolsString::utf8_bad_replace($sBuffer,' ',true);
 				if (!$oBuffer = slToolsDOM::openFromString($sBuffer)){
-					libxml_disable_entity_loader(false);
 					throw new Exc('inputstream_corrupted_input',$sBuffer);				}
 			}
-			libxml_disable_entity_loader(false);
 
 		}
 		 		$this->oInputXML = $oBuffer;
