@@ -7,8 +7,16 @@ class POP3Account extends CacheAccount
 
 	 
 	 
-  public function __construct($user, $accountID,$server ,$port = 110,$username,$password,$primary, $properties = false)
-  {
+  public function __construct(
+	$user,
+	$accountID,
+	$server,
+	$port = 110,
+	$username = '',
+	 	$password = '',
+	$primary = false,
+	$properties = false
+  ) {
 
                           $this->folderClassName = 'POP3Folder';
         parent::__construct($user, $accountID,$server, $port, 
