@@ -85,6 +85,7 @@ abstract class PrivateAndPublic extends XMLRequestInterface
 				}
 				break;
 		}
+		return true;
 	}
 
 	protected function exeInputXML()
@@ -319,7 +320,8 @@ abstract class PrivateAndPublic extends XMLRequestInterface
 							foreach ($signatures['@childnodes']['item'] as $key => $item) {
 								$signature = $item['@childnodes']['text'][0]['@value'];
 								$signature = preg_replace("/\?sid=([^&\s\"']*)('|\"|\s|&)/si","?sid=".$_SESSION['SID']."$2",$signature);
-								$signature = preg_replace("/fullpath=({account})(\/[^&\s\"']*)/si","fullpath=".urlencode($_SESSION['EMAIL'])."$2",$signature);
+								$signature = preg_replace("/\?dlsess=([^&\s\"']*)('|\"|\s|&)/si","?dlsess=".$_SESSION['DLSESS']."$2",$signature);
+								$signature = preg_replace("/fullpath=({account})(((\/)|(%2F))[^&\s\"']*)/si","fullpath=".urlencode($_SESSION['EMAIL'])."$2",$signature);
 								$signatures['@childnodes']['item'][$key]['@childnodes']['text'][0]['@value'] = $signature;
 							}
 							$aData['data'] .= Tools::makeXMLStringFromTree($signatures, $sResourceName, true);
@@ -498,7 +500,7 @@ abstract class PrivateAndPublic extends XMLRequestInterface
 								foreach($aActions as $key => $aAction){
 									$signature = slToolsString::basicSanitizeHTML($aAction['data']['text']);
 									 									if($this->sPP=='public'){
-										 										$signature = preg_replace("/\&amp;fullpath=(.*?)(\/[^&\s\"']*)/si","&amp;fullpath={account}$2",$signature);
+										 										$signature = preg_replace("/\&amp;fullpath=(.*?)(((\/)|(%2F))[^&\s\"']*)/si","&amp;fullpath={account}$2",$signature);
 									}
 									$aActions[$key]['data']['text'] = $signature;
 									$aActions[$key]['dataTree']['@childnodes']['text'][0]['@value'] = $signature;
@@ -638,10 +640,10 @@ abstract class PrivateAndPublic extends XMLRequestInterface
 			}
 		}
 		if($logo = $aData['layout_settings']['@childnodes']['item'][0]['@childnodes']['login_logo'][0]['@value']){
-			$aData['layout_settings']['@childnodes']['item'][0]['@childnodes']['logo_uid'][0]['@value'] = md5(uniqid());
+			$aData['layout_settings']['@childnodes']['item'][0]['@childnodes']['logo_uid'][0]['@value'] = uniqid();
 		}
 		if($background = $aData['layout_settings']['@childnodes']['item'][0]['@childnodes']['login_background_name'][0]['@value']){
-			$aData['layout_settings']['@childnodes']['item'][0]['@childnodes']['login_background_uid'][0]['@value'] = md5(uniqid());
+			$aData['layout_settings']['@childnodes']['item'][0]['@childnodes']['login_background_uid'][0]['@value'] = uniqid();
 		}
 
 		@$_SESSION['user']->imipDefaults();
