@@ -5,6 +5,8 @@ class RemindersAccount extends Account
 	public $folderClassName;
 	public $acc_type;
 	public $aSyncedFolders;
+	public $user;
+	public $account;
 
     public function __construct(&$user,$accountID,&$primaryAccount)
 	{
