@@ -5,7 +5,6 @@ define('SETTINGS_ELM','settings');
 define('RET_STRING',0x0);
 define('RET_ARRAY',0x1);
 
-
  
 
 class StorageCache
@@ -95,7 +94,8 @@ class Storage
 		'phonetelex'=>'LCTPHNTELEX',
 		'phonehearing'=>'LCTPHNHEARING',
 		'phonesip'=>'',
-		'fullname'=>''
+		'fullname'=>'',
+		'profilepicture'=>''
 	);
 
 	static public function getLicenseType()
@@ -149,9 +149,13 @@ class Storage
 	{
 		  		if ($merge['@childnodes']) foreach($merge['@childnodes'] as $keyIndex => $keyValue){
  			$count = 0;
-			
-			if (is_countable($data['@childnodes'][$keyIndex]) && count($data['@childnodes'][$keyIndex])>1) throw new Exc('settings_duplicity');
-			
+
+			 			if (is_countable($data['@childnodes'][$keyIndex]) && count($data['@childnodes'][$keyIndex])>1) {
+				$node = $data['@childnodes'][$keyIndex][0];
+				unset($data['@childnodes'][$keyIndex]);
+				$data['@childnodes'][$keyIndex][0] = $node; 
+				log_buffer("Settings duplicate key - $keyIndex - just first node is used", "EXTENDED");
+			}
 			if ($righttype && isset($restriction['@childnodes'][$keyIndex][0]['@attributes'][$righttype]))
 				$keyRights = $restriction['@childnodes'][$keyIndex][0]['@attributes'][$righttype];
 			else 
@@ -161,8 +165,7 @@ class Storage
 				switch($keyIndex){
 	 				case 'logo_uid':
 	 				case 'login_background_uid':
-	 					@$data['@childnodes'][$keyIndex][$count]['@value'] = $data['@childnodes'][$keyIndex][$count]['@value'].$merge['@childnodes'][$keyIndex][$count]['@value'];
-	 					break;
+	 					 	 					break;
  				}
  			}
 				
@@ -347,26 +350,6 @@ class Storage
 		return $oDOM;
 	}
 
-	static private function getDOM($path,$resource)
-	{
-		$file = $resource.storeageext;
-
-		if (!file_exists($path.$file))
-			return false;
-
-		log_buffer("Storage::getDOM() DOMXML::open(".$path.$file.");","EXTENDED");
-		try{
-			slSystem::import('tools/dom');
-			@$oDOM = slToolsDOM::open($path.$file,true);
-		}catch(Exc $e){
-			$loaderState = libxml_disable_entity_loader(false);
-			$loaderState = $loaderState?'true':'false';
-			log_buffer("Storage::getDOM() - Corrupted file[entity_loader:".$loaderState."]:".$path.$file,"EXTENDED");
-			throw new Exc('storage_get_dom');
-		}
-		return $oDOM;
-	}
-
 	static public function getArray($path,$resource = false,$sfile = SETTINGS_FILE)
 	{
 		$loadFailed = null;
@@ -411,7 +394,7 @@ class Storage
 					log_buffer("Storage::getArray() CREATING DEFAULT SETTINGS FILE","EXTENDED");
 					if(!is_dir($path)){
 						slSystem::import('tools/filesystem');
-						 						slToolsIcewarp::mkdir_r($path, 0777, true);
+						slToolsFilesystem::mkdir_r($path, 0777, true);
 					}
 					$default_content = file_get_contents(DEFAULT_SETTINGS_FILE);
 					$default_content = str_replace('%%ACCOUNT_ID%%',$_SESSION['EMAIL'],$default_content);
@@ -438,9 +421,7 @@ class Storage
 				self::$cache[$path.$file]->set($settingsDOM);
 			}catch(Exception $e){
 				$loadFailed = true;
-				$loaderState = libxml_disable_entity_loader(false);
-				$loaderState = $loaderState?'true':'false';
-				log_buffer("Storage::getArray() Corrupted file:[EntityLoader:".$loaderState."]: ".$path.$file,"EXTENDED");
+				log_buffer("Storage::getArray() Corrupted file: ".$path.$file,"EXTENDED");
 			}
 		}
 		@$oDOM = self::$cache[$path.$file]->get();
@@ -558,6 +539,7 @@ class Storage
 	static public function getCertificateExtraCerts($cert)
 	{
 		$i = 0;
+		$extracerts = [];
 		do{
 			$start = strpos($cert,'-----BEGIN CERTIFICATE-----');
 			$end = strpos($cert,'-----END CERTIFICATE-----')+strlen('-----END CERTIFICATE-----');
@@ -573,6 +555,7 @@ class Storage
 	
 	static public function parseCertificates($content)
 	{
+		
 		$result = array();
 		$content = str_replace(CRLF,LF,$content);
 		$certificates = explode('-'.LF.LF,$content);
@@ -596,11 +579,69 @@ class Storage
 				$result[$itm['id']] = $itm;
 			}
 		}
+		
 		if(count($result)==0){
 			return false;
 		}
 		return $result;
 	}
+
+	static public function parseContactCertificates($content)
+	{
+		$end = false;
+		$position = 0;
+		$result = [];
+		while(!$end){
+			$pubkey_found = false;
+			$itm = [];
+			$pubkey = self::getPEMBlock($content, 'BEGIN CERTIFICATE', 'END CERTIFICATE', $position);
+			if ($pubkey <> ''){
+				$pubkey_found = true;
+			}
+			$cert = '-----BEGIN CERTIFICATE-----'.LF;
+			$cert.= $pubkey.LF;
+			$cert.= '-----END CERTIFICATE-----';
+			$itm['cert'] = $cert;
+			$itm['id'] = md5($cert);			
+			$end = (!$pubkey_found);
+			if(!$end){
+				$itm['info'] = self::getCertificateInfo($cert, $itm['ainfo'],$itm['xmlinfo']);
+				$result[$itm['id']] = $itm;
+			} 
+		}
+		if(count($result)==0){
+			return false;
+		}
+		return $result;
+	} 
+
+
+
+	static public function getPEMBlock(string $pem, string $begin_, string $end_, int &$startPos = 0): string 
+	{
+		$result = '';
+	
+		 		$i = mb_stripos($pem, $begin_, $startPos);
+		if ($i === false) {
+			 			$startPos = strlen($pem);
+	
+			return $result;
+		}
+		$i = mb_stripos($pem, LF, $i);
+	
+		 		$j = mb_stripos($pem, $end_, $i);
+		for ($k = $j; $k >= 1; $k--) {
+			if ($pem[$k] === LF) {
+				break;
+			}
+		}
+		$j = $k;
+		
+		 		$result = trim(mb_substr($pem, $i, $j - $i));
+		 		$startPos = $j;
+	
+		return $result;
+	}
 	
 	static public function getCurrentCertificate($list)
 	{
@@ -700,8 +741,11 @@ class Storage
 		return trim($result);
 	}
 	
-	static public function parseUploadedCertificate($folder,$file,$passphrase = false)
-	{
+	static public function parseUploadedCertificate(
+		$folder,
+		$file,
+		 		$passphrase = false
+	) {
 			$certFile = $_SESSION['user']->getAttachments($folder,$file);
 			$certExt = substr( $certFile['name'], strrpos($certFile['name'],'.')+1);
 			$certExt = strtolower( $certExt );
@@ -1029,7 +1073,7 @@ class Storage
 			case 'AUTORESPONDER':
 				return array('U_RESPOND','U_RESPONDPERIOD','U_REPLYFROM','U_RESPONDBETWEENFROM','U_RESPONDBETWEENTO','U_RESPONDONLYIFTOME','U_RESPONDERCONTENT','U_NORESPONDFOR');
 			case 'FORWARDER':
-				return array('U_FORWARDTO','U_FORWARDOLDER','U_FORWARDOLDERDAYS','U_FORWARDOLDERTO','U_NULL');
+				return array('U_FORWARDTO','U_FORWARDOLDER','U_FORWARDOLDERDAYS','U_FORWARDOLDERTO','U_NULL','U_DONOTFORWARDSPAM');
 			case 'FILTER_RULES':
 				return array('U_RULESCONTENTXML');
 			case 'TEAMCHAT_NOTIFY':
@@ -1135,9 +1179,9 @@ class Storage
 							foreach($filter->CONDITION as $condition){
 								if($condition->EXPRESSION==10 && $condition->HEADERTYPE==33){
 									$time = explode(":",$condition->XML->TIMES->FROM); 
-									$fromtime = ($time[0]*60+$time[1])/1440;
+									$fromtime = count($time) === 2 ? ($time[0]*60+$time[1])/1440 : 0;
 									$time = explode(":",$condition->XML->TIMES->TO);
-									$totime = ($time[0]*60+$time[1])/1440;
+									$totime = count($time) === 2 ? ($time[0]*60+$time[1])/1440 : 0;
 									$fromdate = slToolsDate::strtodate($condition->XML->DATES->FROM);
 									$todate = slToolsDate::strtodate($condition->XML->DATES->TO);
 
@@ -1256,13 +1300,14 @@ class Storage
 	{
 		$domains = self::getAvailableDomains();
 		
+		$arr = [];
 		if($aActions) foreach($aActions as $action)
 			$arr[] = $action['data']['domain'];
 		
 		slSystem::import('tools/filesystem');
 		
 		if($domains) foreach($domains as $dom){
-			if (!@in_array($dom['domain'],$arr) && $dom['domain']){
+			if ($dom['domain'] && !in_array($dom['domain'],$arr)){
 				slToolsFilesystem::rmdir($_SESSION['CONFIGPATH'].'_webmail/'.$dom['domain']);
 			}else{
 				slToolsFilesystem::mkdir($_SESSION['CONFIGPATH'].'_webmail/'.$dom['domain']);
@@ -1328,6 +1373,12 @@ class Storage
     			case 'dbconn':
 	    		case 'dbuser':
 	    		case 'dbpass':
+	    		case 'alfrescourl':
+	    		case 'alfrescosso':
+	    		case 'alfrescotoken':
+	    		case 'alfrescousername':
+	    		case 'alfrescosuperuser':
+	    		case 'alfrescosuperuserpassword':
 	    			break;
 	    			
 	    		default:
@@ -1340,6 +1391,7 @@ class Storage
 		$return['@childnodes']['item'][0]['@childnodes']['http_secure_cookie'][0]['@value'] = $api->GetProperty('C_Webmail_HTTPSecureCookie');
 		$return['@childnodes']['item'][0]['@childnodes']['session_cookie'][0]['@value'] = $api->GetProperty('C_WebMail_SessionCookie');
 		$return['@childnodes']['item'][0]['@childnodes']['teamchat_notify'][0]['@value'] = $api->GetProperty('C_TeamChat_Enable_Notification_Mails');
+		$return['@childnodes']['item'][0]['@childnodes']['cloudapi_hostname'][0]['@value'] = $api->GetProperty('c_cloudapi_hostname');
 
   		$return = Tools::makeXMLStringFromTree($return,'global_settings',true);
   		return $return;
@@ -1615,6 +1667,11 @@ class Storage
 	{
 		if($aFilter){
 			$sAttributes = $oGWAccount->gwAPI->filterToAttributes($aFilter);
+		} else {
+			$aFilter = [];
+		}
+		if (!$aFilter['sql']) {
+			$aFilter['sql'] = '';
 		}
 		$tags = $oGWAccount->gwAPI->FunctionCall(
 			"GetTagDetailList",
@@ -1665,19 +1722,45 @@ class Storage
 		$_SESSION['tags'][$value] = null;
 	}
 	
-	static public function editTag(&$oGWAccount,$uid,$value,$color = '')
+	static public function editTag(&$oGWAccount,$uid,$value,$color = '',$imap = false)
 	{
 		$oldValue = base64_decode($uid);
 		if(!$value){
 			$value = $oldValue;
 		}
-		$oGWAccount->gwAPI->FunctionCall(
-			"AddTag",
-			$oGWAccount->sGWSessionID,
-			$value,
-			$oldValue,
-			$color?'tagcolor='.$color:''
-		);
+		 		 		if($imap && ($oldValue!=$value)){
+			$aFilter['sql'] = '';
+			if(!$color){
+				$aFilter['sql'] = '(tagname = \''.$oldValue.'\') || ';
+			}
+			$aFilter['sql'] = '(tagname = \''.$value.'\')';
+			$tags = self::getAvailableTags($oGWAccount, $aFilter);
+			if(is_array($tags)){
+				foreach($tags as $key => $tag){
+					 					if($tag['TAGNAME']==$oldValue){
+						$color = $tag['TAGCOLOR'];
+					}
+					 					if($tag['TAGNAME']==$value){
+						return;
+					}
+				}
+			}
+			 			$oGWAccount->gwAPI->FunctionCall(
+				"AddTag",
+				$oGWAccount->sGWSessionID,
+				$value,
+				'',
+				$color?'tagcolor='.$color:''
+			);			
+		 		}else{
+			 			$oGWAccount->gwAPI->FunctionCall(
+				"AddTag",
+				$oGWAccount->sGWSessionID,
+				$value,
+				$oldValue,
+				$color?'tagcolor='.$color:''
+			);
+		}
 	}
 	
 	static public function deleteTag(&$oGWAccount,$uid)
@@ -1854,6 +1937,8 @@ class Storage
 				$aliases[slToolsPHP::htmlspecialchars($accountAlias['email'])]['name'] = $accountAlias['name'];
 			}
 			$aliases[slToolsPHP::htmlspecialchars($accountAlias['email'])]['email'] = $accountAlias['email'];
+			$aliases[slToolsPHP::htmlspecialchars($accountAlias['email'])]['type'] = $accountAlias['type'];
+			$aliases[slToolsPHP::htmlspecialchars($accountAlias['email'])]['primary'] = $accountAlias['primary'];
 		}
 		foreach($personalAliases as $personalAlias){
 			$aliases[slToolsPHP::htmlspecialchars($personalAlias['email'])] = $personalAlias;
@@ -1899,9 +1984,11 @@ class Storage
 		}
 		if($aAccounts){
 			foreach($aAccounts as $accountID => $oAccount){
-				 				$aliases[slToolsPHP::htmlspecialchars($accountID)]['email'] = $oAccount->accountID;
+				 				 				$aliases[slToolsPHP::htmlspecialchars($accountID)]['primary'] = !!$oAccount->primary;
+				$aliases[slToolsPHP::htmlspecialchars($accountID)]['email'] = $oAccount->accountID;
 				$aliases[slToolsPHP::htmlspecialchars($accountID)]['name'] = $oAccount->fullname?$oAccount->fullname:$oAccount->description;
 				$aliases[slToolsPHP::htmlspecialchars($accountID)]['enabled'] = true;
+				$aliases[slToolsPHP::htmlspecialchars($accountID)]['type'] = 'account';
 			}
 		}
 		return $aliases;
@@ -1924,6 +2011,7 @@ class Storage
 				}
 				$aliases[slToolsPHP::htmlspecialchars($email)]['email'] = $email;
 				$aliases[slToolsPHP::htmlspecialchars($email)]['enabled'] = true;
+				$aliases[slToolsPHP::htmlspecialchars($email)]['type'] = 'user';
 			}
 		}
 		return $aliases;
@@ -1953,6 +2041,7 @@ class Storage
 					$aliases[slToolsPHP::htmlspecialchars($email)]['name'] = $name;
 				}
 				$aliases[slToolsPHP::htmlspecialchars($email)]['enabled'] = true;
+				$aliases[slToolsPHP::htmlspecialchars($email)]['type'] = 'global';
 			}
 		}
 		return $aliases;
@@ -2048,12 +2137,12 @@ class Storage
 			$check['user'] = $user;
 			$check['email'] = $email;
 			$check['action'] = 'confirm_personality';
-			$hash_check = MerakGWAPI::createURLLine($check);
+			$hash_check = MerakGWAPI::CreateURLLine($check);
 			$hash_check = rawurlencode(base64_encode(gzcompress($hash_check)));
 			$check['hash'] = $hash_check;
-			$hash_check = MerakGWAPI::createURLLine($check);
+			$hash_check = MerakGWAPI::CreateURLLine($check);
 			$hash_check = base64_encode(gzcompress($hash_check));
-			$hash_check = $hash_check.md5(WebmailIqAuth::getServerData('private'));
+			$hash_check = $hash_check.md5(WebmailRSA::instance()->getServerData('private'));
 			
 			if($hash!=$hash_check){
 				throw new Exc('personality_confirmation_invalid_hashid');
@@ -2106,15 +2195,16 @@ class Storage
 			);
 		}
 		 		$api = createobject('api');
-		$webmail_url = explode('?',$_SERVER['HTTP_REFERER'] ?? $api->getProperty('c_webmail_url'))[0];
-		$sPrivateKey = WebmailIqAuth::getServerData('private');
+		$webmail_url = (explode('?',$_SERVER['REQUEST_URI'] ? 'https://' . $_SERVER['HTTP_HOST'] . dirname(dirname($_SERVER['REQUEST_URI'])) : $api->getProperty('c_webmail_url'))[0]) . '/';
+		$webmail_url = preg_replace('/\/\/$/', '/', $webmail_url);
+		$sPrivateKey = WebmailRSA::instance()->getServerData('private');
 		$pathdata['user'] = $user;
 		$pathdata['email'] = $email;
 		$pathdata['action'] = 'confirm_personality';
-		$sHash = MerakGWAPI::createURLLine($pathdata);
+		$sHash = MerakGWAPI::CreateURLLine($pathdata);
 		$sHash = rawurlencode(base64_encode(gzcompress($sHash)));
 		$pathdata['hash'] = $sHash;
-		$sHash = MerakGWAPI::createURLLine($pathdata);
+		$sHash = MerakGWAPI::CreateURLLine($pathdata);
 		$sHash = rawurlencode(base64_encode(gzcompress($sHash)));
 		$sHash = $sHash.md5($sPrivateKey);
 		
@@ -2156,13 +2246,13 @@ class Storage
 		}
 		$check['salt'] = file_get_contents($userdir.'resetpwd.dat');
 
-		$hash_check = MerakGWAPI::createURLLine($check);
+		$hash_check = MerakGWAPI::CreateURLLine($check);
 		$hash_check = rawurlencode(base64_encode(gzcompress($hash_check)));
 		$check['hash'] = $hash_check;
 		
-		$hash_check = MerakGWAPI::createURLLine($check);
+		$hash_check = MerakGWAPI::CreateURLLine($check);
 		$hash_check = base64_encode(gzcompress($hash_check));
-		$hash_check = rawurlencode($hash_check).md5(WebmailIqAuth::getServerData('private'));
+		$hash_check = rawurlencode($hash_check).md5(WebmailRSA::instance()->getServerData('private'));
 
 		if($hash!=$hash_check){
 			throw new Exc('reset_password_invalid_hashid');
@@ -2223,7 +2313,7 @@ class Storage
 		$account->Open($user);
 		$password = $account->GetProperty('u_password');
 		$webmail_url = $api->getProperty('c_webmail_url');
-		$sPrivateKey = WebmailIqAuth::getServerData('private');
+		$sPrivateKey = WebmailRSA::instance()->getServerData('private');
 		$pathdata['user'] = $user;
 		$pathdata['action'] = 'reset_password';
 		$pathdata['salt'] = Tools::my_uniqid('WMResetPwd',true);
@@ -2233,10 +2323,10 @@ class Storage
 		if(!file_put_contents($maildir.'~webmail/resetpwd.dat',$pathdata['salt'])){
 			throw new Exc('reset_password_uid_file');
 		}
-		$sHash = MerakGWAPI::createURLLine($pathdata);
+		$sHash = MerakGWAPI::CreateURLLine($pathdata);
 		$sHash = rawurlencode(base64_encode(gzcompress($sHash)));
 		$pathdata['hash'] = $sHash;
-		$sHash = MerakGWAPI::createURLLine($pathdata);
+		$sHash = MerakGWAPI::CreateURLLine($pathdata);
 		$sHash = rawurlencode(base64_encode(gzcompress($sHash)));
 		$sHash = $sHash.md5($sPrivateKey);
 		
@@ -2338,6 +2428,7 @@ class Storage
 				$result[slToolsPHP::htmlspecialchars($personality)]['email'] = $personality;
 				$result[slToolsPHP::htmlspecialchars($personality)]['enabled'] = 1;
 				$result[slToolsPHP::htmlspecialchars($personality)]['deleteable'] = 1;
+				$result[slToolsPHP::htmlspecialchars($personality)]['type'] = 'personal';
 			}
 		}
 		return $result;
@@ -2394,13 +2485,50 @@ class Storage
 					$signature = str_ireplace('%'.$key.'%', strip_tags($fullname), $signature);
 					$templateData['fullname'] = $fullname;
 					break;
+				case 'profilepicture':
+					$signature = preg_replace_callback('/%'.$key.'(?:(?:&amp;|&)(.*?)(?:(?:&amp;|&)(.*?)(?:(?:&amp;|&)(.*?))?)?)?%/', function($match) use ($oMyCard) {
+						try {
+							$profilePicture = $oMyCard->aAddons['attachment']->getData()[0]['ATTNAME'];
+							if (!$profilePicture) {
+								return '';
+							}
+						} catch(Error $e) {
+							log_buffer("Storage::translateSignature() profilepicture: Missing profile picture", "EXTENDED");
+							return '';
+						}
+
+						$avatarPath = DOWNLOAD_PHP . '?dlsess=' . $_SESSION['DLSESS'] . '&class=attachment&fullpath=' . urlencode($_SESSION['EMAIL'] . '/@@mycard@@/' . $oMyCard->itemID . '/' . $profilePicture);
+						$aStyle = [
+							'object-fit' => 'cover',
+							'object-position' => 'center'
+						];
+						if (intval($match[1] ?? '')) {
+							$aStyle['width'] = intval($match[1]) . 'px';
+							$aStyle['height'] = intval($match[1]) . 'px';
+						}
+						if (intval($match[2] ?? '')) {
+							$aStyle['height'] = intval($match[2]) . 'px';
+						}
+						if (intval($match[3] || '')) {
+							$aStyle['border-radius'] = intval($match[3]) . 'px';
+						}
+						$sStyle = '';
+						foreach($aStyle as $prop => $value) {
+							$sStyle .= $prop . ': ' . $value . ';';
+						}
+
+						return '<img src="' . $avatarPath . '" style="' . $sStyle .'" />';
+						
+					}, $signature ?? '');
+					break;
 				default:
 					$templateData[$key] = slToolsPHP::htmlspecialchars(strip_tags($item[$translate] ?? ''));
-					$signature = str_ireplace('%'.$key.'%', $templateData[$key], $signature);
+					$signature = str_ireplace('%'.$key.'%', $templateData[$key], $signature ?? '');
 					break;
 			}
 		}
 		 		$signature = preg_replace("/\?sid=([^&\s\"']{0,})('|\"|\s|&)/si","?sid=".$_SESSION['SID']."$2",$signature);
+		$signature = preg_replace("/\?dlsess=([^&\s\"']{0,})('|\"|\s|&)/si","?dlsess=".$_SESSION['DLSESS']."$2",$signature);
 		 		$signature = preg_replace("/\&amp;fullpath=({account})(%2F[^&\s\"']{0,})/si","&amp;fullpath=".urlencode($_SESSION['EMAIL'])."$2",$signature);
 		$signature = template($signature,$templateData,true);
 
