@@ -14,7 +14,7 @@ class TagsItem extends Item
 	
 	public function edit($aItem)
 	{
-		return Storage::editTag($this->folder->account->gwAccount,$this->uid,$aItem['tagname'],$aItem['tagcolor']);
+		return Storage::editTag($this->folder->account->gwAccount,$this->uid,$aItem['tagname'],$aItem['tagcolor'],($aItem['tagimap']=='1') || ($aItem['tagimap']=='true'));
 	}
 	
 	public function delete()
@@ -29,7 +29,7 @@ class TagsItem extends Item
 	
 	public function getMessage()
 	{
-		
+		return '';
 	}
 }
 
