@@ -156,10 +156,6 @@ class Tools
 		$oldname ='';
 		if (is_object($oXMLNode) && $oXMLNode->hasChildNodes()) {
 			foreach($oXMLNode->childNodes as $node) {
-				 				if ($oldname != $node->nodeName) {
-					$nCount = 0;
-				}
-				$oldname = $node->nodeName;
 				if ($node->nodeName=='#text') {
 					
 					if($encode){
@@ -168,6 +164,10 @@ class Tools
 						$itm['@value'] = $node->nodeValue;
 					}
 				} else {
+					 					if ($oldname != $node->nodeName) {
+						$nCount = 0;
+					}
+					$oldname = $node->nodeName;
 					if ($node->hasAttributes() && $uid = $node->getAttribute('uid')) {
 						$itm['@childnodes'][strtolower($node->nodeName)][$uid] = self::makeTreeFromXML($node, $encode);
 					} else {
@@ -188,8 +188,14 @@ class Tools
 	 
 	public static function str2JSArrayNew($xmlString) {
 		$xml = self::getXmlFromString($xmlString);
-		$array = Tools::xmlToArray($xml);
-		return json_encode($array);
+		if($xml){
+			$array = Tools::xmlToArray($xml);
+		}
+		if($array){
+			return json_encode($array);
+		}else{
+			return false;
+		}
 	}
 
 	 
@@ -275,7 +281,7 @@ class Tools
 			}
 		}
 	}
-	public static function makeXMLStringFromTree($aArray,$sTagName = 'documentElement',$specialchar = false)
+	public static function makeXMLStringFromTree($aArray,$sTagName = 'documentElement',$specialchar = false, $depth = 0)
 	{
 		 		$sStart = $sEnd = $sValue = $sReturn = $sAttributes = '';
 		 		if (isset($aArray['@attributes'])) {
@@ -286,12 +292,13 @@ class Tools
 		 		if (isset($aArray['@childnodes'])) {
 			foreach($aArray['@childnodes'] as $chName => $chValue) {
 				foreach($chValue as $chVal) {
-					$sValue.= Tools::makeXMLStringFromTree($chVal,$chName,$specialchar);
+					$sValue.= "\n" . str_repeat("\t", $depth) . Tools::makeXMLStringFromTree($chVal,$chName,$specialchar, $depth + 1);
 				}
 			}
+			$sValue.= "\n" . str_repeat("\t", max(0, $depth - 1));
 		}
 		 		if (isset($aArray['@value'])) {
-			$sValue .= ($specialchar?slToolsPHP::htmlspecialchars($aArray['@value']):$aArray['@value']);
+			$sValue .= trim($specialchar?slToolsPHP::htmlspecialchars($aArray['@value']):$aArray['@value']);
 		}
 		 		if (!isset($aArray['@value']) && !isset($aArray['@childnodes'])) {
 			$sReturn = '<'.$sTagName.$sAttributes.'/>';
@@ -374,11 +381,16 @@ class Tools
 	}
 	public static function parseFullPath($fullPath,$class)
 	{
+		$fullPath = str_replace(['../', '..\\'], '', $fullPath);
 	    if($class == 'item' && $fullPath[0] == '@') $class = 'special';
+        $attData['fullpath'] = $fullPath;
+        $attData['class'] = $class;
 		switch($class) {
 			 			case 'image':
 			case 'uploaded_certificate':
 			case 'file':
+			case 'pdf':
+			case 'thumbnail':
 				$lastSlash = strrpos($fullPath,"/");
 				$attData['item'] = substr($fullPath,$lastSlash+1);
 				$attData['folder'] = substr($fullPath,0,$lastSlash);
@@ -411,13 +423,12 @@ class Tools
 			case 'attachment':
 			case 'gwattachment':
 			case 'cid':
-			case 'url':
 			case 'attachmentticket':
 			case 'revision':
 				$lastSlash = strrpos($fullPath,"/");
 				$preLastSlash = strrpos(substr($fullPath,0,$lastSlash),"/");
 				$firstSlash = strpos($fullPath,"/");
-				$attData['part'] = substr($fullPath,$lastSlash+1);
+				$attData['part'] = urldecode(substr($fullPath,$lastSlash+1));
 				$attData['item'] = substr($fullPath,$preLastSlash+1,$lastSlash-$preLastSlash-1);
 				$attData['account'] = substr($fullPath,0,$firstSlash);
 				$attData['folder'] = substr($fullPath,$firstSlash+1,$preLastSlash-$firstSlash-1);
@@ -448,6 +459,7 @@ class Tools
 			case 'logo':
 			case 'jitsi_logo':
 			case 'background':
+			case 'favicon':
 			case 'icechat_background':
 				$attData['host'] = $fullPath;
 				break;
@@ -460,6 +472,10 @@ class Tools
             case 'special':
                 if(!preg_match('/^(?P<account>[^\\/]+)\\/(?P<folder>.*)\\/(?P<item>[^\\/]+)$/', $fullPath, $attData)) throw new Exc(404, 'File not found');
                 break;
+			case 'link':
+			case 'url':
+				$attData['url'] = $fullPath;
+				break;
 			default:
 				throw new Exc('class_unknown',$class);
 				break;
@@ -639,7 +655,7 @@ class Tools
 			return false;
 		}
 		 		if(!self::$host_domain){
-			if(!$host){
+			if(!$host && filter_var('http://' . $_SERVER['HTTP_HOST'], FILTER_VALIDATE_URL)) {
 				$host = $_SERVER['HTTP_HOST'];
 			}
 			if ($pos = strpos($host, ":")){
@@ -812,42 +828,43 @@ class Tools
 	
 	
 	static function pem2der($pem_data) {
-   $begin = "CERTIFICATE-----";
-   $end   = "-----END";
-   $pem_data = substr($pem_data, strpos($pem_data, $begin)+strlen($begin));   
-   $pem_data = substr($pem_data, 0, strpos($pem_data, $end));
-   $der = base64_decode($pem_data);
-   return $der;
-}
+		$begin = "CERTIFICATE-----";
+		$end   = "-----END";
+		$pem_data = substr($pem_data, strpos($pem_data, $begin)+strlen($begin));   
+		$pem_data = substr($pem_data, 0, strpos($pem_data, $end));
+		$der = base64_decode($pem_data);
+		return $der;
+	}
 
- static public function der2pem($der_data) {
-   $pem = chunk_split(base64_encode($der_data), 64, "\n");
-   $pem = "-----BEGIN CERTIFICATE-----\n".$pem."-----END CERTIFICATE-----\n";
-   return $pem;
-}
+	static public function der2pem($der_data) {
+		$pem = chunk_split(base64_encode($der_data), 64, "\n");
+		$pem = "-----BEGIN CERTIFICATE-----\n".$pem."-----END CERTIFICATE-----\n";
+		return $pem;
+	}
 	static public function dbSortValue($value)
 	{
 		return trim(str_replace(array('\'','"','<'),'',$value));
 	}
-	static public function my_fputcsv ($fh, array $fields, $delimiter = ',', $enclosure = '"', $newline = "\n") {
-    $delimiter_esc = preg_quote($delimiter, '/');
-    $enclosure_esc = preg_quote($enclosure, '/');
-  
-    $output = array();
-    foreach ($fields as $field) {
-        $output[] = preg_match("/(?:${delimiter_esc}|${enclosure_esc}|\s)/", $field) ? (
-            $enclosure . str_replace($enclosure, $enclosure . $enclosure, $field) . $enclosure
-        ) : $field;
-    }
-  
-    fwrite($fh, join($delimiter, $output) . $newline);
-  }
- static public function my_str_getcsv($data, $delimiter = ',', $enclosure = '"', $newline = "\n",&$quote_open){
+
+	static public function my_fputcsv($fh, array $fields, $delimiter = ',', $enclosure = '"', $newline = "\n") {
+		$output = [];
+		foreach ($fields as $field) {
+			 			$field = preg_replace('/' . $enclosure . '/', $enclosure . $enclosure, $field);
+			if (in_array($field[0], ['=', '+', '-', '@', "\r", "\t"])) {
+				$field = "'" . $field;
+			}
+			$output[] = $enclosure . $field . $enclosure;
+		}
+
+		fwrite($fh, join($delimiter, $output) . $newline);
+	}
+
+	static public function my_str_getcsv($data, $delimiter = ',', $enclosure = '"', $newline = "\n",&$quote_open = false){
  		$data = str_replace("\r\n","\n",$data);
         $pos = $last_pos = -1;
         $end = strlen($data);
         $row = 0;
-                 $trim_quote = false;
+        $trim_quote = false;
 
         $return = array();
 
@@ -873,7 +890,7 @@ class Tools
                                          --$length;
                 }
 
-                                 $return[$row][] = ($length > 0) ? str_replace($enclosure . $enclosure, $enclosure, substr($data, $last_pos, $length)) : '';
+                                 $return[$row][] = ($length > 0) ? preg_replace('/^\'([=+0@\r\t])/', '$1', str_replace($enclosure . $enclosure, $enclosure, substr($data, $last_pos, $length)), 1) : '';
 
                                  if ($done){
                     break;
@@ -910,66 +927,6 @@ class Tools
         return $return[0];
     }
 	
-	static public function getMimeType ($filename) 
-	{
-		$ext = strrpos($filename,'.');
-		$type = substr( $filename, $ext + 1 );
-		$type = strtolower($type);
-		
-		$mimetypes = array(
-		'doc' => 'application/msword',
-		'bin' => 'application/octet-stream',
-		'lha' => 'application/octet-stream',
-		'exe' => 'application/octet-stream',
-		'dll' => 'application/octet-stream',
-		'pdf' => 'application/pdf',
-		'eps' => 'application/postscript',
-		'ps' => 'application/postscript',
-		'xls' => 'application/vnd.ms-excel',
-		'ppt' => 'application/vnd.ms-powerpoint',
-		'swf' => 'application/x-shockwave-flash',
-		'tar' => 'application/x-tar',
-		'zip' => 'application/zip',
-		'mid' => 'audio/midi',
-		'midi' => 'audio/midi',
-		'mp2' => 'audio/mpeg',
-		'mp3' => 'audio/mpeg',
-		'm3u' => 'audio/x-mpegurl',
-		'bmp' => 'image/bmp',
-		'gif' => 'image/gif',
-		'ief' => 'image/ief',
-		'jpeg' => 'image/jpeg',
-		'jpg' => 'image/jpeg',
-		'jpe' => 'image/jpeg',
-		'png' => 'image/png',
-		'tiff' => 'image/tiff',
-		'tif' => 'image/tiff',
-		'xbm' => 'image/x-xbitmap',
-		'wrl' => 'model/vrml',
-		'vrml' => 'model/vrml',
-		'css' => 'text/css',
-		'html' => 'text/html',
-		'htm' => 'text/html',
-		'asc' => 'text/plain',
-		'txt' => 'text/plain',
-		'rtx' => 'text/richtext',
-		'rtf' => 'text/rtf',
-		'sgml' => 'text/sgml',
-		'sgm' => 'text/sgml',
-		'xml' => 'text/xml',
-		'mpeg' => 'video/mpeg',
-		'mpg' => 'video/mpeg',
-		'mpe' => 'video/mpeg',
-		'qt' => 'video/quicktime',
-		'mov' => 'video/quicktime',
-		'avi' => 'video/x-msvideo',
-		'rar' => 'application/rar',
-		'svg'=>'image/svg+xml'
-		);
-
-		return ( $mimetypes[$type]?$mimetypes[$type]:'application/octet-stream');
-	}
-
 	static public function getDeliveryReport($email,$message_id,$unix_time)
 	{
 		$api = createobject('api');
@@ -1090,7 +1047,7 @@ class Tools
 		if(!$ext){
 			if($attachment['type']=='message/rfc822'){
 				$attachment['name'] = str_replace(array('/','\\','?','%','*',':','|','"','<','>','.'),'',$attachment['name']);
-				$name = $attachment['name'].'.eml';
+				$name = $attachment['name'].messageext;
 			}else{
 				$name = $attachment['name'];
 			}
@@ -1171,4 +1128,4 @@ class Tools
     }
 }
 
-?>
\ No newline at end of file
+?>
