@@ -25,8 +25,7 @@ class User
 			'P'=>'templates'
 		);
 	static public	$mailFlags = array(
-				'D'=>0x200,			 				'S'=>0x100,			 				'H'=>0x800,			 				'P'=>0x80000000000
-			);
+				'D'=>0x200,			 				'S'=>0x100,			 				'H'=>0x800,			 				'P'=>0x20000		 			);
 		
 	static public	$gwFolders = array(
 			'C'=>'contacts',
@@ -34,30 +33,34 @@ class User
 			'T'=>'tasks',
 			'N'=>'notes',
 			'J'=>'journal',
-			'F'=>'files'
+			'F'=>'files',
 		);
 
 	public $protocol;
 
 	 
-	protected function __construct($userID, $username, $password, $hashid, $mAccount)
-	{
+	protected function __construct(
+		$userID,
+		$username,
+		 		$password,
+	) {
 		 		$this->aAccounts = array();
 		$this->userID	= $userID;
 		$this->username  = $username;
 		$this->password  = slToolsCrypt::encryptSymmetric($password);
-		$this->email  = $mAccount->EmailAddress;
 		$this->attachments = array();
+	}
+
+	private function init($mAccount)
+	{
+		$this->email  = $mAccount->EmailAddress;
 
-		 		$this->getData($mAccount);
-		
 		 		$filename = User::getDir().UNAVAILABLE_FILE;
 		if(file_exists($filename)) {
 			slSystem::import('tools/icewarp');
 			slToolsIcewarp::iw_delete($filename);
-		}
-		 		$this->loadAccounts($_SESSION['MAILBOX'],$password, $mAccount);
-		
+		}	
+
 		 		$primary = $this->getAccount($_SESSION['EMAIL']);
 		 		if($primary && $primary->gwAccount){
 			$timezone = $primary->gwAccount->gwAPI->FunctionCall(
@@ -98,14 +101,18 @@ class User
 			$juliantime = MerakGWAPI::iso86012calendartime($iso8601);
 			$juliandate = MerakGWAPI::iso86012calendardate($iso8601);
 			$time2 = MerakGWAPI::calendar2unixTime($juliandate, $juliantime);
-			$dt = $time2-$time1;
-			$dt = $dt/60;
+			$dt = 0;
+			if ($time2) {
+				$dt = $time2-$time1;
+				$dt = $dt/60;
+			}
 			$_SESSION['CLIENT_TIMEZONE_OFFSET'] = $dt;
 		}	
 
 		
 		if(!$_SESSION['LAST_LOGIN_LOAD']){
 			$oAccount = $this->getAccount($_SESSION['EMAIL']);
+			$oAccount->username = $oAccount->username ?? $this->email;
 			$aProperties['current_time'] = time();
 			if ($oAccount->last_time) {
 				$aProperties['last_time'] = $oAccount->last_time;
@@ -124,12 +131,17 @@ class User
 					$aProperties['colors_converted'] = 1;
 				}
 			}
+			if (!$_SESSION['GUEST_ACCOUNT'] && (!$oAccount->default_folders_created || $oAccount->default_folders_created!=1)) {
+				$aProperties['default_folders_created'] = $this->createDefaultFolders($oAccount);
+			}
 			
 			$_SESSION['LAST_LOGIN_LOAD'] = true;
 		}
 		$aProperties['current_ip'] = $_SERVER['REMOTE_ADDR'];
 		
 		$this->editAccount($_SESSION['EMAIL'],$aProperties);
+
+		 		$this->syncDefaultFoldersStorage();
 	}
 
 	 
@@ -141,8 +153,7 @@ class User
 
 	private function cleanUp(){
 		$folder_old = User::getDir();
-		icewarp_delete_files($folder_old,'.msg',true,0);
-	}
+		 		 	}
 	
 	private function convertColors($oAccount)
 	{
@@ -272,14 +283,19 @@ class User
 			$_SESSION['IM_SUPPORT'] = ($_SESSION['TCONLY'] || $_SESSION['GUEST_ACCOUNT'])?false:$user->getProperty("U_IMSupport");
 			$_SESSION['IM_HISTORY_SUPPORT'] = ($_SESSION['TCONLY'] || $_SESSION['GUEST_ACCOUNT'])?false:$api->getProperty("C_IM_UserHistory");
 			$_SESSION['SMS_SUPPORT'] = $_SESSION['TCONLY']?false:$user->getProperty("U_SMSSupport");
+			$_SESSION['GW_SUPPORT'] = $user->GetProperty("u_gwsupport");
 			$_SESSION['GWTRASH_SUPPORT'] = $_SESSION['TCONLY']?false:$api->GetProperty("C_GW_KeepDeletedItems");
 			$_SESSION['SHARING_SUPPORT'] = $user->getProperty("U_SharingAvailable");
 			$_SESSION['SHARED_PREFIX']	 = $api->GetProperty("C_GW_SharedAccountPrefix");
+			if($_SESSION['SHARED_PREFIX']==''){
+				throw new Exc(500,'C_GW_SharedAccountPrefix is empty!');
+			}
 			$_SESSION['DELIVERY_REPORT_SUPPORT'] = $_SESSION['TCONLY']?false:$api->getProperty('C_Accounts_Global_Accounts_DeliveryReports');
 			$_SESSION['FULLTEXT_SUPPORT'] = filter_var($api->getProperty('C_System_Services_Fulltext_Enabled'), FILTER_VALIDATE_BOOLEAN);
 			$_SESSION['TELEMETRY_SUPPORT'] = $api->getProperty('c_system_tools_wcstatistics_enabled');
 			$_SESSION['SMTP_HEADER_FUNCTION'] = $api->getProperty('c_mail_smtp_other_headerfunctions');
 			$_SESSION['SMTP_SSLMETHOD'] = $api->getProperty('c_system_adv_ext_sslservermethod');
+			$_SESSION['RECALL_MESSAGES_ENABLED'] = $api->getProperty('c_process_recall_messages');
 			
 			 			if($_SESSION['IM_SUPPORT']){
 				$xmpp = $user->getProperty('u_xmpp_data');
@@ -305,17 +321,19 @@ class User
 			
 			$_SESSION['SIP_CALLERALIAS'] = $api->GetProperty('c_system_services_sip_calleralias');
 			 			$authMode = $user->getProperty("U_AuthMode");
-			if($authMode!=1 && $authMode!=2){
+			if($authMode!=1){
 				$_SESSION['PASSWORD_EXPIRED'] = $user->getProperty("U_PasswordExpired");
 			}
 			 			if($user->GetProperty('U_WebDAVSupport')){
 				$_SESSION['WEBDAV_URL'] = $api->GetProperty('C_GW_WebDAVURL');
 			}
-			 			$_SESSION['WEBMAIL_URL'] = $api->GetProperty('C_Webmail_URL');
+			 			$_SESSION['WEBMAIL_URL'] = self::getWebmailURL();
 			
 			 			$_SESSION['WEBADMIN_URL'] = $api->GetProperty('C_Webadmin_URL');
 
-			 			$_SESSION['TEAMCHATAPI_URL'] = (($_SERVER['HTTPS']=='ON' || ($_SESSION['ALWAYSHTTPS'] ?? false)) ? 'https' : 'http') .'://'.$_SERVER['HTTP_HOST'].'/teamchatapi/';
+			 			$_SESSION['TEAMCHATAPI_URL'] = filter_var((($_SERVER['HTTPS']=='ON' || ($_SESSION['ALWAYSHTTPS'] ?? false)) ? 'https' : 'http') .'://'.$_SERVER['HTTP_HOST'].'/teamchatapi/', FILTER_VALIDATE_URL);
+
+			 			$_SESSION['ICEWARPMOBILEAPP_URL'] = $user->GetProperty('u_icewarpmobileapp_url');
 
 			 			if ($quota = $user->GetProperty("U_MailboxQuota")) {
 				$_SESSION['DISK_QUOTA']  = $quota;
@@ -379,8 +397,10 @@ class User
 			$_SESSION['SETTINGS_API'] = true;
 			$_SESSION['USER_ALIAS_LIST_ARRAY'] = explode(';', $user->GetProperty('u_aliaslist'));
 			$_SESSION['USER_ALIAS_ARRAY'] = explode(';', $user->GetProperty('u_alias'));
-			$_SESSION['gw_keepdeleteditemsforceexpiration'] = $user->GetProperty('c_gw_keepdeleteditemsforceexpiration');
+			$_SESSION['gw_keepdeleteditemsforceexpiration'] = $api->GetProperty('c_gw_keepdeleteditemsforceexpiration');
             $_SESSION['U_USERNAME'] = $user->GetProperty('u_mailbox');
+
+			 			$_SESSION['CUSTOMBACKUP'] = $user->GetProperty('u_custombackup');
 		}
 	}
 	
@@ -408,9 +428,7 @@ class User
 				slSystem::import('tools/dom');
 				@$xml = slToolsDOM::open(GLOBAL_SETTINGS_FILE,true);
 			}catch(Exc $e){
-				$loaderState = libxml_disable_entity_loader(false);
-				$loaderState = $loaderState?'true':'false';
-				log_buffer("User::settingsGlobal() Corrupted file:[EntityLoader:".$loaderState."]: ".GLOBAL_SETTINGS_FILE,"EXTENDED");
+				log_buffer("User::settingsGlobal() Corrupted file: ".GLOBAL_SETTINGS_FILE,"EXTENDED");
 				throw new Exc('user_settings_global');
 			}
 			
@@ -434,12 +452,16 @@ class User
 			 			$_SESSION['DBCONN'] = $set['dbconn'][0]['@value'];
 			$_SESSION['DBUSER'] = $set['dbuser'][0]['@value'];
 			$_SESSION['DBPASS'] = $set['dbpass'][0]['@value'];
-			$_SESSION['DBSYNTAX'] = strtolower($set['dbsyntax'][0]['@value']);
+			if (isset($set['dbsyntax'][0]['@value'])) {
+				$_SESSION['DBSYNTAX'] = strtolower($set['dbsyntax'][0]['@value']);
+			}
 			if(!$_SESSION['DBCONN']) {
 				$_SESSION['DBCONN'] = 'sqlite:%WMUSERPATH%cache.db';
 			}
 			$arr = explode(":",$_SESSION['DBCONN']);
-			$_SESSION['DBTYPE'] = strtoupper($arr[0]);
+			if ($arr[0]) {
+				$_SESSION['DBTYPE'] = strtoupper($arr[0]);
+			}
 			if($arr[1]) {
 				$_SESSION['PURECONN'] = $arr[1];
 			}
@@ -593,8 +615,10 @@ static public function settingsUser()
 		$defaultdata = Storage::getDefaults('restrictions');
 		$userdata = Storage::getDefaults('restrictions');
 		$data = WebmailIqPrivate::get('restrictions',$defaultdata,$domaindata,$userdata,0);
-		$restrictions = trim(strtoupper($data['@childnodes']['item'][0]
-		['@childnodes']['disable_gw_types'][0]['@value']));
+		$restrictions = '';
+		if ($data['@childnodes']['item'][0]['@childnodes']['disable_gw_types'][0]['@value']) {
+			trim(strtoupper($data['@childnodes']['item'][0]['@childnodes']['disable_gw_types'][0]['@value']));
+		}
 
 		for($i = 0;$i < strlen($restrictions);$i++) {
 			$_SESSION['RESTRICTIONS'][strval($restrictions[$i])] = true;
@@ -613,7 +637,12 @@ static public function settingsUser()
 			$_SESSION['RESTRICTIONS']['S'] = true;
 			$_SESSION['RESTRICTIONS']['P'] = true;
 		}
-		
+		 		if($_SESSION['EXTERNAL']){
+			$_SESSION['RESTRICTIONS']['M'] = true;
+			$_SESSION['RESTRICTIONS']['Q'] = true;
+			$_SESSION['RESTRICTIONS']['B'] = true;
+			$_SESSION['RESTRICTIONS']['W'] = true;
+		}
 
 		if($data['@childnodes']['item'][0]
 		['@childnodes']['disable_virtual'][0]['@value'] == 1){
@@ -689,8 +718,11 @@ static public function settingsUser()
 		 
 	}
 	 
-	public function loadAccounts($user,$pass,$mAccount)
-	{
+	public function loadAccounts(
+		$user,
+		 		$pass,
+		$mAccount
+	) {
 		slSystem::import('tools/string');
 		$primaryProtocol = ($mAccount->getProperty("U_AccountType")==0)?'local':'imap';
 		$email = $mAccount->EmailAddress;
@@ -750,6 +782,9 @@ static public function settingsUser()
 				if($doc->getNode('colors_converted',$accountNode)){
 					$primaryProperties['colors_converted'] = $doc->getNodeValue('colors_converted', $accountNode);
 				}
+				if($doc->getNode('default_folders_created',$accountNode)){
+					$primaryProperties['default_folders_created'] = $doc->getNodeValue('default_folders_created', $accountNode);
+				}
 				if($doc->getNode('sentfolder',$accountNode)){
 					$primaryProperties['sentfolder'] = $doc->getNodeValue('sentfolder', $accountNode);
 				}
@@ -757,6 +792,8 @@ static public function settingsUser()
 					$primaryProperties['trashfolder'] = $doc->getNodeValue('trashfolder', $accountNode);
 				}
 				$primaryProperties['secondary'] = $_SESSION['SECONDARY'];
+				$primaryProperties['created'] = $doc->getNodeValue('created', $accountNode);
+				$primaryProperties['modified'] = $doc->getNodeValue('modified', $accountNode);
 			}else{
 				$pass = false;
 
@@ -776,7 +813,9 @@ static public function settingsUser()
 					'password' => $pass,
 					'valid'=>$accountNode->getAttribute('valid')?true:false,
 					'sentfolder'=>$doc->getNodeValue('sentfolder', $accountNode),
-					'trashfolder'=>$doc->getNodeValue('trashfolder', $accountNode)
+					'trashfolder'=>$doc->getNodeValue('trashfolder', $accountNode),
+					'created'=>$doc->getNodeValue('created', $accountNode),
+					'modified'=>$doc->getNodeValue('modified', $accountNode)
 				);
 			}
 		}
@@ -858,15 +897,29 @@ static public function settingsUser()
 					break;
 			}
 			icewarp_releaselock($background?$lockID_background:$lockID);
-		}catch(Exc $e){
+		} catch(Exc $e){
 			if(!$background){
-				icewarp_releaselock($background?$lockID_background:$lockID);
+				icewarp_releaselock($lockID);
 			}
 			throw $e;
+		} catch(PDOException $e) {
+			icewarp_releaselock($background?$lockID_background:$lockID);
+			throw $e;
 		}
 		return true;
 	}
 	
+	public function checkMailboxQuota($size)
+	{
+		$mAccount = new IceWarpAccount();
+		$mAccount->Open($_SESSION['EMAIL']);
+		$current_usage = $mAccount->GetProperty('U_MailboxSize');
+		if($size + $current_usage > $_SESSION['DISK_QUOTA'] ){
+			return true;
+		}
+		return false;
+	}
+	
 	public function getVersion($background = false)
 	{
 		$nonsensetranslate = array(
@@ -932,8 +985,6 @@ static public function settingsUser()
 						$cacheFile = substr(DB_MIGRATION_SOURCE,7);
 					}
 					$u = false;
-
-					
 					$check = new Cache($u,array('connection'=>"sqlite:".$cacheFile,'query_log'=>$query_log));
 					if($background){
 						$check->checkTablesBackground();
@@ -953,8 +1004,23 @@ static public function settingsUser()
 	}
 	
 	 
-	public static function login($username, $password, $hashid = "", $method = "",$bSaveLoginTime = true,$bEmailAddressLogin='',$cookie = false,$sExternalIP = false,$sDisableIPCheck = false,$sLanguage = false,$newSessionPrefix='',$forceSessionID = '',$removeUploadDir = true)
-	{
+	public static function login(
+		$username,
+		 		$password,
+		$hashid = "",
+		$method = "",
+		$bSaveLoginTime = true,
+		$bEmailAddressLogin='',
+		$cookie = false,
+		$sExternalIP = false,
+		$sDisableIPCheck = false,
+		$sLanguage = false,
+		$newSessionPrefix='',
+		$forceSessionID = '',
+		$removeUploadDir = true,
+		$sDeviceId = '',
+		$is_external = false
+	) {
 		if($_SESSION['SESSION_COOKIE']){
 			$set_cookie = true;
 		}
@@ -967,6 +1033,7 @@ static public function settingsUser()
 		 		$mAccount = new MerakAccount();
 
 		$_SESSION['WMSESS'] = '';
+		$_SESSION['EXTERNAL'] = $is_external;
 
 		self::settingsGlobal();
 		 		if($bEmailAddressLogin==='') {
@@ -993,7 +1060,7 @@ static public function settingsUser()
 		}
 
 		if (!$auth){
-			self::checkLoginKeyPair();
+			WebmailRSA::instance()->checkLoginKeyPair();
 			if(isset($_COOKIE['permanentLogin'])){
 				session_destroy();
 			}
@@ -1012,7 +1079,7 @@ static public function settingsUser()
 			throw new Exc('wm_disabled','wm_disabled',false,true,'login');
 		}
 		
-		$_SESSION['WMSESS'] = 'user='.$mAccount->EmailAddress.'&ip='.$_SERVER['REMOTE_ADDR'].'&time='.time().'&password='.slToolsCrypt::encryptSymmetric($password);
+		$_SESSION['WMSESS'] = 'user='.urlencode($mAccount->EmailAddress).'&ip='.urlencode($_SERVER['REMOTE_ADDR']).'&time='.time().'&password='.slToolsCrypt::encryptSymmetric($password);
 		$_SESSION['DOMAIN'] = substr($_SESSION['EMAIL'],strrpos($_SESSION['EMAIL'],'@')+1);
 		if($_SESSION['DOMAIN']!=$mAccount->Domain){
 			$_SESSION['SLAVE_DOMAIN'] = $mAccount->Domain;
@@ -1022,13 +1089,21 @@ static public function settingsUser()
 			$_SESSION['LANGUAGE'] = $sLanguage;
 		}
 		$_SESSION['TCONLY'] = $_REQUEST['tconly'];
-		
-		$user = new User(0, $username, $password, $hashid, $mAccount);
-		$_SESSION['user'] = $user;
+		if($sDeviceId){
+			$_SESSION['GW']['DEVICE_ID'] = $sDeviceId;
+		}
+		$user = new User(0, $username, $password, $hashid);
+		$user->getData($mAccount);
+		$user->loadAccounts($_SESSION['MAILBOX'],$user->getPassword(), $mAccount);
+		$_SESSION['user'] = &$user;
+		$user->init($mAccount);
 		$_SESSION['EXTERNAL_IP'] = $sExternalIP;
 		$_SESSION['IP_CHECK'] = !$sDisableIPCheck;
+		
 		$_SESSION['jscheck'] = stripos($_REQUEST['lastLogin'],'tablet')===false && stripos($_REQUEST['lastLogin'],'pda')===false;
 		$_SESSION['USERDIR'] = $mAccount->getProperty('u_fullmailboxpath');
+		
+
 		 		$data = Storage::getUserData();
 		
 		log_buffer("User::login() line 872 User::syncDefaultFolders()","EXTENDED");
@@ -1050,6 +1125,7 @@ static public function settingsUser()
 		}
 		 		@unlink($_SESSION['USERDIR'].'import_status.dat');
 		$_SESSION['SID'] = session_id();
+		self::getDlSess(true);
 		if($cookie){
 			if($_COOKIE['auth_cookie']){
 				$_SESSION['AUTH_COOKIE'] = $cookie;
@@ -1072,7 +1148,7 @@ static public function settingsUser()
 					break;
 				}
 			}
-			if(!$teamChatFolderExists){
+			if(!$_SESSION['EXTERNAL'] && !$teamChatFolderExists){
 				session_destroy();
 				throw new Exc('no_teamchat_folders');
 			}
@@ -1088,6 +1164,13 @@ static public function settingsUser()
 
 		return $_SESSION['SID'];
 	}
+
+	public static function getDlSess($replace = false) {
+		if (!$_SESSION['DLSESS'] || $replace) {
+			$_SESSION['DLSESS'] = slToolsCrypt::encryptSymmetric($_SESSION['SID']);
+		}
+		return $_SESSION['DLSESS'];
+	}
 	
 	 
 	public function logout($unsetSession = true)
@@ -1145,7 +1228,11 @@ static public function settingsUser()
 		$api = createobject('API');
 		$api->SetProperty('C_System_Logging_Maintenance_Identity',$_SESSION['EMAIL'].'/WebMail');
 		$version = $api->getProperty('C_Version');
-		if($_SESSION['WM_VERSION']!=$version){
+		if ($_SESSION['WM_VERSION']) {
+			preg_match('/^\d+\.\d+\.\d+\.\d+(?:\s+RC\d+)?(?:\s+\(\d+\-\d+\-\d+\))?/', $_SESSION['WM_VERSION'], $v1);
+		}
+		preg_match('/^\d+\.\d+\.\d+\.\d+(?:\s+RC\d+)?(?:\s+\(\d+\-\d+\-\d+\))?/', $version, $v2);
+		if(!$_SESSION['WM_VERSION'] || ($v1[0] != $v2[0])) {
 			$_SESSION['DB_CHECKED'] = false;
 			$_SESSION['DB_CHECKED_BACKGROUND'] = false;
 			$attachments = $_SESSION['user']->attachments;
@@ -1246,20 +1333,31 @@ static public function settingsUser()
 		return $uploadFolder;
 	}
 	 
-	private function removeUploadDir($folder = '')
+	private function removeUploadDir($folder = '', $api = null)
 	{
+		if (!$api) {
+			$api  = createobject('api');
+		}
 		if($folder) {
 			$dir = User::getDir() . '~upload/' . md5($folder);
 			if(!is_dir($dir)){
 				$dir = User::getDir() . '~upload/' . $folder;
 			}
+			$api->DeleteDirRecWithUpdate($dir);
 		}else{
-			$dir = User::getDir() . '~upload/';
+			 			$now12h = time() - (12*60*60);
+			foreach (glob(User::getDir() . '~upload/*') as $path) {
+				if ($now12h > filemtime($path)) {
+					if (is_dir($path)) {
+						$this->removeUploadDir(basename($path), $api);
+					}else{
+						$api->DeleteFileWithUpdate($path);
+					}
+				}
+			}
+
 		}
-		
-		
-		$api  = createobject('api');
-		$api->DeleteDirRecWithUpdate($dir);
+	
 	}
 	 
 	public function getAccounts()
@@ -1287,6 +1385,7 @@ static public function settingsUser()
 		if($_SESSION['DISABLE_OTHERACCOUNTS']){
 			throw new Exc('other_accounts_disabled');
 		}
+		self::processDigestInProperties($properties);
 		 		if (!isset($properties['username'])){
 			throw new Exc('account_no_username');
 		}
@@ -1334,6 +1433,7 @@ static public function settingsUser()
 			$accountNode->appendChild(new DOMElement('password', slToolsCrypt::encryptSymmetric(slToolsPHP::htmlspecialchars($properties['password']))));
 			$accountNode->appendChild(new DOMElement('sentfolder', slToolsPHP::htmlspecialchars($properties['sentfolder'])));
 			$accountNode->appendChild(new DOMElement('trashfolder', slToolsPHP::htmlspecialchars($properties['trashfolder'])));
+			$accountNode->appendChild(new DOMElement('created', slToolsPHP::htmlspecialchars(time())));
 			if($properties['valid']){
 				$accountNode->setAttribute('valid', 1);
 			}
@@ -1393,7 +1493,9 @@ static public function settingsUser()
 		if(!$primary && $_SESSION['DISABLE_OTHERACCOUNTS']){
 			throw new Exc('other_accounts_disabled');
 		}
+		self::processDigestInProperties($properties);
 		$aProperties = $properties;
+
 				 		 		 		slSystem::import('tools/dom');
 		$doc = slToolsDOM::open(User::getDir() . accountsfile);
 		if (!is_object($doc)) {
@@ -1435,13 +1537,17 @@ static public function settingsUser()
 		if($properties['fullname']){
 			$properties['fullname'] = slToolsString::removeHTML($properties['fullname']);
 		}
-		
+
+		$properties['modified'] = time();
+		$properties['created'] = $account->created ?? time();
+
 		if(isset($properties)) {
 			foreach ($properties as $property => $value) {
 				if($property=='oldpassword' || $property=='acl'){
 					continue;
 				}
 				$value = slToolsPHP::htmlspecialchars($value);
+
 				if($property == 'password'){
 					$value = slToolsCrypt::encryptSymmetric($value);
 					$properties['password'] = $value;
@@ -1470,9 +1576,8 @@ static public function settingsUser()
 					 					if (isset($aProperties['password'])) {
 						if(!$plainPassword){
 							$password = '';
-							 							@$privateKey = openssl_pkey_get_private(file_get_contents(WM_CONFIGPATH.'private.key'));	
-							openssl_private_decrypt(WebmailIqAuth::hexStringToBinary($aProperties['password']), $password, $privateKey);
-							openssl_private_decrypt(WebmailIqAuth::hexStringToBinary($aProperties['oldpassword']), $oldPassword, $privateKey);
+							 							$password = WebmailRSA::instance()->rsaToPlain($aProperties['password']);
+							$oldPassword = WebmailRSA::instance()->rsaToPlain($aProperties['oldpassword']);
 							
 							$oldInfo = Tools::parseURL($oldPassword);
 							if($oldInfo['p']){
@@ -1670,17 +1775,20 @@ static public function settingsUser()
 		$account->charset = $_SESSION['DEFAULTCHARSET'];
 		if ($properties['gw']==true) {
 			 
-			$gwAccount = new GroupWareAccount($properties['accountID'], $properties['password'], $account);
+			$gwAccount = new GroupWareAccount($properties['accountID'], $properties['password'], $account, $_SESSION['EXTERNAL']);
 			$account->appendGroupWareAccount($gwAccount);
 		}
 		$account->description = $properties['description'];
 		$account->fullname = $properties['fullname'];
 		$account->sentfolder = $properties['sentfolder'];
 		$account->trashfolder = $properties['trashfolder'];
+		$account->created = $properties['created'];
+		$account->modified = $properties['modified'];
 		if ($primary) {
 			
 			$account->isGuest = $_SESSION['GUEST_ACCOUNT'];
 			$account->colors_converted = $properties['colors_converted'];
+			$account->default_folders_created = $properties['default_folders_created'];
 			 			if ($properties['rss']==true) {
 				if($account->protocol=='IMAP'){
 					$rssAccount = new RSSAccount($this,$properties['accountID'].'_rss');
@@ -1702,7 +1810,7 @@ static public function settingsUser()
 			}else{
 				$_SESSION['QUARANTINE_SUPPORT'] = false;
 				if($mAccount->GetProperty('U_ASSupport')){
-					$api = createobject('api');
+					$api = createobject('api','com.icewarp.webclient');
 					$_SESSION['WLIST_ENABLE'] = $api->GetProperty('C_AS_SpamWList');
 					$_SESSION['BLIST_ENABLE'] = $api->GetProperty('C_AS_SpamBList');
 				}else{
@@ -1725,7 +1833,7 @@ static public function settingsUser()
 			}
 			 			$virtualAccount = new VirtualAccount($this,$properties['accountID'],$account);
 			$account->folders['virtual'] = $virtualAccount->getFolders();
-			$account->virtualAccount = &$virtualAccount;
+			$account->virtualAccount = $virtualAccount;
 			$account->secondary = $properties['secondary'];
 		}
 		$this->aAccounts[$properties['accountID']] = $account;
@@ -1744,12 +1852,13 @@ static public function settingsUser()
 		if($xml){
 		  $signup_ips = $xml->item->signup_ips;
 		  if($signup_ips){
+			$ipChecker = new slToolsIpChecker();
 		  	$result = false;
 		  	$signup_ips = explode(';',$signup_ips);
 		  	foreach($signup_ips as $allowed_ip_mask){
-		  		if (slToolsString::cidrMatch($ip,$allowed_ip_mask))
-		  		  return true;
+				$ipChecker->addIpRange($allowed_ip_mask);
 		  	}
+			$result = $ipChecker->isIpInRange($ip);
 		  }	
 		}
 		return $result;	
@@ -1800,6 +1909,13 @@ static public function settingsUser()
 	public function testAccount($aProperties)
 	{
 		$aProperties['accountID'] = strtolower($aProperties['email']);
+		if(isset($aProperties['digest'])){
+			$pwd = WebmailRSA::instance()->rsaToPlain($aProperties['digest']);
+			if($pwd){
+				Tools::parsePassword($pwd);
+				$aProperties['password'] = $pwd; 
+			}
+		}
 		 		if(!isset($aProperties['password'])){
 			try{
 				$oAccount = $this->getAccount($aProperties['accountID']);
@@ -1825,6 +1941,45 @@ static public function settingsUser()
 		}
 		unset($this->aAccounts[$aProperties['accountID']]);
 	}
+
+	static public function inviteAccounts(array $aInviteList)
+	{
+		 		if($_SESSION['ACCOUNT'] == 0){
+			throw new Exc('account_permission');
+		}
+		$url .= User::getWebmailURL().'?invite=';
+		$email = $_SESSION['EMAIL'];
+		$name = $_SESSION['FULLNAME'];
+		$domain = $_SESSION['DOMAIN'];
+		$currentSessionId = session_id();
+		$gwAccount = $_SESSION['user']->getAccount($_SESSION['EMAIL'])->gwAccount;
+		$result = [];
+
+		 		foreach($aInviteList as $invitee){
+			 			$signupSessionId = session_create_id('signup-');
+			$inviteURL = $url.urlencode($signupSessionId).'#join';
+			$result[$invitee] = [];
+			$result[$invitee]['email'] = $invitee;
+			$result[$invitee]['sid'] = $signupSessionId;
+			$result[$invitee]['url'] = slToolsPHP::htmlspecialchars($inviteURL);
+			$gwAccount->gwAPI->FunctionCall('InviteUser', $gwAccount->sGWSessionID, $invitee, $inviteURL);
+		}
+		session_write_close();
+
+		 		foreach($result as $invite){
+			session_id($invite['sid']);
+			session_start();
+			$_SESSION['INVITE_SESSION'] = 1;
+			$_SESSION['INVITEE_EMAIL'] = $invite['email'];
+			$_SESSION['INVITOR_EMAIL'] = $email;
+			$_SESSION['INVITOR_NAME'] = $name;
+			$_SESSION['INVITOR_DOMAIN'] = $domain;
+			session_write_close();
+		}
+		session_id($currentSessionId);
+		session_start();
+		return $result;
+	}
 	
 	 
 	public function getAttachments($folder = false,$file = false)
@@ -1852,9 +2007,13 @@ static public function settingsUser()
 		 
 		$dir = $this->getUploadDir($md5folder);
 		 
-		$fName = $_FILES[$uploadFileID]['name'];
+		if (class_exists('Normalizer') && method_exists('Normalizer', 'normalize')) {
+			$fName = Normalizer::normalize($_FILES[$uploadFileID]['name'], Normalizer::NFC);
+		} else {
+			$fName = $_FILES[$uploadFileID]['name'];
+		}
 		$file =  Tools::randomFileName($dir.'/');
-		 		log_buffer("Upload session [".($_SESSION['SID']?$_SESSION['SID']:'Unauthorized')."] PARAMETERS [ItemID:".$itemID.",FolderID:".$folderID.",Filepath:".$file."]","DEBUG");
+		 		log_buffer("Upload session [".($_SESSION['SID']?$_SESSION['SID']:'Unauthorized')."] PARAMETERS [ItemID:".$itemID.",FolderID:".$folder.",Filepath:".$file."]","DEBUG");
 		
 		if (!move_uploaded_file($_FILES[$uploadFileID]['tmp_name'], $file)) {
 			return false;
@@ -1866,13 +2025,15 @@ static public function settingsUser()
 				'file' => $file,
 				'name' => $fName,
 				'type' => $_FILES[$uploadFileID]['type'],
-				'cid'  => $cid
+				'cid'  => $cid,
+				'size' => $_FILES[$uploadFileID]['size']
 			);
 		} else {
 			$this->attachments[$folder][$itemID] = array(
 				'file' => $file,
 				'name' => $fName,
-				'type' => $_FILES[$uploadFileID]['type']
+				'type' => $_FILES[$uploadFileID]['type'],
+				'size' => $_FILES[$uploadFileID]['size']
 			);
 		}
 		return true;
@@ -1889,7 +2050,8 @@ static public function settingsUser()
 			'name'=> $attName,
 			'type'=> $attType,
 			'delete'=>false,
-			'method'=>$sMethod
+			'method'=>$sMethod,
+			'size' => strlen($attString)
 		);
 		return true;
 	}
@@ -1900,7 +2062,8 @@ static public function settingsUser()
 			'file' => $attFile,
 			'name'=> $attName,
 			'type'=> $attType,
-			'delete'=>$delete
+			'delete'=>$delete,
+			'size'=>filesize($attFile)
 		);
 	}
 	
@@ -2003,9 +2166,9 @@ static public function settingsUser()
 		 		self::setDefaultFoldersDefaults();
 	}
 
-	protected function getFolderNameWithoutAccount($folderName)
+	protected function getFolderNameWithoutAccount($folderName = '')
 	{
-		if(!preg_match('#^([^@\\/]+@[^@\\/]+)?[\\/]*(?P<folder>.*)#', $folderName, $matches)) return false;
+		if(!$folderName || !preg_match('#^([^@\\/]+@[^@\\/]+)?[\\/]*(?P<folder>.*)#', $folderName, $matches)) return false;
 		return $matches['folder'];
 	}
 	
@@ -2103,22 +2266,102 @@ static public function settingsUser()
 			}
 		}
 	}
-	
-	static public function checkLoginKeyPair()
+
+	static public function addTokenToTicket(string $ticket) : string
 	{
-		$publicfile = WM_CONFIGPATH.'public.key';
-		$privatefile = WM_CONFIGPATH.'private.key';
-		$public = file_get_contents($publicfile);
-		$private = file_get_contents($privatefile);
-		$pkey = openssl_pkey_get_private($private);
-		$details = openssl_pkey_get_details($pkey);
-		$modulus = WebmailIqAuth::binaryToHexString(strrev($details['rsa']['n']));
-		if($modulus!=$public){
-			log_buffer("Invalid key-pair","EXTENDED");
-			@unlink($publicfile);
-			@unlink($privatefile);
-			throw new Exc('login_invalid_keypair');
+		if($ticket == ''){
+			return $ticket;
 		}
+		$urlEncoded = preg_match('#https?%3A//#si', $ticket, $matches);
+
+		$tokenEquals = 'token=';
+		$question = '?';
+		$amp = '&';
+		$ticketEquals = 'ticket=';
+		$collaboration = '/collaboration/';
+		if($urlEncoded){
+			$tokenEquals = urlencode($tokenEquals);
+			$question = urlencode($question);
+			$amp = urlencode($amp);
+			$ticketEquals = urlencode($ticketEquals);
+			$collaboration = urlencode($collaboration); 
+		}
+
+		$token = $_SESSION['TEAMCHAT_TOKEN'];
+		if($token 
+			 			&& (strpos($ticket, $ticketEquals)!==false)
+			 			&& (strpos($ticket, $collaboration)===false
+			 			&& (strpos($ticket, $tokenEquals)===false))
+		){
+			if (strpos($ticket, $question) === false) {
+				$ticket .= $question;
+			} else {
+				$ticket .= $amp;
+			}
+			$ticket.=$tokenEquals;
+			if($urlEncoded){
+				$ticket.= urlencode(urlencode($token));
+			}else{
+				$ticket.= urlencode($token);
+			}
+		}
+		return $ticket;
+	}
+	 	 	 	 	static public function getWebmailURL($replace = '')
+	{
+		$script = $_SERVER['SCRIPT_NAME'];
+		$script = str_replace(WEBMAIL_PHP, $replace, $script);
+		$script = preg_replace('/-.._._.--.._[0-9]{0,}\//si','',$script);
+		$pos = strrpos($script, 'index.html');
+		if($pos!==false){
+			$script = substr($script, 0, $pos);
+		}
+		 		if($replace==''){
+			$script = rtrim($script,'/').'/';
+		}
+		$result = filter_var('http' . (($_SERVER['HTTPS'] == 'ON' || ($_SESSION['ALWAYSHTTPS'] ?? false)) ? 's' : '') . '://' . $_SERVER['HTTP_HOST'] . $script, FILTER_VALIDATE_URL);
+		return $result;
+	}
+
+	static public function processDigestInProperties(&$properties)
+	{
+		if(isset($properties['digest'])){
+			$password = WebmailRSA::instance()->rsaToPlain($properties['digest']);
+			if($password){
+				Tools::parsePassword($password);
+				$properties['password'] = $password;
+			}
+			unset($properties['digest']);
+		}
+	}
+
+	private function createDefaultFolders($oAccount)
+	{
+		$result = true;
+		$oAccount->sync();
+		$this->setDefaultFoldersDefaults();
+		$default_folders = $this->getDefaultFolderList();
+		if($default_folders){
+			foreach($default_folders as $type => $name){
+				try{
+					$webmailType = isset(self::$gwFolders[$type])?'gw':'main';
+					$oAccount->getFolder($name,	$webmailType);
+				}catch(Exc $e){
+					log_buffer("User::createDefaultFolders() - creating default folder: $name ($type)","EXTENDED");
+					try{
+						$param['name'] = $name;
+						$param['type'] = isset(self::$gwFolders[$type])?$type:'M';
+						$oFolder = $oAccount->createFolder($param);
+						$oFolder->setDefault($type, false);
+						$result = $result && ($oFolder->name == $name);
+					}catch(Exc $e){
+						log_buffer("User::createDefaultFolders() - ERROR creating default folder: $name ($type) code: ".$e->wmcode. " message:". $e->message,"EXTENDED");
+						$result = false;
+					}
+				}
+			}
+		}
+		return $result;
 	}
 }
 ?>
