@@ -15,8 +15,22 @@ class VirtualFolder extends Folder
 	public $primary;
 	public $contentType;
 	public $account;
+	public $scope;
+	public $scopeValue;
+	public $publicRoot;
+	public $groupOwner;
+	private $groupGWID;
 
-    public function __construct(&$account, $name, $type, &$folders, $primary, $search = false,$sync = false,$autoSubscribe = false,$sharetype = false)
+	private static $aScopes = array(
+		'personal'=>1,
+		'public'=>1,
+		'shared'=>1,
+		'archive'=>1,
+		'all'=>1,
+		'subtree'=>1
+	);
+
+    public function __construct(&$account, $name, $type, &$folders, $primary, $search = false,$sync = false,$autoSubscribe = false,$sharetype = false, $scope = '', $scopeValue = '')
 	{
 		$this->type = 'V';
 		$this->name = $name;
@@ -30,8 +44,23 @@ class VirtualFolder extends Folder
 		$this->sync = $sync;
 		$this->sharetype = $sharetype;
 		$this->autoSubscribe = $autoSubscribe;
+		if($scope){
+			$this->setScope($scope,$scopeValue);
+		}
 		$this->init($folders);
 	}
+
+	static public function createFromFolderInfo(&$account, $name, $info)
+	{
+		$scopeType = strtolower(str_replace('@@','',$info['scope']));
+		$scopeValue = $info['value'];
+		$folders = [];
+		if($scopeType=='subtree'){
+			$root = $account->account->getFolder($scopeValue);
+			$info['type'] = $root->getType();
+		}
+		return new VirtualFolder($account, $name, $info['type'],$folders,null, false, false, false, false, $scopeType, $scopeValue);
+	}
 	
 	
 	public function init($folders)
@@ -40,7 +69,7 @@ class VirtualFolder extends Folder
 			case 'M':
 				switch(strtolower($this->account->account->protocol)){
 					case 'imap':
-						$this->class = 'IMAPItem';
+				 		$this->class = 'IMAPItem';
 					break;
 					case 'local':
 						$this->class = 'LocalPOPItem';
@@ -92,7 +121,6 @@ class VirtualFolder extends Folder
 			case 'I':
 				$folder = $this->getPrimary();
 				return GroupWareItem::import($folder, $type, $data, false);
-			break;
 		}
 	}
 	
@@ -100,8 +128,48 @@ class VirtualFolder extends Folder
 	{
 		unset($this->folders);
 		unset($this->nonExistingFolders);
-		if(is_array($folders) && !empty($folders)){
- 			foreach($folders as $folder){
+		 		$scope = $this->getScope();
+		if(!$folders){
+
+			$publicFolder = $this->getScopeValue();
+			$sharedAccount = substr($publicFolder,strlen($_SESSION['SHARED_PREFIX']));
+			$folderID = '@@'.strtoupper($scope).'@@#'.$this->getType();
+			 			if(in_array($this->getType(),['M', 'R'])){
+				switch($scope){
+					case 'public':
+						foreach($account->folders['main'] as $id => $folder){
+							if($folder->isPublic() && ($folder->getType()=='M' || $folder->getType()=='R') && stripos(strtoupper($folder->name), $publicFolder)===0){
+								$folders[$id] = $id;
+							}
+						}
+						break;
+					case 'personal':
+						foreach($account->folders['main'] as $id => $folder){
+							if(($folder->getType()==$this->getType()) 
+							&& !$folder->isShared() 
+							&& !$folder->isPublic() 
+							&& !$folder->isArchiveSubfolder()
+							&& !$account->isDualFolder($folder)){
+								$folders[$id] = $id;
+							}
+						}
+						break;
+					case 'shared':
+						foreach($account->folders['main'] as $id => $folder){
+							if($folder->isShared($sharedAccount) && ($folder->getType()=='M' || $folder->getType()=='R')){
+								$folders[$id] = $id;
+							}
+						}
+						break;
+				}
+			}
+		}
+		 		$acc = $account;
+		if ($account->gwAccount) {
+			$acc = $account->gwAccount;
+		}
+		if($folders){
+			foreach($folders as $folder){
 				if(!$this->autoSubscribe){
 					try{
 						$oFolder = $account->getFolder($folder);
@@ -116,7 +184,13 @@ class VirtualFolder extends Folder
 							$oFolder = $primaryAccount->getFolder($folder);
 							if($oFolder->type=='V'){
 								if($oFolder->isEmpty()){
-									$this->folders['@@ALL@@#'.$this->getType()] = '@@ALL@@#'.$this->getType();
+									if($this->getType()!='M' && $this->getType()!='R'){
+										$folderID = '@@ALL@@#'.$this->getType();
+										$this->folders[$folderID] = new GroupWareFolder($acc, $this->name, $this->getType());
+										if(!$this->primary){
+											$this->primary = $folderID;
+										}
+									}
 								}else{
 									foreach($oFolder->folders as $fdr){
 										$this->folders[$fdr->name] = $fdr;
@@ -128,10 +202,13 @@ class VirtualFolder extends Folder
 						}
 					}
 				}else{
-					$oFolder = new GroupWareFolder($account,$folder,'E');
+					$oFolder = new GroupWareFolder($acc,$folder, $this->getType());
 					$this->folders[$folder] = $oFolder;
 				}
 			}
+		}		
+		if(!isset($this->primary)){
+			$this->findPrimary();
 		}
 	}
 	
@@ -150,7 +227,6 @@ class VirtualFolder extends Folder
 			case 'M':
 				$item = $this->createItemFromID($itemID,$cache);
 				return $item;
-				break;
 			case 'C':
 			case 'E':
 			case 'T':
@@ -178,27 +254,26 @@ class VirtualFolder extends Folder
 	}
 	public function getItems( &$aFilterTag = array(), $createObject = true, $ignoreHidden = true)
 	{
-		if(!$this->alreadySearched && $this->search){
-            if($this->getType() == 'M' && $_SESSION['FULLTEXT_SUPPORT']){
-                $imapSearch = new \server\inc\imap\Search($this, $this->search);
+		if(!$this->alreadySearched && ($this->search || $this->getScope()=='subtree')){
+			$searchCondition = ($this->search ? $this->search : $aFilterTag['search']);
+            if($this->getType() == 'M'){
+                $search = new \server\inc\imap\Search($this, $searchCondition ?? '*');
+            }else{
+                $search = new \server\inc\gw\Search($this, $searchCondition);
             }
-            if(!$imapSearch instanceof \server\inc\imap\Search || !$imapSearch->isSearchEnabled() || ($condition = $imapSearch->getSearchSql()) === false) {
-                require_once(SHAREDLIB_PATH . 'tools/search.php');
-                $search = new SearchTool();
-                $search->setType($this->getType());
-                $search->setAccount($this->account);
-                $search->setFolder($this);
-
-                $condition = $search->parse($this->search);
-            }
-			
-			if($aFilterTag['sql']){
-				$aFilterTag['sql'] ='('.$aFilterTag['sql'].') AND ('.$condition.')';
-			}else{
-				$aFilterTag['sql'] = '('.$condition.')';
+			if ($searchCondition) {
+				if(!$search->isSearchEnabled() || ($condition = $search->getSearchSql()) === false) {
+					$condition = $search->sqlFallback($aFilterTag, $this->account->account);
+				}
 			}
 			
-			
+			if($condition){
+				if($aFilterTag['sql']){
+					$aFilterTag['sql'] ='('.$aFilterTag['sql'].') AND ('.$condition.')';
+				}else{
+					$aFilterTag['sql'] = '('.$condition.')';
+				}
+			}
 			if((strpos($aFilterTag['sql'],'{TAG}')!==false && $this->getType()=='M')
 				|| (strpos($aFilterTag['sql'],'{FULLTEXT}')!==false && $this->getType()=='M')){
 				return $this->search($aFilterTag,true);
@@ -220,7 +295,7 @@ class VirtualFolder extends Folder
 
 		if($this->folderID == '__@@VIRTUAL@@__/__@@MEETINGS@@__'){
             if(!empty($aFilterTag['sql'] ?? null)) $aFilterTag['sql'] = '('.$aFilterTag['sql'].') AND ';
-            $aFilterTag['sql'] = ($aFilterTag['sql'] ?? '') . 'EvnMeetingId IS NOT NULL AND EvnMeetingId <> \'\'';
+            $aFilterTag['sql'] = ($aFilterTag['sql'] ?? '') . '(EvnMeetingId IS NOT NULL AND EvnMeetingId <> \'\')';
             $aFilterTag['holidays'] = false;
         }
 		switch($this->contentType){
@@ -268,6 +343,7 @@ class VirtualFolder extends Folder
 		}
 		return $items;
 	}
+
 	public function createItemFromID($itemID,&$cache = array())
 	{
 		$account = &$this->account;
@@ -304,33 +380,31 @@ class VirtualFolder extends Folder
 	
 	public function countItems($flags = 0, $positive = true,$filter = "",$search = false, $fields = '',$folderID = false,$folder = false,$alreadySearched = false)
 	{
-        if($this->getType() == 'M' && $_SESSION['FULLTEXT_SUPPORT']){
-            $imapSearch = new \server\inc\imap\Search($this, $this->search);
-        }
-        if(!$imapSearch instanceof \server\inc\imap\Search || !$imapSearch->isSearchEnabled() || ($condition = $imapSearch->getSearchSql()) === false) {
-            require_once(SHAREDLIB_PATH.'tools/search.php');
-            $searchTool = new SearchTool();
-            $searchTool->setType($this->getType());
-            $searchTool->setAccount($this->account);
-            $searchTool->setFolder($this);
-            if($this->contentType == 'M'){
-                $this->search = str_replace('fulltext:', '', $this->search);
-                $this->search = str_replace('tag:', 'taglist:', $this->search);
-            }
-            $condition = $searchTool->parse($this->search);
-        }
-		if($flags == Item::FLAG_SEEN 
-			&&	(strpos($condition,'{FULLTEXT}')!==false 
-				|| strpos($condition,'{TAG}')!==false
-				)
-		){
-			return 0;
-		}
-		if($this->search && !$alreadySearched){
-			if($filter){
-				$filter ='('.$filter.') AND ('.$condition.')';
+
+		if(($this->search || $this->getScope()=='subtree') && !$alreadySearched){
+			if($this->getType() == 'M'){
+				$searchTool = new \server\inc\imap\Search($this, ($this->search ? $this->search : $search) ?? '*');
 			}else{
-				$filter = '('.$condition.')';
+				$searchTool = new \server\inc\gw\Search($this, ($this->search ? $this->search : $search));
+			}
+			if(!$searchTool->isSearchEnabled() || ($condition = $searchTool->getSearchSql()) === false) {
+				$aFilterTag = [];
+				$aFilterTag['search'] = $this->search;
+				$condition = $searchTool->sqlFallback($aFilterTag, $this->account->account);
+			}
+			if($flags == Item::FLAG_SEEN 
+				&&	(strpos($condition,'{FULLTEXT}')!==false 
+					|| strpos($condition,'{TAG}')!==false
+					)
+			){
+				return 0;
+			}
+			if($condition){
+				if($filter){
+					$filter ='('.$filter.') AND ('.$condition.')';
+				}else{
+					$filter = '('.$condition.')';
+				}
 			}
 		}
 		
@@ -345,8 +419,7 @@ class VirtualFolder extends Folder
 						$flags,
 						$positive,
 						$filter,
-						$search,
-						$fields
+						$search
 					);
 				}catch(Exception $e){
 					
@@ -370,16 +443,15 @@ class VirtualFolder extends Folder
 					$search,
 					$fields
 				);
-				break;
 		}
+		return 0;
 	}
-	 	public function deleteItems( $oItems = false,$cache = false, $delayed = 'auto', $reason = '', $ignore_reason = false, $skip_trash = false)
+	public function deleteItems( $oItems = false,$cache = false, $delayed = 'auto', $reason = '', $ignore_reason = false, $skip_trash = false)
 	{
 		if(is_array($oItems) && !empty($oItems)){
 			foreach($oItems as $oItem){
-				$item = $this->getItem($oItem->itemID,NO_ADDONS,0,0,'*',$part_id, $cache);
-				$aItemsToDelete[$oItem->folder->folderID]['items'][] = $item;
-				$aItemsToDelete[$oItem->folder->folderID]['folder'] = $item->folder;
+				 				$aItemsToDelete[$oItem->folder->folderID]['items'][] = $oItem;
+				$aItemsToDelete[$oItem->folder->folderID]['folder'] = $oItem->folder;
 			}
 			if($aItemsToDelete)foreach($aItemsToDelete as $folderItems){
 				try{
@@ -556,6 +628,7 @@ class VirtualFolder extends Folder
 				if(!is_object($folder)){
 					throw new Exc('no_folder');
 				}
+				
 				$rights = $folder->getMyRights();
 				$rights |= Folder::RIGHT_FOLDER_READ;
 				$rights |= Folder::RIGHT_FOLDER_WRITE;
@@ -590,20 +663,23 @@ class VirtualFolder extends Folder
 	public function search( &$aFilterTag,$alreadySearched = false)
 	{
 		$sPhrase = $aFilterTag['fulltext'];
-		if(!$alreadySearched && $this->search){
-			require_once(SHAREDLIB_PATH.'tools/search.php');
-			$search = new SearchTool();
-			$search->setType($this->getType());
-			$search->setAccount($this->account);
-			$search->setFolder($this);
-			$condition = $search->parse($this->search);
-			if($aFilterTag['sql']){
-				$aFilterTag['sql'] ='('.$aFilterTag['sql'].') AND ('.$condition.')';
-			}else{
-				$aFilterTag['sql'] = '('.$condition.')';
-			}
+		if(!$alreadySearched && ($this->search || $this->getScope()=='subtree')){
+            if($this->getType() == 'M'){
+                $searchTool = new \server\inc\imap\Search($this, $this->search ?? '*');
+            }else{
+                $searchTool = new \server\inc\gw\Search($this, $this->search);
+            }
+            if(!$searchTool->isSearchEnabled() || ($aFilterTag['sql'] = $searchTool->getSearchSql()) === false) {
+                $condition = $searchTool->sqlFallback($aFilterTag, $this->account->account);
+                if($condition){
+					if($aFilterTag['sql']){
+						$aFilterTag['sql'] ='('.$aFilterTag['sql'].') AND ('.$condition.')';
+					}else{
+						$aFilterTag['sql'] = '('.$condition.')';
+					}
+				}
+            }
 		}
-		
 		 		if($this->isEmpty()){
 			$folders = $this->account->account->folders['main'];
 			$allFolders = true;
@@ -644,6 +720,7 @@ class VirtualFolder extends Folder
 				foreach($folders as $folder){
 					$filter['sql'] = $aFilterTag['sql'];
 					$filter['fulltext'] = $sPhrase;
+                    $prefix = '';
                     if($allFolders && strtoupper($_SESSION['DBTYPE']) == 'MYSQL'){
                         $filter['complicated'] = true;
                     }
@@ -711,6 +788,11 @@ class VirtualFolder extends Folder
 		$sNewSubscribed = isset($parameters['subscription'])?$parameters['subscription']:false;
 		$sNewShareType = isset($parameters['sharetype'])?$parameters['sharetype']:false;
 		$virtual = VirtualHandler::instance($this->account);
+		if(!empty($sNewFolders)){
+			$sNewShareType = false;
+		}else{
+			$sNewShareType = 'all';
+		}
 		$virtual->editFolder(
 			$this->name,
 			$sNewName, 
@@ -720,33 +802,16 @@ class VirtualFolder extends Folder
 			$sNewSubscribed,
 			$sNewShareType
 		);
-		$name = $sNewName?$sNewName:$this->name;
+		$this->name = $sNewName?$sNewName:$this->name;
+		$this->primary = $sNewPrimary?$sNewPrimary:$this->primary;
+		$this->search = $sNewSearch!==false?$sNewSearch:$this->search;
+		$this->sync = $sNewSubscribed?$sNewSubscribed:$this->sync;
+		$this->sharetype = $sNewShareType;
+
 		$fdrs = $this->folders?$this->folders:array();
 		$folders = $setFolders?$sNewFolders:array_keys($fdrs);
-		$primary = $sNewPrimary?$sNewPrimary:$this->primary;
-		$search = $sNewSearch!==false?$sNewSearch:$this->search;
-		$sync = $sNewSubscribed?$sNewSubscribed:$this->sync;
-		$sharetype = $sNewShareType?$sNewShareType:$this->sharetype;
 		
-		$fdr = new VirtualFolder(
-			$this->account,
-			$name,
-			$this->contentType,
-			$folders,
-			$primary,
-			$search,
-			$sync,
-			false,
-			$sharetype
-		);
-		$this->name = $name;
-		$this->primary = $primary;
-		$this->search = $search;
-		$this->sync = $sync;
-		$this->sharetype = $sharetype;
 		$this->init($folders);
-		unset($this->account->folders[$this->name]);
-		$this->account->folders[$name] = $fdr;
 		
 	}
 	
@@ -757,6 +822,10 @@ class VirtualFolder extends Folder
 	
 	public function getPrimary()
 	{
+		$this->findPrimary();
+		if(!$this->primary){
+			throw new Exc('folder_virtual_no_primary');
+		}
 		 		if(!is_object($this->folders[$this->primary])){
 			if(!$this->isEmpty()){
 				$newPrimary = reset($this->folders);
@@ -782,7 +851,6 @@ class VirtualFolder extends Folder
 					);
 				}
 				return $contacts;
-			break;
 			case 'M':
 			case 'E':
 			case 'T':
@@ -793,7 +861,6 @@ class VirtualFolder extends Folder
 			case 'Y':
 			case 'I':
 				throw new Exception('virtual_getcontactlist_type');
-			break;
 		}
 	}
 	
@@ -813,7 +880,6 @@ class VirtualFolder extends Folder
 			case 'Y':
 			case 'I':
 				throw new Exception('virtual_saveaddressestofile_type');
-			break;
 		}
 	}
 	
@@ -841,7 +907,7 @@ class VirtualFolder extends Folder
 					$ifilename.='.eml';
 					$ifile = $item->getMessageFile();
 					if(!file_exists($ifile)){
-						$item->autoCreateMessage($ifile);
+						$ifile = $item->autoCreateMessage($ifile);
 					}
 					$zip->addFile($ifile,$ifilename);
 				}
@@ -871,6 +937,10 @@ class VirtualFolder extends Folder
 	
 	public function __call($name,$args)
 	{
+        $primary = $this->getPrimary();
+        if(is_callable([$primary, $name])){
+            return call_user_func_array([$primary, $name], $args);
+        }
 		echo "Virtual folder error: unsupported method";
 		print_R(array($name,$args));die();
 	}
@@ -893,6 +963,98 @@ class VirtualFolder extends Folder
         }
         return $sql;
     }
+
+	public function getItemsByIds($ids)
+	{
+		switch($this->getType()){
+			case 'M':
+				$cache = Cache::instance($this->account->account->user);
+				$result = $cache->getItemsByIds($this, $ids, $this->class);
+				if(!empty($result)){
+					foreach($result as $key => $item){
+						$realfolder = $cache->getFolderById($this->account->account ,$this->account->account->folderClassName, $item->original_folder_id);
+						$result[$key]->folder = $realfolder;
+					}
+				}
+			break;
+			default:
+				$filter = [];
+				$id = $this->getType()=='C'?'ITM_ID':'EVN_ID';
+				if($ids){
+					$filter['sql'] = $id.' IN ("'.join('", "',$ids).'")';
+				}
+				$result = $this->getItems($filter);
+			break;
+		}
+		return $result;
+	}
+
+		
+	public function setScope($scope, $value = '')
+	{
+		if(isset(self::$aScopes[$scope])){
+			$this->scopeValue = '';
+			$this->scope = $scope;
+			switch($scope){
+				case 'shared':
+				case 'public':
+				case 'subtree':
+					$this->scopeValue = $value;
+					break;
+			}
+		}else{
+			throw new Exc('invalid_scope');
+		}
+	}
+
+	public function getScope()
+	{
+		return $this->scope;
+	}
+
+	public function getScopeValue()
+	{
+		return $this->scopeValue;
+	}
+
+
+	public function getScopeLocation()
+	{
+		switch($this->getScope()){
+
+			case 'personal':
+				$result = '(personal)';
+				break;
+			case 'archive':
+				$result = '(subtree "'.$_SESSION['ARCHIVE_INTEGRATE_NAME'].'")';
+				break;
+			case 'shared':
+				$result = '(subtree "'.$_SESSION['SHARED_PREFIX'].$this->scopeValue.'")';
+				break;
+			case 'subtree':
+			case 'public':
+				$result = '(subtree "'.$this->scopeValue.'")';
+				break;
+			default:
+				$result = '(personal)';
+				break;
+		}
+		return $result;
+	}
+
+	public function findPrimary()
+	{
+		if(!$this->primary){
+			if(is_array($this->folders) && !empty($this->folders)){
+				if($this->getType()=='M'){
+					$this->primary = reset($this->folders)->name; 
+				}else{
+					$this->primary = reset($this->folders)->folderID;
+				}
+			}
+		}
+	}
+
 }
 
 ?>
\ No newline at end of file
