@@ -20,6 +20,7 @@ class WebmailIqAccounts extends XMLRequestInterface
 	public $sTwoFactorCode;
 	public $sTwoFactorPassword;
 	public $sTwoFactorKey;
+	public $aInviteList = [];
 
 	 
 	public function __construct($oDOMQuery,&$oDOMDoc,&$attrs)
@@ -49,6 +50,7 @@ class WebmailIqAccounts extends XMLRequestInterface
 				'refresh'=>1,
 				'test'=>1,
 				'signup'=>1,
+				'invite'=>1,
 				'subscribe'=>1,
 				'unsubscribe'=>1,
 				'subscribe_folder'=>1,
@@ -56,7 +58,8 @@ class WebmailIqAccounts extends XMLRequestInterface
 				'reset_two_factor'=>1,
 				'confirm_two_factor'=>1,
 				'setup_two_factor'=>1,
-				'manual_two_factor'=>1
+				'manual_two_factor'=>1,
+				'set_custombackup'=>1
 			);
 			$this->sAction = $oDOMAccount->getAttribute("action"); 
 			$this->aAttrs['action'] = $this->sAction;
@@ -120,12 +123,15 @@ class WebmailIqAccounts extends XMLRequestInterface
 					throw new Exc('empty_captcha');
 				}
 			break;
+			case 'invite':
+				foreach($this->oDOMDoc->query('accounts:invite', $oDOMAccount) as $oDOMInvite){
+					$this->aInviteList[] = $oDOMInvite->nodeValue;
+				}
+				break;
 			case 'sync':
 			case 'refresh':
-				if($oDOMForceGW = $this->oDOMDoc->getNode('accounts:force_gw',$oDOMAccount)){
-					$this->bForceGWReload =  $this->oDOMDoc->getNodeValue('accounts:force_gw',$oDOMAccount);
-					$this->bForceGWReload = $this->bForceGWReload?true:false;
-				}
+				$force = $oDOMAccount->getAttribute('force');
+				$this->bForceGWReload = ($force == 'true') || ($force == '1');
 				break;
 			case 'setup_two_factor':
 					if($oDOMTwoFactorType = $this->oDOMDoc->getNode('accounts:two_factor_type',$oDOMAccount)){
@@ -153,7 +159,10 @@ class WebmailIqAccounts extends XMLRequestInterface
 					$this->sTwoFactorKey =  $this->oDOMDoc->getNodeValue('accounts:two_factor_key',$oDOMAccount);
 				}
 			break;
+			default:
+				return false;
 		}
+		return true;
 	}
 	 
 	private function exeInputXML()
@@ -176,6 +185,9 @@ class WebmailIqAccounts extends XMLRequestInterface
 					default:
 					 					if($this->sAccountID){
 						$account = &$aAccounts[$this->sAccountID];
+						if(!$account){
+							throw new Exc('account_invalid_id', $this->sAccountID);
+						}
 						if($account->gwAccount && $account->gwAccount->bLogged){
 							$account->gwAccount->getAcl();
 							$account->acl = $account->gwAccount->acl;
@@ -224,7 +236,7 @@ class WebmailIqAccounts extends XMLRequestInterface
 							$aProperties['acl'] = $this->acl;
 						}
 						$oAccount = $oUser->getAccount($sAid);
-						if(!$oAccount->primary && ($aProperties['username'] || $aProperties['password'])){
+						if(!$oAccount->primary && ($aProperties['username'] || $aProperties['password'] || $aProperties['digest'])){
 							$oUser->testAccount($aProperties);
 						}
 						$aProperties['valid'] = 1;
@@ -269,9 +281,15 @@ class WebmailIqAccounts extends XMLRequestInterface
 								$this->aData["mbox_usage"] = $size?$size:'0';
 							}
 						}
-
 						log_buffer("QUOTA end","EXTENDED");
-						$this->aData['folder']['num'] = WebmailIqFolders::cnvFolders($folders,false,$_SESSION['refreshed']);
+						$this->aData['folder']['num'] = WebmailIqFolders::cnvFolders($folders,false,$_SESSION['refreshed'],$this->bForceGWReload);
+
+						if ($oAccount->primary){
+							if($oAccount->gwAccount->gwAPI->deviceID){
+								$this->aData['device_id'] = $oAccount->gwAccount->gwAPI->deviceID;
+							}
+						}
+
 						$_SESSION['refreshed'] = false;
 						$this->sTemplateFile='webmailiqfolders_get';
 							
@@ -284,6 +302,8 @@ class WebmailIqAccounts extends XMLRequestInterface
 					 					case 'signup':
 						User::signupAccount($aProperties,$this->sCaptchaUID);
 					break;
+					case 'invite':
+						$this->aData['invite'] = User::inviteAccounts($this->aInviteList);
 					 					case 'subscribe':
 						 						$oAccount = &$oUser->getAccount($this->sAccountID);
 						if($oAccount->gwAccount && $oAccount->gwAccount->bLogged){
@@ -346,6 +366,13 @@ class WebmailIqAccounts extends XMLRequestInterface
 						$oAccount = $oUser->getAccount($this->sAccountID);
 						$this->aData['totp_secret'] = $oAccount->twoFactorManual($this->sTwoFactorKey);
 						break;
+					case 'set_custombackup':
+						$api = new MerakAccount();
+						if($api->Open($this->sAccountID)) {
+							$api->SetProperty("u_custombackup", $aProperties['custombackup']);
+							$api->Save();
+						}
+						break;
 				}
 			break;
 		}
@@ -367,16 +394,23 @@ class WebmailIqAccounts extends XMLRequestInterface
 			}
 			 			$aAcc = array(); 
 			$aAcc["id"] = $key;
+			$logged = false;
 			$aAcc["gw"] = $oAccount->isGroupWare($logged)?'true':'false';
 			if (!$logged){
 				$aAcc["gw"] = 'false';
 			}
+			if($aAcc['gw']){
+				if($gwAPI && $gwAPI->getIsExternal()){
+					$aAcc['external'] = true;
+				}
+			}
 			$aAcc["port"] = $oAccount->port;
 			$aAcc["protocol"] = (strtolower($oAccount->protocol)=='local' || strtolower($oAccount->protocol)=='pop3')?'pop3':'imap';
 			$aAcc["username"] = $oAccount->username;
 			$aAcc["sentfolder"] = $oAccount->sentfolder;
 			$aAcc["trashfolder"] = $oAccount->trashfolder;
-
+			$aAcc["created"] = $oAccount->created;
+			$aAcc["modified"] = $oAccount->modified;
 			if (!$oAccount->primary){
 				if($oAccount->acc_type=='rss'){
 					$aAcc["acc_type"] = $oAccount->acc_type;
@@ -438,7 +472,8 @@ class WebmailIqAccounts extends XMLRequestInterface
 				if($oAccount->gwAccount->gwAPI->deviceID){
 					$aAcc['device_id'] = $oAccount->gwAccount->gwAPI->deviceID;
 				}
-				 				$aAcc["gwtrash_support"] = $_SESSION['GWTRASH_SUPPORT'] ? 'true' : 'false';
+				 				$aAcc["gw_support"] = $_SESSION['GW_SUPPORT'] ? 'true' : 'false';
+				$aAcc["gwtrash_support"] = $_SESSION['GWTRASH_SUPPORT'] ? 'true' : 'false';
 				$aAcc["meeting_support"] = $_SESSION['MEETING_SUPPORT'] ? 'true' : 'false';
 				$aAcc["conferenceparam_encrypted"] = $_SESSION['CONFERENCEPARAM_ENCRYPTED'];
 				$aAcc["recording_support"] = $_SESSION['RECORDING_SUPPORT'] ? 'true' : 'false';
@@ -471,6 +506,7 @@ class WebmailIqAccounts extends XMLRequestInterface
 				$aAcc['groupchat_support'] = $_SESSION['GROUPCHAT_SUPPORT'] ? 'true' : 'false';
 				$aAcc['two_factor_support'] = $_SESSION['TWO_FACTOR_SUPPORT'];
 				$aAcc['two_factor_enabled'] = $_SESSION['TWO_FACTOR_ENABLED'] ? 'true' : 'false';
+				$aAcc['recall_messages_enabled'] = $_SESSION['RECALL_MESSAGES_ENABLED'] ? 'true' : 'false';
 				$aAcc['smtp_header_function'] = $_SESSION['SMTP_HEADER_FUNCTION'] ? 'true' : 'false';
 				if($_SESSION['ARCHIVE_INTEGRATE']){
 					$aAcc['archive_prefix'] = $_SESSION['ARCHIVE_INTEGRATE_NAME'];
@@ -490,7 +526,9 @@ class WebmailIqAccounts extends XMLRequestInterface
 				if($_SESSION['TEAMCHATAPI_URL']){
 					$aAcc['teamchatapi_url'] = $_SESSION['TEAMCHATAPI_URL'];
 				}
-			
+				if($_SESSION['ICEWARPMOBILEAPP_URL']){
+					$aAcc['icewarpmobileapp_url'] = $_SESSION['ICEWARPMOBILEAPP_URL'];
+				}			
 				$aAcc['client_url'] = $_SESSION['WEBMAIL_URL'];
 				if($logged){
 					if(empty($_SESSION['GW_OWNERID'])){
@@ -511,6 +549,11 @@ class WebmailIqAccounts extends XMLRequestInterface
 				if($_SESSION['ICEWARP_SERVER_ID']){
 					$aAcc['icewarp_server_id'] = $_SESSION['ICEWARP_SERVER_ID'];
 				}
+				$aAcc['custombackup'] = $_SESSION['CUSTOMBACKUP'] ? 'true' : 'false';
+				$aAcc['quarantine_support'] = $_SESSION['QUARANTINE_SUPPORT'];
+				$api = createobject('API');
+				$aAcc['version'] = $api->GetProperty('C_Version');
+				$aAcc['nice_version'] = $api->GetProperty('C_Niceversion');
 			}
 			
 			if (!$aAcc["description"]){
