@@ -14,11 +14,14 @@ class WebmailIqAuth extends XMLRequestInterface
 		'sid' => 1,
 		'troubleshootingsession' => 1,
 		'deletetroubleshooting' => 1,
+		'externalsid' => 1,
 	);
 
 	public $oDOMMethod;
 	public $oDOMCommand;
 	public $oDOMURI;
+	public $oDOMDeviceID;
+	public $sDeviceID;
 	public $oDOMDigest;
 	public $sDigest;
 	public $oDOMLanguage;
@@ -88,6 +91,9 @@ class WebmailIqAuth extends XMLRequestInterface
 		if ($this->oDOMLanguage= $this->oDOMDoc->getNode("auth:language",$this->oDOMQuery)) {
 			$this->sLanguage = $this->oDOMDoc->getNodeValue("auth:language",$this->oDOMQuery);
 		}
+		if ($this->oDOMDeviceID = $this->oDOMDoc->getNode("auth:device_id",$this->oDOMQuery)) {
+			$this->sDeviceID = $this->oDOMDoc->getNodeValue("auth:device_id",$this->oDOMQuery);
+		}
 		if ($this->oDOMExternalIP = $this->oDOMDoc->getNode("auth:ip",$this->oDOMQuery)) {
 			$this->sExternalIP = $this->oDOMDoc->getNodeValue("auth:ip",$this->oDOMQuery);
 		}
@@ -130,12 +136,12 @@ class WebmailIqAuth extends XMLRequestInterface
 	}
 
      
-	public static function createSidByAToken(string $aToken, $language, $remember = false, $ctz = null, string $interface = 'BASIC')
+	public static function createSidByAToken(string $aToken, $language, $remember = false, $ctz = null, string $interface = 'BASIC', $device_id = '', $is_external = false)
 	{
 		$cookie = new slToolsCookie();
 		$user = IceWarpAPI::instance()->FunctionCall("GetTokenEmail", $aToken);
 		$pass = IceWarpAPI::instance()->FunctionCall("GetTokenPassword", $aToken);
-		$sid = User::login($user, $pass,"","",true,"",false, false, false, $language, $remember ? 'wm-perm' : 'wm');
+		$sid = User::login($user, $pass,"","",true,"",false, false, false, $language, $remember ? 'wm-perm' : 'wm-','', true, $device_id,$is_external);
 		$_SESSION['jscheck'] = true;
 		if($ctz) $_SESSION['CTZ'] = $ctz;
 		$cookie->setcookie('PHPSESSID_'.strtoupper($interface),$sid,time() + 31556926,'/');
@@ -152,8 +158,7 @@ class WebmailIqAuth extends XMLRequestInterface
     protected function getActionRsa()
 	{
 		$this->aData['method'] = "RSA";
-		self::checkServerKeys();
-		@$this->aData['hashId'] = file_get_contents(WM_CONFIGPATH.'public.key');
+		@$this->aData['hashId'] = WebmailRSA::instance()->getServerData('public');
 		@$this->aData['timestamp'] = time();
 	}
 
@@ -180,10 +185,17 @@ class WebmailIqAuth extends XMLRequestInterface
 		}
 	}
 
-	protected function getActionSid()
+	protected function getActionSid($external = false)
 	{
-		$this->aData['digest'] = self::createSidByAToken($this->authToken, $this->sLanguage, $this->remember);
-		$this->aData['method'] = "SID";
+		$sSID = self::createSidByAToken($this->authToken, $this->sLanguage, $this->remember, null, 'BASIC', $this->sDeviceID, $external);
+		$this->aData['digest'] = $sSID;
+		$this->aData['method'] = $external?'externalsid':"SID";
+		$this->aData['dlsess'] = $_SESSION['DLSESS'];
+	}
+
+	protected function getActionExternalSid()
+	{
+		$this->getActionSid(true);
 	}
 
 	protected function getActionSip()
@@ -217,7 +229,7 @@ class WebmailIqAuth extends XMLRequestInterface
 		$alias = reset($aliasArray);
 		$key = 'user=' . urlencode($alias);
 		$key .= '&pass=' . urlencode($sPassword);
-		$key .= '&uri=' . urlencode($this->sURI);
+		$key .= '&uri=' . urlencode($this->sURI ?? '');
 		$key .= '&algorithm=md5-sess';
 		$key .= '&command=' . urlencode($this->sCommand);
 		$this->aData['digest'] = base64_encode($api->CryptData('DIGEST-MD5', $key,  base64_decode($this->sHashID), true));
@@ -244,7 +256,7 @@ class WebmailIqAuth extends XMLRequestInterface
 		$_SESSION['created'] = time();
 		$_SESSION['validity'] = $this->troubleshootingValidity;
 		$_SESSION['expire'] = $expire = $_SESSION['created'] + $this->troubleshootingValidity;
-		$_SESSION['user'] =  $oUser->email;
+		$_SESSION['user'] =  $oUser;
 		$_SESSION['password'] = $oUser->getPassword(true);
 		session_write_close();
 
@@ -262,6 +274,7 @@ class WebmailIqAuth extends XMLRequestInterface
 		if ($method == 'rsa') return $this->getActionRsa();
 		if ($method == 'xmpp') return $this->getActionXmpp();
 		if ($method == 'sid') return $this->getActionSid();
+		if ($method == 'externalsid') return $this->getActionExternalSid();
 		if ($method == 'sip') return $this->getActionSip();
 		if ($method == 'websocket') return $this->getActionWebsocket();
 		if ($method == 'meeting') return $this->getActionMeeting();
@@ -285,6 +298,7 @@ class WebmailIqAuth extends XMLRequestInterface
 			throw new Exc('troubleshooting session invalid or expired');
 		}
 		$this->aAttrs['sid'] = User::login(($_SESSION['user'] instanceof User) ? $_SESSION['user']->email : $_SESSION['user'], slToolsCrypt::decryptSymmetric($_SESSION['password']), '', $this->sMethod, true,'', false, false, false,false, 'wm-');
+		$this->aAttrs['dlsess'] = $_SESSION['DLSESS'];
 	}
 
 	protected function setActionDeleteTroubleshooting()
@@ -353,6 +367,7 @@ class WebmailIqAuth extends XMLRequestInterface
 			$lastLanguage = $parse[0];
 		}
 		$this->aAttrs['sid'] = User::login($login, slToolsCrypt::decryptDbPassword($result->pass), "", "", true, "", false, false, false, $lastLanguage ? $lastLanguage : '', $_GET['remember'] ? 'wm-perm' : 'wm');
+		$this->aAttrs['dlsess'] = $_SESSION['DLSESS'];
 		$_SESSION['LOGOUT_REFERER'] = IceWarpAPI::instance()->getProperty('C_Webmail_URL');
 	}
 
@@ -365,6 +380,10 @@ class WebmailIqAuth extends XMLRequestInterface
 
 		 		$user = User::load($this->sSessionID);
 
+		if($this->sDeviceID){
+			$gwAPI = $user->getAccount($_SESSION['EMAIL'])->gwAccount->gwAPI;
+			$gwAPI->setDeviceID($this->sDeviceID);
+		}
 		if (strpos($this->sSessionID, 'wm-perm') !== false) {
 			 			$session_version = $_SESSION['WM_VERSION'];
 			$current_version = $api->GetProperty('c_version');
@@ -374,10 +393,11 @@ class WebmailIqAuth extends XMLRequestInterface
 				session_destroy();
 				throw new Exc('session_no_user');
 			}
-			 			$_SESSION['SETTINGS_API'] = false;
-			$user->getData();
 		}
 
+		 		$_SESSION['SETTINGS_API'] = false;
+		$user->getData();
+
 		if (!$this->bSessionKeep) {
 			unset($_SESSION['SID']);
 			$session_backup = $_SESSION;
@@ -415,6 +435,7 @@ class WebmailIqAuth extends XMLRequestInterface
 		$_SESSION['SID'] = $newSID;
 		$this->aAttrs['sid'] = $newSID;
 		$this->aAttrs['digest'] = $newSID;
+		$this->aAttrs['dlsess'] = User::getDlSess(true);
 	}
 
 	 
@@ -453,12 +474,20 @@ class WebmailIqAuth extends XMLRequestInterface
 		$digest = $this->oDOMDoc->getNodeValue("auth:digest", $this->oDOMQuery);
 		$cookie = false;
 
+		$api  = createobject('api');
+        $merakAccount = createobject('account');
+        $merakAccount->Open($userName);
+        if(filter_var($merakAccount->getProperty("U_2F_ENABLED"), FILTER_VALIDATE_BOOLEAN) && !preg_match('/\/server\/webmail.php$/', $_SERVER['SCRIPT_NAME'])) {
+			throw new Exc('auth_two_factor_required');
+		}
+
 		$this->setValFromDOMDoc($cookie, 'auth:cookie',$this->oDOMQuery);
 
 		 		if ($this->sMethod == 'rsa') {			$this->setActionRsa($cookies, $digest, $userName, $cookie);
 		} else {  
 			$sessionPrefix = ($this->autoLogin) ? 'wm-perm' : 'wm-';
 			$this->aAttrs['sid'] = User::login($userName, $digest, $hashID, $this->sMethod, true, '', $cookie, $this->sExternalIP, $this->sDisableIPCheck, $this->sLanguage, $sessionPrefix, $this->sUserCheck ? $this->sSessionID : '');
+			$this->aAttrs['dlsess'] = $_SESSION['DLSESS'];
 		}
 
 		if ($this->autoLogin) {
@@ -477,9 +506,7 @@ class WebmailIqAuth extends XMLRequestInterface
 	protected function setActionRsa($cookies, $digest, $userName, $cookie)
 	{
 		$hashID = $this->oDOMDoc->getNodeValue("auth:hashid", $this->oDOMQuery);
-		$privateData = self::getServerData('private');
-		$privateKey = self::getServerKey($privateData);
-		openssl_private_decrypt(self::hexStringToBinary($digest), $password, $privateKey);
+		$password = WebmailRSA::instance()->rsaToPlain($digest);
 		$pInfo = Tools::parseURL($password);
 		$password = urldecode($password);
 		if (is_array($pInfo) && !empty($pInfo)) {
@@ -494,6 +521,7 @@ class WebmailIqAuth extends XMLRequestInterface
 		$sessionPrefix = ($this->autoLogin) ? 'wm-perm' : 'wm-';
 		if(!$id){  
 			$this->aAttrs['sid'] = User::login($userName, $password, $hashID, null, true, '', $cookie, $this->sExternalIP, $this->sDisableIPCheck, $this->sLanguage, $sessionPrefix, $this->sUserCheck ? $this->sSessionID : '');
+			$this->aAttrs['dlsess'] = $_SESSION['DLSESS'];
 			return;
 		}
 
@@ -512,81 +540,7 @@ class WebmailIqAuth extends XMLRequestInterface
 		}
 		$_SESSION['SID'] = $sSID;
 		$this->aAttrs['sid'] = $sSID;
-	}
-	
-	public static function checkServerKeys() 
-	{
-		if(is_file(WM_CONFIGPATH.'public.key') && filesize(WM_CONFIGPATH.'public.key') < 512){
-			@unlink(WM_CONFIGPATH.'public.key');
-			@unlink(WM_CONFIGPATH.'private.key');
-		}
-		 		if (!is_file(WM_CONFIGPATH.'private.key')) {
-			$pair = icewarp_openssl_generate_RSA(2048);
-			if (!file_exists(WM_CONFIGPATH)) {
-				slSystem::import('tools/filesystem');
-				slToolsFilesystem::mkdir_r(WM_CONFIGPATH);
-			}
-			if(!@file_put_contents(WM_CONFIGPATH.'private.key', $pair['Private-PEM'])) return false;
-			return (bool)@file_put_contents(WM_CONFIGPATH.'public.key', ltrim($pair['modulus'],'0'));
-		}
-		if (!is_file(WM_CONFIGPATH.'public.key')) {
-			@unlink(WM_CONFIGPATH.'private.key');
-			throw new Exc('rsa_create_key_pair');
-		}
-	}
-	
-	public static function getServerKey($data)
-	{
-		$privateKey = openssl_pkey_get_private($data);
-		return $privateKey;
-	}
-	public static function getServerData($type = 'private')
-	{
-		slSystem::import('tools/filesystem');
-		slToolsFilesystem::securePath($type);
-		self::checkServerKeys();
-		@$privateData = file_get_contents(WM_CONFIGPATH.$type.'.key');
-		return $privateData;
-	}
-	public static function binaryToHexString($binary)
-	{
-		$binary = strrev($binary);
-		$valueToHexChar = array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f');
-		$binaryLength = strlen($binary);
-		$result = '';
-		for ($i = 0; $i < $binaryLength; $i++) {
-			$result .= $valueToHexChar[ord($binary[$i]) >> 4];
-			$result .= $valueToHexChar[ord($binary[$i]) & 0x0F];
-		}
-		return ltrim($result, '0');
-	}
-	public static function hexStringToBinary($hexString) {
-		$result = '';
-		$hexStringLength = strlen($hexString);
-		$i = 0;
-		if ($hexStringLength % 2) {
-			$result = chr(self::hexCharValue($hexString[0]));
-			$i = 1;
-		}
-		for (; $i < $hexStringLength; $i += 2) {
-			$result .= chr(self::hexCharValue($hexString[$i])*16 + self::hexCharValue($hexString[$i+1]));
-		}
-		return $result;
-	}
-
-	public static function hexCharValue($hexChar) {
-		if ($hexChar == '') return 0;
-		if ($hexChar <= '9') {
-			if ($hexChar >= '0') return ord($hexChar) - ord('0');
-			throw new Exc('auth_invalid_char');
-		} else if ($hexChar <= 'F') {
-			if ($hexChar >= 'A') return ord($hexChar) - ord('A') + 10;
-			throw new Exc('auth_invalid_char');
-		} else if ($hexChar <= 'f') {
-			if ($hexChar >= 'a') return ord($hexChar) - ord('a') + 10;
-			throw new Exc('auth_invalid_char');
-		}
-		throw new Exc('auth_invalid_char');
+		$this->aAttrs['dlsess'] = User::getDlSess(true);
 	}
 }
 ?>
