@@ -18,7 +18,11 @@ class WebmailIqFolders extends XMLRequestInterface
         'remove_member' => 1,
         'add_member' => 1,
         'notify_limit' => 1,
-		'refresh' => 1
+		'refresh' => 1,
+		'leave' => 1,
+		'makeowner' => 1,
+		'resetowner' => 1,
+		'autofollow' => 1,
     );
 
     private $oDOMAccount;      private $oDOMFolder;       private $sAID;          private $sFID;      
@@ -44,6 +48,7 @@ class WebmailIqFolders extends XMLRequestInterface
     public $oDOMDoc;
     public $gwFolder;
     public $sFolderName;
+	public $sFolderOwner;
     public $sFType;
     public $sChannelURL;
     public $bCloneRoom;
@@ -57,7 +62,7 @@ class WebmailIqFolders extends XMLRequestInterface
     public $sShareType;
     public $bNotify;
     public $virtual;
-    public $acl;
+    public $acl = [];
     public $removeAcl;
     public $sDestinationAccount;
     public $sDestinationFolder;
@@ -66,6 +71,9 @@ class WebmailIqFolders extends XMLRequestInterface
     public $sComment;
     public $special;
     public static $addCount = false;
+    public $autofollow;
+	public $bForce = false;
+	public $oDistrib;
 
 	 
     public function __construct($oDOMQuery, &$oDOMDoc, &$attrs)
@@ -82,7 +90,6 @@ class WebmailIqFolders extends XMLRequestInterface
 	 
 	private function checkInputXML()
 	{
-		
 		         if (!$this->oDOMAccount = $this->oDOMDoc->getNode("folders:account", $this->oDOMQuery)) {
             throw new Exc('folder_missing_tag', 'account');
 		}
@@ -96,6 +103,8 @@ class WebmailIqFolders extends XMLRequestInterface
 				}elseif (!$this->aAccounts[$this->sAID]) {
 					throw new Exc('account_invalid_id', $this->sAID);
 				}
+				 				$force = $this->oDOMAccount->getAttribute('force');
+				$this->bForce = ($force == 'true') || ($force == '1');
 				                 if ($this->oDOMFolder = $this->oDOMDoc->getNode("folders:folder", $this->oDOMAccount)) {
 					$this->sFID = $this->oDOMFolder->getAttribute('uid');
 					$this->sSearch = $this->oDOMDoc->getNodeValue("folders:search", $this->oDOMFolder);
@@ -113,7 +122,8 @@ class WebmailIqFolders extends XMLRequestInterface
                     throw new Exc('folder_missing_tag', 'action');
 				}
 				$this->aAttrs['action'] = $this->sAction;
-				                 if (!isset($this->aActions[$this->sAction])) {
+				 
+                if (!isset($this->aActions[$this->sAction])) {
                     throw new Exc('folder_invalid_action', $this->sAction);
 				}
 				
@@ -214,7 +224,7 @@ class WebmailIqFolders extends XMLRequestInterface
                                 $email = $oDOMRight->getAttribute('email');
 								$this->acl[$email] = Folder::rightsToBitValue($oDOMRight->nodeValue);
 							}
-                            if (count($this->acl) == 0) {
+                            if (empty($this->acl) || count($this->acl) == 0) {
 								$this->removeAcl = true;
 							}
 						}
@@ -281,6 +291,13 @@ class WebmailIqFolders extends XMLRequestInterface
 					case 'remove_member':
                         $this->sRemoveAccount = $this->oDOMDoc->getNodeValue("folders:account", $this->oDOMFolder);
 					break;
+					case 'makeowner':
+					case 'resetowner':
+						$this->sFolderOwner = $this->oDOMDoc->getNodeValue("folders:folder", $this->oDOMAccount);
+					break;
+					case 'autofollow':
+						$this->autofollow = filter_var($this->oDOMDoc->getNodeValue("folders:autofollow", $this->oDOMFolder) ?? false, FILTER_VALIDATE_BOOLEAN);
+						break;
 					default:
 					break;
 				}
@@ -326,20 +343,28 @@ class WebmailIqFolders extends XMLRequestInterface
 				 
 				$folder->rights = $account->getMyRights($this->sFID);
 				Folder::checkRights($folder, Folder::RIGHT_ADMIN);
-				$folder->acl = $account->getAcl($this->sFID, $folder->getType() == 'I' || $folder->getType() == 'Y');
-				$sizeFolder = $account->getFolder('__@@SIZE@@__/' . $folder->groupOwner);
-				$folder->size = $sizeFolder->getSize();
-				$folder->quota = $sizeFolder->getQuota();
-				$inherited = true;
-				if ($folder->acl) {
-					@$someUserRight = reset($folder->acl);
-					if ($someUserRight & Folder::RIGHT_INHERITED) {
-						$inherited = true;
-					} else {
-						$inherited = false;
+				$folder->acl = $account->getAcl($this->sFID, true, true);
+				$folder->inherited = false;
+
+				 				if ($folder->acl) {
+					 					if (!empty($folder->acl['@'])) {
+
+						 						$folder->inherited = true;
+
+						 						unset($folder->acl['@']);
 					}
+					 					@$someUserRight = reset($folder->acl);
+					if ($someUserRight['rights'] & Folder::RIGHT_INHERITED) {
+						$folder->inherited = true;
+					}
+				}				
+				try {
+					$sizeFolder = $account->getFolder('__@@SIZE@@__/' . $folder->groupOwner);
+					$folder->size = $sizeFolder->getSize();
+					$folder->quota = $sizeFolder->getQuota();
+				} catch (Exc $exc) {
+
 				}
-				$folder->inherited = $inherited;
 			} catch (Exc $e) {
 
 			}
@@ -384,7 +409,7 @@ class WebmailIqFolders extends XMLRequestInterface
 		}
 		$this->sTemplateFile = 'webmailiqfolders_get';
 		$this->aData['aid'] = $this->sAID;
-		$this->aData['folder'] = $this->cnvFolders($folders, false, true);  	}
+		$this->aData['folder'] = $this->cnvFolders($folders, false, true, $this->bForce);  	}
 
 	 
 	protected function checkNewFolderForTC($oAccount)
@@ -515,6 +540,7 @@ class WebmailIqFolders extends XMLRequestInterface
 				throw new Exc($e->wmcode, $e->getMessage());
 			}
 			 		}
+
 		 		if ($this->acl) {
 			$oAccount->setAcl($this->acl, $this->sFID);
 		} else {
@@ -524,7 +550,7 @@ class WebmailIqFolders extends XMLRequestInterface
 		}
 		if ($sFolderType == 'M' || $sFolderType == 'R') {
 			$oAccount->sync();
-		} else {
+		} else if($this->oFolder->type!='V'){
 			if ($oAccount->account) {
 				$oAccount->account->sync(true);
 			}
@@ -533,15 +559,6 @@ class WebmailIqFolders extends XMLRequestInterface
 		$this->oFolder->rights = $oAccount->getMyRights($folderID);
 		$folders = $this->cnvFolders(array($folderID => $this->oFolder));
 		$this->aData = reset($folders);
-		if (isset($parameters['search'])) {
-			$this->aData['search'] = $parameters['search'];
-		}
-		if ($this->oFolder->sharetype) {
-			$this->aData['sharetype'] = $this->oFolder->sharetype;
-		}
-		$this->aData['subscribed'] = $this->oFolder->groupChatSubscribed;
-		$this->aData['relative_path'] = $this->oFolder->groupRelativeFolder;
-		$this->aData['teamchat_notify'] = $this->oFolder->teamChatNotify;
 	}
 
 	protected function setActionDelete($oUser, $oAccount, $sFolderType)
@@ -564,7 +581,7 @@ class WebmailIqFolders extends XMLRequestInterface
 		$this->oFolder->sync();
 		log_buffer("SYNC folder " . $this->sFID . " end","EXTENDED");
 		$this->aData['aid']  = $this->sAID;
-		$this->aData['folder']['num'] = WebmailIqFolders::cnvFolders([$this->oFolder],false,$_SESSION['refreshed']);
+		$this->aData['folder']['num'] = WebmailIqFolders::cnvFolders([$this->oFolder],false,$_SESSION['refreshed'],$this->bForce);
 		$_SESSION['refreshed'] = false;
 		$this->sTemplateFile='webmailiqfolders_get';
 
@@ -589,7 +606,19 @@ class WebmailIqFolders extends XMLRequestInterface
 		$oDestAccount = $oUser->getAccount($this->sDestinationAccount);
 		$oDestFolder = $oDestAccount->getFolder($this->sDestinationFolder);
 		$this->oFolder = &$oAccount->getFolder($this->sFID);
-		$this->oFolder->moveItems($oDestFolder);
+		$folders = [$this->oFolder];
+		if ($this->oFolder instanceof VirtualFolder) {
+			$folders = $this->oFolder->folders;
+			$this->sSearch = (empty($this->sSearch) ? '' : $this->sSearch) . ' ' . $this->oFolder->search;
+		}
+		foreach ($folders as $folder) {
+			if(empty($this->sSearch)) {
+				$result = $folder->moveItems($oDestFolder);
+			} else {
+				$result = $folder->moveFiltered($this->sSearch, $oDestFolder);
+			}
+		}
+		return $result;
 	}
 
 	protected function setActionCopyall($oUser, $oAccount, $sFolderType)
@@ -597,21 +626,47 @@ class WebmailIqFolders extends XMLRequestInterface
 		$oDestAccount = $oUser->getAccount($this->sDestinationAccount);
 		$oDestFolder = $oDestAccount->getFolder($this->sDestinationFolder);
 		$this->oFolder = &$oAccount->getFolder($this->sFID);
-		$this->oFolder->copyItems($oDestFolder);
+		$folders = [$this->oFolder];
+		if ($this->oFolder instanceof VirtualFolder) {
+			$folders = $this->oFolder->folders;
+			$this->sSearch = (empty($this->sSearch) ? '' : $this->sSearch) . ' ' . $this->oFolder->search;
+		}
+		foreach ($folders as $folder) {
+			if(empty($this->sSearch)) {
+				$result = $folder->copyItems($oDestFolder);
+			} else {
+				$result = $folder->copyFiltered($this->sSearch, $oDestFolder);
+			}
+		}
+		return $result;
 	}
 
 	protected function setActionMarkasread($oUser, $oAccount, $sFolderType)
 	{
+		$oItems = null;
 		$flag = Item::FLAG_SEEN;
 		 		$this->oFolder = &$oAccount->getFolder($this->sFID);
-		$this->oFolder->markItems($flag);
+		if($this->sSearch) {
+			$oItems = $this->oFolder->getFilteredItems($this->sSearch);
+			if(empty($oItems)){
+				return true;
+			}
+		}
+		$this->oFolder->markItems($flag, $oItems);
 	}
 
 	protected function setActionMarkasunread($oUser, $oAccount, $sFolderType)
 	{
+		$oItems = null;
 		$flag = Item::FLAG_SEEN;
 		 		$this->oFolder = &$oAccount->getFolder($this->sFID);
-		$this->oFolder->unmarkItems($flag);
+		if($this->sSearch) {
+			 $oItems = $this->oFolder->getFilteredItems($this->sSearch);
+			if(empty($oItems)){
+				return true;
+			}
+		}
+		$this->oFolder->unmarkItems($flag, $oItems);
 	}
 
 	protected function setActionSaveItems($oUser, $oAccount, $sFolderType)
@@ -667,6 +722,18 @@ class WebmailIqFolders extends XMLRequestInterface
 		$oAccount->gwAccount->sendFolderLimitNotification($this->sFID, $this->sComment);
 	}
 
+	protected function setActionMakeOwner($oUser, $oAccount, $sFolderType)
+	{
+		$this->oFolder = &$oAccount->getFolder($this->sFID);
+		$this->oFolder->makeOwner($this->sFolderOwner);
+	}
+
+	protected function setActionResetOwner($oUser, $oAccount, $sFolderType)
+	{
+		$this->oFolder = &$oAccount->getFolder($this->sFID);
+		$this->oFolder->resetOwner($this->sFolderOwner);
+	}
+
 	 
 	protected function exeSetInputXML()
 	{
@@ -676,7 +743,7 @@ class WebmailIqFolders extends XMLRequestInterface
 			if ($this->sFType == 'QL' || $this->sFType == 'VS') {
 				$sFolderType = $this->sFType;
 			} else {
-				$sFolderType = strtoupper($this->sFType{0});
+				$sFolderType = strtoupper(substr($this->sFType, 0, 1));
 			}
 		} else {
 			$folder = $oAccount->getFolder($this->sFID);
@@ -712,7 +779,9 @@ class WebmailIqFolders extends XMLRequestInterface
 		elseif ($this->sAction == 'remove_member') $this->setActionRemoveMember($oUser, $oAccount, $sFolderType);
 		elseif ($this->sAction == 'add_member') $this->setActionAddMember($oUser, $oAccount, $sFolderType);
 		elseif ($this->sAction == 'notify_limit') $this->setActionNotifyLimit($oUser, $oAccount, $sFolderType);
-
+		elseif ($this->sAction == 'makeowner') $this->setActionMakeOwner($oUser, $oAccount, $sFolderType);
+		elseif ($this->sAction == 'resetowner') $this->setActionResetOwner($oUser, $oAccount, $sFolderType);
+		
 		$this->aData['aid'] = $this->sAID;
 		$this->aData['fid'] = $this->oFolder->name;
 		$this->aData['rights'] = Folder::rightsToString($this->oFolder->rights, $this->oFolder->name);
@@ -728,31 +797,39 @@ class WebmailIqFolders extends XMLRequestInterface
 		}
 	}
 
-	 
-	public static function cnvFolders($folders, $interface = false, $actual = false)
+		 		 
+	public static function cnvFolders($folders, $interface = false, $actual = false, $returnRecent = false)
 	{
 		if (!is_array($folders)) return;
 		$aItems = array();
 		$emailFolders = [];
-		foreach ($folders as $k => $v) {
+        foreach ($folders as $k => $v) {
 			$aItm = array();
                        
-			 			if (!isset($v->gw) || !$v->gw) {
-				$emailFolders[] = $v;
-				 			} else {
+			 			if ((!isset($v->gw) || !$v->gw)) {
+				 				if(!$v->account || !$v->account->isGuest){
+					$emailFolders[$v->name] = $v;
+				}
+			 			} else {
                 $aItm['rights'] = Folder::rightsToString($v->rights, $v->name);
-                if ($actual) {
+                if ($actual || $returnRecent) {
                     if ($v->groupChatUnread == 0) {
 						$aItm['force_recent'] = 1;
 					}
                     $aItm['recent'] = $v->groupChatUnread;
 				}
+				if($v->getType()=='I' && !$returnRecent){
+					unset($aItm['force_recent']);
+					unset($aItm['recent']);
+				}
 				$aItm['owner'] = $v->groupOwner;
 				$aItm['relative_path'] = $v->groupRelativeFolder;
 				$aItm['subscribed'] = $v->groupChatSubscribed;
 				$aItm['groupchat_lastreadid'] = $v->groupChatLastReadID;
 				$aItm['groupchat_lastactivity'] = $v->groupChatLastActivity;
 				$aItm['teamchat_notify'] = $v->teamChatNotify;
+				$aItm['autofollow'] = ($v->fdrGroupChatAutoFollow ?? false) ? 1:0;
+				$aItm['created'] = $v->fdrCreated;
                 if ($v->type != 'V') {
 					$aItm['private_root'] = $v->isPrivateRoot();
 				}
@@ -760,6 +837,7 @@ class WebmailIqFolders extends XMLRequestInterface
             if ($v->type == 'V') {
 				$aItm['tagname'] = 'virtual';
 				$aItm['search'] = $v->search;
+				$aItm['subscribed'] = $v->sync;
 				$aItm['sharetype'] = $v->sharetype;
 				$v->rights = $v->getMyRights();
                 $aItm['rights'] = Folder::rightsToString($v->rights, $v->name);
@@ -791,11 +869,11 @@ class WebmailIqFolders extends XMLRequestInterface
 				}
             	$aItm['folders'] = $fdrs;
 			}
-			$aItm['name'] = $v->name;
+            $aItm['name'] = $v->name;
 
 			$aItm['id'] = $v->name;
-			 			if ($interface) {
-				if ($v->localizedName) {
+			             if ($interface) {
+                if ($v->localizedName) {
 					$aItm['localizedName'] = $v->localizedName;
 				}
 				if($v->getType() == 'C'){
@@ -808,7 +886,9 @@ class WebmailIqFolders extends XMLRequestInterface
             if (isset($v->acl) && $v->acl) {
                 foreach ($v->acl as $owner => $right) {
 					$r['email'] = $owner;
-					$r['right'] = Folder::rightsToString($right);
+					$r['right'] = Folder::rightsToString($right['rights'] ?? $right);
+					$r['name'] = $right['name'];
+					$r['isGuest'] = $right['isGuest'];
 					$aItm['acl'][] = $r;
 				}
 			}
@@ -821,7 +901,7 @@ class WebmailIqFolders extends XMLRequestInterface
 			$aItm['archive'] = $v->isArchive();
 			$aItm['spam'] = $v->isSPAM();
 			$aItm['resource'] = $v->isResource();
-			$aItm['default'] = $v->account->primary || $v->getType() != 'M' ? $v->isDefault():true;
+			$aItm['default'] = $v->account && $v->account->primary || $v->getType() != 'M' ? $v->isDefault():true;
 			$aItm['default_type'] = $v instanceof IMAPFolder ? $v->getTypeFromAttributes() : $v->defaultType;
             $aItm['search'] = $v->search ?? false;
             $aItm['display'] = $v->display ?? false;
@@ -835,22 +915,43 @@ class WebmailIqFolders extends XMLRequestInterface
 			$aItems[spl_object_hash($v)] = $aItm;
 		}
 		$counts = [];
-		$cache = Cache::instance($_SESSION['user']);
-		$counts = $cache->getEmailFoldersCounts($emailFolders, self::$addCount);
-		foreach ($emailFolders as $emailFolder) {
-			$aItm = & $aItems[spl_object_hash($emailFolder)];
-			$aItm['rights'] = Folder::rightsToString($emailFolder->rights, $emailFolder->name);
-			 			if ($emailFolder->type == 'R') {
+		if($emailFolders){
+			$cache = Cache::instance($_SESSION['user']);
+			$counts = $cache->getEmailFoldersCounts($emailFolders, self::$addCount);
+			foreach ($emailFolders as $emailFolder) {
+				$aItm = & $aItems[spl_object_hash($emailFolder)];
 				$aItm['rights'] = Folder::rightsToString($emailFolder->rights, $emailFolder->name);
-				$aItm['channel'] = $emailFolder->channel;
+				 				if ($emailFolder->type == 'R') {
+					$aItm['channel'] = $emailFolder->channel;
+				}
+				if (($emailFolder->isDefault && $emailFolder->defaultType == 'D') || self::$addCount || ($emailFolder->type=='SNOOZED')) {
+					$aItm['count'] = intval($counts[$emailFolder->folderID]['messages'] ?? 0);
+				}
+				$aItm['recent'] = intval($counts[$emailFolder->folderID]['countUnread'] ?? 0);
+				
+				if($emailFolder->type == 'V' && $emailFolder->sync){
+					$aItm['recent'] = $emailFolder->countItems(Item::FLAG_SEEN, false);
+				}
+
+				if ($aItm['recent'] == 0) {
+ 					$aItm['force_recent'] = 1;
+				}
+				$aItm['subscribed'] = (bool)($emailFolder->sync ?? false);
 			}
-			if (($emailFolder->isDefault && $emailFolder->defaultType == 'D') || self::$addCount) {
-				$aItm['count'] = intval($counts[$emailFolder->folderID]['messages'] ?? 0);
-			}
-			$aItm['recent'] = intval($counts[$emailFolder->folderID]['countUnread'] ?? 0);
-			$aItm['subscribed'] = (bool)($emailFolder->sync ?? false);
 		}
-         		return $aItems;
+    	return $aItems;
+	}
+
+	public function setActionLeave($oUser, $oAccount, $sFolderType)
+	{
+		$folder = $oAccount->getFolder($this->sFID);
+		$oAccount->gwAPI->FunctionCall('LeaveFolder', $folder->openAccess());
+	}
+
+	public function setActionAutoFollow($oUser, $oAccount, $sFolderType)
+	{
+		$folder = $oAccount->getFolder($this->sFID);
+		$oAccount->gwAPI->FunctionCall('SetTeamchatRoomAutoFollow', $folder->openAccess(), 'autofollow='.($this->autofollow ? '1':'0'));
 	}
 }
 ?>
