@@ -9,7 +9,8 @@ class WebmailIqTools extends XMLRequestInterface
         'cancel_delivery' => 1,
         'distrib' => 1,
         'weather' => 1,
-        'signature' => 1
+        'signature' => 1,
+        'geo' => 1
     );
 
     public $oDOMQuery;
@@ -84,6 +85,23 @@ class WebmailIqTools extends XMLRequestInterface
     }
 
      
+    protected function getActionGeo(DOMElement $oDOMTool, string & $sResponse)
+    {
+        $ip = self::getClientIp();
+        if(!$ip) throw new Exc('not_supported');
+        $arrContextOptions = ['ssl' => ['verify_peer' => false, 'verify_peer_name' => false], 'http' => ['timeout' => 5]];          $result = file_get_contents('https://get.geojs.io/v1/ip/geo/' . $ip . '.json', false, stream_context_create($arrContextOptions));
+        if (json_decode($result)) {
+            $sResponse .= template('inc/templates/tools_geo.tpl', ['geo' => $result]);
+        } else {
+            throw new Exc('wrong_geo', 'IP: ' . $ip . ' ' . $result);
+        }
+    }
+
+         public static function getClientIp() {
+        return $_SERVER['HTTP_CLIENT_IP'] ?? $_SERVER['HTTP_X_FORWARDED_FOR'] ?? $_SERVER['HTTP_X_FORWARDED'] ?? $_SERVER['HTTP_FORWARDED_FOR'] ?? $_SERVER['HTTP_FORWARDED'] ?? $_SERVER['REMOTE_ADDR'] ?? false;
+    }
+
+     
     protected function getActionTimezone(DOMElement $oDOMTool, string & $sResponse)
     {
         $result = array();
@@ -201,11 +219,11 @@ class WebmailIqTools extends XMLRequestInterface
             $city = $this->oDOMDoc->getNodeValue('tools:city', $weatherItem);
             if (Storage::checkCityWeather($oAccount->gwAccount, $city, 'weather', $info, true)) {
                 $items['items'] = $info;
+                $items = Tools::htmlspecialchars_array($items);
+                $sResponse .= template('inc/templates/tools_weather.tpl', $items);
             }
                          break;
         }
-        $items = Tools::htmlspecialchars_array($items);
-        $sResponse .= template('inc/templates/tools_weather.tpl', $items);
     }
 
      
@@ -264,7 +282,7 @@ class WebmailIqTools extends XMLRequestInterface
             }
             $rights = Folder::rightsToBitValue($rights);
             $oItem = $this->oFolder->getItem($uid);
-            $url = $oItem->getPublicUrl($aid, $rights);
+            $url = $oItem->getPublicUrl($aid, $rights, false, 'OFFICETICKET');
             $sResponse .= '<ticket>' . slToolsPHP::htmlspecialchars($url) . '</ticket>';
         }
     }
