@@ -14,6 +14,15 @@ if(!defined('SHAREDLIB_PATH')) {
 }
 
 require_once(SHAREDLIB_PATH . '/tools/cookie.php');
+require_once('inc/tools.php');
+require_once('inc/defines.php');
+require_once('inc/require.php');
+
+$wad = Tools::getWebAdminSession($_GET['wa_sid']);
+if (!$wad) {
+    header($_SERVER['SERVER_PROTOCOL'] . ' 400 BAD REQUEST', true, 400);
+    die();
+}
 
 foreach (getallheaders() as $key => $value){
     if($key == 'Host') continue;
@@ -25,4 +34,4 @@ $cookie->setcookie('waid',$_GET['wa_sid'],false, '/');
 $cookie->setcookie('wall',$_GET['language'] ?? 'en',false, '/');
 
 header("Location: " . urldecode($_GET['redirect_uri'] ?? $_SERVER['HTTP_ORIGIN'] ?? $_SERVER['HTTP_REFERER'] ?? $_SERVER['REMOTE_ADDR']));
-die();
\ No newline at end of file
+die();
