@@ -1,6 +1,7 @@
 <?php
  
 
+   define('API_CREATE_RETRY', true);   define('API_CREATE_TIMEOUT', 10);  
  if($argv) foreach($argv as $arg){
 	if(strpos($arg,'parameters=')===0){
 		$parameters = str_replace("parameters=","",$arg);
@@ -26,7 +27,7 @@ require_once('inc/defines.php');
 
 @session_start();
 try{
-	WebmailIqAuth::checkServerKeys();
+	WebmailRSA::instance()->checkServerKeys();
 	User::settingsGlobal();
 	$api = IceWarpAPI::instance('UpgradeTask');
 	$_SESSION['QUERY_LOG'] = $api->getProperty('c_system_sqllogtype');		
